﻿using System;
using System.IO;

namespace CWIDatas
{

    public enum ランダム選択対象 : byte
    {
        パーティ全体 = 0x01, 敵全体, フィールド全体
    }

    public enum 選択対象状態 : byte
    { 行動可能, 行動不可, 生存, 非生存, 
        健康, 負傷, 重症, 意識不明, 
        中毒, 眠り, 呪縛, 麻痺石化, 
        混乱, 激昂, 勇猛, 恐慌 }

    /// <summary>
    /// ランダム選択コンテントのデータ
    /// </summary>
    public class RandomSelectContent : BaseContent
    {
        #region プロパティ
        /// <summary>
        /// 選択対象
        /// </summary>
        public ランダム選択対象 Target
        {
            get { return (ランダム選択対象)b; }
        }

        /// <summary>
        /// 選択対象レベルを考慮するか
        /// </summary>
        public bool WithLevel
        {
            get { return (flag & 0x01) != 0x00; }
        }
        byte flag = 0x00;

        /// <summary>
        /// 対象最小レベル
        /// </summary>
        private const uint TARGET_MIN_LEVEL = 1;
        /// <summary>
        /// 対象最大レベル
        /// </summary>
        private const uint TARGET_MAX_LEVEL = 99;

        /// <summary>
        /// 選択対象レベルFrom
        /// </summary>
        public uint LevelFrom
        {
            get { return ui; }
            set
            {
                if (value < TARGET_MIN_LEVEL) ui = TARGET_MIN_LEVEL;
                else if (value > TARGET_MAX_LEVEL) ui = TARGET_MAX_LEVEL;
                else ui = value;
            }
        }
        /// <summary>
        /// 選択対象レベルTo
        /// </summary>
        public uint LevelTo
        {
            get { return uiTo; }
            set
            {
                if (value < TARGET_MIN_LEVEL) uiTo = TARGET_MIN_LEVEL;
                else if (value > TARGET_MAX_LEVEL) uiTo = TARGET_MAX_LEVEL;
                else uiTo = value;
            }
        }
        uint uiTo;

        /// <summary>
        /// 選択対象状態を考慮するか
        /// </summary>
        public bool WithState
        {
            get { return (flag & 0x02) != 0x00; }
        }

        /// <summary>
        /// 選択対象状態
        /// </summary>
        public 選択対象状態 State
        {
            get { return (選択対象状態)b; }
        }
        #endregion

        #region コンストラクタ
        public RandomSelectContent() : base(ContentType.RandomSelect) 
        { ui = TARGET_MIN_LEVEL; uiTo = TARGET_MIN_LEVEL; }
        #endregion

        #region リードライト
        public override void Read(BinaryReader r)
        {
            // 選択対象
            b = r.ReadByte();
            // 詳細条件フラグ
            flag = r.ReadByte();
            if (WithLevel)
            {
                // 選択対象レベル
                ui = r.ReadUInt32();
                uiTo = r.ReadUInt32();
            }
            if (WithState)
            {
                // 選択対象の状態
                b2 = r.ReadByte();
            }
        }
        public override void Write(BinaryWriter w)
        {
            // 選択対象
            w.Write(b);
            // 詳細条件フラグ
            w.Write(flag);
            if (WithLevel)
            {
                // 選択対象レベル
                w.Write(ui);
                w.Write(uiTo);
            }
            if (WithState)
            {
                // 選択対象の状態
                w.Write(b2);
            }
        }
        #endregion
    }
}
