﻿using System;
using System.IO;

namespace CWIDatas
{
    /// <summary>
    /// 背景位置・サイズ変更コンテントのデータ
    /// </summary>
    public class ViewPosSizeContent : BaseContent
    {
        #region プロパティ
        /// <summary>
        /// 位置変更を考慮するか
        /// </summary>
        public bool WithPos
        {
            get { return (flag & 0x01) != 0x00; }
        }
        byte flag = 0x00;
        /// <summary>
        /// サイズ変更を考慮するか
        /// </summary>
        public bool WithSize
        {
            get { return (flag & 0x02) != 0x00; }
        }
        byte posUnit;
        uint posxParam;
        uint posyParam;
        byte sizeUnit;
        uint sizeWidthParam;
        uint sizeHeightParam;
        #endregion

        #region コンストラクタ
        public ViewPosSizeContent()
            : base(ContentType.ViewPosSize) {}
        #endregion

        #region リードライト
        public override void Read(BinaryReader r)
        {
            // セル名
            str = ((SjisCBinaryReader)r).ReadSmallString();
            // 詳細条件フラグ
            flag = r.ReadByte();
            if (WithPos)
            {
                // 位置変更
                posUnit = r.ReadByte();
                posxParam = ((SjisCBinaryReader)r).Read80UInt32();
                posyParam = ((SjisCBinaryReader)r).Read80UInt32();
            }
            if (WithSize)
            {
                // サイズ変更
                sizeUnit = r.ReadByte();
                sizeWidthParam = ((SjisCBinaryReader)r).Read80UInt32();
                sizeHeightParam = ((SjisCBinaryReader)r).Read80UInt32();
            }
        }
        public override void Write(BinaryWriter w)
        {
            // セル名
            ((SjisCBinaryWriter)w).WriteSmallString(str);
            // 詳細条件フラグ
            w.Write(flag);
            if (WithPos)
            {
                // 位置変更
                w.Write(posUnit);
                ((SjisCBinaryWriter)w).Write80UInt32(posxParam);
                ((SjisCBinaryWriter)w).Write80UInt32(posyParam);
            }
            if (WithSize)
            {
                // サイズ変更
                w.Write(sizeUnit);
                ((SjisCBinaryWriter)w).Write80UInt32(sizeWidthParam);
                ((SjisCBinaryWriter)w).Write80UInt32(sizeHeightParam);
            }
        }
        #endregion

    }
}
