﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using CWIDatas;
using CWI.Dialog;
using CWI.Card;
using System.Text.RegularExpressions;  // for Regex 

namespace CWI
{
    /// <summary>
    /// キャラクターデータ.wcp
    /// </summary>
    public class CharacterData : DataWithIO
    {
        public enum StateMind : byte
        { 正常 = 0, 眠り, 混乱, 激昂, 勇敢, 恐慌 }

        #region フィールド
        byte widid;

        ImageData image;
        string name;

        uint numberANDversion;

        byte resi_physical;
        byte resi_magic;
        byte resi_body;
        byte resi_mind;
        byte weak_unholy;
        byte weak_magiclife;
        byte resi_hot;
        byte resi_cold;
        byte weak_hot;
        byte weak_cold;

        uint level;
        int nazo1_0; // Mate*.widにはここに謎の０がある
        string expression;
        int hitpoint;
        int hitpointMax;

        int state_paralyse;
        int state_poison;

        int add_avoid;
        int add_resist;
        int add_armor;

        int char_dex;
        int char_agl;
        int char_int;
        int char_str;
        int char_vit;
        int char_min;

        int char_aggressive;
        int char_cheerful;
        int char_brave;
        int char_cautious;
        int char_trickish;

        byte state_mind; // 眠り:1, 混乱:2, 激昂:3, 勇敢:4, 恐慌:5
        uint state_mind_round;
        uint state_bind_round;
        uint state_silent_round;
        uint state_disclose_round;
        uint state_magicprotect_round;
        int state_power;
        uint state_power_round;
        int state_avoid;
        uint state_avoid_round;
        int state_resist;
        uint state_resist_round;
        int state_armor;
        uint state_armor_round;
        List<WidCard> itemList;
        List<WidCard> skillList;
        List<WidCard> beastList;
        List<CouponData> couponList;

        byte stateReverse;
        byte stateVanish;
        #endregion

        #region 追加フィールド
        int imageSize;
        int nameSize;
        #endregion

        #region プロパティ
        /// <summary>
        /// Mate.widのみ２。宿の冒険者情報は既に０。
        /// </summary>
        public WidIDtype WidID
        {
            get { return (WidIDtype)widid; }
        }
        /// <summary>
        /// 画像
        /// </summary>
        public ImageData Image
        {
            get { return image; }
            set { image = value; }
        }
        /// <summary>
        /// カードの名前
        /// </summary>
        public string CardName
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// データバージョン
        /// </summary>
        public uint Version
        {
            get { return numberANDversion / CWData.VersionOffset; }
            set { numberANDversion = FileNumber + value * CWData.VersionOffset; }
        }

        /// <summary>
        /// カード番号
        /// </summary>
        public uint FileNumber
        {
            get { return numberANDversion % CWData.VersionOffset; }
            set { numberANDversion = value + Version * CWData.VersionOffset; }
        }


        /// <summary>
        /// 武器が効かない
        /// </summary>
        public bool ResistPhysical
        {
            get { return !(resi_physical == 0); }
            set { resi_physical = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 魔法が効かない
        /// </summary>
        public bool ResistMagic
        {
            get { return !(resi_magic == 0); }
            set { resi_magic = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 命（肉体）を持たない
        /// </summary>
        public bool ResistBody
        {
            get { return !(resi_body == 0); }
            set { resi_body = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 精神を持たない
        /// </summary>
        public bool ResistMind
        {
            get { return !(resi_mind == 0); }
            set { resi_mind = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 不浄な存在
        /// </summary>
        public bool WeakUnholy
        {
            get { return !(weak_unholy == 0); }
            set { weak_unholy = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 魔法生物
        /// </summary>
        public bool WeakMagiclife
        {
            get { return !(weak_magiclife == 0); }
            set { weak_magiclife = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 炎に耐性がある
        /// </summary>
        public bool ResistHot
        {
            get { return !(resi_hot == 0); }
            set { resi_hot = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 冷気に耐性がある
        /// </summary>
        public bool ResistCold
        {
            get { return !(resi_cold == 0); }
            set { resi_cold = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 炎に弱い
        /// </summary>
        public bool WeakHot
        {
            get { return !(weak_hot == 0); }
            set { weak_hot = (byte)(value ? 1 : 0); }
        }
        /// <summary>
        /// 冷気に弱い
        /// </summary>
        public bool WeakCold
        {
            get { return !(weak_cold == 0); }
            set { weak_cold = (byte)(value ? 1 : 0); }
        }


        /// <summary>
        /// レベル
        /// </summary>
        public uint Level
        {
            get { return (uint)level; }
            set { level = (ushort)value; }
        }
        /// <summary>
        /// 解説
        /// </summary>
        public string Expression
        {
            get { return expression; }
            set { expression = value; }
        }
        /// <summary>
        /// HP。０以上、最大HP以下に調節される
        /// </summary>
        public int HitPoint
        {
            get { return hitpoint; }
            set
            {
                if (value < 0)
                    hitpoint = 0;
                else if (value > MaxHitPoint)
                    hitpoint = MaxHitPoint;
                else
                    hitpoint = value;
            }
        }
        /// <summary>
        /// 最大ＨＰ
        /// </summary>
        public int MaxHitPoint
        {
            get { return hitpointMax; }
            set { hitpointMax = value; }
        }


        /// <summary>
        /// 麻痺の値。０以上、最大値以下に補正される
        /// </summary>
        public int Paralyse
        {
            get { return state_paralyse; }
            set
            {
                if (value < 0)
                    state_paralyse = 0;
                else if (value > CWData.StateParalyseMax)
                    state_paralyse = CWData.StateParalyseMax;
                else
                    state_paralyse = value;
            }
        }
        /// <summary>
        /// 毒の値。０以上、最大値以下に補正される
        /// </summary>
        public int Poison
        {
            get { return state_poison; }
            set
            {
                if (value < 0)
                    state_poison = 0;
                else if (value > CWData.StatePoisonMax)
                    state_poison = CWData.StatePoisonMax;
                else
                    state_poison = value;
            }
        }


        /// <summary>
        /// キャスト性能としての回避修正
        /// </summary>
        public int AvoidBonus
        {
            get { return add_avoid; }
            set { add_avoid = value; }
        }
        /// <summary>
        /// キャスト性能としての抵抗修正
        /// </summary>
        public int ResistBonus
        {
            get { return add_resist; }
            set { add_resist = value; }
        }
        /// <summary>
        /// キャスト性能としての防御修正
        /// </summary>
        public int ArmorBonus
        {
            get { return add_armor; }
            set { add_armor = value; }
        }


        /// <summary>
        /// 器用度
        /// </summary>
        public int Dex
        {
            get { return char_dex; }
            set { char_dex = value; }
        }
        /// <summary>
        /// 敏捷性
        /// </summary>
        public int Agl
        {
            get { return char_agl; }
            set { char_agl = value; }
        }
        /// <summary>
        /// 知力
        /// </summary>
        public int Int
        {
            get { return char_int; }
            set { char_int = value; }
        }
        /// <summary>
        /// 筋力
        /// </summary>
        public int Str
        {
            get { return char_str; }
            set { char_str = value; }
        }
        /// <summary>
        /// 生命力
        /// </summary>
        public int Vit
        {
            get { return char_vit; }
            set { char_vit = value; }
        }
        /// <summary>
        /// 精神力
        /// </summary>
        public int Min
        {
            get { return char_min; }
            set { char_min = value; }
        }


        /// <summary>
        /// 好戦性（＝-平和性）
        /// </summary>
        public int Aggressive
        {
            get { return char_aggressive; }
            set { char_aggressive = value; }
        }
        /// <summary>
        /// 社交性（＝-内向性）
        /// </summary>
        public int Cheerful
        {
            get { return char_cheerful; }
            set { char_cheerful = value; }
        }
        /// <summary>
        /// 勇猛性（＝-臆病性）
        /// </summary>
        public int Brave
        {
            get { return char_brave; }
            set { char_brave = value; }
        }
        /// <summary>
        /// 慎重性（＝-大胆性）
        /// </summary>
        public int Cautious
        {
            get { return char_cautious; }
            set { char_cautious = value; }
        }
        /// <summary>
        /// 狡猾性（＝-正直性）
        /// </summary>
        public int Trickish
        {
            get { return char_trickish; }
            set { char_trickish = value; }
        }


        /// <summary>
        /// 精神異常種類
        /// </summary>
        public StateMind MindStates
        {
            get { return (StateMind)state_mind; }
            set { state_mind = (byte)value; }
        }
        /// <summary>
        /// 精神異常効果ラウンド数
        /// </summary>
        public uint MindRound
        {
            get { return state_mind_round; }
            set { state_mind_round = value; }
        }


        /// <summary>
        /// 束縛状態効果ラウンド数
        /// </summary>
        public uint BindRound
        {
            get { return state_bind_round; }
            set { state_bind_round = value; }
        }
        /// <summary>
        /// 沈黙状態効果ラウンド数
        /// </summary>
        public uint SilentRound
        {
            get { return state_silent_round; }
            set { state_silent_round = value; }
        }
        /// <summary>
        /// 暴露状態効果ラウンド数
        /// </summary>
        public uint DiscloseRound
        {
            get { return state_disclose_round; }
            set { state_disclose_round = value; }
        }
        /// <summary>
        /// 魔法無効化状態効果ラウンド数
        /// </summary>
        public uint InvalidateMagicRound
        {
            get { return state_magicprotect_round; }
            set { state_magicprotect_round = value; }
        }

        /// <summary>
        /// 行動力修正状態の効果強度
        /// </summary>
        public int MagicPowerBonus
        {
            get { return state_power; }
            set { state_power = value; }
        }
        /// <summary>
        /// 行動力修正状態の効果ラウンド数
        /// </summary>
        public uint MagicPowerRound
        {
            get { return state_power_round; }
            set { state_power_round = value; }
        }
        /// <summary>
        /// 回避修正状態の効果強度
        /// </summary>
        public int MagicAvoidBonus
        {
            get { return state_avoid; }
            set { state_avoid = value; }
        }
        /// <summary>
        /// 回避修正状態の効果ラウンド数
        /// </summary>
        public uint MagicAvoidRound
        {
            get { return state_avoid_round; }
            set { state_avoid_round = value; }
        }
        /// <summary>
        /// 抵抗修正状態の効果強度
        /// </summary>
        public int MagicResistBonus
        {
            get { return state_resist; }
            set { state_resist = value; }
        }
        /// <summary>
        /// 抵抗修正状態の効果ラウンド数
        /// </summary>
        public uint MagicResistRound
        {
            get { return state_resist_round; }
            set { state_resist_round = value; }
        }
        /// <summary>
        /// 防御力修正状態の効果強度
        /// </summary>
        public int MagicArmorBonus
        {
            get { return state_armor; }
            set { state_armor = value; }
        }
        /// <summary>
        /// 防御力修正状態の効果ラウンド数
        /// </summary>
        public uint MagicArmorRound
        {
            get { return state_armor_round; }
            set { state_armor_round = value; }
        }

        /// <summary>
        /// 所持アイテムカード
        /// </summary>
        public List<WidCard> Items
        {
            get
            {
                if (itemList == null)
                    itemList = new List<WidCard>();
                return itemList;
            }
        }
        /// <summary>
        /// 所持スキルカード
        /// </summary>
        public List<WidCard> Skills
        {
            get
            {
                if (skillList == null)
                    skillList = new List<WidCard>();
                return skillList;
            }
        }
        /// <summary>
        /// 所持召喚獣カード
        /// </summary>
        public List<WidCard> Beasts
        {
            get
            {
                if (beastList == null)
                    beastList = new List<WidCard>();
                return beastList;
            }
        }

        /// <summary>
        /// 所有クーポン
        /// </summary>
        public List<CouponData> Coupons
        {
            get
            {
                if (couponList == null)
                    couponList = new List<CouponData>();
                return couponList;
            }
        }

        /// <summary>
        /// 裏返り状態（クーポンと冗長）
        /// </summary>
        public bool IsAbsent
        {
            get { return stateReverse == 1; }
            set { stateReverse = value ? (byte)1 : (byte)0; }
        }
        /// <summary>
        /// 対象消去又はエネミーが逃走かフラグにより消滅を表す。エンジン処理用
        /// </summary>
        public bool IsVanished
        {
            get { return stateVanish == 1; }
            set { stateVanish = value ? (byte)1 : (byte)0; }
        }

        #endregion

        #region 便利のためのプロパティ。リードオンリ
        /// <summary>
        /// 生存 := not離脱 ＆ 生命力＞0 ＆ 麻痺＝0
        /// </summary>
        public bool IsLive
        {
            get { return !IsAbsent && !IsVanished && HitPoint > 0 && Paralyse == 0; }
        }
        /// <summary>
        /// 睡眠 := 生存 ＆ 睡眠
        /// </summary>
        public bool IsSleep
        {
            get { return IsLive && MindRound > 0 && MindStates == StateMind.眠り; }
        }
        /// <summary>
        /// 動ける := 生存 ＆ not呪縛 ＆ not睡眠
        /// </summary>
        public bool IsMovable
        {
            get { return IsLive && BindRound == 0 && (MindRound == 0 || MindStates != StateMind.眠り); }
        }

        #endregion

        #region 追加プロパティ
        /// <summary>
        /// 正確なフルパスのファイル名
        /// </summary>
        string fileName = string.Empty;
        public string FileName
        {
            get
            {
                return fileName;
            }
            set
            {
                fileName = value;
            }
        }

        /// <summary>
        /// EP
        /// </summary>
        public int EP
        {
            get
            {
                CouponData cp = GetCouponData(CWData.STR_EP);
                if (cp != null)
                {
                    return cp.Point;
                }
                else
                {
                    return 0;
                }
            }
            set
            {
                SetCoupon(CWData.STR_EP, value);  
            }
        }

        #endregion

        #region コンストラクタ
        /// <summary>
        /// 組み込みデータや１から作成する時のコンストラクタ
        /// </summary>
        public CharacterData() 
        { 
            image = new ImageData(); 
        }

        /// <summary>
        /// wcpのコンストラクタ(wch一部呼出して直接利用)
        /// </summary>
        /// <param name="file"></param>
        public CharacterData(string file)
        {
            // 拡張子は.wcp
            FileName = file;

            // インデックスの拡張子は .wch
            string iciFile =
            Path.ChangeExtension(file,
                                 FilePath.ExtIndexCharacterData);

            try
            {

                IndexCharacterImage ici = new IndexCharacterImage(iciFile);

                //キャラクタデータに追加、描画
                name = ici.Name;
                image = ici.Image;

                //キャラクタデータ読み込み
                Read2(fileName);

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("待機PCの読み込みに失敗：" + ex.Message);
            }

        }

        #endregion

        #region リードライト

        /// <summary>
        /// ファイルをオープンし、自分のデータを読み込みます。
        /// </summary>
        /// <param name="file">読み込むファイルの名前（フルパス推奨）</param>
        public void Read2(string file)
        {
            using (BinaryReader r = new SjisCBinaryReader(File.OpenRead(file)))
            {
                Read2(r);
            }
        }

        /// <summary>
        /// 全部読む場合
        /// </summary>
        /// <param name="r"></param>
        public override void Read(System.IO.BinaryReader r)
        {
            widid = r.ReadByte();

            image.Read(r);
            name = r.ReadString();

            numberANDversion = r.ReadUInt32();

            resi_physical = r.ReadByte();
            resi_magic = r.ReadByte();
            resi_body = r.ReadByte();
            resi_mind = r.ReadByte();
            weak_unholy = r.ReadByte();
            weak_magiclife = r.ReadByte();
            resi_hot = r.ReadByte();
            resi_cold = r.ReadByte();
            weak_hot = r.ReadByte();
            weak_cold = r.ReadByte();

            level = r.ReadUInt32();
            
            if (Version == 4)
                nazo1_0 = r.ReadInt32();

            expression = ((SjisCBinaryReader)r).ReadStringCRLF();
            hitpoint = r.ReadInt32();
            hitpointMax = r.ReadInt32();

            state_paralyse = r.ReadInt32();
            state_poison = r.ReadInt32();
            add_avoid = r.ReadInt32();
            add_resist = r.ReadInt32();
            add_armor = r.ReadInt32();

            char_dex = r.ReadInt32();
            char_agl = r.ReadInt32();
            char_int = r.ReadInt32();
            char_str = r.ReadInt32();
            char_vit = r.ReadInt32();
            char_min = r.ReadInt32();

            ReadCommon(r);
        }

        /// <summary>
        /// .wch ですでに読み込んでいるimageやnameはファイルポインタを移動して回避
        /// </summary>
        /// <param name="r"></param>
        public void Read2(System.IO.BinaryReader r)
        {
            widid = r.ReadByte();
            
            //image飛ばし
            imageSize = r.ReadInt32();
            if (imageSize != 0)
                r.BaseStream.Seek(imageSize, SeekOrigin.Current);

            //name飛ばし
            nameSize = r.ReadInt32();
            if (nameSize != 0)
                r.BaseStream.Seek(nameSize, SeekOrigin.Current);

            numberANDversion = r.ReadUInt32();

            resi_physical = r.ReadByte();
            resi_magic = r.ReadByte();
            resi_body = r.ReadByte();
            resi_mind = r.ReadByte();
            weak_unholy = r.ReadByte();
            weak_magiclife = r.ReadByte();
            resi_hot = r.ReadByte();
            resi_cold = r.ReadByte();
            weak_hot = r.ReadByte();
            weak_cold = r.ReadByte();

            level = r.ReadUInt32();

            if (Version == 4)
                nazo1_0 = r.ReadInt32();

            expression = ((SjisCBinaryReader)r).ReadStringCRLF();
            hitpoint = r.ReadInt32();
            hitpointMax = r.ReadInt32();

            state_paralyse = r.ReadInt32();
            state_poison = r.ReadInt32();
            add_avoid = r.ReadInt32();
            add_resist = r.ReadInt32();
            add_armor = r.ReadInt32();

            char_dex = r.ReadInt32();
            char_agl = r.ReadInt32();
            char_int = r.ReadInt32();
            char_str = r.ReadInt32();
            char_vit = r.ReadInt32();
            char_min = r.ReadInt32();

            ReadCommon(r);
        }

        private void ReadCommon(System.IO.BinaryReader r)
        {
            char_aggressive = r.ReadInt32();
            char_cheerful = r.ReadInt32();
            char_brave = r.ReadInt32();
            char_cautious = r.ReadInt32();
            char_trickish = r.ReadInt32();

            state_mind = r.ReadByte();
            state_mind_round = r.ReadUInt32();
            state_bind_round = r.ReadUInt32();
            state_silent_round = r.ReadUInt32();
            state_disclose_round = r.ReadUInt32();
            state_magicprotect_round = r.ReadUInt32();
            state_power = r.ReadInt32();
            state_power_round = r.ReadUInt32();
            state_avoid = r.ReadInt32();
            state_avoid_round = r.ReadUInt32();
            state_resist = r.ReadInt32();
            state_resist_round = r.ReadUInt32();
            state_armor = r.ReadInt32();
            state_armor_round = r.ReadUInt32();

            uint count = r.ReadUInt32();
            for (; count > 0; count--)
            {
                WidCard d = new WidCard(WidIDtype.アイテム);
                d.Read(r);
                Items.Add(d);
            }
            count = r.ReadUInt32();
            for (; count > 0; count--)
            {
                WidCard d = new WidCard(WidIDtype.スキル);
                d.Read(r);
                Skills.Add(d);
            }
            count = r.ReadUInt32();
            for (; count > 0; count--)
            {
                WidCard d = new WidCard(WidIDtype.召喚獣);
                d.Read(r);
                Beasts.Add(d);
            }
            count = r.ReadUInt32();
            for (; count > 0; count--)
            {
                CouponData d = new CouponData();
                d.Read(r);
                Coupons.Add(d);
            }
        }

        public override void Write(System.IO.BinaryWriter w)
        {
            w.Write(widid);
            image.Write(w);
            w.Write(name);
            w.Write(numberANDversion);
            w.Write(resi_physical);
            w.Write(resi_magic);
            w.Write(resi_body);
            w.Write(resi_mind);
            w.Write(weak_unholy);
            w.Write(weak_magiclife);
            w.Write(resi_hot);
            w.Write(resi_cold);
            w.Write(weak_hot);
            w.Write(weak_cold);
            w.Write(level);
            if (Version == 4)
                w.Write(nazo1_0);
            ((SjisCBinaryWriter)w).WriteCRLF(expression);
            w.Write(hitpoint);
            w.Write(hitpointMax);
            w.Write(state_paralyse);
            w.Write(state_poison);
            w.Write(add_avoid);
            w.Write(add_resist);
            w.Write(add_armor);
            w.Write(char_dex);
            w.Write(char_agl);
            w.Write(char_int);
            w.Write(char_str);
            w.Write(char_vit);
            w.Write(char_min);
            w.Write(char_aggressive);
            w.Write(char_cheerful);
            w.Write(char_brave);
            w.Write(char_cautious);
            w.Write(char_trickish);
            w.Write(state_mind);
            w.Write(state_mind_round);
            w.Write(state_bind_round);
            w.Write(state_silent_round);
            w.Write(state_disclose_round);
            w.Write(state_magicprotect_round);
            w.Write(state_power);
            w.Write(state_power_round);
            w.Write(state_avoid);
            w.Write(state_avoid_round);
            w.Write(state_resist);
            w.Write(state_resist_round);
            w.Write(state_armor);
            w.Write(state_armor_round);
            w.Write(Items.Count);
            foreach (WidCard d in Items)
                d.Write(w);
            w.Write(Skills.Count);
            foreach (WidCard d in Skills)
                d.Write(w);
            w.Write(Beasts.Count);
            foreach (WidCard d in Beasts)
                d.Write(w);
            w.Write(Coupons.Count);
            foreach (CouponData d in Coupons)
                d.Write(w);
        }

        #endregion

        #region メソッド
        
        /// <summary>
        /// 全快する（永続でない召喚獣は消滅する）
        /// </summary>
        public void RefleshAll()
        {
            //　HP
            this.HitPoint = this.MaxHitPoint;
            
            //　能力変化
            this.MagicArmorBonus = 0;
            this.MagicArmorRound = 0;
            this.MagicAvoidBonus = 0;
            this.MagicAvoidRound = 0;
            this.MagicPowerBonus = 0;
            this.MagicPowerRound = 0;
            this.MagicResistBonus = 0;
            this.MagicResistRound = 0;
            
            //　精神状態
            this.MindStates = StateMind.正常;
            this.MindRound = 0;

            //　魔法による状態変化　効果時間
            this.SilentRound = 0;
            this.InvalidateMagicRound = 0;
            this.DiscloseRound = 0;
            this.BindRound = 0;

            // 中毒解除・麻痺解除　
            this.Poison = 0;
            this.Paralyse = 0;

            // 精神力はスキルの使用回数を回復（技能回復）
            foreach (WidCard skill in this.Skills)
            {
                skill.SkillRemainUseCount = GetSkillMaxUseCount(skill.Level);
            }

            // アクションカード消去
            // 永続召喚獣(付帯能力召喚獣)ではない召喚獣削除
            for (int i = this.Beasts.Count - 1; i >= 0; i--)
            {
                if (!this.Beasts[i].HutaiBeast)
                {
                    this.Beasts.RemoveAt(i);
                }
            }

        }

        /// <summary>
        /// 技能最大使用回数を得る
        /// </summary>
        /// <param name="skillLv">スキルレベル</param>
        /// <returns></returns>
        private uint GetSkillMaxUseCount(uint skillLv)
        {
            int diff = (int)this.Level - (int)skillLv;

            switch (diff)
            {
                case -2:
                    return 2;
                case -1:
                    return 3;
                case 0:
                    return 5;
                case 1:
                    return 7;
                case 2:
                    return 8;
                default:
                    if (diff > 0)
                    {
                        return 9;
                    }
                    else
                    {
                        return 1;
                    }
            }

        }

        /// <summary>
        /// 所持確定していない手持ちカードの削除
        /// </summary>
        public void DeleteNonFixHandCards()
        {
            for (int i = this.Skills.Count - 1; i >= 0; i--)
            {
                if (!this.Skills[i].FixPossession)
                {
                    this.Skills.RemoveAt(i);
                }
            }

            for (int i = this.Items.Count - 1; i >= 0; i--)
            {
                if (!this.Items[i].FixPossession)
                {
                    this.Items.RemoveAt(i);
                }

            }

            for (int i = this.Beasts.Count - 1; i >= 0; i--)
            {
                if (!this.Beasts[i].FixPossession)
                {
                    this.Beasts.RemoveAt(i);
                }

            }
        }

        /// <summary>
        /// 番号クーポン削除
        /// </summary>
        public void DeleteNumberCoupons()
        {
            for (int i = 0; i < 6; i++)
            {
                RemoveCouponData(CWData.CouponEnumerateStr[i]);
                RemoveCouponData(CWData.CouponEnumerateOld[i]);
            }
        }

        /// <summary>
        /// 時限クーポン削除
        /// </summary>
        public void DeleteTempCoupons()
        {
            //時限クーポン（「：」「；」クーポン）削除
            for (int i = this.Coupons.Count - 1; i >= 0; i--)
            {
                if (this.Coupons[i].Name.StartsWith("：") || this.Coupons[i].Name.StartsWith("；"))
                {
                    this.Coupons.RemoveAt(i);
                } 

            }

        }

        /// <summary>
        /// ~クーポン復帰
        /// </summary>
        public void ReturnTildeCoupons()
        {
            //~クーポンを~なしに
            for (int i = 0 ;i < this.Coupons.Count; i ++)
            {
                if (this.Coupons[i].Name.StartsWith("~"))
                {
                    this.Coupons[i].Name = this.Coupons[i].Name.TrimStart('~');
                }

            }

        }

        /// <summary>
        /// HandCard の最大所持枚数を取得する
        /// </summary>
        /// <param name="tefuda"></param>
        /// <returns></returns>
        public uint GetMaxHandCard(手札種類 tefuda)
        {

            uint maxCardNum = 0;

            switch (tefuda)
            {
                case 手札種類.所持スキル:
                    maxCardNum = (this.Level + 1) / 2 + 2;
                    break;
                case 手札種類.所持アイテム:
                    maxCardNum = (this.Level + 1) / 2 + 2;
                    break;
                case 手札種類.所有召喚獣:
                    maxCardNum = (this.Level + 5) / 4;
                    break;
                default:
                    PartyMaker.CWIErrorMessage("ありえない手札種類です。（CharacterData）");
                    return (uint)0;
            }

            // 最大枚数
            if (maxCardNum > 10)
            {
                maxCardNum = 10;
            }

            return maxCardNum;
        }

        /// <summary>
        /// 親になれるか判定
        /// </summary>
        /// <returns></returns>
        public bool CanMakeChild()
        {
            Age age = GetAge();
            foreach (CouponData cp in this.Coupons)
            {
                if (cp != null)
                {
                    if (cp.Name.StartsWith(CWData.STR_EP))
                    {
                        switch (age)
                        {
                            case Age.Old:
                                if (cp.Point >= 20)
                                {
                                    return true;
                                }
                                else
                                {
                                    return false;
                                }

                            case Age.Adult:
                                if (cp.Point >= 30)
                                {
                                    return true;
                                }
                                else
                                {
                                    return false;
                                }

                            case Age.Young:
                                if (cp.Point >= 40)
                                {
                                    return true;
                                }
                                else
                                {
                                    return false;
                                }
                            case Age.Child:
                                return false;
                            default:
                                // Age未定
                                return false;
                        }
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// Ageを取得
        /// </summary>
        /// <returns></returns>
        public Age GetAge()
        {
            string s;
            foreach (CouponData cp in this.Coupons)
            {
                if (cp != null)
                {
                    s = cp.Name;
                    if (s.Equals(CWData.STR_CHILD))
                    {
                        return Age.Child;
                    }
                    else if (s.Equals(CWData.STR_YOUNG))
                    {
                        return Age.Young;
                    }
                    else if (s.Equals(CWData.STR_ADULT))
                    {
                        return Age.Adult;
                    }
                    else if (s.Equals(CWData.STR_OLD))
                    {
                        return Age.Old;
                    }　
                }
            }

            return Age.Common;
        }

        /// <summary>
        /// Sexを取得
        /// </summary>
        /// <returns></returns>
        public Sex GetSex()
        {
            string s;
            foreach (CouponData cp in this.Coupons)
            {
                if (cp != null)
                {
                    s = cp.Name;
                    if (s.Equals(CWData.STR_MALE))
                    {
                        return Sex.Male;
                    }
                    else if (s.Equals(CWData.STR_FEMALE))
                    {
                        return Sex.Female;
                    }
                }
            }

            return Sex.Common;
        }

        /// <summary>
        /// クーポン取得
        /// </summary>
        /// <param name="couponName">クーポン名</param>
        /// <returns>クーポン</returns>
        public CouponData GetCouponData(string couponName)
        {
            foreach (CouponData cp in this.Coupons)
            {
                if(cp.Name.Equals(couponName))
                {
                    return cp;
                }
            }
            return null;
        }

        /// <summary>
        /// クーポン設定
        /// </summary>
        /// <param name="couponName">クーポン名</param>
        /// <param name="couponPoint">点数</param>
        public void SetCoupon(string couponName, int couponPoint)
        {
            foreach (CouponData cp in this.Coupons)
            {
                if (cp.Name.Equals(couponName))
                {
                    cp.Point = couponPoint;
                    return;
                }
            }

            CouponData cpp = new CouponData();
            cpp.Name = couponName;
            cpp.Point = couponPoint;
            this.Coupons.Add(cpp);
        }

        /// <summary>
        /// クーポン削除
        /// </summary>
        /// <param name="couponName">クーポン名</param>
        public void RemoveCoupon(string couponName)
        {
            for(int i = this.Coupons.Count - 1;i >=0;i--)
            {
                CouponData cp = this.Coupons[i];
                if (cp.Name.Equals(couponName))
                {
                    this.Coupons.RemoveAt(i);
                    return;
                }
            }
        }

        /// <summary>
        /// ＠Ｇのつくクーポンを取得
        /// </summary>
        /// <returns></returns>
        public string GetGeneString()
        {
            string s;
            foreach (CouponData cp in this.Coupons)
            {
                if (cp != null)
                {
                    s = cp.Name;
                    if (Regex.IsMatch(s, CWData.GENE_HEADER))
                    {
                        return s;
                    }
                }
            }

            return CWData.GENE_HEADER + "0000000000";
        }

        /// <summary>
        /// クーポンの存在確認
        /// </summary>
        /// <param name="coupString">クーポン名</param>
        /// <returns></returns>
        public bool CheckCoupon(string coupString)
        {
            return FindCouponData(coupString) != null;
        }

        /// <summary>
        /// キャラが持つクーポン名nameのクーポンデータを探して返す。
        /// </summary>
        /// <param name="name">探すクーポン名</param>
        /// <returns>見つかったCouponData。見つからなければnullを返す</returns>
        private CouponData FindCouponData(string name)
        {
            foreach (CouponData cou in this.Coupons)
            {
                if (cou.Name == name)
                    return cou;
            }
            return null;
        }

        /// <summary>
        /// キャラが持つクーポン名nameのクーポンデータを探して削除
        /// なければ何もしない
        /// </summary>
        /// <param name="name">探すクーポン名</param>
        public void RemoveCouponData(string name)
        {
            CouponData cp = FindCouponData(name);
            if (cp != null)
                this.Coupons.Remove(cp);
        }
        #endregion

        //class CharacterData
    }
}
