using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using CWIDatas;

namespace CWI
{
    public class GeneData
    {
        private BitVector32 field = new BitVector32(0);
        public BitVector32 Field
        {
            set
            {
                field = value;
            }

            get
            {
                return field;
            }
        }

        public static readonly int BIT1 = 1;
        public static readonly int BIT2 = 2;
        public static readonly int BIT3 = 4;
        public static readonly int BIT4 = 8;
        public static readonly int BIT5 = 16;
        public static readonly int BIT6 = 32;
        public static readonly int BIT7 = 64;
        public static readonly int BIT8 = 128;
        public static readonly int BIT9 = 256;
        public static readonly int BIT10 = 512;
        public static readonly int BIT11 = 1024;

        public GeneData()
        {

        }

        public GeneData(int initValue)
        {
            field = new BitVector32(initValue);
        }

        public GeneData(string geneString)
        {
            field = new BitVector32(0);
            SetGeneString(geneString);
        }

        public void SetGeneString(string geneString)
        {
            try
            {
                if (geneString.StartsWith(CWData.GENE_HEADER)
                    && geneString.Trim(CWData.GENE_HEADER.ToCharArray()).Length == 10)
                {
                    field = new BitVector32(Convert.ToInt32(Util1.StringReverse(geneString.Trim(CWData.GENE_HEADER.ToCharArray())), 2));
                }
                else
                {
                    // `񂪂Ȃ
                    field = new BitVector32(0);
                }
            }
            catch (Exception)
            {
                PartyMaker.CWIErrorMessage("`qN[|ԈĂ܂(GeneData)");
            }
        }

        public string GetGeneString()
        {
            return CWData.GENE_HEADER + Util1.StringReverse(Convert.ToString(field.Data, 2).PadLeft(10, '0'));
        }

        public void RotateRight()
        {
            if (field[BIT10] == true)
            {
                field = new BitVector32((field.Data * 2) % BIT11 + BIT1);
            }
            else
            {
                field = new BitVector32(field.Data * 2);
            }
        }

        public void RotateLeft()
        {
            if (field[BIT1] == true)
            {
                field = new BitVector32((field.Data / 2) + BIT10);
            }
            else
            {
                field = new BitVector32(field.Data / 2);
            }
        }

        public GeneData XorGenes(GeneData pairGene)
        {
            int thisGeneData = this.field.Data;
            int pairGeneData = pairGene.field.Data;
            return new GeneData(thisGeneData ^ pairGeneData);
        }

        /// <summary>
        /// ʂ̐ȅꍇɃ_Ńrbg𗧂Ă
        /// </summary>
        public void SetRandomBit()
        {

            // rbgĂƂ
            int pos = DiceRoll.RandomInt(10);
            switch (pos)
            {
                case 0:
                    field[BIT1] = true;
                    break;
                case 1:
                    field[BIT2] = true;
                    break;
                case 2:
                    field[BIT3] = true;
                    break;
                case 3:
                    field[BIT4] = true;
                    break;
                case 4:
                    field[BIT5] = true;
                    break;
                case 5:
                    field[BIT6] = true;
                    break;
                case 6:
                    field[BIT7] = true;
                    break;
                case 7:
                    field[BIT8] = true;
                    break;
                case 8:
                    field[BIT9] = true;
                    break;
                case 9:
                    field[BIT10] = true;
                    break;
            }

        }

        /// <summary>
        /// rbg]
        /// </summary>
        public void ReverseBit()
        {
            // 1111111111 XORĔ]
            field = new BitVector32(field.Data ^ (BIT11 - 1));
        }

        /// <summary>
        /// CWIDatas.TYPE_(f^) ̃tO𔽓]
        /// </summary>
        /// <param name="typeString"></param>
        public void TypeFlagReverse(string typeString)
        {
            if (typeString == CWData.TYPE_NORMAL)
            {
                field[BIT1] = !field[BIT1];
            }
            else if (typeString == CWData.TYPE_ALLMIGHTY)
            {
                field[BIT2] = !field[BIT2];
            } 
            else if (typeString == CWData.TYPE_RESOURCEFUL)
            {
                field[BIT5] = !field[BIT5];
            } 
            else if (typeString == CWData.TYPE_BRAVE)
            {
                field[BIT3] = !field[BIT3];
            } 
            else if (typeString == CWData.TYPE_DARING)
            {
                field[BIT4] = !field[BIT4];
            } 
            else if (typeString == CWData.TYPE_TACTIC)
            {
                field[BIT6] = !field[BIT6];
            }
            else if (typeString == CWData.TYPE_MATCHLESS)
            {
                field[BIT9] = !field[BIT9];
            }
            else if (typeString == CWData.TYPE_GENIUS)
            {
                field[BIT10] = !field[BIT10];
            }
            else if (typeString == CWData.TYPE_BRILLIANT)
            {
                field[BIT8] = !field[BIT8];
            }
            else if (typeString == CWData.TYPE_ORDINARY)
            {
                field[BIT7] = !field[BIT7];
            } 
            
        }

        /// <summary>
        /// CWIDatas.TYPE_(f^) ̃tO𗧂Ă
        /// </summary>
        /// <param name="typeString"></param>
        public void SetTypeFlag(string typeString)
        {
            if (typeString == CWData.TYPE_NORMAL)
            {
                field[BIT1] = true;
            }
            else if (typeString == CWData.TYPE_ALLMIGHTY)
            {
                field[BIT2] = true;
            }
            else if (typeString == CWData.TYPE_RESOURCEFUL)
            {
                field[BIT5] = true;
            }
            else if (typeString == CWData.TYPE_BRAVE)
            {
                field[BIT3] = true;
            }
            else if (typeString == CWData.TYPE_DARING)
            {
                field[BIT4] = true;
            }
            else if (typeString == CWData.TYPE_TACTIC)
            {
                field[BIT6] = true;
            }
            else if (typeString == CWData.TYPE_MATCHLESS)
            {
                field[BIT9] = true;
            }
            else if (typeString == CWData.TYPE_GENIUS)
            {
                field[BIT10] = true;
            }
            else if (typeString == CWData.TYPE_BRILLIANT)
            {
                field[BIT8] = true;
            }
            else if (typeString == CWData.TYPE_ORDINARY)
            {
                field[BIT7] = true;
            }

        }

        /// <summary>
        /// rbgĂ鐔𐔂
        /// </summary>
        /// <returns></returns>
        public int CountBits()
        {
            int count = 0;

            for (int mask = 1; mask < BIT11; mask *= 2)
            {
                if (field[mask])
                {
                    count++;
                }
            }

            return count;
        }


        /// <summary>
        /// {^+B̃rbgĂ鐔𐔂
        /// </summary>
        /// <returns></returns>
        public int CountBaseBits()
        {
            int count = 0;

            for (int mask = 1; mask < BIT8; mask *= 2)
            {
                if (field[mask])
                {
                    count++;
                }
            }

            return count;
        }
    }
}
