﻿using System;
using System.Drawing.Imaging;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using System.Windows.Forms;
using CWIDatas;
using CWI.Dialog;
using CWI.Card;
using System.Diagnostics;

namespace CWI
{
    /// <summary>
    /// ソート種類
    /// </summary>
    public enum ソート種類
    {
        アイテム_スキル_召喚獣 = 0,
        スキル_アイテム_召喚獣,
        アイテム_召喚獣_スキル,
        スキル_召喚獣_アイテム,
        召喚獣_アイテム_スキル,
        召喚獣_スキル_アイテム
    }

    /// <summary>
    /// ソート対象
    /// </summary>
    public enum ソート対象
    {
        カード置場 = 0,
        荷物袋
    }

    /// <summary>
    /// 性別（不明はCommon扱いしておきます）
    /// </summary>
    public enum Sex 
    {
        Male, Female, Common
    }

    /// <summary>
    /// 年代（不明はCommon扱いしておきます）
    /// </summary>
    public enum Age 
    {
        Child, Young, Adult, Old, Common
    }

    /// <summary>
    /// PCソート種類
    /// </summary>
    public enum PCソート種類
    {
        名前昇順 = 0,
        名前降順,
        レベル昇順,
        レベル降順,
        カード名,
        クーポン

    }

    /// <summary>
    /// Description of InnData
    /// 宿の環境（オプション）設定情報であり、宿クラス
    /// Environment.wyd
    /// </summary>
    public class InnData : DataWithIO
    {
        public static readonly uint MaxCardSpeed = 8;
        public static readonly uint MaxViewSpeed = 8;
        public static readonly uint MaxMessageSpeed = 8;
        public static readonly uint DefCardSpeed = 4;
        public static readonly uint DefViewSpeed = 4;
        public static readonly uint DefMessageSpeed = 4;
        public static readonly uint MaxMessageBoardLight = 6;
        public static readonly uint DefMessageBoardLight = 3;
        public static readonly uint DefYadoMoney = 4000;

        public const string DATAVERSION_08 = "DATAVERSION_08";
        public const string DATAVERSION_10 = "DATAVERSION_10";
        public const string DATAVERSION_11 = "DATAVERSION_11";
        public const string DATAVERSION_12 = "DATAVERSION_12";

        public enum BackGroundAnimationEnum : byte
        { なし, スレッド, フェード, シェーブ }

        public enum DispScenarioEnum : byte
        { 隠蔽終了済非表示, 隠蔽非表示, 全て表示, 適応レベルのみ }

        public enum DispScenarioEnumV12 : byte
        { 隠蔽終了済非表示, 適応レベルのみ, 隠蔽非表示, 全て表示 }

        #region プロパティ

        /// <summary>
        /// バージョン
        /// </summary>
        public string Version
        {
            get { return versionString; }
            set { versionString = value; }
        }
        string versionString;

        /// <summary>
        /// デバッグ宿か
        /// </summary>
        public bool IsDebugInn
        {
            get { return (game_or_debug == 2); }

            set
            {
                if (value)
                {
                    game_or_debug = 2;
                }
                else
                {
                    game_or_debug = 1;
                }
            }
        }
        byte game_or_debug;

        /// <summary>
        /// デバッグ宿に設定
        /// </summary>
        public void SetDebugInn()
        {
            game_or_debug = 2;
        }

        /// <summary>
        /// 通常宿に設定
        /// </summary>
        public void SetNormalInn()
        {
            game_or_debug = 1;
        }

        /// <summary>
        /// オプションのカード動画ウェイト。０で最速
        /// </summary>
        public uint CardSpeed
        {
            get { return speed_card; }
            set
            {
                if (value > MaxCardSpeed)
                    speed_card = MaxCardSpeed;
                else if (value < 0)
                    speed_card = 0;
                else
                    speed_card = value;
            }
        }
        uint speed_card;

        /// <summary>
        /// オプションのカード動画ウェイト。０で最速
        /// </summary>
        public uint CardSpeedBattle
        {
            get { return speed_card_battle; }
            set
            {
                if (value > MaxCardSpeed)
                    speed_card_battle = MaxCardSpeed;
                else if (value < 0)
                    speed_card_battle = 0;
                else
                    speed_card_battle = value;
            }
        }
        uint speed_card_battle;

        /// <summary>
        /// オプションの背景表示動画ウェイト。０で最速
        /// </summary>
        public uint ViewSpeed
        {
            get { return speed_view; }
            set
            {
                if (value > MaxViewSpeed)
                    speed_view = MaxViewSpeed;
                else if (value < 0)
                    speed_view = 0;
                else
                    speed_view = value;
            }
        }
        uint speed_view;

        /// <summary>
        /// オプションのメッセージ表示動画ウェイト。０で最速
        /// </summary>
        public uint MessageSpeed
        {
            get { return speed_message; }
            set
            {
                if (value > MaxMessageSpeed)
                    speed_message = MaxMessageSpeed;
                else if (value < 0)
                    speed_message = 0;
                else
                    speed_message = value;
            }
        }
        uint speed_message;

        public bool BgmON
        {
            get { return (on_bgm == 1); }
            set { on_bgm = (byte)(value ? 1 : 0); }
        }
        byte on_bgm;

        public bool SoundON
        {
            get { return (on_sound == 1); }
            set { on_sound = (byte)(value ? 1 : 0); }
        }
        byte on_sound;

        public bool ReduceCardCorrectionON
        {
            get { return (on_smallcard == 1); }
            set { on_smallcard = (byte)(value ? 1 : 0); }
        }
        byte on_smallcard;

        public bool MagnifyCardCorrectionON
        {
            get { return (on_largecard == 1); }
            set { on_largecard = (byte)(value ? 1 : 0); }
        }
        byte on_largecard;

        #region その他
        public bool AutoLoadON
        {
            get { return (on_autoload == 1); }
            set { on_autoload = (byte)(value ? 1 : 0); }
        }
        byte on_autoload;

        public bool BackGroundCancelON
        {
            get { return (on_cancellback == 1); }
            set { on_cancellback = (byte)(value ? 1 : 0); }
        }
        byte on_cancellback;

        public bool MoneyEffectON
        {
            get { return (on_moneyeffect == 1); }
            set { on_moneyeffect = (byte)(value ? 1 : 0); }
        }
        byte on_moneyeffect;

        public bool SkipEventByRightClickON
        {
            get { return (on_eventjump == 1); }
            set { on_eventjump = (byte)(value ? 1 : 0); }
        }
        byte on_eventjump;

        public bool KeepMaxLevelON
        {
            get { return (on_keepmaxlevel == 1); }
            set { on_keepmaxlevel = (byte)(value ? 1 : 0); }
        }
        byte on_keepmaxlevel;

        /// <summary>
        /// 選択モードでの背景効果
        /// </summary>
        public bool BackGroundEffect
        {
            get { return (backGroundEffect == 1); }
            set { backGroundEffect = (byte)(value ? 1 : 0); }
        }
        byte backGroundEffect = 0;
        #endregion

        /// <summary>
        /// シナリオ表示選択
        /// ～VERSION11
        /// 隠蔽・終了済を表示しない：０、隠蔽を表示しない：１
        /// 全て：２、レベルのみ３
        /// VERSION12
        /// 隠蔽・終了済を表示しない：０、レベルのみ１、
        /// 隠蔽を表示しない：２、全て：３
        /// </summary>
        public DispScenarioEnum DispScenarioEffect
        {
            get { return (DispScenarioEnum)on_backnet; }
            set { on_backnet = (byte)value; }
        }
        public DispScenarioEnumV12 DispScenarioEffectV12
        {
            get { return (DispScenarioEnumV12)on_backnet; }
            set { on_backnet = (byte)value; }
        }
        byte on_backnet;

        /// <summary>
        /// メッセージボードの明るさ。０が最も暗い
        /// </summary>
        public uint MessageLight
        {
            get { return messageLight; }
            set
            {
                if (value > MaxMessageBoardLight)
                    messageLight = MaxMessageBoardLight;
                else if (value < 0)
                    messageLight = 0;
                else
                    messageLight = value;
            }
        }
        uint messageLight;

        /// <summary>
        /// 装飾フォントフラグ
        /// </summary>
        public bool DecoFontON
        {
            get { return (on_decoFont == 1); }
            set { on_decoFont = (byte)(value ? 1 : 0); }
        }
        byte on_decoFont;

        /// <summary>
        /// 背景アニメーション
        /// </summary>
        public BackGroundAnimationEnum BackGroundAnimation
        {
            get { return (BackGroundAnimationEnum)backanime; }
            set { backanime = (byte)value; }
        }
        byte backanime;

        /// <summary>
        /// 終了済みシナリオリスト。未割り当てなら割り当てを行う
        /// </summary>
        public StringCollection FinishScenarios
        {
            get
            { 
                return finishScenarioStringList;
            }
        }
        StringCollection finishScenarioStringList = new StringCollection();

        /// <summary>
        /// 進行中のシナリオリスト。未割り当てなら割り当てを行う
        /// </summary>
        public StringCollection ProgressScenarios
        {
            get
            { 
                return progressScenarioStringList;
            }
        }
        StringCollection progressScenarioStringList = new StringCollection();

        /// <summary>
        /// ゴシップリスト。未割り当てなら割り当てを行う
        /// </summary>
        public StringCollection Gossips
        {
            get
            {  
                return gossipStringList;
            }
        }
        StringCollection gossipStringList = new StringCollection();

        /// <summary>
        /// 宿の実在ファイルカードリスト。未割り当てなら割り当てを行う
        /// </summary>
        public List<RenamedCardData> BankBoxCards
        {
            get
            {   
                return yadoRenamedCardList;
            }
            set
            {
                yadoRenamedCardList = value;
            }
        }
        List<RenamedCardData> yadoRenamedCardList = new List<RenamedCardData>();

        /// <summary>
        /// 宿のインデックスカードリスト。未割り当てなら割り当てを行う
        /// </summary>
        public List<CardIndexData> CardIndexDatas
        {
            get
            {
                return yadoIndexedCardList;
            }
            set
            {
                yadoIndexedCardList = value;
            }
        }
        List<CardIndexData> yadoIndexedCardList = new List<CardIndexData>();

        /// <summary>
        /// 宿金額
        /// </summary>
        public uint YadoMoney
        {
            get { return yadoMoney; }
            set
            {
                if (value > CWData.MaxMoneyValue)
                    yadoMoney = CWData.MaxMoneyValue;
                else if (value < 0)
                    yadoMoney = 0;
                else yadoMoney = value;
            }
        }
        uint yadoMoney;

        /// <summary>
        /// オートロード対象パーティー
        /// </summary>
        public string AutoLoadParty
        {
            get { return loadPartyName; }
            set { loadPartyName = value; }
        }
        string loadPartyName;

        #endregion

        #region プロパティ（ＰＣ関連）

        /// <summary>
        /// 待機PC
        /// </summary>
        List<CharacterData> characters = new List<CharacterData>();
        public List<CharacterData> Characters
        {
            get
            {
                return characters;
            }
            set
            {
                characters = value;
            }
        }

        /// <summary>
        /// パーティー
        /// </summary>
        List<PartyData> parties = new List<PartyData>();
        public List<PartyData> Parties
        {
            get
            {
                return parties;
            }
            set
            {
                parties = value;
            }

        }

        #endregion

        #region プロパティ（カレントパーティー関連）
        /// <summary>
        /// カレントパーティー
        /// </summary>
        PartyData currentParty = null;
        public PartyData CurrentParty
        {
            get
            {
                return currentParty;
            }
            set
            {
                currentParty = value;
            }
        }
        #endregion

        #region プロパティ（宿関連）
        /// <summary>
        /// 名前
        /// </summary>
        string name = null;
        public string Name
        {
            get
            {
                return name;
            }
            set
            {
                // Name
                name = value;
            }
        }

        /// <summary>
        /// フォルダ
        /// </summary>
        string folder = null;
        public string Folder
        {
            get
            {
                return folder;
            }
            set
            {
                folder = value;

                // EnvironmentFile
                // このファイルだけは直接宿書き込み（~宿ではない）。~にはファイルが出来ない
                EnvironmentFile = FilePath.GetYadoEnvironmentFile(folder);

                // Read 
                try
                {
                    Read(EnvironmentFile);

                }
                catch (Exception ex)
                {
                    PartyMaker.CWIErrorMessage("宿環境ファイルの読み込みに失敗：" + ex.Message);
                }

                // TempFolder
                tempFolder = FilePath.GetYadoTempFolder(name);

                // BackupFolder
                backupFolder = FilePath.GetYadoBackupFolder(name);

            }

        }

        /// <summary>
        /// 環境ファイル
        /// </summary>
        string environmentFile = null;
        public string EnvironmentFile
        {
            get
            {
                return environmentFile;
            }
            set
            {
                environmentFile = value;
            }
        }

        /// <summary>
        /// ~宿フォルダ
        /// </summary>
        string tempFolder = null;
        public string TempFolder
        {
            get
            {
                return tempFolder;
            }
            set
            {
                tempFolder = value;
            }

        }

        /// <summary>
        /// _宿フォルダ（バックアップ）
        /// </summary>
        string backupFolder = null;
        public string BackupFolder
        {
            get
            {
                return backupFolder;
            }
            set
            {
                backupFolder = value;
            }

        }

        /// <summary>
        /// ~宿フォルダ
        /// </summary>
        public bool AutoLoad
        {
            get
            {
                return AutoLoadON && (AutoLoadParty != string.Empty);
            }
        }

        /// <summary>
        /// WidCard
        /// </summary>
        List<WidCard> widCards = null;
        bool readWidCards = false; 
        public List<WidCard> WidCards
        {
            get
            {
                if (!readWidCards)
                {
                    widCards = new List<WidCard>();

                    WidCard wc;
                    foreach (CardIndexData cid in CardIndexDatas)
                    {
                        switch (cid.CardKind)
                        {
                            case CardIndexData.Kind.item:
                                wc = new WidCard(WidIDtype.アイテム);
                                break;
                            case CardIndexData.Kind.skill:
                                wc = new WidCard(WidIDtype.スキル);
                                break;
                            case CardIndexData.Kind.beast:
                                wc = new WidCard(WidIDtype.召喚獣);
                                break;
                            default:
                                PartyMaker.CWIErrorMessage("このような種類のカードはありません。");
                                return null;
                        }

                        try
                        {
                            wc.Read(FilePath.GetYadoWidFile(TempFolder, cid.RenamedName));
                            widCards.Add(wc);
                        }
                        catch
                        {

                            PartyMaker.CWIErrorMessage("手札カードの読み込みに失敗：" + cid.RenamedName + " " + cid.CardKind.ToString());
                            return null;
                        }
                    }

                    readWidCards = true;
                }

                return widCards;
            }
            set
            {
                widCards = value;
            }
        }

        /// <summary>
        /// 他宿
        /// </summary>
        InnData otherInn = null;
        public InnData OtherInn
        {
            get
            {
                return otherInn;
            }
            set
            {
                otherInn = value;
            }
        }

        #endregion

        #region コンストラクタ
        public InnData(string name, bool isNewInn, bool isDebugYado)
        {
            // 宿は名前に対して一意に決まるので
            // nameでもそんなに危なくないが、出来れば下のYadoSelectDataで。
            // 各種操作
            this.IsDebugInn = isDebugYado;

            this.Name = name;

            if (isNewInn)
            {
                //FolderとするとEnvironmentFileを読みにいってしまう・・・
                this.folder = FilePath.GetSystemYadoFolder() + "\\" + name;
                
                // EnvironmentFile
                // このファイルだけは直接宿書き込み（~宿ではない）。
                // ~にはファイルが出来ない
                EnvironmentFile = FilePath.GetYadoEnvironmentFile(folder);

                // TempFolder
                this.tempFolder = FilePath.GetYadoTempFolder(name);

                // BackupFolder
                this.backupFolder = FilePath.GetYadoBackupFolder(name);

                this.Version = DATAVERSION_10;
                
                this.CardSpeed = DefCardSpeed;
                this.ViewSpeed = DefViewSpeed;
                this.MessageSpeed = DefMessageSpeed;
                
                // 音響効果
                this.BgmON = true;
                this.SoundON = true;
                
                this.ReduceCardCorrectionON = true;
                this.MagnifyCardCorrectionON = true;
                
                // その他
                this.AutoLoadON = true;
                this.BackGroundCancelON = true;
                this.MoneyEffectON = true;
                this.SkipEventByRightClickON = true;
                this.KeepMaxLevelON = true;
                this.BackGroundEffect = true;

                this.DispScenarioEffect = DispScenarioEnum.隠蔽終了済非表示;

                this.MessageLight = DefMessageBoardLight;
                this.DecoFontON = false;

                this.BackGroundAnimation = BackGroundAnimationEnum.スレッド;

                finishScenarioStringList = new StringCollection();

                progressScenarioStringList = new StringCollection();

                gossipStringList = new StringCollection();

                BankBoxCards = new List<RenamedCardData>();
                CardIndexDatas = new List<CardIndexData>();

                this.YadoMoney = DefYadoMoney;
                this.AutoLoadParty = string.Empty;

            }
            else
            {
                this.Folder = FilePath.GetSystemYadoFolder() + "\\" + name;
            }
        }

        public InnData(string name)
        {
            // 宿は名前に対して一意に決まるので
            // nameでもそんなに危なくないが、出来れば下のYadoSelectDataで。
            // 各種操作
            SetNormalInn();

            this.Name = name;

            this.Folder = FilePath.GetSystemYadoFolder() + "\\" + name;
        }

        public InnData(YadoSelectData selectedYado)
        {
            this.IsDebugInn = selectedYado.IsDebugYado;

            this.Name = selectedYado.Name;
            this.Folder = selectedYado.Folder;
        }
        #endregion

        #region リードライト
        public override void Read(System.IO.BinaryReader r)
        {
            versionString = r.ReadString();
            game_or_debug = r.ReadByte();

            if (versionString == DATAVERSION_12)
            {
                // （画面）速度調整
                byte dummy = r.ReadByte();
                speed_card = (uint)dummy;
                dummy = r.ReadByte();
                speed_view = (uint)dummy;
                dummy = r.ReadByte();
                speed_message = (uint)dummy;
                dummy = r.ReadByte();
                speed_card_battle = (uint)dummy;

                // カードの画像修正
                // （画面）縮小カード
                on_smallcard = r.ReadByte();
                // （画面）拡大カード
                on_largecard = r.ReadByte();

                // その他（システム・画面）
                // （システム）パーティのオートロード
                on_autoload = r.ReadByte();
                // （システム）背景クリックによるキャンセル
                on_cancellback = r.ReadByte();
                // （画面）所持金変更時の視覚効果
                on_moneyeffect = r.ReadByte();
                // （システム）背景クリックによるイベント間のジャンプ
                on_eventjump = r.ReadByte();
                // （システム）レベルを最大値に維持
                on_keepmaxlevel = r.ReadByte();
                // （画面）選択モードでの背景効果
                backGroundEffect = r.ReadByte();

                // （システム）貼紙の表示
                //隠蔽・終了済を表示しない：０、レベルのみ１、
                //隠蔽を表示しない：２、全て：３
                on_backnet = r.ReadByte();

                // （画面）メッセージボード
                messageLight = (uint)r.ReadByte();
                on_decoFont = r.ReadByte();

                // （画面）背景の切り替え方式
                backanime = r.ReadByte();

                //（終了・進行中）シナリオリスト、宿ゴシップ
                finishScenarioStringList = ((SjisCBinaryReader)r).Read80StringCollection();
                progressScenarioStringList = ((SjisCBinaryReader)r).Read80StringCollection();
                gossipStringList = ((SjisCBinaryReader)r).Read80StringCollection();
            }
            else
            {
                // （画面）速度調整
                speed_card = r.ReadUInt32();
                speed_view = r.ReadUInt32();
                speed_message = r.ReadUInt32();


                // 音響効果
                // （オーディオ）BGMを鳴らすチェック
                on_bgm = r.ReadByte();
                // （オーディオ）効果音を鳴らすチェック
                on_sound = r.ReadByte();

                // カードの画像修正
                // （画面）縮小カード
                on_smallcard = r.ReadByte();
                // （画面）拡大カード
                on_largecard = r.ReadByte();

                // その他（システム・画面）
                // （システム）パーティのオートロード
                on_autoload = r.ReadByte();
                // （システム）背景クリックによるキャンセル
                on_cancellback = r.ReadByte();

                if (versionString == DATAVERSION_11 ||
                    versionString == DATAVERSION_10)
                {
                    // （画面）所持金変更時の視覚効果
                    on_moneyeffect = r.ReadByte();
                    // （システム）背景クリックによるイベント間のジャンプ
                    on_eventjump = r.ReadByte();
                    // （システム）レベルを最大値に維持
                    on_keepmaxlevel = r.ReadByte();
                }

                if (versionString == DATAVERSION_11)
                {
                    // （画面）選択モードでの背景効果
                    backGroundEffect = r.ReadByte();
                }

                // （システム）貼紙の表示
                //隠蔽・終了済を表示しない：０、隠蔽を表示しない：１
                //、全て：２
                //DATAVERSION_10以上では、レベルのみ３
                on_backnet = r.ReadByte();

                // （画面）メッセージボード
                messageLight = r.ReadUInt32();
                on_decoFont = r.ReadByte();

                // （画面）背景の切り替え方式
                backanime = r.ReadByte();

                //if (versionString != DATAVERSION_08)
                //{
                //（終了・進行中）シナリオリスト、宿ゴシップ
                finishScenarioStringList = ((SjisCBinaryReader)r).ReadStringCollection();
                progressScenarioStringList = ((SjisCBinaryReader)r).ReadStringCollection();
                gossipStringList = ((SjisCBinaryReader)r).ReadStringCollection();
                //}
            }

            // カード情報
            uint count;
            if (versionString != DATAVERSION_08)
            {
                count = r.ReadUInt32();
                for (; count > 0; count--)
                {
                    RenamedCardData d = new RenamedCardData();
                    d.Read(r);
                    BankBoxCards.Add(d);
                }
            }

            if (versionString == DATAVERSION_12)
            {
                //count = r.ReadByte();
                count = ((SjisCBinaryReader)r).Read80UInt32();
                for (; count > 0; count--)
                {
                    CardIndexData d = new CardIndexData();
                    d.Read(r);
                    CardIndexDatas.Add(d);
                }
                // 宿所持金
                yadoMoney = r.ReadUInt32();
                // 選択中のパーティー名
                loadPartyName = ((SjisCBinaryReader)r).ReadByteString();
            }
            else if (versionString != DATAVERSION_08)
            {
                count = r.ReadUInt32();
                for (; count > 0; count--)
                {
                    CardIndexData d = new CardIndexData();
                    d.Read(r);
                    CardIndexDatas.Add(d);
                }
                // 宿所持金
                yadoMoney = r.ReadUInt32();
                // 選択中のパーティー名
                loadPartyName = r.ReadString();
            }

        }

        public override void Write(System.IO.BinaryWriter w)
        {
            w.Write(versionString);
            w.Write(game_or_debug);

            if (versionString == DATAVERSION_12)
            {
                // （画面）速度調整
                w.Write((byte)speed_card);
                w.Write((byte)speed_view);
                w.Write((byte)speed_message);
                w.Write((byte)speed_card_battle);
            }
            else
            {
                // 速度調整
                w.Write(speed_card);
                w.Write(speed_view);
                w.Write(speed_message);

                // 音響効果
                w.Write(on_bgm);
                w.Write(on_sound);
            }

            // カードの画像修正
            w.Write(on_smallcard);
            w.Write(on_largecard);

            // その他
            w.Write(on_autoload);
            w.Write(on_cancellback);

            if (versionString != DATAVERSION_08)
            {
                w.Write(on_moneyeffect);
                w.Write(on_eventjump);
                w.Write(on_keepmaxlevel);
            }
            if (versionString == DATAVERSION_11 || versionString == DATAVERSION_12)
            {
                w.Write(backGroundEffect);
            }

            // シナリオ表示選択
            w.Write(on_backnet);

            // メッセージボード           
            if (versionString == DATAVERSION_12)
            {
                w.Write((byte)messageLight);
            }
            else
            {
                w.Write(messageLight);
            }
            w.Write(on_decoFont);

            // 背景アニメーション種別
            w.Write(backanime);

            if (versionString == DATAVERSION_12)
            {
                ((SjisCBinaryWriter)w).Write80(FinishScenarios);
                ((SjisCBinaryWriter)w).Write80(ProgressScenarios);
                ((SjisCBinaryWriter)w).Write80(Gossips);
            }
            else
            {
                if (versionString != DATAVERSION_08)
                {
                    ((SjisCBinaryWriter)w).Write(FinishScenarios);
                    ((SjisCBinaryWriter)w).Write(ProgressScenarios);
                    ((SjisCBinaryWriter)w).Write(Gossips);
                }
            }

            w.Write(BankBoxCards.Count);
            foreach (RenamedCardData d in BankBoxCards)
                d.Write(w);

            if (versionString == DATAVERSION_12)
            {
                ((SjisCBinaryWriter)w).Write80UInt32((uint)CardIndexDatas.Count);
                foreach (CardIndexData d in CardIndexDatas)
                    d.Write(w);
                w.Write(yadoMoney);
                ((SjisCBinaryWriter)w).WriteByteString(loadPartyName);
            }
            else
            {
                w.Write(CardIndexDatas.Count);
                foreach (CardIndexData d in CardIndexDatas)
                    d.Write(w);
                w.Write(yadoMoney);
                w.Write(loadPartyName);
            }
        }

        #endregion

        #region メソッド

        #region ファイル関連

        public DirectoryInfo CopyFolderToTemp()
        {
            DirectoryInfo ret = null;

            try
            {
                ret = CopyYadoFolder(Folder, TempFolder);

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("宿のコピーに失敗しました。" + ex.Message);
            }

            return ret;
        }

        public void BackupYadoFolder()
        {
            CopyYadoFolder(Folder, BackupFolder);
        }

        public void AfterTempFolderProc()
        {

            AfterSelectMethod();
        }

        private void AfterSelectMethod()
        {
            // ~宿にEnvironmentFileはない
            FilePath.DeleteFile(TempFolder + "\\" + Path.GetFileName(EnvironmentFile));

            if (AutoLoad)
            {
                // CurrentParty は~宿にファイルを持たない
                DeletePartyFile(CurrentParty);
            }

        }

        private DirectoryInfo CopyYadoFolder(string oldFolder, string newFolder)
        {
            DirectoryInfo diDst = new DirectoryInfo(newFolder);
            DirectoryInfo diSrc = new DirectoryInfo(oldFolder);

            try
            {

                // コピー
                string newPath = newFolder + "\\";
                string fastCopy = FilePath.ApplicationFolder + "\\fastcopy.exe";
                if (File.Exists(fastCopy))
                {
                    Process ps = null;
                    try
                    {
                        //FastCopy使用
                        /*
                            DestDir 指定の末尾に '\' があるかどうかによる挙動の違い
                            Source 指定が１ディレクトリのみで、DestDir 指定の末尾に '\' がない場合

                            Source ディレクトリ自体はコピーせず、Source ディレクトリ配下の内容を DestDir 配下にコピーします。
                            （DestDir\SourceDirの中身） 

                            それ以外の場合

                            Source ディレクトリ自体を含めて、DestDir 配下にコピーします。
                            （DestDir\SourceDir\SourceDirの中身）
                         
                        // forceCopy版
                        if (diDst.Exists)
                        {
                            foreach (FileInfo fi in diDst.GetFiles())
                            {
                                fi.Delete();
                            }
                        }
                        else
                        {
                            // コピー先のフォルダ作成
                            diDst.Create();
                        }

                        ProcessStartInfo info = new ProcessStartInfo(fastCopy, "/cmd=force_copy /auto_close /speed=full \"" + oldFolder + "\" /to=\"" + newFolder + "\"");
                        ps = Process.Start(info);
                        ps.WaitForExit();
                        ps.Close();
                        ps.Dispose();
                         * 
                            とりあえずsyncで同期をとってみるver
                        ProcessStartInfo info = new ProcessStartInfo(fastCopy, "/cmd=sync /auto_close /speed=full \"" + oldFolder + "\" /to=\"" + newFolder + "\"");
                        ps = Process.Start(info);
                        ps.WaitForExit();
                        ps.Close();
                        ps.Dispose();
                         * 
                         */

                        //syncで同期をとってみるver
                        ProcessStartInfo info = new ProcessStartInfo(fastCopy, "/cmd=sync /auto_close /speed=full \"" + oldFolder + "\" /to=\"" + newFolder + "\"");
                        info.WindowStyle = ProcessWindowStyle.Hidden;
                        ps = Process.Start(info);
                        ps.WaitForExit();
                        ps.Close();
                        ps.Dispose();
                    }
                    catch (Exception)
                    {
                        // ここで例外を処理します
                        // 今までのコピー

                        if (diDst.Exists)
                        {
                            foreach (FileInfo fi in diDst.GetFiles())
                            {
                                fi.Delete();
                            }
                        }
                        else
                        {
                            // コピー先のフォルダ作成
                            diDst.Create();
                        }

                        foreach (FileInfo fi in diSrc.GetFiles())
                        {
                            fi.CopyTo(newPath + fi.Name);
                        }
                    }
                }
                else
                {
                    // 今までのコピー

                    if (diDst.Exists)
                    {
                        foreach (FileInfo fi in diDst.GetFiles())
                        {
                            fi.Delete();
                        }
                    }
                    else
                    {
                        // コピー先のフォルダ作成
                        diDst.Create();
                    }

                    foreach (FileInfo fi in diSrc.GetFiles())
                    {
                        fi.CopyTo(newPath + fi.Name);
                    }
                }

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("宿のコピーに失敗しました。" + ex.Message);
            }

            return diDst;
            
        }

        /// <summary>
        /// 待機PC読み込み
        /// </summary>
        public void ReadPcs(DirectoryInfo di)
        {
            // 待機PC読み出し

            try
            {
                LoadAllPC(di);

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("PCファイルの読込に失敗しました。" 
                    + ex.Message);
            }
        }

        /// <summary>
        /// パーティー読み込み
        /// </summary>
        public void ReadParties(DirectoryInfo di)
        {
            try
            {

                // パーティー読み出し
                LoadAllParty(di);

                // オートロード
                if (AutoLoad)
                {
                    // Partiesの中からパーティー名でカレントパーティーを発見
                    // 同じパーティー名が２以上あるときは(2)で書かれているので注意
                    CurrentParty = SelectAutoParty(AutoLoadParty);

                }
                else
                {
                    CurrentParty = null;
                }

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("PTファイルの読込に失敗しました。" 
                    + ex.Message);
            }

        }

        /// <summary>
        /// ~宿削除
        /// </summary>
        public void DeleteTempFolder()
        {
            FilePath.DeleteFolder(TempFolder);
        }

        /// <summary>
        /// 宿セーブ
        /// </summary>
        public void Save()
        {
            try
            {
                string path = string.Empty;
                // 現在の宿の状態を保存する

                // ~宿から宿作成
                CopyYadoFolder(TempFolder, Folder);

                if (CurrentParty != null)
                {
                    // カレントパーティーがある
                    try
                    {
                        // パーティーセーブ・オートロード設定
                        SaveCurrentPartyPrimeFile();

                        AutoLoadON = true;
                        // fileNameには ".wpt" がついているので外す
                        string baseName = Path.GetFileNameWithoutExtension(CurrentParty.FileName);
                        AutoLoadParty = baseName;
                    }
                    catch (Exception ex)
                    {
                        PartyMaker.CWIErrorMessage("パーティ情報の書き込みに失敗：" + ex.Message);
                    }
                }
                else
                {
                    // カレントパーティーは存在しない
                    // オートロード設定

                    AutoLoadON = false;
                    AutoLoadParty = string.Empty;
                }

                // Environment.wyd を~なしの宿に直接保存
                path = FilePath.GetYadoEnvironmentFile(Folder);

                // ないが一応削除を入れて
                FilePath.DeleteFile(path);
                Write(path);

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("~宿の保存に失敗しました" + ex.Message);
            }

        }

        /// <summary>
        /// 対象他宿待機PC設定
        /// </summary>
        public void SetOtherInnCharacter(InnData otherInn)
        {
            try
            {
                // 対象他宿
                this.OtherInn = otherInn;

                if (Directory.Exists(OtherInn.TempFolder))
                {
                    // 既にあった場合は中のファイルを削除する
                    foreach (string file in Directory.GetFiles(OtherInn.TempFolder))
                    {
                        File.Delete(file);
                    }
                }
                else
                {
                    // ~宿作成
                    Directory.CreateDirectory(OtherInn.TempFolder);
                }

                // キャラクタカードだけコピー
                foreach (string file in Directory.GetFiles(OtherInn.Folder, "*.wc*"))
                {
                    File.Copy(file, OtherInn.TempFolder + "\\" + FilePath.FileNameFromPath(file), true);
                }


                // 他宿待機PC全読み込み
                OtherInn.Characters.Clear();

                // 待機ＰＣ
                string[] charaFiles = Directory.GetFiles(OtherInn.TempFolder,
                                      FilePath.MaskCharacterData);
                foreach (string file in charaFiles)
                {
                    OtherInn.Characters.Add(new CharacterData(file));
                }

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("他宿待機PCの読み込みに失敗しました。" + ex.Message);
            }
        }

        /// <summary>
        /// 対象他宿待機PCクリア
        /// </summary>
        public void ClearOtherInnCharacter()
        {

            try
            {
                // 他宿 ファイルセーブ
                foreach (string file in Directory.GetFiles(OtherInn.Folder, "*.wc*"))
                {
                    FilePath.DeleteFile(file);
                }

                foreach (string file in Directory.GetFiles(OtherInn.TempFolder))
                {
                    string path = OtherInn.Folder + "\\" + FilePath.FileNameFromPath(file);
                    File.Copy(file, path, true);
                }

                // Environment.wyd をある宿に直接保存
                string path2 = FilePath.GetYadoEnvironmentFile(OtherInn.Folder);

                FilePath.DeleteFile(path2);
                OtherInn.Write(path2);

                // 削除する
                OtherInn.DeleteTempFolder();

                this.OtherInn = null;
            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("他宿のセーブに失敗しました。" + ex.Message);
            }
        }

        /// <summary>
        /// 対象他宿Card設定
        /// </summary>
        public void SetOtherInnCard(InnData otherInn)
        {
            try
            {
                // 他宿
                this.OtherInn = otherInn;

                if (Directory.Exists(OtherInn.TempFolder))
                {
                    // 既にあった場合は中のファイルを削除する
                    foreach (string file in Directory.GetFiles(OtherInn.TempFolder))
                    {
                        File.Delete(file);
                    }
                }
                else
                {
                    // ~宿作成
                    Directory.CreateDirectory(OtherInn.TempFolder);
                }

                // 新規作成
                foreach (string file in Directory.GetFiles(OtherInn.Folder, "*.wid"))
                {
                    File.Copy(file, OtherInn.TempFolder + "\\" + FilePath.FileNameFromPath(file), true);
                }

                // WidCardが読み込まれていない時のためにダミー行
                int count = this.WidCards.Count; 
                count = this.OtherInn.WidCards.Count;
            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("他宿Cardの読み込みに失敗しました。" + ex.Message);
            }
        }

        /// <summary>
        /// 対象他宿カードクリア
        /// </summary>
        public void ClearOtherInnCard()
        {
            try
            {
                // 他宿 ファイルセーブ
                foreach (string file in Directory.GetFiles(OtherInn.Folder, "*.wid"))
                {
                    FilePath.DeleteFile(file);
                }

                foreach (string file in Directory.GetFiles(OtherInn.TempFolder))
                {
                    string path = OtherInn.Folder + "\\" + FilePath.FileNameFromPath(file);
                    File.Copy(file, path, true);
                }

                // Environment.wyd をある宿に直接保存
                string path2 = FilePath.GetYadoEnvironmentFile(OtherInn.Folder);

                FilePath.DeleteFile(path2);
                OtherInn.Write(path2);

                // 削除する
                OtherInn.DeleteTempFolder();

                this.OtherInn = null;
            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("他宿のセーブに失敗しました。" + ex.Message);
            }

        }

        /// <summary>
        /// 待機PCのBMP画像をData\Face\性別・年代フォルダにセーブ
        /// ファイル名は"PM_" + (RenameName) + ".bmp"
        /// </summary>
        public void PCBmpSave(bool changeFlg)
        {
            foreach (CharacterData cd in Characters)
            {
                CharaBmpSave(cd, changeFlg);
            }

            foreach (PartyData pd in Parties)
            {
                foreach (CharacterData cd in pd.Characters)
                {
                    CharaBmpSave(cd, changeFlg);
                }
            }
        }

        /// <summary>
        /// キャラクタの画像を保存
        /// </summary>
        /// <param name="cd"></param>
        /// <param name="changeFlg"></param>
        private void CharaBmpSave(CharacterData cd, bool changeFlg)
        {

            if (cd.Image == null
                || cd.Image.baseBitmap == null)
            {
                return;
            }

            string folder = FilePath.GetDataFaceFolder(cd.GetSex(), cd.GetAge());

            string fileName = cd.FileName;
            if (fileName.Equals(string.Empty))
            {
                //Party内のキャラクタはFileName== string.Empty
                fileName = FilePath.GetCharacterFileName(TempFolder, cd.CardName);
            }

            string pmFilename = folder
                + "\\PM_"
                + Path.GetFileNameWithoutExtension(fileName)
                + ".bmp";
            cd.Image.baseBitmap.Save(pmFilename, ImageFormat.Bmp);

            Util3.FixBitmap(pmFilename);
        }

        const int itemGossipKeta = 6;
        const string itemGossipHeader = "＄＄";
        const int itemEnd = 59;
        /// <summary>
        /// Itemを＄＄Gossip化
        /// </summary>
        public void ItemToGossip()
        {
            for (int itemIndex = 1; itemIndex <= itemEnd; itemIndex++)
            {
                WidCard baseCard = new WidCard();
                try
                {
                    baseCard.Read(FilePath.GetFileResource("Item" + itemIndex));
                }
                catch (Exception)
                {
                    PartyMaker.CWIErrorMessage("リソースが読み込めません。:" + "Item" + itemIndex);
                }

                string itemName = baseCard.CardName;
                string itemExp = baseCard.Expression;
                string headName = itemGossipHeader + itemName + "_";

                int itemCount = 0;
                int index = -1;
                for(int i = 0;i < this.Gossips.Count;i++)
                {
                    string gossip = this.Gossips[i];
                    if (gossip.Contains(headName))
                    {
                        itemCount = int.Parse(gossip.Substring(headName.Length));
                        index = i;
                        break;
                    }
                }

                uint cardCount = this.GetCardCount(itemName, itemExp);
                if (cardCount > 0)
                {
                    // 指定カード削除
                    DeleteAllTargetCards(itemName, itemExp);
                    // ゴシップ設定
                    string format = "D" + itemGossipKeta.ToString();

                    if (GetCardCount(itemName, itemExp) > 0)
                        PartyMaker.CWIErrorMessage("カード削除に失敗しました。(ItemToGoship)");
                    if (index != -1)
                    {
                        // ゴシップあり
                        this.Gossips[index] = headName + (itemCount + cardCount).ToString(format);
                    }
                    else
                    {
                        // ゴシップなし
                        this.Gossips.Add(headName + (itemCount + cardCount).ToString(format));
                    }
                }
            }
        }

        /// <summary>
        /// ＄＄GossipをItem化
        /// </summary>
        public void GossipToItem()
        {
            for (int itemIndex = 1; itemIndex <= itemEnd; itemIndex++)
            {
                WidCard baseCard = new WidCard();
                try
                {
                    baseCard.Read(FilePath.GetFileResource("Item" + itemIndex));
                }
                catch (Exception)
                {
                    PartyMaker.CWIErrorMessage("リソースが読み込めません。:" + "Item" + itemIndex);
                }

                string itemName = baseCard.CardName;
                string itemExp = baseCard.Expression;
                string headName = itemGossipHeader + itemName + "_";

                int itemCount = 0;
                for (int i = this.Gossips.Count - 1; i >= 0; i--)
                {
                    string gossip = this.Gossips[i];
                    if (gossip.Contains(headName))
                    {
                        itemCount = int.Parse(gossip.Substring(headName.Length));
                        this.Gossips.RemoveAt(i);
                        break;
                    }
                }

                if (itemCount != 0)
                {
                    uint cardCount = this.GetCardCount(itemName, itemExp);
                    if (cardCount > 0)
                    {
                        // カードあり
                        int index = GetCardIndex(itemName, itemExp);
                        this.AddRenamedCardData(this.BankBoxCards, WidCards[index], itemCount);
                    }
                    else
                    {
                        // カードなし
                        try
                        {
                            // ファイル書き出し
                            string filename = FilePath.GetWidFileName(WidIDtype.アイテム,
                                this.TempFolder, itemName);

                            baseCard.FileName = filename;
                            baseCard.Write(filename);

                        }
                        catch (Exception)
                        {
                            PartyMaker.CWIErrorMessage("Widファイルが読み込めません。（GoshipToItem）");
                        }
                        this.AddRenamedCardData(this.BankBoxCards, baseCard, itemCount);
                    }
                }

            }

        }

        /// <summary>
        /// StarDustの状態
        /// </summary>
        public enum StarDustState
        {
            None,
            Coupon,
            Gossip
        }

        const string starDustHeader = "§";
        const int starDustKeta = 2;
        const int starDustMax = 50;
        const int starDustEnd = 7;
        /// <summary>
        /// StarDustの管理情報チェック
        /// </summary>
        /// <returns></returns>
        public StarDustState CheckStarDust()
        {
            StarDustState state = StarDustState.None;

            if (this.Gossips.Contains("SD_管理情報_クーポン"))
            {
                state = StarDustState.Coupon;
            }
            else if (this.Gossips.Contains("SD_管理情報_ゴシップ"))
            {
                state = StarDustState.Gossip;
            }

            return state;
        }

        /// <summary>
        /// ロータスの数
        /// </summary>
        /// <returns></returns>
        public int CountLotus()
        {
            int count = 0;

            PartyData pd = this.CurrentParty;
            if (pd == null) return count;

            foreach (CharacterData cd in pd.Characters)
            {
                foreach (CouponData coup in cd.Coupons)
                {
                    if (coup.Name == "＿ロータス(Star Dust)")
                    {
                        count++;
                        break;
                    }
                }
            }
            
            return count;
        }

        /// <summary>
        /// ロータス
        /// </summary>
        /// <returns></returns>
        public CharacterData FindLotus()
        {
            PartyData pd = this.CurrentParty;

            if (pd == null) return null;
            foreach (CharacterData cd in pd.Characters)
            {
                foreach (CouponData coup in cd.Coupons)
                {
                    if (coup.Name == "＿ロータス(Star Dust)")
                    {
                        return cd;
                    }
                }
            }
            

            return null;
        }

        /// <summary>
        /// カレントパーティー荷物袋Itemを§Gossip,§Coupon化
        /// </summary>
        public void ItemToStarDust()
        {
            StarDustState state = CheckStarDust();
            CharacterData lotus = null;
            if (state == StarDustState.Coupon)
            {
                lotus = FindLotus();
            }

            for (int itemIndex = 1; itemIndex <= starDustEnd; itemIndex++)
            {
                WidCard baseCard = new WidCard();
                try
                {
                    baseCard.Read(FilePath.GetFileResource("Item" + itemIndex));
                }
                catch (Exception)
                {
                    PartyMaker.CWIErrorMessage("リソースが読み込めません。:" + "Item" + itemIndex);
                }

                string itemName = baseCard.CardName;
                string itemExp = baseCard.Expression;
                string headName = starDustHeader + itemName + "_";

                int itemCount = 0;
                int index = -1;

                if (state == StarDustState.Gossip)
                {
                    // ゴシップ
                    for (int i = 0; i < this.Gossips.Count; i++)
                    {
                        string gossip = this.Gossips[i];
                        if (gossip.Contains(headName))
                        {
                            itemCount = int.Parse(gossip.Substring(headName.Length));
                            index = i;
                            break;
                        }
                    }
                }
                else if (state == StarDustState.Coupon)
                {
                    // クーポン
                    for (int i = 0; i < lotus.Coupons.Count; i++)
                    {
                        string coupon = lotus.Coupons[i].Name;
                        if (coupon.Contains(headName))
                        {
                            itemCount = int.Parse(coupon.Substring(headName.Length));
                            index = i;
                            break;
                        }
                    }
                }

                if (itemCount >= starDustMax)
                {
                    continue;
                }

                uint cardCount = this.GetCardCount(itemName, itemExp);
                if (cardCount > 0)
                {
                    // カード置場か荷物袋にカードがあって、
                    // 預けている枚数がMaxでない場合

                    if (itemCount + cardCount <= starDustMax)
                    {
                        // 合計がMax以下の場合
                        // 指定カードをカレント荷物袋から全削除
                        DeleteAllTargetCurrentBagCards(itemName, itemExp);
                    }
                    else
                    {
                        // 合計がMaxを超える場合
                        // 指定カードをカレント荷物袋から指定枚数だけ削除
                        cardCount = starDustMax - (uint)itemCount;
                        DeleteTargetCurrentBagCards(itemName, itemExp, cardCount);
                    }
                }

                // ゴシップ設定
                string format = "D" + starDustKeta.ToString();

                if (state == StarDustState.Gossip)
                {
                    // ゴシップ
                    if (index != -1)
                    {
                        // ゴシップあり
                        this.Gossips[index] = headName + (itemCount + cardCount).ToString(format);
                    }
                    else
                    {
                        // ゴシップなし
                        this.Gossips.Add(headName + (itemCount + cardCount).ToString(format));
                    }
                }
                else if (state == StarDustState.Coupon)
                {
                    // クーポン
                    if (index != -1)
                    {
                        // クーポンあり
                        lotus.Coupons[index].Name =
                            headName + (itemCount + cardCount).ToString(format);
                    }
                    else
                    {
                        // クーポンなし
                        CouponData coup = new CouponData();
                        coup.Name = headName + (itemCount + cardCount).ToString(format);
                        coup.Point = 0;
                        lotus.Coupons.Add(coup);
                    }
                }

            }
        }

        /// <summary>
        /// §Gossip,§CouponをItem化
        /// </summary>
        public void StarDustToItem()
        {
            StarDustState state = CheckStarDust();
            CharacterData lotus = null;
            if (state == StarDustState.Coupon)
            {
                lotus = FindLotus();
            }

            for (int itemIndex = 1; itemIndex <= starDustEnd; itemIndex++)
            {
                WidCard baseCard = new WidCard();
                try
                {
                    baseCard.Read(FilePath.GetFileResource("Item" + itemIndex));
                }
                catch (Exception)
                {
                    PartyMaker.CWIErrorMessage("リソースが読み込めません。:" + "Item" + itemIndex);
                }

                string itemName = baseCard.CardName;
                string itemExp = baseCard.Expression;
                string headName = starDustHeader + itemName + "_";

                int itemCount = 0;
                int index = -1;

                if (state == StarDustState.Gossip)
                {
                    // ゴシップ
                    for (int i = this.Gossips.Count - 1; i >= 0; i--)
                    {
                        string gossip = this.Gossips[i];
                        if (gossip.Contains(headName))
                        {
                            itemCount = int.Parse(gossip.Substring(headName.Length));
                            index = i;
                            this.Gossips[i] = headName + "00";
                            break;
                        }
                    }
                }
                else if (state == StarDustState.Coupon)
                {
                    // クーポン
                    for (int i = lotus.Coupons.Count - 1; i >= 0; i--)
                    {
                        string coupon = lotus.Coupons[i].Name;
                        if (coupon.Contains(headName))
                        {
                            itemCount = int.Parse(coupon.Substring(headName.Length));
                            index = i;
                            lotus.Coupons[i].Name = headName + "00";
                            break;
                        }
                    }
                }

                if (itemCount != 0)
                {
                    uint cardCount = this.GetCardCount(itemName, itemExp);
                    if (cardCount > 0)
                    {
                        // カードあり
                        int widIndex = GetCardIndex(itemName, itemExp);
                        this.AddRenamedCardData(this.CurrentParty.BagCards, WidCards[widIndex], itemCount);
                    }
                    else
                    {
                        // カードなし
                        try
                        {
                            // ファイル書き出し
                            string filename = FilePath.GetWidFileName(WidIDtype.アイテム,
                                this.TempFolder, itemName);

                            baseCard.FileName = filename;
                            baseCard.Write(filename);

                        }
                        catch (Exception)
                        {
                            PartyMaker.CWIErrorMessage("Widファイルが読み込めません。（StarDustToItem）");
                        }
                        this.AddRenamedCardData(this.CurrentParty.BagCards, baseCard, itemCount);
                    }
                }
            }
        }


        #endregion

        #region カード関連

        #region カードリスト検索・取得

        /// <summary>
        /// 名前と解説からカードが何枚有るかを取得する
        /// </summary>
        /// <param name="name">名前</param>
        /// <param name="exp">解説</param>
        /// <returns>枚数</returns>
        public uint GetCardCount(string name, string exp)
        {
            uint count = 0;
            CardIndexData cid;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.Name == name &&
                    cid.Expression == exp)
                {
                    count += cid.Count;
                    break;
                }
            }

            return count;
        }

        /// <summary>
        /// 名前と解説からカードIndexを取得する
        /// </summary>
        /// <param name="name">名前</param>
        /// <param name="exp">解説</param>
        /// <returns>Index</returns>
        public int GetCardIndex(string name, string exp)
        {
            CardIndexData cid;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.Name == name &&
                    cid.Expression == exp)
                {
                    return i;
                }
            }

            return -1;
        }

        /// <summary>
        /// 名前と解説からカード置場カードが何枚有るかを取得する
        /// </summary>
        /// <param name="name">名前</param>
        /// <param name="exp">解説</param>
        /// <returns>枚数</returns>
        public int GetYadoCardCount(string name, string exp)
        {
            int count = 0;
            CardIndexData cid;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.Name == name &&
                    cid.Expression == exp)
                {
                    foreach (RenamedCardData rcd in this.BankBoxCards)
                    {
                        if (rcd.RenamedName == cid.RenamedName)
                        {
                            count++;
                        }
                    }
                    break;
                }
            }

            return count;
        }

        /// <summary>
        /// リネーム名から現在の宿のIndexCardDataを探す。
        /// </summary>
        /// <param name="renamedName">リネームしたカード名</param>
        /// <returns>見つけたIndexCardData、見つからない場合 null</returns>
        public IndexCardData FindIndexedCardData(string renamedName)
        {
            IndexCardData icd = new IndexCardData();
            CardIndexData cid;
            for(int i = 0;i < CardIndexDatas.Count;i++)
            {
                cid = CardIndexDatas[i];
                if (cid.RenamedName == renamedName)
                {
                    icd.Index = i;
                    icd.CardData = cid;
                    icd.WidData = WidCards[i];
                    
                    return icd;
                }
            }


            icd.Index = -1;
            icd.CardData = null;
            icd.WidData = null;
            return icd;
        }

        /// <summary>
        /// RenamedCardDataから現在の宿のIndexCardDataを探す。
        /// </summary>
        /// <param name="renameCard">RenamedCardData</param>
        /// <returns>見つけたIndexCardData、見つからない場合 null</returns>
        public IndexCardData FindIndexedCardData(RenamedCardData renameCard)
        {
            IndexCardData icd = new IndexCardData();
            CardIndexData cid;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.RenamedName == renameCard.RenamedName)
                {
                    icd.Index = i;
                    icd.CardData = cid;
                    icd.ItemData = renameCard;
                    icd.WidData = WidCards[i];
                    BagHandCard hc = new BagHandCard(icd.WidData, renameCard);
                    hc.UpdateImage();
                    icd.HandCard = hc;
                    return icd;
                }
            }

            icd.Index = -1;
            icd.CardData = null;
            icd.ItemData = null;
            icd.WidData = null;
            return icd;
        }

        /// <summary>
        /// cardと一致するCardIndexDataを現在の宿から取得する。
        /// </summary>
        /// <param name="card">探したいWidCardオブジェクト</param>
        /// <returns>見つけたCardIndexData、見つからない場合null</returns>
        public IndexCardData FindIndexedCardData(WidCard card)
        {
            IndexCardData icd = new IndexCardData();
            CardIndexData cid;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (card.CardName == cid.Name &&
                    card.Expression == cid.Expression)
                {
                    icd.Index = i;
                    icd.CardData = cid;
                    icd.WidData = WidCards[i];
                    return icd;
                }
            }
            icd.Index = -1;
            icd.CardData = null;
            icd.WidData = null;
            return icd;
        }

        #endregion

        #region カード一括移動
        /// <summary>
        /// 他宿カードが一括移動
        /// </summary>
        public void MoveAllOtherInnCard()
        {
            // リストコピー
            List<RenamedCardData> localList = new List<RenamedCardData>();
            localList.AddRange(OtherInn.BankBoxCards);

            WidCard widCard;
            IndexCardData icd;
            foreach (RenamedCardData rcd in localList)
            {

                icd = OtherInn.FindIndexedCardData(rcd.RenamedName);

                // WidCard 複製
                widCard = new WidCard(icd.WidData);
                switch (icd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = rcd.UseCount;
                }

                // 他宿の元のファイルとリスト削除
                OtherInn.DeleteRenamedCardData(OtherInn.BankBoxCards, rcd);

                // 宿のファイルとリスト追加
                AddRenamedCardData(BankBoxCards, widCard, 1);
            }

            OtherInn.BankBoxCards.Clear();

        }
        /// <summary>
        /// 宿カードが全部他宿へ
        /// </summary>
        public void MoveAllCardToOtherInn()
        {
            // リストコピー
            List<RenamedCardData> localList = new List<RenamedCardData>();
            localList.AddRange(BankBoxCards);

            WidCard widCard;
            IndexCardData icd;
            foreach (RenamedCardData rcd in localList)
            {
                icd = FindIndexedCardData(rcd.RenamedName);

                // WidCard 複製
                widCard = new WidCard(icd.WidData);
                switch (icd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = rcd.UseCount;
                }

                // 宿のファイルとリスト削除
                DeleteRenamedCardData(BankBoxCards, rcd);
                // 他宿のファイルとリスト追加
                OtherInn.AddRenamedCardData(OtherInn.BankBoxCards, widCard, 1);
            }

            BankBoxCards.Clear();
        }

        /// <summary>
        /// カレントパーティーの荷物袋カードが一括移動
        /// </summary>
        public void MoveAllCardFromPartyBag()
        {
            // リストコピー
            List<RenamedCardData> localList = new List<RenamedCardData>();
            localList.AddRange(CurrentParty.BagCards);

            WidCard widCard;
            IndexCardData icd;
            foreach (RenamedCardData rcd in localList)
            {

                icd = FindIndexedCardData(rcd.RenamedName);

                // WidCard 複製
                widCard = new WidCard(icd.WidData);
                switch (icd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = rcd.UseCount;
                }

                // カレントパーティーの荷物袋の元のファイルとリスト削除
                DeleteRenamedCardData(CurrentParty.BagCards, rcd);

                // 宿のファイルとリスト追加
                AddRenamedCardData(BankBoxCards, widCard, 1);
            }

            CurrentParty.BagCards.Clear();

        }
        /// <summary>
        /// 宿カードが全部カレントパーティーの荷物袋へ
        /// </summary>
        public void MoveAllCardToPartyBag()
        {
            // リストコピー
            List<RenamedCardData> localList = new List<RenamedCardData>();
            localList.AddRange(BankBoxCards);

            WidCard widCard;
            IndexCardData icd;
            foreach (RenamedCardData rcd in localList)
            {
                icd = FindIndexedCardData(rcd.RenamedName);

                // WidCard 複製
                widCard = new WidCard(icd.WidData);
                switch (icd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = rcd.UseCount;
                }

                // 宿のファイルとリスト削除
                DeleteRenamedCardData(BankBoxCards, rcd);

                // カレントパーティーの荷物袋のファイルとリスト追加
                AddRenamedCardData(CurrentParty.BagCards, widCard, 1);
            }

            BankBoxCards.Clear();
        }

        /// <summary>
        /// 名前と解説が一致するカードをカード置場と全荷物袋から全削除
        /// </summary>
        /// <param name="name"></param>
        /// <param name="exp"></param>
        private void DeleteAllTargetCards(string name, string exp)
        {

            CardIndexData cid;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.Name == name &&
                    cid.Expression == exp)
                {

                    if (GetCardCount(name, exp) > 0)
                    {
                        //カード置場から全削除
                        DeleteTargetYadoCards(i);
                    }

                    if (GetCardCount(name, exp) > 0)
                    {
                        //荷物袋から全削除
                        DeleteTargetPartyBagCards(i);
                    }

                    break;
                }
            }


        }

        /// <summary>
        /// 名前と解説が一致するカードをカード置場と全荷物袋から指定枚数だけ削除
        /// </summary>
        /// <param name="name"></param>
        /// <param name="exp"></param>
        /// <param name="eraseCount"></param>
        private void DeleteTargetCards(string name, string exp,uint eraseCount )
        {

            CardIndexData cid;
            uint zanSuu = eraseCount;
            uint count = 0;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.Name == name &&
                    cid.Expression == exp)
                {
                    count = GetCardCount(name, exp);

                    if (count > 0)
                    {
                        if (zanSuu >= count)
                        {
                            //カード置場から全削除
                            DeleteTargetYadoCards(i);
                            zanSuu -= count;
                            if (zanSuu == 0)
                            {
                                return;
                            }
                        }
                        else
                        {
                            //カード置場から削除
                            DeleteTargetYadoCards(i, zanSuu);
                            return;
                        }
                    }

                    count = GetCardCount(name, exp);
                    if (count > 0)
                    {
                        if (zanSuu >= count)
                        {
                            //荷物袋から全削除
                            DeleteTargetPartyBagCards(i);
                        }
                        else
                        {
                            //カード置場から削除
                            DeleteTargetPartyBagCards(i, zanSuu);
                            return;
                        }
                    }

                    break;
                }
            }


        }


        /// <summary>
        /// 名前と解説が一致するカードをカレント荷物袋から全削除
        /// </summary>
        /// <param name="name"></param>
        /// <param name="exp"></param>
        private void DeleteAllTargetCurrentBagCards(string name, string exp)
        {

            CardIndexData cid;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.Name == name &&
                    cid.Expression == exp)
                {
                    if (GetCardCount(name, exp) > 0)
                    {
                        //カレント荷物袋から全削除
                        DeleteTargetCurrentPartyBagCards(i);
                    }

                    break;
                }
            }
        }

        /// <summary>
        /// 名前と解説が一致するカードをカレント荷物袋から指定枚数だけ削除
        /// </summary>
        /// <param name="name"></param>
        /// <param name="exp"></param>
        /// <param name="eraseCount"></param>
        private void DeleteTargetCurrentBagCards(string name, string exp, uint eraseCount)
        {

            CardIndexData cid;
            uint zanSuu = eraseCount;
            uint count = 0;
            for (int i = 0; i < CardIndexDatas.Count; i++)
            {
                cid = CardIndexDatas[i];
                if (cid.Name == name &&
                    cid.Expression == exp)
                {
                    count = GetCardCount(name, exp);

                    if (count > 0)
                    {
                        if (zanSuu >= count)
                        {
                            //カレント荷物袋から全削除
                            DeleteTargetCurrentPartyBagCards(i);
                        }
                        else
                        {
                            //カレント荷物袋から指定枚数削除
                            DeleteTargetCurrentPartyBagCards(i, zanSuu);
                            return;
                        }
                    }

                    break;
                }
            }


        }

        /// <summary>
        /// PCカードをカード置場へ移動
        /// </summary>
        public void MoveAllPCCardToInnCard()
        {
            foreach (CharacterData cd in this.Characters)
            {
                // 手持ちの全カードをカード置場へ移動
                foreach (WidCard skill in cd.Skills)
                {
                    // ホールドは外しておく
                    skill.HoldCard = false;

                    // 宿カードリスト追加・ファイル追加
                    AddRenamedCardData(this.BankBoxCards, skill, 1);
                }

                // キャラクタ所持カードリスト削除
                cd.Skills.Clear();

                foreach (WidCard item in cd.Items)
                {
                    // ホールドは外しておく
                    item.HoldCard = false;

                    // 宿カードリスト追加・ファイル追加
                    AddRenamedCardData(this.BankBoxCards, item, 1);
                }

                // キャラクタ所持カードリスト削除
                cd.Items.Clear();

                foreach (WidCard beast in cd.Beasts)
                {
                    // ホールドは外しておく
                    beast.HoldCard = false;

                    // 宿カードリスト追加・ファイル追加
                    AddRenamedCardData(this.BankBoxCards, beast, 1);
                }

                // キャラクタ所持カードリスト削除
                cd.Beasts.Clear();
                // ファイルに結果反映
                AddCharacterFile(cd);
            }
        }
        #endregion

        #region カード追加
        /// <summary>
        /// 荷物袋かカード置場にカードを追加する。追加位置はリストの先頭。
        /// renamedCardsはPartyData.bagCards か InnData.renamedCardList
        /// </summary>
        /// <param name="renamedCards">カードを追加する先のリスト</param>
        /// <param name="card">追加したいカード</param>
        /// <param name="count">追加する枚数</param>
        public void AddRenamedCardData(List<RenamedCardData> renamedCards,
            WidCard card, int count)
        {
            if (count <= 0) return;

            //宿フォルダのインデックスカードを検索
            IndexCardData icd = FindIndexedCardData(card);
            CardIndexData indexedCard = icd.CardData;
            
            if (indexedCard == null)
            {
                indexedCard = NewIndexedCardData(card);
                indexedCard.Count = (uint)count;

                //WidCardsが読み込まれていないときにCardIndexDatasを
                //先にするとエラーが起こる
                WidCards.Insert(0, card);
                CardIndexDatas.Insert(0, indexedCard);

                //~宿フォルダへリネームした名前でwidCardを保存
                try
                {
                    card.Write(FilePath.GetNameWithWid(TempFolder, indexedCard.RenamedName));
                }
                catch
                {
                    PartyMaker.CWIErrorMessage("カードのセーブに失敗。強制終了");
                }
                //荷物袋カードの追加
                while (count > 0)
                {
                    renamedCards.Add( NewRenamedCardData(card.ItemRemainUseCount, indexedCard.RenamedName));
                    count--;
                }
            }
            else
            {
                indexedCard.Count += (uint)count;
                //荷物袋カードの追加
                while (count > 0)
                {
                    renamedCards.Add( NewRenamedCardData(card.ItemRemainUseCount, indexedCard.RenamedName));
                    count--;
                }
            }
        }

        /// <summary>
        /// 荷物袋と宿金庫に使われるRenamedCardDataオブジェクトをnewして返す。
        /// カードを追加したい時はAddRenamedCardDataを使う。public化は禁止。
        /// </summary>
        /// <param name="useCount">残り使用回数（アイテム以外無意味）</param>
        /// <param name="renamed">実際に保存されるファイル名</param>
        /// <returns>新しいRenamedCardDataオブジェクト</returns>
        public RenamedCardData NewRenamedCardData(uint useCount, string renamed)
        {
            RenamedCardData renamedCard = new RenamedCardData();
            renamedCard.UseCount = useCount;
            renamedCard.RenamedName = renamed;
            renamedCard.FixPossession = false;
            return renamedCard;
        }

        //宿カードのIndexedCardDataデータをnewする
        //card : データ元となるwidカードオブジェクト
        //return : new したIndexedCardDataのインスタンス
        public CardIndexData NewIndexedCardData(WidCard card)
        {
            CardIndexData cardIndex;
            cardIndex = new CardIndexData();
            cardIndex.Name = card.CardName;
            cardIndex.Expression = card.Expression;
            switch (card.WidID)
            {
                case WidIDtype.スキル:
                    cardIndex.CardKind = CardIndexData.Kind.skill;
                    break;
                case WidIDtype.アイテム:
                    cardIndex.CardKind = CardIndexData.Kind.item;
                    break;
                case WidIDtype.召喚獣:
                    cardIndex.CardKind = CardIndexData.Kind.beast;
                    break;
            }
            cardIndex.RenamedName = FilePath.GetRenamedCardName(TempFolder, cardIndex.Name);
            cardIndex.Count = 1;
            return cardIndex;
        }

        #endregion

        #region カード削除
        /// <summary>
        /// 指定インデックスのカードをカード置場から全削除
        /// </summary>
        /// <param name="cardIndex">カードインデックス</param>
        private void DeleteTargetYadoCards(int cardIndex)
        {
            uint delCount = 0;
            CardIndexData cid = CardIndexDatas[cardIndex];
            List<RenamedCardData> list = this.BankBoxCards;
            for (int index = list.Count - 1; index >= 0; index--)
            {
                RenamedCardData rcd = list[index];
                if (rcd.RenamedName == cid.RenamedName)
                {
                    list.Remove(rcd);
                    delCount++;
                }
            }

            if (cid.Count < delCount)
            {
                PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了(InnData.DeleteTargetYadoCards)");
            }
            cid.Count -= delCount;

            //０であればインデックスとファイルを消す
            if (cid.Count == 0)
            {
                // オブジェクト削除
                CardIndexDatas.RemoveAt(cardIndex);
                WidCards.RemoveAt(cardIndex);

                //~宿フォルダからリネームした名前のwidCardを削除
                try
                {
                    File.Delete(FilePath.GetNameWithWid(TempFolder, cid.RenamedName));
                }
                catch
                {
                    PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                }
            }
        }

        /// <summary>
        /// 指定インデックスのカードをカード置場から指定枚数だけ削除
        /// </summary>
        /// <param name="cardIndex">カードインデックス</param>
        /// <param name="cardCount">カード枚数</param>
        private void DeleteTargetYadoCards(int cardIndex, uint cardCount)
        {
            uint delCount = 0;
            CardIndexData cid = CardIndexDatas[cardIndex];
            List<RenamedCardData> list = this.BankBoxCards;
            for (int index = list.Count - 1; index >= 0; index--)
            {
                RenamedCardData rcd = list[index];
                if (rcd.RenamedName == cid.RenamedName)
                {
                    list.Remove(rcd);
                    delCount++;
                    if (delCount == cardCount) break;
                }
            }

            if (cid.Count < delCount)
            {
                PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了(InnData.DeleteTargetYadoCards)");
            }
            cid.Count -= delCount;

            //０であればインデックスとファイルを消す
            if (cid.Count == 0)
            {
                // オブジェクト削除
                CardIndexDatas.RemoveAt(cardIndex);
                WidCards.RemoveAt(cardIndex);

                //~宿フォルダからリネームした名前のwidCardを削除
                try
                {
                    File.Delete(FilePath.GetNameWithWid(TempFolder, cid.RenamedName));
                }
                catch
                {
                    PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                }
            }
        }

        /// <summary>
        /// 指定インデックスのカードを全荷物袋から全削除
        /// </summary>
        /// <param name="cardIndex">カードインデックス</param>
        private void DeleteTargetPartyBagCards(int cardIndex)
        {
            CardIndexData cid = CardIndexDatas[cardIndex];
            List<RenamedCardData> list;

            int currentIndex = -1;
            if (this.currentParty != null) currentIndex = this.GetPartyIndex(this.CurrentParty);

            for (int pIndex = 0; pIndex < this.Parties.Count; pIndex++)
            {

                uint cardCount = this.GetCardCount(cid.Name, cid.Expression);
                if (cardCount == 0) break;

                uint delCount = 0;
                PartyData pd = this.Parties[pIndex];
                list = pd.BagCards;
                for (int index = list.Count - 1; index >= 0; index--)
                {
                    RenamedCardData rcd = list[index];
                    if (rcd.RenamedName == cid.RenamedName)
                    {
                        list.Remove(rcd);
                        delCount++;
                    }
                }

                if (delCount == 0) continue;

                if (pIndex != currentIndex)
                {
                    // 安全のためインデックス確認
                    int index = GetPartyIndex(pd);

                    // パーティファイル追加
                    AddParyFile(pd);

                    // 保存したことでファイル名が付いているので、リストに関連付け直し
                    Parties[index] = pd;
                }

                if (cid.Count < delCount)
                {
                    PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了(InnData.DeleteTargetPartyBagCards)");
                }
                cid.Count -= delCount;
                //０であればインデックスとファイルを消す
                if (cid.Count == 0)
                {
                    // オブジェクト削除
                    CardIndexDatas.RemoveAt(cardIndex);
                    WidCards.RemoveAt(cardIndex);

                    //~宿フォルダからリネームした名前のwidCardを削除
                    try
                    {
                        File.Delete(FilePath.GetNameWithWid(TempFolder, cid.RenamedName));
                    }
                    catch
                    {
                        PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                    }

                    return;
                }
            }
        }

        /// <summary>
        /// 指定インデックスのカードを全荷物袋から指定枚数だけ削除
        /// </summary>
        /// <param name="cardIndex">カードインデックス</param>
        /// <param name="cCount">カード枚数</param>
        private void DeleteTargetPartyBagCards(int cardIndex, uint cCount)
        {
            CardIndexData cid = CardIndexDatas[cardIndex];
            List<RenamedCardData> list;

            uint zanCount = cCount;
            int currentIndex = -1;
            if(this.currentParty != null) currentIndex = this.GetPartyIndex(this.CurrentParty);

            for(int pIndex = 0;pIndex < this.Parties.Count; pIndex++)
            {

                uint cardCount = this.GetCardCount(cid.Name, cid.Expression);
                if (cardCount == 0) break;

                uint delCount = 0;
                PartyData pd = this.Parties[pIndex];
                list = pd.BagCards;
                for (int index = list.Count - 1; index >= 0; index--)
                {
                    RenamedCardData rcd = list[index];
                    if (rcd.RenamedName == cid.RenamedName && zanCount > 0)
                    {
                        list.Remove(rcd);
                        delCount++;
                        zanCount--;
                    }
                }

                if (delCount == 0) continue;

                if (pIndex != currentIndex)
                {
                    // 安全のためインデックス確認
                    int index = GetPartyIndex(pd);

                    // パーティファイル追加
                    AddParyFile(pd);

                    // 保存したことでファイル名が付いているので、リストに関連付け直し
                    Parties[index] = pd;
                }

                if (cid.Count < delCount)
                {
                    PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了(InnData.DeleteTargetPartyBagCards)");
                }
                cid.Count -= delCount;
                //０であればインデックスとファイルを消す
                if (cid.Count == 0)
                {
                    // オブジェクト削除
                    CardIndexDatas.RemoveAt(cardIndex);
                    WidCards.RemoveAt(cardIndex);

                    //~宿フォルダからリネームした名前のwidCardを削除
                    try
                    {
                        File.Delete(FilePath.GetNameWithWid(TempFolder, cid.RenamedName));
                    }
                    catch
                    {
                        PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                    }

                    return;
                }
            }
        }

        /// <summary>
        /// 指定インデックスのカードをカレントパーティー荷物袋から全削除
        /// </summary>
        /// <param name="cardIndex">カードインデックス</param>
        private void DeleteTargetCurrentPartyBagCards(int cardIndex)
        {
            CardIndexData cid = CardIndexDatas[cardIndex];
            List<RenamedCardData> list;

            uint cardCount = this.GetCardCount(cid.Name, cid.Expression);
            if (cardCount == 0) return;

            uint delCount = 0;
            PartyData pd = this.CurrentParty;
            list = pd.BagCards;
            for (int index = list.Count - 1; index >= 0; index--)
            {
                RenamedCardData rcd = list[index];
                if (rcd.RenamedName == cid.RenamedName)
                {
                    list.Remove(rcd);
                    delCount++;
                }
            }

            if (delCount == 0) return;

            if (cid.Count < delCount)
            {
                PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了(InnData.DeleteTargetPartyBagCards)");
            }
            cid.Count -= delCount;
            //０であればインデックスとファイルを消す
            if (cid.Count == 0)
            {
                // オブジェクト削除
                CardIndexDatas.RemoveAt(cardIndex);
                WidCards.RemoveAt(cardIndex);

                //~宿フォルダからリネームした名前のwidCardを削除
                try
                {
                    File.Delete(FilePath.GetNameWithWid(TempFolder, cid.RenamedName));
                }
                catch
                {
                    PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                }

                return;
            }
            
        }

        /// <summary>
        /// 指定インデックスのカードをカレントパーティー荷物袋から指定枚数だけ削除
        /// </summary>
        /// <param name="cardIndex">カードインデックス</param>
        /// <param name="cCount">カード枚数</param>
        private void DeleteTargetCurrentPartyBagCards(int cardIndex, uint cCount)
        {
            CardIndexData cid = CardIndexDatas[cardIndex];
            List<RenamedCardData> list;

            uint zanCount = cCount;

            uint cardCount = this.GetCardCount(cid.Name, cid.Expression);
            if (cardCount == 0) return;

            uint delCount = 0;
            PartyData pd = this.CurrentParty;
            list = pd.BagCards;
            for (int index = list.Count - 1; index >= 0; index--)
            {
                RenamedCardData rcd = list[index];
                if (rcd.RenamedName == cid.RenamedName && zanCount > 0)
                {
                    list.Remove(rcd);
                    delCount++;
                    zanCount--;
                }
            }

            if (delCount == 0) return;

            if (cid.Count < delCount)
            {
                PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了(InnData.DeleteTargetPartyBagCards)");
            }
            cid.Count -= delCount;
            //０であればインデックスとファイルを消す
            if (cid.Count == 0)
            {
                // オブジェクト削除
                CardIndexDatas.RemoveAt(cardIndex);
                WidCards.RemoveAt(cardIndex);

                //~宿フォルダからリネームした名前のwidCardを削除
                try
                {
                    File.Delete(FilePath.GetNameWithWid(TempFolder, cid.RenamedName));
                }
                catch
                {
                    PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                }

                return;
            }
            
        }


        /// <summary>
        /// 金庫か荷物袋のカードを一枚削除する
        /// </summary>
        /// <param name="renamedCards">削除元の金庫か荷物袋</param>
        /// <param name="rcard">削除するカード</param>
        public void DeleteRenamedCardData(List<RenamedCardData> renamedCards, RenamedCardData rcard)
        {
            IndexCardData icd = FindIndexedCardData(rcard.RenamedName);
            CardIndexData cardIndex = icd.CardData;
            renamedCards.Remove(rcard);

            if (cardIndex == null)
                return;
            //宿カードインデックスの参照を１つ減らし、０であればインデックスとファイルを消す
            cardIndex.Count--;
            if (cardIndex.Count == 0)
            {

                // オブジェクト削除
                CardIndexDatas.Remove(cardIndex);
                WidCards.Remove(icd.WidData);

                //~宿フォルダからリネームした名前のwidCardを削除
                try
                {
                    File.Delete(FilePath.GetNameWithWid(TempFolder, rcard.RenamedName));
                }
                catch
                {
                    PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                }
            }

        }

        /// <summary>
        /// 荷物袋か金庫からcardと一致するカードを一枚削除する
        /// </summary>
        /// <param name="renamedCards">削除元となるリスト</param>
        /// <param name="card">データ元となるwidカード</param>
        /// <returns>削除できたらtrue</returns>
        public bool DeleteRenamedCard(List<RenamedCardData> renamedCards, WidCard card)
        {
            foreach (RenamedCardData rd in renamedCards)
            {
                IndexCardData icd = FindIndexedCardData(rd.RenamedName);
                CardIndexData cid = icd.CardData;
                if (cid == null)
                    continue;//ありえないはず

                if (((cid.CardKind == CardIndexData.Kind.item && card.WidID == WidIDtype.アイテム) ||
                    (cid.CardKind == CardIndexData.Kind.skill && card.WidID == WidIDtype.スキル) ||
                    (cid.CardKind == CardIndexData.Kind.beast && card.WidID == WidIDtype.召喚獣)) &&
                    cid.Name == card.CardName &&
                    cid.Expression == card.Expression)
                {
                    renamedCards.Remove(rd);
                    cid.Count--;
                    if (cid.Count == 0)
                    {
                        CardIndexDatas.Remove(cid);
                        WidCards.Remove(icd.WidData);

                        //~宿フォルダからリネームした名前のwidCardを削除
                        try
                        {
                            File.Delete(FilePath.GetNameWithWid(TempFolder, cid.RenamedName));
                        }
                        catch
                        {
                            PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                        }
                    }
                    return true;
                }
            }
            return false;
        }

        #endregion

        #region カード入れ替え

        #region １枚単位のカード入れ替え
        /// <summary>
        /// 宿カード置場カードの入れ替えを行う
        /// </summary>
        /// <param name="oldIndex">入れ替えるインデックス１</param>
        /// <param name="newIndex">入れ替えるインデックス２</param>
        public void SwapCard(int oldIndex, int newIndex)
        {
            SwapRenamedCard(BankBoxCards, oldIndex, newIndex);
        }

        /// <summary>
        /// 荷物袋カードの入れ替えを行う
        /// </summary>
        /// <param name="oldIndex">入れ替えるインデックス１</param>
        /// <param name="newIndex">入れ替えるインデックス２</param>
        public void SwapPartyBagCard(int oldIndex, int newIndex)
        {
            SwapRenamedCard(CurrentParty.BagCards, oldIndex, newIndex);
        }
        
        /// <summary>
        /// RenamedCardの入れ替えを行う
        /// </summary>
        /// <param name="list">入れ替える対象</param>
        /// <param name="oldIndex">入れ替えるインデックス１</param>
        /// <param name="newIndex">入れ替えるインデックス２</param>
        private void SwapRenamedCard(List<RenamedCardData> list,
            int oldIndex, int newIndex)
        {
            RenamedCardData temp = list[newIndex];
            list[newIndex] = list[oldIndex];
            list[oldIndex] = temp;
        }

        /// <summary>
        /// 宿カード置場カードの移動を行う
        /// </summary>
        /// <param name="oldIndex">移動元位置</param>
        /// <param name="insertPos">移動先位置</param>
        public void MoveCard(int oldIndex, int insertPos)
        {
            MoveRenamedCard(BankBoxCards, oldIndex, insertPos);
        }

        /// <summary>
        /// 荷物袋カードの移動を行う
        /// </summary>
        /// <param name="oldIndex">移動元位置</param>
        /// <param name="insertPos">移動先位置</param>
        public void MovePartyBagCard(int oldIndex, int insertPos)
        {
            MoveRenamedCard(CurrentParty.BagCards, oldIndex, insertPos);
        }

        /// <summary>
        /// RenamedCardの移動を行う
        /// </summary>
        /// <param name="list">移動対象</param>
        /// <param name="oldIndex">移動元位置</param>
        /// <param name="insertPos">移動先位置</param>
        private void MoveRenamedCard(List<RenamedCardData> list,
            int oldIndex, int insertPos)
        {
            if (insertPos == oldIndex ||
               insertPos == oldIndex + 1)
            {
                return;
            }

            if (insertPos > oldIndex)
            {
                insertPos--;
            }

            RenamedCardData temp = list[oldIndex];
            list.RemoveAt(oldIndex);

            if (insertPos < list.Count)
            {
                list.Insert(insertPos, temp);
            }
            else
            {
                list.Add(temp);
            }
        }


        /// <summary>
        /// キャラクタの所持カードの入れ替えを行う
        /// </summary>
        /// <param name="charNoOld">カレントパーティー内のキャラクタの位置old</param>
        /// <param name="charNoNew">カレントパーティー内のキャラクタの位置new</param>
        /// <param name="tefuda">手札種類</param>
        /// <param name="oldIndex">
        /// 入れ替えるインデックスold
        /// -1：最後尾に追加
        /// -2～-(index+2)～～-Count-1：index位置に挿入
        /// </param>
        /// <param name="newIndex">
        /// 入れ替えるインデックスnew
        /// </param>
        /// <returns>入れ替え成功 true、入れ替え失敗 false</returns>
        public bool SwapCharacterCard(int charNoOld,int charNoNew,手札種類 tefuda,int oldIndex, int newIndex)
        {
            CharacterData charDataOld = CurrentParty.Characters[charNoOld];
            CharacterData charDataNew = CurrentParty.Characters[charNoNew];

            int maxOld = (int)charDataOld.GetMaxHandCard(tefuda);
            int maxNew = (int)charDataNew.GetMaxHandCard(tefuda);

            List<WidCard> listOld = GetCardTypeList(charDataOld, tefuda);
            List<WidCard> listNew = GetCardTypeList(charDataNew, tefuda);

            WidCard tempOld;
            WidCard tempNew;
            if (oldIndex < 0)
            {
                if (charNoOld != charNoNew 
                    && maxOld == listOld.Count)
                {
                    // 移動元と先が違って、最大枚数の場合
                    return false;
                }

                tempNew = listNew[newIndex];
                
                if (charNoOld != charNoNew)
                {
                    // 移動元と先が違う場合
                    // ホールドは外しておく
                    tempNew.HoldCard = false;
                }

                listNew.RemoveAt(newIndex);

                if (oldIndex == -1 || oldIndex < -listOld.Count - 1)
                {
                    // 最後尾追加
                    listOld.Add(tempNew);
                }
                else
                {
                    // 指定位置に挿入
                    listOld.Insert(-oldIndex - 2, tempNew);
                }
            }
            else if (newIndex < 0)
            {
                if (charNoOld != charNoNew
                    && maxNew == listNew.Count)
                {
                    // 移動元と先が違って、最大枚数の場合
                    return false;
                }

                tempOld = listOld[oldIndex];
                
                if (charNoOld != charNoNew)
                {
                    // 移動元と先が違う場合
                    // ホールドは外しておく
                    tempOld.HoldCard = false;
                }
                
                listOld.RemoveAt(oldIndex);

                if (newIndex == -1 || newIndex < -listNew.Count - 1)
                {
                    // 最後尾追加
                    listNew.Add(tempOld);
                }
                else
                {
                    // 指定位置に挿入
                    listNew.Insert(-newIndex - 2, tempOld);
                }
            }
            else
            {
                tempOld = listOld[oldIndex];
                tempNew = listNew[newIndex];

                if (charNoOld != charNoNew)
                {
                    // 移動元と先が違う場合
                    // ホールドは外しておく
                    tempOld.HoldCard = false;
                    tempNew.HoldCard = false;
                }

                listOld[oldIndex] = tempNew;
                listNew[newIndex] = tempOld;

            }

            if (tefuda == 手札種類.所持スキル)
            {
                Util3.SetAllSkillRecover(charDataOld);
                Util3.SetAllSkillRecover(charDataNew);
            }

            return true;
        }

        /// <summary>
        /// 他宿CardとCardの入れ替えを行う
        /// </summary>
        /// <param name="oIndex">他宿Cardインデックス</param>
        /// <param name="cIndex">Cardインデックス</param>
        public void SwapOtherInnCardAndCard(int oIndex, int cIndex)
        {
            RenamedCardData innCard;
            RenamedCardData otherInnCard;
            IndexCardData innIcd;
            IndexCardData otherInnIcd;
            WidCard widCard;
            WidCard widCard2;

            if (oIndex == -1)
            {
                // 宿カードをリスト削除・ファイル削除
                innCard = BankBoxCards[cIndex];
                innIcd = FindIndexedCardData(innCard);

                // WidCard 複製
                widCard = new WidCard(innIcd.WidData);
                switch (innIcd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                DeleteRenamedCardData(BankBoxCards, BankBoxCards[cIndex]);

                // 他宿カードリスト追加・ファイル追加
                OtherInn.AddRenamedCardData(OtherInn.BankBoxCards, widCard, 1);

            }
            else if (cIndex == -1)
            {
                // 他宿カードリスト削除・ファイル削除
                otherInnCard = OtherInn.BankBoxCards[oIndex];
                otherInnIcd = OtherInn.FindIndexedCardData(otherInnCard);

                // WidCard 複製
                widCard2 = new WidCard(otherInnIcd.WidData);
                switch (otherInnIcd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard2.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard2.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard2.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard2.WidID == WidIDtype.アイテム)
                {
                    widCard2.ItemRemainUseCount = otherInnIcd.ItemData.UseCount;
                }

                OtherInn.DeleteRenamedCardData(OtherInn.BankBoxCards, OtherInn.BankBoxCards[oIndex]);

                // 宿カードリスト追加・ファイル追加
                AddRenamedCardData(BankBoxCards, widCard2, 1);
            }
            else
            {
                // 宿カードと他宿カードの入れ替え

                // ファイルはあらかじめ消しておく
                otherInnCard = OtherInn.BankBoxCards[oIndex];
                otherInnIcd = OtherInn.FindIndexedCardData(otherInnCard);

                // WidCard 複製
                widCard2 = new WidCard(otherInnIcd.WidData);
                switch (otherInnIcd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard2.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard2.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard2.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard2.WidID == WidIDtype.アイテム)
                {
                    widCard2.ItemRemainUseCount = otherInnIcd.ItemData.UseCount;
                }

                OtherInn.DeleteRenamedCardData(OtherInn.BankBoxCards, OtherInn.BankBoxCards[oIndex]);

                innCard = BankBoxCards[cIndex];
                innIcd = FindIndexedCardData(innCard);

                // WidCard 複製
                widCard = new WidCard(innIcd.WidData);
                switch (innIcd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                DeleteRenamedCardData(BankBoxCards, BankBoxCards[cIndex]);

                // お互いにファイルを追加

                OtherInn.AddRenamedCardData(OtherInn.BankBoxCards, widCard, 1);

                AddRenamedCardData(BankBoxCards, widCard2, 1);

                // リストの最後に追加されたので順番を上昇させる

                RenamedCardData temp;
                for (int i = BankBoxCards.Count - 1; i > cIndex; i--)
                {
                    temp = BankBoxCards[i];
                    BankBoxCards[i] = BankBoxCards[i - 1];
                    BankBoxCards[i - 1] = temp;
                }


                for (int i = OtherInn.BankBoxCards.Count - 1; i > oIndex; i--)
                {
                    temp = OtherInn.BankBoxCards[i];
                    OtherInn.BankBoxCards[i] = OtherInn.BankBoxCards[i - 1];
                    OtherInn.BankBoxCards[i - 1] = temp;
                }
            }

        }

        /// <summary>
        /// 荷物袋CardとCardの入れ替えを行う
        /// </summary>
        /// <param name="oIndex">荷物袋Cardインデックス</param>
        /// <param name="cIndex">Cardインデックス</param>
        public void SwapPartyBagCardAndCard(int oIndex, int cIndex)
        {
            RenamedCardData innCard;
            RenamedCardData partyBagCard;
            IndexCardData innIcd;
            IndexCardData partyBagIcd;
            WidCard widCard;
            WidCard widCard2;

            if (oIndex == -1)
            {
                // カード置場から荷物袋に移動

                // 宿カードをリスト削除・ファイル削除
                innCard = BankBoxCards[cIndex];
                innIcd = FindIndexedCardData(innCard);

                // WidCard 複製
                widCard = new WidCard(innIcd.WidData);
                switch (innIcd.CardData.CardKind)
	            {
		            case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                     break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                     break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                     break;
                    default:
                     break;
	            }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                DeleteRenamedCardData(BankBoxCards, BankBoxCards[cIndex]);

                // 荷物袋カードリスト追加・ファイル追加
                AddRenamedCardData(CurrentParty.BagCards, widCard, 1);

            }
            else if (cIndex == -1)
            {
                // 荷物袋カードリスト削除・ファイル削除
                partyBagCard = CurrentParty.BagCards[oIndex];
                partyBagIcd = FindIndexedCardData(partyBagCard);

                // WidCard 複製
                widCard = new WidCard(partyBagIcd.WidData);
                switch (partyBagIcd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = partyBagIcd.ItemData.UseCount;
                }

                DeleteRenamedCardData(CurrentParty.BagCards, CurrentParty.BagCards[oIndex]);

                // 宿カードリスト追加・ファイル追加
                AddRenamedCardData(BankBoxCards, widCard, 1);
            }
            else
            {
                // 宿カードと荷物袋カードの入れ替え

                // ファイルはあらかじめ消しておく
                partyBagCard = CurrentParty.BagCards[oIndex];
                partyBagIcd = FindIndexedCardData(partyBagCard);

                // WidCard 複製
                widCard2 = new WidCard(partyBagIcd.WidData);
                switch (partyBagIcd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard2.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard2.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard2.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard2.WidID == WidIDtype.アイテム)
                {
                    widCard2.ItemRemainUseCount = partyBagIcd.ItemData.UseCount;
                }

                DeleteRenamedCardData(CurrentParty.BagCards, CurrentParty.BagCards[oIndex]);

                innCard = BankBoxCards[cIndex];
                innIcd = FindIndexedCardData(innCard);

                // WidCard 複製
                widCard = new WidCard(innIcd.WidData);
                switch (innIcd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case CardIndexData.Kind.item:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case CardIndexData.Kind.beast:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                DeleteRenamedCardData(BankBoxCards, BankBoxCards[cIndex]);

                // お互いにファイルを追加

                AddRenamedCardData(CurrentParty.BagCards, widCard, 1);

                AddRenamedCardData(BankBoxCards, widCard2, 1);

                // リストの最後に追加されたので順番を上昇させる

                RenamedCardData temp;
                for (int i = BankBoxCards.Count - 1; i > cIndex; i--)
                {
                    temp = BankBoxCards[i];
                    BankBoxCards[i] = BankBoxCards[i - 1];
                    BankBoxCards[i - 1] = temp;
                }


                for (int i = CurrentParty.BagCards.Count - 1; i > oIndex; i--)
                {
                    temp = CurrentParty.BagCards[i];
                    CurrentParty.BagCards[i] = CurrentParty.BagCards[i - 1];
                    CurrentParty.BagCards[i - 1] = temp;
                }
            }

        }

        /// <summary>
        /// キャラクタ所持CardとCardの入れ替えを行う
        /// </summary>
        /// <param name="charNo">カレントパーティー内のキャラクタの位置</param>
        /// <param name="tefuda">手札種類</param>
        /// <param name="oIndex">キャラクタ所持Cardインデックス</param>
        /// <param name="cIndex">Cardインデックス</param>
        /// <returns>入れ替え成功 true、入れ替え失敗 false</returns>
        public bool SwapCharacterCardAndCard(int charNo,手札種類 tefuda,int oIndex, int cIndex)
        {
            RenamedCardData innCard;
            WidCard widCard;
            IndexCardData innIcd;
            CharacterData charData;
            List<WidCard> list;
            RenamedCardData temp;

            // 手札種類によって分岐
            charData = CurrentParty.Characters[charNo];

            list = GetCardTypeList(charData, tefuda);

            if (oIndex == -1)
            {
                // カード置場からキャラクタに移動

                // 最大枚数チェック
                if (list.Count >= charData.GetMaxHandCard(tefuda))
                {
                    // 最大枚数に到達しているので追加できない
                    return false;
                }

                // 宿カードを検索
                innCard = BankBoxCards[cIndex];
                innIcd = FindIndexedCardData(innCard);

                // WidCard 複製
                widCard = new WidCard(innIcd.WidData);
                switch (tefuda)
                {
                    case 手札種類.所持スキル:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case 手札種類.所持アイテム:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case 手札種類.所有召喚獣:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }
           
                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                // 宿カードをリスト削除・ファイル削除
                DeleteRenamedCardData(BankBoxCards, BankBoxCards[cIndex]);

                // キャラクタ所持カードリスト追加
                list.Add(widCard);
            }
            else if (cIndex == -1)
            {
                // キャラクタからカード置場に移動

                // キャラクタ所持カード検索
                widCard = list[oIndex];
                // ホールドは外しておく
                widCard.HoldCard = false;

                // キャラクタ所持カードリスト削除
                list.RemoveAt(oIndex);

                // 宿カードリスト追加・ファイル追加
                AddRenamedCardData(BankBoxCards, widCard, 1);
            }
            else
            {
                // 宿カードを検索
                innCard = BankBoxCards[cIndex];
                innIcd = FindIndexedCardData(innCard);

                // WidCard 複製
                WidCard innWidCard = new WidCard(innIcd.WidData);
                switch (tefuda)
                {
                    case 手札種類.所持スキル:
                        innWidCard.SetCardType(WidIDtype.スキル);
                        break;
                    case 手札種類.所持アイテム:
                        innWidCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case 手札種類.所有召喚獣:
                        innWidCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }

                // アイテムの場合は
                if (innWidCard.WidID == WidIDtype.アイテム)
                {
                    innWidCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                // 宿カードをリスト削除・ファイル削除
                DeleteRenamedCardData(BankBoxCards, BankBoxCards[cIndex]);

                // キャラクタ所持カード検索
                WidCard cWidCard = list[oIndex];
                // ホールドは外しておく
                cWidCard.HoldCard = false;

                // キャラクタ所持カードリスト削除
                list.RemoveAt(oIndex);

                // キャラクタ所持カードリストに挿入
                list.Insert(oIndex, innWidCard);

                //  宿カードリスト追加・ファイル追加
                AddRenamedCardData(BankBoxCards, cWidCard, 1);
                
                for (int i = BankBoxCards.Count - 1; i > cIndex; i--)
                {
                    temp = BankBoxCards[i];
                    BankBoxCards[i] = BankBoxCards[i - 1];
                    BankBoxCards[i - 1] = temp;
                }
            }

            if (tefuda == 手札種類.所持スキル)
            {
                Util3.SetAllSkillRecover(charData);
            }

            return true;

        }

        /// <summary>
        /// キャラクタ所持Cardをすべてカード置場に移動
        /// </summary>
        /// <param name="charNo">カレントパーティー内のキャラクタの位置</param>
        public void MoveAllCharacterCardToYado(int charNo)
        {
            CharacterData cd = this.CurrentParty.Characters[charNo];

            int count = 0;
            count = cd.Skills.Count;
            for (int i = count - 1; i >= 0; i--)
            {
                this.SwapCharacterCardAndCard(charNo, 手札種類.所持スキル, i, -1);
            }
            count = cd.Items.Count;
            for (int i = count - 1; i >= 0; i--)
            {
                this.SwapCharacterCardAndCard(charNo, 手札種類.所持アイテム, i, -1);
            }
            count = cd.Beasts.Count;
            for (int i = count - 1; i >= 0; i--)
            {
                this.SwapCharacterCardAndCard(charNo, 手札種類.所有召喚獣, i, -1);
            }
        }

        /// <summary>
        /// キャラクタ所持Cardと荷物袋Cardの入れ替えを行う
        /// </summary>
        /// <param name="charNo">カレントパーティー内のキャラクタの位置</param>
        /// <param name="tefuda">手札種類</param>
        /// <param name="oIndex">キャラクタ所持Cardインデックス</param>
        /// <param name="cIndex">荷物袋Cardインデックス</param>
        /// <returns>入れ替え成功 true、入れ替え失敗 false</returns>
        public bool SwapCharacterCardAndPartyBagCard(int charNo, 手札種類 tefuda, int oIndex, int cIndex)
        {
            RenamedCardData partyBagCard;
            WidCard widCard;
            IndexCardData innIcd;
         
            CharacterData charData;
            List<WidCard> list;

            RenamedCardData temp;

            // 手札種類によって分岐
            charData = CurrentParty.Characters[charNo];

            list = GetCardTypeList(charData, tefuda);

            if (oIndex == -1)
            {
                // 荷物袋からキャラクタに移動

                // 荷物袋カードを検索
                partyBagCard = CurrentParty.BagCards[cIndex];
                innIcd = FindIndexedCardData(partyBagCard);

                // WidCard 複製
                widCard = new WidCard(innIcd.WidData);
                switch (tefuda)
                {
                    case 手札種類.所持スキル:
                        widCard.SetCardType(WidIDtype.スキル);
                        break;
                    case 手札種類.所持アイテム:
                        widCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case 手札種類.所有召喚獣:
                        widCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }

                // アイテムの場合は
                if (widCard.WidID == WidIDtype.アイテム)
                {
                    widCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                // 最大枚数チェック
                if (list.Count >= charData.GetMaxHandCard(tefuda))
                {
                    // 最大枚数に到達しているので追加できない
                    return false;
                }

                // 荷物袋カードをリスト削除・ファイル削除
                DeleteRenamedCardData(CurrentParty.BagCards, CurrentParty.BagCards[cIndex]);

                // キャラクタ所持カードリスト追加
                list.Add(widCard);
            }
            else if (cIndex == -1)
            {
                // キャラクタから荷物袋に移動

                // キャラクタ所持カード検索
                widCard = list[oIndex];
                // ホールドは外しておく
                widCard.HoldCard = false;

                // キャラクタ所持カードリスト削除
                list.RemoveAt(oIndex);

                // 荷物袋カードリスト追加・ファイル追加
                AddRenamedCardData(CurrentParty.BagCards, widCard, 1);
            }
            else
            {
                // 荷物袋カードを検索
                partyBagCard = CurrentParty.BagCards[cIndex];
                innIcd = FindIndexedCardData(partyBagCard);

                // WidCard 複製
                WidCard innWidCard = new WidCard(innIcd.WidData);
                switch (tefuda)
                {
                    case 手札種類.所持スキル:
                        innWidCard.SetCardType(WidIDtype.スキル);
                        break;
                    case 手札種類.所持アイテム:
                        innWidCard.SetCardType(WidIDtype.アイテム);
                        break;
                    case 手札種類.所有召喚獣:
                        innWidCard.SetCardType(WidIDtype.召喚獣);
                        break;
                    default:
                        break;
                }

                // アイテムの場合は
                if (innWidCard.WidID == WidIDtype.アイテム)
                {
                    innWidCard.ItemRemainUseCount = innIcd.ItemData.UseCount;
                }

                // 荷物袋カードをリスト削除・ファイル削除
                DeleteRenamedCardData(CurrentParty.BagCards, CurrentParty.BagCards[cIndex]);

                // キャラクタ所持カード検索
                WidCard cWidCard = list[oIndex];
                // ホールドは外しておく
                cWidCard.HoldCard = false;

                // キャラクタ所持カードリスト削除
                list.RemoveAt(oIndex);

                // キャラクタ所持カードリストに挿入
                list.Insert(oIndex, innWidCard);

                //  荷物袋カードをリスト追加・ファイル追加
                AddRenamedCardData(CurrentParty.BagCards, cWidCard, 1);

                for (int i = CurrentParty.BagCards.Count - 1; i > cIndex; i--)
                {
                    temp = CurrentParty.BagCards[i];
                    CurrentParty.BagCards[i] = CurrentParty.BagCards[i - 1];
                    CurrentParty.BagCards[i - 1] = temp;
                }
            }

            if (tefuda == 手札種類.所持スキル)
            {
                Util3.SetAllSkillRecover(charData);
            }

            return true;

        }

        /// <summary>
        /// キャラクタ所持Cardをすべて荷物袋に移動
        /// </summary>
        /// <param name="charNo">カレントパーティー内のキャラクタの位置</param>
        public void MoveAllCharacterCardToPartyBag(int charNo)
        {
            CharacterData cd = this.CurrentParty.Characters[charNo];

            int count = 0;
            count = cd.Skills.Count;
            for (int i = count - 1; i >= 0; i--)
            {
                this.SwapCharacterCardAndPartyBagCard(charNo, 手札種類.所持スキル, i, -1);
            }
            count = cd.Items.Count;
            for (int i = count - 1; i >= 0; i--)
            {
                this.SwapCharacterCardAndPartyBagCard(charNo, 手札種類.所持アイテム, i, -1);
            }
            count = cd.Beasts.Count;
            for (int i = count - 1; i >= 0; i--)
            {
                this.SwapCharacterCardAndPartyBagCard(charNo, 手札種類.所有召喚獣, i, -1);
            }
        }

        public List<WidCard> GetCardTypeList(CharacterData cd, 手札種類 tefuda)
        {
            List<WidCard> list;
            switch (tefuda)
            {
                case 手札種類.所持スキル:
                    list = cd.Skills;
                    break;
                case 手札種類.所持アイテム:
                    list = cd.Items;
                    break;
                case 手札種類.所有召喚獣:
                    list = cd.Beasts;
                    break;
                default:
                    PartyMaker.CWIErrorMessage("ありえない手札種類です。（InnData:GetCardTypeList）");
                    return null;
            }

            return list;
        }

        #endregion

        #region カードのソート

        /// <summary>
        /// ソート対象をソートする(ver.2)
        /// </summary>
        /// <param name="sortObject">ソート対象</param>
        /// <param name="sortType">ソート種類</param>
        public void SortCards(ソート対象 sortObject, ソート種類 sortType)
        {
            List<RenamedCardData> list;
            if (sortObject == ソート対象.カード置場)
            {
                //カード置場
                list = BankBoxCards;
            }
            else
            {
                //荷物袋
                list = CurrentParty.BagCards;
            }

            List<RenamedCardData> listItem = new List<RenamedCardData>();
            List<RenamedCardData> listSkill = new List<RenamedCardData>();
            List<RenamedCardData> listBeast = new List<RenamedCardData>();

            List<IndexCardData> indexItem = new List<IndexCardData>();
            List<IndexCardData> indexSkill = new List<IndexCardData>();
            List<IndexCardData> indexBeast = new List<IndexCardData>();

            IndexCardData icd;
            foreach (RenamedCardData rcd in list)
            {
                icd = FindIndexedCardData(rcd);
                if (icd.Index == -1)
                {
                    PartyMaker.CWIErrorMessage("カード情報が取得できません（ソート）");
                }

                if (icd.WidData.WidID == WidIDtype.アイテム)
                {
                    listItem.Add(rcd);
                    indexItem.Add(icd);
                }
                else if (icd.WidData.WidID == WidIDtype.スキル)
                {
                    listSkill.Add(rcd);
                    indexSkill.Add(icd);
                }
                else if (icd.WidData.WidID == WidIDtype.召喚獣)
                {
                    listBeast.Add(rcd);
                    indexBeast.Add(icd);
                }
                else
                {

                    // データバージョン（CWエンジンの種類）によってはキャストが所持しているカードのWidID値が0クリア
                    // されていて、荷物袋やカード置場に移動する際に設定してあげないと
                    // いけないかもしれない。要注意
                    PartyMaker.CWIErrorMessage("カード種類が設定されていません（ソート）");
                }
            }

            RenamedCardData temp;
            IndexCardData tempIcd;
            //Item Sort
            for (int i = listItem.Count - 1; i > 0; i--)
            {
                for (int j = 0; j < i; j++)
                {
                    //名前順で比較
                    if (indexItem[j].WidData.CardName.CompareTo(indexItem[j+1].WidData.CardName) > 0)
                    {
                        //Swap rcd
                        temp = listItem[j];
                        listItem[j] = listItem[j+1];
                        listItem[j + 1] = temp;
                        //Swap icd
                        tempIcd = indexItem[j];
                        indexItem[j] = indexItem[j + 1];
                        indexItem[j + 1] = tempIcd;
                    }    
                }
            }
            //Swap Skill
            for (int i = listSkill.Count - 1; i > 0; i--)
            {
                for (int j = 0; j < i; j++)
                {
                    //名前順で比較
                    if (indexSkill[j].WidData.CardName.CompareTo(indexSkill[j + 1].WidData.CardName) > 0)
                    {
                        //Swap rcd
                        temp = listSkill[j];
                        listSkill[j] = listSkill[j + 1];
                        listSkill[j + 1] = temp;
                        //Swap icd
                        tempIcd = indexSkill[j];
                        indexSkill[j] = indexSkill[j + 1];
                        indexSkill[j + 1] = tempIcd;
                    }
                }
            }
            // Swap Beast
            for (int i = listBeast.Count - 1; i > 0; i--)
            {
                for (int j = 0; j < i; j++)
                {
                    //名前順で比較
                    if (indexBeast[j].WidData.CardName.CompareTo(indexBeast[j + 1].WidData.CardName) > 0)
                    {
                        //Swap rcd
                        temp = listBeast[j];
                        listBeast[j] = listBeast[j + 1];
                        listBeast[j + 1] = temp;
                        //Swap icd
                        tempIcd = indexBeast[j];
                        indexBeast[j] = indexBeast[j + 1];
                        indexBeast[j + 1] = tempIcd;
                    }
                }
            }

            // SortType 順に結合
            list.Clear();
            switch (sortType)
            {
                case ソート種類.アイテム_スキル_召喚獣:
                    list.AddRange(listItem);
                    list.AddRange(listSkill);
                    list.AddRange(listBeast);
                    break;
                case ソート種類.アイテム_召喚獣_スキル:
                    list.AddRange(listItem);
                    list.AddRange(listBeast);
                    list.AddRange(listSkill);
                    break;
                case ソート種類.スキル_召喚獣_アイテム:
                    list.AddRange(listSkill);
                    list.AddRange(listBeast);
                    list.AddRange(listItem);
                    break;
                case ソート種類.スキル_アイテム_召喚獣:
                    list.AddRange(listSkill);
                    list.AddRange(listItem);
                    list.AddRange(listBeast);
                    break;
                case ソート種類.召喚獣_アイテム_スキル:
                    list.AddRange(listBeast);
                    list.AddRange(listItem);
                    list.AddRange(listSkill);
                    break;
                case ソート種類.召喚獣_スキル_アイテム:
                    list.AddRange(listBeast);
                    list.AddRange(listSkill);
                    list.AddRange(listItem);
                    break;
                default:
                    // ありえない列挙型値です。
                    PartyMaker.CWIErrorMessage("ありえない列挙型値です（ソート）");
                    break;
            }
        }


        #endregion

        #endregion

        #region カード→PC→PT
        public void MakeShelf()
        {
            //カード置場
            List<RenamedCardData> list = new List<RenamedCardData>(BankBoxCards.ToArray());

            List<RenamedCardData> listItem = new List<RenamedCardData>();
            List<RenamedCardData> listSkill = new List<RenamedCardData>();
            List<RenamedCardData> listBeast = new List<RenamedCardData>();

            List<IndexCardData> indexItem = new List<IndexCardData>();
            List<IndexCardData> indexSkill = new List<IndexCardData>();
            List<IndexCardData> indexBeast = new List<IndexCardData>();

            IndexCardData icd;
            foreach (RenamedCardData rcd in list)
            {
                icd = FindIndexedCardData(rcd);
                if (icd.Index == -1)
                {
                    PartyMaker.CWIErrorMessage("カード情報が取得できません（ソート）");
                }

                if (icd.WidData.WidID == WidIDtype.アイテム)
                {
                    listItem.Add(rcd);
                    indexItem.Add(icd);
                }
                else if (icd.WidData.WidID == WidIDtype.スキル)
                {
                    listSkill.Add(rcd);
                    indexSkill.Add(icd);
                }
                else if (icd.WidData.WidID == WidIDtype.召喚獣)
                {
                    listBeast.Add(rcd);
                    indexBeast.Add(icd);
                }
                else
                {
                    // データバージョン（CWエンジンの種類）によってはキャストが所持しているカードのWidID値が0クリア
                    // されていて、荷物袋やカード置場に移動する際に設定してあげないと
                    // いけないかもしれない。要注意
                    PartyMaker.CWIErrorMessage("カード種類が設定されていません（ソート）");
                }

                // 宿カードをリスト削除・ファイル削除
                DeleteRenamedCardData(BankBoxCards, rcd);
            }

            int iCount = listItem.Count;
            int sCount = listSkill.Count;
            int bCount = listBeast.Count;

            int maxCount = 0;
            if (iCount >= sCount)
            {
                if (iCount >= bCount)
                {
                    maxCount = iCount;
                }
                else
                {
                    maxCount = bCount;
                }
            }
            else
            {
                if (sCount >= bCount)
                {
                    maxCount = sCount;
                }
                else
                {
                    maxCount = bCount;
                }
            }

            int boxCount = (maxCount + 9) / 10;

            int shelfCount = (boxCount + 5) / 6;

            if (shelfCount == 0)
            {
                PartyMaker.CWIErrorMessage("エラーです。（棚作成）");
            }

            using (CharacterDlg charDlg = new CharacterDlg(this))
            {
                int totalIndex = 0;
                for (int i = 0; i < shelfCount; i++)
                {
                    int pCharaCount = (boxCount >= 6) ? 6 : boxCount;
                    PartyData pd = new PartyData();

                    for (int j = 0; j < pCharaCount; j++)
                    {
                        CharacterData cd = charDlg.MakeBox();

                        for (int sIndex = totalIndex * 10
                            ; sIndex < totalIndex * 10 + 10
                            && sIndex < sCount; sIndex++)
                        {
                            cd.Skills.Add(indexSkill[sIndex].WidData);
                        }

                        for (int iIndex = totalIndex * 10
                            ; iIndex < totalIndex * 10 + 10
                            && iIndex < iCount; iIndex++)
                        {
                            cd.Items.Add(indexItem[iIndex].WidData);
                        }

                        for (int bIndex = totalIndex * 10
                            ; bIndex < totalIndex * 10 + 10
                            && bIndex < bCount; bIndex++)
                        {
                            cd.Beasts.Add(indexBeast[bIndex].WidData);
                        }

                        pd.Characters.Add(cd);
                        totalIndex ++;
                    }

                    // パーティーリストに棚と名前を付けて追加
                    pd.PartyName = "棚";
                    Parties.Add(pd);

                    // パーティーリスト中のインデックス取得
                    int pIndex = GetPartyIndex(pd);

                    // パーティファイル追加
                    AddParyFile(pd);

                    // 保存したことでファイル名が付いているので、リストに関連付け直し
                    Parties[pIndex] = pd;

                    boxCount -= 6;
                    if (boxCount <= 0)
                    {
                        break;
                    }
                }
            }

        }

        public void AllBoxPartyBreakUp()
        {
            List<PartyData> pts = new List<PartyData>(Parties.ToArray());

            // カレントパーティーは対象から外す
            pts.Remove(CurrentParty);

            foreach (PartyData pd in pts)
            {
                if (pd.PartyName != "棚" && pd.PartyName != "箱一行")
                {
                    continue;
                }

                // ファイル削除とリスト削除
                int pIndex = GetPartyIndex(pd);
                DeletePartyFile(pd);
                Parties.RemoveAt(pIndex);

                foreach (CharacterData cd in pd.Characters)
                {
                    if (cd.CardName == "箱")
                    {
                        // アイテムを取り出した後
                        // オブジェクトはパーティーとともにガベージコレクト
                        foreach (WidCard wc in cd.Skills)
                        {
                            // ホールドは外しておく
                            wc.HoldCard = false;

                            // 宿カードリスト追加・ファイル追加
                            AddRenamedCardData(BankBoxCards, wc, 1);

                        } 
                        
                        foreach (WidCard wc in cd.Items)
                        {
                            // ホールドは外しておく
                            wc.HoldCard = false;

                            // 宿カードリスト追加・ファイル追加
                            AddRenamedCardData(BankBoxCards, wc, 1);

                        } 
                        
                        foreach (WidCard wc in cd.Beasts)
                        {
                            // ホールドは外しておく
                            wc.HoldCard = false;

                            // 宿カードリスト追加・ファイル追加
                            AddRenamedCardData(BankBoxCards, wc, 1);

                        }

                    }
                    else
                    {
                        // 箱以外のパーティーが混ざっていた場合
                        // パーティーPCを待機PCに
                        cd.FileName = string.Empty; // パーティーPCなので元ファイルはない
                        // ファイル作成
                        AddCharacterFile(cd);
                        //キャラクタリスト追加
                        Characters.Add(cd);
                    }
                }

                // パーティーアイテムを宿の倉庫へ
                BankBoxCards.AddRange(pd.BagCards);

                // パーティー所持金を宿の金庫へ
                YadoMoney += pd.Money;
            }
        }
        #endregion

        #region パーティー関連

        /// <summary>
        /// カレントパーティーを~の付いていない宿にセーブ
        /// </summary>
        public void SaveCurrentPartyPrimeFile()
        {
            // カレントパーティファイル追加
            AddPartyCommon(CurrentParty, folder);
        }

        /// <summary>
        /// カレントパーティーをセーブ
        /// </summary>
        public void SaveCurrentParty()
        {
            // パーティーリスト中のインデックス取得
            int index = GetPartyIndex(CurrentParty);
            // カレントパーティファイル追加
            AddParyFile(CurrentParty);
            // リストに関連付け
            Parties[index] = CurrentParty;
        }

        /// <summary>
        /// パーティーを~宿にファイル追加
        /// </summary>
        /// <param name="party"></param>
        private void AddParyFile(PartyData party) 
        {
            AddPartyCommon(party, tempFolder);
        }

        /// <summary>
        /// パーティーを~の付いていない宿にファイル直接追加
        /// </summary>
        /// <param name="party">パーティーデータ</param>
        /// <param name="yadoName">宿名</param>
        private void AddPrimePartyFile(PartyData party, string yadoName)
        {
            AddPartyCommon(party, FilePath.GetSystemYadoFolder() + "\\" + yadoName);
        }

        /// <summary>
        /// パーティーファイル追加（共通部）
        /// </summary>
        /// <param name="party"></param>
        /// <param name="folderPath"></param>
        private void AddPartyCommon(PartyData party,string folderPath)
        {
            try
            {
                // あったら元のファイル削除
                DeletePartyFile(party);

                string filePath;
                string indexFilePath;

                filePath = FilePath.GetPartyFileName(folderPath, party.PartyName);

                // 書き込んだファイル名を取得
                party.FileName = filePath;
                party.Write(filePath);

                // PartyIndexData
                indexFilePath = Path.ChangeExtension(filePath, FilePath.ExtIndexPartyData);
                PartyIndexData pid = Util1.MakePartyIndexData(this, party);

                // Save
                pid.Write(indexFilePath);

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("パーティ情報の書き込みに失敗：" + ex.Message);
            }
        }

        /// <summary>
        /// 全パーティー読み出し
        /// </summary>
        private void LoadAllParty(DirectoryInfo di)
        {
            try
            {
                Parties.Clear();

                // パーティー
                foreach (FileInfo fi in di.GetFiles(FilePath.MaskPartyData))
                {
                    Parties.Add(new PartyData(fi.FullName));
                }

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("パーティーの読み込みに失敗しました。" + ex.Message);
            }

        }

        /// <summary>
        /// パーティーリストからパーティー選択
        /// </summary>
        /// <param name="partyName"></param>
        /// <returns></returns>
        private PartyData SelectAutoParty(string AutoPartyRenameName)
        {
            string filePath = tempFolder + "\\" + AutoPartyRenameName + FilePath.ExtPartyData;
           
            foreach (PartyData pd in Parties)
            {
                if (pd.FileName == filePath)
                {
                    return pd;
                }
            }

            PartyMaker.CWIErrorMessage("パーティーリストにパーティーは存在しません。");
            return null;
        }

        /// <summary>
        /// パーティーリストからパーティーのインデックス取得
        /// </summary>
        /// <param name="partyData"></param>
        /// <returns></returns>
        public int GetPartyIndex(PartyData partyData)
        {
            for(int i = 0; i < Parties.Count;i++)
            {
                if (Parties[i] == partyData)
                {
                    return i;
                }
            }

            // 消さないように
            PartyMaker.CWIErrorMessage("パーティ情報がリスト中に見つかりません。");
            return -1;
        }

        /// <summary>
        /// カレントパーティー解散
        /// </summary>
        public void BreakParty()
        {
            int index = GetPartyIndex(CurrentParty);

            PartyData party = Parties[index];

            foreach (CharacterData cd in party.Characters)
            {
                // パーティーPCを待機PCに
                // ファイル作成
                cd.FileName = string.Empty; // パーティーPCなので元ファイルはない
                AddCharacterFile(cd);
                //キャラクタリスト追加
                Characters.Add(cd);
            }

            List<CharacterData> list = party.Characters;

            // パーティーアイテムを宿の倉庫へ
            BankBoxCards.AddRange(party.BagCards);

            // パーティー所持金を宿の金庫へ
            YadoMoney += party.Money;

            // パーティーファイル削除
            DeletePartyFile(party);

            // パーティーリストから外す
            Parties.RemoveAt(index);

            // カレントパーティークリア
            CurrentParty = null;

        }

        /// <summary>
        /// カレントでないパーティー解散
        /// </summary>
        public void BreakOtherParty(PartyData party)
        {

            foreach (CharacterData cd in party.Characters)
            {
                // パーティーPCを待機PCに
                // ファイル作成
                cd.FileName = string.Empty; // パーティーPCなので元ファイルはない
                AddCharacterFile(cd);
                //キャラクタリスト追加
                Characters.Add(cd);
            }

            // パーティーアイテムを宿の倉庫へ
            BankBoxCards.AddRange(party.BagCards);

            // パーティー所持金を宿の金庫へ
            YadoMoney += party.Money;

            // パーティーファイル削除
            DeletePartyFile(party);

            // パーティーリストから外す
            Parties.Remove(party);

        }

        /// <summary>
        /// カレントでないパーティーからキャラクタが１人離脱
        /// </summary>
        public void RemoveOtherPartyChara(PartyData party,CharacterData chara)
        {
            //PartiesでのIndex
            int index = this.GetPartyIndex(party);

            // 一度パーティーファイル削除
            this.DeletePartyFile(party);
            
            // パーティーPCを待機PCに
            // ファイル作成
            chara.FileName = string.Empty; // パーティーPCなので元ファイルはない
            this.AddCharacterFile(chara);
            //キャラクタリスト追加
            this.Characters.Add(chara);

            // パーティーから削除
            party.Characters.Remove(chara);

            // パーティーファイル追加（現状でファイル作成）
            this.AddParyFile(party);

            // 問題ないとは思うが再度参照
            Parties[index] = party;

        }

        /// <summary>
        /// カレントパーティー以外のパーティーを全解散させる
        /// </summary>
        /// <param name="removeCard">手持ちのカードをカード置場に移動させる？</param>
        public void BreakAllParty(bool removeCard)
        {
            List<PartyData> pts = new List<PartyData>(Parties.ToArray());

            // カレントパーティーは対象から外す
            pts.Remove(CurrentParty);

            foreach (PartyData pd in pts)
            {
                int pIndex = GetPartyIndex(pd);
                DeletePartyFile(pd);
                Parties.RemoveAt(pIndex);

                if (removeCard)
                {
                    foreach (CharacterData cd in pd.Characters)
                    {
                        // パーティーPCを待機PCに
                        // ファイル作成
                        cd.FileName = string.Empty; // パーティーPCなので元ファイルはない

                        // 手持ちの全カードをカード置場へ移動
                        foreach (WidCard skill in cd.Skills)
                        {
                            // ホールドは外しておく
                            skill.HoldCard = false;

                            // 宿カードリスト追加・ファイル追加
                            AddRenamedCardData(this.BankBoxCards, skill, 1);
                        }

                        // キャラクタ所持カードリスト削除
                        cd.Skills.Clear();

                        foreach (WidCard item in cd.Items)
                        {
                            // ホールドは外しておく
                            item.HoldCard = false;

                            // 宿カードリスト追加・ファイル追加
                            AddRenamedCardData(this.BankBoxCards, item, 1);
                        }

                        // キャラクタ所持カードリスト削除
                        cd.Items.Clear();

                        foreach (WidCard beast in cd.Beasts)
                        {
                            // ホールドは外しておく
                            beast.HoldCard = false;

                            // 宿カードリスト追加・ファイル追加
                            AddRenamedCardData(this.BankBoxCards, beast, 1);
                        }

                        // キャラクタ所持カードリスト削除
                        cd.Beasts.Clear();

                        AddCharacterFile(cd);
                        //キャラクタリスト追加
                        Characters.Add(cd);
                    }
                }
                else
                {
                    foreach (CharacterData cd in pd.Characters)
                    {
                        // パーティーPCを待機PCに
                        // ファイル作成
                        cd.FileName = string.Empty; // パーティーPCなので元ファイルはない
                        AddCharacterFile(cd);
                        //キャラクタリスト追加
                        Characters.Add(cd);
                    }
                }

                // パーティーアイテムを宿の倉庫へ
                BankBoxCards.AddRange(pd.BagCards);

                // パーティー所持金を宿の金庫へ
                YadoMoney += pd.Money;
            }
        }

        /// <summary>
        /// パーティー作成
        /// </summary>
        /// <param name="list">キャラクタリスト</param>
        public void CreateParty(List<CharacterData> list)
        {
            // パーティー作成する前には必ず
            // 解散もしくはセーブ後 CurrentParty == null にしてあること
            if (list.Count > 0)
            {
                PartyData pd = new PartyData();
                foreach (CharacterData cd in list)
                {
                    int index = GetCharacterIndex(cd);

                    // ファイル削除
                    DeleteCharacterFile(cd);

                    // リストから参照削除
                    Characters.RemoveAt(index);

                    // Partyに追加するときにはFileNameをクリアしておく
                    cd.FileName = string.Empty;
                    pd.Characters.Add(cd);
                }

                pd.PartyName = pd.Characters[0].CardName + FilePath.PartyInitialNameEnd;
                Parties.Add(pd);
                CurrentParty = pd;

            }
            else
            {

                PartyMaker.CWIErrorMessage("キャラクタを選択せずにパーティーを作成しようとしました。" 
                    + System.Reflection.MethodBase.GetCurrentMethod().Name);
            }
                       
        }

        /// <summary>
        /// 全待機PCをパーティー化
        /// </summary>
        /// <param name="moveCard">カード置場のカードを詰められるだけ詰める？</param>
        public void CreatePartyFromAllPC(bool moveCard)
        {
            PartyData pd;
            CharacterData setCd = null;
            int pcCount = Characters.Count;
            List<CharacterData> charaList = new List<CharacterData>();
            if (pcCount > 0)
            {
                if (moveCard)
                {
                    //カード置場
                    List<RenamedCardData> list = new List<RenamedCardData>(BankBoxCards.ToArray());

                    List<RenamedCardData> listItem = new List<RenamedCardData>();
                    List<RenamedCardData> listSkill = new List<RenamedCardData>();
                    List<RenamedCardData> listBeast = new List<RenamedCardData>();

                    List<IndexCardData> indexItem = new List<IndexCardData>();
                    List<IndexCardData> indexSkill = new List<IndexCardData>();
                    List<IndexCardData> indexBeast = new List<IndexCardData>();

                    IndexCardData icd;
                    foreach (RenamedCardData rcd in list)
                    {
                        icd = FindIndexedCardData(rcd);
                        if (icd.Index == -1)
                        {
                            PartyMaker.CWIErrorMessage("カード情報が取得できません（ソート）");
                        }

                        if (icd.WidData.WidID == WidIDtype.アイテム)
                        {
                            listItem.Add(rcd);
                            indexItem.Add(icd);
                        }
                        else if (icd.WidData.WidID == WidIDtype.スキル)
                        {
                            listSkill.Add(rcd);
                            indexSkill.Add(icd);
                        }
                        else if (icd.WidData.WidID == WidIDtype.召喚獣)
                        {
                            listBeast.Add(rcd);
                            indexBeast.Add(icd);
                        }
                        else
                        {
                            // データバージョン（CWエンジンの種類）によってはキャストが所持しているカードのWidID値が0クリア
                            // されていて、荷物袋やカード置場に移動する際に設定してあげないと
                            // いけないかもしれない。要注意
                            PartyMaker.CWIErrorMessage("カード種類が設定されていません（ソート）");
                        }

                        //ここでは削除しない。
                    }

                    // カード置場でのそれぞれのカード種類の数
                    int iCount = listItem.Count;
                    int sCount = listSkill.Count;
                    int bCount = listBeast.Count;
                    int iIndex = 0;
                    int sIndex = 0;
                    int bIndex = 0;
                    int setCount = 0;

                    while (Characters.Count > 0)
                    {
                        charaList.Clear();

                        for (int i = 0; i < 6 && i < Characters.Count; i++)
                        {
                            setCd = Characters[i];

                            // ここでカードを詰められるだけ詰める。
                            setCount = (int)setCd.GetMaxHandCard(手札種類.所持スキル)
                                - setCd.Skills.Count;
                            for (int j = 0; j < setCount && sIndex < sCount; j++, sIndex++)
                            {
                                indexSkill[sIndex].WidData.FileName = string.Empty;
                                setCd.Skills.Add(indexSkill[sIndex].WidData);
                            }

                            setCount = (int)setCd.GetMaxHandCard(手札種類.所持アイテム)
                                - setCd.Items.Count;
                            for (int j = 0; j < setCount && iIndex < iCount; j++, iIndex++)
                            {
                                indexItem[iIndex].WidData.FileName = string.Empty;
                                setCd.Items.Add(indexItem[iIndex].WidData);
                            }

                            setCount = (int)setCd.GetMaxHandCard(手札種類.所有召喚獣)
                                - setCd.Beasts.Count;
                            for (int j = 0; j < setCount && bIndex < bCount; j++, bIndex++)
                            {
                                indexBeast[bIndex].WidData.FileName = string.Empty;
                                setCd.Beasts.Add(indexBeast[bIndex].WidData);
                            }

                            charaList.Add(setCd);
                        }

                        pd = new PartyData();

                        foreach (CharacterData cd in charaList)
                        {
                            int index = GetCharacterIndex(cd);

                            // ファイル削除
                            DeleteCharacterFile(cd);

                            // リストから参照削除
                            Characters.RemoveAt(index);

                            // Partyに追加するときにはFileNameをクリアしておく
                            cd.FileName = string.Empty;
                            pd.Characters.Add(cd);
                        }

                        // パーティーリストにパーティー名を付けて追加
                        pd.PartyName = pd.Characters[0].CardName + FilePath.PartyInitialNameEnd;
                        Parties.Add(pd);

                        // パーティーリスト中のインデックス取得
                        int pIndex = GetPartyIndex(pd);
                        // パーティファイル追加
                        AddParyFile(pd);
                        // 保存したことでファイル名が付いているので、リストに関連付け直し
                        Parties[pIndex] = pd;
                    }

                    // 宿カードをリスト削除・ファイル削除
                    for (int j = 0; j < sIndex; j++)
                    {
                        DeleteRenamedCardData(BankBoxCards, listSkill[j]);
                    }
                    for (int j = 0; j < iIndex; j++)
                    {
                        DeleteRenamedCardData(BankBoxCards, listItem[j]);
                    } 
                    for (int j = 0; j < bIndex; j++)
                    {
                        DeleteRenamedCardData(BankBoxCards, listBeast[j]);
                    }

                }
                else
                {
                    while (Characters.Count > 0)
                    {
                        charaList.Clear();

                        for (int i = 0; i < 6 && i < Characters.Count; i++)
                        {
                            setCd = Characters[i];
                            charaList.Add(setCd);
                        }


                        pd = new PartyData();

                        foreach (CharacterData cd in charaList)
                        {
                            int index = GetCharacterIndex(cd);

                            // ファイル削除
                            DeleteCharacterFile(cd);

                            // リストから参照削除
                            Characters.RemoveAt(index);

                            // Partyに追加するときにはFileNameをクリアしておく
                            cd.FileName = string.Empty;
                            pd.Characters.Add(cd);
                        }

                        // パーティーリストにパーティー名を付けて追加
                        pd.PartyName = pd.Characters[0].CardName + FilePath.PartyInitialNameEnd;
                        Parties.Add(pd);

                        // パーティーリスト中のインデックス取得
                        int pIndex = GetPartyIndex(pd);
                        // パーティファイル追加
                        AddParyFile(pd);
                        // 保存したことでファイル名が付いているので、リストに関連付け直し
                        Parties[pIndex] = pd;
                    }
                }
            }
            else
            {

                PartyMaker.CWIErrorMessage("待機PCがいないのにパーティーを作成しようとしました。"
                    + System.Reflection.MethodBase.GetCurrentMethod().Name);
            }
        }

        /// <summary>
        /// カレントパーティー宿移動
        /// </summary>
        /// <param name="data">移動先宿選択データ</param>
        public void MoveCurrentPartyToOtherInn(YadoSelectData data)
        {

            int rIndex = GetPartyIndex(CurrentParty);

            if (rIndex >= 0 && rIndex < Parties.Count)
            {
                // オブジェクトは保持しておく
                PartyData party = CurrentParty;

                //CurrentPartyから荷物袋カードを全部取り出す。

                // リストコピー
                List<RenamedCardData> localList = new List<RenamedCardData>();
                localList.AddRange(CurrentParty.BagCards);

                List<WidCard> cardList = new List<WidCard>();
                IndexCardData icd;
                foreach (RenamedCardData rcd in localList)
                {
                    icd = FindIndexedCardData(rcd.RenamedName);

                    // 荷物袋のファイルとリスト削除
                    DeleteRenamedCardData(CurrentParty.BagCards, rcd);

                    // カードを取っておく
                    cardList.Add(icd.WidData);

                }

                CurrentParty.BagCards.Clear();


                // まずSave する前にCurrentPartyを外しておく
                // ファイル削除(元々ないが安全のため)
                this.DeletePartyFile(CurrentParty);

                // パーティーリストから削除
                this.Parties.RemoveAt(rIndex);
                // カレントパーティークリア
                this.CurrentParty = null;


                // 移動先宿
                otherInn = new InnData(data);

                // ~移動先宿作成
                DirectoryInfo di = otherInn.CopyFolderToTemp();

                otherInn.ReadParties(di);

                otherInn.AfterTempFolderProc();

                if (otherInn.CurrentParty != null)
                {
                    // CurrentPartyでなくす。
                    otherInn.SaveCurrentParty();
                    // カレントパーティー変更
                    otherInn.CurrentParty = null;
                    
                }

                // カレントパーティー変更
                otherInn.Parties.Add(party);

                otherInn.CurrentParty = party;
                // カレントパーティーはファイルがあってはいけない
                // 来たばかりなので無い。

                foreach (WidCard wc in cardList)
                {
                    // 他宿のカレントパーティーの荷物袋のファイルとリスト追加

                    otherInn.AddRenamedCardData(otherInn.CurrentParty.BagCards,
                        wc, 1);
                }
                
                // カレントパーティーを削除した状態でSave 
                Save();

                // 他宿もセーブ
                otherInn.Save();
            }
            else
            {
                // 消さないように
                PartyMaker.CWIErrorMessage("パーティー情報がパーティーリスト中に見つかりません。(宿移動)");
            }
        }

        /// <summary>
        /// パーティーファイルの削除
        /// </summary>
        /// <param name="cd"></param>
        public void DeletePartyFile(PartyData party)
        {
            string path;
            string indexPath;

            if (party != null && party.FileName != string.Empty)
            {
                path = party.FileName;
                indexPath = Path.ChangeExtension(path, FilePath.ExtIndexPartyData);
                FilePath.DeleteFile(path);
                FilePath.DeleteFile(indexPath);
            }

        }

        /// <summary>
        /// 宿にある同じパーティーファイルの削除
        /// </summary>
        /// <param name="cd">保存済で宿と~宿で同期が取れているパーティーデータ</param>
        private void DeletePrimePartyFile(PartyData party)
        {
            string path;
            string indexPath;

            if (party.FileName != string.Empty)
            {

                path = Folder + "\\" + FilePath.FileNameFromPath(party.FileName);
                indexPath = Path.ChangeExtension(path, FilePath.ExtIndexPartyData);
                FilePath.DeleteFile(path);
                FilePath.DeleteFile(indexPath);
            }
        }

        #endregion

        #region Character 関連

        #region Character 入れ替え
        /// <summary>
        /// パーティーキャラクタリストの入れ替えを行う
        /// </summary>
        /// <param name="oldIndex">入れ替えるインデックス１</param>
        /// <param name="newIndex">入れ替えるインデックス２</param>
        public void SwapPartyCharacter(int oldIndex, int newIndex)
        {
            List<CharacterData> list = CurrentParty.Characters;
            
            CharacterData temp = list[newIndex];
            list[newIndex] = list[oldIndex];
            list[oldIndex] = temp;
        }

        /// <summary>
        /// パーティーキャラクタの挿入移動を行う
        /// </summary>
        /// <param name="oldIndex">元の位置</param>
        /// <param name="insertPos">挿入位置</param>
        public void MovePartyCharacter(int oldIndex, int insertPos)
        {
            if (insertPos == oldIndex 
                || insertPos == (oldIndex + 1))
            {
                // 移動はしない
                return;
            }

            List<CharacterData> list = CurrentParty.Characters;
            int count = list.Count;
            CharacterData cd = list[oldIndex];

            list.RemoveAt(oldIndex);

            if (insertPos == count)
            {
                // 挿入位置が最後尾
                list.Add(cd);
            } 
            else if (insertPos < oldIndex)
            {
                // 挿入位置が元の位置より前
                list.Insert(insertPos, cd);
            }
            else
            {
                // 挿入位置が元の位置より後
                // 自分を削除した分挿入位置が１つ前にずれる
                list.Insert(insertPos - 1, cd);

            }

        }

        /// <summary>
        /// キャラクタリストの入れ替えを行う
        /// </summary>
        /// <param name="oldIndex">入れ替えるインデックス１</param>
        /// <param name="newIndex">入れ替えるインデックス２</param>
        public void SwapCharacter(int oldIndex, int newIndex)
        {
            List<CharacterData> list = Characters;

            CharacterData temp = list[newIndex];
            list[newIndex] = list[oldIndex];
            list[oldIndex] = temp;
        }

        /// <summary>
        /// 他宿待機PCリストの入れ替えを行う
        /// </summary>
        /// <param name="oldIndex">入れ替えるインデックス１</param>
        /// <param name="newIndex">入れ替えるインデックス２</param>
        public void SwapOtherInnCharacter(int oldIndex, int newIndex)
        {
            List<CharacterData> list = OtherInn.Characters;

            CharacterData temp = list[newIndex];
            list[newIndex] = list[oldIndex];
            list[oldIndex] = temp;
        }

        /// <summary>
        /// パーティーキャラクタとキャラクタの入れ替えを行う
        /// カレントパーティーに１人以上いる場合の処理
        /// </summary>
        /// <param name="pIndex">パーティーインデックス</param>
        /// <param name="cIndex">キャラクタインデックス</param>
        public void SwapPartyPCAndPC(int pIndex, int cIndex)
        {
            CharacterData innPC;
            CharacterData partyPC;
            if (pIndex == -1)
            {
                // 待機PCをキャラクタリストから外す
                innPC = Characters[cIndex];
                Characters.RemoveAt(cIndex);
                // ファイルは消しておく
                DeleteCharacterFile(innPC);
                // パーティーにPC移動
                innPC.FileName = string.Empty;
                CurrentParty.Characters.Add(innPC);

            }
            else if (pIndex < -1 && pIndex > - CurrentParty.Characters.Count - 2)
            {
                // InsertPCtoParty

                // 待機PCをキャラクタリストから外す
                innPC = Characters[cIndex];
                Characters.RemoveAt(cIndex);

                // ファイルは消しておく
                DeleteCharacterFile(innPC);
                // パーティーにPC挿入
                innPC.FileName = string.Empty;

                //   0    1    2 ..      index ...     Count-1
                //-2   -3   -4   -(index+2)  ...  -Count-1 | -1

                CurrentParty.Characters.Insert( - pIndex - 2 , innPC);

            }
            else if (cIndex == -1)
            {
                // パーティーPCの離脱
                partyPC = CurrentParty.Characters[pIndex];
                CurrentParty.Characters.RemoveAt(pIndex);
                // 待機PCファイル生成
                partyPC.FileName = string.Empty;
                AddCharacterFile(partyPC);
                //キャラクタリスト追加
                Characters.Add(partyPC);

            }
            else
            {
                // 待機PCとパーティーPCの入れ替え

                partyPC = CurrentParty.Characters[pIndex];
                innPC = Characters[cIndex];
                partyPC.FileName = string.Empty;
                ReplaceCharacter(innPC, partyPC);

                // パーティーにPC移動
                innPC.FileName = string.Empty;
                CurrentParty.Characters[pIndex] = innPC;

            }

        }

        /// <summary>
        /// 他宿待機PCキャラクタとキャラクタの入れ替えを行う
        /// </summary>
        /// <param name="rIndex">他宿待機PCキャラクタインデックス</param>
        /// <param name="cIndex">キャラクタインデックス</param>
        public void SwapOtherPCAndPC(int oIndex, int cIndex)
        {
            CharacterData innPC;
            CharacterData otherInnPC;
            if (oIndex == -1)
            {
                // 待機PCをキャラクタリストから外す
                innPC = Characters[cIndex];
                Characters.RemoveAt(cIndex);
                // 他宿待機PCファイル生成
                OtherInn.AddCharacterFile(innPC);
                // キャラクタリスト追加
                OtherInn.Characters.Add(innPC);
            }
            else if (cIndex == -1)
            {
                // 他宿待機PCを他宿待機PCキャラクタリストから外す
                otherInnPC = OtherInn.Characters[oIndex];
                OtherInn.Characters.RemoveAt(oIndex);
                // 待機PCファイル生成
                AddCharacterFile(otherInnPC);
                // キャラクタリスト追加
                Characters.Add(otherInnPC);
            }
            else
            {
                // 待機PCと他宿待機PCの入れ替え

                // ファイルはあらかじめ消しておく
                otherInnPC = OtherInn.Characters[oIndex];
                DeleteCharacterFile(otherInnPC);
                innPC = Characters[cIndex];
                DeleteCharacterFile(innPC);

                // お互いにファイルを追加
                AddCharacterFile(otherInnPC);
                OtherInn.AddCharacterFile(innPC);

                // 入れ替え
                OtherInn.Characters[oIndex] = innPC;
                Characters[cIndex] = otherInnPC;
            }

        }

        /// <summary>
        /// 待機PCのソートです。
        /// </summary>
        /// <param name="sortType"></param>
        /// <param name="selectString"></param>
        public void PCSort(PCソート種類 sortType,String selectString)
        {
            List<CharacterData> list = new List<CharacterData>(Characters.ToArray());

            CharacterData temp;
            //Character Sort
            for (int i = list.Count - 1; i > 0; i--)
            {
                for (int j = 0; j < i; j++)
                {
                    // 名前順で比較
                    // >0 の時に交換するので昇順
                    if (list[j].CardName.CompareTo(list[j + 1].CardName) > 0)
                    {
                        //Swap Character
                        temp = list[j];
                        list[j] = list[j + 1];
                        list[j + 1] = temp;
                    }
                }
            }

            switch (sortType)
            {
                case PCソート種類.名前昇順:
                    Characters = list;
                    return;
                case PCソート種類.名前降順:
                    list.Reverse();
                    Characters = list;
                    return;
                default:
                    break;
            }

            if (sortType == PCソート種類.クーポン)
            {
                List<CharacterData> listAri = new List<CharacterData>();
                List<CharacterData> listNashi = new List<CharacterData>();
                bool isOK = false;
                foreach (CharacterData cd in list)
                {
                    isOK = false;
                    foreach (CouponData coup in cd.Coupons)
                    {
                        if (coup.Name.Contains(selectString))
                        {
                            isOK = true;
                            break;
                        }
                    }

                    if (isOK)
                    {
                        listAri.Add(cd);
                    }
                    else
                    {
                        listNashi.Add(cd);
                    }
                }

                list.Clear();
                list.AddRange(listAri);
                list.AddRange(listNashi);

                Characters = list;
                
            }
            else if (sortType == PCソート種類.カード名)
            {
                List<CharacterData> listAri = new List<CharacterData>();
                List<CharacterData> listNashi = new List<CharacterData>();
                bool isOK = false;
                foreach (CharacterData cd in list)
                {
                    isOK = false;

                    foreach (WidCard wc in cd.Skills)
                    {
                        if (wc.CardName.Contains(selectString))
                        {
                            isOK = true;
                            break;
                        }
                    }

                    if (!isOK)
                    {
                        foreach (WidCard wc in cd.Items)
                        {
                            if (wc.CardName.Contains(selectString))
                            {
                                isOK = true;
                                break;
                            }
                        }
                    }

                    if (!isOK)
                    {
                        foreach (WidCard wc in cd.Beasts)
                        {
                            if (wc.CardName.Contains(selectString))
                            {
                                isOK = true;
                                break;
                            }
                        }
                    }

                    if (isOK)
                    {
                        listAri.Add(cd);
                    }
                    else
                    {
                        listNashi.Add(cd);
                    }
                }

                list.Clear();
                list.AddRange(listAri);
                list.AddRange(listNashi);

                Characters = list;
            }
            else
            {
                // Character Level Sort
                for (int i = list.Count - 1; i > 0; i--)
                {
                    for (int j = 0; j < i; j++)
                    {
                        // レベルで比較
                        // 前の方が大きい時にSwapしているのでLV昇順
                        if (list[j].Level > list[j + 1].Level)
                        {
                            //Swap Character
                            temp = list[j];
                            list[j] = list[j + 1];
                            list[j + 1] = temp;
                        }
                    }
                }

                if (sortType == PCソート種類.レベル昇順)
                {
                    Characters = list;
                }
                else
                {
                    list.Reverse();
                    Characters = list;
                }
            }
        }

        #endregion

        /// <summary>
        /// キャラクタリストからキャラクタのインデックス取得
        /// </summary>
        /// <param name="chara"></param>
        /// <returns></returns>
        public int GetCharacterIndex(CharacterData chara)
        {
            for (int i = 0; i < Characters.Count; i++)
            {
                if (Characters[i] == chara)
                {
                    return i;
                }
            }

            // 消さないように
            PartyMaker.CWIErrorMessage("キャラクタ情報がリスト中に見つかりません。");
            return -1;
        }

        /// <summary>
        /// 待機PC全読み込み
        /// </summary>
        private void LoadAllPC(DirectoryInfo di)
        {
            Characters.Clear();

            // 待機ＰＣ
            foreach (FileInfo fi in di.GetFiles(FilePath.MaskCharacterData))
            {
                Characters.Add(new CharacterData(fi.FullName));
            }
        }

        /// <summary>
        /// シナリオフォルダ内のMateを待機PCに追加
        /// </summary>
        /// <param name="scenaFolder">シナリオフォルダ</param>
        public void GetMates(string scenaFolder)
        {
            DirectoryInfo di = new DirectoryInfo(scenaFolder);
            FileInfo[] fis = di.GetFiles("Mate*.wid", SearchOption.TopDirectoryOnly);
            foreach (FileInfo fi in fis)
            {
                CharacterData cd = new CharacterData();
                try
                {
                    cd.Read(fi.FullName);
                    // MatePCを待機PCに
                    cd.FileName = string.Empty; // MatePCなので元ファイルはない
                    // ファイル作成
                    AddCharacterFile(cd);
                    //キャラクタリスト追加
                    Characters.Add(cd);
                }
                catch (IOException)
                {
                    PartyMaker.CWIErrorMessage("Mateの読み込みに失敗");
                }
            }
        }

        /// <summary>
        /// キャラクタをファイル追加
        /// </summary>
        /// <param name="cd"></param>
        public void AddCharacterFile(CharacterData cd)
        {
            AddCharacterCommon(cd,tempFolder);
        }

        /// <summary>
        /// キャラクタを~の付いていない宿にファイル直接追加
        /// </summary>
        /// <param name="cd">キャラクタデータ</param>
        /// <param name="yadoName">宿名</param>
        private void AddPrimeCharacterFile(CharacterData cd,string yadoName)
        {
            AddCharacterCommon(cd,FilePath.GetSystemYadoFolder() + "\\" + yadoName);
        }

        /// <summary>
        /// キャラクタファイル追加の共通部分
        /// </summary>
        /// <param name="cd">キャラクタデータ</param>
        /// <param name="folderPath">指定フォルダのパス</param>
        private void AddCharacterCommon(CharacterData cd,string folderPath)
        {
            string filePath;
            string indexFilePath;

            DeleteCharacterFile(cd);

            filePath = FilePath.GetCharacterFileName(folderPath, cd.CardName);

            indexFilePath = Path.ChangeExtension(filePath, FilePath.ExtIndexCharacterData);

            //wcp
            //wcpには画像を書き込まない
            cd.Image.NoSave = true;
            cd.Write(filePath);
            cd.Image.NoSave = false;
            // 書き込んだファイル名を取得
            cd.FileName = filePath;

            //wch
            IndexCharacterData icd = Util1.MakeCharacterIndexData(cd);
            icd.Write(indexFilePath);
        }

        /// <summary>
        /// アルバムファイル追加
        /// </summary>
        /// <param name="cd">アルバムキャラクタデータ</param>
        public void AddAlbumFile(AlbumCharacter ac)
        {
            string filePath;

            if (ac.FileName != string.Empty)
            {
                FilePath.DeleteFile(ac.FileName);
            }

            filePath = FilePath.GetAlbumFileName(tempFolder, ac.Name);

            //wrm
            ac.Write(filePath);
            // 書き込んだファイル名を取得
            ac.FileName = filePath;

        }

        /// <summary>
        /// キャラクタ置き換え
        /// </summary>
        /// <param name="oldCd"></param>
        /// <param name="newCd"></param>
        private void ReplaceCharacter(CharacterData oldCd, CharacterData newCd)
        {
            int oldIndex = GetCharacterIndex(oldCd);

            DeleteCharacterFile(oldCd);

            AddCharacterFile(newCd);

            // List Replace
            Characters[oldIndex] = newCd;

        }

        /// <summary>
        /// 待機PCの単独削除
        /// </summary>
        /// <param name="index"></param>
        public void DeleteCharacter(int index)
        {
            if (index >= 0 && index < Characters.Count)
            {
                CharacterData cd = Characters[index];
                DeleteCharacterFile(cd);
                Characters.RemoveAt(index);
            }
            else
            {
                PartyMaker.CWIErrorMessage("キャラクタ情報がPCリスト中に見つかりません。(削除)");
            }
        }

        /// <summary>
        /// キャラクタファイルの削除
        /// </summary>
        /// <param name="cd"></param>
        private void DeleteCharacterFile(CharacterData cd)
        {
            string path;
            string indexPath;

            if (cd.FileName != string.Empty)
            {
                path = cd.FileName;
                indexPath = Path.ChangeExtension(path, FilePath.ExtIndexCharacterData);
                FilePath.DeleteFile(path);
                FilePath.DeleteFile(indexPath);
            }
        }

        /// <summary>
        /// 宿にある同じキャラクタファイルの削除
        /// </summary>
        /// <param name="cd">保存済で宿と~宿で同期が取れているキャラクタデータ</param>
        private void DeletePrimeCharacterFile(CharacterData cd)
        {
            string path;
            string indexPath;

            if (cd.FileName != string.Empty)
            {

                path = Folder + "\\" + FilePath.FileNameFromPath(cd.FileName);
                indexPath = Path.ChangeExtension(path, FilePath.ExtIndexCharacterData);
                FilePath.DeleteFile(path);
                FilePath.DeleteFile(indexPath);
            }
        }

        /// <summary>
        /// 他宿待機PCが一括移動
        /// </summary>
        public void MoveAllOtherInnPC()
        {
            foreach (CharacterData cd in OtherInn.Characters)
            {
                // ファイル作成（元のファイルは削除）
                AddCharacterFile(cd);
                // キャラクタリスト追加
                Characters.Add(cd);
            }

            OtherInn.Characters.Clear();

        }
        
        /// <summary>
        /// 待機PCが全員他宿へ
        /// </summary>
        public void MoveAllPCToOtherInn()
        {

            foreach (CharacterData cd in Characters)
            {
                // ファイル作成（元のファイルは削除）
                OtherInn.AddCharacterFile(cd);
                // 他宿待機PCキャラクタリスト追加
                OtherInn.Characters.Add(cd);
            }

            Characters.Clear();
        }

        /// <summary>
        /// PC単独宿移動
        /// </summary>
        /// <param name="innChara">移動対象キャラクタデータ</param>
        /// <param name="moveYadoName">移動先宿名</param>
        public void MovePCToOtherInn(CharacterData innChara,string moveYadoName)
        {

            int rIndex = GetCharacterIndex(innChara);

            if (rIndex >= 0 && rIndex < Characters.Count)
            {
                // まずSaveする前にキャラクタを宿から外す
                DeleteCharacterFile(innChara);
                // キャラクタリストから削除
                Characters.RemoveAt(rIndex);

                // この状態でSave
                Save();

                // ~宿と宿のファイルを削除する必要がなくなっている

                // ファイルを追加
                AddPrimeCharacterFile(innChara, moveYadoName);

            }
            else
            {
                // 消さないように
                PartyMaker.CWIErrorMessage("キャラクタ情報がPCリスト中に見つかりません。(宿移動)");
            }
        }
        
        #endregion

        #region WidCard
        /// <summary>
        /// WidCardを再度読み込むためにフラグを落とす
        /// </summary>
        public void ResetWidCard()
        {
            readWidCards = false;
        }
        #endregion

        #endregion

        #endregion

        //class InnData
    }

}
