﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using CWIDatas;
using CWI.Dialog;

namespace CWI
{
    /// <summary>
    /// Description of PartyData
    /// </summary>
    public class PartyData : DataWithIO
    {
        static readonly uint characterCount_base_v5 = 30;
        static readonly uint characterCount_base_v4 = 10;


        #region フィールド
        int version;
        List<CharacterData> characterList;
        List<CharacterData> vanishCharacterList;
        string partyName;
        List<RenamedCardData> bagCardList;
        uint money;
        uint moneyAtStart;
        byte inScenario;
        ScenarioData scenarioData;
        #endregion

        #region プロパティ
        public List<CharacterData> Characters
        {
            get
            {
                if (characterList == null)
                    characterList = new List<CharacterData>();
                return characterList;
            }
        }
        /// <summary>
        /// 対象消去によって消滅したキャラクターデータ
        /// </summary>
        public List<CharacterData> VanishCharacters
        {
            get
            {
                if (vanishCharacterList == null)
                    vanishCharacterList = new List<CharacterData>();
                return vanishCharacterList;
            }
        }
        public string PartyName
        {
            get { return partyName; }
            set { partyName = value; }
        }
        public List<RenamedCardData> BagCards
        {
            get
            {
                if (bagCardList == null)
                    bagCardList = new List<RenamedCardData>();
                return bagCardList;
            }
        }
        public uint Money
        {
            get { return money; }
            set
            {
                if (value > CWData.MaxMoneyValue)
                    money = CWData.MaxMoneyValue;
                else if (value < 0)
                    money = 0;
                else
                    money = value;
            }
        }
        /// <summary>
        /// シナリオ開始時の所持金（F9時に戻す）
        /// </summary>
        public uint MoneyAtStart
        {
            get { return moneyAtStart; }
            set { moneyAtStart = value; }
        }
        /// <summary>
        /// ScenarioDataのセットにより自動的に設定される
        /// </summary>
        public bool InScenario
        {
            get { return inScenario != 0; }
        }

        public ScenarioData PlayingScenarioData
        {
            get { return scenarioData; }
            set
            {
                scenarioData = value;
                if (value != null)
                {
                    inScenario = 1;
                    Summary = new SummaryData();
                    try
                    {
                        Summary.Read(FilePath.GetSummaryFile(scenarioData.ScenarioFolder));
                    }
                    catch (Exception)
                    {

                        PartyMaker.CWIErrorMessage("サマリファイルの読み込みに失敗(PlayingScenarioData)：");
                    }
                }
                else
                {
                    inScenario = 0;
                    Summary = null;
                }
            }
        }


        #endregion

        #region 追加プロパティ
        /// <summary>
        /// 正確なフルパスのファイル名
        /// </summary>
        string fileName = string.Empty;
        public string FileName
        {
            get
            {
                return fileName;
            }
            set
            {
                fileName = value;
            }
        }

        /// <summary>
        /// サマリデータ
        /// </summary>
        SummaryData summary;
        public SummaryData Summary
        {
            get
            {
                return summary;
            }
            set
            {
                summary = value;
            }
        }

        string scenarioBGMFile;
        public string ScenarioBGMFile
        {
            get
            {
                return scenarioBGMFile;

            }
            set
            {
                scenarioBGMFile = value;
            }
        }

        string scenarioLevelText;
        public string ScenarioLevelText
        {
            get
            {
                return scenarioLevelText;

            }
            set
            {
                scenarioLevelText = value;
            }
        }

        #endregion

        #region コンストラクタ

        /// <summary>
        /// 新規作成用指定なしコンストラクタ
        /// </summary>
        public PartyData()
        {

        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="file">ファイル名</param>
        public PartyData(string file)
        {
            try
            {
                Read(file);
                SetSenarioData();
            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("パーティ情報ファイルの読み込みに失敗：" + ex.Message);
            }
        }

        /// <summary>
        /// シナリオデータ設定
        /// </summary>
        private void SetSenarioData()
        {
            // シナリオ情報取得
            if (InScenario)
            {
                //シナリオ中
                scenarioBGMFile = PlayingScenarioData.ScenarioFolder
                    + "\\" + PlayingScenarioData.BGMFile;

                if (Directory.Exists(PlayingScenarioData.ScenarioFolder) == false)
                {
                    bool scenarioNotFound = true;
                    //CardWirth自体がフォルダ移動していた場合の窮余策
                    int index = PlayingScenarioData.ScenarioFolder.IndexOf(FilePath.SystemScenarioFolderName);
                    if (index >= 0)
                    {
                        index += FilePath.SystemScenarioFolderName.Length;
                        string newScenarioFolder = FilePath.GetScenarioFolder() +
                                                   PlayingScenarioData.ScenarioFolder.Substring(index);
                        if (Directory.Exists(newScenarioFolder) &&
                           File.Exists(FilePath.GetSummaryFile(newScenarioFolder)))
                        {
                            PlayingScenarioData.ScenarioFolder = newScenarioFolder;
                            scenarioNotFound = false;
                        }
                    }

                    if (scenarioNotFound)
                    {
                        PartyMaker.CWIErrorMessage(this.PartyName + "の進行中シナリオ：" +
                            Environment.NewLine + PlayingScenarioData.ScenarioFolder + "が存在しません。");
                    }
                }

                Summary = new SummaryData();
                //Summaryを読み込む
                try
                {

                    Summary.Read(FilePath.GetSummaryFile(PlayingScenarioData.ScenarioFolder));

                    //対象レベルの文字
                    if (Summary.LevelLow == Summary.LevelHigh)//対象レベルがどちらも一緒なら
                    {
                        if (Summary.LevelLow == 0)
                        {
                            scenarioLevelText = string.Empty;
                        }
                        else
                        {
                            scenarioLevelText = " 対象レベル : " + Summary.LevelLow;//最低レベルだけ
                        }
                    }
                    else
                    {
                        scenarioLevelText = " 対象レベル : " + Summary.LevelLow + " ～ " + Summary.LevelHigh;//通常表示
                    }

                    // 正常取得して終了
                    return;

                }
                catch (Exception)
                {
                    PartyMaker.CWIErrorMessage(this.PartyName + "の進行中シナリオ：" +
                        Environment.NewLine + PlayingScenarioData.ScenarioFolder + "が読み込めません。");
                }
            }

            scenarioBGMFile = string.Empty;
            Summary = null;
            scenarioLevelText = string.Empty;
            return;
        }

        /// <summary>
        /// シナリオデータ設定(外部からシナリオデータ渡し)
        /// </summary>
        public void SetSenarioData(ScenarioData sd)
        {
            PlayingScenarioData = sd;
            SetSenarioData();
        }

        #endregion

        #region リードライト
        public override void Read(string file)
        {
            this.FileName = file;
            base.Read(file);
        }

        public override void Read(System.IO.BinaryReader r)
        {
            uint temp = r.ReadUInt32();

            uint count;
            if (temp > 16)
            {
                count = temp - characterCount_base_v5;
                version = 5;
            }
            else
            {
                count = temp - characterCount_base_v4;
                version = 4;
            }

            for (; count > 0; count--)
            {
                CharacterData d = new CharacterData();
                d.Read(r);
                Characters.Add(d);
            }

            if (version == 5)
            {
                count = r.ReadUInt32();
                for (; count > 0; count--)
                {
                    CharacterData d = new CharacterData();
                    d.Read(r);
                    VanishCharacters.Add(d);
                }
            }
            else
            {
                //count = r.ReadUInt32();
                //for (; count > 0; count--)
                //{
                //    CharacterData d = new CharacterData();
                //    d.Read(r);
                //    VanishCharacters.Add(d);
                //}
            }

            if (version != 4)
            {
                // PartyName
                partyName = r.ReadString();
                count = r.ReadUInt32();
                for (; count > 0; count--)
                {
                    RenamedCardData d = new RenamedCardData();
                    d.Read(r);
                    BagCards.Add(d);
                }

            }
            else
            {
                //count = r.ReadByte();
                //for (; count > 0; count--)
                //{
                //    RenamedCardData d = new RenamedCardData();
                //    d.Read(r);
                //    BagCards.Add(d);
                //}
            }

            // money
            money = r.ReadUInt32();

            moneyAtStart = r.ReadUInt32();

            // inScenario
            inScenario = r.ReadByte();

            if (InScenario)
            {
                scenarioData = new ScenarioData();
                scenarioData.Read(r);
            }
        }

        public override void Write(System.IO.BinaryWriter w)
        {
            w.Write((uint)(characterCount_base_v5 + Characters.Count));
            foreach (CharacterData d in Characters)
                d.Write(w);
            w.Write(VanishCharacters.Count);
            foreach (CharacterData d in VanishCharacters)
                d.Write(w);
            w.Write(partyName);
            w.Write(BagCards.Count);
            foreach (RenamedCardData d in BagCards)
                d.Write(w);
            w.Write(money);
            w.Write(moneyAtStart);
            w.Write(inScenario);
            if (InScenario)
                scenarioData.Write(w);
        }
        #endregion

        //class PartyData
    }
}
