﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Runtime.Serialization.Formatters.Binary;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    /// <summary>
    /// BGMSelecter用フォームクラス
    /// </summary>
    public partial class BGMSelectDlg : Form
    {
        #region メンバ

        private PartyMaker _pm;
        private DirectoryInfo di;
        private string midi;
        private List<Okiniri> list = new List<Okiniri>();

        private string[] taishoFiles
            = { 
                "DefInn.mid",
                "DefOpen.mid",
                "DefBattle.mid",
                "DefOver.mid",
                "RYUNE.MID",
                "HerosMarch.mid",
                "BeyonedMyHeart.mid",
                "Uno.mid",
                "ScarletPlain.mid",
                "SenceOfDeath.mid"
            };

        private string[] backupFiles
            = { 
                "_DefInn.mid",
                "_DefOpen.mid",
                "_DefBattle.mid",
                "_DefOver.mid",
                "_RYUNE.MID",
                "_HerosMarch.mid",
                "_BeyonedMyHeart.mid",
                "_Uno.mid",
                "_ScarletPlain.mid",
                "_SenceOfDeath.mid"
            };

        private string[] comboNames
            = { 
                "cmbInn",
                "cmbOpen",
                "cmbBattle",
                "cmbOver",
                "cmbRYUNE",
                "cmbHeroes",
                "cmbBeyond",
                "cmbUno",
                "cmbScarlet",
                "cmbSence"
            };

        private ComboBox[] combos;

        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public BGMSelectDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public BGMSelectDlg(PartyMaker pm)
        {
            InitializeComponent();

            _pm = pm;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _pm.Left + (_pm.Width - this.Width) / 2;
            this.Top =
                _pm.Top + (_pm.Height - this.Height) / 2;
            combos =   
            new ComboBox[]{
                    cmbInn,
                    cmbOpen,
                    cmbBattle,
                    cmbOver,
                    cmbRYUNE,
                    cmbHeroes,
                    cmbBeyond,
                    cmbUno,
                    cmbScarlet,
                    cmbSence
            };
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void BGMSelectDlg_Load(object sender, EventArgs e)
        {
            MultiMedia.SoundStop();

            midi = FilePath.GetDataMidiFolder();
            di = new DirectoryInfo(midi);
            midi += "\\";

            List<string> fileNames = new List<string>();
            foreach (FileInfo fi in
                di.GetFiles("*.mid*", SearchOption.AllDirectories))
            {
                fileNames.Add(fi.Name);
            }

            foreach (string item in taishoFiles)
            {
                if (!File.Exists(
                    midi + "_" + item))
                {
                    // デフォルトファイル　バックアップ
                    File.Copy(midi + item, midi + "_" + item);
                }
            }

            foreach (string back in backupFiles)
            {
                if (fileNames.Contains(back))
                {
                    // バックアップファイルは対象外
                    fileNames.Remove(back);
                }
            }

            foreach (string back in taishoFiles)
            {
                if (fileNames.Contains(back))
                {
                    // 対象ファイルは共通ファイル名から一度外す
                    fileNames.Remove(back);
                }
            }

            List<string> cList;

            for (int i = 0; i < 10; i++)
            {
                cList = new List<string>(fileNames);
                cList.Add(taishoFiles[i]);

                combos[i].DataSource = cList;
            }

            string root = FilePath.CW_PATH;
            if (File.Exists(root + "\\okiniri.cache"))
            {
                // キャッシュから現在の状態とお気に入りを読み込み
                using(FileStream fs = new FileStream(
                    root + "\\okiniri.cache",
                    FileMode.Open)
                    )
                {
                    BinaryFormatter
                        bf = new BinaryFormatter();
                    
                     list = (List<Okiniri>)bf.Deserialize(fs);
                    
                }

                Okiniri now = list[0];
                this.lblLastAccess.Text = "最終アクセス：" + now.LastUpdate;

                SetCurrentIndex(this.cmbInn, now.DefInn);
                SetCurrentIndex(this.cmbOpen, now.DefOpen);
                SetCurrentIndex(this.cmbBattle, now.DefBattle);
                SetCurrentIndex(this.cmbOver, now.DefOver);
                SetCurrentIndex(this.cmbRYUNE, now.RYUNE);
                SetCurrentIndex(this.cmbHeroes, now.HerosMarch);
                SetCurrentIndex(this.cmbBeyond, now.BeyonedMyHeart);
                SetCurrentIndex(this.cmbUno, now.Uno);
                SetCurrentIndex(this.cmbScarlet, now.ScarletPlain);
                SetCurrentIndex(this.cmbSence, now.SenceOfDeath);

                for (int i = 1; i < list.Count; i++)
                {
                    listBox1.Items.Add(list[i].LastUpdate);
                }
            }
            else
            {
                for (int i = 0; i < 10; i++)
                {
                    SetCurrentIndex(combos[i], taishoFiles[i]);
                }

                Okiniri now = GetNow();
                now.name = "現在の状態";
                list.Add(now);
            }

        }

        private void SetCurrentIndex(ComboBox cb,string val)
        {

            MultiMedia.SoundStop();

            string temp = val;
            List<string> taishoList 
                = new List<string>(taishoFiles);

            // 選択
            int index = 0;
            foreach (object item in cb.Items)
            {
                if (item.Equals(temp))
                {
                    cb.SelectedIndex = index;
                    break;
                }

                index++;
            }


            // 元のファイルはバックアップファイル
            if (taishoList.Contains(temp))
            {
                temp = "_" + temp;
            }

            for (int i = 0; i < comboNames.Length; i++)
            {
                if (cb.Name.Equals(comboNames[i]))
                {
                    if (cb.SelectedIndex < 0 || cb.SelectedIndex >= cb.Items.Count)
                    {
                        // ファイル名を変更などで選択されて
                        // いない場合
                        int ind = 0;
                        foreach (object item in cb.Items)
                        {
                            if (item.Equals(taishoFiles[i]))
                            {
                                cb.SelectedIndex = ind;
                                break;
                            }

                            ind++;
                        }
                         
                        break;
                    }

                    File.Copy(midi + temp, midi + taishoFiles[i], true);
                    break;
                }
            }
        }


        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BGMSelectDlg_Shown(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BGMSelectDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            // コピー前には止めておく
            MultiMedia.SoundStop();

            string root = FilePath.CW_PATH;
            // キャッシュに現在の状態とお気に入りを保存
            // 現在の状態
            Okiniri now = GetNow();
            now.name = "現在の状態";
            list[0] = now;

            List<string> taishoList
              = new List<string>(taishoFiles);
            // Def にコピーしておく
            for (int i = 0; i < combos.Length; i++)
            {
                string temp = combos[i].SelectedValue.ToString();
                // 元のファイルはバックアップファイル
                if (taishoList.Contains(temp))
                {
                    temp = "_" + temp;
                }

                File.Copy(midi + temp, midi + taishoFiles[i], true);
   
            }

            using (FileStream fs = new FileStream(
                root + "\\okiniri.cache",
                FileMode.Create)
                )
            {
                
                    BinaryFormatter
                        bf = new BinaryFormatter();
                    bf.Serialize(fs, list);
                
            }


            //曲を流す
            MultiMedia.PlayMusicFile(
                midi + cmbInn.SelectedValue, _pm.soundMute
                );
        }

        #endregion

        #region クリックイベント

        private void btnInn_Click(object sender, EventArgs e)
        {

            //曲を流す
            string midfile = cmbInn.SelectedValue.ToString().Equals(taishoFiles[0])
                ? backupFiles[0] : cmbInn.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {

            //曲を流す
            string midfile = cmbOpen.SelectedValue.ToString().Equals(taishoFiles[1])
                ? backupFiles[1] : cmbOpen.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnBattle_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbBattle.SelectedValue.ToString().Equals(taishoFiles[2])
                ? backupFiles[2] : cmbBattle.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnOver_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbOver.SelectedValue.ToString().Equals(taishoFiles[3])
                ? backupFiles[3] : cmbOver.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnRYUNE_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbRYUNE.SelectedValue.ToString().Equals(taishoFiles[4])
                ? backupFiles[4] : cmbRYUNE.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnHeroes_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbHeroes.SelectedValue.ToString().Equals(taishoFiles[5])
                ? backupFiles[5] : cmbHeroes.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnBeyond_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbBeyond.SelectedValue.ToString().Equals(taishoFiles[6])
                ? backupFiles[6] : cmbBeyond.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnUno_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbUno.SelectedValue.ToString().Equals(taishoFiles[7])
                ? backupFiles[7] : cmbUno.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnScarlet_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbScarlet.SelectedValue.ToString().Equals(taishoFiles[8])
                ? backupFiles[8] : cmbScarlet.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnSence_Click(object sender, EventArgs e)
        {
            //曲を流す
            string midfile = cmbSence.SelectedValue.Equals(taishoFiles[9])
                ? backupFiles[9] : cmbSence.SelectedValue.ToString();
            MultiMedia.PlayMusicFile(
                midi + midfile, _pm.soundMute
                );
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            MultiMedia.SoundStop();
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnOkiniAdd_Click(object sender, EventArgs e)
        {
            Okiniri data = GetNow();
            data.name = "お気に入り";
            listBox1.Items.Add(data.LastUpdate);
            list.Add(data);
        }

        private void btnOkiniDel_Click(object sender, EventArgs e)
        {
            int index = listBox1.SelectedIndex;
            if (index >= 0 
                && index < list.Count - 1)
            {
                listBox1.Items.RemoveAt(index);
                list.RemoveAt(index + 1);
                if (index < list.Count - 1)
                {
                    listBox1.SelectedIndex = index;
                }
                else if (list.Count > 1)
                {
                    listBox1.SelectedIndex = list.Count - 2;
                } 
            }
        }

        private void btnHanei_Click(object sender, EventArgs e)
        {
            int index = listBox1.SelectedIndex;
            if (index >= 0
                && index < list.Count - 1)
            {
                Okiniri now = list[index + 1];
                SetCurrentIndex(this.cmbInn, now.DefInn);
                SetCurrentIndex(this.cmbOpen, now.DefOpen);
                SetCurrentIndex(this.cmbBattle, now.DefBattle);
                SetCurrentIndex(this.cmbOver, now.DefOver);
                SetCurrentIndex(this.cmbRYUNE, now.RYUNE);
                SetCurrentIndex(this.cmbHeroes, now.HerosMarch);
                SetCurrentIndex(this.cmbBeyond, now.BeyonedMyHeart);
                SetCurrentIndex(this.cmbUno, now.Uno);
                SetCurrentIndex(this.cmbScarlet, now.ScarletPlain);
                SetCurrentIndex(this.cmbSence, now.SenceOfDeath);

            }
        }

        private void btnDefault_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < 10; i++)
            {
                SetCurrentIndex(combos[i], taishoFiles[i]);
            }
        }

        private Okiniri GetNow()
        {
            Okiniri data = new Okiniri();
            data.LastUpdate = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss");
            data.DefInn = cmbInn.SelectedValue.ToString();
            data.DefOpen = this.cmbOpen.SelectedValue.ToString();
            data.DefBattle = this.cmbBattle.SelectedValue.ToString();
            data.DefOver = this.cmbOver.SelectedValue.ToString();
            data.RYUNE = this.cmbRYUNE.SelectedValue.ToString();
            data.HerosMarch = this.cmbHeroes.SelectedValue.ToString();
            data.BeyonedMyHeart =
                this.cmbBeyond.SelectedValue.ToString();
            data.Uno = this.cmbUno.SelectedValue.ToString();
            data.ScarletPlain
                = this.cmbScarlet.SelectedValue.ToString();
            data.SenceOfDeath
                = this.cmbSence.SelectedValue.ToString();
            return data;
        }
        
        #endregion

    }

    /// <summary>
    /// お気に入り
    /// </summary>
    [Serializable]
    public class Okiniri
    {
        public string name;

        public string LastUpdate;

        public string DefInn;
        public string DefOpen;
        public string DefBattle;
        public string DefOver;
        public string RYUNE;
        public string HerosMarch;
        public string BeyonedMyHeart;
        public string Uno;
        public string ScarletPlain;
        public string SenceOfDeath;
    }
}