﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Threading;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class BmpSelectDlg : Form
    {
        #region メンバ

        private const int FORM_MIN_WIDTH = 640;
        private const int FORM_MIN_HEIGHT = 480;

        private int disp_num_x = 8;

        //選択したインデックス
        public int SelectedIndex = 0;

        private List<string> pathList;
        public List<string> PathList
        {
            set
            {
                pathList = value;
            }

            get
            {
                if (pathList == null)
                {
                    pathList = new List<string>();
                }
                return pathList;
            }
        }

        // 現在のインデックス
        private int index = 0;

        private static volatile ProgressDlg progress;

        private CharacterData cd = null;

        private bool isBaseCard = false;
        public List<int> ExceptIndexList = null;

        private List<CharacterData> charaList = null;
        private List<PartyData> bmpList = null;
        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public BmpSelectDlg()
        {
            // 通常使用しない
            InitializeComponent();

            progress = new ProgressDlg(this);

            MouseWheel += new MouseEventHandler(Bitmap_MouseWheel);
        }

        /// <summary>
        /// コンストラクタ（新規キャラクタ作成）
        /// </summary>
        public BmpSelectDlg(List<string> pathList)
        {
            InitializeComponent();

            progress = new ProgressDlg(this);

            MouseWheel += new MouseEventHandler(Bitmap_MouseWheel);

            this.PathList = pathList;
        }

        
        /// <summary>
        /// コンストラクタ（拡大ＰＣ画像作成）
        /// </summary>
        public BmpSelectDlg(List<string> pathList, string text) : this(pathList)
        {
            this.label1.Text = text;
        }

        /// <summary>
        /// コンストラクタ（キャラクタデザイン編集）
        /// </summary>
        public BmpSelectDlg(List<string> pathList, CharacterData cd)
        {
            InitializeComponent();

            progress = new ProgressDlg(this);

            MouseWheel += new MouseEventHandler(Bitmap_MouseWheel);

            this.PathList = pathList;
            this.cd = cd;
        }

        /// <summary>
        /// コンストラクタ（ベースカード置換）
        /// </summary>
        public BmpSelectDlg(List<string> pathList,bool isBaseCard) 
            : this(pathList)
        {
            this.isBaseCard = isBaseCard;
            this.ExceptIndexList = new List<int>();
        }

        /// <summary>
        /// コンストラクタ（PC選択）
        /// </summary>
        public BmpSelectDlg(List<CharacterData> list)
        {

            InitializeComponent();

            progress = new ProgressDlg(this);

            MouseWheel += new MouseEventHandler(Bitmap_MouseWheel);

            charaList = list;
        }

        /// <summary>
        /// コンストラクタ（パーティーメンバ一覧）
        /// </summary>
        /// <param name="list"></param>
        public BmpSelectDlg(List<PartyData> list)
        {

            InitializeComponent();

            progress = new ProgressDlg(this);

            MouseWheel += new MouseEventHandler(Bitmap_MouseWheel);

            bmpList = list;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void BmpSelectDlg_Load(object sender, EventArgs e)
        {

            // マウス・カーソルの位置をスクリーン中央に移動させる
            int centerX = Screen.PrimaryScreen.Bounds.Right / 2;
            int centerY = Screen.PrimaryScreen.Bounds.Bottom / 2;
            Point cursolPosition = new Point(centerX, centerY);
            
            this.Location = new Point(centerX - this.Width / 2, centerY - this.Height / 2);

            // デフォルト最大化表示
            this.WindowState = FormWindowState.Maximized;

            toolTip1.UseFading = true;
            toolTip1.InitialDelay = 100;
            toolTip1.ShowAlways = false;
            toolTip1.Active = true;
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BmpSelectDlg_Shown(object sender, EventArgs e)
        {
            ShowBitmaps();
        }

        /// <summary>
        /// ビットマップ表示
        /// </summary>
        public void ShowBitmaps()
        {
            
            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "画像を読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            // 画像表示
            index = 0;
            PictureBox pic;

            pnlBitmap.AutoScrollPosition = new Point(0, 0);
            pnlBitmap.Controls.Clear();

            int count = 0;

            if (charaList != null)
            {
                //キャラクタ選択
                count = charaList.Count;
            }
            else if (bmpList != null)
            {
                // パーティーメンバ一覧
                count = 0;
                foreach(PartyData pd in bmpList){
                    count += pd.Characters.Count;
                }
            }
            else
            {

                count = PathList.Count;

                if (this.cd != null)
                {
                    count++;
                }
            }

            this.Text = string.Empty;

            if (count != 0)
            {
                if (!isBaseCard)
                {

                    if (charaList != null)
                    {
                        //キャラクタ選択
                        foreach (CharacterData chara in charaList)
                        {
                            pic = new PictureBox();
                            pic.MouseHover += new EventHandler(pnlBitmap_MouseHover);
                            pic.MouseDown += new MouseEventHandler(pnlBitmap_MouseDown);
                            pic.Tag = chara.CardName;
                            pic.Enabled = true;
                            pic.Size = new Size(CWData.Bitmap_Width, CWData.Bitmap_Height);

                            pic.Image = chara.Image.bitmap;

                            pic.Location = new Point(CWData.Bitmap_Width * (index % disp_num_x)
                                , CWData.Bitmap_Height * (index / disp_num_x));

                            index++;
                            pnlBitmap.Controls.Add(pic);

                            progress.Value = 100 * index / count;
                        }
                    }
                    else if (bmpList != null)
                    {
                        // パーティーメンバ一覧
                        for(int i = 0;i < bmpList.Count ;i++)
                        {
                            PartyData pd = bmpList[i];
                            foreach (CharacterData chara in pd.Characters)
                            {
                                pic = new PictureBox();
                                pic.MouseHover += new EventHandler(pnlBitmap_MouseHover);
                                pic.MouseDown += new MouseEventHandler(pnlBitmap_MouseDown);
                                pic.Tag = (i+1) + ":" + chara.CardName;
                                pic.Enabled = true;
                                pic.Size = new Size(CWData.Bitmap_Width, CWData.Bitmap_Height);

                                pic.Image = chara.Image.bitmap;

                                pic.Location = new Point(CWData.Bitmap_Width * (index % disp_num_x)
                                    , CWData.Bitmap_Height * (index / disp_num_x));

                                index++;
                                pnlBitmap.Controls.Add(pic);

                                progress.Value = 100 * index / count;
                            }
                        }
                    }
                    else
                    {
                        //Faceフォルダ画像選択
                        foreach (string path in PathList)
                        {
                            pic = new PictureBox();
                            pic.MouseHover += new EventHandler(pnlBitmap_MouseHover);
                            pic.MouseDown += new MouseEventHandler(pnlBitmap_MouseDown);
                            pic.Tag = path;
                            pic.Enabled = true;
                            pic.Size = new Size(CWData.Bitmap_Width, CWData.Bitmap_Height);

                            pic.Image = new Bitmap(path);

                            pic.Location = new Point(CWData.Bitmap_Width * (index % disp_num_x)
                                , CWData.Bitmap_Height * (index / disp_num_x));

                            index++;
                            pnlBitmap.Controls.Add(pic);

                            progress.Value = 100 * index / count;
                        }
                    }
                }
                else
                {
                    WidCard wc = null;

                    foreach (string path in PathList)
                    {
                        pic = new PictureBox();
                        pic.MouseHover += new EventHandler(pnlBitmap_MouseHover);
                        pic.MouseDown += new MouseEventHandler(pnlBitmap_MouseDown);
                        pic.Enabled = true;
                        pic.Tag = path;
                        pic.Size = new Size(CWData.Bitmap_Width,
                                            CWData.Bitmap_Height);

                        // WidIDtypeはﾜｶﾗﾅｲ
                        wc = new WidCard();
                        try
                        {
                            wc.Read(path);
                            pic.Image = wc.Image.bitmap;
                        }
                        catch
                        {
                            pic.Image = CWIEngine.Properties.Resources.Back_アクション;
                            ExceptIndexList.Add(index);
                        }

                        pic.Location =
                            new Point(CWData.Bitmap_Width * (index % disp_num_x)
                            , CWData.Bitmap_Height * (index / disp_num_x));

                        index++;
                        pnlBitmap.Controls.Add(pic);

                        progress.Value = 100 * index / count;
                    }
                }

                if (charaList != null)
                {
                    this.Text = "キャラ選択： " + index + " キャラ ";
                }
                else if (bmpList != null)
                {
                    this.Text = "パーティーメンバ：" + count + " キャラ ";
                }
                else
                {
                    this.Text = "画像選択： " + index + " ファイル ";
                }
            }

            if (this.cd != null)
            {
                // cd自身の画像を最後に追加
                pic = new PictureBox();
                pic.MouseHover += new EventHandler(pnlBitmap_MouseHover);
                pic.MouseDown += new MouseEventHandler(pnlBitmap_MouseDown);
                pic.Enabled = true;
                pic.Tag = cd.CardName;
                pic.Size = new Size(CWData.Bitmap_Width, CWData.Bitmap_Height);

                if (cd.Image != null && cd.Image.baseBitmap != null)
                {
                    pic.Image = cd.Image.baseBitmap;
                }

                pic.Location = new Point(CWData.Bitmap_Width * (index % disp_num_x)
                    , CWData.Bitmap_Height * (index / disp_num_x));

                index++;
                pnlBitmap.Controls.Add(pic);

                if (this.Text.Equals(string.Empty))
                {
                    this.Text = "画像選択： 1 ファイル"; 
                }
                else
                {
                    this.Text += "+ 1";
                }

                progress.Value = 100 * index / count;     
            }

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            // FormClosingに集約
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BmpSelectDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            // リソース開放があったらここで
            pnlBitmap.Controls.Clear();
        }

        /// <summary>
        /// リサイズ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BmpSelectDlg_Resize(object sender, EventArgs e)
        {

            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {

                this.pnlBitmap.Width = this.Width - 30;
                this.pnlBitmap.Height = this.Height - 70;

                this.btnEnd.Location = new Point(
                    (this.ClientSize.Width - this.btnEnd.Width) / 2,
                    this.ClientSize.Height - 28);

                this.label1.Location = new Point(
                    12,
                    this.ClientSize.Height - 21);

                int old_num_x = this.disp_num_x;
                this.disp_num_x = (this.pnlBitmap.Width - 10) / CWData.Bitmap_Width;

                if (old_num_x != this.disp_num_x)
                {
                    SetLocationPnlBitmap();
                }

            }
        }


        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {
            backgroundWorker1.CancelAsync();
            while (this.backgroundWorker1.IsBusy)
            {
                Thread.Sleep(1);
            }

            backgroundWorker1.RunWorkerAsync();
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();
            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion

        #region 表示と配置

        /// <summary>
        /// 画像再配置
        /// </summary>
        private void SetLocationPnlBitmap()
        {

            int nowX = -pnlBitmap.AutoScrollPosition.X;
            int nowY = -pnlBitmap.AutoScrollPosition.Y;

            pnlBitmap.AutoScrollPosition = new Point(0, 0);
            index = 0;
            foreach (Control ctrl in pnlBitmap.Controls)
            {
                ctrl.Location = new Point(CWData.Bitmap_Width * (index % disp_num_x)
                    , CWData.Bitmap_Height * (index / disp_num_x));
                index++;
            }

            pnlBitmap.AutoScrollPosition = new Point(nowX, nowY);

        }

        /// <summary>
        /// 移動量分だけスクロール(画像)
        /// </summary>
        /// <param name="delta"></param>
        private void ScrollPnlBitmap(int delta)
        {
            int nowX = -pnlBitmap.AutoScrollPosition.X;
            int nowY = -pnlBitmap.AutoScrollPosition.Y;
            nowY += delta;
            if (nowY < 0)
            {
                nowY = 0;
            }

            if (nowY > CWData.Bitmap_Height * ((pnlBitmap.Controls.Count - 1) / disp_num_x + 1))
            {
                nowY = CWData.Bitmap_Height * ((pnlBitmap.Controls.Count - 1) / disp_num_x + 1);
            }

            pnlBitmap.AutoScrollPosition = new Point(nowX, nowY);
            
        }

        #endregion

        #region マウスイベント
        /// <summary>
        /// マウスホイールの移動量を取得
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Bitmap_MouseWheel(object sender, MouseEventArgs e)
        {
            ScrollPnlBitmap(-e.Delta);
        }


        /// <summary>
        ///  画像パネル　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlBitmap_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                // 処理対象
                Point pt = pnlBitmap.PointToClient(Cursor.Position);
                int indexOfX = (-pnlBitmap.AutoScrollPosition.X + pt.X) / CWData.Bitmap_Width;
                int indexOfY = (-pnlBitmap.AutoScrollPosition.Y + pt.Y) / CWData.Bitmap_Height;

                index = indexOfY * disp_num_x + indexOfX;

                if (indexOfX < disp_num_x &&
                    index < pnlBitmap.Controls.Count + ((cd != null) ? 1 : 0))
                {
                    // 範囲内
                    this.SelectedIndex = index;
                    this.DialogResult = DialogResult.OK;
                    this.Close();

                }
            }
            else if (e.Button == MouseButtons.Right)
            {
                // 右クリックはキャンセルにしました。
                // 閉じるボタンが遠かったので
                this.DialogResult = DialogResult.Cancel;
                this.Close();
            }
        }

        /// <summary>
        ///  画像パネル　マウスホバーイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlBitmap_MouseHover(object sender, EventArgs e)
        {

            // 処理対象
            Point pt = pnlBitmap.PointToClient(Cursor.Position);
            int indexOfX = (-pnlBitmap.AutoScrollPosition.X + pt.X) / CWData.Bitmap_Width;
            int indexOfY = (-pnlBitmap.AutoScrollPosition.Y + pt.Y) / CWData.Bitmap_Height;

            index = indexOfY * disp_num_x + indexOfX;

            if (indexOfX < disp_num_x &&
                index < pnlBitmap.Controls.Count + ((cd != null) ? 1 : 0))
            {
                // 範囲内
                toolTip1.SetToolTip(pnlBitmap.Controls[index], (string)pnlBitmap.Controls[index].Tag);
                toolTip1.Show((string)pnlBitmap.Controls[index].Tag, pnlBitmap.Controls[index]);
            }
            else
            {
                toolTip1.Hide(pnlBitmap.Controls[index]);
            }
        }
        #endregion



    }
}