﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;

using CWIDatas;
using CWI.Card;
using CWI.Button;
using CWI;
using CWI.Dialog;

namespace CWI.Dialog
{
    public partial class CardInfoDialog : CWI.Dialog.SelectableForm
    {
        // 表示すべき要素
        private bool isWidCard = true;

        private bool isScenario = false;
        private InnData inn = null;

        private CharacterData chara = null;

        private WidCard data;
        private List<WidCard> datas = new List<WidCard>();

        private HandCard hcData;
        private List<HandCard> hcDatas = new List<HandCard>();

        public int SelectedIndex = 0;

        public CardInfoDialog()
        {
            // 通常使用しない
            InitializeComponent();
        }

        public CardInfoDialog(List<WidCard> datas, int index)
        {
            SetWidCards(datas);
            base.index = index;
        }

        public CardInfoDialog(InnData inn, List<WidCard> datas, int index)
        {
            this.inn = inn;
            SetScenarioWidCards(datas);
            base.index = index;
        }

        public CardInfoDialog(List<HandCard> hcDatas, int index)
        {
            SetHandCards(hcDatas);
            base.index = index;
        }

        public CardInfoDialog(bool paramOpt, 
            CharacterData chara,List<HandCard> hcDatas, int index)
        {
            //丸秘チェックありの場合だけキャラを持っておく
            if(paramOpt) this.chara = chara;
            SetHandCards(hcDatas);
            base.index = index;
        }

        private void SetWidCards(List<WidCard> datas)
        {
            // キャラクタ所持カードの表示（デコや背景はTODO）

            InitializeComponent();

            this.datas = datas;
            count = datas.Count;

            //ボタン設定
            BtnSelect.Visible = false;

            BtnCwxEditor.Visible = false;

            BtnCancel.Text = "閉じる";
            tablist.Add(BtnCancel);

            // Count数に合わせて＜＞ボタン表示・非表示
            SetUpData();

        }

        private void SetScenarioWidCards(List<WidCard> datas)
        {
            // キャラクタ所持カードの表示（デコや背景はTODO）

            InitializeComponent();

            this.datas = datas;
            count = datas.Count;

            //ボタン設定
            BtnSelect.Text = "取得";
            BtnSelect.Visible = true;
            tablist.Add(BtnSelect);

            BtnCwxEditor.Text = "情報";
            BtnCwxEditor.Visible = true;
            tablist.Add(BtnCwxEditor);

            BtnCancel.Text = "閉じる";
            tablist.Add(BtnCancel);

            this.isScenario = true;

            SetUpPosition();

            // Count数に合わせて＜＞ボタン表示・非表示
            SetUpData();

        }

        private void SetHandCards(List<HandCard> hcDatas)
        {

            InitializeComponent();

            isWidCard = false;

            this.hcDatas = hcDatas;
            count = hcDatas.Count;

            //ボタン設定
            BtnSelect.Visible = false;

            BtnCwxEditor.Visible = false;

            if (chara != null)
            {
                this.BtnEffectView.Visible = true;
                BtnEffectView.Text = "効果";
                tablist.Add(BtnEffectView);
            }

            BtnCancel.Text = "閉じる";
            tablist.Add(BtnCancel);

            if (chara != null)
            {
                SetUpPosition();
            }

            // Count数に合わせて＜＞ボタン表示・非表示
            SetUpData();
        }

        protected override void SetUpData()
        {
            base.SetUpData();

            this.groupBox1.Visible = !(count == 0);

        }


        protected override void doCancel()
        {
            this.SelectedIndex = index;
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }


        protected override void doSelectPage()
        {
            if (isScenario)
            {
                //inn.WidCardsにアクセスしたことが無かったらいけないので
                List<WidCard> cards = inn.WidCards;
                if (inn.CurrentParty != null)
                {
                    inn.AddRenamedCardData(inn.CurrentParty.BagCards, datas[index], 1);
                }
                else
                {
                    inn.AddRenamedCardData(inn.BankBoxCards, datas[index], 1);
                }
            }
        }

        protected override void ShowPage()
        {
            if (count > 0)
            {

                if (isWidCard)
                {
                    data = datas[index];

                    // 枠の部分のカード名
                    this.groupBox1.Text = data.CardName;

                    // 枠右下のシナリオ（著作者）
                    string scenario = string.Empty;
                    if (!data.ScenarioName.Equals(string.Empty))
                    {
                        scenario += data.ScenarioName;

                        if (!data.AuthorName.Equals(string.Empty))
                        {
                            scenario += "(" + data.AuthorName + ")";
                        }
                    }

                    this.lblScenario.Text = scenario;

                    // 情報
                    txtExpression.Text = data.Expression;

                    ViewBitmap = new Bitmap(ViewPanel.Width, ViewPanel.Height);
                    using (Graphics g = Graphics.FromImage(ViewBitmap))
                    {

                        //画像があったら
                        if (data.Image != null && data.Image.bitmap != null)
                        {
                            g.DrawImage(data.Image.bitmap, 5, 17);
                        }

                        using (Font smallFont = new Font("ＭＳ 明朝", 9, FontStyle.Bold))
                        {
                            StringFormat format = new StringFormat();
                            format.Alignment = StringAlignment.Center;
                            // カード左上にカード名
                            g.DrawString(data.CardName, smallFont, SolidBlack, new Rectangle(2, 2, 84, 15), format);

                            // レアリティは背景画像に変更するのでコメントアウト
                            //g.DrawString(data.Rarity.ToString(), smallFont, SolidBlack, 30, 117);

                        }

                    }

                }
                else
                {
                    //HandCard

                    hcData = hcDatas[index];
                    data = hcData.widData;

                    // 枠の部分のカード名
                    this.groupBox1.Text = data.CardName;

                    // 枠右下のシナリオ（著作者）
                    string scenario = string.Empty;
                    if (!data.ScenarioName.Equals(string.Empty))
                    {
                        scenario += data.ScenarioName;

                        if (!data.AuthorName.Equals(string.Empty))
                        {
                            scenario += "(" + data.AuthorName + ")";
                        }
                    }

                    this.lblScenario.Text = scenario;

                    // 情報
                    txtExpression.Text = data.Expression;

                    ViewBitmap = new Bitmap(ViewPanel.Width, ViewPanel.Height);
                    using (Graphics g = Graphics.FromImage(ViewBitmap))
                    {

                        //画像があったら
                        if (hcData.CardImage != null)
                        {
                            g.DrawImage(hcData.CardImage, 5, 8);
                        }

                        if (hcData.CardDecoImage != null)
                        {
                            g.DrawImage(hcData.CardDecoImage, 5, 8);
                        }
                    }



                }
            }
            else
            {

                ViewBitmap = new Bitmap(ViewPanel.Width, ViewPanel.Height);

            }

            ViewPanel.Invalidate();
        }

        private void CardInfoDialog_Shown(object sender, EventArgs e)
        {

            // マウス・カーソルの位置をクライアント中央に移動させる
            Cursor.Position = new Point(this.Location.X + this.Width / 2, this.Location.Y + this.Height);

            ShowPage();
        }

        private void BtnCwxEditor_Click(object sender, EventArgs e)
        {
            if (isScenario && count > 0)
            {
                string cwxeditor = FilePath.ApplicationFolder +
                        "\\cwxeditor.exe";
                string filename = datas[index].FileName;

                if (!string.IsNullOrEmpty(filename) &&
                    System.IO.File.Exists(cwxeditor))
                {
                    cwxeditor = "\"" + cwxeditor + "\"";
                    string args = "\"" + filename + "\"";
                    System.Diagnostics.Process.Start(cwxeditor, args);
                }
                else
                {
                    MessageDlg.ShowInfoDialog("cwxeditorがバリアントエンジンと同じ\r\n"
                        + "フォルダにありません。");
                }
            }
        }

        private void BtnEffectView_Click(object sender, EventArgs e)
        {
            if (count > 0)
            {
                //HandCard
                hcData = hcDatas[index];
                data = hcData.widData;

                if (chara != null &&
                    (data.WidID == WidIDtype.スキル ||
                     data.WidID == WidIDtype.アイテム ||
                     data.WidID == WidIDtype.召喚獣))
                {
                    //丸秘チェックありの場合だけキャラを持っている
                    
                    using (ExpressionDlg exd = new ExpressionDlg(this, chara, data))
                    {
                        exd.ShowDialog();
                    }



                }
            }

        }
    }

}

