﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class CharacterDlg : Form
    {
        #region メンバ・プロパティ
        private Form _owner;
        private PartyMaker _pm = null;
        private InnData inn;
        private int[] rotateTimes = { 0, 4, 1, 5, 2, 6, 3 };
        private GeneData newGene;
        private bool haveParent = false;

        private CharacterData newCharacter = new CharacterData();
        public CharacterData Cha
        {
            set
            {
                newCharacter = value;
            }
            get
            {
                return newCharacter;
            }
        }

        private int INIT_PHYS_PARAM = 6;
        private int INIT_PHTS_BOX_PARAM = 1;
        private int PHYS_PARAM_MAX = 12;
        private int PHYS_PARAM_SPECIAL_MAX = 15;
        private int MIND_PARAM_MIN = -4;
        private int MIND_PARAM_MAX = 4;

        private DataTable dtChara;

        private int bmpIndex = 0;

        private List<string> featureBmpList = new List<string>();
        public List<string> FeatureBmpList
        {
            set
            {
                featureBmpList = value;
            }
            get
            {
                return featureBmpList;
            }
        }

        private int fatherIndex = 0;

        private List<DataWithIO> fatherList = new List<DataWithIO>();
        public List<DataWithIO> FatherList
        {
            set
            {
                fatherList = value;
            }
            get
            {
                return fatherList;
            }
        }

        private int motherIndex = 0;

        private List<DataWithIO> motherList = new List<DataWithIO>();
        public List<DataWithIO> MotherList
        {
            set
            {
                motherList = value;
            }
            get
            {
                return motherList;
            }
        }

        private double aggressive = 0.0;
        private double cheerful = 0.0;
        private double brave = 0.0;
        private double cautious = 0.0;
        private double trickish = 0.0;

        private string[,] characterName = 
        {
            {"秀麗","醜悪","高貴の出","下賎の出"},
            {"都会育ち","田舎育ち","裕福","貧乏"},
            {"厚き信仰","不心得者","誠実","不実"},
            {"冷静沈着","猪突猛進","貪欲","無欲"},
            {"献身的","利己的","秩序派","混沌派"},
            {"進取派","保守派","神経質","鈍感"},
            {"好奇心旺盛","無頓着","過激","穏健"},
            {"楽観的","悲観的","勤勉","遊び人"},
            {"陽気","内気","派手","地味"},
            {"高慢","謙虚","上品","粗野"},
            {"武骨","繊細","硬派","軟派"},
            {"お人好し","ひねくれ者","名誉こそ命","愛に生きる"}
        };

        private List<string> characterList = new List<string>();

        private string talent = string.Empty;

        private BmpSelectDlg modaless = null;

        #endregion

        #region コンストラクタ
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public CharacterDlg(InnData inn)
        {
            InitializeComponent();

            this.inn = inn;
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public CharacterDlg(Form owner,InnData inn)
        {
            InitializeComponent();

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            this.inn = inn;
            
            _pm = _owner as PartyMaker;
        }

        #endregion

        #region フォームイベント

        /// <summary>
        /// フォームロード
        /// </summary>
        private void CharacterDlg_Load(object sender, EventArgs e)
        {
            if (_pm != null && _pm.paramOption)
            {
                btnExpression.Text = "適性";
            }
            

            new MessageDlg();
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CharacterDlg_Shown(object sender, EventArgs e)
        {
            // 初期設定
            txtName.Text = string.Empty;
            radMale.Checked = true;
            radYoung.Checked = true;
            radNormal.Checked = true;

            // 初期設定（グリッド）
            dgvCharacter.RowHeadersVisible = false;
            dgvCharacter.ColumnHeadersVisible = false;
            
            dtChara = new DataTable();
            DataColumn dataCol;
            for (int i = 0; i < 4; i++)
            {
                dataCol = new DataColumn();
                dataCol.ColumnName = "Check" + (i + 1);
                dataCol.DataType = typeof(bool);
                dtChara.Columns.Add(dataCol);


                dataCol = new DataColumn();
                dataCol.ColumnName = "Text" + (i + 1);
                dataCol.DataType = typeof(string);
                dtChara.Columns.Add(dataCol);
            }

            DataRow dr;

            for (int i = 0; i < 12; i++)
            {
                dr = dtChara.NewRow();
                dr[0] = false; 
                dr[2] = false; 
                dr[4] = false; 
                dr[6] = false;
                dr[1] = characterName[i,0];
                dr[3] = characterName[i,1];
                dr[5] = characterName[i,2];
                dr[7] = characterName[i,3];
                dtChara.Rows.Add(dr);
            }

            dgvCharacter.DataSource = dtChara;

            // バインド後のグリッド調整
            dgvCharacter.Columns[0].Width = 20;
            dgvCharacter.Columns[2].Width = 20;
            dgvCharacter.Columns[4].Width = 20;
            dgvCharacter.Columns[6].Width = 20;
            dgvCharacter.Columns[1].Width = 70;
            dgvCharacter.Columns[3].Width = 60;
            dgvCharacter.Columns[5].Width = 64;
            dgvCharacter.Columns[7].Width = 62;
            dgvCharacter.Columns[1].ReadOnly = true;
            dgvCharacter.Columns[3].ReadOnly = true;
            dgvCharacter.Columns[5].ReadOnly = true;
            dgvCharacter.Columns[7].ReadOnly = true;

            //3種類のチェック状態を表示できないようにする
            DataGridViewCheckBoxColumn column
                = (DataGridViewCheckBoxColumn)dgvCharacter.Columns[0];
            column.ThreeState = false;
            column = (DataGridViewCheckBoxColumn)dgvCharacter.Columns[2];
            column.ThreeState = false; 
            column =　(DataGridViewCheckBoxColumn)dgvCharacter.Columns[4];
            column.ThreeState = false;
            column = (DataGridViewCheckBoxColumn)dgvCharacter.Columns[6];
            column.ThreeState = false;



            // 初期設定（血縁パネル）
            // アルバムキャラクタ
            AlbumCharacter albumChara; 
            foreach (string file in Directory.GetFiles(inn.TempFolder, FilePath.MaskAlbumCharacterData))
            {
                try
                {
                    albumChara = new AlbumCharacter(file);

                    if (albumChara.CanMakeChild())
                    {
                        Sex seibetsu = albumChara.GetSex();
                        if (seibetsu == Sex.Male)
                        {
                            FatherList.Add(albumChara);
                        }
                        else if (seibetsu == Sex.Female)
                        {
                            MotherList.Add(albumChara);
                        }
                    }
                }
                catch (Exception ex)
                {
                    PartyMaker.CWIErrorMessage("アルバムメンバーファイルの読み込みに失敗：" + ex.Message);
                }

                
            }
            // 待機PC
            foreach (CharacterData chara in inn.Characters)
            {
                if (chara.CanMakeChild())
                {
                    Sex seibetsu = chara.GetSex();
                    if (seibetsu == Sex.Male)
                    {
                        FatherList.Add(chara);
                    }
                    else if (seibetsu == Sex.Female)
                    {
                        MotherList.Add(chara);
                    }
                }   
            }

            if (FatherList.Count == 0 
                && MotherList.Count == 0)
            {
                pnlBlood.Visible = false;
            }
            else
            {
                pnlBlood.Visible = true;
                fatherIndex = FatherList.Count;
                motherIndex = MotherList.Count;
                ShowFatherBmp();
                ShowMotherBmp();
            }

            // バリアント対応 
            radNormal.Text = CWData.DISP_NORMAL;
            radBannou.Text = CWData.DISP_ALLMIGHTY;
            radYuushou.Text = CWData.DISP_BRAVE;
            radGouketsu.Text = CWData.DISP_DARING;
            radChishou.Text = CWData.DISP_RESOURCEFUL;
            radSakushi.Text = CWData.DISP_TACTIC;
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CharacterDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 開放するリソースがあれば
            if (modaless != null)
            {
                modaless.Dispose();
            }
        }


        #endregion

        #region クリックイベント

        /// <summary>
        /// 説明ボタン　クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExpression_Click(object sender, EventArgs e)
        {
            using (ExpressionDlg func = new ExpressionDlg(this, _pm.paramOption))
            {
                func.ShowDialog();
            }
        }
        /// <summary>
        /// 名前自動ボタン　クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAutoName_Click(object sender, EventArgs e)
        {
            string str;
            string[] strs;
            if (radMale.Checked)
            {
                str = FilePath.GetTextStream("MaleNames");
            }
            else
            {
                str = FilePath.GetTextStream("FemaleNames");
            }
            strs = str.Replace("\r\n", "\n").Split('\n');
            int rndIndex = DiceRoll.RandomInt(strs.Length);
            this.txtName.Text = strs[rndIndex];

        }
        /// <summary>
        /// 閉じるボタン　クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            if (DialogResult.Yes
                == MessageDlg.ShowYesNoDialog(
                "キャラクタを破棄します。\r\nよろしいですか？"))
            {
                this.DialogResult = DialogResult.Cancel;
                this.Close();
            }

        }

        private void btnCharacterLeft_Click(object sender, EventArgs e)
        {
            bmpIndex--;
            if (bmpIndex < 0)
            {
                bmpIndex = FeatureBmpList.Count - 1;
            }
            ShowFeatureBmp();
        }

        private void btnCharacterRight_Click(object sender, EventArgs e)
        {
            bmpIndex++;
            if (bmpIndex >= FeatureBmpList.Count)
            {
                bmpIndex = 0;
            }
            ShowFeatureBmp();
        }

        private void picCharacter_Click(object sender, EventArgs e)
        {
            // モーダレス化に伴い、コメントアウト
            ////using (BmpSelectDlg func = new BmpSelectDlg(FeatureBmpList))
            ////{
            ////    if (func.ShowDialog() == DialogResult.OK)
            ////    {
            ////        bmpIndex = func.SelectedIndex;
            ////        ShowFeatureBmp();
            ////    }
            ////}

            modaless = new BmpSelectDlg(FeatureBmpList);
            modaless.FormClosed += new FormClosedEventHandler(modaless_FormClosed);
            modaless.Show();
        }

        /// <summary>
        /// モーダレスフォームを閉じた時に走るイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void modaless_FormClosed(object sender, FormClosedEventArgs e)
        {
            bmpIndex = modaless.SelectedIndex;
            ShowFeatureBmp();
            modaless = null;
        }

        private void ShowFeatureBmp()
        {
            Bitmap bitmap = new Bitmap(FeatureBmpList[bmpIndex]);
            bitmap.MakeTransparent(bitmap.GetPixel(0, 0));
            picCharacter.Image = bitmap;
        }

        /// <summary>
        /// 個性自動ボタン　クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCharacteristic_Click(object sender, EventArgs e)
        {
            Random rnd = new Random();
            for (int i = 0; i < 12; i++)
            {
                for (int j = 0; j < 2; j++)
                {
                    int val = rnd.Next(100);
                    // 30%バージョン
                    if (val >= 30)
                    {
                        this.dgvCharacter.Rows[i].Cells[j * 4].Value = false;
                        this.dgvCharacter.Rows[i].Cells[j * 4 + 2].Value = false;
                    }
                    else if (val >= 15)
                    {
                        this.dgvCharacter.Rows[i].Cells[j * 4].Value = true;
                        this.dgvCharacter.Rows[i].Cells[j * 4 + 2].Value = false;
                    }
                    else
                    {
                        this.dgvCharacter.Rows[i].Cells[j * 4].Value = false;
                        this.dgvCharacter.Rows[i].Cells[j * 4 + 2].Value = true;
                    }
                }
            }
        }

        private void btnRandom_Click(object sender, EventArgs e)
        {
            //ランダムにキャラクタを生成
            Random rnd = new Random();
            do
            {
                if (rnd.Next(2) == 0)
                {
                    radMale.Checked = true;
                }
                else
                {
                    radFemale.Checked = true;
                }

                switch (rnd.Next(4))
                {
                    case 0:
                        radChild.Checked = true;
                        break;
                    case 1:
                        radYoung.Checked = true;
                        break;
                    case 2:
                        radAdult.Checked = true;
                        break;
                    default:
                        radOld.Checked = true;
                        break;
                }
            }
            while (!pnlFeature.Visible);

            bmpIndex = rnd.Next(FeatureBmpList.Count);
            ShowFeatureBmp();

            btnAutoName.PerformClick();

            switch (rnd.Next(6))
            {
                case 0:
                    this.radNormal.Checked = true;
                    break;
                case 1:
                    this.radYuushou.Checked = true;
                    break;
                case 2:
                    this.radChishou.Checked = true;
                    break;
                case 3:
                    this.radBannou.Checked = true;
                    break;
                case 4:
                    this.radGouketsu.Checked = true;
                    break;
                default:
                    this.radSakushi.Checked = true;
                    break;
            }

            btnCharacteristic.PerformClick();

            //if (RegistConfirm() == true)
            //{
            //    btnRegist_Click(sender, e);
            //}
        }

        #region 親設定メソッド
        private void btnFatherLeft_Click(object sender, EventArgs e)
        {
            fatherIndex--;
            if (fatherIndex < 0)
            {
                fatherIndex = FatherList.Count;
            }

            ShowFatherBmp();

        }

        private void btnFatherRight_Click(object sender, EventArgs e)
        {
            fatherIndex++;
            if (fatherIndex > FatherList.Count)
            {
                fatherIndex = 0;
            }
            ShowFatherBmp();
        }

        private void ShowFatherBmp()
        {
            Bitmap bitmap;
            if (fatherIndex == FatherList.Count)
            {
                bitmap = new ImageData(CWIEngine.Properties.Resources.Card_一般男).bitmap;

                picFather.Image = bitmap;


                this.lblFatherName.Text = "一般男性";
                this.lblFatherExp.Text = "";
            }
            else
            {
                DataWithIO io = FatherList[fatherIndex];
                if (io is CharacterData)
                {
                    // キャラクタ
                    CharacterData chara = (CharacterData)io;
                    bitmap = chara.Image.bitmap;
                    this.lblFatherName.Text = chara.CardName;
                    switch (chara.GetAge())
                    {
                        case Age.Old:
                            this.lblFatherExp.Text = "消費EP：20 (残：" + chara.EP + ")";
                            break;
                        case Age.Adult:
                            this.lblFatherExp.Text = "消費EP：30 (残：" + chara.EP + ")";
                            break;
                        case Age.Young:
                            this.lblFatherExp.Text = "消費EP：40 (残：" + chara.EP + ")";
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない年代です。（ShowFatherBmp）");
                            break;
                    }
                }
                else
                {
                    // アルバムキャラクタ
                    AlbumCharacter albumChara = (AlbumCharacter)io;
                    bitmap = albumChara.Image.bitmap;
                    this.lblFatherName.Text = albumChara.Name;
                    this.lblFatherExp.Text = "消費EP：10 (残：" + albumChara.EP + ")";
                }

                picFather.Image = bitmap;
            }
        }

        private void btnMotherLeft_Click(object sender, EventArgs e)
        {

            motherIndex--;
            if (motherIndex < 0)
            {
                motherIndex = MotherList.Count;
            }

            ShowMotherBmp();
        }

        private void btnMotherRight_Click(object sender, EventArgs e)
        {

            motherIndex++;
            if (motherIndex > MotherList.Count)
            {
                motherIndex = 0;
            }
            ShowMotherBmp();
        }

        private void ShowMotherBmp()
        {
            Bitmap bitmap;
            if (motherIndex == MotherList.Count)
            {
                bitmap = new ImageData(CWIEngine.Properties.Resources.Card_一般女).bitmap;
                picMother.Image = bitmap;


                this.lblMotherName.Text = "一般女性";
                this.lblMotherExp.Text = "";
            }
            else
            {
                DataWithIO io = MotherList[motherIndex];
                if (io is CharacterData)
                {
                    // キャラクタ
                    CharacterData chara = (CharacterData)io;
                    bitmap = chara.Image.bitmap;
                    this.lblMotherName.Text = chara.CardName;
                    switch (chara.GetAge())
                    {
                        case Age.Old:
                            this.lblMotherExp.Text = "消費EP：20 (残：" + chara.EP + ")";
                            break;
                        case Age.Adult:
                            this.lblMotherExp.Text = "消費EP：30 (残：" + chara.EP + ")";
                            break;
                        case Age.Young:
                            this.lblMotherExp.Text = "消費EP：40 (残：" + chara.EP + ")";
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない年代です。（ShowMotherBmp）");
                            break;
                    }
                }
                else
                {
                    // アルバムキャラクタ
                    AlbumCharacter albumChara = (AlbumCharacter)io;
                    bitmap = albumChara.Image.bitmap;
                    this.lblMotherName.Text = albumChara.Name;
                    this.lblMotherExp.Text = "消費EP：10 (残：" + albumChara.EP + ")";
                }

                picMother.Image = bitmap;
            }

        }
        #endregion

        #region 登録関連メソッド
        /// <summary>
        /// 登録ボタン　クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRegist_Click(object sender, EventArgs e)
        {
            // 登録する
            SetNewCharacter();

            // 待機PCファイル生成
            Cha.FileName = string.Empty;
            inn.AddCharacterFile(Cha);
            //キャラクタリスト追加
            inn.Characters.Add(Cha);

            // DialogResult
            this.DialogResult = DialogResult.OK;
            // 終了
            this.Close();
        }

        public bool RegistConfirm()
        {
            return MessageDlg.ShowYesNoDialog(
                "登録してもよろしいですか？")
                == DialogResult.Yes;
        }
        #endregion

        private void dgvCharacter_CellClick(object sender, DataGridViewCellEventArgs e)
        {

            if (e.RowIndex >= 0 && e.RowIndex < 12 && e.ColumnIndex % 2 == 1)
            {                
                if (dgvCharacter.Rows[e.RowIndex].Cells[e.ColumnIndex - 1].Value.Equals(true))
                {
                    dgvCharacter.Rows[e.RowIndex].Cells[e.ColumnIndex - 1].Value = false;
                } 
                else 
                {
                    dgvCharacter.Rows[e.RowIndex].Cells[e.ColumnIndex - 1].Value = true;
                    dgvCharacter_CellValueChanged(sender, new DataGridViewCellEventArgs(e.ColumnIndex - 1,e.RowIndex));
                }

            }
        }

        #endregion

        #region チェンジイベント

        private void txtName_TextChanged(object sender, EventArgs e)
        {
            btnRegist.Enabled = !txtName.Text.Equals(string.Empty) && pnlFeature.Visible;
        }

        private void radAgeAndSex_CheckedChanged(object sender, EventArgs e)
        {
            if (((RadioButton)sender).Checked)
            {
                if (
                    (radMale.Checked || radFemale.Checked)
                    &&
                    (radChild.Checked || radYoung.Checked || radAdult.Checked || radOld.Checked)
                  )
                {

                    // 8パターンでファイル読み込み
                    // なかったら pnlFeature.Visible = false;
                    // あったらリスト化して表示
                    Sex sex = radMale.Checked ? Sex.Male : Sex.Female;
                    Age age;

                    if (radChild.Checked)
                    {
                        age = Age.Child;
                    }
                    else if (radYoung.Checked)
                    {
                        age = Age.Young;
                    }
                    else if (radAdult.Checked)
                    {
                        age = Age.Adult;
                    }
                    else
                    {
                        age = Age.Old;
                    }

                    string folder = FilePath.GetDataFaceFolder(sex, age);
                    FeatureBmpList.Clear();
                    FeatureBmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskBitmap));
                    FeatureBmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskPng));
                    FeatureBmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskGif));

                    if (FeatureBmpList.Count > 0)
                    {
                        pnlFeature.Visible = true;
                        bmpIndex = 0;
                        ShowFeatureBmp();
                    }
                    else
                    {
                        pnlFeature.Visible = false;
                    }

                    if (modaless != null)
                    {
                        // 画像表示ダイアログが出ていたら
                        modaless.PathList = FeatureBmpList;
                        modaless.ShowBitmaps();
                    }
                }
                else
                {
                    pnlFeature.Visible = false;
                }

                btnRegist.Enabled = !txtName.Text.Equals(string.Empty) && pnlFeature.Visible;
            }
        }

        /// <summary>
        /// グリッドの値変更時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvCharacter_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            //Thanks to DOBON.NET

            if (e.RowIndex >= 0 && e.RowIndex < 12 && e.ColumnIndex % 2 == 0)
            {
                if (dgvCharacter.Rows[e.RowIndex].Cells[e.ColumnIndex].Value.Equals(true))
                {
                    // チェックが入った時
                    switch (e.ColumnIndex)
                    {
                        case 0:
                            if (dgvCharacter.Rows[e.RowIndex].Cells[2].Value.Equals(true))
                            {
                                dgvCharacter.Rows[e.RowIndex].Cells[2].Value = false;
                            }
                            break;
                        case 2:
                            if (dgvCharacter.Rows[e.RowIndex].Cells[0].Value.Equals(true))
                            {
                                dgvCharacter.Rows[e.RowIndex].Cells[0].Value = false;
                            }
                            break;
                        case 4:
                            if (dgvCharacter.Rows[e.RowIndex].Cells[6].Value.Equals(true))
                            {
                                dgvCharacter.Rows[e.RowIndex].Cells[6].Value = false;
                            }
                            break;
                        case 6:
                            if (dgvCharacter.Rows[e.RowIndex].Cells[4].Value.Equals(true))
                            {
                                dgvCharacter.Rows[e.RowIndex].Cells[4].Value = false;
                            }
                            break;
                        default:
                            return;
                    }

                }
                
            } 
        }

        /// <summary>
        /// とりあえずのグリッドの内容変更を捉えるイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvCharacter_CurrentCellDirtyStateChanged(object sender, EventArgs e)
        {
            //Thanks to DOBON.NET

            // 現在のアクティブセルがチェックボックスで、
            // 現在のセルがコミットされていない
            if (dgvCharacter.CurrentCellAddress.X % 2 == 0 &&
                dgvCharacter.IsCurrentCellDirty)
            {
                //コミットする
                //これにより変更直後にCellValueChanged が走るようになる
                dgvCharacter.CommitEdit(DataGridViewDataErrorContexts.Commit);
            }

        }

        #endregion

        #region メソッド

        /// <summary>
        /// 新規キャラクタ設定
        /// </summary>
        public void SetNewCharacter()
        {
            Cha = new CharacterData();

            aggressive = 0.0;
            cheerful = 0.0;
            brave = 0.0;
            cautious = 0.0;
            trickish = 0.0;

            // 名前
            Cha.CardName = this.txtName.Text;
            Cha.SetCoupon("＿" + Cha.CardName, 0);
            // 画像
            Cha.Image = new ImageData(new Bitmap(FeatureBmpList[bmpIndex]));

            // 初期設定（パラメータ）
            // Physical
            // CWPyでの中世Ⅰ型（人間）を初期値とする
            Cha.Dex = INIT_PHYS_PARAM;
            Cha.Agl = INIT_PHYS_PARAM;
            Cha.Int = INIT_PHYS_PARAM;
            Cha.Str = INIT_PHYS_PARAM;
            Cha.Vit = INIT_PHYS_PARAM;
            Cha.Min = INIT_PHYS_PARAM;

            // Mental
            // aggressive　など
            // 初期値は0なのでそのまま

            // Enhance
            // 拡張パラメータも初期値は0なのでそのまま

            // 性別
            SetSex();
            // 年代
            SetAge();
            // 親
            SetParents();
            // 素質
            SetTalent();
            // 属性
            SetAttribute();
            // 解説（属性分）
            SetDescription();
            // 特別クーポン
            SetSpecialCoupon();
            // パラメタ範囲制限
            ParamWrap();
            // HP
            SetLife();
            // データバージョン
            SetVersion();
        }

        /// <summary>
        /// 性別設定
        /// </summary>
        private void SetSex()
        {
            if (radMale.Checked)
            {
                Cha.SetCoupon(CWData.STR_MALE, 0);
                Cha.Str += 1;
                aggressive += 0.5;
            }
            else
            {
                Cha.SetCoupon(CWData.STR_FEMALE, 0);
                Cha.Dex += 1;
                cautious += 0.5;
            }
        }


        /// <summary>
        /// 年代設定
        /// </summary>
        private void SetAge()
        {
            if (radChild.Checked)
            {
                Cha.SetCoupon(CWData.STR_CHILD, 0);
                Cha.Level = 1;
                Cha.Dex += 1;
                Cha.Agl += 1;
                Cha.Str -= 1;
                Cha.Vit -= 1;
                cheerful += 0.5;
                cautious -= 0.5;

            }
            else if (radYoung.Checked)
            {
                Cha.SetCoupon(CWData.STR_YOUNG, 0);
                Cha.Level = 1;
            }
            else if (radAdult.Checked)
            {
                Cha.SetCoupon(CWData.STR_ADULT, 0);
                Cha.SetCoupon(CWData.ADULT_COUPON_NAME, 2);
                Cha.Level = 2;
                Cha.Vit -= 1;
                aggressive -= 0.5;
                cautious += 0.5;

            }
            else
            {
                Cha.SetCoupon(CWData.STR_OLD, 0);
                Cha.SetCoupon(CWData.OLD_COUPON_NAME, 4);
                Cha.Level = 2;
                Cha.Dex -= 1;
                Cha.Agl -= 1;
                Cha.Int += 1;
                Cha.Str -= 1;
                Cha.Vit -= 1;
                Cha.Min += 1;
                aggressive -= 0.5;
                brave -= 0.5;
                cautious += 0.5;
                trickish += 0.5;

            }

        }

        /// <summary>
        /// 親設定
        /// </summary>
        private void SetParents()
        {
            GeneData fatherGene;
            GeneData motherGene;
            string fgeneStr = CWData.GENE_HEADER + "0000000000";
            string mgeneStr = CWData.GENE_HEADER + "0000000000";
            CouponData tempData;
            int fatherTimes = 0;
            int motherTimes = 0;

            if (pnlBlood.Visible && fatherIndex != FatherList.Count)
            {
                // 父親が割り当てられている。
                haveParent = true;
                DataWithIO father = FatherList[fatherIndex];
                if (father is AlbumCharacter)
                {
                    AlbumCharacter acFather = (AlbumCharacter)father;

                    // EP消費
                    acFather.EP = acFather.EP - 10;

                    // 交配経験数
                    fgeneStr = acFather.GetGeneString();
                    tempData = acFather.GetCouponData(fgeneStr);
                    fatherTimes = tempData.Point;
                    acFather.SetCoupon(fgeneStr, fatherTimes + 1);

                    // アルバムファイルにEP消費・交配経験数を反映
                    inn.AddAlbumFile(acFather);

                    // 名前
                    Cha.SetCoupon("父：" + acFather.Name, 0);

                }
                else
                {
                    CharacterData cdFather = (CharacterData)father;

                    // EP消費
                    switch (cdFather.GetAge())
                    {
                        case Age.Old:
                            cdFather.EP = cdFather.EP - 20;
                            break;

                        case Age.Adult:
                            cdFather.EP = cdFather.EP - 30;
                            break;

                        case Age.Young:
                            cdFather.EP = cdFather.EP - 40;
                            break;
                    }

                    // 交配経験数
                    fgeneStr = cdFather.GetGeneString();
                    tempData = cdFather.GetCouponData(fgeneStr);
                    fatherTimes = tempData.Point;
                    cdFather.SetCoupon(fgeneStr, fatherTimes + 1);

                    // ファイルにEP消費・交配経験数を反映
                    inn.AddCharacterFile(cdFather);

                    // 名前
                    Cha.SetCoupon("父：" + cdFather.CardName, 0);
                }

                fatherGene = new GeneData(fgeneStr);

                // コドンの得点分だけローテーションします。
                for (int i = 0; i < fatherTimes; i++)
                {
                    fatherGene.RotateLeft();
                }

            }
            else
            {
                // 父親が割り当てられていない。
                fatherGene = new GeneData(0);
                fatherGene.SetRandomBit();
            }

            if (pnlBlood.Visible && motherIndex != MotherList.Count)
            {
                // 母親が割り当てられている。
                haveParent = true;
                DataWithIO mother = MotherList[motherIndex];
                if (mother is AlbumCharacter)
                {
                    AlbumCharacter acMother = (AlbumCharacter)mother;
                    // EP消費
                    acMother.EP = acMother.EP - 10;

                    // 交配経験数
                    mgeneStr = acMother.GetGeneString();
                    tempData = acMother.GetCouponData(mgeneStr);
                    motherTimes = tempData.Point;
                    acMother.SetCoupon(mgeneStr, motherTimes + 1);

                    // アルバムファイルにEP消費・交配経験数を反映
                    inn.AddAlbumFile(acMother);

                    // 名前
                    Cha.SetCoupon("母：" + acMother.Name, 0);
                }
                else
                {
                    CharacterData cdMother = (CharacterData)mother;

                    // EP消費
                    switch (cdMother.GetAge())
                    {
                        case Age.Old:
                            cdMother.EP = cdMother.EP - 20;
                            break;

                        case Age.Adult:
                            cdMother.EP = cdMother.EP - 30;
                            break;

                        case Age.Young:
                            cdMother.EP = cdMother.EP - 40;
                            break;
                    }

                    // 交配経験数
                    mgeneStr = cdMother.GetGeneString();
                    tempData = cdMother.GetCouponData(mgeneStr);
                    motherTimes = tempData.Point;
                    cdMother.SetCoupon(mgeneStr, motherTimes + 1);

                    // ファイルにEP消費・交配経験数を反映
                    inn.AddCharacterFile(cdMother);

                    // 名前
                    Cha.SetCoupon("母：" + cdMother.CardName, 0);
                }


                motherGene = new GeneData(mgeneStr);

                // コドンの得点分だけローテーションします。
                // (10回以上（EP足りるの？）は変化なし)
                if (motherTimes > 10) motherTimes = 0;
                for (int i = 0; i < motherTimes; i++)
                {
                    motherGene.RotateRight();
                }

            }
            else
            {
                // 母親が割り当てられていない。
                motherGene = new GeneData(0);
                motherGene.SetRandomBit();
            }

            newGene = fatherGene.XorGenes(motherGene);
        }

        /// <summary>
        /// 素質設定
        /// </summary>
        private void SetTalent()
        {
            talent = string.Empty;

            if (radNormal.Checked)
            {
                talent = CWData.TYPE_NORMAL;
            }
            else if (this.radBannou.Checked)
            {
                talent = CWData.TYPE_ALLMIGHTY;
            }
            else if (this.radYuushou.Checked)
            {
                talent = CWData.TYPE_BRAVE;
            }
            else if (this.radGouketsu.Checked)
            {
                talent = CWData.TYPE_DARING;
            }
            else if (this.radChishou.Checked)
            {
                talent = CWData.TYPE_RESOURCEFUL;
            }
            else if (this.radSakushi.Checked)
            {
                talent = CWData.TYPE_TACTIC;
            }

            // 特殊型判定・特殊型論理否定演算（凡庸型以外）・素質情報反映
            int count = newGene.CountBits();
            if (count == 10)
            {
                talent = CWData.TYPE_GOD;
                newGene.ReverseBit();
            }
            else if (count >= 8)
            {
                talent = CWData.TYPE_HERO;
                newGene.ReverseBit();
            }
            else if (count >= 6)
            {

                newGene.ReverseBit();

                if (talent == CWData.TYPE_NORMAL || talent == CWData.TYPE_ALLMIGHTY)
                {
                    talent = CWData.TYPE_BRILLIANT;
                }
                else if (talent == CWData.TYPE_BRAVE || talent == CWData.TYPE_DARING)
                {
                    talent = CWData.TYPE_MATCHLESS;
                }
                else
                {
                    talent = CWData.TYPE_GENIUS;
                }

                // 上位型の素質情報反映
                newGene.SetTypeFlag(talent);
            }
            else if (count == 0 && haveParent)
            {
                string oldTalent = talent;
                talent = CWData.TYPE_ORDINARY;

                // 論理和を書くのが面倒なのでフラグ立てます
                newGene.SetTypeFlag(oldTalent);
                newGene.SetTypeFlag(CWData.TYPE_ORDINARY);
                newGene.SetTypeFlag(CWData.TYPE_BRILLIANT);
                newGene.SetTypeFlag(CWData.TYPE_MATCHLESS);
                newGene.SetTypeFlag(CWData.TYPE_GENIUS);

            }
            else
            {
                // 標準型の素質情報反映
                newGene.SetTypeFlag(talent);
            }

            Cha.SetCoupon(talent, 0);

            if (talent == CWData.TYPE_NORMAL)
            {
                // 標準型
                Cha.Min += 1;

                aggressive -= 0.5;
                cautious += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_ALLMIGHTY)
            {
                // 万能型
                Cha.Dex += 1;
                Cha.Agl += 1;
                Cha.Min -= 1;

                cheerful += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_BRAVE)
            {
                // 勇将型
                Cha.Dex -= 1;
                Cha.Int -= 1;
                Cha.Str += 2;

                brave += 1.0;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_DARING)
            {
                // 豪傑型
                Cha.Dex -= 2;
                Cha.Agl -= 1;
                Cha.Int -= 2;
                Cha.Str += 3;
                Cha.Vit += 1;
                Cha.Min -= 1;

                aggressive += 0.5;
                brave += 0.5;
                cautious -= 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_RESOURCEFUL)
            {
                // 知将型
                Cha.Int += 2;
                Cha.Str -= 1;
                Cha.Vit -= 1;

                cautious += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_TACTIC)
            {
                // 策士型
                Cha.Agl -= 1;
                Cha.Int += 3;
                Cha.Str -= 2;
                Cha.Vit -= 2;

                cautious += 0.5;
                trickish += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_BRILLIANT)
            {
                // 英明型
                Cha.Dex += 1;
                Cha.Agl += 1;
                Cha.Int += 1;
                Cha.Str += 1;
                Cha.Vit += 1;
                Cha.Min += 1;

                cautious += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_MATCHLESS)
            {
                // 無双型
                Cha.Agl += 1;
                Cha.Str += 3;
                Cha.Vit += 2;

                aggressive += 0.5;
                brave += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_GENIUS)
            {
                // 天才型
                Cha.Dex += 1;
                Cha.Int += 3;
                Cha.Min += 2;

                cautious += 0.5;
                trickish += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 10);

            }
            else if (talent == CWData.TYPE_ORDINARY)
            {
                // 凡庸型
                Cha.Dex -= 2;
                Cha.Agl -= 2;
                Cha.Int -= 2;
                Cha.Str -= 2;
                Cha.Vit -= 2;
                Cha.Min -= 2;

                brave -= 0.5;
                cautious += 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 12);

            }
            else if (talent == CWData.TYPE_HERO)
            {
                // 英雄型
                Cha.Dex += 1;
                Cha.Agl += 1;
                Cha.Int += 2;
                Cha.Str += 2;
                Cha.Vit += 1;
                Cha.Min += 2;

                cheerful += 0.5;
                brave += 0.5;
                trickish -= 0.5;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 12);

            }
            else if (talent == CWData.TYPE_GOD)
            {
                // 神仙型
                Cha.Dex += 2;
                Cha.Agl += 2;
                Cha.Int += 2;
                Cha.Str += 2;
                Cha.Vit += 2;
                Cha.Min += 2;

                Cha.SetCoupon(CWData.STR_LEVEL_MAX, 15);

            }

            // 解説に素質型を追加
            Cha.Expression = "TEXT\r\n　　　　　　　　" + talent.TrimStart('＿') + "\n\n";
        }

        /// <summary>
        /// 属性設定
        /// </summary>
        private void SetAttribute()
        {
            string attr;
            characterList.Clear();

            #region チェックボックスの判定
            for (int i = 0; i < 12; i++)
            {
                for (int j = 0; j < 4; j++)
                {
                    if (dgvCharacter.Rows[i].Cells[j * 2].Value.Equals(true))
                    {
                        attr = characterName[i, j];
                        // 解説用リスト
                        characterList.Add(attr);
                        // クーポン設定
                        Cha.SetCoupon("＿" + attr, 0);

                        // 属性ごとのパラメタ変更
                        if (attr == characterName[0, 0])
                        {
                            //秀麗
                            Cha.Vit -= 1;
                            cheerful += 0.5;
                        }
                        else if (attr == characterName[0, 1])
                        {
                            //醜悪
                            Cha.Vit += 1;
                            cheerful -= 0.5;
                        }

                        else if (attr == characterName[0, 2])
                        {
                            //高貴の出
                            aggressive -= 0.5;
                            brave += 0.5;
                        }
                        else if (attr == characterName[0, 3])
                        {
                            //下賎の出
                            cautious -= 0.5;
                            trickish += 0.5;
                        }

                        else if (attr == characterName[1, 0])
                        {
                            //都会育ち
                            Cha.Int += 1;
                            Cha.Vit -= 1;
                            cheerful += 0.5;
                            trickish += 0.5;
                        }
                        else if (attr == characterName[1, 1])
                        {
                            //田舎育ち
                            Cha.Agl -= 1;
                            Cha.Vit += 1;
                            trickish -= 0.5;
                        }

                        else if (attr == characterName[1, 2])
                        {
                            //裕福
                            Cha.Min -= 1;
                            aggressive -= 0.5;
                            trickish -= 0.5;
                        }
                        else if (attr == characterName[1, 3])
                        {
                            //貧乏
                            Cha.Min += 1;
                            aggressive += 0.5;
                            brave -= 0.5;
                        }

                        else if (attr == characterName[2, 0])
                        {
                            //厚き信仰
                            Cha.Int -= 1;
                            Cha.Min += 1;
                            brave += 0.5;
                            trickish -= 0.5;
                        }
                        else if (attr == characterName[2, 1])
                        {
                            //不心得者
                            cautious += 0.5;
                            trickish += 0.5;
                        }

                        else if (attr == characterName[2, 2])
                        {
                            //誠実
                            brave += 0.5;
                            trickish -= 0.5;
                        }
                        else if (attr == characterName[2, 3])
                        {
                            //不実
                            brave -= 0.5;
                            trickish += 0.5;
                        }

                        else if (attr == characterName[3, 0])
                        {
                            //冷静沈着
                            Cha.Agl -= 1;
                            Cha.Int += 1;
                            cautious += 0.5;
                            trickish += 0.5;
                        }
                        else if (attr == characterName[3, 1])
                        {
                            //猪突猛進
                            Cha.Agl += 1;
                            Cha.Min -= 1;
                            cautious -= 0.5;
                        }

                        else if (attr == characterName[3, 2])
                        {
                            //貪欲
                            Cha.Vit += 1;
                            Cha.Min -= 1;
                            aggressive += 0.5;
                            brave -= 0.5;
                            cautious -= 0.5;
                        }
                        else if (attr == characterName[3, 3])
                        {
                            //無欲
                            aggressive -= 0.5;
                        }

                        else if (attr == characterName[4, 0])
                        {
                            //献身的
                            Cha.Vit -= 1;
                            Cha.Min += 1;
                            aggressive -= 0.5;
                        }
                        else if (attr == characterName[4, 1])
                        {
                            //利己的
                            Cha.Dex -= 1;
                            Cha.Agl += 1;
                            aggressive += 0.5;
                            cheerful -= 0.5;
                            trickish += 0.5;
                        }

                        else if (attr == characterName[4, 2])
                        {
                            //秩序派
                            aggressive += 0.5;
                            trickish -= 0.5;
                        }
                        else if (attr == characterName[4, 3])
                        {
                            //混沌派
                            Cha.Str += 1;
                            Cha.Min -= 1;
                            aggressive += 0.5;
                            trickish += 0.5;
                        }

                        else if (attr == characterName[5, 0])
                        {
                            //進取派
                            Cha.Agl += 1;
                            Cha.Vit -= 1;
                            brave += 0.5;
                            cautious -= 0.5;
                        }
                        else if (attr == characterName[5, 1])
                        {
                            //保守派
                            Cha.Str -= 1;
                            Cha.Min += 1;
                            aggressive -= 0.5;
                            cautious += 0.5;
                        }

                        else if (attr == characterName[5, 2])
                        {
                            //神経質
                            Cha.Agl += 1;
                            Cha.Str -= 1;
                            cheerful -= 0.5;
                            cautious += 0.5;
                        }
                        else if (attr == characterName[5, 3])
                        {
                            //鈍感
                            Cha.Int -= 1;
                            Cha.Vit += 1;
                        }

                        else if (attr == characterName[6, 0])
                        {
                            //好奇心旺盛
                            Cha.Dex += 1;
                            Cha.Vit -= 1;
                        }
                        else if (attr == characterName[6, 1])
                        {
                            //無頓着
                            Cha.Agl -= 1;
                            Cha.Min += 1;
                            cheerful -= 0.5;
                        }

                        else if (attr == characterName[6, 2])
                        {
                            //過激
                            Cha.Str += 1;
                            Cha.Vit -= 1;
                            aggressive += 0.5;
                            cautious -= 0.5;
                        }
                        else if (attr == characterName[6, 3])
                        {
                            //穏健
                            aggressive -= 0.5;
                            cautious += 0.5;
                        }

                        else if (attr == characterName[7, 0])
                        {
                            //楽観的
                            Cha.Dex += 1;
                            Cha.Agl -= 1;
                            brave += 0.5;
                            cautious -= 0.5;
                        }
                        else if (attr == characterName[7, 1])
                        {
                            //悲観的
                            Cha.Int += 1;
                            Cha.Min -= 1;
                            brave -= 0.5;
                            cautious += 0.5;
                        }

                        else if (attr == characterName[7, 2])
                        {
                            //勤勉
                            Cha.Dex -= 1;
                            Cha.Vit += 1;
                        }
                        else if (attr == characterName[7, 3])
                        {
                            //遊び人
                            Cha.Dex += 1;
                            Cha.Int -= 1;
                            cheerful += 0.5;
                            trickish += 0.5;
                        }

                        else if (attr == characterName[8, 0])
                        {
                            //陽気
                            cheerful += 0.5;
                        }
                        else if (attr == characterName[8, 1])
                        {
                            //内気
                            brave -= 0.5;
                        }

                        else if (attr == characterName[8, 2])
                        {
                            //派手
                            Cha.Agl += 1;
                            Cha.Int -= 1;
                            cheerful += 0.5;
                            cautious -= 0.5;
                        }
                        else if (attr == characterName[8, 3])
                        {
                            //地味
                            Cha.Str -= 1;
                            Cha.Vit += 1;
                            brave -= 0.5;
                        }

                        else if (attr == characterName[9, 0])
                        {
                            //高慢
                            Cha.Dex -= 1;
                            Cha.Min += 1;
                            aggressive += 0.5;
                            cheerful -= 0.5;
                        }
                        else if (attr == characterName[9, 1])
                        {
                            //謙虚
                            Cha.Dex -= 1;
                            Cha.Int += 1;
                            cautious += 0.5;
                        }

                        else if (attr == characterName[9, 2])
                        {
                            //上品
                            Cha.Int += 1;
                            Cha.Str -= 1;
                            aggressive -= 0.5;
                            cheerful += 0.5;
                        }
                        else if (attr == characterName[9, 3])
                        {
                            //粗野
                            Cha.Int -= 1;
                            Cha.Str += 1;
                            aggressive += 0.5;
                            cheerful -= 0.5;
                        }

                        else if (attr == characterName[10, 0])
                        {
                            //武骨
                            Cha.Dex -= 1;
                            Cha.Str += 1;
                            cheerful -= 0.5;
                            brave += 0.5;
                        }
                        else if (attr == characterName[10, 1])
                        {
                            //繊細
                            Cha.Dex += 1;
                            Cha.Str -= 1;
                            brave -= 0.5;
                            cautious += 0.5;
                        }

                        else if (attr == characterName[10, 2])
                        {
                            //硬派
                            Cha.Agl -= 1;
                            Cha.Str += 1;
                            brave += 0.5;
                            trickish -= 0.5;
                        }
                        else if (attr == characterName[10, 3])
                        {
                            //軟派
                            Cha.Dex += 1;
                            Cha.Min -= 1;
                            cheerful += 0.5;
                            brave -= 0.5;
                        }

                        else if (attr == characterName[11, 0])
                        {
                            //お人好し
                            cheerful += 0.5;
                            trickish -= 0.5;
                        }
                        else if (attr == characterName[11, 1])
                        {
                            //ひねくれ者
                            cheerful -= 0.5;
                        }

                        else if (attr == characterName[11, 2])
                        {
                            //名誉こそ命
                            brave += 0.5;
                            cautious -= 0.5;
                            trickish -= 0.5;
                        }
                        else if (attr == characterName[11, 3])
                        {
                            //愛に生きる
                            aggressive -= 0.5;
                        }
                    }
                }
            } // End For
            #endregion
        }

        /// <summary>
        /// 解説設定
        /// </summary>
        private void SetDescription()
        {
            for (int i = 0; i < characterList.Count; i++)
            {
                if (i % 3 == 2)
                {
                    Cha.Expression += characterList[i] + "\n";
                }
                else
                {
                    Cha.Expression += characterList[i].PadRight(7, '　');
                }
            }

        }

        /// <summary>
        /// 特別クーポン設定
        /// </summary>
        private void SetSpecialCoupon()
        {
            Cha.SetCoupon(CWData.STR_LEVEL_BASE, (int)Cha.Level);
            Cha.SetCoupon(CWData.STR_EP, 0);
            Cha.SetCoupon(newGene.GetGeneString(), 0);
        }

        /// <summary>
        /// パラメータラップ
        /// </summary>
        private void ParamWrap()
        {
            if (IsNormalType(talent))
            {
                Cha.Dex = Util1.NumWrap(Cha.Dex, 1, PHYS_PARAM_MAX);
                Cha.Agl = Util1.NumWrap(Cha.Agl, 1, PHYS_PARAM_MAX);
                Cha.Str = Util1.NumWrap(Cha.Str, 1, PHYS_PARAM_MAX);
                Cha.Int = Util1.NumWrap(Cha.Int, 1, PHYS_PARAM_MAX);
                Cha.Vit = Util1.NumWrap(Cha.Vit, 1, PHYS_PARAM_MAX);
                Cha.Min = Util1.NumWrap(Cha.Min, 1, PHYS_PARAM_MAX);
            }
            else
            {
                Cha.Dex = Util1.NumWrap(Cha.Dex, 1, PHYS_PARAM_SPECIAL_MAX);
                Cha.Agl = Util1.NumWrap(Cha.Agl, 1, PHYS_PARAM_SPECIAL_MAX);
                Cha.Str = Util1.NumWrap(Cha.Str, 1, PHYS_PARAM_SPECIAL_MAX);
                Cha.Int = Util1.NumWrap(Cha.Int, 1, PHYS_PARAM_SPECIAL_MAX);
                Cha.Vit = Util1.NumWrap(Cha.Vit, 1, PHYS_PARAM_SPECIAL_MAX);
                Cha.Min = Util1.NumWrap(Cha.Min, 1, PHYS_PARAM_SPECIAL_MAX);
            }


            Cha.Aggressive = Util1.NumWrap(aggressive, MIND_PARAM_MIN, MIND_PARAM_MAX);
            Cha.Brave = Util1.NumWrap(brave, MIND_PARAM_MIN, MIND_PARAM_MAX);
            Cha.Cheerful = Util1.NumWrap(cheerful, MIND_PARAM_MIN, MIND_PARAM_MAX);
            Cha.Cautious = Util1.NumWrap(cautious, MIND_PARAM_MIN, MIND_PARAM_MAX);
            Cha.Trickish = Util1.NumWrap(trickish, MIND_PARAM_MIN, MIND_PARAM_MAX);

        }

        /// <summary>
        /// 一般的な素質型かどうか
        /// </summary>
        /// <param name="talent">素質型・特殊型</param>
        /// <returns></returns>
        private bool IsNormalType(string talent)
        {
            if (talent == CWData.TYPE_NORMAL
            || talent == CWData.TYPE_ALLMIGHTY
            || talent == CWData.TYPE_BRAVE
            || talent == CWData.TYPE_DARING
            || talent == CWData.TYPE_RESOURCEFUL
            || talent == CWData.TYPE_TACTIC)
            {
                return true;
            }
            else
            {
                return false;
            }

        }

        /// <summary>
        /// HP設定
        /// </summary>
        private void SetLife()
        {
            Cha.MaxHitPoint = (Cha.Vit / 2 + 4) * ((int)Cha.Level + 1) + Cha.Min / 2;
            Cha.HitPoint = Cha.MaxHitPoint;
        }

        /// <summary>
        /// データバージョン設定
        /// </summary>
        private void SetVersion()
        {
            Cha.Version = CWData.DataVersion5;
        }

        /// <summary>
        /// 箱PCを作ります。CharacterDlg生成直後に使用してください。
        /// </summary>
        /// <returns></returns>
        public CharacterData MakeBox()
        {
            Cha = new CharacterData();

            // 名前
            Cha.CardName = "箱";
            Cha.SetCoupon("＿" + "箱", 0);

            // 画像
            Cha.Image = new ImageData(CWIEngine.Properties.Resources.hako);

            // Male-Child
            radChild.Checked = true;
            // 親は設定そのままで一般
            // 性別
            SetSex();
            // 年代
            SetAge();

            // レベル修正
            Cha.Level = 99;
            
            // 親
            SetParents();

            // 標準型で設定しておく
            radNormal.Checked = true;

            // 素質
            SetTalent();
            // レベル設定変更
            Cha.SetCoupon(CWData.STR_LEVEL_MAX, 99);

            // 属性
            // グリッドの設定がされないのでなし。

            // 解説はこれで固定
            Cha.Expression = "TEXT\r\n　カードを置くための箱です。\n";

            // 特別クーポン
            SetSpecialCoupon();

            // パラメータ
            // Physical
            // 戦闘使用出来ないように1固定
            Cha.Dex = INIT_PHTS_BOX_PARAM;
            Cha.Agl = INIT_PHTS_BOX_PARAM;
            Cha.Int = INIT_PHTS_BOX_PARAM;
            Cha.Str = INIT_PHTS_BOX_PARAM;
            Cha.Vit = INIT_PHTS_BOX_PARAM;
            Cha.Min = INIT_PHTS_BOX_PARAM;

            aggressive = 0.0;
            brave = 0.0;
            cautious = 0.0;
            cheerful = 0.0;
            trickish = 0.0;

            // パラメタ範囲制限
            ParamWrap();
            // HP
            SetLife();
            // データバージョン
            SetVersion();
            
            Cha.FileName = string.Empty;

            return Cha;
        }

        #endregion
    }
}