﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.IO;

using CWIDatas;
using CWI.Card;
using CWI.Button;
using CWI;

namespace CWI.Dialog
{
    public partial class EditDialog : CWI.Dialog.SelectableForm
    {
        #region フィールド
        private bool isSelectable = false;

        #endregion

        #region プロパティ

        private CharacterData data;
        private int bmpIndex = 0;
        private List<string> FeatureBmpList;
        #endregion

        public EditDialog()
        {
            // 通常使用しない
            InitializeComponent();
        }

        public EditDialog(CharacterData cd, bool isSelectable)
        {
            InnToCharas(cd, isSelectable);
        }

        private void InnToCharas(CharacterData cd, bool isSelectable)
        {

            InitializeComponent();

            this.data = cd;
            this.isSelectable = isSelectable;

            //ボタン設定
            BtnSelect.Visible = true;
            BtnSelect.Text = "決定";
            tablist.Add(BtnSelect);


            BtnCancel.Visible = true;
            BtnCancel.Text = "中止";
            tablist.Add(BtnCancel);

            SetUpPosition();//位置合わせ

            SetUpData(); 
            
        }

        protected override void SetUpData()
        {
            string folder = FilePath.GetDataFaceFolder(data.GetSex(), data.GetAge());
            FeatureBmpList = new List<string>();
            FeatureBmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskBitmap));
            FeatureBmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskPng));
            FeatureBmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskGif));

            bmpIndex = FeatureBmpList.Count;


            txtName.Text = data.CardName;

            string exp = data.Expression;

            if (data.Expression.StartsWith("TEXT\r\n"))
            {
                exp = data.Expression.Substring(6, data.Expression.Length - 6);
            }

            txtExp.Text = exp;

            count = 0;
            base.SetUpData();
        }

        protected override void doSelectPage()
        {
            this.data.CardName = txtName.Text;
            this.data.Expression = "TEXT\r\n" + txtExp.Text;
            if (bmpIndex != FeatureBmpList.Count)
            {
                this.data.Image = new ImageData(new Bitmap(FeatureBmpList[bmpIndex]));
            }

            this.DialogResult = DialogResult.OK;
            this.Close();
        }
        
        protected override void ShowPage()
        {
            using (Graphics g = Graphics.FromImage(ViewBitmap))
            {
                using (Bitmap book = (Bitmap)Image.FromFile(FilePath.GetBillFile()))
                {
                    g.DrawImage(book, ViewPanel.ClientRectangle);
                }

                using (Font smallFont = new Font("ＭＳ 明朝", 9, FontStyle.Bold))
                {
                    using (StringFormat format = new StringFormat())
                    {
                        format.Alignment = StringAlignment.Center;//中揃え
                        Rectangle rectCount 
                            = new Rectangle(10, 20, this.ClientSize.Width, smallFont.Height);

                        g.DrawString("Resident Resistration", smallFont, SolidBlack, rectCount, format);
                        rectCount
                            = new Rectangle(10, 40, this.ClientSize.Width, smallFont.Height);
                        g.DrawString("Name", smallFont, SolidBlack, rectCount, format);
                        rectCount
                           = new Rectangle(10, 80, this.ClientSize.Width, smallFont.Height);
                        g.DrawString("Image", smallFont, SolidBlack, rectCount, format);
                        rectCount
                           = new Rectangle(10, 200, this.ClientSize.Width, smallFont.Height);
                        g.DrawString("Comment", smallFont, SolidBlack, rectCount, format);

                        //g.DrawString((index + 1) + " / " + count, smallFont, SolidBlack, rectCount, format);//数の描画
                    }

                }

                //画像があったら。ない場合ってあるのかな？
                if (data.Image != null && data.Image.bitmap != null)
                {
                    if (bmpIndex == FeatureBmpList.Count)
                    {
                        picCharacter.Image = data.Image.bitmap;
                    }
                    else
                    {
                        Bitmap bitmap = new Bitmap(FeatureBmpList[bmpIndex]);
                        bitmap.MakeTransparent(bitmap.GetPixel(0, 0));
                        picCharacter.Image = bitmap;
                    }
                }
                
            }
            ViewPanel.Invalidate();

        }

        private void EditDialog_Shown(object sender, EventArgs e)
        {

            // マウス・カーソルの位置をクライアント中央に移動させる
            Cursor.Position = new Point(this.Location.X + this.Width / 2, this.Location.Y + this.Height); 

            ShowPage();
        }

        private void btnCharacterLeft_Click(object sender, EventArgs e)
        {

            bmpIndex--;
            if (bmpIndex < 0)
            {
                bmpIndex = FeatureBmpList.Count;
            }
            ShowPage();
        }

        private void btnCharacterRight_Click(object sender, EventArgs e)
        {

            bmpIndex++;
            if (bmpIndex > FeatureBmpList.Count)
            {
                bmpIndex = 0;
            }
            ShowPage();
        }

        private void picCharacter_Click(object sender, EventArgs e)
        {
            // ここではモーダルで使用します。
            using (BmpSelectDlg func = new BmpSelectDlg(FeatureBmpList,data))
            {
                if (func.ShowDialog() == DialogResult.OK)
                {
                    bmpIndex = func.SelectedIndex;
                    ShowPage();
                }
            }
        }

    }
}
