﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class ExtHandWindow : Form
    {
        #region メンバ

        // カード置場・荷物袋の一列の枚数
        private int disp_num_x = 7;

        // キャラクタの各カードの一列の枚数
        private int disp_cnum_x = 2;
        private const int CNUM_DEFAULT = 2;

        private int old_num_x = 7;
        private int old_cnum_x = 2;

        private FormWindowState oldState = FormWindowState.Normal;

        private int grpWidth;
        private int grpHeight;
        // 宿
        private InnData inn = null;

        private Form owner;
        private PartyMaker _pm;

        // 現在のインデックス
        private int index = 0;

        // マウスの押されたインデックス
        private int mouseDownIndex = -1;
        // マウスの押されたパネル
        private Panel mouseDownPanel = null;
        // マウスの押された座標
        private Point mouseDownPoint = Point.Empty;
        // キャラクタの数
        private int characterCount = 0;

        private List<int> maxSkillNum = new List<int>();
        private List<int> maxItemNum = new List<int>();
        private List<int> maxBeastNum = new List<int>();

        private static volatile ProgressDlg progress;

        // 文字エンコーディング
        Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");

        //　リムーブされても回収されないようにつなぎ止めておくリスト
        List<TabPage> list = new List<TabPage>();


        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ExtHandWindow()
        {
            // 通常使用しない
            InitializeComponent();

            this.MouseWheel += new MouseEventHandler(Inn_MouseWheel);

            progress = new ProgressDlg(this);
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ExtHandWindow(Form owner, InnData inn)
        {
            InitializeComponent();

            this.MouseWheel += new MouseEventHandler(Inn_MouseWheel);

            progress = new ProgressDlg(this);

            this.inn = inn;

            this.owner = owner;
            this._pm = owner as PartyMaker;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void ExtHandWindow_Load(object sender, EventArgs e)
        {

            // マウス・カーソルの位置をスクリーン中央に移動させる
            int centerX = Screen.PrimaryScreen.Bounds.Right / 2;
            int centerY = Screen.PrimaryScreen.Bounds.Bottom / 2;
            Point cursolPosition = new Point(centerX, centerY);
            this.Location = new Point(centerX - this.Width / 2, centerY - this.Height / 2);

            // ソート種類.アイテム_スキル_召喚獣
            cmbOrder.SelectedIndex = 0;

            // アイテム最大化表示
            this.WindowState = FormWindowState.Maximized;
            

            tabYado.SelectedIndex = 0;

            foreach (TabPage page in tabYado.TabPages)
            {
                
                list.Add(page);
            }

            tabYado.ItemSize = new Size(CWData.TAB_HEADER_WIDTH, CWData.TAB_HEADER_HEIGHT);

            // 一度走らせる
            ExtHandWindow_Resize(sender, e);
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExtHandWindow_Shown(object sender, EventArgs e)
        {
            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "宿カードを読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            if (inn.CurrentParty == null || inn.CurrentParty.InScenario)
            {
                // カレントパーティーなし　もしくは
                // シナリオ進行中
                this.btnToPartyBag.Enabled = false;
                this.btnToYadoCard.Enabled = false;

            }
            else
            {
                // 宿待機状態
                this.btnToPartyBag.Enabled = true;
                this.btnToYadoCard.Enabled = true;
            }

            // 全宿カード表示
            SetUpYadoCard();

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();
        }

        private void ExtHandWindow_ResizeBegin(object sender, EventArgs e)
        {
            old_num_x = this.disp_num_x;
            old_cnum_x = this.disp_cnum_x;
        }

        private void ExtHandWindow_ResizeEnd(object sender, EventArgs e)
        {
            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {

                ExtHandWindow_Resize(sender, e);

                TabPage select = tabYado.SelectedTab;
                if (select == this.tabYadoCard)
                {
                    this.SetLocationPnl(this.pnlYadoCard);
                }
                else if (select == this.tabPartyBag)
                {
                    this.SetLocationPnl(this.pnlPartyBag);
                    
                }
                else if (select == this.tabCharacter1)
                {
                    this.grpSkill1.Bounds =
                        new Rectangle(6, 6, grpWidth, grpHeight);
                    this.grpItem1.Bounds =
                        new Rectangle(12 + grpWidth, 6, grpWidth, grpHeight);
                    this.grpBeast1.Bounds =
                        new Rectangle(18 + grpWidth * 2, 6, grpWidth, grpHeight);

                    this.pnlSkill1.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlItem1.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlBeast1.Size = new Size(grpWidth - 6, grpHeight - 18);

                    this.SetLocationPnl(this.pnlSkill1);
                    this.SetLocationPnl(this.pnlItem1);
                    this.SetLocationPnl(this.pnlBeast1);

                }
                else if (select == this.tabCharacter2)
                {
                    this.grpSkill2.Bounds =
                        new Rectangle(6, 6, grpWidth, grpHeight);
                    this.grpItem2.Bounds =
                        new Rectangle(12 + grpWidth, 6, grpWidth, grpHeight);
                    this.grpBeast2.Bounds =
                        new Rectangle(18 + grpWidth * 2, 6, grpWidth, grpHeight);

                    this.pnlSkill2.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlItem2.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlBeast2.Size = new Size(grpWidth - 6, grpHeight - 18);

                    this.SetLocationPnl(this.pnlSkill2);
                    this.SetLocationPnl(this.pnlItem2);
                    this.SetLocationPnl(this.pnlBeast2);

                }
                else if (select == this.tabCharacter3)
                {
                    this.grpSkill3.Bounds =
                        new Rectangle(6, 6, grpWidth, grpHeight);
                    this.grpItem3.Bounds =
                        new Rectangle(12 + grpWidth, 6, grpWidth, grpHeight);
                    this.grpBeast3.Bounds =
                        new Rectangle(18 + grpWidth * 2, 6, grpWidth, grpHeight);

                    this.pnlSkill3.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlItem3.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlBeast3.Size = new Size(grpWidth - 6, grpHeight - 18);

                    this.SetLocationPnl(this.pnlSkill3);
                    this.SetLocationPnl(this.pnlItem3);
                    this.SetLocationPnl(this.pnlBeast3);

                }
                else if (select == this.tabCharacter4)
                {
                    this.grpSkill4.Bounds =
                        new Rectangle(6, 6, grpWidth, grpHeight);
                    this.grpItem4.Bounds =
                        new Rectangle(12 + grpWidth, 6, grpWidth, grpHeight);
                    this.grpBeast4.Bounds =
                        new Rectangle(18 + grpWidth * 2, 6, grpWidth, grpHeight);

                    this.pnlSkill4.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlItem4.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlBeast4.Size = new Size(grpWidth - 6, grpHeight - 18);


                    this.SetLocationPnl(this.pnlSkill4);
                    this.SetLocationPnl(this.pnlItem4);
                    this.SetLocationPnl(this.pnlBeast4);

                }
                else if (select == this.tabCharacter5)
                {
                    this.grpSkill5.Bounds =
                        new Rectangle(6, 6, grpWidth, grpHeight);
                    this.grpItem5.Bounds =
                        new Rectangle(12 + grpWidth, 6, grpWidth, grpHeight);
                    this.grpBeast5.Bounds =
                        new Rectangle(18 + grpWidth * 2, 6, grpWidth, grpHeight);

                    this.pnlSkill5.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlItem5.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlBeast5.Size = new Size(grpWidth - 6, grpHeight - 18);


                    this.SetLocationPnl(this.pnlSkill5);
                    this.SetLocationPnl(this.pnlItem5);
                    this.SetLocationPnl(this.pnlBeast5);

                }
                else if (select == this.tabCharacter6)
                {
                    this.grpSkill6.Bounds =
                        new Rectangle(6, 6, grpWidth, grpHeight);
                    this.grpItem6.Bounds =
                        new Rectangle(12 + grpWidth, 6, grpWidth, grpHeight);
                    this.grpBeast6.Bounds =
                        new Rectangle(18 + grpWidth * 2, 6, grpWidth, grpHeight);

                    this.pnlSkill6.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlItem6.Size = new Size(grpWidth - 6, grpHeight - 18);
                    this.pnlBeast6.Size = new Size(grpWidth - 6, grpHeight - 18);

                    this.SetLocationPnl(this.pnlSkill6);
                    this.SetLocationPnl(this.pnlItem6);
                    this.SetLocationPnl(this.pnlBeast6);

                }

            }
        }

        private void ExtHandWindow_Resize(object sender, EventArgs e)
        {
            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {
                if (oldState != this.WindowState)
                {
                    oldState = this.WindowState;
                    ExtHandWindow_ResizeEnd(sender, e);
                }

                this.tabYado.Width = this.Width - 20;
                this.tabYado.Height = this.Height - 68;

                this.btnToPartyBag.Location
                = new Point(7, this.ClientSize.Height - 28);

                this.btnToYadoCard.Location
                = new Point(85, this.ClientSize.Height - 28);

                this.btnClose.Location = new Point(
                    (this.ClientSize.Width - this.btnClose.Width) / 2,
                    this.ClientSize.Height - 28);

                this.btnSort.Location = new Point(
                    this.ClientSize.Width - 85, this.ClientSize.Height - 28);

                this.cmbOrder.Location = new Point(
                    this.ClientSize.Width - 85 - 174, this.ClientSize.Height - 26);

                this.cmbSend.Location = new Point(
                    this.cmbSend.Location.X, this.ClientSize.Height - 26);


                Size pageSize = new Size(this.Width - 28,this.Height - 93);
                this.tabYadoCard.Size = pageSize;
                this.tabPartyBag.Size = pageSize;
                this.tabCharacter1.Size = pageSize;
                this.tabCharacter2.Size = pageSize;
                this.tabCharacter3.Size = pageSize;
                this.tabCharacter4.Size = pageSize;
                this.tabCharacter5.Size = pageSize;
                this.tabCharacter6.Size = pageSize;

                this.pnlYadoCard.Width = this.tabYadoCard.Width - 6;
                this.pnlYadoCard.Height = this.tabYadoCard.Height - 6;

                this.pnlPartyBag.Width = this.tabPartyBag.Width - 6;
                this.pnlPartyBag.Height = this.tabPartyBag.Height - 6;
     
                this.disp_num_x = (this.pnlYadoCard.Width - 10) / CWData.HAND_CARD_WIDTH;


                this.grpWidth = (this.tabYadoCard.ClientSize.Width - 24) / 3;
                this.grpHeight = this.tabYadoCard.ClientSize.Height - 12;

                this.disp_cnum_x = (this.grpWidth - 6 - 10) / CWData.HAND_CARD_WIDTH;

            }

        }

        private void tabYado_SelectedIndexChanged(object sender, EventArgs e)
        {
            
            ExtHandWindow_ResizeEnd(sender, e);
        }

        private void ExtHandWindow_KeyDown(object sender, KeyEventArgs e)
        {
            //KeyCode + Modifiers = KeyData
            if ((e.Modifiers & Keys.Control) == Keys.Control)
            {

                if (e.KeyCode == Keys.Left)
                {
                    if (tabYado.SelectedIndex != 0)
                    {
                        tabYado.SelectedIndex--;

                    }

                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Right)
                {
                    if (inn.CurrentParty != null)
                    {
                        if (tabYado.SelectedIndex < characterCount + 1)
                        {
                            tabYado.SelectedIndex++;
                        }
                    }

                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Up)
                {
                    if (tabYado.SelectedIndex == 0)
                    {
                        ScrollPnl(pnlYadoCard, -100);
                    }
                    else if (tabYado.SelectedIndex == 1)
                    {
                        ScrollPnl(pnlPartyBag, -100);
                    }

                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Down)
                {
                    if (tabYado.SelectedIndex == 0)
                    {
                        ScrollPnl(pnlYadoCard, 100);
                    }
                    else if (tabYado.SelectedIndex == 1)
                    {
                        ScrollPnl(pnlPartyBag, 100);
                    }

                    e.Handled = true;
                }
            }

            if (e.KeyCode == Keys.PageUp)
            {
                if (tabYado.SelectedIndex == 0)
                {
                    ScrollPnl(pnlYadoCard, -100);
                }
                else if (tabYado.SelectedIndex == 1)
                {
                    ScrollPnl(pnlPartyBag, -100);
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.PageDown)
            {
                if (tabYado.SelectedIndex == 0)
                {
                    ScrollPnl(pnlYadoCard, 100);
                }
                else if (tabYado.SelectedIndex == 1)
                {
                    ScrollPnl(pnlPartyBag, 100);
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Home)
            {
                if (tabYado.SelectedIndex == 0)
                {
                    ScrollPnl(pnlYadoCard, -CWData.HAND_CARD_HEIGHT * ((pnlYadoCard.Controls.Count - 1) / disp_num_x + 1));
                }
                else if (tabYado.SelectedIndex == 1)
                {
                    ScrollPnl(pnlPartyBag, -CWData.HAND_CARD_HEIGHT * ((pnlPartyBag.Controls.Count - 1) / disp_num_x + 1));
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.End)
            {
                if (tabYado.SelectedIndex == 0)
                {
                    ScrollPnl(pnlYadoCard, CWData.HAND_CARD_HEIGHT * ((pnlYadoCard.Controls.Count - 1) / disp_num_x + 1));
                }
                else if (tabYado.SelectedIndex == 1)
                {
                    ScrollPnl(pnlPartyBag, CWData.HAND_CARD_HEIGHT * ((pnlPartyBag.Controls.Count - 1) / disp_num_x + 1));
                }

                e.Handled = true;
            }
        }

        #endregion

        #region ボタンイベント

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// パーティー荷物袋の全カードをカード置場へボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllInn_Click(object sender, EventArgs e)
        {
            int count = inn.CurrentParty.BagCards.Count;

            if (count > 0)
            {
                // 進捗ダイアログ開始
                StartProgress();
                // 進捗ダイアログ初期処理待ち
                WaitProgress();
                progress.Message = "荷物袋の全カードをカード置場に移動中です。";
                progress.Maximum = 100;
                progress.Minimum = 0;
                progress.Value = 0;


                // 全荷物袋カードを置場カードへ
                inn.MoveAllCardFromPartyBag();


                // 表示反映
                DispCard dc;


                Control[] array = new Control[pnlPartyBag.Controls.Count];
                pnlPartyBag.Controls.CopyTo(array, 0); 
                
                pnlPartyBag.Controls.Clear();

                this.tabPartyBag.Text = "荷物(0)";

                // 表示用カード追加・配置修正
                pnlYadoCard.SuspendLayout();

                int nowX = -pnlYadoCard.AutoScrollPosition.X;
                int nowY = -pnlYadoCard.AutoScrollPosition.Y;

                pnlYadoCard.AutoScrollPosition = new Point(0, 0);

                progress.Value = 50;

                index = 0;

                IndexCardData icd;

                pnlYadoCard.Controls.AddRange(array);
                
                for (int i = inn.BankBoxCards.Count - count; i < inn.BankBoxCards.Count; i++)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = " 荷物袋：" + (index + 1) + "枚目を移動しています。";

                    icd = inn.FindIndexedCardData(inn.BankBoxCards[i]);

                    // DispCard を Newせずに移して、SetHcする。
                    // dc = new DispCard(icd.HandCard);
                    dc = (DispCard)(array[index]);
                    dc.SetHc(icd.HandCard);

                    dc.Location = new Point(CWData.HAND_CARD_WIDTH
                        * (i % disp_num_x), CWData.HAND_CARD_HEIGHT * (i / disp_num_x));

                    // Add しない
                    // pnlYadoCard.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    index++;
                    progress.Value = 50 + index * 50 / count;
                }

                pnlYadoCard.AutoScrollPosition = new Point(nowX, nowY);

                pnlYadoCard.ResumeLayout();

                tabYado.SelectedIndex = 0;
                tabYado.Update();

                this.tabYadoCard.Text = "置場(" + inn.BankBoxCards.Count + ")";

                // 進捗ダイアログ終了
                EndProgress();

            }
        }

        /// <summary>
        /// 全カード荷物袋へ移動ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllMove_Click(object sender, EventArgs e)
        {
            int count = inn.BankBoxCards.Count;
            if (count > 0)
            {
                // 進捗ダイアログ開始
                StartProgress();
                // 進捗ダイアログ初期処理待ち
                WaitProgress();
                progress.Message = "カード置場の全カードを荷物袋に移動中です。";
                progress.Maximum = 100;
                progress.Minimum = 0;
                progress.Value = 0;

                // 全カード置場カードをカレントパーティーの荷物袋に移動
                inn.MoveAllCardToPartyBag();

                // 表示反映
                DispCard dc;

                Control[] array = new Control[pnlYadoCard.Controls.Count];
                pnlYadoCard.Controls.CopyTo(array, 0);
                
                pnlYadoCard.Controls.Clear();
                
                this.tabYadoCard.Text = "置場(0)";


                // 表示用カード追加・配置修正

                pnlPartyBag.SuspendLayout();

                int nowX = -pnlPartyBag.AutoScrollPosition.X;
                int nowY = -pnlPartyBag.AutoScrollPosition.Y;

                pnlPartyBag.AutoScrollPosition = new Point(0, 0);

                progress.Value = 50;

                index = 0;

                IndexCardData icd;

                pnlPartyBag.Controls.AddRange(array);

                for (int i = inn.CurrentParty.BagCards.Count - count; i < inn.CurrentParty.BagCards.Count; i++)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = "カード置場：" + (index + 1) + "枚目を移動しています。";

                    icd = inn.FindIndexedCardData(inn.CurrentParty.BagCards[i]);


                    // DispCard を Newせずに移して、SetHcする。
                    // dc = new DispCard(icd.HandCard);
                    dc = (DispCard)(array[index]);
                    dc.SetHc(icd.HandCard);

                    dc.Location = new Point(CWData.HAND_CARD_WIDTH
                        * (i % disp_num_x), CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                    
                    // Add しない
                    // pnlPartyBag.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    index++;
                    progress.Value = 50 + index * 50 / count;
                }

                pnlPartyBag.AutoScrollPosition = new Point(nowX, nowY);

                pnlPartyBag.ResumeLayout();

                tabYado.SelectedIndex = 1;
                tabYado.Update();

                this.tabPartyBag.Text = "荷物(" + inn.CurrentParty.BagCards.Count + ")";

                // 進捗ダイアログ終了
                EndProgress();
            }
        }

        /// <summary>
        /// ソートボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSort_Click(object sender, EventArgs e)
        {
            try
            {
                if (cmbOrder.SelectedIndex < 0 && cmbOrder.SelectedIndex > 5)
                {
                    // ありえない
                    MessageDlg.ShowInfoDialog("ソート種類の選択に誤りがあります。");
                    return;
                }
                //else
                //{
                //    if (DialogResult.Yes != MessageDlg.ShowYesNoDialog(
                //        "ソートは時間がかかる場合があります。\r\n 実行してよろしいですか？"
                //        ))
                //    {
                //        return;
                //    }
                //}

                int nowX;
                int nowY;
                IndexCardData icd;
                DispCard dc;
                int count;

                switch (tabYado.SelectedIndex)
                {
                    case 0:
                        count = inn.BankBoxCards.Count;
                        if (count > 0)
                        {
                            // カード置場

                            // 表示反映
                            // 進捗ダイアログ開始
                            StartProgress();
                            // 進捗ダイアログ初期処理待ち
                            WaitProgress();
                            progress.Message = "カード置場をソート中です。";
                            progress.Maximum = 100;
                            progress.Minimum = 0;
                            progress.Value = 0;

                            inn.SortCards(ソート対象.カード置場, (ソート種類)(cmbOrder.SelectedIndex));

                            // 個数が同じなのでクリアせずにSetHcする。
                            GC.Collect(2);

                            progress.Value = 50;

                            // 表示用カード追加・配置修正
                            nowX = -pnlYadoCard.AutoScrollPosition.X;
                            nowY = -pnlYadoCard.AutoScrollPosition.Y;

                            pnlYadoCard.AutoScrollPosition = new Point(0, 0);

                            for (int i = 0; i < count; i++)
                            {
                                // プログレスバーのメッセージを変更する
                                progress.Message = "カード置場：" + (i + 1) + "枚目を表示しています。";

                                icd = inn.FindIndexedCardData(inn.BankBoxCards[i]);

                                // DispCard は　New　せずにSetHcする。
                                dc = (DispCard)(pnlYadoCard.Controls[i]);
                                dc.SetHc(icd.HandCard);

                                //プログレスバーの値を変更する
                                
                                progress.Value = 50 + (i + 1) * 50 / count;
                            }

                            pnlYadoCard.AutoScrollPosition = new Point(nowX, nowY);

                            // 進捗ダイアログ終了
                            EndProgress();
                        }

                        break;
                    case 1:
                        if (inn.CurrentParty != null)
                        {
                            // 荷物袋
                            count = inn.CurrentParty.BagCards.Count;
                            if (count > 0)
                            {
                                // 表示反映
                                // 進捗ダイアログ開始
                                StartProgress();
                                // 進捗ダイアログ初期処理待ち
                                WaitProgress();
                                progress.Message = "荷物袋をソート中です。";
                                progress.Maximum = 100;
                                progress.Minimum = 0;
                                progress.Value = 0;

                                inn.SortCards(ソート対象.荷物袋, (ソート種類)(cmbOrder.SelectedIndex));

                                // 個数が同じなのでクリアせずにSetHcする。
                                GC.Collect(2);

                                progress.Value = 50;

                                // 表示用カード追加・配置修正
                                nowX = -pnlPartyBag.AutoScrollPosition.X;
                                nowY = -pnlPartyBag.AutoScrollPosition.Y;

                                pnlPartyBag.AutoScrollPosition = new Point(0, 0);

                                for (int i = 0; i < count; i++)
                                {
                                    // プログレスバーのメッセージを変更する
                                    progress.Message = "荷物袋：" + (i + 1) + "枚目を表示しています。";

                                    icd = inn.FindIndexedCardData(inn.CurrentParty.BagCards[i]);

                                    // DispCard は　New　せずにSetHcする。
                                    dc = (DispCard)(pnlPartyBag.Controls[i]);
                                    dc.SetHc(icd.HandCard);

                                    //プログレスバーの値を変更する
                                    progress.Value = 50 + (i + 1) * 50 / count;
                                }

                                pnlPartyBag.AutoScrollPosition = new Point(nowX, nowY);

                                // 進捗ダイアログ終了
                                EndProgress();
                            }
                        }
                        break;
                    default:
                        MessageDlg.ShowInfoDialog("キャラクタの所持カードのソートは未対応です。"
                            );
                        break;
                }

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("システムエラー（ソート）\r\n" + ex.ToString());
            }
        }

        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {
            backgroundWorker1.RunWorkerAsync();
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();
            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion

        #region 表示と配置
        /// <summary>
        /// 宿Card設定
        /// </summary>
        private void SetUpYadoCard()
        {

            int num;
            int num2;


            progress.Message = " カードを読込中です。";

            //プログレスバーの値を変更する
            progress.Value = 0;

            // WidCardが読み込まれていない時のためにダミー行
            inn.ResetWidCard();
            int dummyCount = inn.WidCards.Count;

            //プログレスバーの値を変更する
            progress.Value = 10;

            // カード置場
            pnlYadoCard.Controls.Clear();
            pnlYadoCard.AutoScrollPosition = new Point(0, 0);
            int count = inn.BankBoxCards.Count;


            this.pnlYadoCard.SuspendLayout();

            index = 0;
            IndexCardData icd;
            DispCard dc;
            SkillHandCard shc;
            ItemHandCard ihc;
            BeastHandCard bhc;

            progress.Message = " カード置場を表示中です。";

            foreach (RenamedCardData rcd in inn.BankBoxCards)
            {
                // プログレスバーのメッセージを変更する
                progress.Message = inn.Name + " カード置場：" + (index + 1) + "枚目を表示中です。";

                icd = inn.FindIndexedCardData(rcd);

                dc = new DispCard(icd.HandCard);
                dc.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x)
                , CWData.HAND_CARD_HEIGHT * (index / disp_num_x));

                index++;
                pnlYadoCard.Controls.Add(dc);

                //プログレスバーの値を変更する
                progress.Value = 10 + index * 30 / count;
            }

            this.pnlYadoCard.ResumeLayout();

            this.tabYadoCard.Text = "置場(" + inn.BankBoxCards.Count + ")";


            cmbSend.Items.Add("カード置場");

            // 荷物袋
            if (inn.CurrentParty != null)
            {

                progress.Message = " 荷物袋を表示中です。";

                pnlPartyBag.Controls.Clear();
                pnlPartyBag.AutoScrollPosition = new Point(0, 0);

                this.pnlPartyBag.SuspendLayout();

                count = inn.CurrentParty.BagCards.Count;

                index = 0;
                foreach (RenamedCardData rcd in inn.CurrentParty.BagCards)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = inn.Name + " 荷物袋：" + (index + 1) + "枚目を表示中です。";

                    icd = inn.FindIndexedCardData(rcd);

                    dc = new DispCard(icd.HandCard);
                    dc.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x)
                    , CWData.HAND_CARD_HEIGHT * (index / disp_num_x));

                    index++;
                    pnlPartyBag.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    progress.Value = 40 + index * 30 / count;
                }

                this.pnlPartyBag.ResumeLayout();

                count = inn.CurrentParty.Characters.Count;
                this.tabPartyBag.Text = "荷物(" + inn.CurrentParty.BagCards.Count + ")";
                //num = sjisEnc.GetByteCount(this.tabPartyBag.Text);
                //num2 = this.tabPartyBag.Text.Length;
                //this.tabPartyBag.Text = this.tabPartyBag.Text.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                
                cmbSend.Items.Add("荷物袋");

            }
            else
            {
                count = 0;
            }



            //プログレスバーの値を変更する
            progress.Value = 70;

            // 各キャラクタ
            characterCount = count;

            string cardName;
            //int num = sjisEnc.GetByteCount(str);
            // tabPage 削除
            switch (count)
            {
                default:
                    // ありえない
                    PartyMaker.CWIErrorMessage("カレントパーティーのキャラクタの人数に誤りがあります。（拡張ハンドウインドウ）");
                    this.Close();
                    break;
                case 0:
                    this.tabYado.TabPages.Remove(this.tabPartyBag);
                    this.tabYado.TabPages.Remove(this.tabCharacter1);
                    this.tabYado.TabPages.Remove(this.tabCharacter2);
                    this.tabYado.TabPages.Remove(this.tabCharacter3);
                    this.tabYado.TabPages.Remove(this.tabCharacter4);
                    this.tabYado.TabPages.Remove(this.tabCharacter5);
                    this.tabYado.TabPages.Remove(this.tabCharacter6);
                    break;

                case 1:
                    cardName = inn.CurrentParty.Characters[0].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter1.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    this.tabYado.TabPages.Remove(this.tabCharacter2);
                    this.tabYado.TabPages.Remove(this.tabCharacter3);
                    this.tabYado.TabPages.Remove(this.tabCharacter4);
                    this.tabYado.TabPages.Remove(this.tabCharacter5);
                    this.tabYado.TabPages.Remove(this.tabCharacter6);
                    break;

                case 2:
                    cardName = inn.CurrentParty.Characters[0].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter1.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[1].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter2.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    this.tabYado.TabPages.Remove(this.tabCharacter3);
                    this.tabYado.TabPages.Remove(this.tabCharacter4);
                    this.tabYado.TabPages.Remove(this.tabCharacter5);
                    this.tabYado.TabPages.Remove(this.tabCharacter6);
                    break;

                case 3:
                    cardName = inn.CurrentParty.Characters[0].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter1.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[1].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter2.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[2].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter3.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    this.tabYado.TabPages.Remove(this.tabCharacter4);
                    this.tabYado.TabPages.Remove(this.tabCharacter5);
                    this.tabYado.TabPages.Remove(this.tabCharacter6);
                    break;


                case 4:
                    cardName = inn.CurrentParty.Characters[0].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter1.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[1].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter2.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[2].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter3.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[3].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter4.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    this.tabYado.TabPages.Remove(this.tabCharacter5);
                    this.tabYado.TabPages.Remove(this.tabCharacter6);
                    break;

                case 5:
                    cardName = inn.CurrentParty.Characters[0].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter1.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[1].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter2.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[2].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter3.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[3].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter4.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[4].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter5.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    this.tabYado.TabPages.Remove(this.tabCharacter6);
                    break;

                case 6:
                    cardName = inn.CurrentParty.Characters[0].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter1.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[1].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter2.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[2].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter3.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[3].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter4.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[4].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter5.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    cardName = inn.CurrentParty.Characters[5].CardName;
                    num = sjisEnc.GetByteCount(cardName);
                    num2 = cardName.Length;
                    this.tabCharacter6.Text = cardName.PadLeft((12 - num) / 2 + num2, ' ').PadRight((12 - num) + num2, ' ');
                    break;
            }


            int charCount = 0;
            for (int i = 0; i < count; i++)
            {
                charCount += inn.CurrentParty.Characters[i].Skills.Count;
                charCount += inn.CurrentParty.Characters[i].Items.Count;
                charCount += inn.CurrentParty.Characters[i].Beasts.Count;

                cmbSend.Items.Add(inn.CurrentParty.Characters[i].CardName);
            }


            Panel skillPanel = new Panel();
            Panel itemPanel = new Panel();
            Panel beastPanel = new Panel();
            GroupBox skillGroup = new GroupBox();
            GroupBox itemGroup = new GroupBox();
            GroupBox beastGroup = new GroupBox();

            index = 0;
            for (int i = 0; i < count; i++)
            {
                switch (i)
                {
                    case 0:
                        skillPanel = this.pnlSkill1;
                        itemPanel = this.pnlItem1;
                        beastPanel = this.pnlBeast1;

                        skillGroup = this.grpSkill1;
                        itemGroup = this.grpItem1;
                        beastGroup = this.grpBeast1;

                        break;

                    case 1:
                        skillPanel = this.pnlSkill2;
                        itemPanel = this.pnlItem2;
                        beastPanel = this.pnlBeast2;

                        skillGroup = this.grpSkill2;
                        itemGroup = this.grpItem2;
                        beastGroup = this.grpBeast2;
                        break;

                    case 2:
                        skillPanel = this.pnlSkill3;
                        itemPanel = this.pnlItem3;
                        beastPanel = this.pnlBeast3;

                        skillGroup = this.grpSkill3;
                        itemGroup = this.grpItem3;
                        beastGroup = this.grpBeast3;
                        break;

                    case 3:
                        skillPanel = this.pnlSkill4;
                        itemPanel = this.pnlItem4;
                        beastPanel = this.pnlBeast4;

                        skillGroup = this.grpSkill4;
                        itemGroup = this.grpItem4;
                        beastGroup = this.grpBeast4;
                        break;

                    case 4:
                        skillPanel = this.pnlSkill5;
                        itemPanel = this.pnlItem5;
                        beastPanel = this.pnlBeast5;

                        skillGroup = this.grpSkill5;
                        itemGroup = this.grpItem5;
                        beastGroup = this.grpBeast5;
                        break;

                    case 5:
                        skillPanel = this.pnlSkill6;
                        itemPanel = this.pnlItem6;
                        beastPanel = this.pnlBeast6;

                        skillGroup = this.grpSkill6;
                        itemGroup = this.grpItem6;
                        beastGroup = this.grpBeast6;
                        break;
                }

                skillPanel.Controls.Clear();
                skillPanel.AutoScrollPosition = new Point(0, 0);
                itemPanel.Controls.Clear();
                itemPanel.AutoScrollPosition = new Point(0, 0);
                beastPanel.Controls.Clear();
                beastPanel.AutoScrollPosition = new Point(0, 0);

                progress.Message = " キャラクタ" + i.ToString() + "　カードを表示中です。";

                skillPanel.SuspendLayout();
                itemPanel.SuspendLayout();
                beastPanel.SuspendLayout();

                CharacterData charaData = inn.CurrentParty.Characters[i];
                //Skills
                int skillIndex = 0;
                foreach (WidCard wc in charaData.Skills)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = inn.Name + " キャラクタカード：" + (index + 1) + "枚目を表示中です。";


                    shc = new SkillHandCard(wc);

                    shc.UpdateImage(GameParam.GetCardFacterColor(charaData, shc.widData.BodyFacter, shc.widData.MindFacter),
                             GameParam.GetSkillRemainColor(shc.widData.SkillRemainUseCount,
                                       GameParam.GetMaxSkillUseCount(charaData.Level, shc.widData.Level)));

                    dc = new DispCard(shc);
                    dc.Location = new Point(CWData.HAND_CARD_WIDTH * (skillIndex % disp_cnum_x)
                    , CWData.HAND_CARD_HEIGHT * (skillIndex / disp_cnum_x));

                    skillIndex++;
                    index++;

                    skillPanel.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    progress.Value = 70 + index * 30 / charCount;
                }

                skillGroup.Text = "技能カード " + skillIndex + "/" + charaData.GetMaxHandCard(手札種類.所持スキル);
                

                //Items
                int itemIndex = 0;
                foreach (WidCard wc in charaData.Items)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = inn.Name + " キャラクタカード：" + (index + 1) + "枚目を表示中です。";


                    ihc = new ItemHandCard(wc);

                    ihc.UpdateImage(GameParam.GetCardFacterColor(charaData, ihc.widData.BodyFacter, ihc.widData.MindFacter));


                    dc = new DispCard(ihc);
                    dc.Location = new Point(CWData.HAND_CARD_WIDTH * (itemIndex % disp_cnum_x)
                    , CWData.HAND_CARD_HEIGHT * (itemIndex / disp_cnum_x));

                    itemIndex++;
                    index++;

                    itemPanel.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    progress.Value = 70 + index * 30 / charCount;
                }

                itemGroup.Text = "アイテムカード " + itemIndex + "/" + charaData.GetMaxHandCard(手札種類.所持アイテム);


                //Beasts
                int beastIndex = 0;
                foreach (WidCard wc in charaData.Beasts)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = inn.Name + " キャラクタカード：" + (index + 1) + "枚目を表示中です。";


                    bhc = new BeastHandCard(wc);
                    bhc.UpdateImage(GameParam.GetCardFacterColor(charaData, bhc.widData.BodyFacter, bhc.widData.MindFacter));

                    dc = new DispCard(bhc);
                    dc.Location = new Point(CWData.HAND_CARD_WIDTH * (beastIndex % disp_cnum_x)
                    , CWData.HAND_CARD_HEIGHT * (beastIndex / disp_cnum_x));

                    beastIndex++;
                    index++;

                    beastPanel.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    progress.Value = 70 + index * 30 / charCount;
                }

                beastGroup.Text = "召喚獣カード " + beastIndex + "/" + charaData.GetMaxHandCard(手札種類.所有召喚獣);

                skillPanel.ResumeLayout();
                itemPanel.ResumeLayout();
                beastPanel.ResumeLayout();

            }


            cmbSend.Items.Add("ゴミ箱");

            cmbSend.SelectedIndex = 0;

            //プログレスバーの値を変更する
            progress.Value = 100;

        }

        /// <summary>
        /// パネル汎用 パネル内コントロール再配置
        /// </summary>
        /// <param name="pnl"></param>
        private void SetLocationPnl(Panel pnl)
        {

            if (pnl != null)
            {

                pnl.SuspendLayout();

                int nowX = -pnl.AutoScrollPosition.X;
                int nowY = -pnl.AutoScrollPosition.Y;

                pnl.AutoScrollPosition = new Point(0, 0);
                index = 0;


                if (pnl == pnlYadoCard || pnl == pnlPartyBag)
                {
                    foreach (Control ctrl in pnl.Controls)
                    {
                        ctrl.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x),
                            CWData.HAND_CARD_HEIGHT * (index / disp_num_x));
                        index++;
                    }
                }
                else
                {
                    foreach (Control ctrl in pnl.Controls)
                    {
                        ctrl.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_cnum_x),
                            CWData.HAND_CARD_HEIGHT * (index / disp_cnum_x));
                        index++;
                    }
                }

                pnl.AutoScrollPosition = new Point(nowX, nowY);


                pnl.ResumeLayout();
            }
        }
 
        /// <summary>
        /// 移動量分だけスクロール
        /// </summary>
        /// <param name="pnl"></param>
        /// <param name="delta"></param>
        private void ScrollPnl(Panel pnl,int delta)
        {

            if (pnl != null)
            {
                int nowX = -pnl.AutoScrollPosition.X;
                int nowY = -pnl.AutoScrollPosition.Y;
                nowY += delta;
                if (nowY < 0)
                {
                    nowY = 0;
                }

                if (pnl == pnlYadoCard || pnl == pnlPartyBag)
                {
                    if (nowY > CWData.HAND_CARD_HEIGHT * ((pnl.Controls.Count - 1) / disp_num_x + 1))
                    {
                        nowY = CWData.HAND_CARD_HEIGHT * ((pnl.Controls.Count - 1) / disp_num_x + 1);
                    }
                }
                else
                {
                    if (nowY > CWData.HAND_CARD_HEIGHT * ((pnl.Controls.Count - 1) / disp_cnum_x + 1))
                    {
                        nowY = CWData.HAND_CARD_HEIGHT * ((pnl.Controls.Count - 1) / disp_cnum_x + 1);
                    }


                }

                pnl.AutoScrollPosition = new Point(nowX, nowY);
            }
        }

        /// <summary>
        /// カードの単独移動を表示に反映する
        /// </summary>
        /// <param name="inn"></param>
        /// <param name="oldPanel"></param>
        /// <param name="newPanel"></param>
        /// <param name="oldIndex"></param>
        private void DisplayCardMove(InnData inn, Panel oldPanel,Panel newPanel, int oldIndex)
        {
            int oldX = -oldPanel.AutoScrollPosition.X;
            int oldY = -oldPanel.AutoScrollPosition.Y;
            int nowX = -newPanel.AutoScrollPosition.X;
            int nowY = -newPanel.AutoScrollPosition.Y;

            oldPanel.AutoScrollPosition = new Point(0, 0);
            newPanel.AutoScrollPosition = new Point(0, 0);

            DispCard addDc;
            int newIndex;

            パネル属性 pnlType;

            if (oldPanel != newPanel)
            {
                if (oldPanel == this.pnlPartyBag)
                {
                    // 荷物袋から表示削除
                    pnlPartyBag.Controls.RemoveAt(oldIndex);

                    // カード個数修正
                    this.tabPartyBag.Text = "荷物(" + pnlPartyBag.Controls.Count + ")";

                    for (int i = oldIndex; i < pnlPartyBag.Controls.Count; i++)
                    {
                        // 荷物袋Cardの元の位置より後ろのカードだけ配置修正
                        pnlPartyBag.Controls[i].Location =
                            new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                            , CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                    }

                    if (newPanel == this.pnlYadoCard)
                    {
                        // 荷物袋Cardからカード置場へ

                        // すでにSwap処理でListの最後に追加されているので
                        // それを元に表示用カードを作成
                        newIndex = inn.BankBoxCards.Count - 1;

                        HandCard hc = GetBagHandCard(newPanel, newIndex);
                        addDc = new DispCard(hc);

                        addDc.Location = new Point(
                            CWData.HAND_CARD_WIDTH * (newIndex % disp_num_x),
                            CWData.HAND_CARD_HEIGHT * (newIndex / disp_num_x));
                        pnlYadoCard.Controls.Add(addDc);

                        // カード個数修正
                        this.tabYadoCard.Text = "置場(" + pnlYadoCard.Controls.Count + ")";
                    }
                    else
                    {
                        // 荷物袋Cardからキャラクタへ
                        pnlType = GetPanelTypeNo(newPanel);

                        // すでにSwap処理でListの最後に追加されているので
                        // それを元に表示用カードを作成]
                        newIndex = GetCharacterPanelCount(pnlType) - 1;

                        addDc = new DispCard(GetCharacterHandCard(pnlType,newIndex));

                        addDc.Location = new Point(
                            CWData.HAND_CARD_WIDTH * (newIndex % disp_cnum_x),
                            CWData.HAND_CARD_HEIGHT * (newIndex / disp_cnum_x));
                        newPanel.Controls.Add(addDc);

                        SetDispCardCount(pnlType);
                    }

                }
                else if (oldPanel == this.pnlYadoCard)
                {
                    // カード置場から表示削除
                    pnlYadoCard.Controls.RemoveAt(oldIndex);

                    // カード個数修正
                    this.tabYadoCard.Text = "置場(" + pnlYadoCard.Controls.Count + ")";

                    for (int i = oldIndex; i < pnlYadoCard.Controls.Count; i++)
                    {
                        // カード置場の元の位置より後ろのカードだけ配置修正
                        pnlYadoCard.Controls[i].Location =
                            new Point(
                            CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                          , CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                    }

                    if (newPanel == this.pnlPartyBag)
                    {
                        // カード置場から荷物袋Cardへ

                        // すでにSwap処理でListの最後に追加されているので
                        // それを元に表示用カードを作成
                        newIndex = inn.CurrentParty.BagCards.Count - 1;

                        HandCard hc = GetBagHandCard(newPanel, newIndex);
                        addDc = new DispCard(hc);

                        addDc.Location = new Point(
                            CWData.HAND_CARD_WIDTH * (newIndex % disp_num_x),
                            CWData.HAND_CARD_HEIGHT * (newIndex / disp_num_x));

                        pnlPartyBag.Controls.Add(addDc);


                        // カード個数修正
                        this.tabPartyBag.Text = "荷物(" + pnlPartyBag.Controls.Count + ")";
                    }
                    else
                    {
                        // カード置場からキャラクタへ 
                        pnlType = GetPanelTypeNo(newPanel);

                        // すでにSwap処理でListの最後に追加されているので
                        // それを元に表示用カードを作成
                        newIndex = GetCharacterPanelCount(pnlType) - 1;

                        HandCard hc = GetCharacterHandCard(pnlType, newIndex);
                        addDc = new DispCard(hc);

                        addDc.Location = new Point(
                            CWData.HAND_CARD_WIDTH * (newIndex % disp_cnum_x),
                            CWData.HAND_CARD_HEIGHT * (newIndex / disp_cnum_x));

                        newPanel.Controls.Add(addDc);

                        SetDispCardCount(pnlType);
                    }
                }
                else
                {
                    パネル属性 pnlTypeOld = GetPanelTypeNo(oldPanel);

                    SetDispCardCount(pnlTypeOld);

                    oldPanel.Controls.RemoveAt(oldIndex);
                    for (int i = oldIndex; i < oldPanel.Controls.Count; i++)
                    {
                        // キャラクタカードの元の位置より後ろのカードだけ配置修正
                        oldPanel.Controls[i].Location =
                            new Point(CWData.HAND_CARD_WIDTH * (i % disp_cnum_x)
                            , CWData.HAND_CARD_HEIGHT * (i / disp_cnum_x));
                    }

                    if (newPanel == this.pnlYadoCard)
                    {
                        // キャラクタからカード置場へ

                        // すでにSwap処理でListの最後に追加されているので
                        // それを元に表示用カードを作成
                        newIndex = inn.BankBoxCards.Count - 1;

                        addDc = new DispCard(GetBagHandCard(newPanel, newIndex));

                        addDc.Location = new Point(
                            CWData.HAND_CARD_WIDTH * (newIndex % disp_num_x),
                            CWData.HAND_CARD_HEIGHT * (newIndex / disp_num_x));

                        pnlYadoCard.Controls.Add(addDc);

                        // カード個数修正
                        this.tabYadoCard.Text = "置場(" + pnlYadoCard.Controls.Count + ")";
                    }
                    else if (newPanel == this.pnlPartyBag)
                    {
                        // キャラクタから荷物袋Cardへ

                        // すでにSwap処理でListの最後に追加されているので
                        // それを元に表示用カードを作成
                        newIndex = inn.CurrentParty.BagCards.Count - 1;

                        addDc = new DispCard(GetBagHandCard(newPanel, newIndex));

                        addDc.Location = new Point(
                            CWData.HAND_CARD_WIDTH * (newIndex % disp_num_x),
                            CWData.HAND_CARD_HEIGHT * (newIndex / disp_num_x));
                        pnlPartyBag.Controls.Add(addDc);

                        // カード個数修正
                        this.tabPartyBag.Text = "荷物(" + pnlPartyBag.Controls.Count + ")";
                    }
                    else
                    {
                        // キャラクタからキャラクタへ
                        pnlType = GetPanelTypeNo(newPanel);

                        // すでにSwap処理でListの最後に追加されているので
                        // それを元に表示用カードを作成
                        newIndex = GetCharacterPanelCount(pnlType) - 1;

                        addDc = new DispCard(GetCharacterHandCard(pnlType,newIndex));

                        addDc.Location = new Point(
                            CWData.HAND_CARD_WIDTH * (newIndex % disp_cnum_x),
                            CWData.HAND_CARD_HEIGHT * (newIndex / disp_cnum_x));
                        newPanel.Controls.Add(addDc);

                        SetDispCardCount(pnlType);
                    }

                }
            }

            oldPanel.AutoScrollPosition = new Point(oldX, oldY);

            newPanel.AutoScrollPosition = new Point(nowX, nowY);
        }

        /// <summary>
        /// カードのパネル内単独移動を表示に反映する()
        /// </summary>
        /// <param name="inn"></param>
        /// <param name="panel"></param>
        /// <param name="oldIndex"></param>
        /// <param name="insertPos"></param>
        private void DisplayInnerPanelMove(InnData inn, Panel panel, 
            int oldIndex, int insertPos)
        {

            if (panel == this.pnlPartyBag || panel == this.pnlYadoCard)
            {

                int oldX = -panel.AutoScrollPosition.X;
                int oldY = -panel.AutoScrollPosition.Y;

                panel.AutoScrollPosition = new Point(0, 0);

                // 表示上のカード変更
                // パネル内の枚数も少ないので
                int minIndex = insertPos < oldIndex
                    ? insertPos : oldIndex;

                int maxIndex = (insertPos - 1) > oldIndex
                    ? (insertPos - 1) : oldIndex;

                for (int i = minIndex
                    ; i <= maxIndex
                    ; i++)
                {
                    ((DispCard)panel.Controls[i]).SetHc(
                        GetBagHandCard(panel, i));
                }

                ////// パネルから表示削除
                ////panel.Controls.RemoveAt(oldIndex);

                ////if (insertPos > oldIndex)
                ////{
                ////    // 挿入位置が移動元よりも後

                ////    // 移動元が削除されると挿入位置が１つ前にずれる。 
                ////    insertPos--;

                ////    for (int i = oldIndex; i < insertPos; i++)
                ////    {
                ////        // Cardの移動元の位置から挿入位置より前のカードだけ配置修正
                ////        panel.Controls[i].Location =
                ////            new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                ////            , CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                ////    }
                ////}

                ////List<Control> contList = new List<Control>();
                ////for (int i = panel.Controls.Count - 1; i >= insertPos; i--)
                ////{
                ////    // Cardの挿入位置以降のカードを一度外す
                ////    panel.Controls[i].Location =
                ////        new Point(CWData.HAND_CARD_WIDTH * ((i + 1) % disp_num_x)
                ////        , CWData.HAND_CARD_HEIGHT * ((i + 1) / disp_num_x));

                ////    contList.Add(panel.Controls[i]);

                ////    panel.Controls.RemoveAt(i);
                ////}

                ////addDc = new DispCard(GetBagHandCard(panel, insertPos));

                ////addDc.Location = new Point(
                ////    CWData.HAND_CARD_WIDTH * (insertPos % disp_num_x),
                ////    CWData.HAND_CARD_HEIGHT * (insertPos / disp_num_x));
                ////panel.Controls.Add(addDc);

                ////contList.Reverse();
                ////foreach (Control cont in contList)
                ////{
                ////    panel.Controls.Add(cont);
                ////}

                panel.AutoScrollPosition = new Point(oldX, oldY);

            }
        
        }

        #endregion

        #region マウスイベント

        /// <summary>
        ///  パネル汎用　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Pnl_MouseDown(object sender, MouseEventArgs e)
        {
            int oldX;
            int oldY;

            mouseDownIndex = -1;
            mouseDownPanel = (Panel)sender;
            mouseDownPanel.Select();
            mouseDownPoint = Point.Empty;

            // 処理対象
            int indexOfX = (-mouseDownPanel.AutoScrollPosition.X + e.X) 
                / CWData.HAND_CARD_WIDTH;
            int indexOfY = (-mouseDownPanel.AutoScrollPosition.Y + e.Y) 
                / CWData.HAND_CARD_HEIGHT;

            if (mouseDownPanel == pnlPartyBag 
                || mouseDownPanel == pnlYadoCard)
            {
                index = indexOfY * disp_num_x + indexOfX;
            }
            else
            {
                index = indexOfY * disp_cnum_x + indexOfX;

            }

            if (mouseDownPanel == pnlYadoCard
                || mouseDownPanel == pnlPartyBag)
            {
                // カード置場 か　荷物袋
                if (indexOfX >= disp_num_x
                    || index >= mouseDownPanel.Controls.Count)
                {
                    // 範囲外
                    // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                    if (e.Button == MouseButtons.Right)
                    {
                        List<HandCard> hands = new List<HandCard>();
                        foreach (Object obj in mouseDownPanel.Controls)
                        {
                            hands.Add(((DispCard)obj).hc);
                        }

                        using (CardInfoDialog cid = new CardInfoDialog(hands, 0))
                        {
                            cid.ShowDialog();
                        }
                    }
                }
                else
                {
                    // Cardを指している
                    // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                    if (e.Button == MouseButtons.Right)
                    {

                        List<HandCard> hands = new List<HandCard>();
                        foreach (Object obj in mouseDownPanel.Controls)
                        {
                            hands.Add(((DispCard)obj).hc);
                        }

                        using (CardInfoDialog cid = new CardInfoDialog(hands, index))
                        {
                            cid.ShowDialog();
                        }
                    }
                    else if (e.Button == MouseButtons.Left)
                    {
                        mouseDownIndex = index;
                        mouseDownPoint = new Point(e.X, e.Y);

                    }
                    else if (e.Button == MouseButtons.Middle)
                    {

                        // ゴミ箱送りか指定先に送るか
                        bool moveCard = true;
                        if (characterCount == 0)
                        {
                            if (cmbSend.SelectedIndex > 0)
                            {
                                moveCard = false;
                            }
                        }
                        else
                        {
                            if (cmbSend.SelectedIndex > characterCount + 1)
                            {
                                moveCard = false;
                            }
                        }

                        if (moveCard)
                        {
                            // パネル指定先に送る
                            mouseDownIndex = index;
                            MoveProcessByIndex(cmbSend.SelectedIndex);
                        }
                        else
                        {
                            // カード削除
                            if (MessageDlg.ShowYesNoDialog("カードを削除します。よろしいですか？"
                                ) == DialogResult.Yes)
                            {
                                oldX = mouseDownPanel.AutoScrollPosition.X;
                                oldY = mouseDownPanel.AutoScrollPosition.Y;
                                mouseDownPanel.AutoScrollPosition = new Point(0, 0);
                                if (mouseDownPanel == pnlYadoCard)
                                {
                                    // カード置場　カード削除
                                    inn.DeleteRenamedCardData(inn.BankBoxCards, inn.BankBoxCards[index]);
                                    // 表示反映
                                    // カード置場から表示削除
                                    pnlYadoCard.Controls.RemoveAt(index);

                                    // カード個数修正
                                    this.tabYadoCard.Text = "置場(" + pnlYadoCard.Controls.Count + ")";

                                    for (int i = index; i < pnlYadoCard.Controls.Count; i++)
                                    {
                                        // カード置場の元の位置より後ろのカードだけ配置修正
                                        pnlYadoCard.Controls[i].Location =
                                            new Point(
                                            CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                                          , CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                                    }
                                }
                                else
                                {
                                    // 荷物袋　カード削除
                                    inn.DeleteRenamedCardData(inn.CurrentParty.BagCards, inn.CurrentParty.BagCards[index]);
                                    // 表示反映
                                    // 荷物袋から表示削除
                                    pnlPartyBag.Controls.RemoveAt(index);

                                    // カード個数修正
                                    this.tabPartyBag.Text = "荷物(" + pnlPartyBag.Controls.Count + ")";

                                    for (int i = index; i < pnlPartyBag.Controls.Count; i++)
                                    {
                                        // 荷物袋Cardの元の位置より後ろのカードだけ配置修正
                                        pnlPartyBag.Controls[i].Location =
                                            new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                                            , CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                                    }
                                }
                                mouseDownPanel.AutoScrollPosition = new Point(oldX, oldY);
                            }
                        }
                    }
                }
            }
            else
            {
                // キャラクタカード
                if (indexOfX >= disp_cnum_x
                    || index >= mouseDownPanel.Controls.Count)
                {
                    // 範囲外
                    // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                    if (e.Button == MouseButtons.Right)
                    {
                        List<HandCard> hands = new List<HandCard>();
                        foreach (Object obj in mouseDownPanel.Controls)
                        {
                            hands.Add(((DispCard)obj).hc);
                        }

                        パネル属性 pnlType = GetPanelTypeNo(mouseDownPanel);
                        CharacterData cd = inn.CurrentParty.Characters[pnlType.CharaNo];

                        using (CardInfoDialog cid = new CardInfoDialog(_pm.paramOption
                            , cd, hands, 0))
                        {
                            cid.ShowDialog();
                        }
                    }
                }
                else
                {
                    // Cardを指している
                    // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                    if (e.Button == MouseButtons.Right)
                    {

                        List<HandCard> hands = new List<HandCard>();
                        foreach (Object obj in mouseDownPanel.Controls)
                        {
                            hands.Add(((DispCard)obj).hc);
                        }

                        パネル属性 pnlType = GetPanelTypeNo(mouseDownPanel);
                        CharacterData cd = inn.CurrentParty.Characters[pnlType.CharaNo];

                        using (CardInfoDialog cid = new CardInfoDialog(_pm.paramOption
                            , cd, hands, index))
                        {
                            cid.ShowDialog();
                        }
                    }
                    else if (e.Button == MouseButtons.Left)
                    {
                        mouseDownIndex = index;
                        mouseDownPoint = new Point(e.X, e.Y);

                    }
                    else if (e.Button == MouseButtons.Middle)
                    {
                        // ゴミ箱送りか指定先に送るか
                        bool moveCard = true;
                        if (characterCount == 0)
                        {
                            if (cmbSend.SelectedIndex > 0)
                            {
                                moveCard = false;
                            }
                        }
                        else
                        {
                            if (cmbSend.SelectedIndex > characterCount + 1)
                            {
                                moveCard = false;
                            }
                        }

                        if (moveCard)
                        {
                            // パネル指定先に送る
                            mouseDownIndex = index;
                            MoveProcessByIndex(cmbSend.SelectedIndex);
                        }
                        else
                        {
                            // キャラクタ所持カード　カード削除
                            if (MessageDlg.ShowYesNoDialog("カードを削除します。よろしいですか？"
                                ) == DialogResult.Yes)
                            {


                                oldX = mouseDownPanel.AutoScrollPosition.X;
                                oldY = mouseDownPanel.AutoScrollPosition.Y;
                                mouseDownPanel.AutoScrollPosition = new Point(0, 0);

                                // キャラクタの所持カードをカード置場に移動
                                パネル属性 pnlType = GetPanelTypeNo(mouseDownPanel);
                                CharacterData cd = inn.CurrentParty.Characters[pnlType.CharaNo];
                                
                                List<WidCard> list = null;
                                switch (pnlType.Tefuda)
                                {
                                    case 手札種類.所持スキル:
                                        list = cd.Skills;
                                        break;
                                    case 手札種類.所持アイテム:
                                        list = cd.Items;
                                        break;
                                    case 手札種類.所有召喚獣:
                                        list = cd.Beasts;
                                        break;
                                    default:
                                        PartyMaker.CWIErrorMessage("カードの種類が間違っています。");
                                        return;
                                }
                                // WidCard 削除
                                list.RemoveAt(index);

                                // 表示反映
                                SetDispCardCount(pnlType);

                                mouseDownPanel.Controls.RemoveAt(index);
                                for (int i = index; i < mouseDownPanel.Controls.Count; i++)
                                {
                                    // カードの元の位置より後ろのカードだけ配置修正
                                    mouseDownPanel.Controls[i].Location =
                                        new Point(CWData.HAND_CARD_WIDTH * (i % disp_cnum_x)
                                        , CWData.HAND_CARD_HEIGHT * (i / disp_cnum_x));
                                }


                                mouseDownPanel.AutoScrollPosition = new Point(oldX, oldY);
                            }
                        }
                    }
                }

            }
    
        }

        /// <summary>
        ///  パネル汎用　マウスムーブイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Pnl_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDownPoint != Point.Empty)
            {

                mouseDownPanel = (Panel)sender;

                //ドラッグとしないマウスの移動範囲を取得する
                Rectangle moveRect = new Rectangle(
                    mouseDownPoint.X - SystemInformation.DragSize.Width / 2,
                    mouseDownPoint.Y - SystemInformation.DragSize.Height / 2,
                    SystemInformation.DragSize.Width,
                    SystemInformation.DragSize.Height);
                //ドラッグとする移動範囲を超えたか調べる
                if (!moveRect.Contains(e.X, e.Y))
                {

                    //ドラッグの準備
                    DispCard dcx = (DispCard)(mouseDownPanel.Controls[mouseDownIndex]);

                    //ドラッグ&ドロップ処理を開始する
                    DragDropEffects dde =
                        dcx.DoDragDrop(dcx, DragDropEffects.All);


                    mouseDownPoint = Point.Empty;
                }
            }

        }

        /// <summary>
        /// パネル汎用　マウスアップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Pnl_MouseUp(object sender, MouseEventArgs e)
        {
            mouseDownPoint = Point.Empty;
        }

        /// <summary>
        /// マウスホイールの移動量を取得
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Inn_MouseWheel(object sender, MouseEventArgs e)
        {
            ScrollPnl(mouseDownPanel,- e.Delta);
        }

        /// <summary>
        /// SkillパネルとItemパネルでのダブルクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SIPnl_MouseDoubleClick(object sender, MouseEventArgs e)
        {

            Panel checkPanel = (Panel)sender;

            // 処理対象
            int indexOfX = (-checkPanel.AutoScrollPosition.X + e.X) / CWData.HAND_CARD_WIDTH;
            int indexOfY = (-checkPanel.AutoScrollPosition.Y + e.Y) / CWData.HAND_CARD_HEIGHT;

            index = indexOfY * disp_cnum_x + indexOfX;


            // キャラクタカード
            if (indexOfX < disp_cnum_x
                && index < checkPanel.Controls.Count)
            {
                // Cardを指している
                if (e.Button == MouseButtons.Left)
                {
                    // 左ダブルクリック
                    string lastStr = checkPanel.Name.Substring(checkPanel.Name.Length - 1, 1);
                    int charNo = int.Parse(lastStr) - 1;
                    string typeStr = checkPanel.Name.Substring(3, checkPanel.Name.Length - 4);
                    手札種類 tefuda;
                    if (typeStr == "Skill")
                    {
                        tefuda = 手札種類.所持スキル;
                    }
                    else if (typeStr == "Item")
                    {
                        tefuda = 手札種類.所持アイテム;
                    }
                    else if (typeStr == "Beast")
                    {
                        tefuda = 手札種類.所有召喚獣;
                        // まかりまちがってダブルクリックしてもホールドしません
                        return;
                    }
                    else
                    {
                        PartyMaker.CWIErrorMessage("ありえない手札種類です。（ダブルクリックイベント）");
                        return;
                    }

                    CharacterData charaData = inn.CurrentParty.Characters[charNo];
                    WidCard wc;
                    if (tefuda == 手札種類.所持スキル)
                    {
                        wc = charaData.Skills[index];
                        if (Util2.IsPenaltyCard(wc))
                        {
                            // ペナルティカードならば何もしない
                            //（通常のスキルカードやアイテムカードがペナルティカードとは思えないが）
                            return;
                        }
                        
                        // ホールド状態反転（本当はメソッドを作って処理するべき）
                        wc.HoldCard = !wc.HoldCard;

                        // 表示反映
                        SkillHandCard shc　= new SkillHandCard(wc);

                        shc.UpdateImage(
                            GameParam.GetCardFacterColor(charaData,
                            shc.widData.BodyFacter, shc.widData.MindFacter),
                            GameParam.GetSkillRemainColor(shc.widData.SkillRemainUseCount,
                            GameParam.GetMaxSkillUseCount(charaData.Level, shc.widData.Level)));


                        ((DispCard)checkPanel.Controls[index]).SetHc(shc);

                    }
                    else if (tefuda == 手札種類.所持アイテム)
                    {
                        wc = charaData.Items[index];
                        if (Util2.IsPenaltyCard(wc))
                        {
                            // ペナルティカードならば何もしない
                            //（通常のスキルカードやアイテムカードがペナルティカードとは思えないが）
                            return;
                        }

                        // ホールド状態反転（本当はメソッドを作って処理するべき）
                        wc.HoldCard = !wc.HoldCard;

                        // 表示反映
                        ItemHandCard ihc = new ItemHandCard(wc);

                        ihc.UpdateImage(
                            GameParam.GetCardFacterColor(charaData,
                            ihc.widData.BodyFacter, ihc.widData.MindFacter));

                        ((DispCard)checkPanel.Controls[index]).SetHc(ihc);

                    }
                }
            }

            
        }

        #endregion

        #region ドラッグドロップ関連

        /// <summary>
        /// パネル汎用　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Pnl_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //そうであればドロップ効果をCopyにする
            Panel pnlNew = (Panel) sender;
            if (e.Data.GetDataPresent(typeof(DispCard)) 
                && CanDropPanel(inn,mouseDownPanel,mouseDownIndex,pnlNew))
            {
                // Characterタブシートの別のパネルには直接D&D出来ない。
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                //DispCard型でなければ受け入れない
                e.Effect = DragDropEffects.None;
            }

        }

        /// <summary>
        /// パネル汎用　ドラッグドロップイベント
        /// 拡張ハンドウインドウでは同じパネル間の移動のみ記述
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Pnl_DragDrop(object sender, DragEventArgs e)
        {

            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象
                Panel panelDropped = (Panel)sender;

                Point pointClient = panelDropped.PointToClient(new Point(e.X, e.Y));
                int indexOfX = (-panelDropped.AutoScrollPosition.X + pointClient.X) 
                    / CWData.HAND_CARD_WIDTH;
                int indexOfY = (-panelDropped.AutoScrollPosition.Y + pointClient.Y)
                    / CWData.HAND_CARD_HEIGHT;

                int posOfX = (-panelDropped.AutoScrollPosition.X + pointClient.X)
                    % CWData.HAND_CARD_WIDTH;

                if (panelDropped == pnlYadoCard 
                    || panelDropped == pnlPartyBag)
                {
                    index = indexOfY * disp_num_x + indexOfX;
                }
                else
                {
                    index = indexOfY * disp_cnum_x + indexOfX;
                }

                パネル属性 pnlTypeOld;
                パネル属性 pnlType;
                手札種類 tefuda;

                if (panelDropped == pnlPartyBag)
                {
                    // 荷物袋カードにドロップ
                    if (indexOfX < disp_num_x 
                        && index < pnlPartyBag.Controls.Count)
                    {
                        // 対象は荷物袋カード

                        if (mouseDownPanel == pnlPartyBag)
                        {
                            // 荷物袋カードどうしなのでファイルに変化なし
                                
                            if (posOfX < CWData.Among_Space ||
                                posOfX >= (CWData.HAND_CARD_WIDTH - CWData.Among_Space))
                            {
                                // 対象は荷物袋カードの隙間か前後
                                // カード挿入・移動
                                int insertPos = index;
                                if (posOfX >=
                                    (CWData.HAND_CARD_WIDTH - CWData.Among_Space))
                                {
                                    // カードの右側余白の場合は次のカードとの間なので
                                    // インクリメント
                                    insertPos++;
                                }

                                if (insertPos == mouseDownIndex ||
                                   insertPos == mouseDownIndex + 1)
                                {
                                    return;
                                }

                                inn.MovePartyBagCard(mouseDownIndex, insertPos);

                                DisplayInnerPanelMove(inn, panelDropped, mouseDownIndex, insertPos);

                                return;
                            }
                            else
                            {
                                // 対象は荷物袋カード自身
                                // カードスワップ

                                // 荷物袋カード内移動
                                inn.SwapPartyBagCard(mouseDownIndex, index);

                                // 表示上のカード変更
                                ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                    GetBagHandCard(mouseDownPanel, mouseDownIndex));

                                ((DispCard)panelDropped.Controls[index]).SetHc(
                                    GetBagHandCard(panelDropped, index));

                                return;
                            }
                        }
                        else if (mouseDownPanel == pnlYadoCard)
                        {
                            // カード置場カードと荷物袋カードの交換
                            inn.SwapPartyBagCardAndCard(index, mouseDownIndex);

                            // 表示上のカード変更
                            ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                GetBagHandCard(mouseDownPanel, mouseDownIndex));

                            ((DispCard)panelDropped.Controls[index]).SetHc(
                                GetBagHandCard(panelDropped, index));

                            return;
                        }
                        else
                        {
                            // パーティーPCの手札と荷物袋カードの交換
                            // カードの種類が同じでないといけない
                            pnlTypeOld = GetPanelTypeNo(mouseDownPanel);
                            tefuda = GetCardType(panelDropped, index);
                            if (tefuda == pnlTypeOld.Tefuda)
                            {
                                if (!(inn.SwapCharacterCardAndPartyBagCard(
                                    pnlTypeOld.CharaNo, pnlTypeOld.Tefuda, mouseDownIndex, index)))
                                {
                                    MessageDlg.ShowInfoDialog("交換時エラー");
                                    return;
                                }

                                // 表示上のカード変更

                                ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                    GetCharacterHandCard(pnlTypeOld, mouseDownIndex));

                                ((DispCard)panelDropped.Controls[index]).SetHc(
                                    GetBagHandCard(panelDropped, index));

                                return;
                            }
                            else
                            {
                                // 手札種類
                                MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。"
                                    );

                                return;

                            }
                        }
                        
                    }
                    else
                    {
                        // 対象は荷物袋そのもの
                        if (mouseDownPanel == pnlPartyBag)
                        {
                            // 何も起こらない
                            return;
                        }
                        else if (mouseDownPanel == pnlYadoCard)
                        {
                            // カード置場カードを荷物袋に移動
                            inn.SwapPartyBagCardAndCard(-1, mouseDownIndex);
                        }
                        else
                        {
                            // パーティーPCの手札を荷物袋に移動
                            pnlTypeOld = GetPanelTypeNo(mouseDownPanel);
                            
                            // パーティーPCの所持カードを荷物袋へ
                            if (!(inn.SwapCharacterCardAndPartyBagCard(pnlTypeOld.CharaNo
                                , pnlTypeOld.Tefuda, mouseDownIndex, -1)))
                            {

                                MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。"
                                    );
                                return;
                            }
                        }


                        // カード表示反映
                        DisplayCardMove(inn, mouseDownPanel, panelDropped, mouseDownIndex);
                    }

                }
                else if (panelDropped == pnlYadoCard)
                {
                    // カード置場にドロップ
                    if (indexOfX < disp_num_x 
                        && index < pnlYadoCard.Controls.Count)
                    {
                        // カード置場のカードにドロップ

                        if (mouseDownPanel == pnlYadoCard)
                        {

                            // カード置場カードどうしなのでファイルに変化なし

                            if (posOfX < CWData.Among_Space ||
                                posOfX >= (CWData.HAND_CARD_WIDTH - CWData.Among_Space))
                            {
                                // 対象はカード置場のカードの隙間か前後
                                // カード挿入・移動
                                int insertPos = index;
                                if(posOfX >= 
                                    (CWData.HAND_CARD_WIDTH - CWData.Among_Space))
                                {
                                    // カードの右側余白は次のカードとの隙間なので
                                    // インクリメント
                                    insertPos ++;
                                }

                                if (insertPos == mouseDownIndex ||
                                   insertPos == mouseDownIndex + 1)
                                {
                                    return;
                                }

                                inn.MoveCard(mouseDownIndex, insertPos);

                                DisplayInnerPanelMove(inn, panelDropped, mouseDownIndex, insertPos);

                                return;
                            }
                            else
                            {
                                // 対象はカード置場のカード自身
                                // カードスワップ
                                inn.SwapCard(mouseDownIndex, index);

                                // 表示上のカード変更
                                ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                    GetBagHandCard(mouseDownPanel, mouseDownIndex));

                                ((DispCard)panelDropped.Controls[index]).SetHc(
                                    GetBagHandCard(panelDropped, index));

                                return;
                            }

                        }
                        else if (mouseDownPanel == pnlPartyBag)
                        {
                            // 荷物袋カードとカード置場のカードの交換
                            inn.SwapPartyBagCardAndCard(mouseDownIndex, index);

                            // 表示上のカード変更
                            ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                GetBagHandCard(mouseDownPanel, mouseDownIndex));

                            ((DispCard)panelDropped.Controls[index]).SetHc(
                                GetBagHandCard(panelDropped, index));

                            return;
                        }
                        else
                        {
                            // パーティーPCの手札とカード置場のカードの交換
                            // 種類が同じでないといけない
                            pnlTypeOld = GetPanelTypeNo(mouseDownPanel);
                            tefuda = GetCardType(panelDropped, index);
                            if (tefuda == pnlTypeOld.Tefuda)
                            {
                                if (!(inn.SwapCharacterCardAndCard(
                                    pnlTypeOld.CharaNo, pnlTypeOld.Tefuda, mouseDownIndex, index)))
                                {
                                    MessageDlg.ShowInfoDialog("交換時エラー"
                                        );
                                    return;
                                }

                                // 表示上のカード変更
                                ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                    GetCharacterHandCard(pnlTypeOld, mouseDownIndex));

                                ((DispCard)panelDropped.Controls[index]).SetHc(
                                    GetBagHandCard(panelDropped, index));

                                return;
                            }
                            else
                            {
                                // 手札種類
                                MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。"
                                   );

                                return;

                            }
                        }
                        
                    }
                    else
                    {
                        // カード置場そのものにドロップ
                        if (mouseDownPanel == pnlYadoCard)
                        {
                            // カード置場のカードなので変化なし
                            return;
                        }
                        else if (mouseDownPanel == pnlPartyBag)
                        {
                            // 荷物袋カードをカード置場へ
                            inn.SwapPartyBagCardAndCard(mouseDownIndex, -1);

                        }
                        else
                        {
                            // パーティーPCの所持カードをカード置場に移動
                            pnlTypeOld = GetPanelTypeNo(mouseDownPanel);

                            // キャラクタの所持カードをカード置場に移動
                            if (!(inn.SwapCharacterCardAndCard(
                                pnlTypeOld.CharaNo, pnlTypeOld.Tefuda, mouseDownIndex, -1)))
                            {
                                MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。"
                                    );
                                return;
                            }
                        }
                        // カード表示反映
                        DisplayCardMove(inn, mouseDownPanel, panelDropped, mouseDownIndex);
                    }
                }
                else
                {
                    // パーティーPCの手札にドロップ
                    if (mouseDownPanel == panelDropped)
                    {
                        // 同じパネル間
                        if (indexOfX < disp_cnum_x && index < panelDropped.Controls.Count)
                        {
                            // キャラクタのカードにドロップ
                            // 同じキャラクタの同じ種類のカードどうしなのでファイルに変化なし

                            if (posOfX < CWData.Among_Space ||
                                posOfX >= (CWData.HAND_CARD_WIDTH - CWData.Among_Space))
                            {
                                // 対象はキャラクタのカードの隙間か前後
                                // カード挿入・移動

                                // キャラクタの所持カード
                                pnlTypeOld = GetPanelTypeNo(mouseDownPanel);

                                int insertPos = 0;
                                if (posOfX < CWData.Among_Space)
                                {
                                    // カード挿入・移動(カード左端)
                                    insertPos = index;
                                }
                                else
                                {
                                    // カード挿入・移動(カード右端)
                                    insertPos = index + 1;
                                }

                                if(insertPos == mouseDownIndex ||
                                   insertPos == mouseDownIndex + 1)
                                {
                                    // 何も起こらない
                                    return;
                                }
                                else if (insertPos > mouseDownIndex)
                                {
                                    // パネル内で挿入位置が元の位置より大
                                    insertPos--;
                                }

                                // パネル内カード移動
                                inn.SwapCharacterCard(pnlTypeOld.CharaNo
                                    , pnlTypeOld.CharaNo, pnlTypeOld.Tefuda
                                    , mouseDownIndex, -(insertPos + 2));
                                
                                if (insertPos > mouseDownIndex)
                                {
                                    // パネル内で挿入位置が元の位置より大
                                    // 挿入位置判定を元に戻す。
                                    insertPos++;
                                }

                                // 表示上のカード変更
                                // ここでは枚数が変わっていない
                                // パネル内の枚数も少ないので
                                int minIndex = insertPos < mouseDownIndex
                                    ? insertPos : mouseDownIndex;

                                int maxIndex = (insertPos - 1) > mouseDownIndex
                                    ? (insertPos - 1) : mouseDownIndex;

                                for (int i = minIndex
                                    ; i <= maxIndex
                                    ; i++)
                                {
                                    ((DispCard)panelDropped.Controls[i]).SetHc(
                                        GetCharacterHandCard(pnlTypeOld, i));
                                }

                                return;
                            }
                            else
                            {
                                // 対象はキャラクタのカード自身
                                // カードスワップ

                                // キャラクタの所持カード
                                pnlTypeOld = GetPanelTypeNo(mouseDownPanel);

                                // カードスワップ
                                inn.SwapCharacterCard(pnlTypeOld.CharaNo
                                    , pnlTypeOld.CharaNo, pnlTypeOld.Tefuda, mouseDownIndex, index);

                                // 表示上のカード変更

                                ((DispCard)panelDropped.Controls[mouseDownIndex]).SetHc(
                                    GetCharacterHandCard(pnlTypeOld, mouseDownIndex));

                                ((DispCard)panelDropped.Controls[index]).SetHc(
                                    GetCharacterHandCard(pnlTypeOld, index));

                                return;
                            }
                        }
                        else
                        {
                            // 同じパネル間でカードのないところにドロップしても
                            // 何も起こらない
                            return;
                        }

                    }
                    else
                    {
                        pnlTypeOld = new パネル属性();
                        pnlType = GetPanelTypeNo(panelDropped);

                        // ドラッグ前と違うパネル（パーティーPCの手札パネル）に落とす
                        if (indexOfX < disp_cnum_x && index < panelDropped.Controls.Count)
                        {
                            // パーティーPCの手札にドロップ
                            // いずれにせよ種類に注意

                            if (mouseDownPanel == pnlYadoCard)
                            {
                                // カード置場とパーティーPCの手札の交換
                                tefuda = GetCardType(mouseDownPanel,mouseDownIndex);

                                if (pnlType.Tefuda == tefuda)
                                {
                                    // カード置場カードとキャラクタ手札の交換
                                    if (!(inn.SwapCharacterCardAndCard(
                                        pnlType.CharaNo, pnlType.Tefuda, index, mouseDownIndex)))
                                    {
                                        MessageDlg.ShowInfoDialog("交換時エラー");
                                        return;
                                    }

                                    // 表示上のカード変更
                                    ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                        GetBagHandCard(mouseDownPanel, mouseDownIndex));

                                    ((DispCard)panelDropped.Controls[index]).SetHc(
                                        GetCharacterHandCard(pnlType, index));

                                    return;
                                }
                                else
                                {
                                    // 手札種類
                                    MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。"
                                        );

                                    return;
                                }

                            }
                            else if (mouseDownPanel == pnlPartyBag)
                            {
                                // 荷物袋とパーティーPCの手札の交換
                                tefuda = GetCardType(mouseDownPanel, mouseDownIndex);

                                // 荷物袋カードとキャラクタ手札の交換
                                if (pnlType.Tefuda == tefuda)
                                {
                                    if (!(inn.SwapCharacterCardAndPartyBagCard(
                                        pnlType.CharaNo, pnlType.Tefuda, index, mouseDownIndex)))
                                    {
                                        MessageDlg.ShowInfoDialog("交換時エラー");
                                        return;
                                    }

                                    // 表示上のカード変更
                                    ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                        GetBagHandCard(mouseDownPanel, mouseDownIndex));

                                    ((DispCard)panelDropped.Controls[index]).SetHc(
                                        GetCharacterHandCard(pnlType, index));

                                    return;
                                }
                                else
                                {
                                    // 手札種類
                                    MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。");

                                    return;
                                }
                            }
                            else
                            {
                                // パーティーPCの手札どうしの交換
                                pnlTypeOld = GetPanelTypeNo(mouseDownPanel);
                                if (pnlType.Tefuda == pnlTypeOld.Tefuda)
                                {
                                    inn.SwapCharacterCard(pnlTypeOld.CharaNo
                                        , pnlType.CharaNo
                                        , pnlType.Tefuda
                                        , mouseDownIndex
                                        , index);

                                    // 表示上のカード変更

                                    ((DispCard)mouseDownPanel.Controls[mouseDownIndex]).SetHc(
                                        GetCharacterHandCard(pnlTypeOld, mouseDownIndex));

                                    ((DispCard)panelDropped.Controls[index]).SetHc(
                                        GetCharacterHandCard(pnlType, index));

                                    return;
                                }
                                else
                                {
                                    // 種類が違う
                                    MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。");
                                    return;
                                }
                            }
                        }
                        else
                        {
                            // パーティーPCの手札置場にドロップ
                            // いずれにせよ種類に注意

                            pnlTypeOld = new パネル属性();
                            if (mouseDownPanel == pnlYadoCard)
                            {
                                // カード置場のカードをパーティーPCの手札置場へ

                                tefuda = GetCardType(mouseDownPanel,mouseDownIndex);

                                if (pnlType.Tefuda == tefuda)
                                {
                                    // カード置場カードをキャラクタに移動
                                    if (!(inn.SwapCharacterCardAndCard(
                                        pnlType.CharaNo, pnlType.Tefuda, -1, mouseDownIndex)))
                                    {
                                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                                        return;
                                    }
                                }
                                else
                                {
                                    // 手札種類
                                    MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。");
                                    return;
                                }

                            }
                            else if (mouseDownPanel == pnlPartyBag)
                            {
                                // 荷物袋のカードをパーティーPCの手札置場へ
                                tefuda = GetCardType(mouseDownPanel,mouseDownIndex);

                                // 荷物袋カードをキャラクタに移動
                                if (pnlType.Tefuda == tefuda)
                                {
                                    if (!(inn.SwapCharacterCardAndPartyBagCard(
                                        pnlType.CharaNo, pnlType.Tefuda, -1, mouseDownIndex)))
                                    {
                                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                                        return;
                                    }
                                }
                                else
                                {
                                    // 手札種類
                                    MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。");
                                    return;
                                }

                            }
                            else
                            {
                                // パーティーPCの手札をパーティーPCの手札置場へ

                                pnlTypeOld = GetPanelTypeNo(mouseDownPanel);
                                if (pnlType.Tefuda == pnlTypeOld.Tefuda)
                                {
                                    if (!(inn.SwapCharacterCard(pnlTypeOld.CharaNo
                                        , pnlType.CharaNo
                                        , pnlType.Tefuda
                                        , mouseDownIndex
                                        , -1)))
                                    {
                                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                                        return;

                                    }

                                }
                                else
                                {
                                    // 種類が違う
                                    MessageDlg.ShowInfoDialog("手札種類が違うので移動できません。");
                                    return;
                                }

                            }

                            // カード表示反映
                            DisplayCardMove(inn, mouseDownPanel, panelDropped, mouseDownIndex);

                        }
                    }

                }


            }
        }

        /// <summary>
        /// パネル汎用　クエリコンティニュードロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Pnl_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;

        }

        /// <summary>
        /// タブコントロール　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabYado_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //そうであればドロップ効果をCopyにする
            Point pointClient = tabYado.PointToClient(new Point(e.X, e.Y));
            if (e.Data.GetDataPresent(typeof(DispCard))
                && pointClient.Y < CWData.TAB_HEADER_HEIGHT
                && pointClient.X < CWData.TAB_HEADER_WIDTH * (characterCount + 2)
                && inn.CurrentParty != null)
            {

                index = pointClient.X / CWData.TAB_HEADER_WIDTH;
                tabYado.SelectedIndex = index;
                tabYado.Update();

                if (inn.CurrentParty.InScenario)
                {
                    // シナリオ進行中
                    if (mouseDownPanel == pnlYadoCard)
                    {
                        // 進行中はカード置場からパーティーにカードを移動できない
                        e.Effect = DragDropEffects.None;

                    }
                    else if (pointClient.X < CWData.TAB_HEADER_WIDTH * 1)
                    {
                        // 進行中はパーティーからカード置場にカードを移動できない
                        e.Effect = DragDropEffects.None;
                    }
                    else
                    {
                        // パーティー内ならばOK
                        e.Effect = DragDropEffects.Copy;
                    }
                }
                else
                {
                    // 宿待機状態
                    e.Effect = DragDropEffects.Copy;
                }
            }
            else
            {
                // DispCard型でなければ受け入れない
                // Tabヘッダ範囲外は受け入れない
                // CurrentParty がない場合は受け入れない
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// タブコントロール　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabYado_DragDrop(object sender, DragEventArgs e)
        {
            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象

                Point pointClient = tabYado.PointToClient(new Point(e.X, e.Y));
                int indexOfX = pointClient.X / CWData.TAB_HEADER_WIDTH;
                int indexOfY = pointClient.Y / CWData.TAB_HEADER_HEIGHT;

                if (indexOfY != 0 || indexOfX >= (characterCount + 2))
                {
                    // タブヘッダ範囲外
                    return;
                }

                MoveProcessByIndex(indexOfX);
            }
        }

        /// <summary>
        /// ドロップ先インデックス指定でカード移動
        /// mouseDownPanel と mouseDownIndex は必要です。
        /// </summary>
        /// <param name="indexOfX">ドロップ先インデックス</param>
        private void MoveProcessByIndex(int indexOfX)
        {
            if (indexOfX == 0)
            {
                // カード置場ヘッダにドロップ
                if (mouseDownPanel == pnlYadoCard)
                {
                    // カード置場カードをカード置場にドロップしても変わらない 
                    return;
                }
                else if (mouseDownPanel == pnlPartyBag)
                {
                    // 荷物袋カードをカード置場に移動
                    inn.SwapPartyBagCardAndCard(mouseDownIndex, -1);

                }
                else
                {
                    // キャラクタの所持カードをカード置場に移動
                    パネル属性 pnlType = GetPanelTypeNo(mouseDownPanel);

                    // キャラクタの所持カードをカード置場に移動
                    if (!(inn.SwapCharacterCardAndCard(pnlType.CharaNo, pnlType.Tefuda
                        , mouseDownIndex, -1)))
                    {
                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                        return;
                    }
                }

                // カード表示反映
                DisplayCardMove(inn, mouseDownPanel, pnlYadoCard, mouseDownIndex);

            }
            else if (indexOfX == 1)
            {
                // 荷物袋ヘッダにドロップ
                if (mouseDownPanel == pnlYadoCard)
                {
                    // カード置場カードを荷物袋に移動
                    inn.SwapPartyBagCardAndCard(-1, mouseDownIndex);
                }
                else if (mouseDownPanel == pnlPartyBag)
                {
                    // 荷物袋カードを荷物袋にドロップしても変わらない
                    return;
                }
                else
                {
                    // キャラクタの所持カードを荷物袋に移動
                    パネル属性 pnlType = GetPanelTypeNo(mouseDownPanel);

                    // キャラクタの所持カードを荷物袋に移動
                    if (!(inn.SwapCharacterCardAndPartyBagCard(pnlType.CharaNo, pnlType.Tefuda, mouseDownIndex, -1)))
                    {
                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                        return;
                    }
                }

                // カード表示反映
                DisplayCardMove(inn, mouseDownPanel, pnlPartyBag, mouseDownIndex);

            }
            else
            {
                // キャラクタヘッダにドロップ
                // カードをキャラクタに移動
                CardIndexData.Kind kind;
                パネル属性 pnlTypeOld = new パネル属性();

                int droppedCharIndex = indexOfX - 2;

                if (mouseDownPanel == pnlYadoCard)
                {

                    // カード置場のカードを手札に
                    kind = inn.FindIndexedCardData(inn.BankBoxCards[mouseDownIndex]).CardData.CardKind;

                    pnlTypeOld.Tefuda = ChangeKindType(kind);

                    // ダミー
                    pnlTypeOld.CharaNo = 0;

                    // カード置場カードをキャラクタに移動
                    if (!(inn.SwapCharacterCardAndCard(droppedCharIndex, pnlTypeOld.Tefuda, -1, mouseDownIndex)))
                    {
                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                        return;
                    }

                }
                else if (mouseDownPanel == pnlPartyBag)
                {
                    // 荷物袋のカードを手札に
                    kind = inn.FindIndexedCardData(inn.CurrentParty.BagCards[mouseDownIndex]).CardData.CardKind;

                    pnlTypeOld.Tefuda = ChangeKindType(kind);

                    // ダミー
                    pnlTypeOld.CharaNo = 0;

                    // 荷物袋カードをキャラクタに移動
                    if (!(inn.SwapCharacterCardAndPartyBagCard(droppedCharIndex, pnlTypeOld.Tefuda, -1, mouseDownIndex)))
                    {
                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                        return;
                    }

                }
                else
                {
                    // キャラクタの所持カードをキャラクタに移動
                    pnlTypeOld = GetPanelTypeNo(mouseDownPanel);

                    if (pnlTypeOld.CharaNo == droppedCharIndex)
                    {
                        // 同じキャラクタの場合は移動する必要なし
                        return;
                    }

                    inn.SwapCharacterCard(pnlTypeOld.CharaNo, droppedCharIndex, pnlTypeOld.Tefuda, mouseDownIndex, -1);

                }


                // ドロップ先のパーティーPCパネル
                Panel pnlNew = FindPanelFromTypeIndex(droppedCharIndex, pnlTypeOld.Tefuda);

                // カード表示反映
                DisplayCardMove(inn, mouseDownPanel, pnlNew, mouseDownIndex);
            }

        }


        /// <summary>
        /// タブコントロール　クエリコンティニュードロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabYado_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {

            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;
        }

        #endregion

        #region メソッド
        private int GetCharacterPanelCount(Panel pnl)
        {
            return GetCharacterPanelCount(GetPanelTypeNo(pnl));
        }

        private int GetCharacterPanelCount(パネル属性 pnlType)
        {
            CharacterData charaData = inn.CurrentParty.Characters[pnlType.CharaNo];
            if (pnlType.Tefuda == 手札種類.所持スキル)
            {
                return charaData.Skills.Count;
            }
            else if (pnlType.Tefuda == 手札種類.所持アイテム)
            {
                return charaData.Items.Count;
            }
            else if (pnlType.Tefuda == 手札種類.所有召喚獣)
            {
                return charaData.Beasts.Count;
            }
            else
            {
                PartyMaker.CWIErrorMessage("ありえない手札種類です。ExtHandWindow：GetCharacterPanelCount");
                return 0;
            }
        }

        private bool CanDropPanel(InnData inn,Panel oldPnl, int oldIndex ,Panel newPnl)
        {
            手札種類 tefudaOld;
            手札種類 tefudaNew;

            if (inn.CurrentParty != null && inn.CurrentParty.InScenario)
            {
                // カレントパーティーシナリオ進行中
                if (oldPnl == pnlYadoCard || newPnl == pnlYadoCard)
                {
                    // 宿カードからの補充や宿カードに戻すなどの処理は出来ない
                    return false;
                }
            }

            if (oldPnl == pnlYadoCard 
                || oldPnl == pnlPartyBag)
            {
                tefudaOld = GetCardType(oldPnl, oldIndex);
            }
            else
            {
                tefudaOld = GetPanelTypeNo(oldPnl).Tefuda;
            }

            if (oldPnl == newPnl)
            {
                return true;
            }
            else if (newPnl == pnlYadoCard || newPnl == pnlPartyBag)
            {
                return true;
            }
            else
            {
                tefudaNew = GetPanelTypeNo(newPnl).Tefuda;

                if (tefudaOld == tefudaNew)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            
        }

        private HandCard GetCharacterHandCard(パネル属性 pnlType, int index)
        {
            HandCard hc;
            CharacterData charaData = inn.CurrentParty.Characters[pnlType.CharaNo];
            if (pnlType.Tefuda == 手札種類.所持スキル)
            {
                SkillHandCard shc;

                hc = new SkillHandCard(charaData.Skills[index]);

                shc = (SkillHandCard)hc;
                shc.UpdateImage(
                    GameParam.GetCardFacterColor(charaData,
                    shc.widData.BodyFacter, shc.widData.MindFacter),
                    GameParam.GetSkillRemainColor(shc.widData.SkillRemainUseCount,
                    GameParam.GetMaxSkillUseCount(charaData.Level, shc.widData.Level)));

            }
            else if (pnlType.Tefuda == 手札種類.所持アイテム)
            {
                ItemHandCard ihc;

                hc = new ItemHandCard(charaData.Items[index]);

                ihc = (ItemHandCard)hc;
                ihc.UpdateImage(
                    GameParam.GetCardFacterColor(charaData,
                    ihc.widData.BodyFacter, ihc.widData.MindFacter));

            }
            else if (pnlType.Tefuda == 手札種類.所有召喚獣)
            {
                BeastHandCard bhc;

                hc = new BeastHandCard(charaData.Beasts[index]);

                bhc = (BeastHandCard)hc;

                bhc.UpdateImage(
                    GameParam.GetCardFacterColor(charaData,
                    bhc.widData.BodyFacter, bhc.widData.MindFacter));

            }
            else
            {
                PartyMaker.CWIErrorMessage("ありえない手札種類です。GetCharacterHandCard");
                return null;
            }

            return hc;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pnl">カード置場か荷物袋</param>
        /// <param name="index">インデックス</param>
        /// <returns></returns>
        private HandCard GetBagHandCard(Panel pnl, int index)
        {
            IndexCardData icd;
            if (pnl == pnlPartyBag)
            {
                icd = inn.FindIndexedCardData(inn.CurrentParty.BagCards[index]);
                return icd.HandCard;
            }
            else if (pnl == pnlYadoCard)
            {
                icd = inn.FindIndexedCardData(inn.BankBoxCards[index]);
                return icd.HandCard;
            }
            else
            {

                PartyMaker.CWIErrorMessage("ありえないパネル種類です。");
                return null;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="droppedCharIndex"></param>
        /// <param name="tefuda"></param>
        /// <returns></returns>
        private Panel FindPanelFromTypeIndex(int droppedCharIndex,手札種類 tefuda)
        {
            Panel pnlNew;

            switch (droppedCharIndex)
            {
                case 0:
                    switch (tefuda)
                    {
                        case 手札種類.所持スキル:
                            pnlNew = pnlSkill1;
                            break;
                        case 手札種類.所持アイテム:
                            pnlNew = pnlItem1;
                            break;
                        case 手札種類.所有召喚獣:
                            pnlNew = pnlBeast1;
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない手札種類です。FindPanelFromTypeIndex");
                            return null;
                    }
                    break;
                case 1:
                    switch (tefuda)
                    {
                        case 手札種類.所持スキル:
                            pnlNew = pnlSkill2;
                            break;
                        case 手札種類.所持アイテム:
                            pnlNew = pnlItem2;
                            break;
                        case 手札種類.所有召喚獣:
                            pnlNew = pnlBeast2;
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない手札種類です。FindPanelFromTypeIndex");
                            return null;
                    }
                    break;
                case 2:
                    switch (tefuda)
                    {
                        case 手札種類.所持スキル:
                            pnlNew = pnlSkill3;
                            break;
                        case 手札種類.所持アイテム:
                            pnlNew = pnlItem3;
                            break;
                        case 手札種類.所有召喚獣:
                            pnlNew = pnlBeast3;
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない手札種類です。FindPanelFromTypeIndex");
                            return null;
                    }
                    break;
                case 3:
                    switch (tefuda)
                    {
                        case 手札種類.所持スキル:
                            pnlNew = pnlSkill4;
                            break;
                        case 手札種類.所持アイテム:
                            pnlNew = pnlItem4;
                            break;
                        case 手札種類.所有召喚獣:
                            pnlNew = pnlBeast4;
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない手札種類です。FindPanelFromTypeIndex");
                            return null;
                    }
                    break;
                case 4:
                    switch (tefuda)
                    {
                        case 手札種類.所持スキル:
                            pnlNew = pnlSkill5;
                            break;
                        case 手札種類.所持アイテム:
                            pnlNew = pnlItem5;
                            break;
                        case 手札種類.所有召喚獣:
                            pnlNew = pnlBeast5;
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない手札種類です。FindPanelFromTypeIndex");
                            return null;
                    }
                    break;
                case 5:
                    switch (tefuda)
                    {
                        case 手札種類.所持スキル:
                            pnlNew = pnlSkill6;
                            break;
                        case 手札種類.所持アイテム:
                            pnlNew = pnlItem6;
                            break;
                        case 手札種類.所有召喚獣:
                            pnlNew = pnlBeast6;
                            break;
                        default:
                            PartyMaker.CWIErrorMessage("ありえない手札種類です。FindPanelFromTypeIndex");
                            return null;
                    }
                    break;
                default:
                    PartyMaker.CWIErrorMessage("ありえないキャラクタ人数です。FindPanelFromTypeIndex");
                    return null;
            }

            return pnlNew;
        }

        private 手札種類 ChangeKindType(CardIndexData.Kind kind)
        {
            手札種類 tefuda;
            switch (kind)
            {
                case CardIndexData.Kind.skill:
                    tefuda = 手札種類.所持スキル;
                    break;
                case CardIndexData.Kind.item:
                    tefuda = 手札種類.所持アイテム;
                    break;
                case CardIndexData.Kind.beast:
                    tefuda = 手札種類.所有召喚獣;
                    break;
                default:
                    PartyMaker.CWIErrorMessage("ありえない手札種類です。ChangeKindType");
                    return 手札種類.所持スキル;
            }

            return tefuda;
        }

        private 手札種類 GetCardType(Panel pnl, int index)
        {
            if (pnl == pnlPartyBag)
            {
                return ChangeKindType(
                    inn.FindIndexedCardData(inn.CurrentParty.BagCards[index]).CardData.CardKind);
            }
            else if (pnl == pnlYadoCard)
            {
                return ChangeKindType(
                    inn.FindIndexedCardData(inn.BankBoxCards[index]).CardData.CardKind);
            }
            else
            {
                PartyMaker.CWIErrorMessage("ありえないパネルです。GetCardType");
                return 手札種類.所持スキル;

            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pnl"></param>
        /// <returns></returns>
        private パネル属性 GetPanelTypeNo(Panel pnl)
        {
            パネル属性 pnlType = new パネル属性();
            string pnlName = pnl.Name;
            string lastStr = pnlName.Substring(pnlName.Length - 1, 1);
            pnlType.CharaNo = int.Parse(lastStr) - 1;
            string typeStr = pnlName.Substring(3, pnlName.Length - 4);

            if (typeStr == "Skill")
            {
                pnlType.Tefuda = 手札種類.所持スキル;
            }
            else if (typeStr == "Item")
            {
                pnlType.Tefuda = 手札種類.所持アイテム;
            }
            else if (typeStr == "Beast")
            {
                pnlType.Tefuda = 手札種類.所有召喚獣;
            }
            else
            {
                PartyMaker.CWIErrorMessage("ありえない手札種類です。GetPanelTypeNo");
                return null;
            }

            return pnlType;
        }

        /// <summary>
        /// カード枚数/最大所持枚数表示をおこなう
        /// </summary>
        private void SetDispCardCount(パネル属性 pnlType)
        {
            GroupBox grp = new GroupBox();
            string header = "";
            int count = 0;

            CharacterData cd = inn.CurrentParty.Characters[pnlType.CharaNo];
            switch (pnlType.CharaNo)
            {
                case 0:
                    if (pnlType.Tefuda == 手札種類.所持スキル)
                    {
                        grp = grpSkill1;
                        header = "技能カード ";
                        count = cd.Skills.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所持アイテム)
                    {
                        grp = grpItem1;
                        header = "アイテムカード ";
                        count = cd.Items.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所有召喚獣)
                    {
                        grp = grpBeast1;
                        header = "召喚獣カード ";
                        count = cd.Beasts.Count;
                    }
                    break;
                case 1:
                    if (pnlType.Tefuda == 手札種類.所持スキル)
                    {
                        grp = grpSkill2;
                        header = "技能カード ";
                        count = cd.Skills.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所持アイテム)
                    {
                        grp = grpItem2;
                        header = "アイテムカード ";
                        count = cd.Items.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所有召喚獣)
                    {
                        grp = grpBeast2;
                        header = "召喚獣カード ";
                        count = cd.Beasts.Count;
                    }
                    break;
                case 2:
                    if (pnlType.Tefuda == 手札種類.所持スキル)
                    {
                        grp = grpSkill3;
                        header = "技能カード ";
                        count = cd.Skills.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所持アイテム)
                    {
                        grp = grpItem3;
                        header = "アイテムカード ";
                        count = cd.Items.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所有召喚獣)
                    {
                        grp = grpBeast3;
                        header = "召喚獣カード ";
                        count = cd.Beasts.Count;
                    }
                    break;
                case 3:
                    if (pnlType.Tefuda == 手札種類.所持スキル)
                    {
                        grp = grpSkill4;
                        header = "技能カード ";
                        count = cd.Skills.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所持アイテム)
                    {
                        grp = grpItem4;
                        header = "アイテムカード ";
                        count = cd.Items.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所有召喚獣)
                    {
                        grp = grpBeast4;
                        header = "召喚獣カード ";
                        count = cd.Beasts.Count;
                    }
                    break;
                case 4:
                    if (pnlType.Tefuda == 手札種類.所持スキル)
                    {
                        grp = grpSkill5;
                        header = "技能カード ";
                        count = cd.Skills.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所持アイテム)
                    {
                        grp = grpItem5;
                        header = "アイテムカード ";
                        count = cd.Items.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所有召喚獣)
                    {
                        grp = grpBeast5;
                        header = "召喚獣カード ";
                        count = cd.Beasts.Count;
                    }
                    break;
                case 5:
                    if (pnlType.Tefuda == 手札種類.所持スキル)
                    {
                        grp = grpSkill6;
                        header = "技能カード ";
                        count = cd.Skills.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所持アイテム)
                    {
                        grp = grpItem6;
                        header = "アイテムカード ";
                        count = cd.Items.Count;
                    }
                    else if (pnlType.Tefuda == 手札種類.所有召喚獣)
                    {
                        grp = grpBeast6;
                        header = "召喚獣カード ";
                        count = cd.Beasts.Count;
                    }
                    break;
            }

            grp.Text =
                header + count + "/" + cd.GetMaxHandCard(pnlType.Tefuda);

        }

        #endregion

        #region ドラッグ時パネル表示切替用
        private void pnl_DragLeave(object sender, EventArgs e)
        {

            Point pointClient = tabYado.PointToClient(Cursor.Position);
            int indexOfX = pointClient.X / CWData.TAB_HEADER_WIDTH;
            int indexOfY = pointClient.Y / CWData.TAB_HEADER_HEIGHT;
            if (indexOfY <= 2)
            {
                tabYado.SelectedIndex = indexOfX;
                tabYado.Update();
            }
        }

        #endregion

    }

    public class パネル属性
    {
        public int CharaNo;
        public 手札種類 Tefuda;
    }
}