﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{

    public partial class FunctionDlg : Form
    {
        #region メンバ

        private Form _owner = null;
        private PartyMaker _pm = null;
        private InnData inn = null;
        private PartyData currentParty = null;
        ToolTip toolTip1 = null;

        private int selectedButtonID = 0;
        public int SelectedButtonID
        {
            set
            {
                selectedButtonID = value;
            }
            get
            {
                return selectedButtonID;
            }
        }

        public PCソート種類 SelectedSortType = PCソート種類.名前昇順;

        private SelectedFont selectedfont = SelectedFont.ＭＳ明朝;
        public SelectedFont SelectFont
        {
            set
            {
                selectedfont = value;
            }
            get
            {
                return selectedfont;
            }
        }

        public Ikinari SelectedMoveTo = Ikinari.交易都市リューン;
        public string SelectedFolder = "";

        public int SelectedMoveScenaIndex = -1;
        ToolTip tip;
        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FunctionDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FunctionDlg(Form owner, InnData inn)
        {
            InitializeComponent();

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            this.inn = inn;
            this.currentParty = inn.CurrentParty;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void FunctionDlg_Load(object sender, EventArgs e)
        {
            
            // チェック反映
            _pm = _owner as PartyMaker;
            if (_pm != null)
            {
                this.chkAutoBackup.Checked = _pm.isAutoBackup;
                this.chkResizeBmp.Checked = _pm.canResizeBmp;
                this.chkCABScenario.Checked = _pm.isIncludedCAB;
                this.chkPartyName.Checked = _pm.noChangePTName;
                this.chkAllBreak.Checked = _pm.allPartyBreakToYadoCard;
                this.chkEndWithVariant.Checked = _pm.endBeforeExecVariant;
                this.chkFullScreen.Checked = _pm.fullScreenWhenExecVariant;
                this.chkVariantMp3.Checked = _pm.variantMp3;
                //this.chkExPatch.Checked = _pm.variantPatch;
                this.txtVariant.Text = _pm.execVariant;
            }

            chkResizeBmp_CheckedChanged(null, null);
            chkEndWithVariant_CheckedChanged(null, null);

            if (currentParty == null)
            {
                btnF9.Enabled = false;
                btnIkinari.Enabled = false;
                btnIkinariScena.Enabled = false;
            }
            else if (currentParty.InScenario)
            {
                btnF9.Enabled = true;
                btnIkinari.Enabled = false;
                btnIkinariScena.Enabled = false;
            }
            else
            {
                btnF9.Enabled = false;
                if (inn.IsDebugInn)
                {
                    btnIkinari.Enabled = false;
                    btnIkinariScena.Enabled = false;
                }
                else
                {
                    btnIkinari.Enabled = true;
                    btnIkinariScena.Enabled = true;
                }
            }


            // ツールチップ
            this.components = new System.ComponentModel.Container();
            toolTip1 = new ToolTip(this.components);
            toolTip1.UseFading = true;
            toolTip1.InitialDelay = 100;
            toolTip1.ShowAlways = false;
            toolTip1.Active = true;

            toolTip1.SetToolTip(this.btnBmpGet, "ＰＣ画像をFaceフォルダに収集します。\r\nご利用は個人利用の範囲内にするのが無難です。");
            toolTip1.SetToolTip(this.btnF9, "現在選択中でシナリオ進行中のパーティーを緊急避難します。");
            toolTip1.SetToolTip(this.btnSearch, "シナリオを検索します。");
            toolTip1.SetToolTip(this.btnAllBreakUp, "全パーティーを解散します。");
            toolTip1.SetToolTip(this.btnNewParty, "新規パーティーを作成します。");
            toolTip1.SetToolTip(this.btnMakeShelf, "棚パーティーに箱ＰＣを作成し、カードを格納します。");
            toolTip1.SetToolTip(this.btnItemFromShelf, "棚パーティーを解散し、カードをカード置場に戻します。");
            toolTip1.SetToolTip(this.btnPCSort, "待機ＰＣを特定の条件でソートします。（宿には反映されません。）");
            toolTip1.SetToolTip(this.btnMusic, "Midiフォルダの音楽を表示します。");
            toolTip1.SetToolTip(this.btnBGM, "デフォルトの.midファイルの中身を\r\n別のファイルで置き換えます。");
            toolTip1.SetToolTip(this.btnFontVariant, "フォント名を置き換えて別のフォントで表示される\r\nCardWirthエンジンを作成します。");
            toolTip1.SetToolTip(this.btnIkinari, "いきなりシナリオの途中から開始します。");
            toolTip1.SetToolTip(this.btnIkinariScena, "いきなりシナリオにArea9999を追加し、\r\nそこから開始します。\r\nシナリオからArea9999.widを削除すれば元に戻ります。");

            toolTip1.SetToolTip(this.pnlBitmap, "CardWirth画像用のビットマップに変換します。\r\nファイルやフォルダをドラッグ＆ドロップしてください。");
            toolTip1.SetToolTip(this.txtVariant, "CardWirth起動ボタンやいきなりシナリオ機能などで起動するエンジンを指定してください。\r\nクリックすると選択ダイアログが表示されます。");
        }

        private void InitToolTip()
        {
            tip = new ToolTip();
            tip.SetToolTip(chkVariantMp3, (string)chkVariantMp3.Tag);
        }

        private void Control_MouseHover(object sender, EventArgs e)
        {
        }

        private void Control_MouseLeave(object sender, EventArgs e)
        {
        }


        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FunctionDlg_Shown(object sender, EventArgs e)
        {
            for (int i = 0; i <= (int)PCソート種類.クーポン; i++)
            {
                cmbPCSort.Items.Add(((PCソート種類)i).ToString());
            }
            cmbPCSort.SelectedIndex = 0;

            for (int i = 0; i <= (int)SelectedFont.あくびん; i++)
            {
                cmbFontVariant.Items.Add(((SelectedFont)i).ToString());
            }
            cmbFontVariant.SelectedIndex = 0;


            for (int i = 0; i <= (int)Ikinari.ジーベック銀行; i++)
            {
                //_E_は!に置き換えます。
                // _H_ は - に置き換えます。
                // _F_は！に置き換えます。
                cmbIkinari.Items.Add(((Ikinari)i).ToString()
                    .Replace("_E_", "!").Replace("_H_", "-").Replace("_C_", "　")
                    .Replace("_K_", "（").Replace("_T_", "）").Replace("_G_", "―")
                    .Replace("_S_", "#").Replace("_M_", "。").Replace("_B_", "\"")
                    .Replace("_L_", "「").Replace("_N_", "」").Replace("_A_", "＆").Replace("_Q_", "？")
                    .Replace("_F_", "！").Replace("_Z_", "=").Replace("_Y_", " ").Replace("_D_", "・")
                    .Replace("_I_","5"));
            }
            cmbIkinari.SelectedIndex = 0;

            cmbFace.Items.Add("png");
            cmbFace.Items.Add("bmp");
            cmbFace.SelectedIndex = 0;
        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 0;
            this.Close();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FunctionDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            XmlStateChange();
        }

        #endregion

        #region クリックイベント
        
        private void btnCard_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 1;
            this.Close();
        }
 
        private void btnMove_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 2;
            this.Close();
        }

        private void btnPartyMove_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 3;
            this.Close();
        }

        private void btnF9_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 4;
            this.Close();
        }

        private void btnBmpGet_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 5;
            this.Close();
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 6;
            this.Close();
        }

        private void btnAllBreakUp_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 7;
            this.Close();
        }

        private void btnNewParty_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 8;
            this.Close();

        }

        private void btnMakeShelf_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 9;
            this.Close();
        }

        private void btnItemFromShelf_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 10;
            this.Close();
        }

        private void btnPCSort_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 11;
            SelectedSortType = (PCソート種類)cmbPCSort.SelectedIndex;
            this.Close();
        }


        private void btnMusic_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 13;
            this.Close();
        }

        private void btnBGM_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 14;
            this.Close();
        }

        private void btnFontVariant_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 15;
            SelectFont = (SelectedFont)cmbFontVariant.SelectedIndex;

            this.Close();
        }

        private void btnIkinari_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 16;
            SelectedMoveTo = (Ikinari)cmbIkinari.SelectedIndex;
            this.Close();
        }

        /// <summary>
        /// 起動バリアント指定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtVariant_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = this.txtVariant.Text;
            if (DialogResult.OK == openFileDialog1.ShowDialog())
            {
                this.txtVariant.Text = openFileDialog1.FileName;
            }
        }

        private void btnIkinariScena_Click(object sender, EventArgs e)
        {
            using (ScenarioSearchDlg ssd = new ScenarioSearchDlg(this, this.inn, _pm))
            {
                ssd.ScenarioSelect = true;
                if (ssd.ShowDialog() != DialogResult.OK)
                {
                    return;
                }

                this.SelectedFolder = ssd.SelectedFolder; 
            }
            SelectedButtonID = 17;
            this.Close();
        }
        #endregion

        #region ドラッグドロップ
        /// <summary>
        /// ビットマップ変換パネル　ドラッグエンター
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlBitmap_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがFileDropか調べ、、
            //そうであればドロップ効果をCopyにする
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.Copy;
            else
                //DispCard型でなければ受け入れない
                e.Effect = DragDropEffects.None;   
        }

        /// <summary>
        /// ビットマップ変換パネル　ドラッグドロップ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlBitmap_DragDrop(object sender, DragEventArgs e)
        {
            string[] ss = 
                (string[])e.Data.GetData(DataFormats.FileDrop, false);

            //メッセージボックスが真ん中に来ないのでコメントアウト
            switch (ss.Length)
            {
                case 0:
                    //MessageDlg.ShowInfoDialog("ファイルかフォルダをドラッグ＆ドロップしてください。");
                    return;
                default:
                    if (Directory.Exists(ss[0]))
                    {
                        // Directory
                        DirectoryInfo di = new DirectoryInfo(ss[0]);
                        foreach (FileInfo fi in di.GetFiles("*.*", SearchOption.AllDirectories))
                        {
                            string fullStr = fi.FullName;
                            ImageFileToBitmap(fullStr);
                        }
                    }
                    else
                    {
                        // Files
                        foreach (string s in ss)
                        {
                            ImageFileToBitmap(s);
                        }
                    }
                    break;
            }
            //MessageDlg.ShowInfoDialog("終了しました。");

        }

        /// <summary>
        /// 画像ファイルをビットマップ(かpng)に変換
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ImageFileToBitmap(string fullPath)
        {
            if (fullPath.EndsWith(".jpg")
                               || fullPath.EndsWith(".jpeg")
                               || fullPath.EndsWith(".png")
                               || fullPath.EndsWith(".gif")
                               || fullPath.EndsWith(".bmp")
                               || fullPath.EndsWith(".JPG")
                               || fullPath.EndsWith(".JPEG")
                               || fullPath.EndsWith(".PNG")
                               || fullPath.EndsWith(".GIF")
                               || fullPath.EndsWith(".BMP")
                               )
            {
                Bitmap bmp = new Bitmap(fullPath);
                int pixelSize = Image.GetPixelFormatSize(bmp.PixelFormat);

                string fileName = fullPath.Replace(".jpg", FilePath.ExtBitmap)
                        .Replace(".jpeg", FilePath.ExtBitmap)
                        .Replace(".png", FilePath.ExtBitmap)
                        .Replace(".gif", FilePath.ExtBitmap)
                        .Replace(".JPG", FilePath.ExtBitmap)
                        .Replace(".JPEG", FilePath.ExtBitmap)
                        .Replace(".PNG", FilePath.ExtBitmap)
                        .Replace(".GIF", FilePath.ExtBitmap)
                        .Replace(".BMP", FilePath.ExtBitmap);

                // png形式の場合
                string extName = FilePath.ExtBitmap;
                if (cmbFace.SelectedIndex == 0)
                {
                    fileName.Replace(FilePath.ExtBitmap, FilePath.ExtPng);
                    extName = FilePath.ExtPng;
                }

                //すでに同じ名前のファイルがある場合
                uint num = 1;
                string basename = fileName.Substring(0, fileName.Length - 4);
                string tempname = basename;

                while (File.Exists(tempname + extName))
                {
                    num++;
                    tempname = basename + "(" + num.ToString() + ")";
                }

                //重ならないように名前を変更
                fileName = tempname + extName;
                Graphics g;

                if (this.chkResizeBmp.Checked)
                {
                    int marginWidth = (int)numMarginWidth.Value;
                    int marginHeight = (int)numMarginHeight.Value;
                    // サイズを74x94に変えたい場合
                    if (bmp.Width > marginWidth * 2 &&
                        bmp.Height > marginHeight * 2)
                    {
                        // 余白を取っても残る場合
                        Bitmap dest = null;
                        if (numVertical.Value == 1 && numHorizontal.Value == 1)
                        {
                            // 余白を追加したのでそれを考慮に入れる
                            if (marginWidth + marginHeight == 0)
                            {
                                // 余白がない場合(単なる拡大・縮小)
                                dest = new Bitmap(bmp, CWData.Bitmap_Width, CWData.Bitmap_Height);
                            }
                            else
                            {
                                // 高品質版
                                dest = new Bitmap(CWData.Bitmap_Width, CWData.Bitmap_Height);

                                g = Graphics.FromImage(dest);
                                g.InterpolationMode = InterpolationMode.HighQualityBicubic;

                                // 余白あり
                                Rectangle rect = new Rectangle(marginWidth,
                                                               marginHeight,
                                                               (bmp.Width - marginWidth * 2), 
                                                               (bmp.Height - marginHeight * 2));

                                Bitmap src = bmp.Clone(rect, PixelFormat.Format32bppArgb);
                                g.DrawImage(src, 0, 0, CWData.Bitmap_Width, CWData.Bitmap_Height);

                            }

                            if (extName == FilePath.ExtBitmap)
                            {
                                dest.Save(fileName, ImageFormat.Bmp);
                            }
                            else
                            {
                                dest.MakeTransparent();
                                dest.Save(fileName, ImageFormat.Png);
                            }

                            dest.Dispose();

                            // なぜかbiSizeImageが0になるので修正
                            Util3.FixBitmap(fileName);
                        }
                        else
                        {
                            // 1 * 1 以外の分割をしたい場合
                            int horizontal = (int)numHorizontal.Value;
                            int vertical = (int)numVertical.Value;

                            int bmpWidth = (bmp.Width - marginWidth * 2) / horizontal;
                            int bmpHeight = (bmp.Height - marginHeight * 2) / vertical;

                            for (int div_y = 0; div_y < vertical; div_y++)
                            {
                                for (int div_x = 0; div_x < horizontal; div_x++)
                                {

                                    //重ならないように名前を変更
                                    fileName = tempname + "_" + div_y.ToString("000") +
                                        "_" + div_x.ToString("000") + extName;

                                    Rectangle rect = new Rectangle(marginWidth + bmpWidth * div_x,
                                                                   marginHeight + bmpHeight * div_y,
                                                                   bmpWidth, bmpHeight);

                                    Bitmap src = bmp.Clone(rect, PixelFormat.Format32bppArgb);

                                    // 高品質版
                                    dest = new Bitmap(CWData.Bitmap_Width, CWData.Bitmap_Height);

                                    g = Graphics.FromImage(dest);

                                    g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                                    g.DrawImage(src, 0, 0, CWData.Bitmap_Width, CWData.Bitmap_Height);

                                   if (extName == FilePath.ExtBitmap)
                                    {
                                        dest.Save(fileName, ImageFormat.Bmp);
                                    }
                                    else
                                    {
                                        dest.MakeTransparent();
                                        dest.Save(fileName, ImageFormat.Png);
                                    }

                                    dest.Dispose();

                                    // なぜかbiSizeImageが0になるので修正
                                    Util3.FixBitmap(fileName);
                                }
                            }

                        }
                    }
                }
                else
                {
                    //通常仕様
                    if (extName == FilePath.ExtBitmap)
                    {
                        bmp.Save(fileName, ImageFormat.Bmp);
                    }
                    else
                    {
                        bmp.MakeTransparent();
                        bmp.Save(fileName, ImageFormat.Png);
                    }

                    Util3.FixBitmap(fileName);
                }
                bmp.Dispose();
            }
        }

        /// <summary>
        /// 値変更時イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkResizeBmp_CheckedChanged(object sender, EventArgs e)
        {
            bool isTrue = chkResizeBmp.Checked;
            
            numHorizontal.Enabled = isTrue;
            numVertical.Enabled = isTrue;
            numMarginWidth.Enabled = isTrue;
            numMarginHeight.Enabled = isTrue;
        }
        private void chkEndWithVariant_CheckedChanged(object sender, EventArgs e)
        {
            bool isTrue = chkEndWithVariant.Checked;
            if (isTrue)
            {
                // エンジン起動時はPM終了
                chkFullScreen.Checked = false;
                chkFullScreen.Enabled = false;
            }
            else
            {
                chkFullScreen.Enabled = true;
            }
        }
        #endregion
        
        #region XML状態反映
        private void XmlStateChange()
        {
            if (_pm != null)
            {
                _pm.isAutoBackup = this.chkAutoBackup.Checked;
                _pm.canResizeBmp = this.chkResizeBmp.Checked;
                _pm.isIncludedCAB = this.chkCABScenario.Checked;
                _pm.noChangePTName = this.chkPartyName.Checked;
                _pm.allPartyBreakToYadoCard = this.chkAllBreak.Checked;
                _pm.endBeforeExecVariant = this.chkEndWithVariant.Checked;
                _pm.fullScreenWhenExecVariant = this.chkFullScreen.Checked;
                _pm.variantMp3 = this.chkVariantMp3.Checked;
                //_pm.variantPatch = this.chkExPatch.Checked;
                _pm.execVariant = this.txtVariant.Text;
            }
        }

        #endregion


    }
}