﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Runtime.Serialization.Formatters.Binary;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{

    public partial class FunctionDlg2 : Form
    {
        #region メンバ

        private Form _owner = null;
        private PartyMaker _pm = null;
        private InnData inn = null;
        private PartyData currentParty = null;

        private int selectedButtonID = 0;
        public int SelectedButtonID
        {
            set
            {
                selectedButtonID = value;
            }
            get
            {
                return selectedButtonID;
            }
        }

        public PCソート種類 SelectedSortType = PCソート種類.名前昇順;

        private SelectedFont selectedfont = SelectedFont.ＭＳ明朝;
        public SelectedFont SelectFont
        {
            set
            {
                selectedfont = value;
            }
            get
            {
                return selectedfont;
            }
        }

        public Ikinari SelectedMoveTo = Ikinari.交易都市リューン;

        ToolTip toolTip1 = null;
        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FunctionDlg2()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public FunctionDlg2(Form owner, InnData inn)
        {
            InitializeComponent();

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            this.inn = inn;
            this.currentParty = inn.CurrentParty;

            _pm = _owner as PartyMaker;

            // 状態反映
            if (_pm != null)
            {
                this.chkPartyItem.Checked = _pm.createFullPartyWithYadoCard;
                this.chkSimple.Checked = _pm.isSimpleCardWindow;
                this.chkParam.Checked = _pm.paramOption;
                this.chkMute.Checked = _pm.soundMute;
                this.chkYadoSimple.Checked = _pm.yadoSimple;
            }

            this.cmbPTToChara.Items.Clear();
            this.cmbPTToChara.Items.Add("パーティー離脱");
            this.cmbPTToChara.Items.Add("パーティー解散");
            this.cmbPTToChara.SelectedIndex = 0;

            EnableChange();
        }

        private void EnableChange()
        {
            if (inn.Parties.Count - (inn.CurrentParty != null ? 1 : 0) > 0)
            {
                // カレント以外のパーティーがいる場合
                this.cmbPTToChara.Enabled = true;
                this.btnPTToChara.Enabled = true;
            }
            else
            {
                this.cmbPTToChara.Enabled = false;
                this.btnPTToChara.Enabled = false;
            }

            if (inn.CurrentParty == null || inn.CurrentParty.InScenario)
            {
                // カレントパーティーがあってシナリオ中でない場合のみTrue
                this.btnStarDust.Enabled = false;
                this.btnStarDustToItem.Enabled = false;
            }
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void FunctionDlg2_Load(object sender, EventArgs e)
        {
            btnPartyMove.Enabled = inn.CurrentParty != null &&
                inn.CurrentParty.InScenario == false;

            // ツールチップ
            this.components = new System.ComponentModel.Container();
            toolTip1 = new ToolTip(this.components);
            toolTip1.UseFading = true;
            toolTip1.InitialDelay = 100;
            toolTip1.ShowAlways = false;
            toolTip1.Active = true;

            toolTip1.SetToolTip(this.btnPartinize, "待機ＰＣを一括でパーティー化します。");
            toolTip1.SetToolTip(this.btnCharaCoupon, "２ＰＣを選択してクーポン操作します。");
            toolTip1.SetToolTip(this.btnPTToChara, "パーティーを離脱もしくは解散する対象キャラクタを画像選択します。");
            toolTip1.SetToolTip(this.btnFaceFix, "Faceフォルダのビットマップ画像にサイズが入っていなくて\r\nエンジンエラーが起こる場合に実行してください。");
            toolTip1.SetToolTip(this.btnPartyList, "カレントパーティー装備一覧を表示します。");

            toolTip1.SetToolTip(this.btnPartyMember, "パーティーを組んでいるPCの一覧を表示します。");
            toolTip1.SetToolTip(this.btnCardGet, "全待機PCのカードをカード置場へ移動します。");

            toolTip1.SetToolTip(this.btnCard, "宿間でカードの受け渡しをします。");
            toolTip1.SetToolTip(this.btnMove, "宿間でキャラクタ移動をします。");
            toolTip1.SetToolTip(this.btnPartyMove, "選択中のパーティーを他宿に移動します。");
            
            toolTip1.SetToolTip(this.btnItemBank, "ItemをGossip化する機能を実行します。");
            toolTip1.SetToolTip(this.btnGossipToItem, "GossipをItem化する機能を実行します。");
            toolTip1.SetToolTip(this.btnStarDust, "アイテムをStarDust方式で格納する機能を実行します。");
            toolTip1.SetToolTip(this.btnStarDustToItem, "StarDust方式で格納したアイテムをアイテムに戻す機能を実行します。");
            toolTip1.SetToolTip(this.btnExtPcBmp, "指定した性別・年齢と背景色でPC画像を３つ選択し拡張ＰＣ画像を作成する機能を実行します。");

            toolTip1.SetToolTip(this.chkParam, "？？");

            cmbSexAge.Items.Clear();
            cmbSexAge.Items.Add("Male-CHD");
            cmbSexAge.Items.Add("Male-YNG");
            cmbSexAge.Items.Add("Male-ADT");
            cmbSexAge.Items.Add("Male-OLD");
            cmbSexAge.Items.Add("Male");
            cmbSexAge.Items.Add("Female-CHD");
            cmbSexAge.Items.Add("Female-YNG");
            cmbSexAge.Items.Add("Female-ADT");
            cmbSexAge.Items.Add("Female-OLD");
            cmbSexAge.Items.Add("Female");
            cmbSexAge.Items.Add("Common-CHD");
            cmbSexAge.Items.Add("Common-YNG");
            cmbSexAge.Items.Add("Common-ADT");
            cmbSexAge.Items.Add("Common-OLD");
            cmbSexAge.Items.Add("Common");
            cmbSexAge.SelectedIndex = 0;

            colorDialog1.Color = Color.Fuchsia;
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FunctionDlg2_Shown(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 0;
            this.Close();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FunctionDlg2_FormClosing(object sender, FormClosingEventArgs e)
        {
            XmlStateChange();
        }

        #endregion

        #region クリックイベント

        /// <summary>
        /// パーティー化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartinize_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 1;
            this.Close();
        }

        /// <summary>
        /// キャラクーポン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCharaCoupon_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 2;
            this.Close();
        }

        /// <summary>
        /// パーティー内キャラ選択操作
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPTToChara_Click(object sender, EventArgs e)
        {
            List<PartyData> partyList = new List<PartyData>(inn.Parties);
            if (inn.CurrentParty != null)
            {
                // カレントパーティーは対象外
                partyList.Remove(inn.CurrentParty);
            }

            int pIndex = 0;
            List<int> indexList = new List<int>();
            List<CharacterData> charaList = new List<CharacterData>();
            foreach (PartyData pd in partyList)
            {
                charaList.AddRange(pd.Characters);

                for (int i = 0; i < pd.Characters.Count; i++)
                {
                    indexList.Add(pIndex);
                }

                pIndex++;
            }

            using (BmpSelectDlg selector = new BmpSelectDlg(charaList))
            {
                if (selector.ShowDialog() == DialogResult.OK)
                {
                    PartyData party = partyList[indexList[selector.SelectedIndex]];
                    CharacterData chara = charaList[selector.SelectedIndex];

                    int count = 0;

                    if (!party.Characters.Contains(chara))
                    {
                        return;
                    }

                    switch (this.cmbPTToChara.SelectedIndex)
                    {
                        case 0:
                            // パーティー離脱
                            count = 1;

                            if (party.Characters.Count == 1)
                            {
                                // パーティー解散
                                inn.BreakOtherParty(party);
                            }
                            else
                            {
                                // パーティー離脱
                                inn.RemoveOtherPartyChara(party, chara);
                            }

                            // パーティーメーカーの表示反映
                            _pm.PinPointAddCharacter(count);
                            break;
                        case 1:
                            // パーティー解散
                            count = party.Characters.Count;

                            inn.BreakOtherParty(party);

                            // パーティーメーカーの表示反映
                            _pm.PinPointAddCharacter(count);

                            break;
                        default:
                            break;
                    }

                    EnableChange();

                }
            }
        }

        /// <summary>
        /// Face画像修正ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFaceFix_Click(object sender, EventArgs e)
        {
            string folder = "";
            for (int i = 0; i <= (int)Sex.Common; i++)
            {
                // Male,Female,Common
                for (int j = 0; j <= (int)Age.Old; j++)
                {
                    folder = FilePath.GetDataFaceFolder((Sex)i, (Age)j);

                    // Child, Young, Adult, Old
                    foreach (string filename
                        in Directory.GetFiles(folder
                        ,"*.bmp"
                        ,SearchOption.TopDirectoryOnly))
                    {
                        Util3.FixBitmap(filename);
                    }
                }
            }
            
            // Common
            folder = FilePath.GetDataFaceFolder(Sex.Male, Age.Common);

            foreach (string filename
                in Directory.GetFiles(folder
                ,"*.bmp"
                ,SearchOption.TopDirectoryOnly))
            {
                Util3.FixBitmap(filename);
            }

            MessageDlg.ShowInfoDialog("終了しました。");
        }

        /// <summary>
        /// パーティーデータ一覧　クリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyList_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 3;
            this.Close();
        }

        /// <summary>
        /// パーティーメンバ一覧　クリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyMember_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 7;
            this.Close();

        }

        /// <summary>
        /// カード狩り
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCardGet_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 8;
            this.Close();
        }

        /// <summary>
        /// 他宿とのカード受け渡し
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCard_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 4;
            this.Close();
        }

        /// <summary>
        /// 他宿とのキャラクタ移動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMove_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 5;
            this.Close();
        }

        /// <summary>
        /// カレントパーティー移動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyMove_Click(object sender, EventArgs e)
        {
            SelectedButtonID = 6;
            this.Close();
        }

        /// <summary>
        /// ItemをGossip化する機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnItemBank_Click(object sender, EventArgs e)
        {
            ItemToGossip();
        }

        /// <summary>
        /// GossipをItem化する機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGossipToItem_Click(object sender, EventArgs e)
        {
            GossipToItem();
        }

        /// <summary>
        /// アイテムをStarDust方式で格納する機能
        /// </summary>
        private void btnStarDust_Click(object sender, EventArgs e)
        {
            ItemToStarDust();
        }

        /// <summary>
        /// StarDust方式で格納したアイテムをアイテムに戻す機能
        /// </summary>
        private void btnStarDustToItem_Click(object sender, EventArgs e)
        {
            StarDustToItem();
        }

        /// <summary>
        /// アイテムをStarDust方式で格納
        /// </summary>
        private void ItemToStarDust()
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "荷物袋から取り出して\r\n" +
            "StarDust方式で格納します。よろしいですか？"))
            {

                InnData.StarDustState state = inn.CheckStarDust();
                if (state == InnData.StarDustState.None)
                {
                    MessageDlg.ShowInfoDialog("StarDustを一度もプレイしていません。");
                    return;
                }

                if (state == InnData.StarDustState.Coupon)
                {
                    int lotusCount = inn.CountLotus();
                    if (lotusCount == 0)
                    {
                        MessageDlg.ShowInfoDialog("管理情報：クーポンですが、カレントパーティーに\r\nロータスが存在しません。");
                        return;
                    }
                    else if (lotusCount > 1)
                    {
                        MessageDlg.ShowInfoDialog("管理情報：クーポンでカレントパーティーに\r\nロータスが２人以上います。");
                        return;
                    }
                }

                inn.ItemToStarDust();

                MessageDlg.ShowInfoDialog("終了しました。");
            }
        }

        /// <summary>
        /// StarDust方式で格納したアイテムをアイテムに戻す
        /// </summary>
        private void StarDustToItem()
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "StarDust方式で格納したアイテムを\r\n" +
            "荷物袋に戻します。よろしいですか？"))
            {
                InnData.StarDustState state = inn.CheckStarDust();
                if (state == InnData.StarDustState.None)
                {
                    MessageDlg.ShowInfoDialog("StarDustを一度もプレイしていません。");
                    return;
                }

                if (state == InnData.StarDustState.Coupon)
                {
                    int lotusCount = inn.CountLotus();
                    if (lotusCount == 0)
                    {
                        MessageDlg.ShowInfoDialog("管理情報：クーポンですが、カレントパーティーに\r\nロータスが存在しません。");
                        return;
                    }
                    else if (lotusCount > 1)
                    {
                        MessageDlg.ShowInfoDialog("管理情報：クーポンでカレントパーティーに\r\nロータスが２人以上います。");
                        return;
                    }
                }

                inn.StarDustToItem();

                MessageDlg.ShowInfoDialog("終了しました。");
            }
        }

        /// <summary>
        /// アイテムをゴシップ化
        /// </summary>
        private void ItemToGossip()
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "カード置場と荷物袋から取り出して\r\n" +
            "ゴシップ化します。よろしいですか？"))
            {
                inn.ItemToGossip();

                MessageDlg.ShowInfoDialog("終了しました。");
            }
        }

        /// <summary>
        /// アイテムゴシップをアイテムに戻す
        /// </summary>
        private void GossipToItem()
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "ゴシップ化したアイテムを\r\n" +
            "カード置場に戻します。よろしいですか？"))
            {
                inn.GossipToItem();

                MessageDlg.ShowInfoDialog("終了しました。");
            }
        }

        List<string> bmpList = new List<string>();
        /// <summary>
        /// ｸﾞﾘｭｴﾙｯ！のようなＰＣ画像を作成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExtPcBmp_Click(object sender, EventArgs e)
        {
            colorDialog1.ShowHelp = false;
            colorDialog1.AnyColor = true;

            //int makeHeight = 580;

            int makeWidth = CWData.Bitmap_Width * 5;
            int makeHeight = CWData.Bitmap_Height * 5;

            DialogResult res = colorDialog1.ShowDialog();
            if (res == DialogResult.OK)
            {
                string folder = FilePath.GetDataFaceFolder(cmbSexAge.SelectedItem.ToString());
                bmpList.Clear();
                bmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskBitmap));
                bmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskPng));
                bmpList.AddRange(Directory.GetFiles(folder, FilePath.MaskGif));

                Color bkColor = colorDialog1.Color;
                Bitmap makeBmp = new Bitmap(makeWidth, makeHeight, PixelFormat.Format32bppArgb);

                var array = Enumerable.Repeat<int>(bkColor.ToArgb(), makeWidth * makeHeight);

                if (bmpList.Count > 0)
                {
                    string firstBmpName = "";
                    using (BmpSelectDlg bsd = new BmpSelectDlg(bmpList, "通常表示されるＰＣ画像を選択してください。"))
                    {
                        res = bsd.ShowDialog();
                        if (res != DialogResult.OK) return;

                        firstBmpName = bmpList[bsd.SelectedIndex];

                    }

                    string nextBmpName = "";
                    using (BmpSelectDlg bsd = new BmpSelectDlg(bmpList, "詳細表示で表示されるＰＣ画像を選択してください。"))
                    {
                        res = bsd.ShowDialog();
                        if (res != DialogResult.OK) return;

                        nextBmpName = bmpList[bsd.SelectedIndex];

                    }

                    string lastBmpName = "";
                    using (BmpSelectDlg bsd = new BmpSelectDlg(bmpList, "CWEのキャストで表示されるＰＣ画像を選択してください。"))
                    {
                        res = bsd.ShowDialog();
                        if (res != DialogResult.OK) return;

                        lastBmpName = bmpList[bsd.SelectedIndex];

                    }

                    int nextHeight = 0;
                    ////int firstHeight = makeHeight / 2 - CWData.Bitmap_Height / 2;
                    ////int lastHeight = makeHeight - CWData.Bitmap_Height;

                    int firstHeight = CWData.Bitmap_Height * 2;
                    int lastHeight = CWData.Bitmap_Height * 4;

                    int nextWidth = 0;
                    int firstWidth = CWData.Bitmap_Width * 2;
                    int lastWidth = CWData.Bitmap_Width * 2;


                    Bitmap nextBmp = new Bitmap(nextBmpName);
                    Bitmap firstBmp = new Bitmap(firstBmpName);
                    Bitmap lastBmp = new Bitmap(lastBmpName);
                    Color nextTransColor = nextBmp.GetPixel(0, 0);
                    Color firstTransColor = firstBmp.GetPixel(0, 0);
                    Color lastTransColor = lastBmp.GetPixel(0, 0);


                    Brush bkBrush = new SolidBrush(bkColor);
                    Pen bkPen = new Pen(bkColor);

                    //図形を描画する
                    
                    Graphics g = Graphics.FromImage(makeBmp);

                    //基礎部分
                    g.FillRectangle(bkBrush, 0, 0, makeWidth, makeHeight);

                    g.DrawImage(nextBmp,
                        new Rectangle(nextWidth, nextHeight, CWData.Bitmap_Width, CWData.Bitmap_Height),
                        new Rectangle(0, 0, CWData.Bitmap_Width, CWData.Bitmap_Height), 
                        GraphicsUnit.Pixel);

                    g.DrawImage(firstBmp,
                        new Rectangle(firstWidth, firstHeight, CWData.Bitmap_Width, CWData.Bitmap_Height),
                        new Rectangle(0, 0, CWData.Bitmap_Width, CWData.Bitmap_Height), 
                        GraphicsUnit.Pixel);

                    g.DrawImage(lastBmp,
                        new Rectangle(lastWidth, lastHeight, CWData.Bitmap_Width, CWData.Bitmap_Height),
                        new Rectangle(0, 0, CWData.Bitmap_Width, CWData.Bitmap_Height), 
                        GraphicsUnit.Pixel);

                    //画像の透明色部分
                    for (int height = 0; height < CWData.Bitmap_Height; height++)
                    {
                        for (int width = 0; width < CWData.Bitmap_Width; width++)
                        {
                            Color setCol;

                            setCol = nextBmp.GetPixel(width, height);
                            if (setCol.GetHue() == nextTransColor.GetHue())
                            {
                                g.FillRectangle(bkBrush, width + nextWidth, height + nextHeight, 1, 1);
                            }

                            setCol = firstBmp.GetPixel(width, height);
                            if (setCol.GetHue() == firstTransColor.GetHue())
                            {
                                g.FillRectangle(bkBrush, width + firstWidth, height + firstHeight, 1, 1);
                            }

                            setCol = lastBmp.GetPixel(width, height);
                            if (setCol.GetHue() == lastTransColor.GetHue())
                            {
                                g.FillRectangle(bkBrush, width + lastWidth, height + lastHeight, 1, 1);
                            }

                        }
                    }

                    //旧
                    // 実は74x94の倍数でなければ透明色の位置がずれます。
                    // この場合、（6,16）の１ピクセルを落とします。
                    //g.FillRectangle(bkBrush, 6, 16, 1, 1);


                    g.Dispose();

                    //旧
                    ////for (int height = 0; height < 94; height++)
                    ////{
                    ////    for (int width = 0; width < 74; width++)
                    ////    {
                    ////        Color setCol;

                    ////        setCol = nextBmp.GetPixel(width, height);
                    ////        makeBmp.SetPixel(width, height + nextHeight,
                    ////            setCol == nextTransColor ? bkColor : setCol);

                    ////        setCol = firstBmp.GetPixel(width, height);
                    ////        makeBmp.SetPixel(width, height + firstHeight,
                    ////            setCol == firstTransColor ? bkColor : setCol);

                    ////        setCol = lastBmp.GetPixel(width, height);
                    ////        makeBmp.SetPixel(width, height + lastHeight,
                    ////            setCol == lastTransColor ? bkColor : setCol);
                    ////    }
                    ////}


                    ////for (int height = nextHeight + 94; height < firstHeight; height++)
                    ////{
                    ////    for (int width = 0; width < 74; width++)
                    ////    {
                    ////        makeBmp.SetPixel(width, height, bkColor);
                    ////    }
                    ////}

                    ////for (int height = firstHeight + 94; height < lastHeight; height++)
                    ////{
                    ////    for (int width = 0; width < 74; width++)
                    ////    {
                    ////        makeBmp.SetPixel(width, height, bkColor);
                    ////    }
                    ////}
                    
                    string saveFile = FilePath.GetBmpFileName(folder, Path.GetFileNameWithoutExtension(firstBmpName));

                    makeBmp.Save(saveFile);
                    makeBmp.Dispose();

                    // これ忘れていた 2012/09/30
                    Util3.FixBitmap(saveFile);

                    
                }
            }

        }
    

        #endregion

        #region XML状態反映
        private void XmlStateChange()
        {
            if (_pm != null)
            {
                _pm.paramOption = this.chkParam.Checked;
                _pm.createFullPartyWithYadoCard = this.chkPartyItem.Checked;
                _pm.isSimpleCardWindow = this.chkSimple.Checked;

                //BGMを変更(無音にする場合は消音)
                if(_pm.soundMute != this.chkMute.Checked)
                    MultiMedia.PlayMusicFile(FilePath.GetDefOpenMidi(), this.chkMute.Checked);
                _pm.soundMute = this.chkMute.Checked;
                
                _pm.yadoSimple = this.chkYadoSimple.Checked;
            }
        }

        #endregion

        private void btnKeyCode_Click(object sender, EventArgs e)
        {
            if (!File.Exists(FilePath.ApplicationFolder + "\\Summary.cache"))
            {
                MessageDlg.ShowInfoDialog("シナリオキャッシュがありません。");
                return;
            }

            string scenario = "";
            string str = txtKeyCode.Text;
            ReadCache();
            if (ScenarioCacheDt != null)
            {
                WidArea area = null;
                WidBattle battle = null;
                foreach (DataRow dr in ScenarioCacheDt.Rows)
                {
                    string folder = dr[ColumnNames[7]].ToString();

                    try
                    {
                        DirectoryInfo di = new DirectoryInfo(folder);
                        FileInfo[] fis;
                        try
                        {
                            fis = di.GetFiles("Area*.wid", SearchOption.TopDirectoryOnly);
                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    area = new WidArea();
                                    area.Read(fi.FullName);
                                    if (area.KeyCodeIsIncluded(str))
                                    {
                                        scenario = fi.FullName;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }


                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;

                        try
                        {
                            fis = di.GetFiles("Battle*.wid", SearchOption.TopDirectoryOnly);

                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    battle = new WidBattle();
                                    battle.Read(fi.FullName);
                                    if (battle.KeyCodeIsIncluded(str))
                                    {
                                        scenario = fi.FullName;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }

                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;
                    }
                    catch (Exception)
                    {
                        // 隊長！ファイル読み込めません！！
                        continue;
                    }
                }
            }

            if (scenario != "")
            {
                MessageDlg.ShowInfoDialog("ありました。(" + scenario + ")");
            }
            else
            {
                MessageDlg.ShowInfoDialog("見つかりませんでした。");
            }

        }

        private string[] ColumnNames = 
            {
                "Scenario",
                "Author",
                "Low",
                "High",
                "Version",
                "CanPlay",
                "Expression",
                "Folder",
                "LastWrite",
                "Coupon1",
                "Coupon2",
                "Coupon3",
                "Coupon4",
                "Coupon5"
            };

        private DataTable ScenarioCacheDt = null;
        private void ReadCache()
        {
            //読み込んでデシリアライズする
            FileStream fs = new FileStream(FilePath.ApplicationFolder + "\\Summary.cache",
            FileMode.Open,
            FileAccess.Read);
            BinaryFormatter f = new BinaryFormatter();

            // ダイアログを出すほどの時間がかからない

            object obj = f.Deserialize(fs);
            DataSet ds = (DataSet)obj;
            ScenarioCacheDt = ds.Tables[0];

            f = null;
            fs.Dispose();

        }

        private void btnKeyCodeFirst_Click(object sender, EventArgs e)
        {
            if (!File.Exists(FilePath.ApplicationFolder + "\\Summary.cache"))
            {
                MessageDlg.ShowInfoDialog("シナリオキャッシュがありません。");
                return;
            }

            string scenario = "";
            string str = txtKeyCode.Text;
            ReadCache();
            if (ScenarioCacheDt != null)
            {
                WidArea area = null;
                WidBattle battle = null;
                foreach (DataRow dr in ScenarioCacheDt.Rows)
                {
                    string folder = dr[ColumnNames[7]].ToString();

                    try
                    {
                        DirectoryInfo di = new DirectoryInfo(folder);
                        FileInfo[] fis;
                        try
                        {
                            fis = di.GetFiles("Area*.wid", SearchOption.TopDirectoryOnly);
                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    area = new WidArea();
                                    area.Read(fi.FullName);
                                    if (area.KeyCodeStartsWith(str))
                                    {
                                        scenario = fi.FullName;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }


                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;

                        try
                        {
                            fis = di.GetFiles("Battle*.wid", SearchOption.TopDirectoryOnly);

                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    battle = new WidBattle();
                                    battle.Read(fi.FullName);
                                    if (battle.KeyCodeStartsWith(str))
                                    {
                                        scenario = fi.FullName;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }

                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;

                    }
                    catch (Exception)
                    {
                        continue;
                        // 隊長！ファイル読み込めません！！
                    }
                }
            }

            if (scenario != "")
            {
                MessageDlg.ShowInfoDialog("ありました。(" + scenario + ")");
            }
            else
            {
                MessageDlg.ShowInfoDialog("見つかりませんでした。");
            }
        }

        private void btnCoupon_Click(object sender, EventArgs e)
        {
            if (!File.Exists(FilePath.ApplicationFolder + "\\Summary.cache"))
            {
                MessageDlg.ShowInfoDialog("シナリオキャッシュがありません。");
                return;
            }

            string scenario = "";
            string str = this.txtCoupon.Text;
            ReadCache();
            if (ScenarioCacheDt != null)
            {
                WidArea area = null;
                WidBattle battle = null;
                WidPackage pack = null;
                foreach (DataRow dr in ScenarioCacheDt.Rows)
                {
                    string folder = dr[ColumnNames[7]].ToString();

                    try
                    {
                        DirectoryInfo di = new DirectoryInfo(folder);
                        FileInfo[] fis;
                        try
                        {
                            fis = di.GetFiles("Area*.wid", SearchOption.TopDirectoryOnly);
                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    area = new WidArea();
                                    area.Read(fi.FullName);
                                    string val = area.CouponIsIncluded(str);
                                    if(val != null)
                                    {
                                        scenario = fi.FullName + ":" + val;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }


                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;

                        try
                        {
                            fis = di.GetFiles("Battle*.wid", SearchOption.TopDirectoryOnly);

                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    battle = new WidBattle();
                                    battle.Read(fi.FullName);
                                    string val = battle.CouponIsIncluded(str);
                                    if (val != null)
                                    {
                                        scenario = fi.FullName + ":" + val;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }

                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;

                        try
                        {
                            fis = di.GetFiles("Package*.wid", SearchOption.TopDirectoryOnly);

                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    pack = new WidPackage();
                                    pack.Read(fi.FullName);
                                    string val = pack.CouponIsIncluded(str);
                                    if (val != null)
                                    {
                                        scenario = fi.FullName + ":" + val;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }

                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;
                    }
                    catch (Exception)
                    {
                        // 隊長！ファイル読み込めません！！
                        continue;
                    }
                }
            }

            if (scenario != "")
            {
                MessageDlg.ShowInfoDialog("ありました。(" + scenario + ")");
            }
            else
            {
                MessageDlg.ShowInfoDialog("見つかりませんでした。");
            }

        
        }

        private void btnGossip_Click(object sender, EventArgs e)
        {
            if (!File.Exists(FilePath.ApplicationFolder + "\\Summary.cache"))
            {
                MessageDlg.ShowInfoDialog("シナリオキャッシュがありません。");
                return;
            }

            string scenario = "";
            string str = this.txtGossip.Text;
            ReadCache();
            if (ScenarioCacheDt != null)
            {
                WidArea area = null;
                WidBattle battle = null;
                WidPackage pack = null;
                foreach (DataRow dr in ScenarioCacheDt.Rows)
                {
                    string folder = dr[ColumnNames[7]].ToString();

                    try
                    {
                        DirectoryInfo di = new DirectoryInfo(folder);
                        FileInfo[] fis;
                        try
                        {
                            fis = di.GetFiles("Area*.wid", SearchOption.TopDirectoryOnly);
                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    area = new WidArea();
                                    area.Read(fi.FullName);
                                    string val = area.GossipIsIncluded(str);
                                    if (val != null)
                                    {
                                        scenario = fi.FullName + ":" + val;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }


                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;

                        try
                        {
                            fis = di.GetFiles("Battle*.wid", SearchOption.TopDirectoryOnly);

                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    battle = new WidBattle();
                                    battle.Read(fi.FullName);
                                    string val = battle.GossipIsIncluded(str);
                                    if (val != null)
                                    {
                                        scenario = fi.FullName + ":" + val;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }

                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;

                        try
                        {
                            fis = di.GetFiles("Package*.wid", SearchOption.TopDirectoryOnly);

                            foreach (FileInfo fi in fis)
                            {
                                try
                                {
                                    pack = new WidPackage();
                                    pack.Read(fi.FullName);
                                    string val = pack.GossipIsIncluded(str);
                                    if (val != null)
                                    {
                                        scenario = fi.FullName + ":" + val;
                                        break;
                                    }

                                }
                                catch (Exception)
                                {
                                    // 隊長！ファイル読み込めません！！
                                    continue;
                                }
                            }

                        }
                        catch (Exception)
                        {
                            // 隊長！ファイル読み込めません！！
                        }

                        if (scenario != "") break;
                    }
                    catch (Exception)
                    {
                        // 隊長！ファイル読み込めません！！
                        continue;
                    }
                }
            }

            if (scenario != "")
            {
                MessageDlg.ShowInfoDialog("ありました。(" + scenario + ")");
            }
            else
            {
                MessageDlg.ShowInfoDialog("見つかりませんでした。");
            }

        }



    }
}