﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class JukuBoxDlg : Form
    {
        #region メンバ

        private PartyMaker _pm;

        private static volatile ProgressDlg progress;

        private DataTable dt;

        private string[] ColumnNames = 
            {
                "Filename",
                "FullPath"
            };

        private string[] Captions = 
            {
                "ファイル名",
                "フルパス"
            };

        private bool widthFirst = false;

        private InnData inn;
        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public JukuBoxDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public JukuBoxDlg(PartyMaker pm, InnData inn)
        {
            InitializeComponent();

            _pm = pm;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _pm.Left + (_pm.Width - this.Width) / 2;
            this.Top =
                _pm.Top + (_pm.Height - this.Height) / 2;

            this.inn = inn;

            progress = new ProgressDlg(this);
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void JukuBoxDlg_Load(object sender, EventArgs e)
        {

            // this.WindowState = FormWindowState.Maximized;
            MultiMedia.SoundStop();

            this.cmbSearch1.Items.Add("ファイル名");
            this.cmbSearch1.Items.Add("フルパス");
            this.cmbSearch1.SelectedIndex = 0;
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void JukuBoxDlg_Shown(object sender, EventArgs e)
        {
            if(File.Exists(FilePath.ApplicationFolder + "\\Music.cache"))
            {
                // デシリアしてDataTable取り出し
                ReadCache();

            }else {
                // Data\Midiだけで初期キャッシュを作成
                MakeInitCache();
            }

            DispDt();
        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnCache_Click(object sender, EventArgs e)
        {                
            // シナリオ読み込んでDataTableを作成後、
            // シリアライズしてバイナリファイルに書き出し。
            MakeCache();
            DispDt();
        }

        private void ReadCache()
        { 
            //読み込んでデシリアライズする
            FileStream fs = new FileStream(FilePath.ApplicationFolder + "\\Music.cache",
            FileMode.Open,
            FileAccess.Read);
            BinaryFormatter f = new BinaryFormatter();

            // ダイアログを出すほどの時間がかからない

            object obj = f.Deserialize(fs);
            DataSet ds = (DataSet)obj;
            dt = ds.Tables[0];

            f = null;
            fs.Dispose();

        }

        private void MakeInitCache()
        {

            // 新規DT、Column設定
            PrepareNewDt();

            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "初期ファイルとしてData\\Midiを読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            //@IT
            DirectoryInfo di = new DirectoryInfo(FilePath.GetDataMidiFolder());
            FileInfo[] fis = di.GetFiles("*.mid*", SearchOption.AllDirectories);

            int count = 0;
            foreach (FileInfo fi in fis)
            {
                DataRow dr = dt.NewRow();
                dr[0] = fi.Name;
                dr[1] = fi.FullName;
                dt.Rows.Add(dr);
                progress.Value = 90 * ++count / fis.Length;
            }

            progress.Value = 90;

            // DataTableをDataSetに詰めてシリアライズして保存
            progress.Message = "Music.cacheファイルに保存します。";
            DataSet ds = new DataSet();
            ds.Tables.Add(dt);

            FileStream fs = new FileStream(
            FilePath.ApplicationFolder + "\\Music.cache",
            FileMode.Create,
            FileAccess.Write);
            BinaryFormatter bf = new BinaryFormatter();
            //シリアル化して書き込む
            bf.Serialize(fs, ds);
            fs.Close();
            fs.Dispose();

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

        }

        private void MakeCache()
        {   

            // 新規DT、Column設定
            PrepareNewDt();

            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "シナリオフォルダ(midi)\r\nを読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;
            
            //@IT
            DirectoryInfo di = new DirectoryInfo(FilePath.GetSystemScenarioFolder());
            FileInfo[] fis = di.GetFiles("*.mid*",SearchOption.AllDirectories);
            int count = 0;
            foreach (FileInfo fi in fis)
            {
                DataRow dr = dt.NewRow();
                dr[0] = fi.Name;
                dr[1] = fi.FullName;
                dt.Rows.Add(dr);

                count++;
                progress.Value = 60 * count / fis.Length;
            }


            progress.Value = 60;
            progress.Message = "シナリオフォルダ(mp3)\r\nを読み込み中です。";

            fis = di.GetFiles("*.mp3", SearchOption.AllDirectories);
            count = 0;
            foreach (FileInfo fi in fis)
            {
                DataRow dr = dt.NewRow();
                dr[0] = fi.Name;
                dr[1] = fi.FullName;
                dt.Rows.Add(dr);

                count++;
                progress.Value = 60 + 20 * count / fis.Length;
            }

            progress.Value = 80;
            progress.Message = "Data\\Midiを読み込み中です。";
            di = new DirectoryInfo(FilePath.GetDataMidiFolder());
            fis = di.GetFiles("*.mid*", SearchOption.AllDirectories);

            count = 0;
            foreach (FileInfo fi in fis)
            {
                DataRow dr = dt.NewRow();
                dr[0] = fi.Name;
                dr[1] = fi.FullName;
                dt.Rows.Add(dr);

                count++;
                progress.Value = 80 + 10 * count / fis.Length;
            }
            progress.Value = 90;

            // DataTableをDataSetに詰めてシリアライズして保存
            progress.Message = "Music.cacheファイルに保存します。";
            DataSet ds = new DataSet();
            ds.Tables.Add(dt);

            FileStream fs = new FileStream(
            FilePath.ApplicationFolder + "\\Music.cache",
            FileMode.Create,
            FileAccess.Write);
            BinaryFormatter bf = new BinaryFormatter();
            //シリアル化して書き込む
            bf.Serialize(fs, ds);
            fs.Close();
            fs.Dispose();

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

        }

        private void PrepareNewDt()
        {
            dt = new DataTable();

            dt.CaseSensitive = true;

            //DataColumn 設定
            DataColumn dataCol;


            for (int i = 0; i < ColumnNames.Length; i++)
            {
                dataCol = new DataColumn();
                dataCol.ColumnName = ColumnNames[i];
                dataCol.Caption = Captions[i];

                dataCol.DataType = typeof(string);
                dataCol.DefaultValue = string.Empty;

                dt.Columns.Add(dataCol);
            }
          

        }

        private void DispDt()
        {
            DataTable dtDisp = dt.Copy();
            int dtCount = dtDisp.Rows.Count;

            DataView dv = new DataView(dtDisp);

            //  検索
            string search1Type = string.Empty;
            switch (cmbSearch1.SelectedIndex)
            {
                case 0:
                    search1Type = ColumnNames[0];
                    break;
                default:
                    search1Type = ColumnNames[1];
                    break;
            }

            string search1Filter = txtSearch1.Text.Equals(string.Empty)
                ? string.Empty
                : search1Type + " LIKE '%" + txtSearch1.Text + "%'";

            dv.RowFilter = search1Filter;

            int dvCount = dv.Count;

            dgv1.DataSource = dv;

            lblCount.Text = dvCount + "/" + dtCount;

            if (!widthFirst)
            {
                dgv1.Columns[ColumnNames[0]].Width = 200;
                dgv1.Columns[ColumnNames[1]].Width = 480;
                widthFirst = true;
            }

            for (int i = 0; i < dt.Columns.Count;i++)
            {
                dgv1.Columns[i].HeaderText =
                    dt.Columns[i].Caption;
            }

            this.Activate();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void JukuBoxDlg_FormClosing(object sender, FormClosingEventArgs e)
        {

        }

        private void JukuBoxDlg_Resize(object sender, EventArgs e)
        {
            if (this.Width >= this.MinimumSize.Width
                && this.Height >= this.MinimumSize.Height)
            {

                this.dgv1.Width = this.Width - 30;
                this.dgv1.Height = this.Height - 100;

                this.btnCSV.Location =
                    new Point(this.ClientSize.Width - 12 - 53,
                    this.ClientSize.Height - 28);

                this.btnCopyMidi.Location =
                    new Point(this.ClientSize.Width - 12 - 53 - 92,
                    this.ClientSize.Height - 28);

                this.lblCount.Location =
                    new Point(this.lblCount.Location.X, this.ClientSize.Height - 28);

                this.btnEnd.Location =
                    new Point((this.ClientSize.Width - this.btnEnd.Width) / 2, this.ClientSize.Height - 28);

            }
        }
        #endregion

        #region キーイベント

        private void JukuBoxDlg_KeyDown(object sender, KeyEventArgs e)
        {
            int dvCount = ((DataView)(dgv1.DataSource)).Count;
            if (e.KeyCode == Keys.PageUp)
            {
                if (0 < dgv1.FirstDisplayedScrollingRowIndex - 15)
                {
                    dgv1.FirstDisplayedScrollingRowIndex -= 15;
                }
                else
                {
                    dgv1.FirstDisplayedScrollingRowIndex = 0;
                }
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.PageDown)
            {
                if (dvCount - 1 > dgv1.FirstDisplayedScrollingRowIndex + 15)
                {
                    dgv1.FirstDisplayedScrollingRowIndex += 15;
                }
                else
                {
                    dgv1.FirstDisplayedScrollingRowIndex = dvCount - 1;
                }
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Home)
            {

                dgv1.FirstDisplayedScrollingRowIndex = 0;
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.End)
            {

                dgv1.FirstDisplayedScrollingRowIndex = dvCount - 1;
                e.Handled = true;
            }
        }

        #endregion
        
        #region クリック

        private void btnSearch_Click(object sender, EventArgs e)
        {
            DispDt();
        }

        private void btnCSV_Click(object sender, EventArgs e)
        {
            // テスト用（正確なデータとの比較用）に出力

            //DOBON.NETより

            //CSVで保存するDataTableはすでにある。

            //保存先のCSVファイルのパス
            string csvPath = FilePath.ApplicationFolder + "\\pm_midi.csv";

            //CSVファイルに書き込むときに使うEncoding
            System.Text.Encoding enc =
                System.Text.Encoding.GetEncoding("Shift_JIS");

            //開く
            System.IO.StreamWriter sr =
                new System.IO.StreamWriter(csvPath, false, enc);

            int colCount = dt.Columns.Count;
            int lastColIndex = colCount - 1;

            //レコードを書き込む
            foreach (DataRow row in dt.Rows)
            {
                for (int i = 0; i < colCount; i++)
                {
                    //フィールドの取得
                    string field = row[i].ToString();
                    //"で囲む必要があるか調べる
                    if (field.IndexOf('"') > -1 ||
                        field.IndexOf(',') > -1 ||
                        field.IndexOf('\r') > -1 ||
                        field.IndexOf('\n') > -1 ||
                        field.StartsWith(" ") || field.StartsWith("\t") ||
                        field.EndsWith(" ") || field.EndsWith("\t"))
                    {
                        if (field.IndexOf('"') > -1)
                        {
                            //"を""とする
                            field = field.Replace("\"", "\"\"");
                        }
                        field = "\"" + field + "\"";
                    }
                    //フィールドを書き込む
                    sr.Write(field);
                    //カンマを書き込む
                    if (lastColIndex > i)
                    {
                        sr.Write(',');
                    }
                }
                //改行する
                sr.Write("\r\n");
            }

            //閉じる
            sr.Close();

            sr.Dispose();


        }

        private void dgv1_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex < 0 || e.RowIndex > dgv1.Rows.Count - 1)
            {
                return;
            }

            //曲を流す
            MultiMedia.PlayMusicFile((string)dgv1.Rows[e.RowIndex].Cells[1].Value, _pm.soundMute);

        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            MultiMedia.SoundStop();     
        }

        private void btnCopyMidi_Click(object sender, EventArgs e)
        {

            if (this.dgv1.SelectedRows.Count == 0)
            {
                MessageDlg.ShowInfoDialog("選択されていません。");
                return;
            }

            string fileName =
                this.dgv1.SelectedRows[0].Cells[0].Value.ToString();

            string fullName = 
                this.dgv1.SelectedRows[0].Cells[1].Value.ToString();

            string destFile = FilePath.GetDataMidiFolder() 
                + "\\" + fileName;

            if(fileName.EndsWith(".mp3"))
            {
                MessageDlg.ShowInfoDialog(fileName
                + "\nはData\\Midiへの移動対象外です。");
            } 
            else if (File.Exists(destFile))
            {
                MessageDlg.ShowInfoDialog(fileName
                + "\nはData\\Midiに存在します。");
            } 
            else if 
            (DialogResult.Yes == MessageDlg.ShowYesNoDialog(fullName
                + "\nをData\\Midiにコピーします。よろしいですか？"))
            {
                File.Copy(fullName, destFile);
            }
        }

        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {

            backgroundWorker1.CancelAsync();
            while (this.backgroundWorker1.IsBusy)
            {
                Thread.Sleep(1);
            }

            backgroundWorker1.RunWorkerAsync();
            
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn 
                || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();

            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion
  }
}