﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class MoveCardDlg : Form
    {
        #region メンバ

        private int disp_num_x = 3;
        private int old_num_x = 3;
        private FormWindowState oldState = FormWindowState.Normal;
        
        /// <summary>
        /// 宿
        /// </summary>
        private InnData inn = null;

        /// <summary>
        /// 他宿名
        /// </summary>
        private InnData otherInn = null;


        // 現在のインデックス
        private int index = 0;

        // マウスの押されたインデックス
        private int mouseDownIndex = -1;
        // マウスの押されたパネル
        private Panel mouseDownPanel = null;
        // マウスの押された座標
        private Point mouseDownPoint = Point.Empty;

        private static volatile ProgressDlg progress;

        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MoveCardDlg()
        {
            // 通常使用しない
            InitializeComponent();

            MouseWheel += new MouseEventHandler(OtherInn_MouseWheel);

            progress = new ProgressDlg(this);
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MoveCardDlg(InnData inn, InnData otherInn)
        {
            InitializeComponent();

            MouseWheel += new MouseEventHandler(OtherInn_MouseWheel);

            progress = new ProgressDlg(this);

            this.inn = inn;
            this.otherInn = otherInn;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void MoveCardDlg_Load(object sender, EventArgs e)
        {
            // 宿名反映
            this.grpOtherInn.Text = otherInn.Name + " カード置場(0)";
            this.grpYadoCard.Text = inn.Name + " カード置場(0)";
            this.btnAllInn.Text = inn.Name + " へ";
            this.btnAllMove.Text = otherInn.Name + " へ"; 


            // マウス・カーソルの位置をスクリーン中央に移動させる
            int centerX = Screen.PrimaryScreen.Bounds.Right / 2;
            int centerY = Screen.PrimaryScreen.Bounds.Bottom / 2;
            Point cursolPosition = new Point(centerX, centerY);
            this.Location = new Point(centerX - this.Width / 2, centerY - this.Height / 2);

            this.cmbOrder.SelectedIndex = 0;


            // 最大化表示
            this.WindowState = FormWindowState.Maximized;

            // 一度走らせる
            MoveCardDlg_Resize(sender, e);
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MoveCardDlg_Shown(object sender, EventArgs e)
        {
            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = otherInn.Name + " カード置場を読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            // 対象他宿カード置場設定
            inn.SetOtherInnCard(this.otherInn);

            progress.Value = 20;

            // カード置場
            // カード置場表示
            SetUpYadoCard();

            progress.Value = 60;

            // 他宿カード置場表示
            SetUpOtherInn();

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();
        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            // FormClosingに集約
            this.Close();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MoveCardDlg_FormClosing(object sender, FormClosingEventArgs e)
        {

            // 他宿カード関連はクリア（このとき中では同時に宿はセーブしておかないといけない）
            inn.ClearOtherInnCard();
        }

        /// <summary>
        /// サイズ変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MoveCardDlg_Resize(object sender, EventArgs e)
        {

            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {
                if (oldState != this.WindowState)
                {
                    oldState = this.WindowState;
                    MoveCardDlg_ResizeEnd(sender, e);
                }

                int baseWidth = this.Width / 2;
              
                this.grpOtherInn.Width = baseWidth - 10;
                this.grpOtherInn.Height = this.Height - 63;

                this.grpYadoCard.Location = new Point(baseWidth, this.grpYadoCard.Location.Y);

                this.grpYadoCard.Width = baseWidth - 10;
                this.grpYadoCard.Height = this.Height - 63;


                this.pnlOtherInn.Width = baseWidth - 26;
                this.pnlOtherInn.Height = this.Height - 86;
                this.pnlYadoCard.Width = baseWidth - 26;
                this.pnlYadoCard.Height = this.Height - 86;

                this.btnAllMove.Location
                = new Point(this.btnAllMove.Location.X, this.ClientSize.Height - 28);

                this.btnAllInn.Location
                = new Point(this.btnAllInn.Location.X, this.ClientSize.Height - 28);

                this.btnEnd.Location = new Point(
                    (this.ClientSize.Width - this.btnEnd.Width) / 2,
                    this.ClientSize.Height - 28);

                this.btnSort.Location = new Point(
                    this.ClientSize.Width - 85, this.ClientSize.Height - 28);

                this.cmbOrder.Location = new Point(
                    this.ClientSize.Width - 85 - 174, this.ClientSize.Height - 26);

                this.disp_num_x = (this.pnlYadoCard.Width - 6 - 10) / CWData.HAND_CARD_WIDTH;


            }
            

        }

        private void MoveCardDlg_ResizeBegin(object sender, EventArgs e)
        {
            old_num_x = this.disp_num_x;
        }

        private void MoveCardDlg_ResizeEnd(object sender, EventArgs e)
        {

            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {
                MoveCardDlg_Resize(sender, e);

                SetLocationPnlYadoCard();
                SetLocationPnlOtherInn();
                
            }
        }

        private void MoveCardDlg_KeyDown(object sender, KeyEventArgs e)
        {
            //KeyCode + Modifiers = KeyData
            if ((e.Modifiers & Keys.Control) == Keys.Control)
            {

                if (e.KeyCode == Keys.Up)
                {
                    if (mouseDownPanel == this.pnlOtherInn)
                    {
                        ScrollPnl(pnlOtherInn, -100);
                    }
                    else 
                    {
                        ScrollPnl(pnlYadoCard, -100);
                    }

                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Down)
                {
                    if (mouseDownPanel == this.pnlOtherInn)
                    {
                        ScrollPnl(pnlOtherInn, 100);
                    }
                    else 
                    {
                        ScrollPnl(pnlYadoCard, 100);
                    }

                    e.Handled = true;
                }
            }

            if (e.KeyCode == Keys.PageUp)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, -100);
                }
                else 
                {
                    ScrollPnl(pnlYadoCard, -100);
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.PageDown)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, 100);
                }
                else 
                {
                    ScrollPnl(pnlYadoCard, 100);
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Home)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, -CWData.HAND_CARD_HEIGHT * ((pnlOtherInn.Controls.Count - 1) / disp_num_x + 1));
                }
                else 
                {
                    ScrollPnl(pnlYadoCard, -CWData.HAND_CARD_HEIGHT * ((pnlYadoCard.Controls.Count - 1) / disp_num_x + 1));
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.End)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, CWData.HAND_CARD_HEIGHT * ((pnlOtherInn.Controls.Count - 1) / disp_num_x + 1));
                }
                else 
                {
                    ScrollPnl(pnlYadoCard, CWData.HAND_CARD_HEIGHT * ((pnlYadoCard.Controls.Count - 1) / disp_num_x + 1));
                }

                e.Handled = true;
            }
        }
        /// <summary>
        /// 全カード現在の宿へボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllInn_Click(object sender, EventArgs e)
        {
            int count = inn.OtherInn.BankBoxCards.Count;
            if (count > 0)
            {
                // 進捗ダイアログ開始
                StartProgress();
                // 進捗ダイアログ初期処理待ち
                WaitProgress();
                progress.Message = "全カード " + inn.Name + " に移動中です。";
                progress.Maximum = 100;
                progress.Minimum = 0;
                progress.Value = 0;

                // 全他宿カード置場カードを現在の宿へ
                inn.MoveAllOtherInnCard();

                // 表示反映
                Control[] array = new Control[pnlOtherInn.Controls.Count];
                pnlOtherInn.Controls.CopyTo(array, 0);

                pnlOtherInn.Controls.Clear();

                this.grpOtherInn.Text = otherInn.Name + " カード置場(0)";

                // 表示用カード追加・配置修正
                int nowX = -pnlYadoCard.AutoScrollPosition.X;
                int nowY = -pnlYadoCard.AutoScrollPosition.Y;

                pnlYadoCard.AutoScrollPosition = new Point(0, 0);

                progress.Value = 50;


                IndexCardData icd;
                DispCard dc;

                pnlYadoCard.Controls.AddRange(array);

                index = 0;
                for (int i = inn.BankBoxCards.Count - count; i < inn.BankBoxCards.Count; i++)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = otherInn.Name + " カード置場：" + (index + 1) + "枚目を移動しています。";

                    icd = inn.FindIndexedCardData(inn.BankBoxCards[i]);

                    dc = (DispCard)array[index];
                    dc.SetHc(icd.HandCard);

                    dc.Location = new Point(CWData.HAND_CARD_WIDTH
                        * (i % disp_num_x), CWData.HAND_CARD_HEIGHT * (i / disp_num_x));

                    //プログレスバーの値を変更する
                    index++;
                    progress.Value = 50 + index * 50 / count;
                }

                this.grpYadoCard.Text = inn.Name + " カード置場(" + inn.BankBoxCards.Count + ")";

                pnlYadoCard.AutoScrollPosition = new Point(nowX, nowY);

                // 進捗ダイアログ終了
                EndProgress();

            }
        }

        /// <summary>
        /// 全カード他宿へ移動ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllMove_Click(object sender, EventArgs e)
        {
            int count = inn.BankBoxCards.Count;
            if (count > 0)
            {
                // 進捗ダイアログ開始
                StartProgress();
                // 進捗ダイアログ初期処理待ち
                WaitProgress();
                progress.Message = "全カード " + otherInn.Name + " に移動中です。";
                progress.Maximum = 100;
                progress.Minimum = 0;
                progress.Value = 0;

                // 全カード置場を他宿に移動
                inn.MoveAllCardToOtherInn();

                // 表示反映
                Control[] array = new Control[pnlYadoCard.Controls.Count];
                pnlYadoCard.Controls.CopyTo(array, 0);

                pnlYadoCard.Controls.Clear();

                this.grpYadoCard.Text = inn.Name + " カード置場(0)";

                // 表示用カード追加・配置修正
                int nowX = -pnlOtherInn.AutoScrollPosition.X;
                int nowY = -pnlOtherInn.AutoScrollPosition.Y;

                pnlOtherInn.AutoScrollPosition = new Point(0, 0);

                progress.Value = 50;


                IndexCardData icd;
                DispCard dc;

                pnlOtherInn.Controls.AddRange(array);

                index = 0;
                for (int i = inn.OtherInn.BankBoxCards.Count - count; i < inn.OtherInn.BankBoxCards.Count; i++)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = "カード置場：" + (index + 1) + "枚目を移動しています。";

                    icd = inn.OtherInn.FindIndexedCardData(inn.OtherInn.BankBoxCards[i]);

                    dc = (DispCard)array[index];
                    dc.SetHc(icd.HandCard);

                    dc.Location = new Point(CWData.HAND_CARD_WIDTH
                        * (i % disp_num_x), CWData.HAND_CARD_HEIGHT * (i / disp_num_x));

                    //プログレスバーの値を変更する
                    index++;
                    progress.Value = 50 + index * 50 / count;
                }

                this.grpOtherInn.Text = otherInn.Name + " カード置場(" + inn.OtherInn.BankBoxCards.Count + ")";

                pnlOtherInn.AutoScrollPosition = new Point(nowX, nowY);


                // 進捗ダイアログ終了
                EndProgress();
            }
        }

        /// <summary>
        /// ソートボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSort_Click(object sender, EventArgs e)
        {
            try
            {
                if (cmbOrder.SelectedIndex < 0 && cmbOrder.SelectedIndex > 5)
                {
                    // ありえない
                    MessageDlg.ShowInfoDialog("ソート種類の選択に誤りがあります。");
                    return;
                }
                else
                {
                    if (DialogResult.Yes != MessageDlg.ShowYesNoDialog(
                        "ソートは時間がかかる場合があります。\r\n 実行してよろしいですか？"
                        ))
                    {
                        return;
                    }
                }

                int nowX;
                int nowY;
                IndexCardData icd;
                DispCard dc;
                int count;

                if (mouseDownPanel == pnlYadoCard)
                {
                    count = inn.BankBoxCards.Count;
                    if (count > 0)
                    {
                        // カード置場

                        // 表示反映
                        // 進捗ダイアログ開始
                        StartProgress();
                        // 進捗ダイアログ初期処理待ち
                        WaitProgress();
                        progress.Message = "カード置場をソート中です。";
                        progress.Maximum = 100;
                        progress.Minimum = 0;
                        progress.Value = 0;

                        inn.SortCards(ソート対象.カード置場, (ソート種類)(cmbOrder.SelectedIndex));

                        // 個数が同じなのでクリアせずにSetHcする。
                        GC.Collect(2);

                        progress.Value = 50;

                        // 表示用カード追加・配置修正
                        nowX = -pnlYadoCard.AutoScrollPosition.X;
                        nowY = -pnlYadoCard.AutoScrollPosition.Y;

                        pnlYadoCard.AutoScrollPosition = new Point(0, 0);

                        for (int i = 0; i < count; i++)
                        {
                            // プログレスバーのメッセージを変更する
                            progress.Message = "カード置場：" + (i + 1) + "枚目を表示しています。";

                            icd = inn.FindIndexedCardData(inn.BankBoxCards[i]);

                            // DispCard は　New　せずにSetHcする。
                            dc = (DispCard)(pnlYadoCard.Controls[i]);
                            dc.SetHc(icd.HandCard);

                            //プログレスバーの値を変更する

                            progress.Value = 50 + (i + 1) * 50 / count;
                        }

                        pnlYadoCard.AutoScrollPosition = new Point(nowX, nowY);

                        // 進捗ダイアログ終了
                        EndProgress();
                    }
                }
                else if (mouseDownPanel == pnlOtherInn)
                {
                    count = inn.OtherInn.BankBoxCards.Count;
                    if (count > 0)
                    {
                        // 他宿　カード置場

                        // 表示反映
                        // 進捗ダイアログ開始
                        StartProgress();
                        // 進捗ダイアログ初期処理待ち
                        WaitProgress();
                        progress.Message = otherInn.Name + " カード置場をソート中です。";
                        progress.Maximum = 100;
                        progress.Minimum = 0;
                        progress.Value = 0;

                        inn.OtherInn.SortCards(ソート対象.カード置場, (ソート種類)(cmbOrder.SelectedIndex));

                        // 個数が同じなのでクリアせずにSetHcする。
                        GC.Collect(2);

                        progress.Value = 50;

                        // 表示用カード追加・配置修正
                        nowX = -pnlOtherInn.AutoScrollPosition.X;
                        nowY = -pnlOtherInn.AutoScrollPosition.Y;

                        pnlOtherInn.AutoScrollPosition = new Point(0, 0);

                        for (int i = 0; i < count; i++)
                        {
                            // プログレスバーのメッセージを変更する
                            progress.Message = otherInn.Name + " カード置場：" + (i + 1) + "枚目を表示しています。";

                            icd = inn.OtherInn.FindIndexedCardData(inn.OtherInn.BankBoxCards[i]);

                            // DispCard は　New　せずにSetHcする。
                            dc = (DispCard)(pnlOtherInn.Controls[i]);
                            dc.SetHc(icd.HandCard);

                            //プログレスバーの値を変更する

                            progress.Value = 50 + (i + 1) * 50 / count;
                        }

                        pnlOtherInn.AutoScrollPosition = new Point(nowX, nowY);

                        // 進捗ダイアログ終了
                        EndProgress();
                    }
                }

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("システムエラー（ソート）\r\n" + ex.ToString());
            }
        }


        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {
            backgroundWorker1.RunWorkerAsync();
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();
            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion

        #region 表示と配置
        /// <summary>
        /// カード置場Card設定
        /// </summary>
        private void SetUpYadoCard()
        {

            pnlYadoCard.Controls.Clear();
            pnlYadoCard.AutoScrollPosition = new Point(0, 0);
            int count = inn.BankBoxCards.Count;
            if (count != 0)
            {

                progress.Message = inn.Name + " カード置場を表示中です。";

                this.pnlYadoCard.SuspendLayout();

                index = 0;
                IndexCardData icd;
                DispCard dc;
                foreach (RenamedCardData rcd in inn.BankBoxCards)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = inn.Name + " カード置場：" + (index + 1) + "枚目を移動しています。";

                    icd = inn.FindIndexedCardData(rcd);

                    dc = new DispCard(icd.HandCard);
                    dc.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x)
                    , CWData.HAND_CARD_HEIGHT * (index / disp_num_x));

                    index++;
                    pnlYadoCard.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    progress.Value = 20 + index * 40 / count;
                }

                this.grpYadoCard.Text = inn.Name + " カード置場(" + inn.BankBoxCards.Count +")";

                this.pnlYadoCard.ResumeLayout();

            }

        }
        /// <summary>
        /// 他宿カード置場設定
        /// </summary>
        private void SetUpOtherInn()
        {

            pnlOtherInn.Controls.Clear();
            pnlOtherInn.AutoScrollPosition = new Point(0, 0);
            int count = inn.OtherInn.BankBoxCards.Count;
            if (count != 0)
            {

                progress.Message = otherInn.Name + " カード置場を表示中です。";

                this.pnlOtherInn.SuspendLayout();

                index = 0;
                IndexCardData icd;
                DispCard dc;
                foreach (RenamedCardData rcd in inn.OtherInn.BankBoxCards)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = otherInn.Name + " カード置場：" + (index + 1) + "枚目を移動しています。";

                    icd = inn.OtherInn.FindIndexedCardData(rcd);

                    dc = new DispCard(icd.HandCard);
                    dc.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x)
                    , CWData.HAND_CARD_HEIGHT * (index / disp_num_x));

                    index++;
                    pnlOtherInn.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    progress.Value = 60 + index * 40 / count;
                }


                this.grpOtherInn.Text = otherInn.Name + " カード置場(" + inn.OtherInn.BankBoxCards.Count + ")";

                this.pnlOtherInn.ResumeLayout();

            }

        }
        /// <summary>
        /// カード置場再配置
        /// </summary>
        private void SetLocationPnlYadoCard()
        {

            int nowX = -pnlYadoCard.AutoScrollPosition.X;
            int nowY = -pnlYadoCard.AutoScrollPosition.Y;

            pnlYadoCard.AutoScrollPosition = new Point(0, 0);
            index = 0;
            foreach (Control ctrl in pnlYadoCard.Controls)
            {
                ctrl.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x),
                    CWData.HAND_CARD_HEIGHT * (index / disp_num_x));
                index++;
            }

            pnlYadoCard.AutoScrollPosition = new Point(nowX, nowY);

        }
        /// <summary>
        /// 他宿カード置場再配置
        /// </summary>
        private void SetLocationPnlOtherInn()
        {

            int nowX = -pnlOtherInn.AutoScrollPosition.X;
            int nowY = -pnlOtherInn.AutoScrollPosition.Y;

            pnlOtherInn.AutoScrollPosition = new Point(0, 0);
            index = 0;
            foreach (Control ctrl in pnlOtherInn.Controls)
            {
                ctrl.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x),
                    CWData.HAND_CARD_HEIGHT * (index / disp_num_x));
                index++;
            }

            pnlOtherInn.AutoScrollPosition = new Point(nowX, nowY);

        }

        /// <summary>
        /// 移動量分だけスクロール
        /// </summary>
        /// <param name="pnl"></param>
        /// <param name="delta"></param>
        private void ScrollPnl(Panel pnl, int delta)
        {

            if (pnl != null)
            {
                int nowX = -pnl.AutoScrollPosition.X;
                int nowY = -pnl.AutoScrollPosition.Y;
                nowY += delta;
                if (nowY < 0)
                {
                    nowY = 0;
                }

                if (pnl == pnlYadoCard || pnl == pnlOtherInn)
                {
                    if (nowY > CWData.HAND_CARD_HEIGHT * ((pnl.Controls.Count - 1) / disp_num_x + 1))
                    {
                        nowY = CWData.HAND_CARD_HEIGHT * ((pnl.Controls.Count - 1) / disp_num_x + 1);
                    }
                }

                pnl.AutoScrollPosition = new Point(nowX, nowY);
            }
        }
        
        /// <summary>
        /// カードの単独移動を表示反映する
        /// </summary>
        private void DisplayCardMove(InnData inn, Panel oldPanel, int oldIndex)
        {

            int nowX = -pnlYadoCard.AutoScrollPosition.X;
            int nowY = -pnlYadoCard.AutoScrollPosition.Y;
            int nowOtherInnX = -pnlOtherInn.AutoScrollPosition.X;
            int nowOtherInnY = -pnlOtherInn.AutoScrollPosition.Y;

            pnlYadoCard.AutoScrollPosition = new Point(0, 0);
            pnlOtherInn.AutoScrollPosition = new Point(0, 0);

            DispCard addDc;
            IndexCardData icd;
            int newIndex;
            if (oldPanel == this.pnlOtherInn)
            {
                // 他宿Cardからカード置場へ
                pnlOtherInn.Controls.RemoveAt(oldIndex);

                for (int i = oldIndex; i < pnlOtherInn.Controls.Count; i++)
                {
                    // 他宿Cardの元の位置より後ろのカードだけ配置修正
                    pnlOtherInn.Controls[i].Location = 
                        new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x), CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                }

                // すでにSwap処理でListの最後に追加されているので
                // それを元に表示用カードを作成
                newIndex = inn.BankBoxCards.Count - 1;
                icd = inn.FindIndexedCardData(inn.BankBoxCards[newIndex]);
                addDc = new DispCard(icd.HandCard);
                addDc.Location = new Point(CWData.HAND_CARD_WIDTH * (newIndex % disp_num_x), CWData.HAND_CARD_HEIGHT * (newIndex / disp_num_x));
                pnlYadoCard.Controls.Add(addDc);

            }
            else
            {

                // カード置場から他宿Cardへ
                pnlYadoCard.Controls.RemoveAt(oldIndex);

                for (int i = oldIndex; i < pnlYadoCard.Controls.Count; i++)
                {
                    // カード置場の元の位置より後ろのカードだけ配置修正
                    pnlYadoCard.Controls[i].Location = 
                        new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x), CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                }

                // すでにSwap処理でListの最後に追加されているので
                // それを元に表示用カードを作成
                newIndex = inn.OtherInn.BankBoxCards.Count - 1;
                icd = inn.OtherInn.FindIndexedCardData(inn.OtherInn.BankBoxCards[newIndex]);
                addDc = new DispCard(icd.HandCard);
                addDc.Location = new Point(CWData.HAND_CARD_WIDTH * (newIndex % disp_num_x), CWData.HAND_CARD_HEIGHT * (newIndex / disp_num_x));
                pnlOtherInn.Controls.Add(addDc);
            }

            this.grpYadoCard.Text = inn.Name + " カード置場(" + inn.BankBoxCards.Count + ")";

            this.grpOtherInn.Text = otherInn.Name + " カード置場(" + inn.OtherInn.BankBoxCards.Count + ")";

            pnlYadoCard.AutoScrollPosition = new Point(nowX, nowY);

            pnlOtherInn.AutoScrollPosition = new Point(nowOtherInnX, nowOtherInnY);
        }

        #endregion

        #region マウスイベント

        /// <summary>
        ///  カード置場パネル　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoCard_MouseDown(object sender, MouseEventArgs e)
        {

            mouseDownIndex = -1;
            mouseDownPanel = pnlYadoCard;

            // 処理対象
            int indexOfX = (-pnlYadoCard.AutoScrollPosition.X + e.X) / CWData.HAND_CARD_WIDTH;
            int indexOfY = (-pnlYadoCard.AutoScrollPosition.Y + e.Y) / CWData.HAND_CARD_HEIGHT;

            index = indexOfY * disp_num_x + indexOfX;

            if (indexOfX >= disp_num_x || index >= pnlYadoCard.Controls.Count)
            {
                // 範囲外
                // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                if (e.Button == MouseButtons.Right)
                {
                    List<HandCard> hands = new List<HandCard>();
                    foreach (Object obj in this.pnlYadoCard.Controls)
                    {
                        hands.Add(((DispCard)obj).hc);
                    }

                    using (CardInfoDialog cid = new CardInfoDialog(hands, 0))
                    {
                        cid.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    // その後スクロールできるように現在選択中パネルを入れておく
                    mouseDownPanel = pnlYadoCard;
                }
            }
            else
            {
                // PCを指している
                // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                if (e.Button == MouseButtons.Right)
                {

                    List<HandCard> hands = new List<HandCard>();
                    foreach (Object obj in this.pnlYadoCard.Controls)
                    {
                        hands.Add(((DispCard)obj).hc);
                    }

                    using (CardInfoDialog cid = new CardInfoDialog(hands, index))
                    {
                        cid.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    mouseDownIndex = index;
                    mouseDownPanel = pnlYadoCard;
                    mouseDownPoint = new Point(e.X, e.Y);

                }
                else if (e.Button == MouseButtons.Middle)
                {
                    // パネル指定先に送る
                    mouseDownIndex = index;

                    // カード置場を他宿カード欄に移動
                    inn.SwapOtherInnCardAndCard(-1, mouseDownIndex);

                    // カード表示反映
                    DisplayCardMove(inn, pnlYadoCard, mouseDownIndex);
                }
            }
        }
        /// <summary>
        /// 他宿カードパネル　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_MouseDown(object sender, MouseEventArgs e)
        {

            mouseDownIndex = -1;
            mouseDownPanel = pnlOtherInn;

            // 処理対象
            int indexOfX = (-pnlOtherInn.AutoScrollPosition.X + e.X) / CWData.HAND_CARD_WIDTH;
            int indexOfY = (-pnlOtherInn.AutoScrollPosition.Y + e.Y) / CWData.HAND_CARD_HEIGHT;

            index = indexOfY * disp_num_x + indexOfX;

            if (indexOfX >= disp_num_x || index >= pnlOtherInn.Controls.Count)
            {
                // 範囲外
                // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                if (e.Button == MouseButtons.Right)
                {

                    List<HandCard> hands = new List<HandCard>();
                    foreach (Object obj in this.pnlOtherInn.Controls)
                    {
                        hands.Add(((DispCard)obj).hc);
                    }

                    using (CardInfoDialog cid = new CardInfoDialog(hands, 0))
                    {
                        cid.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    // その後スクロールできるように現在選択中パネルを入れておく
                    mouseDownPanel = pnlOtherInn;

                }
            }
            else
            {
                // マウスの右ボタンだけが押されている時にCardInfoDialogで情報表示
                if (e.Button == MouseButtons.Right)
                {

                    List<HandCard> hands = new List<HandCard>();
                    foreach (Object obj in this.pnlOtherInn.Controls)
                    {
                        hands.Add(((DispCard)obj).hc);
                    }

                    using (CardInfoDialog cid = new CardInfoDialog(hands, index))
                    {
                        cid.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    //マウスの左ボタンだけが押されている時のみドラッグできるようにする
                    mouseDownIndex = index;
                    mouseDownPanel = pnlOtherInn;
                    mouseDownPoint = new Point(e.X, e.Y);
                }
                else if (e.Button == MouseButtons.Middle)
                {
                    // パネル指定先に送る
                    mouseDownIndex = index;

                    // 他宿カードをカード置場に移動
                    inn.SwapOtherInnCardAndCard(mouseDownIndex, -1);

                    // カード表示反映
                    DisplayCardMove(inn, pnlOtherInn, mouseDownIndex);

                }
            }

        }
        /// <summary>
        ///  カード置場パネル　マウスムーブイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoCard_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDownPoint != Point.Empty)
            {
                //ドラッグとしないマウスの移動範囲を取得する
                Rectangle moveRect = new Rectangle(
                    mouseDownPoint.X - SystemInformation.DragSize.Width / 2,
                    mouseDownPoint.Y - SystemInformation.DragSize.Height / 2,
                    SystemInformation.DragSize.Width,
                    SystemInformation.DragSize.Height);
                //ドラッグとする移動範囲を超えたか調べる
                if (!moveRect.Contains(e.X, e.Y))
                {

                    //ドラッグの準備
                    DispCard dcx = (DispCard)(pnlYadoCard.Controls[mouseDownIndex]);

                    //ドラッグ&ドロップ処理を開始する
                    DragDropEffects dde =
                        dcx.DoDragDrop(dcx, DragDropEffects.All);


                    mouseDownPoint = Point.Empty;
                }
            }

        }
        /// <summary>
        ///  他宿カードパネル　マウスムーブイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDownPoint != Point.Empty)
            {
                //ドラッグとしないマウスの移動範囲を取得する
                Rectangle moveRect = new Rectangle(
                    mouseDownPoint.X - SystemInformation.DragSize.Width / 2,
                    mouseDownPoint.Y - SystemInformation.DragSize.Height / 2,
                    SystemInformation.DragSize.Width,
                    SystemInformation.DragSize.Height);
                //ドラッグとする移動範囲を超えたか調べる
                if (!moveRect.Contains(e.X, e.Y) )
                {

                    //ドラッグの準備
                    DispCard dcx = (DispCard)(pnlOtherInn.Controls[mouseDownIndex]);

                    //ドラッグ&ドロップ処理を開始する
                    DragDropEffects dde =
                        dcx.DoDragDrop(dcx, DragDropEffects.All);


                    mouseDownPoint = Point.Empty;
                }
            }

        }
        /// <summary>
        /// 他宿カードパネル　マウスアップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_MouseUp(object sender, MouseEventArgs e)
        {
            mouseDownPoint = Point.Empty;
        }
        /// <summary>
        /// カード置場パネル　マウスアップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoCard_MouseUp(object sender, MouseEventArgs e)
        {
            mouseDownPoint = Point.Empty;
        }
        /// <summary>
        /// マウスホイールの移動量を取得
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OtherInn_MouseWheel(object sender, MouseEventArgs e)
        {
            if (mouseDownPanel == pnlOtherInn)
            {
                ScrollPnl(pnlOtherInn,-e.Delta);
            }
            else 
            {
                ScrollPnl(pnlYadoCard, -e.Delta);

            }
        }
        
        #endregion

        #region ドラッグドロップ関連

        /// <summary>
        /// 他宿カードパネル　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //CurrentParty がInSenarioでないかを調べ、
            //そうであればドロップ効果をCopyにする
            if (e.Data.GetDataPresent(typeof(DispCard)))
                e.Effect = DragDropEffects.Copy;
            else
                //DispCard型でなければ受け入れない
                e.Effect = DragDropEffects.None;
        }
        /// <summary>
        /// 他宿カードパネル　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_DragDrop(object sender, DragEventArgs e)
        {

            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象
                Point pointClient = pnlOtherInn.PointToClient(new Point(e.X,e.Y));
                int indexOfX = (-pnlOtherInn.AutoScrollPosition.X + pointClient.X) / CWData.HAND_CARD_WIDTH;
                int indexOfY = (-pnlOtherInn.AutoScrollPosition.Y + pointClient.Y) / CWData.HAND_CARD_HEIGHT;
                index = indexOfY * disp_num_x + indexOfX;


                // 他宿カードにドロップ
                if (indexOfX >= disp_num_x || index >= pnlOtherInn.Controls.Count)
                {
                    // 他宿カードの空きにドロップ

                    if (mouseDownPanel == pnlYadoCard)
                    {
                        // カード置場を他宿カード欄に移動
                        inn.SwapOtherInnCardAndCard(-1, mouseDownIndex);

                        // カード表示反映
                        DisplayCardMove(inn, pnlYadoCard, mouseDownIndex);
                        
                    }

                }
                else
                {

                    // 対象は他宿カード

                    IndexCardData icdOld;
                    IndexCardData icd;

                    if (mouseDownPanel == pnlYadoCard)
                    {
                        // カード置場カードと他宿カードの入れ替え
                        inn.SwapOtherInnCardAndCard(index, mouseDownIndex);

                        // 表示カードの変更
                        icdOld = inn.OtherInn.FindIndexedCardData(inn.OtherInn.BankBoxCards[index]);
                        ((DispCard)pnlOtherInn.Controls[index]).SetHc(icdOld.HandCard);
                        icd = inn.FindIndexedCardData(inn.BankBoxCards[mouseDownIndex]);
                        ((DispCard)pnlYadoCard.Controls[mouseDownIndex]).SetHc(icd.HandCard);


                    }
                    else
                    {
                        // 他宿カード内移動
                        inn.OtherInn.SwapCard(mouseDownIndex, index);

                        // 表示上のキャラクタ変更
                        icdOld = inn.OtherInn.FindIndexedCardData(inn.OtherInn.BankBoxCards[mouseDownIndex]);
                        ((DispCard)pnlOtherInn.Controls[mouseDownIndex]).SetHc(icdOld.HandCard);

                        icd = inn.OtherInn.FindIndexedCardData(inn.OtherInn.BankBoxCards[index]);
                        ((DispCard)pnlOtherInn.Controls[index]).SetHc(icd.HandCard);

                    }

                }


            }
        }


        /// <summary>
        /// カード置場パネル　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoCard_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //そうであればドロップ効果をMoveにする
            if (e.Data.GetDataPresent(typeof(DispCard)))
                e.Effect = DragDropEffects.Copy;
            else
                //string型でなければ受け入れない
                e.Effect = DragDropEffects.None;
        }
        /// <summary>
        /// カード置場パネル　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoCard_DragDrop(object sender, DragEventArgs e)
        {
            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象
                Point pointClient = pnlYadoCard.PointToClient(new Point(e.X, e.Y));
                int indexOfX = (-pnlYadoCard.AutoScrollPosition.X + pointClient.X) / CWData.HAND_CARD_WIDTH;
                int indexOfY = (-pnlYadoCard.AutoScrollPosition.Y + pointClient.Y) / CWData.HAND_CARD_HEIGHT;
                index = indexOfY * disp_num_x + indexOfX;

                // カード置場にドロップ
                if (indexOfX >= disp_num_x || index >= pnlYadoCard.Controls.Count)
                {

                    // カード置場の空きにドロップ

                    if (mouseDownPanel == pnlOtherInn)
                    {
                        // 他宿カードをカード置場に移動
                        inn.SwapOtherInnCardAndCard(mouseDownIndex, -1);

                        // カード表示反映
                        DisplayCardMove(inn, pnlOtherInn, mouseDownIndex);

                    }

                }
                else
                {
                    IndexCardData icdOld;
                    IndexCardData icd;

                    // カード置場のカードにドロップ
                    if (mouseDownPanel == pnlOtherInn)
                    {
                        // 他宿カードとカード置場カードの入れ替え
                        inn.SwapOtherInnCardAndCard(mouseDownIndex, index);

                        // 表示上のカード変更
                        icdOld = inn.OtherInn.FindIndexedCardData(inn.OtherInn.BankBoxCards[mouseDownIndex]);
                        ((DispCard)pnlOtherInn.Controls[mouseDownIndex]).SetHc(icdOld.HandCard);

                        icd = inn.FindIndexedCardData(inn.BankBoxCards[index]);
                        ((DispCard)pnlYadoCard.Controls[index]).SetHc(icd.HandCard);
                    }
                    else
                    {
                        // カード置場カードどうしなのでファイルに変化なし
                        inn.SwapCard(mouseDownIndex, index);

                        // 表示上のカード変更
                        icdOld = inn.FindIndexedCardData(inn.BankBoxCards[mouseDownIndex]);
                        ((DispCard)pnlYadoCard.Controls[mouseDownIndex]).SetHc(icdOld.HandCard);

                        icd = inn.FindIndexedCardData(inn.BankBoxCards[index]);
                        ((DispCard)pnlYadoCard.Controls[index]).SetHc(icd.HandCard);

                    }

                }

            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;

        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoCard_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;

        }

        #endregion



    }
}