﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class MoveCouponDlg : Form
    {
        #region メンバ

        private Form _owner;
        private List<CouponData> list1 = null;
        private List<CouponData> list2 = null;
        private DataTable dtChara1 = null;
        private DataTable dtChara2 = null;
        private bool couponFlg = true;
        private InnData innInfo;
        private ListType selectedType;
        #endregion

        public enum ListType
        {
            FinishList,
            ProgressList,
            GoshipList
        }

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MoveCouponDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// (宿ゴシップ編集/終了済リスト編集/進行中リスト編集)
        /// </summary>
        /// <param name="owner"></param>
        /// <param name="inn"></param>
        public MoveCouponDlg(Form owner,
            InnData inn, ListType type)
        {
            couponFlg = false;
            InitializeComponent();

            this.innInfo = inn;

            this.btnOK.Location = new Point(this.btnOK.Location.X / 2 - this.btnOK.Width / 2 - 20, 
                this.btnOK.Location.Y);
            this.btnEnd.Location = new Point(this.btnEnd.Location.X / 2 + this.btnEnd.Width / 2 - 20, 
                this.btnEnd.Location.Y);

            this.MinimumSize = new Size(300, 480);
            this.MaximumSize = new Size(300, 480);

            SetPosition(owner);

            this.lblChara2.Visible = false;
            this.btnNew2.Visible = false;
            this.btnDelete2.Visible = false;
            this.btnCopy2to1.Visible = false;
            this.btn2to1.Visible = false;
            this.btnCopy1to2.Visible = false;
            this.btn1to2.Visible = false;
            this.dgvCoupon2.Visible = false;

            this.lblChara1.Text = inn.Name;

            switch (type)
            {
                case ListType.FinishList:
                    this.Text = "終了済リスト編集";
                    break;
                case ListType.ProgressList:
                    this.Text = "進行中リスト編集";
                    break;
                case ListType.GoshipList:
                    this.Text = "宿ゴシップ編集";
                    break;
            }
            selectedType = type;
        }

        /// <summary>
        /// コンストラクタ
        /// (２キャラのクーポンリスト)
        /// </summary>
        /// <param name="owner"></param>
        /// <param name="chara1Name"></param>
        /// <param name="chara2Name"></param>
        /// <param name="list1"></param>
        /// <param name="list2"></param>
        public MoveCouponDlg(Form owner
            ,string chara1Name
            ,string chara2Name
            ,List<CouponData> list1
            ,List<CouponData> list2)
        {
            couponFlg = true;
            InitializeComponent();

            SetPosition(owner);

            this.lblChara1.Text = chara1Name;
            this.lblChara2.Text = chara2Name;

            this.list1 = list1;
            this.list2 = list2;

        }

        private void SetPosition(Form owner)
        {
            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void MoveCouponDlg_Load(object sender, EventArgs e) 
        {

            // 初期設定（グリッド）
            this.dgvCoupon1.RowHeadersVisible = false;
            this.dgvCoupon1.ColumnHeadersVisible = true;

            if (couponFlg == true)
            {
                // クーポン移動・修正
                this.dgvCoupon2.RowHeadersVisible = false;
                this.dgvCoupon2.ColumnHeadersVisible = true;

                dtChara1 = new DataTable();
                dtChara2 = new DataTable();
                DataColumn dataCol;

                string[] colHeaderNames
                = 
                    {
                        "クーポン",
                        "得点",
                    };

                dataCol = new DataColumn();
                dataCol.ColumnName = colHeaderNames[0];

                dataCol.DataType = typeof(string);
                dtChara1.Columns.Add(dataCol);


                dataCol = new DataColumn();
                dataCol.ColumnName = colHeaderNames[1];
                dataCol.DataType = typeof(int);
                dtChara1.Columns.Add(dataCol);


                dataCol = new DataColumn();
                dataCol.ColumnName = colHeaderNames[0];

                dataCol.DataType = typeof(string);
                dtChara2.Columns.Add(dataCol);


                dataCol = new DataColumn();
                dataCol.ColumnName = colHeaderNames[1];
                dataCol.DataType = typeof(int);
                dtChara2.Columns.Add(dataCol);

                DataRow dr;

                foreach (CouponData data in this.list1)
                {
                    dr = dtChara1.NewRow();

                    dr[0] = data.Name;

                    dr[1] = data.Point;

                    dtChara1.Rows.Add(dr);
                }

                foreach (CouponData data in this.list2)
                {
                    dr = dtChara2.NewRow();

                    dr[0] = data.Name;

                    dr[1] = data.Point;

                    dtChara2.Rows.Add(dr);
                }

                this.dgvCoupon1.DataSource = dtChara1;
                this.dgvCoupon2.DataSource = dtChara2;
                this.dgvCoupon1.Visible = true;
                this.dgvCoupon2.Visible = true;

            }
            else
            {
                // 宿ゴシップ修正
                dtChara1 = new DataTable();
                DataColumn dataCol;

                string[] colHeaderNames;
                
                switch (selectedType)
                {
                    case ListType.FinishList:
                        colHeaderNames = new string[] { "終了済シナリオ名" };
                        break;
                    case ListType.ProgressList:
                        colHeaderNames = new string[] { "進行中シナリオ名" };
                        break;
                    case ListType.GoshipList:
                        colHeaderNames  = new string[]{ "ゴシップ名" };
                        break;
                    default:
                        colHeaderNames = null;
                        break;
                 }


                dataCol = new DataColumn();
                dataCol.ColumnName = colHeaderNames[0];

                dataCol.DataType = typeof(string);
                dtChara1.Columns.Add(dataCol);

                DataRow dr;


                switch (selectedType)
                {
                    case ListType.FinishList:
                        foreach (string fin in this.innInfo.FinishScenarios)
                        {
                            dr = dtChara1.NewRow();

                            dr[0] = fin;

                            dtChara1.Rows.Add(dr);
                        }
                        break;
                    case ListType.ProgressList:
                        foreach (string progress in this.innInfo.ProgressScenarios)
                        {
                            dr = dtChara1.NewRow();

                            dr[0] = progress;

                            dtChara1.Rows.Add(dr);
                        }
                        break;
                    case ListType.GoshipList:
                        foreach (string goship in this.innInfo.Gossips)
                        {
                            dr = dtChara1.NewRow();

                            dr[0] = goship;

                            dtChara1.Rows.Add(dr);
                        }
                        break;
                    default:
                        break;
                }


                this.dgvCoupon1.DataSource = dtChara1;

                this.dgvCoupon1.Columns[0].Width = this.dgvCoupon1.Width - 20;

                this.dgvCoupon1.Visible = true;
            }
        }


        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MoveCouponDlg_Shown(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// キャンセルボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            //キャンセル時は反映させない
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }


        /// <summary>
        /// ＯＫボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            if (couponFlg)
            {
                this.list1.Clear();
                this.list2.Clear();
                CouponData data;

                foreach (DataRow dr in this.dtChara1.Rows)
                {
                    data = new CouponData();
                    data.Name = dr[0].ToString();
                    data.Point = int.Parse(dr[1].ToString());
                    list1.Add(data);
                }
                foreach (DataRow dr in this.dtChara2.Rows)
                {
                    data = new CouponData();
                    data.Name = dr[0].ToString();
                    data.Point = int.Parse(dr[1].ToString());
                    list2.Add(data);
                }
            }
            else
            {
                switch (selectedType)
                {
                    case ListType.FinishList:
                        innInfo.FinishScenarios.Clear();
                        foreach (DataRow dr in this.dtChara1.Rows)
                        {
                            innInfo.FinishScenarios.Add(dr[0].ToString());
                        }
                        break;
                    case ListType.ProgressList:
                        innInfo.ProgressScenarios.Clear();
                        foreach (DataRow dr in this.dtChara1.Rows)
                        {
                            innInfo.ProgressScenarios.Add(dr[0].ToString());
                        }
                        break;
                    case ListType.GoshipList:
                        innInfo.Gossips.Clear();
                        foreach (DataRow dr in this.dtChara1.Rows)
                        {
                            innInfo.Gossips.Add(dr[0].ToString());
                        }
                        break;
                    default:
                        break;
                }
            }

            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MoveCouponDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

        /// <summary>
        /// 移動1→2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn1to2_Click(object sender, EventArgs e)
        {

            if (dgvCoupon1.SelectedRows != null)
            {
                // コピー
                if (!Copy12()) return;

                // 削除
                foreach (DataGridViewRow dr1 in
                    ReverseList(dgvCoupon1, dgvCoupon1.SelectedCells))
                {
                    //逆順にしてRemoveAt
                    dgvCoupon1.Rows.RemoveAt(dr1.Index);
                }
            }
        }

        private List<DataGridViewRow> ReverseList(
            DataGridView dgv,
            DataGridViewSelectedCellCollection list) 
        {
            List<DataGridViewRow> newList = new List<DataGridViewRow>();
            List<int> indexes = new List<int>();
            foreach (DataGridViewCell cell in list)
            {
                if (!indexes.Contains(cell.RowIndex))
                {
                    indexes.Add(cell.RowIndex);
                }

                indexes.Sort();
                indexes.Reverse();
            }


            foreach (int index in indexes)
            {
                newList.Add(dgv.Rows[index]);
            }

            return newList;
        }

        /// <summary>
        /// コピー1→2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCopy1to2_Click(object sender, EventArgs e)
        {
            Copy12();
        }

        private bool Copy12()
        {
            if (dgvCoupon1.SelectedCells != null)
            {
                foreach (DataGridViewCell cell1 in dgvCoupon1.SelectedCells)
                {
                    foreach (DataRow dr2 in dtChara2.Rows)
                    {
                        if (dr2[0].ToString().Equals(dgvCoupon1
                            .Rows[cell1.RowIndex]
                            .Cells[0].Value.ToString()))
                        {
                            MessageDlg.ShowInfoDialog("同じ名前のクーポンがあります。" 
                                + Environment.NewLine + dr2[0].ToString());
                            return false;
                        }
                    }
                }


                DataRow dr;

                List<int> selectedRowIndexes = new List<int>();
                foreach (DataGridViewCell cell1 in dgvCoupon1.SelectedCells)
                {
                    if (!selectedRowIndexes.Contains(cell1.RowIndex))
                    {
                        selectedRowIndexes.Add(cell1.RowIndex);
                    }
                }

                // ソートがかかっていなくて逆順で取れていたので修正
                // CardWirth(カードワース)総合スレ Ver.1.19 の 65より
                selectedRowIndexes.Sort();

                foreach (int index in selectedRowIndexes)
                {
                    dr = dtChara2.NewRow();
                    dr[0] = dgvCoupon1.Rows[index].Cells[0].Value.ToString();
                    dr[1] = int.Parse(dgvCoupon1.Rows[index].Cells[1].Value.ToString());
                    dtChara2.Rows.Add(dr);
                }

                dgvCoupon2.DataSource = dtChara2;
            }

            return true;
        }

        /// <summary>
        /// 移動2→1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn2to1_Click(object sender, EventArgs e)
        {

            if (dgvCoupon2.SelectedRows != null)
            {
                // コピー
                if(!Copy21()) return;

                // 削除
                foreach (DataGridViewRow dr2 in
                    ReverseList(dgvCoupon2,dgvCoupon2.SelectedCells))
                {
                    //逆順にしてRemoveAt
                    dgvCoupon2.Rows.RemoveAt(dr2.Index);
                }
            }
        }

        /// <summary>
        /// コピー2→1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCopy2to1_Click(object sender, EventArgs e)
        {
            Copy21();
        }

        private bool Copy21()
        {

            if (dgvCoupon2.SelectedCells != null)
            {

                foreach (DataGridViewCell cell2 in dgvCoupon2.SelectedCells)
                {
                    foreach (DataRow dr1 in dtChara1.Rows)
                    {
                        if (dr1[0].ToString().Equals(
                            dgvCoupon2.Rows[cell2.RowIndex].Cells[0].Value.ToString()))
                        {
                            MessageDlg.ShowInfoDialog("同じ名前のクーポンがあります。"
                                + Environment.NewLine + dr1[0].ToString());
                            return false;
                        }
                    }
                }


                DataRow dr;
                List<int> selectedRowIndexes = new List<int>();
                foreach (DataGridViewCell cell2 in dgvCoupon2.SelectedCells)
                {
                    if (!selectedRowIndexes.Contains(cell2.RowIndex))
                    {
                        selectedRowIndexes.Add(cell2.RowIndex);
                    }
                }

                // ソートがかかっていなくて逆順で取れていたので修正
                // CardWirth(カードワース)総合スレ Ver.1.19 の 65より
                selectedRowIndexes.Sort();

                foreach (int index in selectedRowIndexes)
                {
                    dr = dtChara1.NewRow();
                    dr[0] = dgvCoupon2.Rows[index].Cells[0].Value.ToString();
                    dr[1] = int.Parse(dgvCoupon2.Rows[index].Cells[1].Value.ToString());
                    dtChara1.Rows.Add(dr);
                }

                dgvCoupon1.DataSource = dtChara1;
            }

            return true;
        }

        /// <summary>
        /// 新規1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNew1_Click(object sender, EventArgs e)
        {
            DataRow dr = dtChara1.NewRow();

            if (couponFlg)
            {
                dr[0] = NewCouponName();
                dr[1] = 0;
            }
            else
            {
                dr[0] = NewGoshipName();
            }

            dtChara1.Rows.Add(dr);
            dgvCoupon1.DataSource = dtChara1;
        }

        /// <summary>
        /// 新規2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNew2_Click(object sender, EventArgs e)
        {
            DataRow dr = dtChara2.NewRow();
            dr[0] = NewCouponName();
            dr[1] = 0;
            dtChara2.Rows.Add(dr);
            dgvCoupon2.DataSource = dtChara2;
        }

        private string NewCouponName()
        {
            string baseName = "新規クーポン";
            string tempName = baseName;
            int suffix = 1;
            bool isOK = false;
            bool isNotSame = true;
            while (!isOK)
            {
                isNotSame = true;
                foreach(DataRow dr in this.dtChara1.Rows)
                {
                    if (dr[0].ToString().Equals(tempName))
                    {
                        isNotSame = false;
                        break;
                    }
                }

                if (isNotSame)
                {
                    isOK = true;
                }
                else
                {
                    suffix++;
                    tempName = baseName + "(" + suffix + ")";
                }
            }

            return tempName;
        }

        private string NewGoshipName()
        {
            string baseName = "新規ゴシップ";
            string tempName = baseName;
            int suffix = 1;
            bool isOK = false;
            bool isNotSame = true;
            while (!isOK)
            {
                isNotSame = true;
                foreach (DataRow dr in this.dtChara1.Rows)
                {
                    if (dr[0].ToString().Equals(tempName))
                    {
                        isNotSame = false;
                        break;
                    }
                }

                if (isNotSame)
                {
                    isOK = true;
                }
                else
                {
                    suffix++;
                    tempName = baseName + "(" + suffix + ")";
                }
            }

            return tempName;
        }

        /// <summary>
        /// 削除１
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDelete1_Click(object sender, EventArgs e)
        {
            if (dgvCoupon1.SelectedCells != null)
            {

                List<int> selectedRowIndexes = new List<int>();
                foreach (DataGridViewCell cell1 in dgvCoupon1.SelectedCells)
                {
                    if (!selectedRowIndexes.Contains(cell1.RowIndex))
                    {
                        selectedRowIndexes.Add(cell1.RowIndex);
                    }
                }

                selectedRowIndexes.Sort();
                selectedRowIndexes.Reverse();

                foreach (int index in selectedRowIndexes)
                {
                    dgvCoupon1.Rows.RemoveAt(index);
                }
            }
        }

        /// <summary>
        /// 削除２
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDelete2_Click(object sender, EventArgs e)
        {
            if (dgvCoupon2.SelectedCells != null)
            {

                List<int> selectedRowIndexes = new List<int>();
                foreach (DataGridViewCell cell2 in dgvCoupon2.SelectedCells)
                {
                    if (!selectedRowIndexes.Contains(cell2.RowIndex))
                    {
                        selectedRowIndexes.Add(cell2.RowIndex);
                    }
                }

                selectedRowIndexes.Sort();
                selectedRowIndexes.Reverse();

                foreach (int index in selectedRowIndexes)
                {
                    dgvCoupon2.Rows.RemoveAt(index);
                }
            }
        }

        /// <summary>
        /// セルバリデーティング
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvCoupon_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {

            DataGridView view = (DataGridView)sender;
            if (e.RowIndex >= 0 && e.RowIndex < view.Rows.Count)
            {
                DataGridViewCell current =
                    view.Rows[e.RowIndex].Cells[e.ColumnIndex];

                switch (e.ColumnIndex)
                {
                    case 0:
                        // クーポン名
                        // ゴシップ名
                        string name = current.EditedFormattedValue.ToString();
                        bool isOK = true;

                        foreach (DataGridViewRow row in view.Rows)
                        {
                            if (e.RowIndex == row.Index) continue;
                            if (row.Cells[0].Value.ToString().Equals(name))
                            {
                                isOK = false;
                                break;
                            }
                        }

                        if (!isOK)
                        {
                            string mes;
                            if (couponFlg)
                            {
                                mes = "クーポン名が重なっています。";
                            }
                            else
                            {
                                mes = "ゴシップ名が重なっています。";
                            }
                            MessageDlg.ShowInfoDialog(mes);
                            e.Cancel = true;
                            return;
                        }

                        break;
                    case 1:
                        // 得点
                        try
                        {
                            int point = int.Parse(current.EditedFormattedValue.ToString());
                            if (point < -999 || point > 999)
                            {
                                throw new Exception();
                            }
                        }
                        catch (Exception)
                        {
                            MessageDlg.ShowInfoDialog("得点が範囲外です（-999～999）");
                            e.Cancel = true;
                            return;
                        }
                        break;
                    default:
                        return;
                }
            }
        }

        #endregion

    }
}