﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class MovePCDlg : Form
    {
        #region メンバ

        private int disp_num_x = 3;
        private int old_num_x = 3;
        private FormWindowState oldState = FormWindowState.Normal;
        
        /// <summary>
        /// 宿
        /// </summary>
        private InnData inn = null;

        /// <summary>
        /// 表示用の待機PC達
        /// </summary>
        private List<DispCard> yadoPCs;
        public List<DispCard> YadoPCs
        {
            set
            {
                yadoPCs = value;
            }
            get
            {
                return yadoPCs;
            }
        }

        /// <summary>
        /// 他宿
        /// </summary>
        private InnData otherInn;


        // 現在のインデックス
        private int index = 0;

        // マウスの押されたインデックス
        private int mouseDownIndex = -1;
        // マウスの押されたパネル
        private Panel mouseDownPanel = null;
        // マウスの押された座標
        private Point mouseDownPoint = Point.Empty;

        private static volatile ProgressDlg progress;

        private PartyMaker _pm;

        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MovePCDlg()
        {
            // 通常使用しない
            InitializeComponent();

            MouseWheel += new MouseEventHandler(Inn_MouseWheel);

            progress = new ProgressDlg(this);
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MovePCDlg(PartyMaker pm, InnData inn, List<DispCard> yadoPCs, InnData otherInn)
        {
            InitializeComponent();

            MouseWheel += new MouseEventHandler(Inn_MouseWheel);

            progress = new ProgressDlg(this);

            this.inn = inn;
            this.yadoPCs = yadoPCs;
            this.otherInn = otherInn;
            this._pm = pm;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void MovePCDlg_Load(object sender, EventArgs e)
        {
            // 宿名反映
            this.grpYadoPC.Text = inn.Name + " 待機PC(0)";
            this.grpOtherInn.Text = otherInn.Name + " 待機PC(0)";
            this.btnAllInn.Text = inn.Name + " へ";
            this.btnAllMove.Text = otherInn.Name + " へ"; 

            // マウス・カーソルの位置をスクリーン中央に移動させる
            int centerX = Screen.PrimaryScreen.Bounds.Right / 2;
            int centerY = Screen.PrimaryScreen.Bounds.Bottom / 2;
            Point cursolPosition = new Point(centerX, centerY);
            this.Location = new Point(centerX - this.Width / 2, centerY - this.Height / 2);

            // 最大化表示
            this.WindowState = FormWindowState.Maximized;

            // 一度走らせる
            MovePCDlg_Resize(sender, e);
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MovePCDlg_Shown(object sender, EventArgs e)
        {
            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = otherInn.Name + " 待機PCを読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            // 待機PCはinnに読み込み済

            // 対象他宿待機PC設定
            inn.SetOtherInnCharacter(otherInn);

            // 宿 待機PC表示
            index = 0;
            foreach (DispCard dc in yadoPCs)
            {
                dc.Location = new Point(CWData.DispCard_Width * (index % disp_num_x)
                    , CWData.DispCard_Height * (index / disp_num_x));
                index++;
                pnlYadoPC.Controls.Add(dc);
            }

            this.grpYadoPC.Text = inn.Name + " 待機PC(" + inn.Characters.Count + ")";

            progress.Value = 50;

            // 他宿待機PC表示
            SetUpOtherInn();

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MovePCDlg_FormClosing(object sender, FormClosingEventArgs e)
        {

            List<DispCard> list = new List<DispCard>();
            foreach (Control ctrl in pnlYadoPC.Controls)
            {
                list.Add(ctrl as DispCard);
            }
            yadoPCs = list;

            // 他宿PC関連はクリア
            inn.ClearOtherInnCharacter();
        }

        private void MovePCDlg_KeyDown(object sender, KeyEventArgs e)
        {
            //KeyCode + Modifiers = KeyData
            if ((e.Modifiers & Keys.Control) == Keys.Control)
            {

                if (e.KeyCode == Keys.Up)
                {
                    if (mouseDownPanel == this.pnlOtherInn)
                    {
                        ScrollPnl(pnlOtherInn, -100);
                    }
                    else
                    {
                        ScrollPnl(pnlYadoPC, -100);
                    }

                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Down)
                {
                    if (mouseDownPanel == this.pnlOtherInn)
                    {
                        ScrollPnl(pnlOtherInn, 100);
                    }
                    else
                    {
                        ScrollPnl(pnlYadoPC, 100);
                    }

                    e.Handled = true;
                }
            }

            if (e.KeyCode == Keys.PageUp)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, -100);
                }
                else
                {
                    ScrollPnl(pnlYadoPC, -100);
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.PageDown)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, 100);
                }
                else
                {
                    ScrollPnl(pnlYadoPC, 100);
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Home)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, -CWData.HAND_CARD_HEIGHT * ((pnlOtherInn.Controls.Count - 1) / disp_num_x + 1));
                }
                else
                {
                    ScrollPnl(pnlYadoPC, -CWData.HAND_CARD_HEIGHT * ((pnlYadoPC.Controls.Count - 1) / disp_num_x + 1));
                }

                e.Handled = true;
            }
            else if (e.KeyCode == Keys.End)
            {
                if (mouseDownPanel == this.pnlOtherInn)
                {
                    ScrollPnl(pnlOtherInn, CWData.HAND_CARD_HEIGHT * ((pnlOtherInn.Controls.Count - 1) / disp_num_x + 1));
                }
                else
                {
                    ScrollPnl(pnlYadoPC, CWData.HAND_CARD_HEIGHT * ((pnlYadoPC.Controls.Count - 1) / disp_num_x + 1));
                }

                e.Handled = true;
            }
        }

        private void MovePCDlg_Resize(object sender, EventArgs e)
        {

            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {
                if (oldState != this.WindowState)
                {
                    oldState = this.WindowState;
                    MovePCDlg_ResizeEnd(sender, e);
                }

                int baseWidth = this.Width / 2;

                this.grpOtherInn.Width = baseWidth - 10;
                this.grpOtherInn.Height = this.Height - 63;

                this.grpYadoPC.Location = new Point(baseWidth, this.grpYadoPC.Location.Y);

                this.grpYadoPC.Width = baseWidth - 10;
                this.grpYadoPC.Height = this.Height - 63;


                this.pnlOtherInn.Width = baseWidth - 26;
                this.pnlOtherInn.Height = this.Height - 86;
                this.pnlYadoPC.Width = baseWidth - 26;
                this.pnlYadoPC.Height = this.Height - 86;

                this.btnAllMove.Location
                = new Point(this.ClientSize.Width - this.btnAllMove.Width - 12
                    , this.ClientSize.Height - 28);

                this.btnAllInn.Location
                = new Point(this.btnAllInn.Location.X, this.ClientSize.Height - 28);

                this.btnEnd.Location = new Point(
                    (this.ClientSize.Width - this.btnEnd.Width) / 2,
                    this.ClientSize.Height - 28);

                ////this.btnSort.Location = new Point(
                ////    this.ClientSize.Width - 85, this.ClientSize.Height - 28);

                ////this.cmbOrder.Location = new Point(
                ////    this.ClientSize.Width - 85 - 174, this.ClientSize.Height - 26);

                this.disp_num_x = (this.pnlYadoPC.Width - 6 - 10) / CWData.HAND_CARD_WIDTH;


            }
        }

        private void MovePCDlg_ResizeEnd(object sender, EventArgs e)
        {

            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {
                MovePCDlg_Resize(sender, e);

                SetLocationPnlYadoPC();
                SetLocationPnlOtherInn();

            }
        }

        private void MovePCDlg_ResizeBegin(object sender, EventArgs e)
        {

            old_num_x = this.disp_num_x;
        }

        #endregion

        #region ボタンイベント

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            // FormClosingに集約
            this.Close();
        }
        /// <summary>
        /// 全員現在の宿へボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllInn_Click(object sender, EventArgs e)
        {
            int count = inn.OtherInn.Characters.Count;
            if (count > 0)
            {
                // 進捗ダイアログ開始
                StartProgress();
                // 進捗ダイアログ初期処理待ち
                WaitProgress();
                progress.Message = "全員 " + inn.Name + " に移動中です。";
                progress.Maximum = 100;
                progress.Minimum = 0;
                progress.Value = 0;


                // 全他宿待機PCを現在の宿へ
                inn.MoveAllOtherInnPC();

                // 表示反映
                Control[] array = new Control[pnlOtherInn.Controls.Count];
                pnlOtherInn.Controls.CopyTo(array, 0);

                pnlOtherInn.Controls.Clear();

                this.grpOtherInn.Text = otherInn.Name + " 待機PC(0)";


                // 表示用カード追加・配置修正
                int nowX = -pnlYadoPC.AutoScrollPosition.X;
                int nowY = -pnlYadoPC.AutoScrollPosition.Y;

                pnlYadoPC.AutoScrollPosition = new Point(0, 0);

                progress.Value = 50;

                pnlYadoPC.Controls.AddRange(array);

                DispCard dc;
                index = 0;
                for (int i = inn.Characters.Count - count; i < inn.Characters.Count; i++)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = otherInn.Name + " 待機PC：" + (index + 1) + "人目を移動しています。";

                    dc = (DispCard)array[index];
                    dc.SetCd(inn.Characters[i]);

                    dc.Location = new Point(CWData.DispCard_Width
                        * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));

                    //プログレスバーの値を変更する
                    index++;
                    progress.Value = 50 + index * 50 / count;
                }

                this.grpYadoPC.Text = inn.Name + " 待機PC(" + inn.Characters.Count + ")";

                pnlYadoPC.AutoScrollPosition = new Point(nowX, nowY);

                // 進捗ダイアログ終了
                EndProgress();

            }
        }

        /// <summary>
        /// 全員他宿へ移動ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllMove_Click(object sender, EventArgs e)
        {
            int count = inn.Characters.Count;
            if (count > 0)
            {
                // 進捗ダイアログ開始
                StartProgress();
                // 進捗ダイアログ初期処理待ち
                WaitProgress();
                progress.Message = "全員 " + otherInn.Name + " に移動中です。";
                progress.Maximum = 100;
                progress.Minimum = 0;
                progress.Value = 0;

                // 全待機PCを他宿に移動
                inn.MoveAllPCToOtherInn();

                // 表示反映
                Control[] array = new Control[pnlYadoPC.Controls.Count];
                pnlYadoPC.Controls.CopyTo(array, 0);

                pnlYadoPC.Controls.Clear();

                this.grpYadoPC.Text = inn.Name + " 待機PC(0)";


                // 表示用カード追加・配置修正
                int nowX = -pnlOtherInn.AutoScrollPosition.X;
                int nowY = -pnlOtherInn.AutoScrollPosition.Y;

                pnlOtherInn.AutoScrollPosition = new Point(0, 0);

                progress.Value = 50;

                pnlOtherInn.Controls.AddRange(array);

                DispCard dc; 
                index = 0;
                for (int i = inn.OtherInn.Characters.Count - count; i < inn.OtherInn.Characters.Count; i++)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = "待機PC：" + (index + 1) + "人目を移動しています。";

                    dc = (DispCard)array[index];
                    dc.SetCd(inn.OtherInn.Characters[i]);

                    dc.Location = new Point(CWData.DispCard_Width
                        * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                    

                    //プログレスバーの値を変更する
                    index++;
                    progress.Value = 50 + index * 50 / count;
                }

                this.grpOtherInn.Text = otherInn.Name + " 待機PC(" + inn.OtherInn.Characters.Count + ")";

                pnlOtherInn.AutoScrollPosition = new Point(nowX, nowY);


                // 進捗ダイアログ終了
                EndProgress();
            }
        }

        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {
            backgroundWorker1.RunWorkerAsync();
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();


            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion

        #region 表示と配置
        /// <summary>
        /// 他宿待機PC設定
        /// </summary>
        private void SetUpOtherInn()
        {

            pnlOtherInn.Controls.Clear();
            pnlOtherInn.AutoScrollPosition = new Point(0, 0);
            int count = inn.OtherInn.Characters.Count;
            if (count != 0)
            {

                progress.Message = otherInn.Name + " 待機PCを表示中です。";

                this.pnlOtherInn.SuspendLayout();

                index = 0;
                foreach (CharacterData cd in inn.OtherInn.Characters)
                {
                    // プログレスバーのメッセージを変更する
                    progress.Message = otherInn.Name + " 待機PC：" + (index + 1) + "人目を移動しています。";


                    DispCard dc = new DispCard(cd);
                    dc.Location = new Point(CWData.DispCard_Width * (index % disp_num_x)
                    , CWData.DispCard_Height * (index / disp_num_x));

                    index++;
                    pnlOtherInn.Controls.Add(dc);

                    //プログレスバーの値を変更する
                    progress.Value = 50 + index * 50 / count;
                }

                this.pnlOtherInn.ResumeLayout();

            }

            this.grpOtherInn.Text = otherInn.Name + " 待機PC(" + count + ")";


        }
        /// <summary>
        /// 待機PC再配置
        /// </summary>
        private void SetLocationPnlYadoPC()
        {

            int nowX = -pnlYadoPC.AutoScrollPosition.X;
            int nowY = -pnlYadoPC.AutoScrollPosition.Y;

            pnlYadoPC.AutoScrollPosition = new Point(0, 0);
            index = 0;
            foreach (Control ctrl in pnlYadoPC.Controls)
            {
                ctrl.Location = new Point(CWData.DispCard_Width * (index % disp_num_x),
                    CWData.DispCard_Height * (index / disp_num_x));
                index++;
            }

            pnlYadoPC.AutoScrollPosition = new Point(nowX, nowY);

        }
        /// <summary>
        /// 他宿待機PC再配置
        /// </summary>
        private void SetLocationPnlOtherInn()
        {

            int nowX = -pnlOtherInn.AutoScrollPosition.X;
            int nowY = -pnlOtherInn.AutoScrollPosition.Y;

            pnlOtherInn.AutoScrollPosition = new Point(0, 0);
            index = 0;
            foreach (Control ctrl in pnlOtherInn.Controls)
            {
                ctrl.Location = new Point(CWData.DispCard_Width * (index % disp_num_x),
                    CWData.DispCard_Height * (index / disp_num_x));
                index++;
            }

            pnlOtherInn.AutoScrollPosition = new Point(nowX, nowY);

        }

        private void ScrollPnl(Panel pnl, int delta)
        {
            if (pnl == this.pnlOtherInn)
            {
                ScrollPnlOtherInn(delta);
            }
            else
            {
                ScrollPnlYadoPC(delta);
            }
        }
        /// <summary>
        /// 移動量分だけスクロール(待機PC)
        /// </summary>
        /// <param name="delta"></param>
        private void ScrollPnlYadoPC(int delta)
        {
            int nowX = -pnlYadoPC.AutoScrollPosition.X;
            int nowY = -pnlYadoPC.AutoScrollPosition.Y;
            nowY += delta;
            if (nowY < 0)
            {
                nowY = 0;
            }

            if (nowY > CWData.DispCard_Height * ((pnlYadoPC.Controls.Count - 1) / disp_num_x + 1))
            {
                nowY = CWData.DispCard_Height * ((pnlYadoPC.Controls.Count - 1) / disp_num_x + 1);
            }

            pnlYadoPC.AutoScrollPosition = new Point(nowX, nowY);
            
        }
        /// <summary>
        /// 移動量分だけスクロール(他宿PC)
        /// </summary>
        /// <param name="delta"></param>
        private void ScrollPnlOtherInn(int delta)
        {
            int nowX = -pnlOtherInn.AutoScrollPosition.X;
            int nowY = -pnlOtherInn.AutoScrollPosition.Y;
            nowY += delta;
            if (nowY < 0)
            {
                nowY = 0;
            }

            if (nowY > CWData.DispCard_Height * ((pnlOtherInn.Controls.Count - 1) / disp_num_x + 1))
            {
                nowY = CWData.DispCard_Height * ((pnlOtherInn.Controls.Count - 1) / disp_num_x + 1);
            }

            pnlOtherInn.AutoScrollPosition = new Point(nowX, nowY);

        }
        /// <summary>
        /// PCの単独移動を表示反映する
        /// </summary>
        private void DisplayPCMove(InnData inn,Panel oldPanel,int oldIndex)
        {

            int nowX = -pnlYadoPC.AutoScrollPosition.X;
            int nowY = -pnlYadoPC.AutoScrollPosition.Y;
            int nowOtherInnX = -pnlOtherInn.AutoScrollPosition.X;
            int nowOtherInnY = -pnlOtherInn.AutoScrollPosition.Y;

            pnlYadoPC.AutoScrollPosition = new Point(0, 0);
            pnlOtherInn.AutoScrollPosition = new Point(0, 0);

            DispCard addDc;
            int newIndex;
            if (oldPanel == this.pnlOtherInn)
            {
                // 他宿PCから待機PCへ
                pnlOtherInn.Controls.RemoveAt(oldIndex);
                for (int i = oldIndex; i < pnlOtherInn.Controls.Count; i++)
                {
                    // 他宿PCの元の位置より後ろのカードだけ配置修正
                    pnlOtherInn.Controls[i].Location = 
                        new Point(CWData.DispCard_Width * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                }

                // すでにSwap処理でListの最後に追加されているので
                // それを元に表示用カードを作成
                newIndex = inn.Characters.Count - 1;
                addDc = new DispCard(inn.Characters[newIndex]);
                addDc.Location = new Point(CWData.DispCard_Width * (newIndex % disp_num_x), CWData.DispCard_Height * (newIndex / disp_num_x));
                pnlYadoPC.Controls.Add(addDc);
            }
            else
            {

                // 待機PCから他宿PCへ
                pnlYadoPC.Controls.RemoveAt(oldIndex);
                for (int i = oldIndex; i < pnlYadoPC.Controls.Count; i++)
                {
                    // 待機PCの元の位置より後ろのカードだけ配置修正
                    pnlYadoPC.Controls[i].Location = 
                        new Point(CWData.DispCard_Width * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                }

                // すでにSwap処理でListの最後に追加されているので
                // それを元に表示用カードを作成
                newIndex = inn.OtherInn.Characters.Count - 1;
                addDc = new DispCard(inn.OtherInn.Characters[newIndex]);
                addDc.Location = new Point(CWData.DispCard_Width * (newIndex % disp_num_x), CWData.DispCard_Height * (newIndex / disp_num_x));
                pnlOtherInn.Controls.Add(addDc);

            }


            this.grpYadoPC.Text = inn.Name + " 待機PC(" + inn.Characters.Count + ")";
            this.grpOtherInn.Text = otherInn.Name + " 待機PC(" + inn.OtherInn.Characters.Count + ")";

            pnlYadoPC.AutoScrollPosition = new Point(nowX, nowY);

            pnlOtherInn.AutoScrollPosition = new Point(nowOtherInnX, nowOtherInnY);
        }

        #endregion

        #region マウスイベント

        /// <summary>
        ///  待機PCパネル　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoPC_MouseDown(object sender, MouseEventArgs e)
        {

            mouseDownIndex = -1;

            // 処理対象
            int indexOfX = (-pnlYadoPC.AutoScrollPosition.X + e.X) / CWData.DispCard_Width;
            int indexOfY = (-pnlYadoPC.AutoScrollPosition.Y + e.Y) / CWData.DispCard_Height;

            index = indexOfY * disp_num_x + indexOfX;

            if (indexOfX >= disp_num_x || index >= pnlYadoPC.Controls.Count)
            {
                // 範囲外
                // マウスの右ボタンだけが押されている時にPCDetailDlgで情報表示
                if (e.Button == MouseButtons.Right)
                {
                    using (PcDetailDlg pcd = new PcDetailDlg(_pm, inn,inn.Characters,0))
                    {
                        pcd.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    // その後スクロールできるように現在選択中パネルを入れておく
                    mouseDownPanel = pnlYadoPC;
                }
            }
            else
            {
                // PCを指している
                // マウスの右ボタンだけが押されている時にPCDetailDlgで情報表示
                if (e.Button == MouseButtons.Right)
                {
                    using (PcDetailDlg pcd = new PcDetailDlg(_pm, inn, inn.Characters, index))
                    {
                        pcd.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    mouseDownIndex = index;
                    mouseDownPanel = pnlYadoPC;
                    mouseDownPoint = new Point(e.X, e.Y);

                }
            }
        }
        /// <summary>
        /// 他宿PCパネル　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_MouseDown(object sender, MouseEventArgs e)
        {

            mouseDownIndex = -1;

            // 処理対象
            int indexOfX = (-pnlOtherInn.AutoScrollPosition.X + e.X) / CWData.DispCard_Width;
            int indexOfY = (-pnlOtherInn.AutoScrollPosition.Y + e.Y) / CWData.DispCard_Height;

            index = indexOfY * disp_num_x + indexOfX;

            if (indexOfX >= disp_num_x || index >= pnlOtherInn.Controls.Count)
            {
                // 範囲外
                // マウスの右ボタンだけが押されている時にPCDetailDlgで情報表示
                if (e.Button == MouseButtons.Right)
                {
                    using (PcDetailDlg pcd = new PcDetailDlg(_pm, inn, inn.OtherInn.Characters, 0))
                    {
                        pcd.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    // その後スクロールできるように現在選択中パネルを入れておく
                    mouseDownPanel = pnlOtherInn;

                }
            }
            else
            {
                // マウスの右ボタンだけが押されている時にPCDetailDlgで情報表示
                if (e.Button == MouseButtons.Right)
                {
                    using (PcDetailDlg pcd = new PcDetailDlg(_pm, inn, inn.OtherInn.Characters, index))
                    {
                        pcd.ShowDialog();
                    }
                }
                else if (e.Button == MouseButtons.Left)
                {
                    //マウスの左ボタンだけが押されている時のみドラッグできるようにする
                    mouseDownIndex = index;
                    mouseDownPanel = pnlOtherInn;
                    mouseDownPoint = new Point(e.X, e.Y);
                }
            }

        }
        /// <summary>
        ///  待機PCパネル　マウスムーブイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoPC_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDownPoint != Point.Empty)
            {
                //ドラッグとしないマウスの移動範囲を取得する
                Rectangle moveRect = new Rectangle(
                    mouseDownPoint.X - SystemInformation.DragSize.Width / 2,
                    mouseDownPoint.Y - SystemInformation.DragSize.Height / 2,
                    SystemInformation.DragSize.Width,
                    SystemInformation.DragSize.Height);
                //ドラッグとする移動範囲を超えたか調べる
                if (!moveRect.Contains(e.X, e.Y))
                {

                    //ドラッグの準備
                    DispCard dcx = (DispCard)(pnlYadoPC.Controls[mouseDownIndex]);

                    //ドラッグ&ドロップ処理を開始する
                    DragDropEffects dde =
                        dcx.DoDragDrop(dcx, DragDropEffects.All);


                    mouseDownPoint = Point.Empty;
                }
            }

        }
        /// <summary>
        ///  他宿PCパネル　マウスムーブイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDownPoint != Point.Empty)
            {
                //ドラッグとしないマウスの移動範囲を取得する
                Rectangle moveRect = new Rectangle(
                    mouseDownPoint.X - SystemInformation.DragSize.Width / 2,
                    mouseDownPoint.Y - SystemInformation.DragSize.Height / 2,
                    SystemInformation.DragSize.Width,
                    SystemInformation.DragSize.Height);
                //ドラッグとする移動範囲を超えたか調べる
                if (!moveRect.Contains(e.X, e.Y) )
                {

                    //ドラッグの準備
                    DispCard dcx = (DispCard)(pnlOtherInn.Controls[mouseDownIndex]);

                    //ドラッグ&ドロップ処理を開始する
                    DragDropEffects dde =
                        dcx.DoDragDrop(dcx, DragDropEffects.All);


                    mouseDownPoint = Point.Empty;
                }
            }

        }
        /// <summary>
        /// 他宿PCパネル　マウスアップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_MouseUp(object sender, MouseEventArgs e)
        {
            mouseDownPoint = Point.Empty;
        }
        /// <summary>
        /// 待機PCパネル　マウスアップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoPC_MouseUp(object sender, MouseEventArgs e)
        {
            mouseDownPoint = Point.Empty;
        }
        /// <summary>
        /// マウスホイールの移動量を取得
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Inn_MouseWheel(object sender, MouseEventArgs e)
        {
            if (mouseDownPanel == pnlOtherInn)
            {
                ScrollPnlOtherInn(-e.Delta);
            }
            else if (mouseDownPanel == pnlYadoPC)
            {
                ScrollPnlYadoPC(-e.Delta);

            }
        }
        
        #endregion

        #region ドラッグドロップ関連

        /// <summary>
        /// 他宿PCパネル　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //CurrentParty がInSenarioでないかを調べ、
            //そうであればドロップ効果をCopyにする
            if (e.Data.GetDataPresent(typeof(DispCard)))
                e.Effect = DragDropEffects.Copy;
            else
                //DispCard型でなければ受け入れない
                e.Effect = DragDropEffects.None;
        }
        /// <summary>
        /// 他宿PCパネル　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_DragDrop(object sender, DragEventArgs e)
        {

            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象
                Point pointClient = pnlOtherInn.PointToClient(new Point(e.X,e.Y));
                int indexOfX = (-pnlOtherInn.AutoScrollPosition.X + pointClient.X) / CWData.DispCard_Width;
                int indexOfY = (-pnlOtherInn.AutoScrollPosition.Y + pointClient.Y) / CWData.DispCard_Height;
                index = indexOfY * disp_num_x + indexOfX;


                // 他宿PCにドロップ
                if (indexOfX >= disp_num_x || index >= pnlOtherInn.Controls.Count)
                {
                    // 他宿PCの空きにドロップ

                    if (mouseDownPanel == pnlYadoPC)
                    {
                        // 待機PCを他宿PC欄に移動
                        inn.SwapOtherPCAndPC(-1, mouseDownIndex);

                        // PC表示反映
                        DisplayPCMove(inn, pnlYadoPC, mouseDownIndex);
                        
                    }
 

                }
                else
                {

                    // 対象は他宿PCのカード

                    if (mouseDownPanel == pnlYadoPC)
                    {
                        // 待機PCカードと他宿PCのカードの入れ替え
                        inn.SwapOtherPCAndPC(index, mouseDownIndex);

                        // 表示カードの変更
                        ((DispCard)pnlOtherInn.Controls[index]).SetCd(inn.OtherInn.Characters[index]);
                        ((DispCard)pnlYadoPC.Controls[mouseDownIndex]).SetCd(inn.Characters[mouseDownIndex]);

                    }
                    else
                    {
                        // 他宿PCのカード内移動
                        inn.SwapOtherInnCharacter(mouseDownIndex, index);

                        // 表示上のキャラクタ変更
                        ((DispCard)pnlOtherInn.Controls[mouseDownIndex]).SetCd(inn.OtherInn.Characters[mouseDownIndex]);
                        ((DispCard)pnlOtherInn.Controls[index]).SetCd(inn.OtherInn.Characters[index]);

                    }

                }


            }
        }
        /// <summary>
        /// 待機PCパネル　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoPC_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //そうであればドロップ効果をMoveにする
            if (e.Data.GetDataPresent(typeof(DispCard)))
                e.Effect = DragDropEffects.Copy;
            else
                //string型でなければ受け入れない
                e.Effect = DragDropEffects.None;
        }
        /// <summary>
        /// 待機PCパネル　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoPC_DragDrop(object sender, DragEventArgs e)
        {
            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象
                Point pointClient = pnlYadoPC.PointToClient(new Point(e.X, e.Y));
                int indexOfX = (-pnlYadoPC.AutoScrollPosition.X + pointClient.X) / CWData.DispCard_Width;
                int indexOfY = (-pnlYadoPC.AutoScrollPosition.Y + pointClient.Y) / CWData.DispCard_Height;
                index = indexOfY * disp_num_x + indexOfX;

                // 待機PCにドロップ
                if (indexOfX >= disp_num_x || index >= pnlYadoPC.Controls.Count)
                {

                    // 待機PCの空きにドロップ

                    if (mouseDownPanel == pnlOtherInn)
                    {
                        // 他宿PCのカードを待機PCに移動
                        inn.SwapOtherPCAndPC(mouseDownIndex, -1);

                        // カード表示反映
                        DisplayPCMove(inn, pnlOtherInn, mouseDownIndex);

                    }

                }
                else
                {
                    // 待機PCのカードにドロップ
                    if (mouseDownPanel == pnlOtherInn)
                    {
                        // 他宿PCのカードと待機PCカードの入れ替え
                        inn.SwapOtherPCAndPC(mouseDownIndex, index);

                        // 表示上のカード変更
                        ((DispCard)pnlOtherInn.Controls[mouseDownIndex]).SetCd(inn.OtherInn.Characters[mouseDownIndex]);
                        ((DispCard)pnlYadoPC.Controls[index]).SetCd(inn.Characters[index]);
                    }
                    else
                    {
                        // 待機PCカードどうしなのでファイルに変化なし
                        inn.SwapCharacter(mouseDownIndex, index);

                        // 表示上のカード変更
                        ((DispCard)pnlYadoPC.Controls[mouseDownIndex]).SetCd(inn.Characters[mouseDownIndex]);
                        ((DispCard)pnlYadoPC.Controls[index]).SetCd(inn.Characters[index]);

                    }

                }

            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlOtherInn_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;

        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PnlYadoPC_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;

        }

        #endregion






    }
}