﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using CWI;
using System.IO;
using System.Collections.Specialized;

namespace CWI.Dialog
{
    public partial class NewInnDlg : Form
    {
        public YadoSelectData[] datas = null;
        public NewInnDlg(YadoSelectData[] oldDatas)
        {
            InitializeComponent();
            this.BackgroundImage = CWIEngine.Properties.Resources.MessageDlgBackGround;// フォームの背景を変える
            this.TxtYadoName.SelectAll();
            this.datas = oldDatas;
        }

        private void BtnStop_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Abort;
        }

        private void BtnReg_Click(object sender, EventArgs e)
        {
            string name = this.TxtYadoName.Text;
            if (FilePath.isProhibition(name))
            {
                MessageDlg.ShowInfoDialog("禁則文字が含まれています。");
                return;
            }

            if (datas == null)
            {
                datas = new YadoSelectData[0];
            }

            if (datas.Select(data => data.Name).Contains(name)) {
                MessageDlg.ShowInfoDialog("同名の冒険者の宿が既に存在しています。名前を変更してください。");
                return;
            }

            YadoSelectData ysd = new YadoSelectData();
            ysd.Folder = FilePath.GetSystemYadoFolder() + "\\" + name;
            ysd.Name = FilePath.FileNameFromPath(ysd.Folder); 
            ysd.IsDebugYado = this.RdoDbg.Checked;
            ysd.Members = new StringCollection();
            ysd.LastAccessTime = DateTime.Now;

            InnData inn = new InnData(TxtYadoName.Text, true, this.RdoDbg.Checked);

            if (Directory.Exists(inn.Folder))
            {
                MessageDlg.ShowInfoDialog("すでにその名前の宿フォルダが存在します。");
                return;
            }
            else
            {
                // 宿フォルダ作成
                Directory.CreateDirectory(inn.Folder);

                // ないが一応削除を入れて
                FilePath.DeleteFile(inn.EnvironmentFile);
                inn.Write(inn.EnvironmentFile);

                List<YadoSelectData> lst = new List<YadoSelectData>();
                lst.Add(ysd);
                lst.AddRange(datas);
                datas = lst.ToArray();
            }

            this.DialogResult = DialogResult.OK;
        }

        private void TxtYadoName_TextChanged(object sender, EventArgs e)
        {
            if(TxtYadoName.Text == "")
            {
                this.BtnReg.Enabled = false;
            } else
            {
                this.BtnReg.Enabled = true;
            }
        }

        private void NewInnDlg_Load(object sender, EventArgs e)
        {
            // マウス・カーソルの位置をスクリーン中央に移動させる
            int centerX = Screen.PrimaryScreen.Bounds.Right / 2;
            int centerY = Screen.PrimaryScreen.Bounds.Bottom / 2;
            Point cursolPosition = new Point(centerX, centerY);
            this.Location = new Point(centerX - this.Width / 2, centerY - this.Height / 2);
        }
    }
}
