﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Threading;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class PartyDataListDlg : Form
    {
        #region メンバ

        private const int FORM_MIN_WIDTH = 640;
        private const int FORM_MIN_HEIGHT = 480;
        private const int LABEL_HEIGHT = 20;

        private int disp_num_x = 8;

        private static volatile ProgressDlg progress;

        private PartyData current;

        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public PartyDataListDlg()
        {
            // 通常使用しない
            InitializeComponent();

            progress = new ProgressDlg(this);

            MouseWheel += new MouseEventHandler(PartyData_MouseWheel);
        }

        /// <summary>
        /// コンストラクタ（カレントパーティー）
        /// </summary>
        public PartyDataListDlg(PartyData current)
        {
            InitializeComponent();

            progress = new ProgressDlg(this);

            MouseWheel += new MouseEventHandler(PartyData_MouseWheel);

            this.current = current;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void PartyDataListDlg_Load(object sender, EventArgs e)
        {

            // マウス・カーソルの位置をスクリーン中央に移動させる
            int centerX = Screen.PrimaryScreen.Bounds.Right / 2;
            int centerY = Screen.PrimaryScreen.Bounds.Bottom / 2;
            Point cursolPosition = new Point(centerX, centerY);
            
            this.Location = new Point(centerX - this.Width / 2, centerY - this.Height / 2);

        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyDataListDlg_Shown(object sender, EventArgs e)
        {
            ShowPartyPanel();

            // デフォルト最大化表示
            this.WindowState = FormWindowState.Maximized;
        }

        /// <summary>
        /// パーティーデータ表示
        /// </summary>
        public void ShowPartyPanel()
        {
            
            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "画像を読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            // 画像表示

            pnlPartyData.AutoScrollPosition = new Point(0, 0);
            pnlPartyData.Controls.Clear();

            Label label = null;

            if (current != null)
            {
                this.Text = "パーティーデータ一覧 " + current.PartyName;

                int dispY = 0;

                int cindex = 0;
                int count = current.Characters.Count;
                foreach (CharacterData cd in current.Characters)
                {
                    label = new Label();
                    label.ForeColor = Color.WhiteSmoke;
                    label.Height = LABEL_HEIGHT;
                    label.Text = cd.CardName;
                    label.TextAlign = ContentAlignment.MiddleLeft;
                    label.Location = new Point(0, dispY);
                    dispY += LABEL_HEIGHT;

                    pnlPartyData.Controls.Add(label);
                    int index = 0;
                    for (byte i = 0; i <= (byte)手札種類.所有召喚獣; i++)
                    {
                        label = new Label();
                        label.Height = LABEL_HEIGHT;
                        label.Text = ((手札種類)i).ToString().Substring(2);
                        label.TextAlign = ContentAlignment.MiddleLeft;

                        label.Location = new Point(0, dispY);
                        dispY += LABEL_HEIGHT;

                        pnlPartyData.Controls.Add(label);

                        DispCard dc = null;
                        SkillHandCard shc = null;
                        ItemHandCard ihc = null;
                        BeastHandCard bhc = null;

                        index = 0;
                        switch (i)
                        {
                            case 0:
                                //スキル
                                foreach (WidCard wc in cd.Skills)
                                {
                                    shc = new SkillHandCard(wc);

                                    shc.UpdateImage(
                                        GameParam.GetCardFacterColor(cd,
                                        shc.widData.BodyFacter, shc.widData.MindFacter),
                                        GameParam.GetSkillRemainColor(shc.widData.SkillRemainUseCount,
                                        GameParam.GetMaxSkillUseCount(cd.Level, shc.widData.Level)));

                                    dc = new DispCard(shc);
                                    dc.Location = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x)
                                        , dispY + CWData.HAND_CARD_HEIGHT * (index / disp_num_x));

                                    pnlPartyData.Controls.Add(dc);

                                    index++;
                                }

                                dispY += CWData.HAND_CARD_HEIGHT
                                    * HeightCount(cd.Skills.Count);
                                break;
                            case 1:
                                //アイテム
                                foreach (WidCard wc in cd.Items)
                                {
                                    ihc = new ItemHandCard(wc);

                                    ihc.UpdateImage(
                                        GameParam.GetCardFacterColor(cd,
                                        ihc.widData.BodyFacter, ihc.widData.MindFacter));

                                    dc = new DispCard(ihc);
                                    dc.Location
                                        = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x)
                                        , dispY + CWData.HAND_CARD_HEIGHT * (index / disp_num_x));

                                    pnlPartyData.Controls.Add(dc);

                                    index++;
                                }

                                dispY += CWData.HAND_CARD_HEIGHT
                                    * HeightCount(cd.Items.Count);
                                break;
                            case 2:
                                //召喚獣
                                foreach (WidCard wc in cd.Beasts)
                                {
                                    bhc = new BeastHandCard(wc);

                                    bhc.UpdateImage(
                                        GameParam.GetCardFacterColor(cd,
                                        bhc.widData.BodyFacter, bhc.widData.MindFacter));

                                    dc = new DispCard(bhc);
                                    dc.Location
                                        = new Point(CWData.HAND_CARD_WIDTH * (index % disp_num_x)
                                        , dispY + CWData.HAND_CARD_HEIGHT * (index / disp_num_x));

                                    pnlPartyData.Controls.Add(dc);

                                    index++;
                                }

                                dispY += CWData.HAND_CARD_HEIGHT
                                    * HeightCount(cd.Beasts.Count);
                                break;
                        }
                    }

                    progress.Value = 100 * ++cindex / count;
                }
            }
            else
            {
                this.Text = "パーティーデータ一覧";
            }

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

        }

        private int HeightCount(int count)
        {
            return count != 0 ? (count - 1) / disp_num_x + 1: 0 ; 
        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            // FormClosingに集約
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyDataListDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            // リソース開放があったらここで
        }

        /// <summary>
        /// フォームリサイズ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyDataListDlg_Resize(object sender, EventArgs e)
        {

            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {

                this.pnlPartyData.Width = this.Width - 30;
                this.pnlPartyData.Height = this.Height - 70;

                this.btnEnd.Location = new Point(
                    (this.ClientSize.Width - this.btnEnd.Width) / 2,
                    this.ClientSize.Height - 28);

                int old_num_x = this.disp_num_x;
                this.disp_num_x = (this.pnlPartyData.Width - 10) / CWData.HAND_CARD_WIDTH;

                if (old_num_x != this.disp_num_x)
                {
                    SetLocationPnlObject();
                }

            }
        }


        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {
            backgroundWorker1.CancelAsync();
            while (this.backgroundWorker1.IsBusy)
            {
                Thread.Sleep(1);
            }

            backgroundWorker1.RunWorkerAsync();
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();
            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion

        #region 表示と配置

        /// <summary>
        /// 画像再配置
        /// </summary>
        private void SetLocationPnlObject()
        {

            int nowX = -pnlPartyData.AutoScrollPosition.X;
            int nowY = -pnlPartyData.AutoScrollPosition.Y;

            pnlPartyData.AutoScrollPosition = new Point(0, 0);

            int index = 0;
            int dispY = 0;

            if (current != null)
            {
                foreach (CharacterData cd in current.Characters)
                {
                    //CardName
                    pnlPartyData.Controls[index++].Location = new Point(0, dispY);
                    dispY += LABEL_HEIGHT;

                    //Skill
                    pnlPartyData.Controls[index++].Location = new Point(0, dispY);
                    dispY += LABEL_HEIGHT;

                    for (int i = 0; i < cd.Skills.Count; i++)
                    {
                        pnlPartyData.Controls[index++].Location
                            = new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                                ,dispY + CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                    }

                    dispY += CWData.HAND_CARD_HEIGHT
                        * HeightCount(cd.Skills.Count);

                    //Item
                    pnlPartyData.Controls[index++].Location = new Point(0, dispY);
                    dispY += LABEL_HEIGHT;

                    for (int i = 0; i < cd.Items.Count; i++)
                    {
                        pnlPartyData.Controls[index++].Location
                            = new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                                , dispY + CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                    }

                    dispY += CWData.HAND_CARD_HEIGHT
                        * HeightCount(cd.Items.Count);

                    //Beast
                    pnlPartyData.Controls[index++].Location = new Point(0, dispY);
                    dispY += LABEL_HEIGHT;

                    for (int i = 0; i < cd.Beasts.Count; i++)
                    {
                        pnlPartyData.Controls[index++].Location
                            = new Point(CWData.HAND_CARD_WIDTH * (i % disp_num_x)
                                , dispY + CWData.HAND_CARD_HEIGHT * (i / disp_num_x));
                    }

                    dispY += CWData.HAND_CARD_HEIGHT
                        * HeightCount(cd.Beasts.Count);

                }
            }

            pnlPartyData.AutoScrollPosition = new Point(nowX, nowY);

        }

        /// <summary>
        /// 移動量分だけスクロール
        /// </summary>
        /// <param name="delta"></param>
        private void ScrollPnlPartyData(int delta)
        {
            int nowX = -pnlPartyData.AutoScrollPosition.X;
            int nowY = -pnlPartyData.AutoScrollPosition.Y;

            nowY += delta;

            if (nowY < 0)
            {
                nowY = 0;
            }

            if (nowY > PartyTotalHeight(current))
            {
                nowY = PartyTotalHeight(current);
            }
            
            pnlPartyData.AutoScrollPosition = new Point(nowX, nowY);
            
        }

        private int PartyTotalHeight(PartyData current)
        {
            int dispY = 0;
            if (current == null) return dispY;

            foreach (CharacterData cd in current.Characters)
            {
                dispY += LABEL_HEIGHT * 4 +
                CWData.HAND_CARD_WIDTH *
                (HeightCount(cd.Skills.Count) +
                HeightCount(cd.Items.Count) +
                HeightCount(cd.Beasts.Count));
            }

            return dispY;
        }

        #endregion

        #region マウスイベント
        /// <summary>
        /// マウスホイールの移動量を取得
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyData_MouseWheel(object sender, MouseEventArgs e)
        {
            ScrollPnlPartyData(-e.Delta);
        }
        
        #endregion

    }
}