﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;

using CWIDatas;
using CWI.Card;
using CWI.Button;
using CWI;

namespace CWI.Dialog
{
    public partial class PcDialog : CWI.Dialog.SelectableForm
    {
        #region フィールド
        private const string TO_BE_CONTINUED = "・・・";
        private readonly int MaxMemberCount = 6;

        private bool isSelectable = false;
        

        // 表示すべき要素
        private CharacterData data;
        private List<CharacterData> datas = new List<CharacterData>();

        // 宿
        private InnData inn;

        private Form owner;
        private PartyMaker _pm;
        #endregion

        #region プロパティ
        /// <summary>
        /// 選択キャラクタリスト
        /// </summary>
        private List<CharacterData> selectedDatas = new List<CharacterData>();
        public List<CharacterData> SelectedDatas
        {
            get
            {
                return selectedDatas;
            }
        }
        /// <summary>
        /// 選択キャラクタ実インデックス
        /// </summary>
        private List<int> selectedIndexes = new List<int>();
        public List<int> SelectedIndexes
        {
            get
            {
                return selectedIndexes;
            }
        }

        /// <summary>
        /// 選択宿名(宿移動用)
        /// </summary>
        private string selectYadoNameForMove = string.Empty;
        public string SelectYadoName
        {
            get
            {
                return selectYadoNameForMove;
            }
        }
        /// <summary>
        /// 選択移動キャラクタ(宿移動用)
        /// </summary>
        private CharacterData selectMoveCharacter = null;
        public CharacterData SelectMoveCharacter
        {
            get
            {
                return selectMoveCharacter;
            }
        }

        /// <summary>
        /// 選択キャラクタインデックス(宿移動用)
        /// </summary>
        private int selectedIndex = -1;
        public int SelectedIndex
        {
            get
            {
                return selectedIndex;
            }
        }
        #endregion

        public PcDialog()
        {
            // 通常使用しない
            InitializeComponent();
        }

        public PcDialog(Form owner, InnData inn,bool isSelectable)
        {
            this.owner = owner;
            _pm = owner as PartyMaker;
            InnToCharas(inn, isSelectable);
            index = 0;
        }

        public PcDialog(Form owner, InnData inn, int index, bool isSelectable)
        {
            this.owner = owner;
            _pm = owner as PartyMaker;
            InnToCharas(inn, isSelectable);
            base.index = index;
        }

        private void InnToCharas(InnData inn, bool isSelectable)
        {

            InitializeComponent();

            this.inn = inn;
            this.isSelectable = isSelectable;

            //ボタン設定
            BtnSelect.Visible = isSelectable;

            BtnInfo.Visible = true;

            // 宿移動を外しておく
            BtnInnMove.Visible = false;
            
            // 編集
            BtnEdit.Visible = true;

            // 適性
            BtnAttr.Visible = true;
            BtnAttr.Text = "適性";
            BtnAttr.Width = 53;
            BtnAttr.Height = 23;

            // 詳細
            BtnInfo.Text = "詳細";
            BtnInfo.Width = 53;
            BtnInfo.Height = 23;

            this.BtnDoping.Visible = !isSelectable;

            if (isSelectable)
            {
                // こちらは待機PC
                // 編成追加
                BtnSelect.Text = "編成";
                BtnSelect.Width = 53;
                tablist.Add(BtnSelect);

                // 編集追加
                BtnEdit.Text = "編集";
                BtnEdit.Width = 53;
                BtnEdit.Height = 23;
                tablist.Add(BtnEdit);

                // 適性追加
                if (_pm.paramOption)
                {
                    // 追加
                    tablist.Add(BtnAttr);
                }
                else
                {
                    BtnAttr.Visible = false;
                }

                // 詳細追加
                tablist.Add(BtnInfo);

                count = inn.Characters.Count;
                foreach (CharacterData cd in inn.Characters)
                {
                    datas.Add(cd);
                }
            }
            else
            {

                // こちらはカレントPTのPC
                if (_pm.paramOption)
                {
                    // アンビリーバブル
                    // 制限解除 追加
                    BtnDoping.Text = "解除";
                    BtnDoping.Width = 53;
                    BtnDoping.Height = 23;
                    tablist.Add(BtnDoping);
                }
                else
                {
                    BtnDoping.Visible = false;
                }

                if (inn.CurrentParty != null && inn.CurrentParty.InScenario)
                {
                    // シナリオ進行中は表示しない
                    BtnEdit.Visible = false;
                }
                else
                {
                    // 編集追加
                    BtnEdit.Text = "編集";
                    BtnEdit.Width = 53;
                    BtnEdit.Height = 23;
                    tablist.Add(BtnEdit);
                }

                // 適性追加
                if (_pm.paramOption)
                {
                    // 追加
                    tablist.Add(BtnAttr);
                }
                else
                {
                    BtnAttr.Visible = false;
                }

                // 詳細追加
                tablist.Add(BtnInfo);

                count = inn.CurrentParty.Characters.Count;
                foreach (CharacterData cd in inn.CurrentParty.Characters)
                {
                    datas.Add(cd);
                }
            }


            BtnCancel.Text = "閉じる";
            tablist.Add(BtnCancel);
            SetUpPosition();//位置合わせ

            SetUpData(); 
        }


        protected override void SetUpData()
        {
            base.SetUpData();
        }


        protected override void doCancel()
        {

            if (selectedDatas.Count > 0)
            {
                this.DialogResult = DialogResult.OK;
            }
            else
            {
                this.DialogResult = DialogResult.Cancel;
            }
            
            this.Close();
        }
        

        protected override void doSelectPage()
        {
            if (isSelectable)
            {
                // パーティー編成するときのみ

                // １人でもパーティー編成用に選択されたら宿移動は出来ない
                if(BtnInnMove.Visible) {
                    BtnInnMove.Visible = false;
                    tablist.Remove(BtnInnMove);
                    SetUpPosition();//位置合わせ
                }

                SetSelectData();
            }
        }
        
        /// <summary>
        /// index位置のキャラをパーティー候補に追加
        /// </summary>
        protected void SetSelectData()
        {
            CharacterData selectedChara = datas[index];

            selectedDatas.Add(selectedChara);

            selectedIndexes.Add(inn.GetCharacterIndex(selectedChara));

            datas.RemoveAt(index);

            count = datas.Count;

            if (selectedDatas.Count == MaxMemberCount
                || count == 0)
            {
                this.DialogResult = DialogResult.OK;
                this.Close();
            }

            if (index >= count)
            {
                index = count - 1;
            }

            ShowPage();
        }

        protected override void ShowPage()
        {
            using (Graphics g = Graphics.FromImage(ViewBitmap))
            {
                using (Bitmap book = (Bitmap)Image.FromFile(FilePath.GetBookFile()))
                {
                    g.DrawImage(book, ViewPanel.ClientRectangle);
                }
                
                if (count > 0)
                {
                    data = datas[index];

                    using (Font smallFont = new Font("ＭＳ 明朝", 9, FontStyle.Bold))
                    {
                        string sex = "";//性別 Male or Female
                        string age = "";//世代 Child or Young or Adult or Old
                        string ep = "0";// EP 
                        string type = string.Empty;//素質型
                        string codon = string.Empty;//コドン
                        int times = 0;//交配経験数
                        List<String> coupNames = new List<string>();
                        foreach (CouponData coup in data.Coupons)
                        {//クーポン繰り返し
                            string s = coup.Name;
                            if (!Regex.IsMatch(s, "(＿|＠)"))
                            {//sに　＠か＿が含まれてなかったら
                                coupNames.Add(s);
                            }
                            else
                            {//＠＆＿だけ判断
                                //見たままです
                                if (s.Equals(CWData.STR_MALE)) { sex = "Male"; }
                                if (s.Equals(CWData.STR_FEMALE)) { sex = "Female"; }

                                if (s.Equals(CWData.STR_CHILD)) { age = "Child"; }
                                if (s.Equals(CWData.STR_YOUNG)) { age = "Young"; }
                                if (s.Equals(CWData.STR_ADULT)) { age = "Adult"; }
                                if (s.Equals(CWData.STR_OLD)) { age = "Old"; }

                                if (s.Equals(CWData.TYPE_NORMAL)) { type = CWData.DISP_NORMAL; }
                                if (s.Equals(CWData.TYPE_ALLMIGHTY)) { type = CWData.DISP_ALLMIGHTY; }
                                if (s.Equals(CWData.TYPE_RESOURCEFUL)) { type = CWData.DISP_RESOURCEFUL; }
                                if (s.Equals(CWData.TYPE_BRAVE)) { type = CWData.DISP_BRAVE; }
                                if (s.Equals(CWData.TYPE_DARING)) { type = CWData.DISP_DARING; }
                                if (s.Equals(CWData.TYPE_TACTIC)) { type = CWData.DISP_TACTIC; }
                                if (s.Equals(CWData.TYPE_BRILLIANT)) { type = CWData.DISP_BRILLIANT; }
                                if (s.Equals(CWData.TYPE_MATCHLESS)) { type = CWData.DISP_MATCHLESS; }
                                if (s.Equals(CWData.TYPE_GENIUS)) { type = CWData.DISP_GENIUS; }
                                if (s.Equals(CWData.TYPE_ORDINARY)) { type = CWData.DISP_ORDINARY; }
                                if (s.Equals(CWData.TYPE_HERO)) { type = CWData.DISP_HERO; }
                                if (s.Equals(CWData.TYPE_GOD)) { type = CWData.DISP_GOD; }

                                if (s.Equals(CWData.STR_EP)) { ep = coup.Point.ToString(); }

                                if (Regex.IsMatch(s, CWData.GENE_HEADER + ".*"))
                                {
                                    codon = coup.Name.Trim(CWData.GENE_HEADER.ToCharArray());
                                    times = coup.Point;
                                }
                            }
                        }


                        // History
                        int y = 30;//称号表示用Ｙ座標

                        if (_pm.paramOption)
                        {
                            // 怪しい表示モード
                            y = 25;
                            g.DrawString("パラメータ", smallFont, SolidBlack, 300, y);

                            //１２個は入る
                            y += 15;
                            g.DrawString("器用度  : " + data.Dex, smallFont, SolidBlack, 240, y);// 器用度
                            y += 15;
                            g.DrawString("敏捷性  : " + data.Agl, smallFont, SolidBlack, 240, y);// 敏捷性 
                            y += 15;
                            g.DrawString("知力    : " + data.Int, smallFont, SolidBlack, 240, y);// 知力
                            y += 15;
                            g.DrawString("筋力    : " + data.Str, smallFont, SolidBlack, 240, y);// 筋力
                            y += 15;
                            g.DrawString("生命力  : " + data.Vit, smallFont, SolidBlack, 240, y);// 生命力
                            y += 15;
                            g.DrawString("精神力  : " + data.Min, smallFont, SolidBlack, 240, y);// 精神力
                            y += 15;
                            g.DrawString("回避修正: " + data.AvoidBonus, smallFont, SolidBlack, 240, y);// キャスト性能としての回避修正
                            y += 15;
                            g.DrawString("抵抗修正: " + data.ResistBonus, smallFont, SolidBlack, 240, y);// キャスト性能としての抵抗修正 
                            y += 15;
                            g.DrawString("防御修正: " + data.ArmorBonus, smallFont, SolidBlack, 240, y);//  キャスト性能としての防御修正
                            y += 15;
                            g.DrawString("好戦性  : " + data.Aggressive, smallFont, SolidBlack, 240, y);//  好戦平和性
                            y += 15;
                            g.DrawString("社交性  : " + data.Cheerful, smallFont, SolidBlack, 240, y);//  社交内向性
                            y += 15;
                            g.DrawString("勇猛性  : " + data.Brave, smallFont, SolidBlack, 240, y);//  勇猛臆病性
                            y += 15;
                            g.DrawString("慎重性  : " + data.Cautious, smallFont, SolidBlack, 240, y);//  慎重大胆性
                            y += 15;
                            g.DrawString("狡猾性  : " + data.Trickish, smallFont, SolidBlack, 240, y);//  狡猾正直性

                            y = 40;
                            g.DrawString("武器無効: " + BoolString(data.ResistPhysical), smallFont, SolidBlack, 330, y);// 武器無効
                            y += 15;
                            g.DrawString("魔法無効: " + BoolString(data.ResistMagic), smallFont, SolidBlack, 330, y);// 魔法無効 
                            y += 15;
                            g.DrawString("肉体無し: " + BoolString(data.ResistBody), smallFont, SolidBlack, 330, y);// 命を持たない
                            y += 15;
                            g.DrawString("精神無し: " + BoolString(data.ResistMind), smallFont, SolidBlack, 330, y);// 心を持たない
                            y += 15;
                            g.DrawString("不浄存在: " + BoolString(data.WeakUnholy), smallFont, SolidBlack, 330, y);// 不浄な存在
                            y += 15;
                            g.DrawString("魔法生物: " + BoolString(data.WeakMagiclife), smallFont, SolidBlack, 330, y);// 魔法生物
                            y += 15;
                            g.DrawString("炎耐性　: " + BoolString(data.ResistHot), smallFont, SolidBlack, 330, y);// 耐炎
                            y += 15;
                            g.DrawString("冷気耐性: " + BoolString(data.ResistCold), smallFont, SolidBlack, 330, y);// 耐冷
                            y += 15;
                            g.DrawString("炎弱点　: " + BoolString(data.WeakHot), smallFont, SolidBlack, 330, y);// 弱炎
                            y += 15;
                            g.DrawString("冷気弱点: " + BoolString(data.WeakCold), smallFont, SolidBlack, 330, y);// 弱冷

                            y += 30;
                            g.DrawString("GENE   : " + codon, smallFont, SolidBlack, 330, y);// 遺伝配列
                            y += 15;
                            g.DrawString("TIMES  : " + times, smallFont, SolidBlack, 330, y);// 交配経験数 


                            g.DrawString("Age", smallFont, SolidBlack, 43, 199);
                            g.DrawString(age, smallFont, SolidBlack, 69, 199);
                            g.DrawString("Sex", smallFont, SolidBlack, 43, 214);
                            g.DrawString(sex, smallFont, SolidBlack, 69, 214);
                            g.DrawString("EP ", smallFont, SolidBlack, 43, 229);
                            g.DrawString(ep, smallFont, SolidBlack, 69, 229);
                            g.DrawString(type, smallFont, SolidBlack, 43, 244);

                            LevelInfo info = Util1.GetLevelInfo(data);
                            g.DrawString("MaxLV", smallFont, SolidBlack, 130, 199);
                            g.DrawString(info.MaxLv.ToString(), smallFont, SolidBlack, 170, 199);
                            g.DrawString("MaxHP", smallFont, SolidBlack, 130, 214);
                            g.DrawString(data.MaxHitPoint.ToString(), smallFont, SolidBlack, 170, 214);
                            g.DrawString("Exp", smallFont, SolidBlack, 130, 229);
                            g.DrawString(info.Exp.ToString(), smallFont, SolidBlack, 170, 229);
                            g.DrawString("Next", smallFont, SolidBlack, 130, 244);
                            g.DrawString(info.Next.ToString(), smallFont, SolidBlack, 170, 244);

                        }
                        else
                        {
                            // 通常表示

                            g.DrawString("History", smallFont, SolidBlack, 320, y);
                            y += 15;
                            for (int coupIndex = 0; coupIndex < coupNames.Count && coupIndex < 12; coupIndex++, y += 15)
                            {
                                //１２個
                                g.DrawString(coupNames[coupIndex], smallFont, SolidBlack, 270, y);//描画

                            }

                            if (coupNames.Count > 12)
                            {
                                g.DrawString(TO_BE_CONTINUED, smallFont, SolidBlack, 270, y);//描画
                            }

                            g.DrawString("Age", smallFont, SolidBlack, 104, 199);
                            g.DrawString(age, smallFont, SolidBlack, 130, 199);
                            g.DrawString("Sex", smallFont, SolidBlack, 104, 214);
                            g.DrawString(sex, smallFont, SolidBlack, 130, 214);
                            g.DrawString("EP", smallFont, SolidBlack, 104, 229);
                            g.DrawString(ep, smallFont, SolidBlack, 130, 229);
                            g.DrawString(type, smallFont, SolidBlack, 104, 244);
                        }

                        g.DrawString("Level", smallFont, SolidBlack, 43, 35);
                        g.DrawString("Adventurer", smallFont, SolidBlack, 121, 35);

                        using (StringFormat format = new StringFormat())
                        {
                            format.Alignment = StringAlignment.Center;//中揃え
                            Rectangle rectCount = new Rectangle(0, 254, this.ClientSize.Width, smallFont.Height);
                            g.DrawString((index + 1) + " / " + count, smallFont, SolidBlack, rectCount, format);//数の描画
                        }

                    }

                    // ここら辺は後で要センタリング
                    using (Font levelFont = new Font("ＭＳ 明朝", 16, FontStyle.Bold))
                    {
                        g.DrawString(data.Level.ToString() + " ", levelFont, SolidBlack, 80, 30);//レベル。
                    }

                    using (Font largeFont = new Font("ＭＳ 明朝", 16, FontStyle.Bold))
                    {
                        g.DrawString(data.CardName, largeFont, SolidBlack, 70 + (5 - data.CardName.Length) * 7, 60);//名前
                    }

                    //画像があったら。ない場合ってあるのかな？
                    if (data.Image != null && data.Image.bitmap != null)
                    {
                        g.DrawImage(data.Image.bitmap, 93, 91);

                        // グリュエルで女の子を出す場合
                        g.DrawImage(data.Image.bitmap,
                            // ViewPanel内の表示位置
                            new Rectangle(93, 91,
                                CWData.Bitmap_Width, CWData.Bitmap_Height),
                            new Rectangle(0, 0,
                                CWData.Bitmap_Width, CWData.Bitmap_Height),
                            GraphicsUnit.Pixel);

                        ////// グリュエルで大熊猫を出す場合
                        ////g.DrawImage(data.Image.bitmap,
                        ////    // ViewPanel内の表示位置
                        ////    new Rectangle(93, 91,
                        ////        CWData.Bitmap_Width, CWData.Bitmap_Height),
                        ////    new Rectangle(data.Image.bitmap.Width / 2 - CWData.Bitmap_Width / 2,
                        ////        data.Image.bitmap.Height / 2 - CWData.Bitmap_Height / 2,
                        ////        CWData.Bitmap_Width, CWData.Bitmap_Height),
                        ////    GraphicsUnit.Pixel);

                        ////// グリュエルで女の子（裸）を出す場合
                        ////g.DrawImage(data.Image.bitmap,
                        ////    // ViewPanel内の表示位置
                        ////    new Rectangle(93, 91,
                        ////        CWData.Bitmap_Width, CWData.Bitmap_Height),
                        ////    new Rectangle(data.Image.bitmap.Width / 2 - CWData.Bitmap_Width / 2,
                        ////        data.Image.bitmap.Height - CWData.Bitmap_Height,
                        ////        CWData.Bitmap_Width, CWData.Bitmap_Height),
                        ////    GraphicsUnit.Pixel);
                    }
                }
            }
            ViewPanel.Invalidate();

        }

        private string BoolString(bool state)
        {
            if(state) { 
                return "T";
            } else {
                return "F";
            }
        }

        private void PcDialog_Shown(object sender, EventArgs e)
        {

            // マウス・カーソルの位置をクライアント中央に移動させる
            Cursor.Position = new Point(this.Location.X + this.Width / 2, this.Location.Y + this.Height); 

            ShowPage();
        }

        /// <summary>
        /// 詳細
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnInfo_Click(object sender, EventArgs e)
        {
            using (PcDetailDlg pdd = new PcDetailDlg(_pm, inn, this.datas, this.index))
            {
                pdd.ShowDialog();
                this.index = pdd.SelectedIndex;
            }
            ShowPage();
        }

        /// <summary>
        /// 適性
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnAttr_Click(object sender, EventArgs e)
        {

            using (ExpressionDlg func = new ExpressionDlg(this, this.datas[this.index]))
            {
                func.ShowDialog();
            }
        }


        private void BtnInnMove_Click(object sender, EventArgs e)
        {
            if (isSelectable)
            {

                // 待機PCの場合のみ
                if (inn != null)
                {
                    if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
                    "移動を終了した時に宿を保存しますが、\r\nよろしいですか？"
                    ))
                    {

                        // 保存してもいい場合

                        // 宿選択ダイアログ(元宿名付き)
                        using (YadoSelectDlg ysd = new YadoSelectDlg(this,inn.Name))
                        {
                            if (DialogResult.OK == ysd.ShowDialog())
                            {
                                // 実際の処理は表示PCの削除もあるのでダイアログの外で行う
                                selectYadoNameForMove = ysd.SelectedYado.Name;
                                selectMoveCharacter = datas[index];
                                selectedIndex = index;
                                // キャンセルと同じ扱いにしておく
                                this.DialogResult = DialogResult.Cancel;
                                this.Close();
                            }
                        }

                    }
                }
            }

        }

        private void BtnDelete_Click(object sender, EventArgs e)
        {
            if (isSelectable)
            {
                // 待機PCの場合のみ
                if (inn != null)
                {
                    if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
                    "を削除します。よろしいですか？"
                    ))
                    {
                        // 削除してもいい場合
                        // 他の機能との兼ね合いでここには削除が
                        // 入れにくいのでここではしない
                    }
                }
            }
        }

        private void BtnEdit_Click(object sender, EventArgs e)
        {

            if (!isSelectable)
            {
                //パーティーPC

                using (EditDialog ed = new EditDialog(inn.CurrentParty.Characters[index], isSelectable))
                {
                    if (DialogResult.OK == ed.ShowDialog())
                    {
                        // パーティーメーカーメインの表示カードに反映
                        PartyMaker pm = (PartyMaker)owner;
                        pm.PinPointChangeParty(index);
                        ShowPage();
                    }
                }
            }
            else
            {
                // 待機PC

                using (EditDialog ed = new EditDialog(datas[index], isSelectable))
                {
                    if (DialogResult.OK == ed.ShowDialog())
                    {
                        // ファイルに結果を反映
                        inn.AddCharacterFile(datas[index]);
                        // パーティーメーカーメインの表示カードに反映
                        PartyMaker pm = (PartyMaker)owner;
                        int realIndex = inn.GetCharacterIndex(datas[index]);

                        pm.PinPointChangeCharacter(realIndex);
                        ShowPage();
                    }
                }

            }
        }

        private void BtnDoping_Click(object sender, EventArgs e)
        {
            if (!isSelectable)
            {
                // パーティーPC
                // 個人的な改造用機能のため確認後
                if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
                "選択中のPCのレベル制限解除します。\r\n通常使用しないでください。\r\nよろしいですか？"
                ))
                {
                    inn.CurrentParty.Characters[index].SetCoupon(CWData.STR_LEVEL_MAX, 999);
                }
            }
        }
    }
}
