﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class ScenarioSearchDlg : Form
    {
        #region メンバ
        public bool ScenarioSelect = false;
        public string SelectedFolder = "";

        private Form _owner;
        private PartyMaker _pm = null;

        private static volatile ProgressDlg progress;

        public DataTable ScenarioCacheDt;

        private const string FINISH = "シナリオ終了済";
        private const string PROGRESSION = "シナリオ進行中";
        private const string LACK_OF_COUPONS = "クーポン不足";
        private const string OUT_OF_RANGE = "推奨レベル範囲外";

        private string[] ColumnNames = 
            {
                "Scenario",
                "Author",
                "Low",
                "High",
                "Version",
                "CanPlay",
                "Expression",
                "Folder",
                "LastWrite",
                "Coupon1",
                "Coupon2",
                "Coupon3",
                "Coupon4",
                "Coupon5"
            };

        private string[] Captions = 
            {
                "シナリオ名",
                "作者名",
                "Low",
                "High",
                "Ver",
                "プレイ可能？",
                "解説",
                "フォルダ",
                "更新日時",
                "必要クーポン１",
                "必要クーポン２",
                "必要クーポン３",
                "必要クーポン４",
                "必要クーポン５"
            };


        private string[] CardColumnNames = 
            {
                "CardName",
                "Match",
                "Hand",
                "Expression",
                "BasePath",
                "InnFullName"
            };

        private string[] CardCaptions = 
            {
                "対象カード名",
                "マッチ",
                "所持",
                "解説",
                "元ファイルパス",
                "宿でのファイル名"
            };

        private bool widthFirst = false;

        private InnData inn;

        private List<List<string>> matchList = null;
        ToolTip toolTip1 = null;
        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ScenarioSearchDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ScenarioSearchDlg(Form owner, InnData inn, PartyMaker pm)
        {
            InitializeComponent();

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            this.inn = inn;
            this._pm = pm;

            progress = new ProgressDlg(this);
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void ScenarioSearchDlg_Load(object sender, EventArgs e)
        {

            this.WindowState = FormWindowState.Maximized;

            this.cmbSearch1.Items.Add("シナリオ名");
            this.cmbSearch1.Items.Add("作者名");
            this.cmbSearch1.Items.Add("解説");
            this.cmbSearch1.Items.Add("フォルダ");
            this.cmbSearch1.SelectedIndex = 0;

            this.cmbSearch2.Items.Add("シナリオ名");
            this.cmbSearch2.Items.Add("作者名");
            this.cmbSearch2.Items.Add("解説");
            this.cmbSearch2.Items.Add("フォルダ");
            this.cmbSearch2.SelectedIndex = 1;

            this.txtSearch1.Select();

            if (_pm.paramOption == false)
            {
                btnGetMate.Visible = false;
            }


            // ツールチップ
            this.components = new System.ComponentModel.Container();
            toolTip1 = new ToolTip(this.components);
            toolTip1.UseFading = true;
            toolTip1.InitialDelay = 100;
            toolTip1.ShowAlways = false;
            toolTip1.Active = true;

            toolTip1.SetToolTip(this.btnSearch, "シナリオキャッシュからシナリオを検索します。");
            toolTip1.SetToolTip(this.btnCache, "シナリオキャッシュを作成します。");
            toolTip1.SetToolTip(this.btnGetMate, "シナリオのキャストをＰＣとして取り込みます。");
            toolTip1.SetToolTip(this.btnEditor, "シナリオをcwxeditorかWirthBuilderで開きます。");
            toolTip1.SetToolTip(this.btnCard, "シナリオ内のCardを一覧表示します。");
            toolTip1.SetToolTip(this.btnRyuneChange, "リューンカードの画像をリューン互換のカードの画像と置き換えます。\r\n宿単位で効果があります。");
            toolTip1.SetToolTip(this.btnCSV, "現在表示中の内容をpm.csvに書き出します。");
            
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScenarioSearchDlg_Shown(object sender, EventArgs e)
        {
            if(File.Exists(FilePath.ApplicationFolder + "\\Summary.cache"))
            {
                // デシリアライズしてDataTable取り出し
                ReadCache();
                if (ScenarioCacheDt.Columns.Count != ColumnNames.Length)
                {
                    // ColumnCountが変化している。
                    MakeCache();
                }

            }else {
                // シナリオ読み込んでDataTableを作成後、シリアライズしてバイナリファイルに書き出し。
                MakeCache();
            }

            DispDt();
        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void btnCache_Click(object sender, EventArgs e)
        {
            MakeCache();
            DispDt();
        }

        public void ReadCache()
        { 
            //読み込んでデシリアライズする
            FileStream fs = new FileStream(FilePath.ApplicationFolder + "\\Summary.cache",
            FileMode.Open,
            FileAccess.Read);
            BinaryFormatter f = new BinaryFormatter();

            // ダイアログを出すほどの時間がかからない

            object obj = f.Deserialize(fs);
            DataSet ds = (DataSet)obj;
            ScenarioCacheDt = ds.Tables[0];

            f = null;
            fs.Dispose();

        }

        private void MakeCache()
        {   

            // 新規DT、Column設定
            PrepareNewDt();


            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "シナリオフォルダ構造を読み込み中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;
            
            //@IT
            DirectoryInfo di = new DirectoryInfo(FilePath.GetSystemScenarioFolder());
            FileInfo[] fis = di.GetFiles("*.wsm",SearchOption.AllDirectories);
            FileInfo[] fics = new FileInfo[] { };

            // CodeZine(http://www.codezine.jp/)
            bool canCountCAB = true;
            // DLLの存在を確認
            IntPtr pathPtr;

            if (false == ((PartyMaker)_owner).isIncludedCAB)
            {
                // CAB圧縮シナリオは検索対象に含めない
                canCountCAB = false;
            } 
            else if (PartyMaker.SearchPath(null, "cab32.dll", null, 0, null, out pathPtr) == 0)
            //else if(!File.Exists("cab32.dll"))
            {
                MessageDlg.ShowInfoDialog(
                    "CAB32.DLLが見つかりませんので、\r\n圧縮ファイルはカウントしません。");
                canCountCAB = false;
            }
            else
            {
                try
                {
                    // DLLのチェック
                    ushort ver = PartyMaker.CabGetVersion();
                }
                catch
                {
                    MessageDlg.ShowInfoDialog(
                        "CAB32.DLLがインストールされていませんので、\r\n圧縮ファイルはカウントしません。");
                    canCountCAB = false;
                }
            }

            progress.Value = 20;
            progress.Message = "CAB圧縮シナリオ配置を読み込み中です。";
            

            if (canCountCAB)
            {
                fics = di.GetFiles("*.cab", SearchOption.AllDirectories);
            }

            int countMax = fis.Length + fics.Length;

            if (countMax != 0)
            {
                // summary ファイルを読み込みながら
                // DataRow追加しながらデータ格納

                int count = 0;
                SimpleSummaryData summary = null;
                DataRow dr;

                progress.Message = "シナリオサマリを読み込み中です。";
                progress.Value = 30;

                foreach (FileInfo fi in fis)
                {
                    progress.Value = 30 + 60 * count / countMax;
                    
                    try
                    {
                        summary = new SimpleSummaryData();
                        summary.Read(fi.FullName);

                        dr = ScenarioCacheDt.NewRow();

                        dr[ColumnNames[0]] = summary.ScenarioName;
                        dr[ColumnNames[1]] = summary.Author;
                        dr[ColumnNames[2]] = Convert.ToInt32(summary.LevelLow);
                        dr[ColumnNames[3]] = Convert.ToInt32(summary.LevelHigh);
                        dr[ColumnNames[4]] = Convert.ToInt32(summary.Version);


                        dr[ColumnNames[6]] = summary.ScenarioExpression;
                        dr[ColumnNames[7]] = Path.GetDirectoryName(fi.FullName);
                        dr[ColumnNames[8]] = fi.LastWriteTime.ToString();

                        // NeedCouponCountとNeedCoupons.Countが違うデータがある。
                        for (int i = 0; i < summary.NeedCoupons.Count && i < 5; i++)
                        {
                            dr[ColumnNames[9 + i]] = summary.NeedCoupons[i];
                        }

                        ScenarioCacheDt.Rows.Add(dr);

                    }
                    catch (Exception)
                    {
                        // 隊長！ファイル読み込めません！！
                    }

                    count += 1;
                }

                string archiveFile;

                if (canCountCAB)
                {

                    DirectoryInfo dic = new DirectoryInfo(FilePath.GetSystemTempFolder());
                    FileInfo[] ficcs;
                    foreach (FileInfo fic in fics)
                    {

                        // 指定されたファイルがあるか調べる必要はない

                        progress.Value = 30 + 60 * count / countMax;


                        //動作中かチェック
                        if (PartyMaker.CabGetRunning())
                            PartyMaker.CWIErrorMessage(
                                "CAB32.DLLが現在動作中です。");

                        //展開できるかチェック
                        if (!PartyMaker.CabCheckArchive(fic.FullName, 0))
                        {
                            // 失敗したということはシナリオ書庫ではないのでcontinue

                            count += 1;
                            continue;

                        }


                        //ファイル名とフォルダ名を修正する
                        archiveFile = "\"" + fic.FullName + "\"";
                        //フォルダ名は必ず\で終わる必要がある
                        string extractTo = "\"" + FilePath.GetSystemTempFolder() + "\\\"";

                        StringBuilder output = new StringBuilder(65536);

                        FilePath.CreateFolder(FilePath.GetSystemTempFolder());

                        // 展開する
                        // オプション変更 v1.3.0(01) サイレントに変更
                        int ret = PartyMaker.Cab(IntPtr.Zero,
                            string.Format("-x -i {0} {1} *", archiveFile, extractTo),
                            output, (uint)output.Capacity);

                        //結果
                        if (ret != 0)
                        {
                            //    失敗したということはシナリオ書庫ではないのでcontinue
                            count += 1;
                            continue;
                        }
                        else
                        {
                            dic.Refresh();
                        }

                        ficcs = dic.GetFiles("*.wsm", SearchOption.AllDirectories);

                        foreach (FileInfo ficc in ficcs)
                        {
                            
                            try
                            {
                                summary = new SimpleSummaryData();
                                summary.Read(ficc.FullName);

                                dr = ScenarioCacheDt.NewRow();

                                dr[ColumnNames[0]] = summary.ScenarioName;
                                dr[ColumnNames[1]] = summary.Author;
                                dr[ColumnNames[2]] = Convert.ToInt32(summary.LevelLow);
                                dr[ColumnNames[3]] = Convert.ToInt32(summary.LevelHigh);
                                dr[ColumnNames[4]] = Convert.ToInt32(summary.Version);

                                dr[ColumnNames[6]] = summary.ScenarioExpression;
                                dr[ColumnNames[7]] = archiveFile + Path.GetDirectoryName(ficc.FullName.Replace(FilePath.GetSystemTempFolder(),""));
                                dr[ColumnNames[8]] = ficc.LastWriteTime.ToString();


                                // NeedCouponCountとNeedCoupons.Countが違うデータがある。
                                // v1.3.0(01)
                                for (int i = 0; i < summary.NeedCoupons.Count && i < 5; i++)
                                {
                                    dr[ColumnNames[9 + i]] = summary.NeedCoupons[i];
                                }

                                ScenarioCacheDt.Rows.Add(dr);

                            }
                            catch (Exception)
                            {
                                // 隊長！ファイル読み込めません！！
                            }

                            count += 1;
                        }
                    }

                }

                FilePath.DeleteFolder(FilePath.GetSystemTempFolder());

                progress.Value = 90;

                // DataTableをDataSetに詰めてシリアライズして保存
                progress.Message = "Summary.cacheファイルに保存します。";
                DataSet ds = new DataSet();
                ds.Tables.Add(ScenarioCacheDt);

                FileStream fs = new FileStream(
                FilePath.ApplicationFolder + "\\Summary.cache",
                FileMode.Create,
                FileAccess.Write);
                BinaryFormatter bf = new BinaryFormatter();
                //シリアル化して書き込む
                bf.Serialize(fs, ds);
                fs.Close();
                fs.Dispose();

                progress.Value = 100;

                // 進捗ダイアログ終了
                EndProgress();
            }
            else
            {
                MessageDlg.ShowInfoDialog(
                    "シナリオがありません。");
            }
        }

        private void PrepareNewDt()
        {
            ScenarioCacheDt = new DataTable();

            ScenarioCacheDt.CaseSensitive = true;

            //DataColumn 設定
            DataColumn dataCol;


            for (int i = 0; i < ColumnNames.Length; i++)
            {
                dataCol = new DataColumn();
                dataCol.ColumnName = ColumnNames[i];
                dataCol.Caption = Captions[i];
                if (i > 4 || i < 2)
                {
                    dataCol.DataType = typeof(string);
                    dataCol.DefaultValue = string.Empty;
                }
                else if (i == 3)
                {
                    dataCol.DataType = typeof(int);
                    dataCol.DefaultValue = 999;
                }
                else
                {
                    dataCol.DataType = typeof(int);
                    dataCol.DefaultValue = 0;
                }

                ScenarioCacheDt.Columns.Add(dataCol);
            }
          
        }

        private void DispDt()
        {
            DataTable dtDisp = ScenarioCacheDt.Copy();
            int dtCount = dtDisp.Rows.Count;

            // カレントパーティーがいる場合はプレイ可能かどうかを表示
            if (inn != null && inn.CurrentParty != null)
            {
                // プレイ可能かどうかを表示
                // 平均レベル算出
                double ave;
                uint sum = 0;
                List<string> coupons = new List<string>();
                foreach (CharacterData cd in inn.CurrentParty.Characters)
                {
                    // レベル
                    sum += cd.Level;
                    // クーポン名
                    foreach (CouponData coup in cd.Coupons)
                    {
                        coupons.Add(coup.Name);
                    }
                }
                ave = (double)sum / inn.CurrentParty.Characters.Count;

                lblAverage.Text = "平均レベル：" + ave.ToString("#.##");
                
                bool isFinish = false;
                bool isProgress = false;
                bool isCouponOK = true;
                int low;
                int high;
                foreach (DataRow dr in dtDisp.Rows)
                {

                    // シナリオは終了済みシナリオリストの中にあるか
                    isFinish = false;
                    foreach (string finish in inn.FinishScenarios)
                    {
                        if (finish.Equals(dr[ColumnNames[0]]))
                        {
                            isFinish = true;
                            break;
                        }
                    }

                    if (isFinish)
                    {
                        // 終了印あり
                        dr[ColumnNames[5]] = FINISH;
                        continue;
                    }

                    // シナリオは進行中のシナリオリストの中にあるか
                    isProgress = false;
                    foreach (string progress in inn.ProgressScenarios)
                    {
                        if (progress.Equals(dr[ColumnNames[0]]))
                        {
                            isProgress = true;
                            break;
                        }
                    }

                    if (isProgress)
                    {
                        // シナリオ進行中
                        dr[ColumnNames[5]] = PROGRESSION;
                        continue;
                    }

                    // 必要クーポンを満たしているか
                    int NeedCouponCount = 0;
                    for(int i = 1;i <= 5;i++)
                    {
                        if(dr["Coupon" + i].Equals(string.Empty)) 
                        {
                            break;
                        } else 
                        {
                            NeedCouponCount ++;
                        }
                    }

                    isCouponOK = true;
                    for (int i = 0; i < NeedCouponCount; i ++ )
                    {
                        if (!coupons.Contains(dr["Coupon" + (i+1)].ToString()))
                        {
                            // 含まれていなかったら
                            isCouponOK = false;
                            break;
                        }
                    }

                    if (!isCouponOK)
                    {
                        // 含まれていなかったら
                        dr[ColumnNames[5]] = LACK_OF_COUPONS;
                        continue;
                    }

                    // シナリオの推奨レベルは平均レベルのなかにあるか？
                    // 0～0のしなりおはスルー
                    if (dr[ColumnNames[2]].Equals(0) && dr[ColumnNames[3]].Equals(0))
                    {
                        // スルー
                        continue;
                    }
                    else
                    {
                        low = Int32.Parse(dr[ColumnNames[2]].ToString());
                        high = Int32.Parse(dr[ColumnNames[3]].ToString());
                        if (ave < low || high < ave)
                        {
                            // 推奨レベル範囲外
                            dr[ColumnNames[5]] = OUT_OF_RANGE;
                            continue;
                        }

                    }
                }

            }
            else
            {
                lblAverage.Text = "平均レベル：";
            }


            DataView dv = new DataView(dtDisp);


            //  検索１
            string search1Type = string.Empty;
            switch (cmbSearch1.SelectedIndex)
            {
                case 0:
                    search1Type = ColumnNames[0];
                    break;
                case 1:
                    search1Type = ColumnNames[1];
                    break;
                case 2:
                    search1Type = ColumnNames[6];
                    break;
                default:
                    search1Type = ColumnNames[7];
                    break;
            }

            string likeWord = txtSearch1.Text;
            if (likeWord.IndexOf("'") > -1)
            {
                MessageDlg.ShowInfoDialog("'抜きで検索してください。");
                this.txtSearch1.Focus();
                return;
            }

            string search1Filter = likeWord.Equals(string.Empty)
                ? string.Empty
                : search1Type + " LIKE '%" + likeWord + "%'";


            //  検索２
            string search2Type = string.Empty;
            switch (cmbSearch2.SelectedIndex)
            {
                case 0:
                    search2Type = ColumnNames[0];
                    break;
                case 1:
                    search2Type = ColumnNames[1];
                    break;
                case 2:
                    search2Type = ColumnNames[6];
                    break;
                default:
                    search2Type = ColumnNames[7];
                    break;
            }

            likeWord = txtSearch2.Text;
            if (likeWord.IndexOf("'") > -1)
            {
                MessageDlg.ShowInfoDialog("'抜きで検索してください。");
                this.cmbSearch2.Focus();
                return;
            }

            string search2Filter = likeWord.Equals(string.Empty)
                ? string.Empty
                : search2Type + " LIKE '%" + likeWord + "%'";

            // レベル
            string levelFilter = string.Empty;

            int levelLow = 0;
            int levelHigh = 0;
            bool setLow = false;
            bool setHigh = false;
            setLow = int.TryParse(txtLevelLow.Text, out levelLow);
            setHigh = int.TryParse(txtLevelHigh.Text, out levelHigh);

            if (setLow)
            {
                if (setHigh)
                {
                    // 両方入っている
                    if (levelLow > levelHigh)
                    {
                        // 表示するものがない
                        levelFilter = "1 = 2";

                    }
                    else
                    {

                        levelFilter =
                            "( Low >= " + levelLow + " AND Low <= " + levelHigh
                            + ") OR ( High >= " + levelLow + " AND High <= " + levelHigh
                            + ") OR ( Low < " + levelLow + " AND High > " + levelHigh + ")";
                    }
                }
                else
                {
                    //Lowのみ
                    levelFilter = "High >= " + levelLow;

                }
            }
            else
            {
                if (setHigh)
                {
                    //Highのみ
                    levelFilter = "Low <= " + levelHigh;
                }
                else
                {
                    //どちらもないのでそのまま
                }

            }

            dv.RowFilter = string.Empty;
            dv.RowFilter = StringConcateAnd(dv.RowFilter, search1Filter);
            dv.RowFilter = StringConcateAnd(dv.RowFilter, search2Filter);
            dv.RowFilter = StringConcateAnd(dv.RowFilter, levelFilter);

            int dvCount = dv.Count;

            dgv1.DataSource = dv;

            lblCount.Text = dvCount + "/" + dtCount;

            if (!widthFirst)
            {
                dgv1.Columns[ColumnNames[0]].Width = 140;
                dgv1.Columns[ColumnNames[1]].Width = 90;
                dgv1.Columns[ColumnNames[2]].Width = 40;
                dgv1.Columns[ColumnNames[3]].Width = 40;
                dgv1.Columns[ColumnNames[4]].Width = 40;
                dgv1.Columns[ColumnNames[5]].Width = 120;
                dgv1.Columns[ColumnNames[6]].Width = 400;
                dgv1.Columns[ColumnNames[7]].Width = 600;
                dgv1.Columns[ColumnNames[8]].Width = 120;
                widthFirst = true;
            }

            for (int i = 0; i < ScenarioCacheDt.Columns.Count;i++)
            {
                dgv1.Columns[i].HeaderText =
                    ScenarioCacheDt.Columns[i].Caption;
            }

            this.Activate();
        }

        private string StringConcateAnd(string baseStr,string addStr)
        {
            if (!baseStr.Equals(string.Empty))
            {
                if (!addStr.Equals(string.Empty))
                {
                    return baseStr + " AND " + addStr;
                }
                else
                {
                    return baseStr;
                }
            }
            else
            {
                return addStr;
            }
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScenarioSearchDlg_FormClosing(object sender, FormClosingEventArgs e)
        {

        }

        private void ScenarioSearchDlg_Resize(object sender, EventArgs e)
        {
            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {

                this.dgv1.Width = this.Width - 30;
                this.dgv1.Height = this.Height - 100;

                this.btnGetMate.Location =
                    new Point(this.ClientSize.Width - 5 * 5 - 50 * 5, this.ClientSize.Height - 28);

                this.btnEditor.Location =
                    new Point(this.ClientSize.Width - 5 * 4 - 50 * 4, this.ClientSize.Height - 28);

                this.btnCard.Location =
                    new Point(this.ClientSize.Width - 5 * 3 - 50 * 3, this.ClientSize.Height - 28);

                this.btnRyuneChange.Location =
                    new Point(this.ClientSize.Width - 5 * 2 - 50 * 2, this.ClientSize.Height - 28);

                this.btnCSV.Location = 
                    new Point(this.ClientSize.Width - 5 - 50, this.ClientSize.Height - 28);

                this.btnEnd.Location =
                    new Point((this.ClientSize.Width - this.btnEnd.Width) / 2, this.ClientSize.Height - 28);

                this.lblAverage.Location =
                    new Point(this.lblAverage.Location.X, this.ClientSize.Height - 28);

                this.lblCount.Location =
                    new Point(this.lblCount.Location.X, this.ClientSize.Height - 28);
            }
        }

        private DataSet PrepareNewDt2()
        {  
            int widIndex = 0;
            DataSet ds = new DataSet();
            DataTable dtCard = null;

            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            if (File.Exists(FilePath.ApplicationFolder
                + "\\Replace.cache") 
                && MessageDlg.ShowYesNoDialog("すでにReplace.cacheがあります。\r\nこれを使いますか？")
                == DialogResult.Yes)
            {

                //読み込んでデシリアライズする
                FileStream fs = new FileStream(FilePath.ApplicationFolder
                    + "\\Replace.cache",
                FileMode.Open,
                FileAccess.Read);
                BinaryFormatter f = new BinaryFormatter();

                // ダイアログを出すほどの時間がかからない

                object obj = f.Deserialize(fs);
                DataSet ds1 = (DataSet)obj;

                f = null;
                fs.Dispose();

                fs = new FileStream(FilePath.ApplicationFolder
                    + "\\Replace2.cache",
                FileMode.Open,
                FileAccess.Read);
                f = new BinaryFormatter();
                obj = f.Deserialize(fs);
                List<List<string>> lists = (List<List<string>>)obj;

                matchList = lists;

                progress.Value = 10;

                // 宿が違ってもcacheがつかえるようにチェック状態を変える
                dtCard = ds1.Tables["CardData"];

                string pattern = "*.wid";
                DirectoryInfo diYado = new DirectoryInfo(inn.TempFolder);
                FileInfo[] yadoInfos = diYado.GetFiles(pattern);

                // チェック状態初期化
                foreach (DataRow drr in dtCard.Rows)
                {
                    drr[2] = false;
                    drr[5] = "";
                }

                CompareWidCard cwci;
                foreach (FileInfo fii in yadoInfos)
                {
                    cwci = new CompareWidCard(fii.FullName);

                    foreach (DataRow drr in dtCard.Rows)
                    {
                        if (cwci.CardName.Equals(drr[0]) &&
                            cwci.Expression.Equals(drr[3]))
                        {
                            //宿のカード置場・荷物袋の状態によりチェックを付ける
                            drr[2] = true;
                            drr[5] = fii.Name;
                            break;
                        }
                    }
                }

                progress.Value = 100;

                // 進捗ダイアログ終了
                EndProgress();

                return ds1;

            }

            progress.Message = "データテーブルを用意しています。";
            dtCard = new DataTable("CardData");

            dtCard.CaseSensitive = true;

            //DataColumn 設定
            DataColumn dataCol;

            for (int i = 0; i < CardColumnNames.Length; i++)
            {
                dataCol = new DataColumn();
                if (i == 1)
                {
                    dataCol.DataType = typeof(int);
                    dataCol.DefaultValue = 0;
                }
                else if (i == 2)
                {
                    dataCol.DataType = typeof(bool);
                    dataCol.DefaultValue = false;
                }
                else
                {
                    dataCol.DataType = typeof(string);
                    dataCol.DefaultValue = string.Empty;
                }
                dataCol.ColumnName = CardColumnNames[i];
                dataCol.Caption = CardCaptions[i];

                dtCard.Columns.Add(dataCol);
            }

            ds.Tables.Add(dtCard);

            progress.Value = 5;
            progress.Message = "交易都市リューンなど\r\n対象のシナリオをチェックします。";

            bool isRyune = false;
            bool isKajiya = false;
            int ryuneNo = -1;
            int kajiyaNo = -1;
            for(int i = 0;i < ScenarioCacheDt.Rows.Count 
                && (!isRyune || !isKajiya);i++)
            {
                if (!isRyune 
                    && ScenarioCacheDt.Rows[i][0].Equals("交易都市リューン"))
                {
                    isRyune = true;
                    ryuneNo = i;
                }

                if (!isKajiya 
                    && ScenarioCacheDt.Rows[i][0].Equals("リューンの鍛冶屋"))
                {
                    isKajiya = true;
                    kajiyaNo = i;
                }
            }

            DataRow dr = null;
            if (isRyune && isKajiya)
            {

                progress.Value = 10;
                progress.Message =
                    "宿にリューンカードを持っているかチェックしています。";

                //２シナリオがある！
                string pattern = "*.wid";
                string ryuneFolder = ScenarioCacheDt.Rows[ryuneNo][7].ToString();
                DirectoryInfo di = new DirectoryInfo(ryuneFolder);
                DirectoryInfo diYado = new DirectoryInfo(inn.TempFolder);
                FileInfo[] yadoInfos = diYado.GetFiles(pattern);

                CompareWidCard cwc = null;
                CompareWidCard cwci = null;
                //リューンカード
                foreach (FileInfo fi in di.GetFiles(pattern))
                {
                    if (fi.Name.Substring(0, 5).StartsWith("Skill") ||
                       fi.Name.Substring(0, 5).StartsWith("Beast") ||
                       fi.Name.Substring(0, 4).StartsWith("Item"))
                    {

                        dr = dtCard.NewRow();
                        cwc = new CompareWidCard(fi.FullName);
                        dr[0] = cwc.CardName;
                        dr[1] = 0;
                        dr[3] = cwc.Expression;
                        dr[4] = fi.FullName;
                        foreach (FileInfo fii in yadoInfos)
                        {
                            cwci = new CompareWidCard(fii.FullName);
                            if (cwc.CardName == cwci.CardName &&
                                cwc.Expression == cwci.Expression)
                            {
                                dr[2] = true;
                                dr[5] = fii.Name;
                                break;
                            }
                        }

                        dtCard.Rows.Add(dr);
                    }
                }


                progress.Value = 15;
                progress.Message =
                    "宿に鉱石を持っているかチェックしています。";
                
                //リューンの鍛冶屋の鉱石
                di = new DirectoryInfo(ScenarioCacheDt.Rows[kajiyaNo][7].ToString());
                List<string> list = new List<string>();
                list.Add("Item98.wid");
                list.Add("Item99.wid");
                list.Add("Item100.wid");
                list.Add("Item101.wid");
                list.Add("Item102.wid");
                foreach (FileInfo fi in di.GetFiles(
                    "Item*.wid"))
                {
                    if (!list.Contains(fi.Name)) continue;

                    dr = dtCard.NewRow();

                    cwc = new CompareWidCard(fi.FullName);
                    dr[0] = cwc.CardName;
                    dr[1] = 0;
                    dr[3] = cwc.Expression;
                    dr[4] = fi.FullName;
                    foreach (FileInfo fii in yadoInfos)
                    {
                        cwci = new CompareWidCard(fii.FullName);
                        if (cwc.CardName == cwci.CardName &&
                            cwc.Expression == cwci.Expression)
                        {
                            dr[2] = true;
                            dr[5] = fii.Name;
                            break;
                        }
                    }

                    dtCard.Rows.Add(dr);
                }


                matchList = new List<List<string>>();

                foreach (DataRow drr in dtCard.Rows)
                {
                    matchList.Add(new List<string>());
                }

                progress.Value = 20;
                progress.Message =
                    "シナリオ内に同カード名・\r\n同解説のカードがあるかチェックします。";


                di = new DirectoryInfo(FilePath.GetSystemScenarioFolder());
                FileInfo[] fiSce = di.GetFiles(pattern,SearchOption.AllDirectories);


                progress.Value = 25;
                progress.Message =
                    "シナリオ内に同カード名・\r\n同解説のカードがあるかチェックします。\r\nかなり時間が掛かります。";

                int count = fiSce.Length;
                if (count > 0)
                {
                    int index = 0;
                    widIndex = 0;
                    foreach (FileInfo fisce in fiSce)
                    {
                        if (fisce.Name.Substring(0, 5).StartsWith("Skill") ||
                           fisce.Name.Substring(0, 5).StartsWith("Beast") ||
                           fisce.Name.Substring(0, 4).StartsWith("Item"))
                        {

                            try
                            {
                                cwci = new CompareWidCard(fisce.FullName);
                            }
                            catch(Exception)
                            {
                                widIndex++;
                                progress.Value = 25 + 65 * widIndex / count;
                                continue;
                            }

                            if (cwci.CardName == "" && cwci.Expression == "")
                            {
                                widIndex++;
                                progress.Value = 25 + 65 * widIndex / count;
                                continue;
                            }

                            index = 0;
                            foreach (DataRow drr in dtCard.Rows)
                            {
                                if (cwci.CardName.Equals(drr[0]) &&
                                    cwci.Expression.Equals(drr[3]))
                                {
                                    matchList[index].Add(fisce.FullName);
                                    int value = (int)drr[1];
                                    value++;
                                    drr[1] = value;
                                    break;
                                }

                                index++;
                            }
                        }

                        widIndex++;

                        progress.Value = 25 + 65 * widIndex / count;
                    }

                }

                progress.Value = 90;

                // DataTableをDataSetに詰めてシリアライズして保存
                progress.Message = "Replace.cacheファイルに保存します。";

                FileStream fs = new FileStream(
                FilePath.ApplicationFolder + "\\Replace.cache",
                FileMode.Create,
                FileAccess.Write);
                BinaryFormatter bf = new BinaryFormatter();
                //シリアル化して書き込む
                bf.Serialize(fs, ds);
                fs.Close();
                fs.Dispose();

                fs = new FileStream(
                FilePath.ApplicationFolder + "\\Replace2.cache",
                FileMode.Create,
                FileAccess.Write);
                bf = new BinaryFormatter();
                //シリアル化して書き込む
                bf.Serialize(fs, matchList);
                fs.Close();
                fs.Dispose();

                progress.Value = 100;

                // 進捗ダイアログ終了
                EndProgress();
            }
            else
            {
                MessageDlg.ShowInfoDialog("シナリオキャッシュの中に\r\n置換対象のシナリオがありません。");

                progress.Value = 100;

                // 進捗ダイアログ終了
                EndProgress();
            }

            return ds;

        }

        #endregion

        #region キーイベント

        private void ScenarioSearchDlg_KeyDown(object sender, KeyEventArgs e)
        {
            int dvCount = ((DataView)(dgv1.DataSource)).Count;
            if (e.KeyCode == Keys.PageUp)
            {
                if (0 < dgv1.FirstDisplayedScrollingRowIndex - 15)
                {
                    dgv1.FirstDisplayedScrollingRowIndex -= 15;
                }
                else
                {
                    dgv1.FirstDisplayedScrollingRowIndex = 0;
                }
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.PageDown)
            {
                if (dvCount - 1 > dgv1.FirstDisplayedScrollingRowIndex + 15)
                {
                    dgv1.FirstDisplayedScrollingRowIndex += 15;
                }
                else
                {
                    dgv1.FirstDisplayedScrollingRowIndex = dvCount - 1;
                }
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Home)
            {

                dgv1.FirstDisplayedScrollingRowIndex = 0;
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.End)
            {

                dgv1.FirstDisplayedScrollingRowIndex = dvCount - 1;
                e.Handled = true;
            }
        }

        private void txtLevel_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
            {
                e.Handled = true;
            }

        }

        #endregion
        
        #region クリック

        private void btnSearch_Click(object sender, EventArgs e)
        {
            DispDt();
        }

        private void btnCSV_Click(object sender, EventArgs e)
        {
            // テスト用（正確なデータとの比較用）に出力

            //DOBON.NETより

            //CSVで保存するDataTableはすでにある。

            //保存先のCSVファイルのパス
            string csvPath = FilePath.ApplicationFolder + "\\pm.csv";

            //CSVファイルに書き込むときに使うEncoding
            System.Text.Encoding enc =
                System.Text.Encoding.GetEncoding("Shift_JIS");

            //開く
            System.IO.StreamWriter sr =
                new System.IO.StreamWriter(csvPath, false, enc);

            int colCount = ScenarioCacheDt.Columns.Count;
            int lastColIndex = colCount - 1;

            //レコードを書き込む
            //foreach (DataRow row in ScenarioCacheDt.Rows)
            DataView dv = (DataView)dgv1.DataSource;
            DataTable dtDv = dv.ToTable();
            foreach (DataRow row in dtDv.Rows)
            {
                for (int i = 0; i < colCount; i++)
                {
                    //フィールドの取得
                    string field = row[i].ToString();
                    //"で囲む必要があるか調べる
                    if (field.IndexOf('"') > -1 ||
                        field.IndexOf(',') > -1 ||
                        field.IndexOf('\r') > -1 ||
                        field.IndexOf('\n') > -1 ||
                        field.StartsWith(" ") || field.StartsWith("\t") ||
                        field.EndsWith(" ") || field.EndsWith("\t"))
                    {
                        if (field.IndexOf('"') > -1)
                        {
                            //"を""とする
                            field = field.Replace("\"", "\"\"");
                        }
                        field = "\"" + field + "\"";
                    }
                    //フィールドを書き込む
                    sr.Write(field);
                    //カンマを書き込む
                    if (lastColIndex > i)
                    {
                        sr.Write(',');
                    }
                }
                //改行する
                sr.Write("\r\n");
            }

            //閉じる
            sr.Close();

            sr.Dispose();
        }

        private void dgv1_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex < 0 || e.RowIndex > dgv1.Rows.Count - 1)
            {
                return;
            }

            
            DataGridViewCellCollection cells = dgv1.Rows[e.RowIndex].Cells;
            if (ScenarioSelect)
            {
                string stat = cells[5].Value.ToString();

                // シナリオ
                if (FINISH.Equals(stat))
                {
                    MessageBox.Show("このシナリオは終了済です。");
                    return;
                }
                if (PROGRESSION.Equals(stat))
                {
                    MessageBox.Show("このシナリオは他のパーティーが進行中です。");
                    return;
                }
                if (LACK_OF_COUPONS.Equals(stat))
                {
                    MessageBox.Show("このシナリオはクーポン不足のためいきなり起動できません。");
                    return;
                }

                int version = (int)cells[4].Value;
                if (version != 4)
                {
                    // シナリオバージョンが４でない場合
                    MessageBox.Show("すみませんがこの機能は現在Version４限定となっております。");
                    return;
                }

                string scenaFolder = cells[7].Value.ToString();
                bool isCAB = scenaFolder.StartsWith("\"");
                if (isCAB)
                {
                    // CAB圧縮されている
                    MessageDlg.ShowInfoDialog("この機能は圧縮シナリオは対象外です。");
                    return;
                }

                if (!Directory.Exists(scenaFolder))
                {
                    // シナリオはなくなっていた！
                    MessageDlg.ShowInfoDialog(scenaFolder + " は移動しています。");
                    return;
                }

                this.SelectedFolder = scenaFolder;
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            else
            {
                // シナリオテキスト表示
                using (ScenarioTextDlg std
                    = new ScenarioTextDlg(this, cells[7].Value.ToString()))
                {
                    std.ShowDialog();
                }
            }
        }

        /// <summary>
        /// Editorボタン（シナリオをcwxeditorかWirthBuilderで開く）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCwxEditor_Click(object sender, EventArgs e)
        {
            try
            {
                if (dgv1.SelectedRows.Count > 0)
                {
                    string cwxeditor = FilePath.ApplicationFolder +
                            "\\cwxeditor.exe";
                    string wirthBuilder = FilePath.ApplicationFolder +
                            "\\WirthBuilder.exe";

                    string filename = dgv1.SelectedRows[0].Cells[7].Value.ToString();
                    bool isCab = false;

                    if (filename.StartsWith("\""))
                    {
                        // CABファイル
                        filename = filename.Split('\"')[1];
                        isCab = true;
                    }

                    string args = "";
                    if (!string.IsNullOrEmpty(filename) &&
                        System.IO.File.Exists(cwxeditor))
                    {
                        cwxeditor = "\"" + cwxeditor + "\"";
                        args = "\"" + filename + "\"";
                        System.Diagnostics.Process.Start(cwxeditor, args);
                    }
                    else if (!string.IsNullOrEmpty(filename) &&
                        System.IO.File.Exists(wirthBuilder))
                    {
                        if (isCab)
                        {
                            MessageDlg.ShowInfoDialog("WirthBuilderは\r\n"
                                + "Cab圧縮ファイルを開けません。");
                            return;
                        }
                        args = filename + "\\Summary.wsm";
                        System.Diagnostics.Process.Start(wirthBuilder,args);
                    }
                    else
                    {
                        MessageDlg.ShowInfoDialog("cwxeditorかWirthBuilderが\r\n"
                            + "エンジンと同じフォルダにありません。");
                    }
                }
            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("cwxeditorの呼出でエラー（シナリオ簡易検索）" 
                    + ex.Message);
            }
        }

        /// <summary>
        /// Mateボタン（Mate抽出）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGetMate_Click(object sender, EventArgs e)
        {
            if (dgv1.SelectedRows.Count > 0)
            {

                int oldCharCount = inn.Characters.Count;
                string scenaName = dgv1.SelectedRows[0].Cells[0].Value.ToString();
                inn.GetMates(dgv1.SelectedRows[0].Cells[7].Value.ToString());
                int newCharCount = inn.Characters.Count;

                if (newCharCount - oldCharCount > 0)
                {
                    // PartyMaker 表示反映
                    _pm.ModifyLastSomePC(newCharCount - oldCharCount);

                    MessageDlg.ShowInfoDialog(scenaName + "のMatePCを待機PCに追加しました。");
                }
                else
                {
                    MessageDlg.ShowInfoDialog(scenaName + "にはMatePCはいません。");
                }
            }
        }

        /// <summary>
        /// カードボタン（シナリオカード検索）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCard_Click(object sender, EventArgs e)
        {
            if (dgv1.SelectedRows.Count > 0)
            {
                using (TextSearchDlg tsd = new TextSearchDlg(this, inn,
                      dgv1.SelectedRows[0].Cells[7].Value.ToString()))
                {
                    tsd.ShowDialog();
                }
            }
        }

        /// <summary>
        /// 置換ボタン（リューンカードと同名同解説で画像が違うカードに置換）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRyuneChange_Click(object sender, EventArgs e)
        {
            try
            {
                //現在、最後のBitmap表示以降の表示処理や実際の置換が
                //うまくいかないためEnabled = False

                DataSet ds = PrepareNewDt2();
                using (TextSearchDlg tsd = new TextSearchDlg(this, inn, ds, matchList))
                {
                    tsd.ShowDialog();
                }
            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("カード置換の呼出でエラー\r\n（シナリオ簡易検索）"
                    + ex.Message);
            }
        }

        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {

            backgroundWorker1.CancelAsync();
            while (this.backgroundWorker1.IsBusy)
            {
                Thread.Sleep(1);
            }

            backgroundWorker1.RunWorkerAsync();
            
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();

            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion



  }
}