﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class ScenarioTextDlg : Form
    {
        #region メンバ

        private Form _owner;
        private string folderPath;

        private static volatile ProgressDlg progress;

        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ScenarioTextDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ScenarioTextDlg(Form owner,string folderPath)
        {
            InitializeComponent();

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            this.folderPath = folderPath;

            progress = new ProgressDlg(this);
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void ScenarioTextDlg_Load(object sender, EventArgs e)
        {
            DirectoryInfo di = null;
            FileInfo[] fis = null;
            TabPage tp = null;
            RichTextBox rich = null;
            if (folderPath.StartsWith("\""))
            {
                // CABファイル
            }
            else
            {
                // 普通のフォルダ
                di = new DirectoryInfo(folderPath);
                fis = di.GetFiles("*.txt", SearchOption.AllDirectories);
                foreach (FileInfo fi in fis)
                {
                    tp = new TabPage(fi.Name);
                    rich = new RichTextBox();
                    rich.Dock = DockStyle.Fill;
                    rich.ReadOnly = true;
                    rich.Font = new Font("ＭＳゴシック", 9.0f);
                    using (StreamReader sr = new StreamReader(
                            fi.FullName, Encoding.GetEncoding("Shift_JIS")))
                    {
                        rich.Text = sr.ReadToEnd();
                    }
                    tp.Controls.Add(rich);
                    tabText.TabPages.Add(tp);

                }
            }
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScenarioTextDlg_Shown(object sender, EventArgs e)
        {
            if (folderPath.StartsWith("\""))
            {
                // CABファイル
                // 現在対象外
                MessageDlg.ShowInfoDialog("CAB圧縮ファイルは対象外です。");
            }
        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }


        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScenarioTextDlg_FormClosing(object sender, FormClosingEventArgs e)
        {

        }

        #endregion

        #region キーイベント

        private void ScenarioTextDlg_KeyDown(object sender, KeyEventArgs e)
        {

        }

        #endregion
        
        #region クリック



        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {

            backgroundWorker1.CancelAsync();
            while (this.backgroundWorker1.IsBusy)
            {
                Thread.Sleep(1);
            }

            backgroundWorker1.RunWorkerAsync();
            
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();

            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion

  }
}