﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class SimpleHandWindow : Form
    {
        #region メンバ

        /// <summary>
        /// 宿
        /// </summary>
        private InnData inn = null;

        private static volatile ProgressDlg progress;
        private int maxCount = 99999;
        private int countNow = 0;

        private DataTable dtCard1 = null;
        private DataTable dtCard2 = null;

        private Form _owner;

        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public SimpleHandWindow()
        {
            // 通常使用しない
            InitializeComponent();

            this.MouseWheel += new MouseEventHandler(Inn_MouseWheel);

            progress = new ProgressDlg(this);

        }

        /// <summary>
        /// コンストラクタ
        /// (シンプルハンドウインドウ)
        /// </summary>
        public SimpleHandWindow(Form owner
            , InnData inn)
        {
            InitializeComponent();

            this.MouseWheel += new MouseEventHandler(Inn_MouseWheel);

            progress = new ProgressDlg(this);

            this.inn = inn;

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void SimpleHandWindow_Load(object sender, EventArgs e)
        {

            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "初期設定中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            // いつinn.WidCardsが呼ばれてもいいようにここでフラグを落とす。
            inn.ResetWidCard();

            // ソート種類.アイテム_スキル_召喚獣
            cmbOrder.SelectedIndex = 0;

            // 初期設定（グリッド）
            this.dgvCard1.RowHeadersVisible = false;
            this.dgvCard1.ColumnHeadersVisible = true;
            this.dgvCard2.RowHeadersVisible = false;
            this.dgvCard2.ColumnHeadersVisible = true;

            dtCard1 = new DataTable();
            dtCard2 = new DataTable();
            DataColumn dataCol;

            string[] colHeaderNames
            = 
                    {
                        "カード名",
                        "種別"
                    };

            dataCol = new DataColumn();
            dataCol.ColumnName = colHeaderNames[0];

            dataCol.DataType = typeof(string);
            dtCard1.Columns.Add(dataCol);


            dataCol = new DataColumn();
            dataCol.ColumnName = colHeaderNames[1];
            dataCol.DataType = typeof(string);
            dtCard1.Columns.Add(dataCol);


            dataCol = new DataColumn();
            dataCol.ColumnName = colHeaderNames[0];

            dataCol.DataType = typeof(string);
            dtCard2.Columns.Add(dataCol);


            dataCol = new DataColumn();
            dataCol.ColumnName = colHeaderNames[1];
            dataCol.DataType = typeof(string);
            dtCard2.Columns.Add(dataCol);

            // コンボ
            this.cmbSend.Items.Clear();
            if (inn.CurrentParty != null)
            {
                maxCount = inn.BankBoxCards.Count + inn.CurrentParty.BagCards.Count;

                this.dgvCard2.Enabled = true;
                this.btn2to1.Enabled = true;
                this.btnPartyUp.Enabled = true;
                this.btnPartyDown.Enabled = true;
                this.btnPartyToFirst.Enabled = true;
                this.btnPartyToEnd.Enabled = true;
                this.btnPartySort.Enabled = true;

                this.cmbSend.Items.Add("荷物袋");
                for (int i = 0; i < inn.CurrentParty.Characters.Count; i++)
                {
                    // キャラクタ名
                    this.cmbSend.Items.Add(inn.CurrentParty.Characters[i].CardName);
                }
                this.cmbSend.Enabled = true;
                // この段階で荷物袋カードを読み込み
                this.cmbSend.SelectedIndex = 0;
            }
            else
            {
                maxCount = inn.BankBoxCards.Count;

                this.dgvCard2.Enabled = false;
                this.cmbSend.Enabled = false;
                this.btn2to1.Enabled = false;
                this.btnPartyUp.Enabled = false;
                this.btnPartyDown.Enabled = false;
                this.btnPartyToFirst.Enabled = false;
                this.btnPartyToEnd.Enabled = false;
                this.btnPartySort.Enabled = false;
                this.cmbSend.SelectedIndex = -1;
            }

            //パーティーグリッド
            this.dgvCard2.DataSource = dtCard2;
            this.dgvCard2.Visible = true;

            dgvCard2.Columns[colHeaderNames[0]].Width = 170;
            dgvCard2.Columns[colHeaderNames[1]].Width = 70;
            dgvCard2.Columns[colHeaderNames[0]].SortMode = DataGridViewColumnSortMode.NotSortable; 
            dgvCard2.Columns[colHeaderNames[1]].SortMode = DataGridViewColumnSortMode.NotSortable;

            // カード置場
            SetYadoCard();

            // カード置場グリッド
            this.dgvCard1.DataSource = dtCard1;
            this.dgvCard1.Visible = true;

            dgvCard1.Columns[colHeaderNames[0]].Width = 170;
            dgvCard1.Columns[colHeaderNames[1]].Width = 70;
            dgvCard1.Columns[colHeaderNames[0]].SortMode = DataGridViewColumnSortMode.NotSortable;
            dgvCard1.Columns[colHeaderNames[1]].SortMode = DataGridViewColumnSortMode.NotSortable; 

            SetYadoCount();
            SetPartyCount();

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SimpleHandWindow_Shown(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SimpleHandWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

        #endregion

        #region ボタンクリックイベント

        /// <summary>
        /// ＯＫボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        /// <summary>
        /// 移動1→2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn1to2_Click(object sender, EventArgs e)
        {
            if (dgvCard1.SelectedRows.Count > 0)
            {
                int oldIndex = 0;
                DataRow dr;
                if (this.cmbSend.SelectedIndex == 0)
                {
                    // 対象は荷物袋（荷物袋は無条件移動）

                    // カード置場カードを荷物袋へ
                    oldIndex = dgvCard1.SelectedRows[0].Index;
                    inn.SwapPartyBagCardAndCard(-1, oldIndex);

                    // 表示反映
                    //  1から消去
                    string name = dgvCard1.SelectedRows[0].Cells[0].Value.ToString();
                    string kind = dgvCard1.SelectedRows[0].Cells[1].Value.ToString();

                    dtCard1.Rows.RemoveAt(oldIndex);

                    //  2に追加
                    dr = dtCard2.NewRow();
                    dr[0] = name;
                    dr[1] = kind;
                    dtCard2.Rows.Add(dr);

                }
                else
                {
                    // カード置場カードをキャラクタへ
                    oldIndex = dgvCard1.SelectedRows[0].Index;

                    int charIndex = this.cmbSend.SelectedIndex - 1;
                    CharacterData cd = inn.CurrentParty.Characters[charIndex];

                    手札種類 tefuda = 手札種類.所持スキル;

                    if (dgvCard1.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.SKILL))
                    {
                        tefuda = 手札種類.所持スキル;
                    }
                    else if (dgvCard1.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.ITEM))
                    {
                        tefuda = 手札種類.所持アイテム;
                    }
                    else if (dgvCard1.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.BEAST))
                    {
                        tefuda = 手札種類.所有召喚獣;
                    }
                    else
                    {
                        PartyMaker.CWIErrorMessage("該当するカード種別はありません。(SimpleHandWindow)");
                    }

                    // カード置場カードをキャラクタに移動
                    if (!(inn.SwapCharacterCardAndCard(charIndex, tefuda, -1, oldIndex)))
                    {
                        MessageDlg.ShowInfoDialog("最大枚数を超えて所持できません。");
                        return;
                    }

                    // 表示反映
                    //  1から消去
                    string name = dgvCard1.SelectedRows[0].Cells[0].Value.ToString();
                    string kind = dgvCard1.SelectedRows[0].Cells[1].Value.ToString();

                    dtCard1.Rows.RemoveAt(oldIndex);

                    //  2に表示反映
                    SetCharaCard();
                }


                SetYadoCount();
                SetPartyCount();
            }
        }

        /// <summary>
        /// 移動2→1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn2to1_Click(object sender, EventArgs e)
        {
            int oldIndex = 0;
            DataRow dr;
            if (dgvCard2.SelectedRows.Count > 0)
            {
                // 2→1の移動は無条件
                if (this.cmbSend.SelectedIndex == 0)
                {
                    // 荷物袋

                    // 荷物袋カードをカード置場へ
                    oldIndex = dgvCard2.SelectedRows[0].Index;
                    inn.SwapPartyBagCardAndCard(oldIndex, -1);

                    // 表示反映
                    //  2から消去
                    string name = dgvCard2.SelectedRows[0].Cells[0].Value.ToString();
                    string kind = dgvCard2.SelectedRows[0].Cells[1].Value.ToString();

                    dtCard2.Rows.RemoveAt(oldIndex);

                    //  1に追加
                    dr = dtCard1.NewRow();
                    dr[0] = name;
                    dr[1] = kind;
                    dtCard1.Rows.Add(dr);

                }
                else
                {
                    // キャラクタ
                    int charIndex = this.cmbSend.SelectedIndex - 1;
                    CharacterData cd = inn.CurrentParty.Characters[charIndex];

                    int firstIndex = -1;
                    int lastIndex = -1;

                    手札種類 tefuda = 手札種類.所持スキル;

                    oldIndex = dgvCard2.SelectedRows[0].Index;
                    if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.SKILL))
                    {
                        firstIndex = 0;
                        lastIndex = cd.Skills.Count - 1;
                        tefuda = 手札種類.所持スキル;
                    }
                    else if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.ITEM))
                    {
                        firstIndex = cd.Skills.Count;
                        lastIndex = cd.Skills.Count + cd.Items.Count - 1;
                        tefuda = 手札種類.所持アイテム;
                    }
                    else if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.BEAST))
                    {
                        firstIndex = cd.Skills.Count + cd.Items.Count;
                        lastIndex = cd.Skills.Count + cd.Items.Count + cd.Beasts.Count - 1;
                        tefuda = 手札種類.所有召喚獣;
                    }
                    else
                    {
                        PartyMaker.CWIErrorMessage("該当するカード種別はありません。(SimpleHandWindow)");
                    }

                    if (oldIndex < firstIndex || oldIndex > lastIndex)
                    {
                        PartyMaker.CWIErrorMessage("範囲がおかしいです。(SimpleHandWindow)");
                    }

                    // キャラクタカードをカード置場へ移動
                    inn.SwapCharacterCardAndCard(charIndex, tefuda, oldIndex - firstIndex, -1);

                    // 表示反映
                    //  2から消去
                    string name = dgvCard2.SelectedRows[0].Cells[0].Value.ToString();
                    string kind = dgvCard2.SelectedRows[0].Cells[1].Value.ToString();

                    dtCard2.Rows.RemoveAt(oldIndex);

                    //  1に追加
                    dr = dtCard1.NewRow();
                    dr[0] = name;
                    dr[1] = kind;
                    dtCard1.Rows.Add(dr);

                }


                SetYadoCount();
                SetPartyCount();
            }
        }

        /// <summary>
        /// 移動2→1 全部
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllToYado_Click(object sender, EventArgs e)
        {

            if (this.cmbSend.SelectedIndex == 0)
            {
                // 荷物袋

                // 全荷物袋カードを置場カードへ
                inn.MoveAllCardFromPartyBag();

            }
            else
            {
                // キャラクタ
                int charIndex = this.cmbSend.SelectedIndex - 1;
                inn.MoveAllCharacterCardToYado(charIndex);
            }

            // 表示反映
            dtCard2.Rows.Clear();
            SetYadoCard();

            SetYadoCount();
            SetPartyCount();
        }

        private void btnYadoUp_Click(object sender, EventArgs e)
        {
            if (dgvCard1.SelectedRows.Count > 0)
            {
                // 対象はカード置場カード自身
                // カードスワップ
                int oldIndex = dgvCard1.SelectedRows[0].Index;
                int newIndex = oldIndex - 1;
                if (newIndex < 0) return;

                // カード置場カード内移動
                inn.SwapCard(oldIndex, newIndex);

                // 表示反映
                string nameOld = dgvCard1.Rows[oldIndex].Cells[0].Value.ToString();
                string kindOld = dgvCard1.Rows[oldIndex].Cells[1].Value.ToString();
                dgvCard1.Rows[oldIndex].Cells[0].Value = dgvCard1.Rows[newIndex].Cells[0].Value;
                dgvCard1.Rows[oldIndex].Cells[1].Value = dgvCard1.Rows[newIndex].Cells[1].Value;
                dgvCard1.Rows[newIndex].Cells[0].Value = nameOld;
                dgvCard1.Rows[newIndex].Cells[1].Value = kindOld;

                //newIndexの行を選択する
                dgvCard1.Rows[newIndex].Selected = true;
                if (dgvCard1.FirstDisplayedScrollingRowIndex > 0)
                    dgvCard1.FirstDisplayedScrollingRowIndex --;
            }
        }

        private void btnYadoDown_Click(object sender, EventArgs e)
        {
            if (dgvCard1.SelectedRows.Count > 0)
            {
                // 対象はカード置場カード自身
                // カードスワップ
                int oldIndex = dgvCard1.SelectedRows[0].Index;
                int newIndex = oldIndex + 1;
                if (newIndex >= inn.BankBoxCards.Count) return;

                // カード置場カード内移動
                inn.SwapCard(oldIndex, newIndex);

                // 表示反映
                string nameOld = dgvCard1.Rows[oldIndex].Cells[0].Value.ToString();
                string kindOld = dgvCard1.Rows[oldIndex].Cells[1].Value.ToString();
                dgvCard1.Rows[oldIndex].Cells[0].Value = dgvCard1.Rows[newIndex].Cells[0].Value;
                dgvCard1.Rows[oldIndex].Cells[1].Value = dgvCard1.Rows[newIndex].Cells[1].Value;
                dgvCard1.Rows[newIndex].Cells[0].Value = nameOld;
                dgvCard1.Rows[newIndex].Cells[1].Value = kindOld;

                //newIndexの行を選択する
                dgvCard1.Rows[newIndex].Selected = true;

                if (dgvCard1.FirstDisplayedScrollingRowIndex < dtCard1.Rows.Count - 1)
                    dgvCard1.FirstDisplayedScrollingRowIndex ++;
            }
        }

        private void btnYadoToFirst_Click(object sender, EventArgs e)
        {
            DataRow dr;
            if (dgvCard1.SelectedRows.Count > 0)
            {
                // 対象はカード置場先頭
                int oldIndex = dgvCard1.SelectedRows[0].Index;
                int newIndex = 0;
                if (oldIndex <= newIndex) return;

                inn.MoveCard(oldIndex, newIndex);

                // 表示反映
                string nameOld = dgvCard1.Rows[oldIndex].Cells[0].Value.ToString();
                string kindOld = dgvCard1.Rows[oldIndex].Cells[1].Value.ToString();

                dtCard1.Rows.RemoveAt(oldIndex);

                //  1に追加
                dr = dtCard1.NewRow();
                dr[0] = nameOld;
                dr[1] = kindOld;
                dtCard1.Rows.InsertAt(dr, newIndex);

                //newIndexの行を選択する
                dgvCard1.Rows[newIndex].Selected = true;
                dgvCard1.FirstDisplayedScrollingRowIndex = newIndex;
            }

        }

        private void btnYadoToEnd_Click(object sender, EventArgs e)
        {
            DataRow dr;
            if (dgvCard1.SelectedRows.Count > 0)
            {
                // 対象はカード置場終端
                int oldIndex = dgvCard1.SelectedRows[0].Index;
                int newIndex = dtCard1.Rows.Count - 1;
                if (oldIndex >= newIndex) return;

                inn.MoveCard(oldIndex, newIndex + 1);

                // 表示反映
                string nameOld = dgvCard1.Rows[oldIndex].Cells[0].Value.ToString();
                string kindOld = dgvCard1.Rows[oldIndex].Cells[1].Value.ToString();

                dtCard1.Rows.RemoveAt(oldIndex);

                //  1に追加
                dr = dtCard1.NewRow();
                dr[0] = nameOld;
                dr[1] = kindOld;
                dtCard1.Rows.Add(dr);

                //newIndexの行を選択する
                dgvCard1.Rows[newIndex].Selected = true;
                dgvCard1.FirstDisplayedScrollingRowIndex = newIndex;
            }

        }

        private void btnPartyUp_Click(object sender, EventArgs e)
        {
            if (dgvCard2.SelectedRows.Count > 0)
            {
                if (this.cmbSend.SelectedIndex == 0)
                {
                    // 荷物袋

                    // 対象は荷物袋カード自身
                    // カードスワップ
                    int oldIndex = dgvCard2.SelectedRows[0].Index;
                    int newIndex = oldIndex - 1;
                    if (newIndex < 0) return;

                    // 荷物袋カード内移動
                    inn.SwapCard(oldIndex, newIndex);

                    // 表示反映
                    string nameOld = dgvCard2.Rows[oldIndex].Cells[0].Value.ToString();
                    string kindOld = dgvCard2.Rows[oldIndex].Cells[1].Value.ToString();
                    dgvCard2.Rows[oldIndex].Cells[0].Value = dgvCard2.Rows[newIndex].Cells[0].Value;
                    dgvCard2.Rows[oldIndex].Cells[1].Value = dgvCard2.Rows[newIndex].Cells[1].Value;
                    dgvCard2.Rows[newIndex].Cells[0].Value = nameOld;
                    dgvCard2.Rows[newIndex].Cells[1].Value = kindOld;

                    //newIndexの行を選択する
                    dgvCard2.Rows[newIndex].Selected = true;

                    if (dgvCard2.FirstDisplayedScrollingRowIndex > 0)
                    {
                        dgvCard2.FirstDisplayedScrollingRowIndex--;
                    }

                }
                else
                {
                    // キャラクタ
                    int charIndex = this.cmbSend.SelectedIndex - 1;
                    CharacterData cd = inn.CurrentParty.Characters[charIndex];

                    int firstIndex = -1;
                    int lastIndex = -1;

                    手札種類 tefuda = 手札種類.所持スキル;

                    int oldIndex = dgvCard2.SelectedRows[0].Index;
                    if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.SKILL))
                    {
                        firstIndex = 0;
                        lastIndex = cd.Skills.Count - 1;
                        tefuda = 手札種類.所持スキル;
                    } 
                    else if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.ITEM))
                    {
                        firstIndex = cd.Skills.Count;
                        lastIndex = cd.Skills.Count + cd.Items.Count - 1;
                        tefuda = 手札種類.所持アイテム;
                    }
                    else if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.BEAST))
                    {
                        firstIndex = cd.Skills.Count + cd.Items.Count;
                        lastIndex = cd.Skills.Count + cd.Items.Count + cd.Beasts.Count - 1;
                        tefuda = 手札種類.所有召喚獣;
                    }
                    else
                    {
                        PartyMaker.CWIErrorMessage("該当するカード種別はありません。(SimpleHandWindow)");
                    }
                    
                    if (oldIndex < firstIndex || oldIndex > lastIndex)
                    {
                        PartyMaker.CWIErrorMessage("範囲がおかしいです。(SimpleHandWindow)");
                    }
                    int newIndex = oldIndex - 1;
                    if (newIndex < firstIndex) return;

                    // カードスワップ
                    inn.SwapCharacterCard(charIndex
                        , charIndex, tefuda, oldIndex - firstIndex, newIndex - firstIndex);

                    // 表示反映
                    SetCharaCard();

                    //newIndexの行を選択する
                    dgvCard2.Rows[newIndex].Selected = true;
                    
                    if (dgvCard2.FirstDisplayedScrollingRowIndex > 0)
                        dgvCard2.FirstDisplayedScrollingRowIndex--;
                }
            }
        }

        private void btnPartyDown_Click(object sender, EventArgs e)
        {
            if (dgvCard2.SelectedRows.Count > 0)
            {
                if (this.cmbSend.SelectedIndex == 0)
                {
                    // 荷物袋

                    // 対象は荷物袋カード自身
                    // カードスワップ
                    int oldIndex = dgvCard2.SelectedRows[0].Index;
                    int newIndex = oldIndex + 1;
                    if (newIndex >= inn.CurrentParty.BagCards.Count) return;

                    // 荷物袋カード内移動
                    inn.SwapPartyBagCard(oldIndex, newIndex);

                    // 表示反映
                    string nameOld = dgvCard2.Rows[oldIndex].Cells[0].Value.ToString();
                    string kindOld = dgvCard2.Rows[oldIndex].Cells[1].Value.ToString();
                    dgvCard2.Rows[oldIndex].Cells[0].Value = dgvCard2.Rows[newIndex].Cells[0].Value;
                    dgvCard2.Rows[oldIndex].Cells[1].Value = dgvCard2.Rows[newIndex].Cells[1].Value;
                    dgvCard2.Rows[newIndex].Cells[0].Value = nameOld;
                    dgvCard2.Rows[newIndex].Cells[1].Value = kindOld;

                    //newIndexの行を選択する
                    dgvCard2.Rows[newIndex].Selected = true;
                    if (dgvCard2.FirstDisplayedScrollingRowIndex < dtCard2.Rows.Count - 1)
                        dgvCard2.FirstDisplayedScrollingRowIndex++;

                }
                else
                {
                    // キャラクタ
                    int charIndex = this.cmbSend.SelectedIndex - 1;
                    CharacterData cd = inn.CurrentParty.Characters[charIndex];

                    int firstIndex = -1;
                    int lastIndex = -1;

                    手札種類 tefuda = 手札種類.所持スキル;

                    int oldIndex = dgvCard2.SelectedRows[0].Index;
                    if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.SKILL))
                    {
                        firstIndex = 0;
                        lastIndex = cd.Skills.Count - 1;
                        tefuda = 手札種類.所持スキル;
                    }
                    else if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.ITEM))
                    {
                        firstIndex = cd.Skills.Count;
                        lastIndex = cd.Skills.Count + cd.Items.Count - 1;
                        tefuda = 手札種類.所持アイテム;
                    }
                    else if (dgvCard2.Rows[oldIndex].Cells[1].Value.ToString().Equals(CWData.BEAST))
                    {
                        firstIndex = cd.Skills.Count + cd.Items.Count;
                        lastIndex = cd.Skills.Count + cd.Items.Count + cd.Beasts.Count - 1;
                        tefuda = 手札種類.所有召喚獣;
                    }
                    else
                    {
                        PartyMaker.CWIErrorMessage("該当するカード種別はありません。(SimpleHandWindow)");
                    }

                    if (oldIndex < firstIndex || oldIndex > lastIndex)
                    {
                        PartyMaker.CWIErrorMessage("範囲がおかしいです。(SimpleHandWindow)");
                    }
                    int newIndex = oldIndex + 1;
                    if (newIndex > lastIndex) return;

                    // カードスワップ
                    inn.SwapCharacterCard(charIndex
                        , charIndex, tefuda, oldIndex - firstIndex, newIndex - firstIndex);

                    // 表示反映
                    SetCharaCard();

                    //newIndexの行を選択する
                    dgvCard2.Rows[newIndex].Selected = true;

                    if (dgvCard2.FirstDisplayedScrollingRowIndex < dtCard2.Rows.Count - 1)
                        dgvCard2.FirstDisplayedScrollingRowIndex++;

                }
            }
        }

        private void btnPartyToFirst_Click(object sender, EventArgs e)
        {
            DataRow dr;
            if (this.cmbSend.SelectedIndex == 0)
            {
                // 荷物袋
                if (dgvCard2.SelectedRows.Count > 0)
                {
                    // 対象は荷物袋先頭
                    int oldIndex = dgvCard2.SelectedRows[0].Index;
                    int newIndex = 0;
                    if (oldIndex <= newIndex) return;

                    inn.MovePartyBagCard(oldIndex, newIndex);

                    // 表示反映
                    string nameOld = dgvCard2.Rows[oldIndex].Cells[0].Value.ToString();
                    string kindOld = dgvCard2.Rows[oldIndex].Cells[1].Value.ToString();

                    dtCard2.Rows.RemoveAt(oldIndex);

                    //  1に追加
                    dr = dtCard2.NewRow();
                    dr[0] = nameOld;
                    dr[1] = kindOld;
                    dtCard2.Rows.InsertAt(dr, newIndex);

                    //newIndexの行を選択する
                    dgvCard2.Rows[newIndex].Selected = true;
                    dgvCard2.FirstDisplayedScrollingRowIndex = newIndex;

                }
            }
        }

        private void btnPartyToEnd_Click(object sender, EventArgs e)
        {
            DataRow dr;
            if (this.cmbSend.SelectedIndex == 0)
            {
                // 荷物袋
                if (dgvCard2.SelectedRows.Count > 0)
                {
                    // 対象は荷物袋終端
                    int oldIndex = dgvCard2.SelectedRows[0].Index;
                    int newIndex = dtCard2.Rows.Count - 1;
                    if (oldIndex >= newIndex) return;

                    inn.MovePartyBagCard(oldIndex, newIndex + 1);

                    // 表示反映
                    string nameOld = dgvCard2.Rows[oldIndex].Cells[0].Value.ToString();
                    string kindOld = dgvCard2.Rows[oldIndex].Cells[1].Value.ToString();

                    dtCard2.Rows.RemoveAt(oldIndex);

                    //  1に追加
                    dr = dtCard2.NewRow();
                    dr[0] = nameOld;
                    dr[1] = kindOld;
                    dtCard2.Rows.Add(dr);

                    //newIndexの行を選択する
                    dgvCard2.Rows[newIndex].Selected = true;
                    dgvCard2.FirstDisplayedScrollingRowIndex = newIndex;

                }
            }
        }

        /// <summary>
        /// ソート
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnYadoSort_Click(object sender, EventArgs e)
        {

            try
            {
                if (cmbOrder.SelectedIndex < 0 && cmbOrder.SelectedIndex > 5)
                {
                    // ありえない
                    MessageDlg.ShowInfoDialog("ソート種類の選択に誤りがあります。");
                    return;
                }
                //else
                //{
                //    if (DialogResult.Yes != MessageDlg.ShowYesNoDialog(
                //        "ソートは時間がかかる場合があります。\r\n 実行してよろしいですか？"
                //        ))
                //    {
                //        return;
                //    }
                //}

                int count;

                count = inn.BankBoxCards.Count;
                if (count > 0)
                {
                    // カード置場

                    inn.SortCards(ソート対象.カード置場, (ソート種類)(cmbOrder.SelectedIndex));

                    GC.Collect(2);

                    // 表示反映
                    SetYadoCard();
                }

            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("システムエラー（ソート）\r\n" + ex.ToString());
            }
        }

        /// <summary>
        /// ソート
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartySort_Click(object sender, EventArgs e)
        {
            try
            {
                if (cmbOrder.SelectedIndex < 0 && cmbOrder.SelectedIndex > 5)
                {
                    // ありえない
                    MessageDlg.ShowInfoDialog("ソート種類の選択に誤りがあります。");
                    return;
                }
                //else
                //{
                //    if (DialogResult.Yes != MessageDlg.ShowYesNoDialog(
                //        "ソートは時間がかかる場合があります。\r\n 実行してよろしいですか？"
                //        ))
                //    {
                //        return;
                //    }
                //}

                int count;

                if (inn.CurrentParty != null)
                {
                    // 荷物袋
                    count = inn.CurrentParty.BagCards.Count;
                    if (count > 0)
                    {
                        inn.SortCards(ソート対象.荷物袋, (ソート種類)(cmbOrder.SelectedIndex));

                        // 個数が同じなのでクリアせずにSetHcする。
                        GC.Collect(2);

                        // 表示反映
                        SetPartyCard();
                    }
                }


            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("システムエラー（ソート）\r\n" + ex.ToString());
            }
        }

        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {
            backgroundWorker1.RunWorkerAsync();
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();
            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion
        
        #region その他

        /// <summary>
        /// マウスホイールの移動量を取得
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Inn_MouseWheel(object sender, MouseEventArgs e)
        {
            ///ScrollPnl(mouseDownPanel, -e.Delta);
        }

        private void dgvCard1_SelectionChanged(object sender, EventArgs e)
        {

        }

        private void dgvCard2_SelectionChanged(object sender, EventArgs e)
        {

        }

        private void SetYadoCard()
        {
            bool canProgress = (progress != null && progress.LoadOn && progress.IsHandleCreated);
           
            dtCard1.Rows.Clear();
            DataRow dr;

            if (canProgress)
            {
                progress.Message = "カード置場のカードを読み込み中です。";
            }

            // カード置場
            foreach (RenamedCardData data in inn.BankBoxCards)
            {
                IndexCardData icd = inn.FindIndexedCardData(data);

                dr = dtCard1.NewRow();

                dr[0] = icd.CardData.Name;

                dr[1] = Util3.KindToText(icd.CardData.CardKind);

                dtCard1.Rows.Add(dr);

                if (canProgress)
                {
                    countNow++;
                    progress.Value = 100 * countNow / maxCount;
                }
            }
        }

        private void SetYadoCount()
        {
            this.lblYadoCount.Text = inn.BankBoxCards.Count.ToString();
        }

        private void SetPartyCard()
        {
            bool canProgress = (progress != null && progress.LoadOn && progress.IsHandleCreated);

            dtCard2.Rows.Clear();
            DataRow dr;

            if (canProgress)
            {
                progress.Message = "荷物袋のカードを読み込み中です。";
            }

            // 荷物袋
            foreach (RenamedCardData data in inn.CurrentParty.BagCards)
            {
                IndexCardData icd = inn.FindIndexedCardData(data);

                dr = dtCard2.NewRow();

                dr[0] = icd.CardData.Name;

                dr[1] = Util3.KindToText(icd.CardData.CardKind);

                dtCard2.Rows.Add(dr);

                if (canProgress)
                {
                    countNow++;
                    progress.Value = 100 * countNow / maxCount;
                }
            }

            if (canProgress)
            {
                progress.Message = "";
            }
        }

        private void SetPartyCount()
        {
            if (this.cmbSend.SelectedIndex == 0)
            {
                // 荷物袋
                this.lblPartyCount.Text = inn.CurrentParty.BagCards.Count.ToString();
            }
            else
            {
                // キャラクタ
                int charIndex = this.cmbSend.SelectedIndex - 1;
                CharacterData cd = inn.CurrentParty.Characters[charIndex];
                this.lblPartyCount.Text = cd.Skills.Count.ToString() 
                    + "/" + cd.Items.Count.ToString() 
                    + "/" + cd.Beasts.Count.ToString();
            }
        }

        private void cmbSend_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.cmbSend.SelectedIndex >= 0)
            {
                if (this.cmbSend.SelectedIndex == 0)
                {
                    // 荷物袋
                    SetPartyCard();

                    this.btnPartyToFirst.Enabled = true;
                    this.btnPartyToEnd.Enabled = true;
                    this.btnPartySort.Enabled = true;
                }
                else
                {
                    // キャラクタ
                    SetCharaCard();

                    this.btnPartyToFirst.Enabled = false;
                    this.btnPartyToEnd.Enabled = false;
                    this.btnPartySort.Enabled = false;
                }

                SetPartyCount();
            }
        }

        private void SetCharaCard()
        {
            dtCard2.Rows.Clear();
            DataRow dr;

            // キャラクタ
            int charIndex = this.cmbSend.SelectedIndex - 1;
            CharacterData cd = inn.CurrentParty.Characters[charIndex];

            foreach (WidCard wc in cd.Skills)
            {
                dr = dtCard2.NewRow();

                dr[0] = wc.CardName;
                dr[1] = CWData.SKILL;

                dtCard2.Rows.Add(dr);
            }

            foreach (WidCard wc in cd.Items)
            {
                dr = dtCard2.NewRow();

                dr[0] = wc.CardName;
                dr[1] = CWData.ITEM;

                dtCard2.Rows.Add(dr);
            }

            foreach (WidCard wc in cd.Beasts)
            {
                dr = dtCard2.NewRow();

                dr[0] = wc.CardName;
                dr[1] = CWData.BEAST;

                dtCard2.Rows.Add(dr);
            }
        }

        private void dgvCard1_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                List<HandCard> hands = new List<HandCard>();
                foreach (RenamedCardData data in inn.BankBoxCards)
                {
                    hands.Add(inn.FindIndexedCardData(data).HandCard);
                }

                using (CardInfoDialog cid = new CardInfoDialog(hands, e.RowIndex))
                {
                    cid.ShowDialog();
                    dgvCard1.Rows[cid.SelectedIndex].Selected = true;
                    dgvCard1.FirstDisplayedScrollingRowIndex = cid.SelectedIndex;
                }
            }
        }

        private void dgvCard2_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                int index = e.RowIndex;
                List<HandCard> hands = new List<HandCard>();

                int firstIndex = 0;
                if (this.cmbSend.SelectedIndex > 0)
                {

                    // キャラクタ
                    int charIndex = this.cmbSend.SelectedIndex - 1;
                    CharacterData cd = inn.CurrentParty.Characters[charIndex];


                    HandCard hc;
                    if (dgvCard2.Rows[index].Cells[1].Value.ToString().Equals(CWData.SKILL))
                    {
                        firstIndex = 0;

                        foreach (WidCard wc in cd.Skills)
                        {
                            SkillHandCard shc;

                            hc = new SkillHandCard(wc);

                            shc = (SkillHandCard)hc;
                            shc.UpdateImage(
                                GameParam.GetCardFacterColor(cd,
                                shc.widData.BodyFacter, shc.widData.MindFacter),
                                GameParam.GetSkillRemainColor(shc.widData.SkillRemainUseCount,
                                GameParam.GetMaxSkillUseCount(cd.Level, shc.widData.Level)));

                            hands.Add(shc);
                        }
                    }
                    else if (dgvCard2.Rows[index].Cells[1].Value.ToString().Equals(CWData.ITEM))
                    {
                        firstIndex = cd.Skills.Count;

                        foreach (WidCard wc in cd.Items)
                        {
                            ItemHandCard ihc;

                            hc = new ItemHandCard(wc);

                            ihc = (ItemHandCard)hc;
                            ihc.UpdateImage(
                                GameParam.GetCardFacterColor(cd,
                                ihc.widData.BodyFacter, ihc.widData.MindFacter));

                            hands.Add(ihc);
                        }

                    }
                    else if (dgvCard2.Rows[index].Cells[1].Value.ToString().Equals(CWData.BEAST))
                    {
                        firstIndex = cd.Skills.Count + cd.Items.Count;


                        foreach (WidCard wc in cd.Beasts)
                        {

                            BeastHandCard bhc;

                            hc = new BeastHandCard(wc);

                            bhc = (BeastHandCard)hc;

                            bhc.UpdateImage(
                                GameParam.GetCardFacterColor(cd,
                                bhc.widData.BodyFacter, bhc.widData.MindFacter));

                            hands.Add(bhc);
                        }
                    }
                    else
                    {
                        PartyMaker.CWIErrorMessage("該当するカード種別はありません。(SimpleHandWindow)");
                    }

                    index -= firstIndex;

                }
                else
                {
                    foreach (RenamedCardData data in inn.CurrentParty.BagCards)
                    {
                        hands.Add(inn.FindIndexedCardData(data).HandCard);
                    }
                }

                using (CardInfoDialog cid = new CardInfoDialog(hands, index))
                {
                    cid.ShowDialog();
                    dgvCard2.Rows[cid.SelectedIndex + firstIndex].Selected = true;
                    dgvCard2.FirstDisplayedScrollingRowIndex = cid.SelectedIndex + firstIndex;
                }
            }
        }

        #endregion

    }
}