﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{
    public partial class TextSearchDlg : Form
    {
        #region メンバ

        private Form _owner;

        private static volatile ProgressDlg progress;

        private DataSet ds = new DataSet();

        private bool scenarioFlag = false;
        private string folderPath = string.Empty;
        private List<WidCard> cardList = null;

        private List<List<string>> matchList = null;
        private bool replaceFlag = false;

        #region 宿データ用フィールド
        private string[] CouponColumnNames = 
            {
                "Name",
                "CouponName",
                "Point"
            };

        private string[] CouponCaptions = 
            {
                "PC名",
                "クーポン名",
                "点数"
            };

        private string[] GoshipColumnNames = 
            {
                "GoshipName"
            };

        private string[] GoshipCaptions = 
            {
                "ゴシップ名"
            };

        private string[] CardColumnNames = 
            {
                "Name",
                "CardType",
                "CardName"
            };

        private string[] CardCaptions = 
            {
                "カード所在",
                "カード種類",
                "カード名"
            };
        #endregion

        #region カード置換用フィールド
        private string[] RepColumnNames = 
            {
                "CardName",
                "Match",
                "Hand",
                "Expression",
                "BasePath",
                "InnFullName"
            };

        private string[] RepCaptions = 
            {
                "対象カード名",
                "マッチ",
                "所持",
                "解説",
                "元ファイルパス",
                "宿でのファイル名"
            };
        #endregion

        private InnData inn;

        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public TextSearchDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public TextSearchDlg(Form owner, InnData inn)
            : this(owner, inn, string.Empty)
        { 
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="owner"></param>
        /// <param name="inn"></param>
        /// <param name="folderPath"></param>
        public TextSearchDlg(Form owner, InnData inn, string folderPath)
        {
            InitializeComponent();

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            this.inn = inn;

            progress = new ProgressDlg(this);

            this.scenarioFlag = (false == string.IsNullOrEmpty(folderPath));
            if (this.scenarioFlag)
            {
                this.folderPath = folderPath;
                this.Text = "シナリオカード検索";
            }
            else
            {
                this.Text = "宿テキスト検索";
            }
        }

        /// <summary>
        /// カード置換用コンストラクタ
        /// </summary>
        /// <param name="owner"></param>
        /// <param name="inn"></param>
        /// <param name="ds"></param>
        /// <param name="matchList"></param>
        public TextSearchDlg(Form owner, InnData inn, DataSet ds,
                             List<List<string>> matchList)
            : this(owner, inn, string.Empty)
        {
            this.Text = "カード置換対象リスト";
            this.matchList = matchList;
            replaceFlag = true;
            this.ds = ds;

            this.Width += 340;
            this.dgv1.Width += 340;

            this.btnEnd.Location 
                = new Point(this.Width / 2 - this.btnEnd.Width / 2,
                            this.btnEnd.Location.Y);
        }

        public string SelectedText = null;
        private bool yadoSelectFlg = false;
        private List<string> lst = null;
        /// <summary>
        /// 宿選択用コンストラクタ
        /// </summary>
        /// <param name="yado"></param>
        public TextSearchDlg(Form owner, List<string> yado)
        {
            InitializeComponent();
            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            yadoSelectFlg = true;
            lst = yado;
            this.Text = "宿選択";
        }
        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void TextSearchDlg_Load(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TextSearchDlg_Shown(object sender, EventArgs e)
        {
            if (this.scenarioFlag)
            {
                this.cmbSearch1.Enabled = false;
                this.txtSearch1.Enabled = false;
                this.btnSearch.Enabled = false;

                MakeDt2();
                DispDt2();

            }
            else if (replaceFlag)
            {
                this.cmbSearch1.Enabled = false;
                this.txtSearch1.Enabled = false;
                this.btnSearch.Enabled = false;

                DispDt3();
            }
            else if (yadoSelectFlg)
            {
                this.cmbSearch1.Visible = false;
                this.txtSearch1.Visible = false;
                this.btnSearch.Visible = false;
                this.lblCount.Visible = false;
                this.dgv1.ColumnHeadersVisible = false;
                this.dgv1.Columns.Add("YadoName", "宿名");
                this.dgv1.Columns["YadoName"].Width = 250;
                foreach (string yado in lst)
                {
                    this.dgv1.Rows.Add(yado);
                }
            }
            else 
            {
                this.cmbSearch1.Items.Add("クーポン");
                this.cmbSearch1.Items.Add("宿ゴシップ");
                this.cmbSearch1.Items.Add("カード名");
                this.cmbSearch1.SelectedIndex = 0;

                this.txtSearch1.Select();

                MakeDt();
                DispDt();
            }
        }

        /// <summary>
        /// 閉じるボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void MakeDt()
        {   
            // 新規DT、Column設定
            PrepareNewDt();

            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "PCからデータを読み出し中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            int count = 0;
            foreach (PartyData pd in inn.Parties)
            {
                count += pd.Characters.Count;
            }

            count += inn.Characters.Count;
            int index = 0;

            DataTable dtCoup = ds.Tables["CouponData"];
            DataTable dtGosh = ds.Tables["GoshipData"];
            DataTable dtCard = ds.Tables["CardData"];

            foreach (CharacterData cd in inn.Characters)
            {
                foreach (CouponData coup in cd.Coupons)
                {
                    DataRow dr = dtCoup.NewRow();
                    dr[0] = cd.CardName;
                    dr[1] = coup.Name;
                    dr[2] = coup.Point;
                    dtCoup.Rows.Add(dr);
                }

                foreach (WidCard wc in cd.Skills)
                {
                    DataRow dr = dtCard.NewRow();
                    dr[0] = cd.CardName;
                    dr[1] =
                        "スキル";
                    dr[2] = wc.CardName;
                    dtCard.Rows.Add(dr);
                }

                foreach (WidCard wc in cd.Items)
                {
                    DataRow dr = dtCard.NewRow();
                    dr[0] = cd.CardName;
                    dr[1] =
                        "アイテム";
                    dr[2] = wc.CardName;
                    dtCard.Rows.Add(dr);
                }

                foreach (WidCard wc in cd.Beasts)
                {
                    DataRow dr = dtCard.NewRow();
                    dr[0] = cd.CardName;
                    dr[1] =
                        "召喚獣";
                    dr[2] = wc.CardName;
                    dtCard.Rows.Add(dr);
                }

                index++;
                progress.Value = 80 * index / count;
            }

            progress.Message = "パーティーPCからデータを読み出し中です。";

            foreach (PartyData pd in inn.Parties)
            {
                // パーティーの荷物袋
                List<IndexCardData> bag = new List<IndexCardData>();
                foreach (RenamedCardData rcd in pd.BagCards)
                {
                    IndexCardData icd = inn.FindIndexedCardData(rcd);
                    if (!bag.Contains(icd))
                    {
                        bag.Add(icd);
                    }
                }

                foreach (IndexCardData icd in bag)
                {
                    DataRow dr = dtCard.NewRow();

                    dr[0] = pd.PartyName + " 荷物袋";

                    switch (icd.CardData.CardKind)
                    {
                        case CardIndexData.Kind.skill:
                            dr[1] = "スキル";
                            break;

                        case CardIndexData.Kind.item:
                            dr[1] = "アイテム";
                            break;

                        case CardIndexData.Kind.beast:
                            dr[1] = "召喚獣";
                            break;

                        default:
                            PartyMaker.CWIErrorMessage("カードの種類が間違っています。");
                            break;
                    }

                    dr[2] = icd.WidData.CardName;
                    dtCard.Rows.Add(dr);
                }

                foreach (CharacterData cd in pd.Characters)
                {
                    foreach (CouponData coup in cd.Coupons)
                    {
                        DataRow dr = dtCoup.NewRow();
                        dr[0] = cd.CardName;
                        dr[1] = coup.Name;
                        dr[2] = coup.Point;
                        dtCoup.Rows.Add(dr);
                    }

                    foreach (WidCard wc in cd.Skills)
                    {
                        DataRow dr = dtCard.NewRow();
                        dr[0] = cd.CardName;
                        dr[1] = "スキル";
                        dr[2] = wc.CardName;
                        dtCard.Rows.Add(dr);
                    }

                    foreach (WidCard wc in cd.Items)
                    {
                        DataRow dr = dtCard.NewRow();
                        dr[0] = cd.CardName;
                        dr[1] = "アイテム";
                        dr[2] = wc.CardName;
                        dtCard.Rows.Add(dr);
                    }

                    foreach (WidCard wc in cd.Beasts)
                    {
                        DataRow dr = dtCard.NewRow();
                        dr[0] = cd.CardName;
                        dr[1] = "召喚獣";
                        dr[2] = wc.CardName;
                        dtCard.Rows.Add(dr);
                    }

                    index++;
                    progress.Value = 80 * index / count;
                }
            }

            progress.Value = 80;

            progress.Message = "宿からデータを読み出し中です。";
            index = 0;
            count = inn.Gossips.Count;
            foreach (String str in inn.Gossips)
            {
                DataRow dr = dtGosh.NewRow();
                dr[0] = str;
                dtGosh.Rows.Add(dr);

                index++;
                progress.Value = 80 + 10 * index / count;
            }

            // 宿・カード置場
            List<IndexCardData> okiba = new List<IndexCardData>();
            foreach (RenamedCardData rcd in inn.BankBoxCards)
            {
                IndexCardData icd = inn.FindIndexedCardData(rcd);
                if (!okiba.Contains(icd))
                {
                    okiba.Add(icd);
                }
            }

            foreach (IndexCardData icd in okiba)
            {
                DataRow dr = dtCard.NewRow();

                dr[0] = "カード置場";

                switch (icd.CardData.CardKind)
                {
                    case CardIndexData.Kind.skill:
                        dr[1] = "スキル";
                        break;

                    case CardIndexData.Kind.item:
                        dr[1] = "アイテム";
                        break;

                    case CardIndexData.Kind.beast:
                        dr[1] = "召喚獣";
                        break;

                    default:
                        PartyMaker.CWIErrorMessage("カードの種類が間違っています。");
                        break;
                }

                dr[2] = icd.WidData.CardName;
                dtCard.Rows.Add(dr);
            }

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

        }

        private void PrepareNewDt()
        {
            ds.Clear();

            DataTable dt = new DataTable("CouponData");

            dt.CaseSensitive = true;

            //DataColumn 設定
            DataColumn dataCol;

            for (int i = 0; i < CouponColumnNames.Length; i++)
            {
                dataCol = new DataColumn();
                dataCol.ColumnName = CouponColumnNames[i];
                dataCol.Caption = CouponCaptions[i];
                if (i < 2)
                {
                    dataCol.DataType = typeof(string);
                    dataCol.DefaultValue = string.Empty;
                }
                else if (i == 2)
                {
                    dataCol.DataType = typeof(int);
                    dataCol.DefaultValue = 0;
                }

                dt.Columns.Add(dataCol);
            }

            ds.Tables.Add(dt);

            dt = new DataTable("GoshipData");

            dt.CaseSensitive = true;


            for (int i = 0; i < GoshipColumnNames.Length; i++)
            {
                dataCol = new DataColumn();
                dataCol.ColumnName = GoshipColumnNames[i];
                dataCol.Caption = GoshipCaptions[i];

                dataCol.DataType = typeof(string);
                dataCol.DefaultValue = string.Empty;

                dt.Columns.Add(dataCol);
            }

            ds.Tables.Add(dt);


            dt = new DataTable("CardData");

            dt.CaseSensitive = true;


            for (int i = 0; i < CardColumnNames.Length; i++)
            {
                dataCol = new DataColumn();
                dataCol.ColumnName = CardColumnNames[i];
                dataCol.Caption = CardCaptions[i];

                dataCol.DataType = typeof(string);
                dataCol.DefaultValue = string.Empty;

                dt.Columns.Add(dataCol);
            }

            ds.Tables.Add(dt);
        }

        private void DispDt()
        {
            DataTable dt = new DataTable();

            switch (cmbSearch1.SelectedIndex)
            {
                case 0:
                    dt = ds.Tables["CouponData"];
                    break;
                case 1:
                    dt = ds.Tables["GoshipData"];
                    break;
                case 2:
                    dt = ds.Tables["CardData"];
                    break;
            }

            int dtCount = dt.Rows.Count;

            DataView dv = new DataView(dt);

            //  検索１
            string search1Type = string.Empty;
            switch (cmbSearch1.SelectedIndex)
            {
                case 0:
                    search1Type = "CouponName";
                    break;
                case 1:
                    search1Type = "GoshipName";
                    break;
                case 2:
                    search1Type = "CardName";
                    break;
            }

            string search1Filter = txtSearch1.Text.Equals(string.Empty)
                ? string.Empty
                : search1Type + " LIKE '%" + txtSearch1.Text + "%'";


            dv.RowFilter = string.Empty;
            dv.RowFilter = StringConcateAnd(dv.RowFilter, search1Filter);

            int dvCount = dv.Count;

            dgv1.DataSource = dv;

            lblCount.Text = dvCount + "/" + dtCount;

            if (cmbSearch1.SelectedIndex == 0)
            {
                dgv1.Columns[CouponColumnNames[0]].Width = 80;
                dgv1.Columns[CouponColumnNames[1]].Width = 120;
                dgv1.Columns[CouponColumnNames[2]].Width = 40;
            }
            else if (cmbSearch1.SelectedIndex == 1)
            {
                dgv1.Columns[GoshipColumnNames[0]].Width = 240;
            }
            else
            {
                dgv1.Columns[CardColumnNames[0]].Width = 80;
                dgv1.Columns[CardColumnNames[1]].Width = 60;
                dgv1.Columns[CardColumnNames[2]].Width = 100;

            }

            for (int i = 0; i < dt.Columns.Count;i++)
            {
                dgv1.Columns[i].HeaderText =
                    dt.Columns[i].Caption;
                
            }

            this.Activate();
        }

        private void MakeDt2()
        {
            // 新規DT、Column設定
            PrepareNewDt2();

            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();
            progress.Message = "シナリオからデータを読み出し中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;


            DataTable dtCard = ds.Tables["CardData"];
            DirectoryInfo di = null;
            FileInfo[] fis = null;
            if (folderPath.StartsWith("\""))
            {
                // CABファイル
                MessageDlg.ShowInfoDialog("CABファイルは現在直接扱えません。");


                progress.Value = 100;

                // 進捗ダイアログ終了
                EndProgress();

                this.Close();
            }
            else
            {
                // 普通のフォルダ
                di = new DirectoryInfo(folderPath);
                fis = di.GetFiles("*.wid", SearchOption.TopDirectoryOnly);
                cardList = new List<WidCard>();

                WidCard wc = null;
                DataRow dr = null;
                int index = 0;
                foreach (FileInfo fi in fis)
                {
                    dr = dtCard.NewRow();

                    dr[0] = "シナリオ";

                    if (fi.Name.StartsWith("Skill"))
                    {
                        wc = new WidCard(WidIDtype.スキル);
                        wc.Read(fi.FullName);

                        dr[1] = "スキル";
                    } 
                    else if (fi.Name.StartsWith("Item"))
                    {
                        wc = new WidCard(WidIDtype.アイテム);
                        wc.Read(fi.FullName);

                        dr[1] = "アイテム";
                    }
                    else if (fi.Name.StartsWith("Beast"))
                    {
                        wc = new WidCard(WidIDtype.召喚獣);
                        wc.Read(fi.FullName);

                        dr[1] = "召喚獣";
                    }
                    else
                    {
                        continue;
                    }
                    cardList.Add(wc);

                    dr[2] = wc.CardName;
                    dtCard.Rows.Add(dr);


                    index++;
                    progress.Value = 100 * index / fis.Length;
                }
            }

            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

        }

        private void PrepareNewDt2()
        {
            ds.Clear();

            //DataColumn 設定
            DataColumn dataCol;

            DataTable dt = new DataTable("CardData");

            dt.CaseSensitive = true;


            for (int i = 0; i < CardColumnNames.Length; i++)
            {
                dataCol = new DataColumn();
                dataCol.ColumnName = CardColumnNames[i];
                dataCol.Caption = CardCaptions[i];

                dataCol.DataType = typeof(string);
                dataCol.DefaultValue = string.Empty;

                dt.Columns.Add(dataCol);
            }

            ds.Tables.Add(dt);
        }

        private void DispDt2()
        {
            DataTable dt = new DataTable();

            dt = ds.Tables["CardData"];

            int dtCount = dt.Rows.Count;

            DataView dv = new DataView(dt);

            //  検索

            dv.RowFilter = string.Empty;

            int dvCount = dv.Count;

            dgv1.DataSource = dv;

            lblCount.Text = dvCount + "/" + dtCount;

            dgv1.Columns[CardColumnNames[0]].Width = 80;
            dgv1.Columns[CardColumnNames[1]].Width = 60;
            dgv1.Columns[CardColumnNames[2]].Width = 100;

            for (int i = 0; i < dt.Columns.Count; i++)
            {
                dgv1.Columns[i].HeaderText =
                    dt.Columns[i].Caption;
            }

            this.Activate();
        }

        private void DispDt3()
        {
            DataTable dt = new DataTable();

            dt = ds.Tables["CardData"];

            int dtCount = dt.Rows.Count;

            DataView dv = new DataView(dt);

            //  検索

            dv.RowFilter = string.Empty;

            int dvCount = dv.Count;

            dgv1.DataSource = dv;

            lblCount.Text = dvCount + "/" + dtCount;

            dgv1.Columns[RepColumnNames[0]].Width = 120;
            dgv1.Columns[RepColumnNames[1]].Width = 60;
            dgv1.Columns[RepColumnNames[2]].Width = 40;
            dgv1.Columns[RepColumnNames[3]].Width = 200;
            dgv1.Columns[RepColumnNames[4]].Width = 200;
            dgv1.Columns[RepColumnNames[5]].Width = 100;

            for (int i = 0; i < dt.Columns.Count; i++)
            {
                dgv1.Columns[i].HeaderText =
                    dt.Columns[i].Caption;
            }

            this.Activate();
        }

        private string StringConcateAnd(string baseStr,string addStr)
        {
            if (!baseStr.Equals(string.Empty))
            {
                if (!addStr.Equals(string.Empty))
                {
                    return baseStr + " AND " + addStr;
                }
                else
                {
                    return baseStr;
                }
            }
            else
            {
                return addStr;
            }
        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TextSearchDlg_FormClosing(object sender, FormClosingEventArgs e)
        {

        }

        #endregion

        #region キーイベント

        private void TextSearchDlg_KeyDown(object sender, KeyEventArgs e)
        {
            int dvCount = ((DataView)(dgv1.DataSource)).Count;
            if (e.KeyCode == Keys.PageUp)
            {
                if (0 < dgv1.FirstDisplayedScrollingRowIndex - 15)
                {
                    dgv1.FirstDisplayedScrollingRowIndex -= 15;
                }
                else
                {
                    dgv1.FirstDisplayedScrollingRowIndex = 0;
                }
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.PageDown)
            {
                if (dvCount - 1 > dgv1.FirstDisplayedScrollingRowIndex + 15)
                {
                    dgv1.FirstDisplayedScrollingRowIndex += 15;
                }
                else
                {
                    dgv1.FirstDisplayedScrollingRowIndex = dvCount - 1;
                }
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.Home)
            {

                dgv1.FirstDisplayedScrollingRowIndex = 0;
                e.Handled = true;
            }
            else if (e.KeyCode == Keys.End)
            {

                dgv1.FirstDisplayedScrollingRowIndex = dvCount - 1;
                e.Handled = true;
            }
        }

        #endregion
        
        #region クリック

        private void btnSearch_Click(object sender, EventArgs e)
        {
            if (this.scenarioFlag)
            {
                DispDt2();
            }
            else if (this.replaceFlag)
            {
                DispDt3();
            } 
            else
            {
                DispDt();
            }
        }

        private void dgv1_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {

            if (e.RowIndex >= 0 &&
                e.RowIndex <= dgv1.Rows.Count - 1)
            {
                if (this.scenarioFlag)
                {
                    using (CardInfoDialog cid
                        = new CardInfoDialog(inn, cardList, e.RowIndex))
                    {
                        cid.ShowDialog();
                    }
                }
                else if(this.replaceFlag)
                {

                    DataTable dt = new DataTable();
                    dt = ds.Tables["CardData"];
                    bool exists = (bool)dt.Rows[e.RowIndex][2];

                    List<string> clickedList = matchList[e.RowIndex];
                    if (exists && clickedList.Count > 0)
                    {
                        using (BmpSelectDlg bsd
                            = new BmpSelectDlg(clickedList, true))
                        {
                            if (DialogResult.OK == bsd.ShowDialog())
                            {
                                int selected = bsd.SelectedIndex;

                                if (bsd.ExceptIndexList.Contains(selected))
                                {

                                    MessageDlg.ShowInfoDialog("imageがないので置換え出来ません。");
                                }
                                else
                                {

                                    File.Copy(clickedList[selected], inn.TempFolder + "\\"
                                        + dgv1.Rows[e.RowIndex].Cells[5].Value.ToString(), true);


                                    MessageDlg.ShowInfoDialog(clickedList[selected] + "に置き換えました。");
                                }
                            }
                        }
                    }
                    else
                    {
                        MessageDlg.ShowInfoDialog("宿にカードがないので置換え出来ません。");
                    }
                }
                else if (yadoSelectFlg)
                {
                    SelectedText = (string)dgv1.Rows[e.RowIndex].Cells[0].Value;
                    this.Close();
                }
            }
        }

        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {

            backgroundWorker1.CancelAsync();
            while (this.backgroundWorker1.IsBusy)
            {
                Thread.Sleep(1);
            }

            backgroundWorker1.RunWorkerAsync();
            
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();

            this.Activate();
        }

        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }
        #endregion

  }
}