﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using CWIDatas;
using CWI.Card;
using CWI;

namespace CWI.Dialog
{

    public partial class YadoExtendDlg : Form
    {
        #region メンバ
        private Form _owner = null;
        private InnData inn = null;
        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public YadoExtendDlg()
        {
            // 通常使用しない
            InitializeComponent();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public YadoExtendDlg(Form owner, InnData inn)
        {
            InitializeComponent();

            _owner = owner;
            this.StartPosition = FormStartPosition.Manual;
            this.Left =
            _owner.Left + (_owner.Width - this.Width) / 2;
            this.Top =
                _owner.Top + (_owner.Height - this.Height) / 2;

            this.inn = inn;
        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void YadoExtendDlg_Load(object sender, EventArgs e)
        {
            // 新宿
            SetNewYadoName();

            // バージョン
            cmbVersion.Items.Clear();

            VersionItem item10 = new VersionItem();
            item10.ComboText = "ver1.28";
            item10.Name = InnData.DATAVERSION_10;
            cmbVersion.Items.Add(item10);

            VersionItem item11 = new VersionItem();
            item11.ComboText = "ver1.29～ver1.50";
            item11.Name = InnData.DATAVERSION_11;
            cmbVersion.Items.Add(item11);

            VersionItem item12 = new VersionItem();
            item12.ComboText = "Next(非対応)";
            item12.Name = InnData.DATAVERSION_12;
            cmbVersion.Items.Add(item12);
            
            if (inn.Version == InnData.DATAVERSION_11)
            {
                cmbVersion.SelectedIndex = 1;
                chkSnt6.Checked = inn.BackGroundEffect;
                chkSnt6.Enabled = true;
                groupBox2.Enabled = true;
                lblCardBattle.Visible = false;
                numCardBattle.Visible = false;
                
            }
            else if (inn.Version == InnData.DATAVERSION_12)
            {
                cmbVersion.SelectedIndex = 2;
                chkSnt6.Checked = inn.BackGroundEffect;
                chkSnt6.Enabled = true;
                groupBox2.Enabled = false;
                lblCardBattle.Visible = true;
                numCardBattle.Visible = true;
            }
            else
            {
                //inn.Version == InnData.DATAVERSION_10
                cmbVersion.SelectedIndex = 0;
                chkSnt6.Checked = false;
                chkSnt6.Enabled = false;
                groupBox2.Enabled = true;
                lblCardBattle.Visible = false;
                numCardBattle.Visible = false;
            }

            // 速度調整
            numCardAnime.Value = inn.CardSpeed;
            numBack.Value = inn.ViewSpeed;
            numMessage.Value = inn.MessageSpeed;
            if (inn.Version == InnData.DATAVERSION_12)
            {
                numCardBattle.Value = inn.CardSpeedBattle;
            }
            
            if (inn.Version != InnData.DATAVERSION_12)
            {
                // 音響効果
                this.chkBGM.Checked = inn.BgmON;
                this.chkEffect.Checked = inn.SoundON;
            }

            // 背景の切り替え方式
            cmbBack.Items.Clear();
            for (int i = 0; i <= (int)InnData.BackGroundAnimationEnum.シェーブ; i++)
            {
                cmbBack.Items.Add(((InnData.BackGroundAnimationEnum)i).ToString());
            }
            ////cmbBack.Items.Add("アニメーションなし");
            ////cmbBack.Items.Add("短冊（スレッド）式");
            ////cmbBack.Items.Add("色変換（フェード）式");
            ////cmbBack.Items.Add("ドット置換（シェープ）式）");
            cmbBack.SelectedIndex = (int)inn.BackGroundAnimation;

            // メッセージボード
            // メッセージボードの明るさ。０が最も暗い
            numBoardColor.Value = inn.MessageLight;
            // 装飾フォント
            chkDeco.Checked = inn.DecoFontON;

            // 貼り紙の表示
            cmbScenario.Items.Clear();
            if (inn.Version == InnData.DATAVERSION_12)
            {
                for (int i = 0; i <= (int)InnData.DispScenarioEnumV12.全て表示; i++)
                {
                    cmbScenario.Items.Add(((InnData.DispScenarioEnumV12)i).ToString());
                }
                cmbScenario.SelectedIndex = (int)inn.DispScenarioEffectV12;
            }
            else
            {
                for (int i = 0; i <= (int)InnData.DispScenarioEnum.適応レベルのみ; i++)
                {
                    cmbScenario.Items.Add(((InnData.DispScenarioEnum)i).ToString());
                }
                ////cmbScenario.Items.Add("隠蔽シナリオ、終了済シナリオを表示しない");
                ////cmbScenario.Items.Add("隠蔽シナリオを表示しない");
                ////cmbScenario.Items.Add("全てのシナリオを表示する");
                ////cmbScenario.Items.Add("適応レベルのシナリオのみを表示する");
                cmbScenario.SelectedIndex = (int)inn.DispScenarioEffect;
            }

            // カードの画像補正
            chkHosei1.Checked = inn.ReduceCardCorrectionON;
            chkHosei2.Checked = inn.MagnifyCardCorrectionON;

            // その他
            chkSnt1.Checked = inn.AutoLoadON;
            chkSnt2.Checked = inn.BackGroundCancelON;
            chkSnt3.Checked = inn.MoneyEffectON;
            chkSnt4.Checked = inn.SkipEventByRightClickON;
            chkSnt5.Checked = inn.KeepMaxLevelON;

            // ゲーム宿？
            chkGame.Checked = !inn.IsDebugInn;

        }


        /// <summary>
        /// フォーム表示時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void YadoExtendDlg_Shown(object sender, EventArgs e)
        {


        }

        /// <summary>
        /// フォームが閉じられる時のイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void YadoExtendDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
        }
        #endregion

        #region イベント
        /// <summary>
        /// バージョンチェンジ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbVersion_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbVersion.SelectedIndex == 0)
            {
                this.chkSnt6.Checked = false;
                this.chkSnt6.Enabled = false;
                groupBox2.Enabled = true;
                lblCardBattle.Visible = false;
                numCardBattle.Visible = false;
            }
            else if (cmbVersion.SelectedIndex == 1)
            {
                this.chkSnt6.Enabled = true;
                groupBox2.Enabled = true;
                lblCardBattle.Visible = false;
                numCardBattle.Visible = false;
            }
            else
            {
                this.chkSnt6.Enabled = true;
                groupBox2.Enabled = false;
                lblCardBattle.Visible = true;
                numCardBattle.Visible = true;
            }
        }

        /// <summary>
        /// ゴシップボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGoship_Click(object sender, EventArgs e)
        {
            using (MoveCouponDlg dlg = new MoveCouponDlg(this, inn, MoveCouponDlg.ListType.GoshipList))
            {
                dlg.ShowDialog();
            }
        }

        /// <summary>
        /// 宿文字列を検索します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnInnString_Click(object sender, EventArgs e)
        {
            using (TextSearchDlg tsd = new TextSearchDlg(this, inn))
            {
                tsd.ShowDialog();
            }
        }

        /// <summary>
        /// OKボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            // バージョン
            if (cmbVersion.SelectedIndex == 0)
            {
                inn.Version = InnData.DATAVERSION_10;
            }
            else if (cmbVersion.SelectedIndex == 1)
            {
                inn.Version = InnData.DATAVERSION_11;
            }
            else
            {
                inn.Version = InnData.DATAVERSION_12;
                // 速度調整
                inn.CardSpeedBattle = (uint)numCardBattle.Value;
            }

            // 速度調整
            inn.CardSpeed = (uint)numCardAnime.Value;
            inn.ViewSpeed = (uint)numBack.Value;
            inn.MessageSpeed = (uint)numMessage.Value;

            if (inn.Version != InnData.DATAVERSION_12)
            {
                // 音響効果
                inn.BgmON = this.chkBGM.Checked;
                inn.SoundON = this.chkEffect.Checked;
            }

            // カードの画像補正
            inn.ReduceCardCorrectionON = chkHosei1.Checked;
            inn.MagnifyCardCorrectionON = chkHosei2.Checked;

            // その他
            inn.AutoLoadON = chkSnt1.Checked;
            inn.BackGroundCancelON = chkSnt2.Checked;
            inn.MoneyEffectON = chkSnt3.Checked;
            inn.SkipEventByRightClickON = chkSnt4.Checked;
            inn.KeepMaxLevelON = chkSnt5.Checked;
            inn.BackGroundEffect = chkSnt6.Checked;

            // 貼り紙の表示
            if (inn.Version != InnData.DATAVERSION_12)
            {
                inn.DispScenarioEffect = (InnData.DispScenarioEnum)this.cmbScenario.SelectedIndex;
            }
            else
            {
                inn.DispScenarioEffectV12 = (InnData.DispScenarioEnumV12)this.cmbScenario.SelectedIndex;
            }
            // 背景色
            // メッセージボードのの明るさ。０が最も暗い
            inn.MessageLight = (uint)numBoardColor.Value;
            inn.DecoFontON = chkDeco.Checked;

            // 背景の切り替え方式
            inn.BackGroundAnimation = (InnData.BackGroundAnimationEnum)cmbBack.SelectedIndex;

            // ゲーム宿？
            inn.IsDebugInn = !chkGame.Checked;

            this.Close();
        }

        /// <summary>
        /// キャンセルボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            // 何もせずに終了
            this.Close();
        }
        /// <summary>
        /// 終了印編集
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFinish_Click(object sender, EventArgs e)
        {
            using (MoveCouponDlg dlg = new MoveCouponDlg(this, inn, MoveCouponDlg.ListType.FinishList))
            {
                dlg.ShowDialog();
            }
        }
        /// <summary>
        /// 進行中リスト編集
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnProgress_Click(object sender, EventArgs e)
        {
            using (MoveCouponDlg dlg = new MoveCouponDlg(this, inn, MoveCouponDlg.ListType.ProgressList))
            {
                dlg.ShowDialog();
            }
        }

        /// <summary>
        /// 新宿ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNewYado_Click(object sender, EventArgs e)
        {

            string name = this.txtYadoName.Text;
            if (FilePath.isProhibition(name))
            {
                MessageDlg.ShowInfoDialog("禁則文字が含まれています。");
                return;
            }

            string[] split;
            string strDir;
            List<string> yadoNameList = new List<string>();

            foreach (string folder in
                Directory.GetDirectories(FilePath.GetSystemYadoFolder()))
            {
                split = folder.Split('\\');
                strDir = split[split.Length - 1];
                yadoNameList.Add(strDir);
            }

            if (yadoNameList.Contains(name))
            {
                MessageDlg.ShowInfoDialog("すでにその名前の宿フォルダが存在します。");
                return;
            }

            //新規宿データ作成
            InnData newInn = new InnData(name, true, RdoDbg.Checked);

            if (Directory.Exists(newInn.Folder))
            {
                MessageDlg.ShowInfoDialog("すでにその名前の宿フォルダが存在します。");
                return;
            }
            else
            {
                // 宿フォルダ作成
                Directory.CreateDirectory(newInn.Folder);

                // ないが一応削除を入れて
                FilePath.DeleteFile(newInn.EnvironmentFile);
                newInn.Write(newInn.EnvironmentFile);

                SetNewYadoName();
            }


            MessageDlg.ShowInfoDialog("宿を作成しました。");

        }

        /// <summary>
        /// 宿削除ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnYadoDelete_Click(object sender, EventArgs e)
        {
            // 宿選択ダイアログ(元宿名付き)
            YadoSelectData selectedYado = null;
            string folder = null;
            string tempfolder = null;
            string name = null;
            DialogResult res = DialogResult.No;

            using (YadoSelectDlg ysd = new YadoSelectDlg(this, inn.Name))
            {
                if (DialogResult.OK == ysd.ShowDialog())
                {

                    selectedYado = ysd.SelectedYado;
                    name = selectedYado.Name;
                    folder = selectedYado.Folder;
                    tempfolder = FilePath.GetSystemYadoFolder()
                        + "\\~" + selectedYado.Name;
                    selectedYado = null;

                    res = MessageDlg.ShowYesNoDialog(
                    "宿「" + name + "」を破棄します。よろしいですか？");

                }

            }

            if (DialogResult.Yes == res)
            {
                //外でやらないとつかんでいる可能性あり
                FilePath.DeleteFolder(tempfolder);
                FilePath.DeleteFolder(folder);

                SetNewYadoName();

                MessageDlg.ShowInfoDialog("削除しました。");
            }

        }

        /// <summary>
        /// 新宿名設定
        /// </summary>
        private void SetNewYadoName()
        {

            string basename = "新規宿";
            string tempname = basename;
            uint num = 1;

            while (Directory.Exists(FilePath.GetSystemYadoFolder() + "\\"
                + tempname))
            {
                num++;
                tempname = basename + "(" + num.ToString() + ")";
            }

            txtYadoName.Text = tempname;
        }
        #endregion

        #region クラス
        /// <summary>
        /// バージョンアイテム
        /// </summary>
        public class VersionItem
        {
            /// <summary>
            /// バージョン名
            /// </summary>
            public string Name { get; set; }
            /// <summary>
            /// コンボ表示テキスト
            /// </summary>
            public string ComboText { get; set; }

            public override string ToString()
            {
                return ComboText;
            }
        }
      
        #endregion

    }
}