﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Xml;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Threading;
using System.Linq;

using CWI.Dialog;
using CWIDatas;
using CWI.Card;

namespace CWI
{
    public partial class PartyMaker : Form
    {
        #region メンバ

        private System.Threading.Mutex objMutex;

        /// <summary>
        /// FLAG_TRUE(1)
        /// </summary>
        private readonly int FLAG_TRUE = 1;
        /// <summary>
        /// FLAG_FALSE(0)
        /// </summary>
        private readonly int FLAG_FALSE = 0;

        private readonly int MaxMemberCount = 6;

        private int disp_num_x = 6;

        // 宿
        private InnData inn = null;

        // カードリスト
        private List<HandCard> cardList;
        public List<HandCard> CardList
        {
            get
            {
                if (cardList == null)
                {
                    cardList = new List<HandCard>();
                }

                return cardList;
            }
        }

        // 現在のインデックス
        private int index = 0;

        // マウスの押されたインデックス
        private int mouseDownIndex = -1;
        // マウスの押されたパネル
        private Panel mouseDownPanel = null;
        // マウスの押された座標
        private Point mouseDownPoint = Point.Empty;

        private static volatile ProgressDlg progress;

        // バリアント種類
        public static バリアント種類 variant = バリアント種類.中世ファンタジーⅠ型;

        // シナリオ簡易検索
        // モーダルに戻したので不必要
        //private ScenarioSearchDlg search = null; 

        private string[] variants = null;

        #region 状態保存
        //自動バックアップ
        public bool isAutoBackup = false;
        //BMP変換時にリサイズする
        public bool canResizeBmp = false;
        //CAB圧縮シナリオをシナリオ検索対象に含む
        public bool isIncludedCAB = false;
        //パーティー内での移動はパーティー名を変えない
        public bool noChangePTName = false;
        // 全パーティー解散でカード置場に解散したPTの全カード移動
        public bool allPartyBreakToYadoCard = false;
        // 一括パーティー作成でカード置場のカードを出来るだけパーティーPCに持たせる。
        public bool createFullPartyWithYadoCard = false;
        //カードウィンドウを簡略版で起動
        public bool isSimpleCardWindow = false;
        // 起動バリアントエンジン
        public string execVariant = string.Empty;
        // バリアントエンジン起動する場合にパーティーメーカーを終了するか。
        public bool endBeforeExecVariant = false;
        // PMを終了せずにバリアントエンジン起動する場合にフルスクリーンにするか。
        public bool fullScreenWhenExecVariant = false;
        // フォントバリアントを作成する時にDefInn,Over,Open,Battleを.mp3にする。
        public bool variantMp3 = false;
        // フォントバリアントを作成する時にexpatch.exe があったら適用
        //public bool variantPatch = false;
        // 丸秘
        public bool paramOption = false;
        // 宿選択をシンプルに
        public bool yadoSimple = false;
        // ミュート
        public bool soundMute = false;
        #endregion

        private SelectedFont selectedFont = SelectedFont.ＭＳ明朝;

        private Ikinari selectedMoveTo = Ikinari.交易都市リューン;

        /// <summary>
        /// TRUE("True")
        /// </summary>
        private const string TRUE = "True";
        /// <summary>
        /// FALSE("False")
        /// </summary>
        private const string FALSE = "False";

        private ToolTip toolTip1 = null;

        private bool closingDelete = false;
        #endregion

        #region static

        public static Brush BlackBrush;
        public static Brush WhiteBrush;
        public static Font CWDefaultFont;
        public static StringFormat centerFormat;

        static PartyMaker()
        {
            BlackBrush = new SolidBrush(Color.Black);
            WhiteBrush = new SolidBrush(Color.White);
            CWDefaultFont = new Font("MS UI Gothic", 9F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(128)));

            centerFormat = new StringFormat();//中ぞろえ書式
            centerFormat.Alignment = StringAlignment.Center;

        }

        //DLL の版の取得
        [DllImport("cab32",
           EntryPoint = "CabGetVersion",
            ExactSpelling = true)]
        public extern static ushort CabGetVersion();
        //DLL の実行状況の取得
        [DllImport("cab32",
           EntryPoint = "CabGetRunning",
            ExactSpelling = true)]
        public extern static bool CabGetRunning();
        //書庫のチェック
        [DllImport("cab32",
           EntryPoint = "CabCheckArchive",
            CharSet = CharSet.Ansi,
            ExactSpelling = true)]
        public extern static bool CabCheckArchive(
            [MarshalAs(UnmanagedType.LPStr)]string _szFileName,
            int _iMode);
        //書庫操作一般
        [DllImport("cab32",
            EntryPoint = "Cab",
            CharSet = CharSet.Ansi,
            ExactSpelling = true)]
        public extern static int Cab(
            IntPtr _hwnd,
            [MarshalAs(UnmanagedType.LPStr)]string _szCmdLine,
            [MarshalAs(UnmanagedType.LPStr)]StringBuilder _szOutput,
            uint _dwSize);

        //ファイルを検索
        [DllImport("kernel32",
            EntryPoint = "SearchPath",
            SetLastError = true,
            CharSet = CharSet.Auto,
            ExactSpelling = false)]
        public static extern uint SearchPath(
            [MarshalAs(UnmanagedType.LPTStr)]string lpPath,
            [MarshalAs(UnmanagedType.LPTStr)]string lpFileName,
            [MarshalAs(UnmanagedType.LPTStr)]string lpExtension,
            uint nBufferLength,
            [MarshalAs(UnmanagedType.LPTStr)]StringBuilder lpBuffer,
            out IntPtr lpFilePart);

        // 解像度変更
        [DllImport("user32.dll")]
        public static extern int EnumDisplaySettings(
          string deviceName, int modeNum, ref DEVMODE devMode);
        [DllImport("user32.dll")]
        public static extern int ChangeDisplaySettings(
              ref DEVMODE devMode, int flags);
        [DllImport("user32.dll")]
        public static extern int ChangeDisplaySettings(
              IntPtr devMode, int flags);

        public const int ENUM_CURRENT_SETTINGS = -1;
        //public const int ENUM_REGISTRY_SETTINGS = -2;
        public const int CDS_UPDATEREGISTRY = 0x01;
        public const int CDS_FULLSCREEN = 0x04; 
        public const int DISP_CHANGE_SUCCESSFUL = 0;

        // dmFields
        public const int DM_BITSPERPEL = 0x40000;
        public const int DM_PELSWIDTH = 0x80000;
        public const int DM_PELSHEIGHT = 0x100000;
        public const int DM_DISPLAYFLAGS = 0x200000;
        public const int DM_DISPLAYFREQUENCY = 0x400000;

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct DEVMODE
        {
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
            public string dmDeviceName;
            public short dmSpecVersion;
            public short dmDriverVersion;
            public short dmSize;
            public short dmDriverExtra;
            public int dmFields;
            short dmOrientation;
            short dmPaperSize;
            short dmPaperLength;
            short dmPaperWidth;
            short dmScale;
            short dmCopies;
            short dmDefaultSource;
            short dmPrintQuality;
            short dmColor;
            short dmDuplex;
            short dmYResolution;
            short dmTTOption;
            short dmCollate;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
            public string dmFormName;
            public short dmLogPixels;
            public int dmBitsPerPel;
            public int dmPelsWidth;
            public int dmPelsHeight;
            public int dmDisplayFlags;
            public int dmDisplayFrequency;
            public int dmDisplayOrientation;
        }

        ////public enum DISP_CHANGE : int
        ////{
        ////    Successful = 0,
        ////    Restart = 1,
        ////    Failed = -1,
        ////    BadMode = -2,
        ////    NotUpdated = -3,
        ////    BadFlags = -4,
        ////    BadParam = -5,
        ////    BadDualView = -6
        ////}

        // タスクバー表示・非表示
        [DllImport("User32.dll", EntryPoint = "FindWindow")]
        public static extern Int32 FindWindow(String lpClassName, String lpWindowName);

        [DllImport("User32.dll")]
        public static extern Int32 FindWindowEx(Int32 hWndParent, IntPtr hWndChild, String lpClassName, String lpWindowName);

        [DllImport("User32.dll", EntryPoint = "ShowWindow")]
        public static extern int ShowWindow(Int32 hWnd, int nCmdShow);
        const int SW_HIDE = 0;
        const int SW_SHOW = 5;


        //今のところうまくいかなかったのでコメントアウト
        ////[DllImport("user32.dll")]
        ////[return: MarshalAs(UnmanagedType.Bool)]
        ////private static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter,
        ////    int x, int y, int width, int height, uint flags);

        //////ウィンドウ表示リストのトップに置く
        //////非アクティブになっても常にトップ位置を保持する
        ////static readonly IntPtr HWND_TOPMOST = new IntPtr(-1);

        ////public const uint SWP_SHOWWINDOW = 0x40;
        
        #endregion

        #region コンストラクタ

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public PartyMaker()
        {
            InitializeComponent();

            //テスト中はこのSetApplicationFolderとFilePathのRmCWIリソースをフルパス指定しておく
#if DEBUG
            FilePath.SetApplicationFolder(FilePath.CW_PATH);
#else
            //FilePath.SetApplicationFolder(FilePath.CW_PATH);
            FilePath.SetApplicationFolder(System.Windows.Forms.Application.StartupPath);
#endif

            MouseWheel += new MouseEventHandler(PartyMaker_MouseWheel);

            progress = new ProgressDlg(this);

            // 打ち落とし用バリアントエンジン文字列配列
            variants = new string[]{
                CWData.Variant_CWN,
                CWData.Variant_CW,
                CWData.Variant_CW_AP,
                CWData.Variant_CW_Vip,
                CWData.Variant_CWD_COMN,
                CWData.Variant_CWD_HERO,
                CWData.Variant_CWD_STD,
                CWData.Variant_gs,
                CWData.Variant_Oedo,
                CWData.Variant_Modern,
                CWData.Variant_School,
                CWData.Variant_Dark
            };

            // バリアントチェック
            CheckVariant();

            // バリアント用定数設定
            SetVariantConst(variant);

            // Yadoフォルダがあるかどうかをチェック
            if (!Directory.Exists(FilePath.GetSystemYadoFolder()))
            {
                PartyMaker.CWIErrorMessage(
                    FilePath.SystemYadoFolderName + "フォルダが見つかりません。\r\nフォルダ構成を確認してください。");

            }

            // Dataフォルダがあるかどうかをチェック
            if (!Directory.Exists(FilePath.GetSystemDataFolder()))
            {
                PartyMaker.CWIErrorMessage(
                    FilePath.SystemDataFolderName + "フォルダが見つかりません。\r\nフォルダ構成を確認してください。");

            }

        }

        #endregion

        #region フォーム

        /// <summary>
        /// フォームロード
        /// </summary>
        private void PartyMaker_Load(object sender, EventArgs e)
        {
            // 中央表示
            SetWindowCenter();

            // メッセージボックス使用のため
            new MessageDlg();

            // 二重起動防止
            objMutex = new System.Threading.Mutex(false, "PartyMaker");

            if (objMutex.WaitOne(0, false) == false)
            {
                PartyMaker.CWIErrorMessage("すでにPartyMakerは起動しています。");
            }

            // カードワースエンジン起動中に起動禁
            Process[] ps;
            
            foreach(string engineName in variants)
            {
                ps = Process.GetProcessesByName(engineName.Substring(0, engineName.Length - 4));
                if (ps.Length > 0)
                {
                    PartyMaker.CWIErrorMessage("CardWirthエンジンが起動しています。\r\nPartyMakerは起動できません。");
                }           
            }

            // タイマー開始
            this.timer1.Enabled = true;

            // 拡張コンボ
            this.cmbExtend.Items.Clear();
            this.cmbExtend.Items.Add("宿関連");
            this.cmbExtend.Items.Add("拡張");
            this.cmbExtend.Items.Add("拡張２");
            this.cmbExtend.SelectedIndex = 1;

            // ﾊﾟｰﾃｨｰﾒｲｸコンボ
            this.cmbPartyMake.Items.Clear();
            this.cmbPartyMake.Items.Add("");
            this.cmbPartyMake.Items.Add("ＬＶ");
            this.cmbPartyMake.Items.Add("名前");
            this.cmbPartyMake.Items.Add("ｸｰﾎﾟﾝ名");
            this.cmbPartyMake.SelectedIndex = 0;

            // ツールチップ
            toolTip1 = new ToolTip(this.components);
            toolTip1.UseFading = true;
            toolTip1.InitialDelay = 100;
            toolTip1.ShowAlways = false;
            toolTip1.Active = true;

            toolTip1.SetToolTip(this.btnYado, "宿を選択します。");
            toolTip1.SetToolTip(this.btnSave, "宿を保存します。");
            toolTip1.SetToolTip(this.btnPartySelect, "パーティーを選択します。");
            toolTip1.SetToolTip(this.btnPartyBreakUp, "パーティーを解散します。");
            toolTip1.SetToolTip(this.btnAbort, "パーティの冒険を中断する。");
            toolTip1.SetToolTip(this.btnPartyLeft, "１つ前のパーティーに変更します。");
            toolTip1.SetToolTip(this.btnPartyRight, "１つ後のパーティーに変更します。");
            toolTip1.SetToolTip(this.btnPartyMoney, "パーティーの情報を表示します。");
            toolTip1.SetToolTip(this.btnItem, "パーティーのアイテムを表示します。");
            toolTip1.SetToolTip(this.btnNewCharacter, "新規ＰＣ作成ダイアログを表示します。");
            toolTip1.SetToolTip(this.btnCWExec, "CardWirthエンジンを起動します。");
            toolTip1.SetToolTip(this.btnExtendExec, "宿情報や拡張機能を表示します。");
            toolTip1.SetToolTip(this.btnEnd, "パーティーメーカーを終了します。");

        }

        private void SetWindowCenter()
        {
            // マウス・カーソルの位置をスクリーン中央に移動させる
            int centerX = Screen.PrimaryScreen.Bounds.Right / 2;
            int centerY = Screen.PrimaryScreen.Bounds.Bottom / 2;
            Point cursolPosition = new Point(centerX, centerY);
            this.Location = new Point(centerX - this.Width / 2, centerY - this.Height / 2);
        }

        /// <summary>
        /// バリアントチェック
        /// </summary>
        private void CheckVariant()
        {
            // 同じフォルダにあるバリアントチェック
            if (File.Exists(FilePath.ApplicationFolder + "\\"
                + CWData.Variant_CW_AP))
            {
                // CardWirth_AP.exe
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CW_AP;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_CW256))
            {
                // CardWirth_256.exe
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CW256;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_CW_Vip))
            {
                // CW#Vip_N.exe
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CW_Vip;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_CW_MP))
            {
                // CWMP.exe
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CW_MP;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_CW))
            {
                // SFバリアントはexe名とデータ構造が同じなので
                // 今のところ中世Ⅰ型として扱う
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CW;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_Oedo))
            {
                variant = バリアント種類.大江戸;

                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_Oedo;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_Modern))
            {
                variant = バリアント種類.現代Ⅰ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_Modern;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_School))
            {
                variant = バリアント種類.学園;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_School;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_Dark))
            {
                variant = バリアント種類.妖魔;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_Dark;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                + "\\" + CWData.Variant_gs))
            {
                // グリマスポットバリアントは
                // バリアントの種類としてはCWエンジンと変わらないので
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_gs;
            }
            else if (File.Exists(FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CWD_STD))
            {
                // CW´は
                // バリアントの種類としてはCWエンジンと変わらないので
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CWD_STD;
            }
            else if (File.Exists(FilePath.ApplicationFolder
               + "\\" + CWData.Variant_CWD_HERO))
            {
                // CW´は
                // バリアントの種類としてはCWエンジンと変わらないので
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CWD_HERO;
            }
            else if (File.Exists(FilePath.ApplicationFolder
               + "\\" + CWData.Variant_CWD_COMN))
            {
                // CW´は
                // バリアントの種類としてはCWエンジンと変わらないので
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CWD_COMN;
            }
            else if (File.Exists(FilePath.ApplicationFolder + "\\"
                + CWData.Variant_CWN))
            {
                // CardWirthNext.exe
                variant = バリアント種類.中世ファンタジーⅠ型;
                FilePath.VariantExe = FilePath.ApplicationFolder
                    + "\\" + CWData.Variant_CWN;
            }
            else
            {
                PartyMaker.CWIErrorMessage(
                    "エンジンがありません。\r\nCardWirth.exeかバリアントエンジンと\r\n同じフォルダに置いて下さい。");
            }
        }

        /// <summary>
        /// フォームが閉じる時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyMaker_FormClosing(object sender, FormClosingEventArgs e)
        {
            // リソース開放したい
            if (closingDelete)
            {
                DeleteAllTempFolder();
            }
            MultiMedia.SoundStop();
        }

        /// <summary>
        /// フォーム表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyMaker_Shown(object sender, EventArgs e)
        {
            //チェック状態読み出し
            LoadInitXml();

            //コマンドラインを配列で取得する
            string[] cmds;
            cmds = System.Environment.GetCommandLineArgs();

            if (cmds.Length != 2)
            {
                ShowYadoSelector();
            }
            else
            {
                string yadoName = cmds[1];
                string[] splitName = yadoName.Split('\\');
                if (splitName.Length > 1)
                {
                    // フォルダをドラッグドロップ実行の場合は
                    // 宿名部分だけにする。
                    yadoName = splitName[splitName.Length - 1];
                }

                try
                {
                    inn = new InnData(yadoName);
                }
                catch (Exception)
                {
                    // 宿を読み込めませんでした
                    MessageDlg.ShowInfoDialog("宿を読み込めませんでした");
                }
                LoadNewInn();
            }
        }

        /// <summary>
        /// リサイズ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyMaker_Resize(object sender, EventArgs e)
        {

            if (this.Width >= CWData.FORM_MIN_WIDTH
                && this.Height >= CWData.FORM_MIN_HEIGHT)
            {

                this.grpYadoPc.Width = this.ClientSize.Width - 9 - 15;
                this.grpYadoPc.Height = this.ClientSize.Height - 224 - 14;

                this.pnlPc.Width = this.grpYadoPc.Width - 15 * 2;
                this.pnlPc.Height = this.grpYadoPc.Height - 17 - 15;
                int old_num_x = this.disp_num_x;
                this.disp_num_x = (this.pnlPc.Width - 10) / CWData.DispCard_Width;

                if (old_num_x != this.disp_num_x)
                {
                    SetLocationPnlPc();
                }

            }
        
        }

        /// <summary>
        /// フォーム閉じた後イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyMaker_FormClosed(object sender, FormClosedEventArgs e)
        {
            objMutex.Close();
        }

        /// <summary>
        /// 終了
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEnd_Click(object sender, EventArgs e)
        {
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
                "保存されていないデータはすべて消えてしまいます。\r\nよろしいですか？"))
            {
                closingDelete = true;
                this.Close();
            }
        }

        /// <summary>
        /// キーダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyMaker_KeyDown(object sender, KeyEventArgs e)
        {
            //KeyCode + Modifiers = KeyData
            if ((e.Modifiers & Keys.Control) == Keys.Control)
            {
                if (e.KeyCode == Keys.Up)
                {
                    ScrollPnlPc(-100);
                }
                else if (e.KeyCode == Keys.Down)
                {
                    ScrollPnlPc(100);
                }
            }

            if (e.KeyCode == Keys.PageUp)
            {
                ScrollPnlPc(-100);
            }
            else if(e.KeyCode == Keys.PageDown)
            {
                ScrollPnlPc(100);
            }
            else if (e.KeyCode == Keys.Home)
            {
                ScrollPnlPc(-CWData.DispCard_Height * ((pnlPc.Controls.Count - 1) / disp_num_x + 1)); 
            }
            else if (e.KeyCode == Keys.End)
            {
                ScrollPnlPc(CWData.DispCard_Height * ((pnlPc.Controls.Count - 1) / disp_num_x + 1));
            }
        }
        #endregion

        #region バリアント設定
        /// <summary>
        /// バリアント用定数の設定
        /// </summary>
        public void SetVariantConst(バリアント種類 variant)
        {
            switch (variant)
            {
                case バリアント種類.中世ファンタジーⅠ型:
                    FilePath.SystemYadoFolderName = "Yado";
                    FilePath.SystemScenarioFolderName = "Scenario";
                    FilePath.SetDataFolderName("Data");

                    CWData.TYPE_NORMAL = "＿標準型";
                    CWData.TYPE_ALLMIGHTY = "＿万能型";
                    CWData.TYPE_BRAVE = "＿勇将型";
                    CWData.TYPE_DARING = "＿豪傑型";
                    CWData.TYPE_RESOURCEFUL = "＿知将型";
                    CWData.TYPE_TACTIC = "＿策士型";

                    CWData.TYPE_MATCHLESS = "＿無双型";
                    CWData.TYPE_GENIUS = "＿天才型";
                    CWData.TYPE_BRILLIANT = "＿英明型";
                    CWData.TYPE_ORDINARY = "＿凡庸型";
                    CWData.TYPE_HERO = "＿英雄型";
                    CWData.TYPE_GOD = "＿神仙型";

                    CWData.ADULT_COUPON_NAME = "熟練";
                    CWData.OLD_COUPON_NAME = "老獪";

                    FilePath.PartyInitialNameEnd = "一行";
                    Util1.MoneyUnit = " sp  ";
                    break;
                case バリアント種類.大江戸:
                    FilePath.SystemYadoFolderName = "Naga";
                    FilePath.SystemScenarioFolderName = "Ohanashi";
                    FilePath.SetDataFolderName("D_E1");

                    CWData.TYPE_NORMAL = "＿標準型";
                    CWData.TYPE_ALLMIGHTY = "＿隠密型";
                    CWData.TYPE_BRAVE = "＿剣客型";
                    CWData.TYPE_DARING = "＿豪傑型";
                    CWData.TYPE_RESOURCEFUL = "＿参謀型";
                    CWData.TYPE_TACTIC = "＿策士型";

                    CWData.TYPE_MATCHLESS = "＿剣豪型";
                    CWData.TYPE_GENIUS = "＿賢才型";
                    CWData.TYPE_BRILLIANT = "＿秀英型";
                    CWData.TYPE_ORDINARY = "＿晩成型";
                    CWData.TYPE_HERO = "＿覇道型";
                    CWData.TYPE_GOD = "＿神仙型";

                    CWData.ADULT_COUPON_NAME = "熟練";
                    CWData.OLD_COUPON_NAME = "老獪";


                    FilePath.PartyInitialNameEnd = "一行";
                    Util1.MoneyUnit = " 文  ";
                    break;
                case バリアント種類.現代Ⅰ型:
                    FilePath.SystemYadoFolderName = "yado";
                    FilePath.SystemScenarioFolderName = "Scenario";
                    FilePath.SetDataFolderName("D_M1");

                    CWData.TYPE_NORMAL = "＿標準型";
                    CWData.TYPE_ALLMIGHTY = "＿万能型";
                    CWData.TYPE_BRAVE = "＿勇将型";
                    CWData.TYPE_DARING = "＿豪傑型";
                    CWData.TYPE_RESOURCEFUL = "＿知将型";
                    CWData.TYPE_TACTIC = "＿策士型";

                    CWData.TYPE_MATCHLESS = "＿無双型";
                    CWData.TYPE_GENIUS = "＿天才型";
                    CWData.TYPE_BRILLIANT = "＿英明型";
                    CWData.TYPE_ORDINARY = "＿凡庸型";
                    CWData.TYPE_HERO = "＿英雄型";
                    CWData.TYPE_GOD = "＿神仙型";

                    CWData.ADULT_COUPON_NAME = "熟練";
                    CWData.OLD_COUPON_NAME = "老獪";


                    FilePath.PartyInitialNameEnd = "一行";
                    Util1.MoneyUnit = " sp  ";
                    break;
                case バリアント種類.学園:
                    FilePath.SystemYadoFolderName = "heya";
                    FilePath.SystemScenarioFolderName = "Rumormil";
                    FilePath.SetDataFolderName("D_S1");

                    CWData.TYPE_NORMAL = "＿標準型";
                    CWData.TYPE_ALLMIGHTY = "＿万能型";
                    CWData.TYPE_BRAVE = "＿根性型";
                    CWData.TYPE_DARING = "＿熱血型";
                    CWData.TYPE_RESOURCEFUL = "＿理性型";
                    CWData.TYPE_TACTIC = "＿秀才型";

                    CWData.TYPE_MATCHLESS = "＿無双型";
                    CWData.TYPE_GENIUS = "＿天才型";
                    CWData.TYPE_BRILLIANT = "＿英明型";
                    CWData.TYPE_ORDINARY = "＿努力型";
                    CWData.TYPE_HERO = "＿超人型";
                    CWData.TYPE_GOD = "＿神仙型";

                    CWData.ADULT_COUPON_NAME = "教師";
                    CWData.OLD_COUPON_NAME = "教諭";

                    FilePath.PartyInitialNameEnd = "たち";
                    Util1.MoneyUnit = " sp  ";
                    break;
                case バリアント種類.妖魔:

                    FilePath.SystemYadoFolderName = "mori";
                    FilePath.SystemScenarioFolderName = "EventSet";
                    FilePath.SetDataFolderName("F_dt");

                    CWData.TYPE_NORMAL = "＿多種族";
                    CWData.TYPE_ALLMIGHTY = "＿人獣族";
                    CWData.TYPE_BRAVE = "＿悪鬼族";
                    CWData.TYPE_DARING = "＿蜥蜴族";
                    CWData.TYPE_RESOURCEFUL = "＿妖木族";
                    CWData.TYPE_TACTIC = "＿小悪魔";

                    CWData.TYPE_MATCHLESS = "＿鬼人族";
                    CWData.TYPE_GENIUS = "＿大悪魔";
                    CWData.TYPE_BRILLIANT = "＿人狼族";
                    CWData.TYPE_ORDINARY = "＿妖虫族";
                    CWData.TYPE_HERO = "＿闇の者";
                    CWData.TYPE_GOD = "＿神竜族";

                    CWData.ADULT_COUPON_NAME = "熟練";
                    CWData.OLD_COUPON_NAME = "老獪";


                    FilePath.PartyInitialNameEnd = "の群";
                    Util1.MoneyUnit = " g   ";
                    break;
            }

            // 表示する素質型・特殊型の設定
            SetDispType();

        }

        /// <summary>
        /// 表示する素質型・特殊型の設定
        /// </summary>
        public void SetDispType()
        {
            CWData.DISP_NORMAL = CWData.TYPE_NORMAL.TrimStart('＿');
            CWData.DISP_ALLMIGHTY = CWData.TYPE_ALLMIGHTY.TrimStart('＿');
            CWData.DISP_BRAVE = CWData.TYPE_BRAVE.TrimStart('＿');
            CWData.DISP_DARING = CWData.TYPE_DARING.TrimStart('＿');
            CWData.DISP_RESOURCEFUL = CWData.TYPE_RESOURCEFUL.TrimStart('＿');
            CWData.DISP_TACTIC = CWData.TYPE_TACTIC.TrimStart('＿');

            CWData.DISP_MATCHLESS = CWData.TYPE_MATCHLESS.TrimStart('＿');
            CWData.DISP_GENIUS = CWData.TYPE_GENIUS.TrimStart('＿');
            CWData.DISP_BRILLIANT = CWData.TYPE_BRILLIANT.TrimStart('＿');
            CWData.DISP_ORDINARY = CWData.TYPE_ORDINARY.TrimStart('＿');
            CWData.DISP_HERO = CWData.TYPE_HERO.TrimStart('＿');
            CWData.DISP_GOD = CWData.TYPE_GOD.TrimStart('＿');
        }
        #endregion

        #region 宿操作

        /// <summary>
        /// 宿選択
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnYado_Click(object sender, EventArgs e)
        {

            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
                "保存されていないデータはすべて消えてしまいます。\r\nよろしいですか？"))
            {
                ShowYadoSelector();
            }
        }

        /// <summary>
        /// 宿選択ダイアログ表示・宿選択
        /// </summary>
        private void ShowYadoSelector()
        {

            // 進捗ダイアログ開始
            StartProgress();
            // 進捗ダイアログ初期処理待ち
            WaitProgress();

            progress.Message = "宿ダイアログを表示中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;


            DialogResult res;

            if (!this.yadoSimple)
            {
                using (YadoSelectDlg yadoSelector = new YadoSelectDlg(this))
                {

                    progress.Value = 95;

                    //BGMを変更
                    MultiMedia.PlayMusicFile(FilePath.GetDefOpenMidi(), this.soundMute);

                    progress.Value = 100;

                    // 進捗ダイアログ終了
                    EndProgress();

                    // ダイアログにはOwner設定がいる？
                    yadoSelector.Owner = this;
                    res = yadoSelector.ShowDialog();

                    yadoSelector.Hide();

                    if (res.Equals(DialogResult.OK))
                    {
                        // 宿を選んだ場合                  
                        // 旧~宿削除
                        if (inn != null)
                        {
                            inn.DeleteTempFolder();
                        }
                        // 旧宿消滅とともに新宿設定
                        inn = new InnData(yadoSelector.SelectedYado);

                    }
                    else
                    {
                        if (inn == null)
                        {
                            // 最初にキャンセルされたときは
                            closingDelete = true;
                            this.Close();
                            return;
                        }

                    }

                }

                this.Refresh();
                Thread.Sleep(50);                               // 0.05秒間 待つ

                if (res.Equals(DialogResult.OK))
                {
                    LoadNewInn();
                }

            }
            else
            {
                // 簡略版
                DirectoryInfo di = new DirectoryInfo(FilePath.GetSystemYadoFolder());
                IEnumerable<FileInfo> fis = di.GetFiles(FilePath.Environment_wyd, SearchOption.AllDirectories)
                    .OrderByDescending(dateTime => dateTime.LastAccessTime);
                List<string> lst = new List<string>();
                foreach(FileInfo fi in fis){
                    string yadoName = fi.DirectoryName;
                    string[] splitName = yadoName.Split('\\');
                    if (splitName.Length > 1)
                    {
                        // 宿名部分だけにする。
                        yadoName = splitName[splitName.Length - 1];
                    }
                    lst.Add(yadoName);
                }

                progress.Value = 100;

                // 進捗ダイアログ終了
                EndProgress();

                using (TextSearchDlg tsd = new TextSearchDlg(this, lst))
                {
                    tsd.ShowDialog();
                    string ret = tsd.SelectedText;
                    if (ret != null)
                    {
                        string yadoName = ret;

                        try
                        {
                            inn = new InnData(yadoName);
                        }
                        catch (Exception)
                        {
                            // 宿を読み込めませんでした
                            MessageDlg.ShowInfoDialog("宿を読み込めませんでした");
                        }

                        this.Refresh();
                        Thread.Sleep(50);                               // 0.05秒間 待つ

                        LoadNewInn();
                    }
                    else
                    {
                        this.Close();
                    }
                }
            }


        }

        /// <summary>
        /// 宿をロード
        /// </summary>
        private void LoadNewInn()
        {

            // 進捗ダイアログ開始
            StartProgress();

            // 進捗ダイアログ初期処理待ち
            WaitProgress();

            progress.Message = "~宿作成中です。";
            progress.Maximum = 100;
            progress.Minimum = 0;
            progress.Value = 0;

            // ~宿作成
            DirectoryInfo di = inn.CopyFolderToTemp();

            progress.Value = 20;

            // PC読み込み・パーティー読み込み

            progress.Message = "ＰＣ読み込み中です。";
            inn.ReadPcs(di);

            progress.Value = 40;

            progress.Message = "パーティー読み込み中です。";
            inn.ReadParties(di);

            // カレントパーティー表示
            SetUpCurrentParty(inn.CurrentParty);

            progress.Value = 50;

            // 宿帳PC表示
            SetUpYadoPcData(inn.Characters);

            progress.Value = 90;

            progress.Message = "後処理を実行中です。";

            inn.AfterTempFolderProc();

            progress.Value = 95;

            if (isAutoBackup)
            {
                Thread.Sleep(10);                               // 0.01秒間 待つ
                progress.Message = "宿をバックアップ中です。";
                inn.BackupYadoFolder();
            }
            progress.Value = 100;

            // 進捗ダイアログ終了
            EndProgress();

            // カレント宿設定
            SetUpCurrentInn(inn);
        }

        /// <summary>
        /// 保存ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            if (inn != null)
            {
                if (MessageDlg.ShowYesNoDialog("宿を保存します。よろしいですか？")
                        == DialogResult.Yes)
                {
                    inn.Save();

                    MessageDlg.ShowInfoDialog("正常に保存されました。");
                }
            }
        }

        /// <summary>
        /// 宿カード移動機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCard_Click(object sender, EventArgs e)
        {
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "移動を終了した時に他宿を保存しますが、\r\nよろしいですか？"))
            {
                // 保存してもいい場合
                // 宿選択ダイアログ(元宿名付き)
                using (YadoSelectDlg ysd = new YadoSelectDlg(this,inn.Name))
                {
                    if (DialogResult.OK == ysd.ShowDialog())
                    {
                        InnData selectedInn = new InnData(ysd.SelectedYado);
                        using (MoveCardDlg mvDlg = new MoveCardDlg(inn, selectedInn))
                        {
                            // PC宿移動用ダイアログ表示
                            mvDlg.ShowDialog();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 宿から他宿へ移動機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMove_Click(object sender, EventArgs e)
        {

            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "移動を終了した時に他宿を保存しますが、\r\nよろしいですか？"))
            {
                // 保存してもいい場合
                // 簡易宿選択ダイアログ(元宿名付き)
                DialogResult res = DialogResult.Cancel;
                List<DispCard> list = new List<DispCard>();
                InnData SelectedInn = null;

                using (YadoSelectDlg ysd = new YadoSelectDlg(this,inn.Name))
                {
                    res = ysd.ShowDialog();
                    if (DialogResult.OK == res)
                    {

                        SelectedInn = new InnData(ysd.SelectedYado);

                        // 現在表示中の待機PCはそのままリストへ
                        foreach (Control ctrl in pnlPc.Controls)
                        {
                            list.Add(ctrl as DispCard);
                        }

                        // 戻ってきたときの表示準備でクリア
                        pnlPc.Controls.Clear();
                        pnlPc.AutoScrollPosition = new Point(0, 0);

                    }
                }

                if (DialogResult.OK == res)
                {

                    using (MovePCDlg mvDlg = new MovePCDlg(this, inn, list, SelectedInn))
                    {

                        // PC宿移動用ダイアログ表示
                        mvDlg.ShowDialog();

                        // 待機PC最新状態反映
                        this.pnlPc.SuspendLayout();
                        index = 0;
                        foreach (DispCard dc in mvDlg.YadoPCs)
                        {
                            dc.Location = new Point(CWData.DispCard_Width
                                * (index % disp_num_x), CWData.DispCard_Height * (index / disp_num_x));
                            pnlPc.Controls.Add(dc);
                            index++;
                        }

                        this.pnlPc.ResumeLayout();
                    }
                }

            }
            
        }

        /// <summary>
        /// 宿中のアイテム移動機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnItem_Click(object sender, EventArgs e)
        {
            GC.Collect(2, GCCollectionMode.Forced);
            // 拡張ハンドウインドウ選択ダイアログ
            if (this.isSimpleCardWindow)
            {
                using (SimpleHandWindow shw = new SimpleHandWindow(this, inn))
                {
                    //簡略版ダイアログ表示
                    shw.ShowDialog();
                }
            }
            else
            {
                using (ExtHandWindow ehw = new ExtHandWindow(this, inn))
                {
                    // 拡張ハンドウインドウ選択ダイアログ表示
                    ehw.ShowDialog();
                }
            }
        }

        /// <summary>
        /// 新規キャラクタ作成ボタン　クリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNewCharacter_Click(object sender, EventArgs e)
        {
            int charaNum = inn.Characters.Count;
            using (CharacterDlg charaDlg = new CharacterDlg(this, inn))
            {
                //charaDlg.ShowDialog();
                if(charaDlg.ShowDialog() == DialogResult.OK)
                {
                    // 増えている待機PCを表示反映

                    int nowX = -pnlPc.AutoScrollPosition.X;
                    int nowY = -pnlPc.AutoScrollPosition.Y;

                    pnlPc.AutoScrollPosition = new Point(0, 0);

                    int newIndex = inn.Characters.Count - 1;

                    DispCard addDc = new DispCard(inn.Characters[newIndex]);

                    addDc.Location = 
                        new Point(CWData.DispCard_Width * (newIndex % disp_num_x)
                        , CWData.DispCard_Height * (newIndex / disp_num_x));

                    pnlPc.Controls.Add(addDc);

                    pnlPc.AutoScrollPosition = new Point(nowX, nowY);


                    this.grpYadoPc.Text = "待機PC(" + inn.Characters.Count + ")";
                }
            }

        }

        /// <summary>
        /// CW起動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCWExec_Click(object sender, EventArgs e)
        {
            CWExec();
        }

        #region XML関連

        /// <summary>
        /// フラグ値をXMLに設定
        /// </summary>
        /// <param name="w"></param>
        /// <param name="paramName"></param>
        /// <param name="value"></param>
        private void XMLSet(XmlTextWriter w, string paramName, bool value)
        {
            if (value)
            {
                w.WriteElementString(paramName, TRUE);
            }
            else
            {
                w.WriteElementString(paramName, FALSE);
            }
        }

        /// <summary>
        /// フラグ値をXMLに設定
        /// </summary>
        /// <param name="w"></param>
        /// <param name="paramName"></param>
        /// <param name="value"></param>
        private void XMLSet(XmlTextWriter w, string paramName, string value)
        {
            w.WriteElementString(paramName, value);
        }

        /// <summary>
        /// 宿状態をXML保存
        /// </summary>
        private void SaveInitXml()
        {
            if (inn != null)
            {
                try
                {
                    string xmlFile = FilePath.ApplicationFolder + "\\pm.xml";

                    FilePath.DeleteFile(xmlFile);

                    //XMLファイルを開きます
                    using (FileStream fs
                        = new FileStream(xmlFile, FileMode.Create, FileAccess.Write))
                    {
                        Encoding sjis = Encoding.GetEncoding("Shift-JIS");
                        using (XmlTextWriter w
                            = new XmlTextWriter(fs, sjis))
                        {

                            w.Formatting = Formatting.Indented;
                            w.WriteStartDocument(true);

                            w.WriteStartElement("Root");

                            w.WriteStartElement("Option");

                            {
                                XMLSet(w, "DetailFlag", this.paramOption);

                                XMLSet(w, "AutoBackup", this.isAutoBackup);

                                XMLSet(w, "CanResizeBmp", this.canResizeBmp);

                                XMLSet(w, "IsIncludedCAB", this.isIncludedCAB);

                                XMLSet(w, "NoChangePTName", this.noChangePTName);

                                XMLSet(w, "AllPartyBreakToYadoCard", this.allPartyBreakToYadoCard);

                                XMLSet(w, "CreateFullPartyWithYadoCard", this.createFullPartyWithYadoCard);

                                XMLSet(w, "IsSimpleCardWindow", this.isSimpleCardWindow);

                                XMLSet(w, "ExecVariant", this.execVariant.Equals(string.Empty) ?
                                    FilePath.VariantExe :
                                    this.execVariant);

                                XMLSet(w, "EndPMWithExec", this.endBeforeExecVariant);

                                XMLSet(w, "FullScreenWithExec", this.fullScreenWhenExecVariant);

                                XMLSet(w, "VariantMp3", this.variantMp3);

                                //XMLSet(w, "VariantPatch", this.variantPatch);

                                XMLSet(w, "YadoSelect", this.yadoSimple ? "SIMPLE" : "NORMAL");
                                XMLSet(w, "NoSound", this.soundMute);
                                
                                XMLSet(w, "PartyMakeSelectedIndex", this.cmbPartyMake.SelectedIndex.ToString());
                            }

                            w.WriteEndElement(); //Optionタグを閉じる

                            w.WriteEndElement(); //Rootタグを閉じる 

                            w.WriteEndDocument();

                        }
                    }
                }
                catch (Exception ex)
                {
                    PartyMaker.CWIErrorMessage("設定ファイルの保存に失敗しました" 
                        + ex.Message);
                }
            }
        }

        /// <summary>
        /// 宿状態の読み出し
        /// </summary>
        private void LoadInitXml()
        {

            try
            {
                string xmlFile = FilePath.ApplicationFolder + "\\pm.xml";
                string detailFlag = string.Empty;
                string autoBackup = string.Empty;
                string canResizeBmp = string.Empty;
                string isIncludedCAB = string.Empty;
                string noChangeName = string.Empty;
                string allBreakToCard = string.Empty;
                string createPartyWithCard = string.Empty;
                string simpleCardWindow = string.Empty;
                string endPMWithExec = string.Empty;
                string fullScreenWithExec = string.Empty;
                string variantMidToMp3 = string.Empty;
                string variantExPatch = string.Empty;
                string yadoSelect = string.Empty;
                string noSound = string.Empty;
                string partyMakeSelectedIndex = string.Empty;


                //XMLファイルが存在するか確認
                if (File.Exists(xmlFile))
                {
                    //XMLファイルを開く
                    using (FileStream fStream =
                        new FileStream(xmlFile, FileMode.Open, FileAccess.Read))
                    {
                        using (XmlTextReader reader = new XmlTextReader(fStream))
                        {

                            //XMLファイルを1ノードずつ読み込む
                            while (reader.Read())
                            {
                                reader.MoveToContent();

                                //ノード要素にデータがある場合
                                if (reader.NodeType == XmlNodeType.Element)
                                {
                                    //ノード要素がある場合
                                    if (reader.LocalName.Equals("DetailFlag"))
                                    {
                                        //ノードデータを取得
                                        detailFlag = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("AutoBackup"))
                                    {
                                        autoBackup = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("CanResizeBmp"))
                                    {
                                        canResizeBmp = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("IsIncludedCAB"))
                                    {
                                        isIncludedCAB = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("NoChangePTName"))
                                    {
                                        noChangeName = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("AllPartyBreakToYadoCard"))
                                    {
                                        allBreakToCard = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("CreateFullPartyWithYadoCard"))
                                    {
                                        createPartyWithCard = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("IsSimpleCardWindow"))
                                    {
                                        simpleCardWindow = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("ExecVariant"))
                                    {
                                        this.execVariant = reader.ReadString();
                                        if (File.Exists(FilePath.VariantExe))
                                        {
                                            this.execVariant = FilePath.VariantExe;
                                        }
                                    }
                                    else if (reader.LocalName.Equals("EndPMWithExec"))
                                    {
                                        endPMWithExec = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("FullScreenWithExec"))
                                    {
                                        fullScreenWithExec = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("VariantMp3"))
                                    {
                                        variantMidToMp3 = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("VariantPatch"))
                                    {
                                        variantExPatch = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("YadoSelect"))
                                    {
                                        yadoSelect = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("NoSound"))
                                    {
                                        noSound = reader.ReadString();
                                    }
                                    else if (reader.LocalName.Equals("PartyMakeSelectedIndex"))
                                    {
                                        partyMakeSelectedIndex = reader.ReadString();
                                    }
                                    
                                }

                            }

                        }
                    }

                    this.paramOption = detailFlag.Equals(TRUE);

                    this.isAutoBackup = autoBackup.Equals(TRUE);

                    this.canResizeBmp = canResizeBmp.Equals(TRUE);

                    this.isIncludedCAB = isIncludedCAB.Equals(TRUE);

                    this.noChangePTName = noChangeName.Equals(TRUE);

                    this.allPartyBreakToYadoCard = allBreakToCard.Equals(TRUE);

                    this.createFullPartyWithYadoCard = createPartyWithCard.Equals(TRUE);

                    this.isSimpleCardWindow = simpleCardWindow.Equals(TRUE);

                    if (this.execVariant.Equals(string.Empty))
                    {
                        this.execVariant = FilePath.VariantExe;
                    }

                    this.endBeforeExecVariant = endPMWithExec.Equals(TRUE);
                    this.fullScreenWhenExecVariant = fullScreenWithExec.Equals(TRUE);

                    this.variantMp3 = variantMidToMp3.Equals(TRUE);

                    //this.variantPatch = variantExPatch.Equals(TRUE);

                    this.yadoSimple = yadoSelect.Equals("SIMPLE");
                    this.soundMute = noSound.Equals(TRUE);
                    int index = 0;
                    this.cmbPartyMake.SelectedIndex = int.TryParse(partyMakeSelectedIndex, out index) ? index : 0;
                }
            }
            catch (Exception ex)
            {
                PartyMaker.CWIErrorMessage("設定ファイルの読み込みに失敗しました"
                    + ex.Message);
            }
        
        }
        #endregion

        #endregion

        #region 進捗ダイアログ
        /// <summary>
        /// 進捗ダイアログ表示開始
        /// </summary>
        private void StartProgress()
        {
            backgroundWorker1.RunWorkerAsync();
        }

        /// <summary>
        /// 進捗ダイアログ初期処理待ち
        /// </summary>
        private void WaitProgress()
        {
            while (!progress.LoadOn || !progress.IsHandleCreated) { }
        }

        /// <summary>
        /// 進捗ダイアログ表示終了
        /// </summary>
        private void EndProgress()
        {
            progress.CloseDialog();
            this.Activate();
        }


        /// <summary>
        /// 別スレッド処理１
        /// </summary>
        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            progress = new ProgressDlg(this);
            progress.ShowDialog();

        }

        /// <summary>
        /// 別スレッド処理１終了時処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            progress.Dispose();
        }

        #endregion

        #region シナリオ関連

        /// <summary>
        /// 宿にいるパーティーのシナリオ選択
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSenario_Click(object sender, EventArgs e)
        {
            // 現在使用不可
            if (inn.CurrentParty != null 
                && !inn.CurrentParty.InScenario)
            {
                // 安全のためInSenarioも再度確認後
                // FolderBrowserDialog起動
                string selectedFolder = string.Empty;
                using (FolderBrowserDialog fbd = new FolderBrowserDialog())
                {
                    fbd.ShowNewFolderButton = false;
                    fbd.RootFolder = Environment.SpecialFolder.Desktop;
                    fbd.SelectedPath = FilePath.GetSystemScenarioFolder();

                    // プレイシナリオリスト
                    List<String> playScenarioList = new List<String>();

                    // 全員分のクーポン名リスト
                    List<string> coupons = new List<string>();

                    while (true)
                    {
                        DialogResult res = fbd.ShowDialog();
                        if (!res.Equals(DialogResult.OK))
                        {
                            return;
                        }
                        selectedFolder = fbd.SelectedPath;


                        if (!Directory.Exists(selectedFolder))
                        {
                            //フォルダ自体が削除された
                            MessageDlg.ShowInfoDialog("フォルダが見つかりません。");
                        } 
                        else if (File.Exists(FilePath.GetSummaryFile(selectedFolder)))
                        {
                            //シナリオの追加削除に対応できるように、表示時に情報を取得する
                            //シナリオフォルダである
                            SummaryData summary = new SummaryData();
                            try
                            {
                                summary.Read(FilePath.GetSummaryFile(selectedFolder));

                                // サマリファイルによるシナリオ実行条件チェック

                                // 他のパーティーがそのシナリオを進行中でないかどうかのチェック
                                // TODO:下記のようではなく、宿データの進行中シナリオリストから取得しないといけない
                                playScenarioList.Clear();
                                foreach (string file in Directory.GetFiles(inn.TempFolder, FilePath.MaskIndexPartyData))
                                {
                                    PartyIndexData pIndex = new PartyIndexData();
                                    try
                                    {
                                        pIndex.Read(file);
                                    }
                                    catch (Exception ex)
                                    {
                                        PartyMaker.CWIErrorMessage("宿メンバーファイルの読み込みに失敗：" + ex.Message);
                                    }

                                    if (pIndex.InScenario)
                                    {
                                        playScenarioList.Add(pIndex.PlayingScenarioName);
                                    }
                                }

                                if (playScenarioList.Contains(summary.ScenarioName))
                                {
                                    // 他のパーティーが進行中
                                    MessageDlg.ShowInfoDialog("シナリオ："
                                        + summary.ScenarioName
                                        + " は他のパーティーが進行しています。");
                                }
                                else
                                {

                                    // 終了印チェック
                                    bool isFinish = false;
                                    foreach (string finish in inn.FinishScenarios)
                                    {
                                        if (finish.Equals(summary.ScenarioName))
                                        {
                                            isFinish = true;
                                            break;
                                        }
                                    }

                                    if (isFinish)
                                    {
                                        // 終了印あり
                                        MessageDlg.ShowInfoDialog("シナリオ："
                                            + summary.ScenarioName
                                            + " はすでに終了しています。");
                                    }
                                    else
                                    {
                                        // 全員分のクーポン名
                                        coupons.Clear();
                                        foreach (CharacterData cd in inn.CurrentParty.Characters)
                                        {
                                            foreach (CouponData coup in cd.Coupons)
                                            {
                                                coupons.Add(coup.Name);
                                            }
                                        }

                                        // 必要クーポンチェック
                                        bool isCouponOK = true;
                                        for (int i = 0; i < summary.NeedCouponCount; i++)
                                        {
                                            if (!coupons.Contains(summary.NeedCoupons[i]))
                                            {
                                                // 含まれていなかったら
                                                isCouponOK = false;
                                                break;
                                            }
                                        }

                                        if (!isCouponOK)
                                        {
                                            // 含まれていなかったら
                                            MessageDlg.ShowInfoDialog("シナリオ："
                                                + summary.ScenarioName
                                                + " を選択するのに必要なクーポンが足りません。");
                                        }
                                        else
                                        {

                                            string levelText;//対象レベルの文字
                                            if (summary.LevelLow == summary.LevelHigh)//対象レベルがどちらも一緒なら
                                            {
                                                if (summary.LevelLow == 0)
                                                {
                                                    levelText = string.Empty;
                                                }
                                                else
                                                {
                                                    levelText = " 対象レベル : " + summary.LevelLow;//最低レベルだけ
                                                }
                                            }
                                            else
                                            {
                                                levelText = " 対象レベル : " + summary.LevelLow 
                                                    + " ～ " + summary.LevelHigh;//通常表示
                                            }

                                            if (MessageDlg.ShowYesNoDialog("シナリオ："
                                                + summary.ScenarioName + levelText
                                                + " を選択します。よろしいですか。")
                                                == DialogResult.Yes)
                                            {

                                                //F9用のデータ初期化
                                                inn.CurrentParty.MoneyAtStart = inn.CurrentParty.Money;

                                                ScenarioData scenario = new ScenarioData();
                                                scenario.ScenarioFolder = selectedFolder;


                                                //初期エリア登録
                                                scenario.AreaNumber = summary.AreaNumber;

                                                //フラグ・ステップ初期値登録
                                                foreach (FlagIndexData flag in summary.Flags)
                                                {
                                                    scenario.FlagValues.Add(flag.FlagName + "=" +
                                                        flag.InitialValue.ToString());
                                                }
                                                foreach (StepIndexData step in summary.Steps)
                                                {
                                                    scenario.StepValues.Add(step.StepName + "=" +
                                                        step.InitialValue.ToString());
                                                }

                                                //＿番号のクーポン配布
                                                int i = 0;
                                                foreach (CharacterData chara in inn.CurrentParty.Characters)
                                                {
                                                    if (i > 5) break;
                                                    CouponData cp = new CouponData();
                                                    cp.Name = Util1.CouponEnumerateStr[i++];
                                                    cp.Point = 0;
                                                    chara.Coupons.Insert(0, cp);
                                                }

                                                // BGM登録
                                                scenario.BGMFile = FilePath.GetDefInnMidi();

                                                //先に読み込めるかテストする
                                                WidArea area = new WidArea();
                                                try
                                                {
                                                    area.Read(scenario.ScenarioFolder + "\\" + FilePath.FileArea(scenario.AreaNumber));
                                                }
                                                catch
                                                {
                                                    //読み込み付加の場合は宿に強制送還
                                                    //TODO
                                                    return;
                                                }

                   
                                                //背景を構成
                                                if (area.Cells[0].FileName != "")
                                                    scenario.Cells = area.Cells;//背景継承でない
                                                else
                                                    //背景継承
                                                    foreach (CellData cd in area.Cells)
                                                    {
                                                        scenario.Cells.Add(cd);
                                                    }
                                                Util2.UpdateCells(scenario);

                                                // シナリオデータをパーティに登録
                                                inn.CurrentParty.PlayingScenarioData = scenario;

                                                // カレント宿設定
                                                SetUpCurrentInn(inn);

                                                return;

                                            }
                                        }
                                    }
                                }
                            }
                            catch
                            {
                                // 読み込みに失敗表示
                                MessageDlg.ShowInfoDialog("シナリオの読み込みに失敗しました。");
                            }
                        }
                        else
                        {
                            // ただのフォルダである
                            MessageDlg.ShowInfoDialog("ただのフォルダです。");
                        }


                        // whileループエンド
                    }
                

                }
            }
        }

        /// <summary>
        /// シナリオ簡易検索ボタン押下
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSearch_Click(object sender, EventArgs e)
        {
            // 隠れたりするので
            // モーダルにもどします。
            using (ScenarioSearchDlg ssd = new ScenarioSearchDlg(this, inn, this))
            {
                ssd.ShowDialog();
            }


            ////if (search == null)
            ////{
            ////    search = new ScenarioSearchDlg(this);
            ////    search.FormClosed += new FormClosedEventHandler(Search_FormClosed);
            ////    search.Show();
            ////}
            ////else
            ////{
            ////    // すでにある。開いている。
            ////}
        }

        /// <summary>
        /// シナリオ簡易検索をモーダレス化した場合に閉じる時に呼ばれるメソッド
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Search_FormClosed(object sender, FormClosedEventArgs e)
        {
            //search = null;
        }

        #endregion

        #region パーティー操作

        /// <summary>
        /// パーティー選択
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartySelect_Click(object sender, EventArgs e)
        {
            using (TeamSelectDlg teamSelector = new TeamSelectDlg(inn))
            {

                if (inn != null)
                {

                    DialogResult res = teamSelector.ShowDialog();
                    if (res.Equals(DialogResult.OK))
                    {
                        // カレントパーティー表示
                        SetUpCurrentParty(inn.CurrentParty);

                        // カレント宿設定
                        SetUpCurrentInn(inn);
                    }

                }
            }
        }

        /// <summary>
        /// カレントパーティー変更左
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyLeft_Click(object sender, EventArgs e)
        {

            if (inn.Parties.Count == 0) return;
            int index = 0;

            if (inn.CurrentParty != null)
            {
                index = inn.GetPartyIndex(inn.CurrentParty);
                index--;
                if (index < 0) index = inn.Parties.Count - 1;
                
                // カレントパーティーセーブ
                inn.SaveCurrentParty();
            }

            // カレントパーティー変更
            inn.CurrentParty = inn.Parties[index];
            // カレントパーティーはファイルがあってはいけない
            inn.DeletePartyFile(inn.CurrentParty);

            // カレントパーティー表示
            SetUpCurrentParty(inn.CurrentParty);

            // カレント宿設定
            SetUpCurrentInn(inn);
            
        }

        /// <summary>
        /// カレントパーティー変更右
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyRight_Click(object sender, EventArgs e)
        {

            if (inn.Parties.Count == 0) return;

            int index = 0;
            if (inn.CurrentParty != null)
            {
                index = inn.GetPartyIndex(inn.CurrentParty);
                index++;

                if (index >= inn.Parties.Count) index = 0;

                // カレントパーティーセーブ
                inn.SaveCurrentParty();
            }

            // カレントパーティー変更
            inn.CurrentParty = inn.Parties[index];
            // カレントパーティーはファイルがあってはいけない
            inn.DeletePartyFile(inn.CurrentParty);

            // カレントパーティー表示
            SetUpCurrentParty(inn.CurrentParty);

            // カレント宿設定
            SetUpCurrentInn(inn);
            
        }

        /// <summary>
        /// パーティー解散
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyBreakUp_Click(object sender, EventArgs e)
        {

            //パーティを待機PCに戻す
            //パーティーアイテム・所持金を宿の金庫へ
            if (inn.CurrentParty != null)
            {

                // パーティを待機PCに戻す
                pnlParty.Controls.Clear();

                int count = inn.CurrentParty.Characters.Count;
                // パーティー解散（戻り値は待機PCになったCharacterData）
                inn.BreakParty();

                // 表示用カード追加・配置修正
                ModifyLastSomePC(count);

                // カレント宿設定
                SetUpCurrentInn(inn);
            }
        }

        /// <summary>
        /// 表示用カード追加・配置修正(最後に追加した人数だけ)
        /// </summary>
        /// <param name="count">人数</param>
        public void ModifyLastSomePC(int count)
        {
            int nowX = -pnlPc.AutoScrollPosition.X;
            int nowY = -pnlPc.AutoScrollPosition.Y;

            pnlPc.AutoScrollPosition = new Point(0, 0);

            for (int i = inn.Characters.Count - count; i < inn.Characters.Count; i++)
            {
                DispCard dc = new DispCard(inn.Characters[i]);
                dc.Location = new Point(CWData.DispCard_Width
                    * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                pnlPc.Controls.Add(dc);
            }

            pnlPc.AutoScrollPosition = new Point(nowX, nowY);

        }

        /// <summary>
        /// カレントPTではない全てのPTの解散
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAllBreakUp_Click(object sender, EventArgs e)
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "カレントパーティー以外のパーティーを解散します。\r\n時間がかかるかもしれません。\r\nよろしいですか？"))
            {
                if (inn.Parties.Count > 1 
                    || (inn.Parties.Count == 1 && inn.CurrentParty == null))
                {
                    int pcCount = inn.Characters.Count;
                    // 処理
                    inn.BreakAllParty(this.allPartyBreakToYadoCard);
                    // 表示反映

                    // 表示用カード追加・配置修正
                    int nowX = -pnlPc.AutoScrollPosition.X;
                    int nowY = -pnlPc.AutoScrollPosition.Y;

                    pnlPc.AutoScrollPosition = new Point(0, 0);

                    for (int i = pcCount; i < inn.Characters.Count; i++)
                    {
                        DispCard dc = new DispCard(inn.Characters[i]);
                        dc.Location = new Point(CWData.DispCard_Width
                            * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                        pnlPc.Controls.Add(dc);
                    }

                    pnlPc.AutoScrollPosition = new Point(nowX, nowY);

                    // カレント宿設定
                    SetUpCurrentInn(inn);

                    ////MessageDlg.ShowInfoDialog(
                    ////    "終了しました。");

                }
                else
                {
                    MessageDlg.ShowInfoDialog(
                        "対象のパーティーがありません。");
                }

            }
        }

        /// <summary>
        /// 一括パーティー機能ボタンクリック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPCtoParty_Click(object sender, EventArgs e)
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "全待機PCをパーティーにしてしまいます。\r\n時間がかかるかもしれません。\r\nよろしいですか？"))
            {
                if (inn.Characters.Count > 0)
                {

                    inn.CreatePartyFromAllPC(this.createFullPartyWithYadoCard);

                    pnlPc.Controls.Clear();

                    ////MessageDlg.ShowInfoDialog(
                    ////    "終了しました。");

                    this.grpYadoPc.Text = "待機PC(" + inn.Characters.Count + ")";
                }
                else
                {
                    MessageDlg.ShowInfoDialog(
                        "待機PCはいません。");
                }

            }
        }

        /// <summary>
        /// 新規パーティー作成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNewParty_Click(object sender, EventArgs e)
        {
            if (pnlPc.Controls.Count > 0)
            {
                using (PcDialog pcd = new PcDialog(this,inn,true)) 
                {
                    DialogResult res = pcd.ShowDialog();
                    if(res.Equals(DialogResult.OK))
                    {
                        if (inn.CurrentParty != null)
                        {
                            inn.SaveCurrentParty();
                            inn.CurrentParty = null;
                        }

                        // 新規パーティー作成
                        inn.CreateParty(pcd.SelectedDatas);

                        // メンバー移動表示反映
                        DisplayMemberMove(inn, pnlPc, pcd.SelectedIndexes);

                        // カレント宿設定
                        SetUpCurrentInn(inn);

                    }
                }

            }

        }

        /// <summary>
        /// パーティー所持金、パーティー名変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyMoney_Click(object sender, EventArgs e)
        {
            if (inn.CurrentParty != null)
            {
                using (PartyDataDlg pdd = new PartyDataDlg(inn))
                {
                    DialogResult res = pdd.ShowDialog();
                    if (res.Equals(DialogResult.OK))
                    {
                        // 宿所持金表示
                        lblYadoMoney.Text = "宿所持金：" + inn.YadoMoney.ToString();
                        // パーティー所持金表示
                        lblPartyMoney.Text = "パーティー所持金：" + inn.CurrentParty.Money.ToString();
                        // パーティー名表示
                        lblParty.Text = "パーティー名：" + inn.CurrentParty.PartyName;
                    }
                }
            }
        }

        /// <summary>
        /// カレントパーティー宿間移動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPartyMove_Click(object sender, EventArgs e)
        {
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "移動を終了した時に宿を保存しますが、\r\nよろしいですか？"))
            {

                // 保存してもいい場合
                // 簡易宿選択ダイアログ(元宿名付き)
                using (YadoSelectDlg ysd = new YadoSelectDlg(this,inn.Name))
                {
                    if (DialogResult.OK == ysd.ShowDialog())
                    {
                        // 宿名が指定されていた場合
                        if (!ysd.SelectedYado.Name.Equals(string.Empty))
                        {
                            // 他の宿にパーティー移動
                            inn.MoveCurrentPartyToOtherInn(ysd.SelectedYado);
                            // 表示反映
                            pnlParty.Controls.Clear();
                            // カレント宿設定
                            SetUpCurrentInn(inn);
                        }
                    }
                }

            }
        }
        
        /// <summary>
        /// F9（緊急避難）ボタン押下
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnF9_Click(object sender, EventArgs e)
        {
            if (inn.CurrentParty != null 
                && inn.CurrentParty.InScenario)
            {
                F9Command();
            }
        }

        /// <summary>
        /// F9(緊急避難)
        /// </summary>
        private void F9Command()
        {
            // 一応安全のためにシナリオ中かどうか確認した後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "緊急避難コマンドが発令されました。シナリオを強制\r\n終了します。よろしいですか？"))
            {
                // ゴシップや終了印はそのまま

                // 所持金は1.28ではシナリオ開始時点の状態に戻る
                inn.CurrentParty.Money = inn.CurrentParty.MoneyAtStart;

                // クーポンはシナリオ開始時点の状態に戻る
                // ・シナリオ中に付加される時限クーポンなどが取れている
                // ・~のついたクーポンは元に戻す

                // シナリオ中に手に入れたカードは消滅するが、使用したカードはそのまま
                // つまり　所持確定 = false の 
                // 荷物袋の中の RenamedCardData に対してチェック 
                // WidCardとWidCardに対応するCardIndexData と RenameCardData を削除
                IndexCardData icd;
                for (int i = inn.CurrentParty.BagCards.Count - 1; i >= 0; i--)
                {
                    icd = inn.FindIndexedCardData(inn.CurrentParty.BagCards[i]);
                    if (icd != null)
                    {
                        if (!icd.WidData.FixPossession)
                        {
                            // 所持確定していない

                            // CardIndexData WidDataを削除 ファイルも削除
                            inn.CardIndexDatas.Remove(icd.CardData);
                            inn.WidCards.Remove(icd.WidData);
                            //~宿フォルダからリネームした名前のwidCardを削除
                            try
                            {
                                File.Delete(FilePath.GetNameWithWid(inn.TempFolder, icd.CardData.RenamedName));
                            }
                            catch
                            {
                                PartyMaker.CWIErrorMessage("カードの削除に失敗。強制終了");
                            }

                            // RenamedCardData 削除
                            inn.CurrentParty.BagCards.RemoveAt(i);
                        }
                    }
                    else
                    {
                        // すでに CardIndexData、WidDataは削除済み
                        // RenamedCardData 削除
                        inn.CurrentParty.BagCards.RemoveAt(i);
                    }

                }

                // シナリオ中に消滅したキャストは、パーティの最後尾に復活
                CharacterData[] list = inn.CurrentParty.VanishCharacters.ToArray();
                inn.CurrentParty.Characters.AddRange(list);
                inn.CurrentParty.VanishCharacters.Clear();

                //冒険者処理
                foreach (CharacterData chara in inn.CurrentParty.Characters)
                {
                    // ~クーポン復帰
                    chara.ReturnTildeCoupons();

                    //番号クーポン削除
                    chara.DeleteNumberCoupons();

                    //時限クーポン（「：」「；」クーポン）削除
                    // リバースなども含む
                    chara.DeleteTempCoupons();

                    // シナリオ中に手に入れたカードは消滅するが、使用したカードはそのまま
                    // つまり　所持確定 = false の 
                    // HandCard を削除
                    chara.DeleteNonFixHandCards();

                    //冒険者状態異常クリアというより全快
                    chara.RefleshAll();
                }

                // 宿データの進行中シナリオリストからシナリオ名を削除
                inn.ProgressScenarios.Remove(inn.CurrentParty.Summary.ScenarioName);

                // CurrentParty　からPlayingScenarioDataをなくす(Summary もクリアされる)
                inn.CurrentParty.PlayingScenarioData = null;


                GC.Collect(2);

                // カレント宿設定
                SetUpCurrentInn(inn);
            }
        }

        /// <summary>
        /// パーティーメンバ一覧
        /// </summary>
        private void PartyMemberList()
        {
            using (BmpSelectDlg selector = new BmpSelectDlg(inn.Parties))
            {
                selector.ShowDialog();
            }
        
        }
        /// <summary>
        /// 冒険の中断イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAbort_Click(object sender, EventArgs e)
        {
            if (pnlParty.Controls.Count > 0)
            {
                if (inn.CurrentParty != null)
                {
                    if (inn.CurrentParty.InScenario)
                    {
                        F9Command();
                        return;
                    }

                    inn.SaveCurrentParty();
                    inn.CurrentParty = null;

                    // メンバー移動表示反映
                    pnlParty.Controls.Clear();

                    // カレント宿設定
                    SetUpCurrentInn(inn);
                }

            }
        }
        #endregion

        #region 待機PC関連

        /// <summary>
        /// PCの画像収集機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBmpGet_Click(object sender, EventArgs e)
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "全PCの画像をFaceフォルダに吸い出します。\r\n"
                 + "ファイル名の先頭にPM_をつけて保存します。\r\n"
                + "よろしいですか？"))
            {
                inn.PCBmpSave(true);

                MessageDlg.ShowInfoDialog("終了しました。");
            }
        }

        /// <summary>
        /// キャラクタクーポン移動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCharaCoupon_Click(object sender, EventArgs e)
        {
            List<CharacterData> list = new List<CharacterData>();
            list.AddRange(inn.Characters);
            int charaCount = inn.Characters.Count;
            int ptCharaCount = 0;
            if(inn.CurrentParty != null)
            {
                list.AddRange(inn.CurrentParty.Characters);
                ptCharaCount = inn.CurrentParty.Characters.Count;
            }
            int chara1 = -1;
            int chara2 = -1;
            DialogResult res = DialogResult.Cancel;
            using (BmpSelectDlg bmpDlg = new BmpSelectDlg(list))
            {
                bmpDlg.Text = "キャラクタ１選択";
                res = bmpDlg.ShowDialog();
                if (res != DialogResult.OK)
                {
                    return;
                }
                chara1 = bmpDlg.SelectedIndex;

                bmpDlg.Text = "キャラクタ２選択";
                res = bmpDlg.ShowDialog();
                if (res != DialogResult.OK)
                {
                    return;
                }
                chara2 = bmpDlg.SelectedIndex;

                if (chara1 == chara2)
                {
                    MessageDlg.ShowInfoDialog("同じキャラを選択出来ません。");
                    return;
                }
            }

            using (MoveCouponDlg couponDlg
                = new MoveCouponDlg(this
                                    ,list[chara1].CardName
                                    ,list[chara2].CardName
                                    ,list[chara1].Coupons
                                    ,list[chara2].Coupons))
            {
                ;

                if (couponDlg.ShowDialog() == DialogResult.OK)
                {
                    // ＠レベル原点などが書き換えられていたとしても
                    // 対応しません。

                    if (chara1 < charaCount)
                    {
                        // 待機ＰＣファイルに結果を反映
                        inn.AddCharacterFile(list[chara1]);
                    }

                    if (chara2 < charaCount)
                    {
                        // 待機ＰＣファイルに結果を反映
                        inn.AddCharacterFile(list[chara2]);
                    }
                }
            }
                                                              
        }

        /// <summary>
        /// カード狩り
        /// </summary>
        private void CardGet()
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "全待機PCのカードをカード置場に移動します。\r\n"
                + "よろしいですか？"))
            {
                inn.MoveAllPCCardToInnCard();
            }
        }
        #endregion

        #region アイテム処理

        private void btnMakeShelf_Click(object sender, EventArgs e)
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "カード置場の全カードを棚パーティーに格納します。\r\n時間がかかるかもしれません。\r\nよろしいですか？"))
            {
                if (inn.BankBoxCards.Count > 0)
                {
                    inn.MakeShelf();
                }
                else
                {
                    MessageDlg.ShowInfoDialog(
                        "カード置場にカードがありません。");
                }

            }
        }

        private void btnItemFromShelf_Click(object sender, EventArgs e)
        {
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "棚か箱一行のPTの箱からアイテムを取り出します。\r\n時間がかかるかもしれません。\r\nよろしいですか？"
            ))
            {
                if (inn != null 
                        && (inn.Parties.Count > 1 
                            || (inn.Parties.Count == 1 && inn.CurrentParty == null))
                    )
                {

                    int pcCount = inn.Characters.Count;
                    // 処理
                    inn.AllBoxPartyBreakUp();
                    // 表示反映

                    // 表示用カード追加・配置修正
                    int nowX = -pnlPc.AutoScrollPosition.X;
                    int nowY = -pnlPc.AutoScrollPosition.Y;

                    pnlPc.AutoScrollPosition = new Point(0, 0);

                    for (int i = pcCount; i < inn.Characters.Count; i++)
                    {
                        DispCard dc = new DispCard(inn.Characters[i]);
                        dc.Location = new Point(CWData.DispCard_Width
                            * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                        pnlPc.Controls.Add(dc);
                    }

                    pnlPc.AutoScrollPosition = new Point(nowX, nowY);

                    // カレント宿設定
                    SetUpCurrentInn(inn);


                }
                else
                {
                    MessageDlg.ShowInfoDialog(
                        "カレントパーティー以外の\r\nパーティーがありません。");
                }

            }
        }

        #endregion

        #region 表示と配置

        /// <summary>
        /// カレントパーティーPC設定
        /// </summary>
        /// <param name="list">キャラクタリスト</param>
        private void SetUpCurrentParty(PartyData currentParty)
        {

            pnlParty.Controls.Clear();

            this.pnlParty.SuspendLayout();

            if (currentParty != null)
            {

                index = 0;
                foreach (CharacterData cd in currentParty.Characters)
                {
                    DispCard dispPartyPc = new DispCard(cd);
                    dispPartyPc.Location = new Point(CWData.DispCard_Width * index++, 0);
                    pnlParty.Controls.Add(dispPartyPc);
                }

            }

            this.pnlParty.ResumeLayout();

        }

        /// <summary>
        /// 外部からピンポイントでキャラクタの状態変更をDispCardに反映
        /// </summary>
        /// <param name="index"></param>
        public void PinPointChangeParty(int index)
        {
            DispCard dc = pnlParty.Controls[index] as DispCard;
            if (dc != null)
            {
                dc.SetCd(inn.CurrentParty.Characters[index]);
            }
        }

        /// <summary>
        /// 外部からピンポイントでキャラクタの状態変更をDispCardに反映
        /// </summary>
        /// <param name="index"></param>
        public void PinPointChangeCharacter(int index)
        {
            DispCard dc = pnlPc.Controls[index] as DispCard;
            if (dc != null)
            {
                dc.SetCd(inn.Characters[index]);
            }
        }

        /// <summary>
        /// 外部からピンポイントでキャラクタの状態変更をDispCardに反映
        /// </summary>
        /// <param name="index"></param>
        public void PinPointAddCharacter(int count)
        {
            // 表示反映

            // 表示用カード追加・配置修正
            int nowX = -pnlPc.AutoScrollPosition.X;
            int nowY = -pnlPc.AutoScrollPosition.Y;

            pnlPc.AutoScrollPosition = new Point(0, 0);

            for (int i = inn.Characters.Count - count; i < inn.Characters.Count; i++)
            {
                DispCard dc = new DispCard(inn.Characters[i]);
                dc.Location = new Point(CWData.DispCard_Width
                    * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                pnlPc.Controls.Add(dc);
            }

            pnlPc.AutoScrollPosition = new Point(nowX, nowY);

        }

        /// <summary>
        /// カレント宿設定
        /// </summary>
        /// <param name="inn">宿データ</param>
        private void SetUpCurrentInn(InnData inn)
        {
            if (inn != null)
            {
                // 宿所持金表示
                lblYado.Text = "宿名：" + inn.Name;
                lblYadoMoney.Text = "宿所持金：" + inn.YadoMoney.ToString();

                if (inn.CurrentParty != null)
                {
                    // パーティー名の変更は１人目が変わったときだけにするように変更したい。
                    // 新規のときもパーティー名は他のところで付加されている。
                    // パーティー解散時はきちんとCurrentPartyをnullにしておく。

                    lblParty.Text = "パーティー名：" + inn.CurrentParty.PartyName;
                    lblPartyMoney.Text = "パーティー所持金：" + inn.CurrentParty.Money.ToString();


                    if (inn.CurrentParty.InScenario)
                    {
                        // カレントパーティーはシナリオ中
                        //BGMを変更
                        try
                        {
                            MultiMedia.PlayMusicFile(inn.CurrentParty.ScenarioBGMFile, this.soundMute);
                        }
                        catch (Exception)
                        {
                            MultiMedia.PlayMusicFile(FilePath.GetBeyonedMyHeartMidi(), this.soundMute);
                        }

                        SummaryData summary = inn.CurrentParty.Summary;

                        try
                        {
                            this.grpParty.Text = "カレントパーティー(シナリオ中：" + summary.ScenarioName + inn.CurrentParty.ScenarioLevelText + ")";
                        }
                        catch (Exception)
                        {
                            this.grpParty.Text = "カレントパーティー(シナリオ中）";
                        }
                        

                        btnPartyMoney.Enabled = false;
                        btnPartyBreakUp.Enabled = false;
                        //btnF9.Enabled = true;
                        //btnSenario.Enabled = false;
                    }
                    else
                    {
                        //宿でパーティ状態の再開

                        //BGMを変更
                        MultiMedia.PlayMusicFile(FilePath.GetDefInnMidi(), this.soundMute);
                        this.grpParty.Text = "カレントパーティー(宿)";
                        btnPartyMoney.Enabled = true;
                        btnPartyBreakUp.Enabled = true;
                        //btnF9.Enabled = false;
                        //btnSenario.Enabled = true;

                    }
                }
                else
                {
                    //宿でパーティなし状態の再開
                    ClearPartyLabel();
                    //BGMを変更
                    MultiMedia.PlayMusicFile(FilePath.GetDefInnMidi(), this.soundMute);
                    this.grpParty.Text = "カレントパーティー";
                    btnPartyMoney.Enabled = false;
                    btnPartyBreakUp.Enabled = false;
                    //btnF9.Enabled = false;
                    //btnSenario.Enabled = false;

                }

                this.grpYadoPc.Text = "待機PC(" + inn.Characters.Count + ")";
            }
            else
            {
                lblYado.Text = "宿名：";
                lblYadoMoney.Text = "宿所持金：";
                ClearPartyLabel();
                //BGMを変更
                MultiMedia.PlayMusicFile(FilePath.GetDefInnMidi(), this.soundMute);
                this.grpParty.Text = "カレントパーティー";
                this.grpYadoPc.Text = "待機PC(0)";

                lblYado.Text = "宿名：";
                lblYadoMoney.Text = "宿所持金：";
            }


        }

        /// <summary>
        /// パーティークリア
        /// </summary>
        private void ClearPartyLabel()
        {
            lblParty.Text = "パーティー名：";
            lblPartyMoney.Text = "パーティー所持金：";
        }

        /// <summary>
        /// パーティー名のみ設定
        /// （出来れば１人目が変わっていないときには呼ばないようにする。）
        /// </summary>
        private void SetUpPartyName()
        {
            if (inn.CurrentParty != null)
            {

                string partyName = inn.CurrentParty.Characters[0].CardName + FilePath.PartyInitialNameEnd;
                inn.CurrentParty.PartyName = partyName;

                lblParty.Text = "パーティー名：" + partyName;
            }
        }

        /// <summary>
        /// 宿待機PC設定
        /// </summary>
        /// <param name="list">キャラクタリスト</param>
        private void SetUpYadoPcData(List<CharacterData> innCharacters)
        {
            pnlPc.Controls.Clear();

            this.pnlPc.SuspendLayout();

            index = 0;
            int count = innCharacters.Count;

            int initValue = 0;
            int amongValue = 85;
            if (progress.Value == 50)
            {
                // 宿作成中
                initValue = 50;
                amongValue = 35;
            }

            foreach (CharacterData cd in innCharacters)
            {

                progress.Message = "待機PC：" + (index + 1) + "人目を表示しています。";

                //カードリストに追加、絵画
                DispCard dispPc = new DispCard(cd);
                dispPc.Location = new Point(CWData.DispCard_Width * (index % disp_num_x)
                    , CWData.DispCard_Height * (index / disp_num_x));
                index++;
                pnlPc.Controls.Add(dispPc);

                //プログレスバーの値を変更する
                progress.Value = initValue + index * amongValue / count;

            }

            this.pnlPc.ResumeLayout();

        }

        /// <summary>
        /// 待機PC再配置
        /// </summary>
        private void SetLocationPnlPc()
        {

            int nowX = -pnlPc.AutoScrollPosition.X;
            int nowY = -pnlPc.AutoScrollPosition.Y;

            pnlPc.AutoScrollPosition = new Point(0, 0);
            index = 0;
            foreach (Control ctrl in pnlPc.Controls)
            {
                ctrl.Location = new Point(CWData.DispCard_Width * (index % disp_num_x),
                    CWData.DispCard_Height * (index / disp_num_x));
                index++;
            }

            pnlPc.AutoScrollPosition = new Point(nowX, nowY);

        }

        /// <summary>
        /// カレントパーティー再配置
        /// </summary>
        private void SetLocationPnlParty()
        {

            index = 0;
            foreach (Control ctrl in pnlParty.Controls)
            {
                ctrl.Location = new Point(CWData.DispCard_Width * index++, 0);
            }

        }

        /// <summary>
        /// 移動量分だけスクロール
        /// </summary>
        /// <param name="delta"></param>
        private void ScrollPnlPc(int delta)
        {
            int nowX = -pnlPc.AutoScrollPosition.X;
            int nowY = -pnlPc.AutoScrollPosition.Y;
            nowY += delta;
            if (nowY < 0)
            {
                nowY = 0;
            }

            if (nowY > CWData.DispCard_Height * ((pnlPc.Controls.Count - 1) / disp_num_x + 1) )
            {
                nowY = CWData.DispCard_Height * ((pnlPc.Controls.Count - 1) / disp_num_x + 1);
            }

            pnlPc.AutoScrollPosition = new Point(nowX, nowY);
            
        }

        /// <summary>
        /// PCの単独移動を表示反映する
        /// </summary>
        private void DisplayPCMove(InnData inn,Panel oldPanel,int oldIndex)
        {

            int nowX = -pnlPc.AutoScrollPosition.X;
            int nowY = -pnlPc.AutoScrollPosition.Y;

            pnlPc.AutoScrollPosition = new Point(0, 0);

            DispCard addDc;
            int newIndex;
            if (oldPanel == this.pnlParty)
            {
                // パーティーから宿待機へ
                pnlParty.Controls.RemoveAt(oldIndex);
                for (int i = oldIndex; i < pnlParty.Controls.Count; i++)
                {
                    // パーティーの元の位置より後ろのPCだけ配置修正
                    pnlParty.Controls[i].Location = new Point(CWData.DispCard_Width * i, 0);
                }

                // すでにSwap処理でListの最後に追加されているので
                // それを元に表示用カードを作成
                newIndex = inn.Characters.Count - 1;
                addDc = new DispCard(inn.Characters[newIndex]);
                addDc.Location = new Point(CWData.DispCard_Width * (newIndex % disp_num_x), CWData.DispCard_Height * (newIndex / disp_num_x));
                pnlPc.Controls.Add(addDc);
            }
            else
            {
                // 宿待機からパーティーへ
                pnlPc.Controls.RemoveAt(oldIndex);
                for (int i = oldIndex; i < pnlPc.Controls.Count; i++)
                {
                    // 待機PCの元の位置より後ろのPCだけ配置修正
                    pnlPc.Controls[i].Location = 
                        new Point(CWData.DispCard_Width * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                }

                // すでにSwap処理でListの最後に追加されているので
                // それを元に表示用カードを作成
                newIndex = inn.CurrentParty.Characters.Count - 1;
                addDc = new DispCard(inn.CurrentParty.Characters[newIndex]);
                addDc.Location = new Point(CWData.DispCard_Width * newIndex, 0);
                pnlParty.Controls.Add(addDc);

            }

            this.grpYadoPc.Text = "待機PC(" + inn.Characters.Count + ")";

            pnlPc.AutoScrollPosition = new Point(nowX, nowY);
        }

        /// <summary>
        /// PCの単独パーティー挿入を表示反映する
        /// </summary>
        private void DisplayPCInsert(InnData inn ,int oldIndex,int insertPos)
        {

            int nowX = -pnlPc.AutoScrollPosition.X;
            int nowY = -pnlPc.AutoScrollPosition.Y;

            pnlPc.AutoScrollPosition = new Point(0, 0);

            DispCard addDc;

            // 宿待機からパーティーへ
            pnlPc.Controls.RemoveAt(oldIndex);
            for (int i = oldIndex; i < pnlPc.Controls.Count; i++)
            {
                // 待機PCの元の位置より後ろのPCだけ配置修正
                pnlPc.Controls[i].Location =
                    new Point(CWData.DispCard_Width * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
            }

            // すでにInsert処理でListの途中に追加されているので
            // それを元に表示用カードを作成

            // 最後にある表示用カードを挿入位置に持ってきて
            // その他を１つずつ下げる。
            List<Control> contList = new List<Control>();
            // 現在のパーティーの数より表示は１人少ない
            for (int i = pnlParty.Controls.Count - 1; i >= insertPos; i--)
            {
                Control temp = pnlParty.Controls[i];
                contList.Add(temp);

                temp.Location
                    = new Point(CWData.DispCard_Width * (i + 1), 0);

                // 一度外す
                pnlParty.Controls.RemoveAt(i);
            }

            // 追加
            addDc = new DispCard(inn.CurrentParty.Characters[insertPos]);
            addDc.Location
                = new Point(CWData.DispCard_Width * insertPos, 0);
            pnlParty.Controls.Add(addDc);

            contList.Reverse();
            foreach (Control ctrl in contList)
            {
                // 後ろに付ける
                pnlParty.Controls.Add(ctrl);
            }


            this.grpYadoPc.Text = "待機PC(" + inn.Characters.Count + ")";

            pnlPc.AutoScrollPosition = new Point(nowX, nowY);
        }

        /// <summary>
        /// パーティーPCの位置移動（入れ替えでない）を表示反映する
        /// </summary>
        /// <param name="inn"></param>
        /// <param name="oldIndex"></param>
        /// <param name="insertPos"></param>
        private void DisplayPartyMove(InnData inn, int oldIndex, int insertPos)
        {
            DispCard moveDc;

            // すでにMove処理でListの途中に移動されているので
            // それを元に表示用カードを作成

            // 元の表示用カードは外します。
            pnlParty.Controls.RemoveAt(oldIndex);

            // 最後にある表示用カードを挿入位置に持ってきて
            // その他を１つずつ下げる。
            List<Control> contList = new List<Control>();


            int newIndex = insertPos;

            if (insertPos > oldIndex)
            {
                newIndex--;
                // 挿入位置は元の位置より後
                // 元の位置から挿入位置ー１までは配置修正
                for (int i = oldIndex; i < newIndex; i++)
                {
                    pnlParty.Controls[i].Location =
                        new Point(CWData.DispCard_Width * (i % disp_num_x),
                        CWData.DispCard_Height * (i / disp_num_x));
                }

            }

            // 現在のパーティーの数より表示は１人少ない
            for (int i = pnlParty.Controls.Count - 1; i >= newIndex; i--)
            {
                Control temp = pnlParty.Controls[i];
                contList.Add(temp);

                temp.Location
                    = new Point(CWData.DispCard_Width * (i + 1), 0);

                // 一度外す
                pnlParty.Controls.RemoveAt(i);
            }

            // 追加
            moveDc = new DispCard(inn.CurrentParty.Characters[newIndex]);
            moveDc.Location
                = new Point(CWData.DispCard_Width * newIndex, 0);
            pnlParty.Controls.Add(moveDc);

            contList.Reverse();
            foreach (Control ctrl in contList)
            {
                // 後ろに付ける
                pnlParty.Controls.Add(ctrl);
            }
        }

        /// <summary>
        /// PCのメンバー移動を表示反映する(新規パーティー編成)
        /// </summary>
        private void DisplayMemberMove(InnData inn, Panel oldPanel, List<int> oldIndexList)
        {

            int nowX = -pnlPc.AutoScrollPosition.X;
            int nowY = -pnlPc.AutoScrollPosition.Y;

            if (oldPanel == this.pnlPc)
            {
                if (oldIndexList.Count > 0)
                {

                    pnlPc.AutoScrollPosition = new Point(0, 0);

                    // 宿待機からパーティーへ

                    // パーティーはそのまま反映してもらう
                    SetUpCurrentParty(inn.CurrentParty);
                                      
                    // インデックスをソート
                    oldIndexList.Sort();
                    // インデックスのうち最小
                    int minIndex = oldIndexList[0];
                    // 削除するときは逆順
                    oldIndexList.Reverse();
                    foreach (int i in oldIndexList)
                    {
                        pnlPc.Controls.RemoveAt(i);
                    }

                    for (int i = minIndex; i < pnlPc.Controls.Count; i++)
                    {
                        // 待機PCの元の位置より後ろのPCだけ配置修正
                        pnlPc.Controls[i].Location = new Point(CWData.DispCard_Width 
                            * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                    }


                    this.grpYadoPc.Text = "待機PC(" + inn.Characters.Count + ")";

                    pnlPc.AutoScrollPosition = new Point(nowX, nowY);
                
                }
            }

        }

        #endregion

        #region マウスイベント
        /// <summary>
        ///  PCパネル　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlPc_MouseDown(object sender, MouseEventArgs e)
        {

            mouseDownIndex = -1;
            mouseDownPanel = pnlPc;

            // 処理対象
            int indexOfX = (-pnlPc.AutoScrollPosition.X + e.X) / CWData.DispCard_Width;
            int indexOfY = (-pnlPc.AutoScrollPosition.Y + e.Y) / CWData.DispCard_Height;

            index = indexOfY * disp_num_x + indexOfX;

            if (indexOfX >= disp_num_x || index >= pnlPc.Controls.Count)
            {
                // 待機PCは disp_num_x 人１列
                return;
            }


            if (e.Button == MouseButtons.Left)
            {
                mouseDownIndex = index;
                mouseDownPoint = new Point(e.X, e.Y);

            }
            else if (e.Button == MouseButtons.Middle)
            {
                mouseDownIndex = index;
                // 直接パーティーに追加

                // 待機PCがパーティーに入る
                PcPartyIn(mouseDownIndex);
                
            }
        }

        private void PcPartyIn(int mouseDownIndex)
        {
            CharacterData cdLocal = inn.Characters[mouseDownIndex];
            if (cmbPartyMake.SelectedIndex == 0)
            {
                MouseDownPCToParty(mouseDownIndex);
            }
            else if (cmbPartyMake.SelectedIndex == 1)
            {
                uint lv = cdLocal.Level;
                MouseDownPCToParty(mouseDownIndex);

                List<int> lvCmp = new List<int>();
                foreach (CharacterData cd in inn.Characters)
                {
                    lvCmp.Add((int)(cd.Level - lv));
                }
                while (inn.Characters.Count > 0 && inn.CurrentParty.Characters.Count < 6)
                {
                    int ind = 0;
                    int min = 9999;
                    for (int i = 0; i < lvCmp.Count; i++)
                    {
                        int abs = lvCmp[i] < 0 ? -lvCmp[i] : lvCmp[i];
                        if (min > abs)
                        {
                            min = abs;
                            ind = i;
                        }
                    }
                    MouseDownPCToParty(ind);
                    lvCmp.RemoveAt(ind);
                }
            }
            else if (cmbPartyMake.SelectedIndex == 2)
            {

                string name = cdLocal.FileName;
                MouseDownPCToParty(mouseDownIndex);

                List<string> nameCmp = new List<string>();
                foreach (CharacterData cd in inn.Characters)
                {
                    nameCmp.Add(cd.FileName);
                }
                while (inn.Characters.Count > 0 && inn.CurrentParty.Characters.Count < 6)
                {
                    int ind = 0;
                    int max = 0;
                    for (int i = 0; i < nameCmp.Count; i++)
                    {
                        int count = 0;
                        for (int j = 0; j < nameCmp[i].Length && j < name.Length; j++)
                        {
                            if (nameCmp[i][j] == name[j])
                            {
                                count++;
                            }
                            else
                            {
                                break;
                            }
                        }
                        if (count > max)
                        {
                            max = count;
                            ind = i;
                        }
                    }
                    MouseDownPCToParty(ind);
                    nameCmp.RemoveAt(ind);
                }
            }
            else if (cmbPartyMake.SelectedIndex == 3)
            {
                List<CouponData> coupons = cdLocal.Coupons;
                MouseDownPCToParty(mouseDownIndex);

                List<List<CouponData>> coupCmp = new List<List<CouponData>>();
                foreach (CharacterData cd in inn.Characters)
                {
                    coupCmp.Add(cd.Coupons);
                }
                while (inn.Characters.Count > 0 && inn.CurrentParty.Characters.Count < 6)
                {
                    int ind = 0;
                    int max = 0;
                    for (int i = 0; i < coupCmp.Count; i++)
                    {
                        int abs = 0;
                        foreach (CouponData cd in coupCmp[i])
                        {
                            bool isEqual = false;
                            foreach (CouponData cdmaster in coupons)
                            {
                                if (cd.Name == cdmaster.Name)
                                {
                                    isEqual = true;
                                    break;
                                }
                            }
                            if (isEqual) abs++;
                        }
                        if (max < abs)
                        {
                            max = abs;
                            ind = i;
                        }
                    }
                    MouseDownPCToParty(ind);
                    coupCmp.RemoveAt(ind);
                }
            }
        }

        /// <summary>
        /// パーティーパネル　マウスダウンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlParty_MouseDown(object sender, MouseEventArgs e)
        {

            mouseDownIndex = -1;
            mouseDownPanel = pnlParty;

            // 処理対象
            int indexOfX = e.X / CWData.DispCard_Width;
            int indexOfY = e.Y / CWData.DispCard_Height;
            if (indexOfX >= pnlParty.Controls.Count || indexOfY != 0)
            {
                // カレントパーティーは６人まで
                return;
            }

            index = indexOfX;


            if (e.Button == MouseButtons.Left)
            {

                //マウスの左ボタンだけが押されている時のみドラッグできるようにする
                mouseDownIndex = index;
                mouseDownPoint = new Point(e.X, e.Y);


            }
            else if (e.Button == MouseButtons.Middle)
            {
                mouseDownIndex = index;
                // 直接パーティーに追加

                // パーティーから待機欄へ単独移動
                if (inn.CurrentParty != null && !inn.CurrentParty.InScenario)
                {
                    MouseDownPartyToPC(mouseDownIndex);
                }

            }

        }
        /// <summary>
        ///  PCパネル　マウスムーブイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlPc_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDownPoint != Point.Empty)
            {
                //ドラッグとしないマウスの移動範囲を取得する
                Rectangle moveRect = new Rectangle(
                    mouseDownPoint.X - SystemInformation.DragSize.Width / 2,
                    mouseDownPoint.Y - SystemInformation.DragSize.Height / 2,
                    SystemInformation.DragSize.Width,
                    SystemInformation.DragSize.Height);
                //ドラッグとする移動範囲を超えたか調べる
                if (!moveRect.Contains(e.X, e.Y))
                {

                    // 離脱ではなく削除へ変更
                    grpRemove.Text = "削除";

                    //ドラッグの準備
                    DispCard dcx = (DispCard)(pnlPc.Controls[mouseDownIndex]);

                    //ドラッグ&ドロップ処理を開始する
                    DragDropEffects dde =
                        dcx.DoDragDrop(dcx, DragDropEffects.All);


                    // ドラッグ&ドロップ処理を終了した時に離脱に戻す
                    grpRemove.Text = "離脱";

                    mouseDownPoint = Point.Empty;

                }
            }

        }
        /// <summary>
        ///  パーティーパネル　マウスムーブイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlParty_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouseDownPoint != Point.Empty)
            {
                //ドラッグとしないマウスの移動範囲を取得する
                Rectangle moveRect = new Rectangle(
                    mouseDownPoint.X - SystemInformation.DragSize.Width / 2,
                    mouseDownPoint.Y - SystemInformation.DragSize.Height / 2,
                    SystemInformation.DragSize.Width,
                    SystemInformation.DragSize.Height);
                //ドラッグとする移動範囲を超えたか調べる
                // ついでにCurrentPartyがInSenarioのときはDrag＆Drop不可
                if (!moveRect.Contains(e.X, e.Y) && (inn.CurrentParty == null || !inn.CurrentParty.InScenario))
                {

                    //ドラッグの準備
                    DispCard dcx = (DispCard)(pnlParty.Controls[mouseDownIndex]);

                    //ドラッグ&ドロップ処理を開始する
                    DragDropEffects dde =
                        dcx.DoDragDrop(dcx, DragDropEffects.All);


                    mouseDownPoint = Point.Empty;
                }
            }

        }
        /// <summary>
        /// パーティーパネル　マウスアップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlParty_MouseUp(object sender, MouseEventArgs e)
        {
            // マウスの右ボタンだけが押されている時にPCDialogで情報表示
            if (e.Button == MouseButtons.Right 
                && inn.CurrentParty != null)
            {

                // パーティーが少ない時にパーティー以外をクリックした場合
                if (inn.CurrentParty.Characters.Count <= index)
                {
                    index = 0;
                }

                // PCDialog
                Point backupPoint = new Point(Cursor.Position.X, Cursor.Position.Y);
                PcDialog pcDialog = new PcDialog(this, inn, index, false);
                pcDialog.ShowDialog();
                Cursor.Position = backupPoint;

            }
            
            mouseDownPoint = Point.Empty;
        }
        /// <summary>
        /// PCパネル　マウスアップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlPc_MouseUp(object sender, MouseEventArgs e)
        {
            // マウスの右ボタンだけが押されている時にPCDialogで情報表示
            if (e.Button == MouseButtons.Right 
                && inn.Characters.Count != 0)
            {
                // 待機PCが少ない時にPC以外をクリックした場合
                if (inn.Characters.Count <= index)
                {
                    index = 0;
                }

                // PCDialog
                // 直接パーティー作成できるようにしました。
                using (PcDialog pcDialog = new PcDialog(this, inn, index, true))
                {
                    DialogResult res = pcDialog.ShowDialog();

                    if (res == DialogResult.OK)
                    {
                        if (inn.CurrentParty != null)
                        {
                            inn.SaveCurrentParty();
                            inn.CurrentParty = null;
                        }

                        // 新規パーティー作成
                        inn.CreateParty(pcDialog.SelectedDatas);

                        // メンバー移動表示反映
                        DisplayMemberMove(inn, pnlPc, pcDialog.SelectedIndexes);

                        // カレント宿設定
                        SetUpCurrentInn(inn);
                    }
                    else
                    {
                        // キャンセルされた場合でも
                        // 宿名が指定されていた場合
                        if (!pcDialog.SelectYadoName.Equals(string.Empty))
                        {
                            // 他の宿にPC移動
                            inn.MovePCToOtherInn(pcDialog.SelectMoveCharacter, pcDialog.SelectYadoName);
                            // 表示反映

                            int nowX = -pnlPc.AutoScrollPosition.X;
                            int nowY = -pnlPc.AutoScrollPosition.Y;

                            pnlPc.AutoScrollPosition = new Point(0, 0);

                            // 待機PCの表示削除
                            int oldIndex = pcDialog.SelectedIndex;
                            pnlPc.Controls.RemoveAt(oldIndex);
                            for (int i = oldIndex; i < pnlPc.Controls.Count; i++)
                            {
                                // 待機PCの元の位置より後ろのPCだけ配置修正
                                pnlPc.Controls[i].Location = new Point(CWData.DispCard_Width * (i % disp_num_x), CWData.DispCard_Height * (i / disp_num_x));
                            }

                            pnlPc.AutoScrollPosition = new Point(nowX, nowY);
                        }
                    }

                }
                
            }

            mouseDownPoint = Point.Empty;
            
        }
        /// <summary>
        /// マウスホイールの移動量を取得
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PartyMaker_MouseWheel(object sender, MouseEventArgs e)
        {
            ScrollPnlPc(-e.Delta);
        }
        
        #endregion

        #region ドラッグドロップ関連

        /// <summary>
        /// パーティーパネル　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlParty_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //CurrentParty がInSenarioでないかを調べ、
            //そうであればドロップ効果をCopyにする
            if (e.Data.GetDataPresent(typeof(DispCard)) 
                && (inn.CurrentParty == null || !inn.CurrentParty.InScenario) )
                e.Effect = DragDropEffects.Copy;
            else
                //DispCard型でなければ受け入れない
                e.Effect = DragDropEffects.None;
        }

        /// <summary>
        /// PTパネル　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlParty_DragDrop(object sender, DragEventArgs e)
        {

            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象
                Point pointClient = pnlParty.PointToClient(new Point(e.X,e.Y));
                int indexOfX = (-pnlParty.AutoScrollPosition.X + pointClient.X) / CWData.DispCard_Width;
                int indexOfY = (-pnlParty.AutoScrollPosition.Y + pointClient.Y) / CWData.DispCard_Height;
                int posOfX = (-pnlParty.AutoScrollPosition.X + pointClient.X) % CWData.DispCard_Width;
                index = indexOfX;

                // パーティー欄にドロップ
                if (indexOfY >= 1 
                    || index >= pnlParty.Controls.Count)
                {
                    // 対象はパーティー
                    if (mouseDownPanel == pnlPc)
                    {
                        // 待機PCがパーティーに入る
                        PcPartyIn(mouseDownIndex);
                        return;
                    }
                    else
                    {
                        // 同じパネル内なので何もしない
                        return;
                    }

                }
                else
                {
                    // 対象はパーティーPCかその間や前後
                    if (mouseDownPanel == pnlPc)
                    {
                        // 待機PCが来た
                        if (posOfX >= CWData.Among_Space
                            && posOfX < (CWData.DispCard_Width - CWData.Among_Space))
                        {

                            // 対象はパーティーPC

                            //待機PCとパーティーPCの入れ替え
                            inn.SwapPartyPCAndPC(index, mouseDownIndex);

                            // 表示上のキャラクタ変更
                            ((DispCard)pnlParty.Controls[index]).SetCd(inn.CurrentParty.Characters[index]);
                            ((DispCard)pnlPc.Controls[mouseDownIndex]).SetCd(inn.Characters[mouseDownIndex]);

                            // パーティー名変更
                            if (index == 0
                                && noChangePTName == false)
                            {
                                // 入れ替えたパーティーPCが1人目で
                                // NoChangeフラグが立っていないならば
                                SetUpPartyName();
                            }

                        }
                        else
                        {
                            // 対象はパーティーPC間

                            int insertPos = 0;
                            if (posOfX < CWData.Among_Space)
                            {
                                // カレントパーティーPCの左端
                                insertPos = index;
                            }
                            else
                            {
                                // カレントパーティーPCの右端
                                insertPos = index + 1;
                            }

                            if (insertPos == inn.CurrentParty.Characters.Count)
                            {
                                // 待機PCがパーティーの最後に入る
                                MouseDownPCToParty(mouseDownIndex);
                                return;
                            }

                            if (pnlParty.Controls.Count >= MaxMemberCount)
                            {
                                // 定員オーバーなので何もしない
                                return;
                            }

                            //InsertPCtoParty
                            inn.SwapPartyPCAndPC(-(insertPos + 2), mouseDownIndex);

                            // 表示上のキャラクタ変更
                            DisplayPCInsert(inn, mouseDownIndex, insertPos);

                            // パーティー名変更
                            if (insertPos == 0
                                && noChangePTName == false)
                            {
                                // 待機PCがパーティー1人目になり
                                // NoChangeフラグが立っていないならば
                                SetUpPartyName();
                            }
                        }

                    }
                    else
                    {
                        // パーティーPCが移動
                        if (posOfX >= CWData.Among_Space
                            && posOfX < (CWData.DispCard_Width - CWData.Among_Space))
                        {

                            // パーティーPCどうしの入れ替え
                            inn.SwapPartyCharacter(mouseDownIndex, index);

                            // 表示上のキャラクタ変更
                            ((DispCard)pnlParty.Controls[mouseDownIndex]).SetCd(inn.CurrentParty.Characters[mouseDownIndex]);
                            ((DispCard)pnlParty.Controls[index]).SetCd(inn.CurrentParty.Characters[index]);

                            // パーティー名変更  
                            if ((index == 0 || mouseDownIndex == 0)
                                && noChangePTName == false)
                            {
                                // 入れ替えたパーティーPCのどちらかが1人目で
                                // NoChangeフラグが立っていないならば

                                SetUpPartyName();
                            }
                        }
                        else
                        {

                            int insertPos = 0;
                            if (posOfX < CWData.Among_Space)
                            {
                                // カレントパーティーPCの左端
                                insertPos = index;
                            }
                            else
                            {
                                // カレントパーティーPCの右端
                                insertPos = index + 1;
                            }

                            if (insertPos == mouseDownIndex ||
                                insertPos == (mouseDownIndex + 1))
                            {
                                // 挿入位置が元の位置と同じか１つ多い
                                return;
                            }

                            inn.MovePartyCharacter(mouseDownIndex, insertPos);

                            // 表示反映
                            DisplayPartyMove(inn, mouseDownIndex, insertPos);

                            // パーティー名変更  
                            if ((insertPos == 0 || mouseDownIndex == 0)
                                && noChangePTName == false)
                            {
                                // 入れ替えたパーティーPCのどちらかが1人目で
                                // NoChangeフラグが立っていないならば

                                SetUpPartyName();
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// PC to Party 単独で最後尾に移動　共通処理
        /// </summary>
        /// <param name="mouseDownIndex"></param>
        private void MouseDownPCToParty(int mouseDownIndex)
        {
            // 待機PCがパーティーに入る
            if (pnlParty.Controls.Count >= MaxMemberCount)
            {
                // 定員オーバーなので何もしない
                return;
            }
            else
            {

                if (pnlParty.Controls.Count == 0)
                {
                    // 実は新規パーティー作成処理
                    if (inn.CurrentParty == null)
                    {
                        CharacterData cdLocal = inn.Characters[mouseDownIndex];

                        // 新規パーティー追加
                        List<CharacterData> list = new List<CharacterData>();
                        list.Add(cdLocal);
                        inn.CreateParty(list);

                        // PC表示反映
                        DisplayPCMove(inn, pnlPc, mouseDownIndex);

                        // カレント宿設定
                        SetUpCurrentInn(inn);

                    }

                }
                else
                {
                    // 待機PCを既存のパーティーに追加
                    inn.SwapPartyPCAndPC(-1, mouseDownIndex);

                    // PC表示反映
                    DisplayPCMove(inn, pnlPc, mouseDownIndex);

                    // パーティー名変更の必要なし
                }
            }
        }

        /// <summary>
        /// PCパネル　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlPc_DragEnter(object sender, DragEventArgs e)
        {

            //ドラッグされているデータがDispCard型か調べ、
            //そうであればドロップ効果をMoveにする
            if (e.Data.GetDataPresent(typeof(DispCard)))
                e.Effect = DragDropEffects.Copy;
            else
                //string型でなければ受け入れない
                e.Effect = DragDropEffects.None;
        }

        /// <summary>
        /// PCパネル　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlPc_DragDrop(object sender, DragEventArgs e)
        {
            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                // 処理対象
                Point pointClient = pnlPc.PointToClient(new Point(e.X, e.Y));
                int indexOfX = (-pnlPc.AutoScrollPosition.X + pointClient.X) / CWData.DispCard_Width;
                int indexOfY = (-pnlPc.AutoScrollPosition.Y + pointClient.Y) / CWData.DispCard_Height;
                index = indexOfY * disp_num_x + indexOfX;


                // PC欄にドロップ
                if (indexOfX >= disp_num_x || index >= pnlPc.Controls.Count)
                {

                    // 対象は待機欄
                    if (pnlPc != mouseDownPanel)
                    {
                        // パーティーから待機欄へ単独移動
                        MouseDownPartyToPC(mouseDownIndex);
                    }
                }
                else
                {
                    // 対象は待機PC
                    if (pnlPc != mouseDownPanel)
                    {
                        //パーティーPCと待機PCの入れ替え
                        inn.SwapPartyPCAndPC(mouseDownIndex, index);

                        // 表示上のキャラクタ変更
                        ((DispCard)pnlParty.Controls[mouseDownIndex]).SetCd(inn.CurrentParty.Characters[mouseDownIndex]);
                        ((DispCard)pnlPc.Controls[index]).SetCd(inn.Characters[index]);

                        // パーティー名変更
                        if (mouseDownIndex == 0 && noChangePTName == false)
                        {
                            // 入れ替えたパーティーPCが1人目で
                            // パーティー名変更なしが立っていない場合
                            SetUpPartyName();
                        }

                    }
                    else
                    {
                        // 待機PCどうしなのでファイルに変化なし
                        inn.SwapCharacter(mouseDownIndex, index);

                        // 表示上のキャラクタ変更
                        ((DispCard)pnlPc.Controls[mouseDownIndex]).SetCd(inn.Characters[mouseDownIndex]);
                        ((DispCard)pnlPc.Controls[index]).SetCd(inn.Characters[index]);

                    }

                }

            }
        }

        /// <summary>
        /// Party to PC 単独移動　共通処理
        /// </summary>
        /// <param name="mouseDownIndex"></param>
        private void MouseDownPartyToPC(int mouseDownIndex)
        {
            // パーティーPCが1人離脱
            if (pnlParty.Controls.Count == 1)
            {
                // 実はパーティー解散処理
                // パーティー解散
                inn.BreakParty();

                // パーティー欄クリア
                pnlParty.Controls.Clear();

                // 表示用カード追加
                DispCard dc = new DispCard(inn.Characters[inn.Characters.Count - 1]);
                pnlPc.Controls.Add(dc);

                // 宿帳PC配置修正
                SetLocationPnlPc();

                // カレント宿設定
                SetUpCurrentInn(inn);
                return;
            }
            else
            {
                // パーティーPCが離脱して宿待機へ
                inn.SwapPartyPCAndPC(mouseDownIndex, -1);

                // PC表示反映
                DisplayPCMove(inn, pnlParty, mouseDownIndex);

                // パーティー名変更
                if (mouseDownIndex == 0 && noChangePTName == false)
                {
                    // 離脱したパーティーPCが1人目で
                    // パーティー名変更なしが立っていない場合
                    SetUpPartyName();
                }
            }
        }

        /// <summary>
        /// パーティー離脱パネル　ドラッグエンターイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlRemove_DragEnter(object sender, DragEventArgs e)
        {
            //ドラッグされているデータがDispCard型か調べ、
            //そうであればドロップ効果をMoveにする
            if (e.Data.GetDataPresent(typeof(DispCard)) && pnlParty == mouseDownPanel)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else if (e.Data.GetDataPresent(typeof(DispCard)) && pnlPc == mouseDownPanel)
            {
                e.Effect = DragDropEffects.Move;
            }
            else
            {
                //DispCard型でなければ受け入れない
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// パーティー離脱パネル　ドラッグドロップイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlRemove_DragDrop(object sender, DragEventArgs e)
        {
            //ドロップされたデータがDispCard型か調べる
            if (e.Data.GetDataPresent(typeof(DispCard)))
            {
                if (pnlParty == mouseDownPanel)
                {
                    // ドロップしたのがパーティーPCの場合
                    // パーティーPCが1人離脱
                    if (pnlParty.Controls.Count == 1)
                    {
                        // 実はパーティー解散処理
                        // パーティー解散
                        inn.BreakParty();

                        // パーティー欄クリア
                        pnlParty.Controls.Clear();

                        // 表示用カード追加
                        DispCard dc = new DispCard(inn.Characters[inn.Characters.Count - 1]);
                        pnlPc.Controls.Add(dc);

                        // 宿帳PC配置修正
                        SetLocationPnlPc();

                        // カレント宿設定
                        SetUpCurrentInn(inn);
                    }
                    else
                    {
                        // パーティーPCが離脱して宿待機へ
                        inn.SwapPartyPCAndPC(mouseDownIndex, -1);

                        // PC表示反映
                        DisplayPCMove(inn, pnlParty, mouseDownIndex);

                        // パーティー名変更
                        if (mouseDownIndex == 0)
                        {
                            SetUpPartyName();
                        }
                    }
                }
                else if (pnlPc == mouseDownPanel)
                {
                    // 確認ダイアログ表示

                    if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
                    inn.Characters[mouseDownIndex].CardName + "を削除します。よろしいですか？"
                    ))
                    {
                        // 削除してもいい場合

                        // 待機PC削除
                        inn.DeleteCharacter(mouseDownIndex);
                        // 表示反映
                        int nowX = -pnlPc.AutoScrollPosition.X;
                        int nowY = -pnlPc.AutoScrollPosition.Y;

                        pnlPc.AutoScrollPosition = new Point(0, 0);

                        pnlPc.Controls.RemoveAt(mouseDownIndex);
                        for (int i = mouseDownIndex; i < pnlPc.Controls.Count; i++)
                        {
                            // 待機PCの元の位置より後ろのPCだけ配置修正
                            pnlPc.Controls[i].Location =
                                new Point(CWData.DispCard_Width * (i % disp_num_x)
                                , CWData.DispCard_Height * (i / disp_num_x));
                        }

                        // カレント宿設定
                        SetUpCurrentInn(inn);

                        pnlPc.AutoScrollPosition = new Point(nowX, nowY);

                    }


                }
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlParty_GiveFeedback(object sender, GiveFeedbackEventArgs e)
        {
            //////既定のカーソルを使用しない
            ////e.UseDefaultCursors = false;
            //////ドロップ効果にあわせてカーソルを指定する
            ////if ((e.Effect & DragDropEffects.Move) ==
            ////        DragDropEffects.Move)
            ////    Cursor.Current = moveCursor;
            ////else if ((e.Effect & DragDropEffects.Copy) ==
            ////        DragDropEffects.Copy)
            ////    Cursor.Current = copyCursor;
            ////else if ((e.Effect & DragDropEffects.Link) ==
            ////        DragDropEffects.Link)
            ////    Cursor.Current = linkCursor;
            ////else
            ////    Cursor.Current = noneCursor;

        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlParty_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;

        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlPc_GiveFeedback(object sender, GiveFeedbackEventArgs e)
        {
        ////    //既定のカーソルを使用しない
        ////    e.UseDefaultCursors = false;
        ////    //ドロップ効果にあわせてカーソルを指定する
        ////    if ((e.Effect & DragDropEffects.Move) ==
        ////            DragDropEffects.Move)
        ////        Cursor.Current = moveCursor;
        ////    else if ((e.Effect & DragDropEffects.Copy) ==
        ////            DragDropEffects.Copy)
        ////        Cursor.Current = copyCursor;
        ////    else if ((e.Effect & DragDropEffects.Link) ==
        ////            DragDropEffects.Link)
        ////        Cursor.Current = linkCursor;
        ////    else
        ////        Cursor.Current = noneCursor;

        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pnlPc_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            //マウスの右ボタンが押されていればドラッグをキャンセル
            //"2"はマウスの右ボタンを表す
            if ((e.KeyState & 2) == 2)
                e.Action = DragAction.Cancel;

        }

        #endregion

        #region ユーティリティっぽいの

        #region CW起動
        /// <summary>
        /// CW起動
        /// </summary>
        private void CWExec()
        {
            string yadoName = string.Empty;
            if (inn != null)
            {
                // ~宿削除
                yadoName = inn.Name;
                inn.DeleteTempFolder();
                // ダメ押しの全~宿削除
                DeleteAllTempFolder();
            }

            //CW起動前には音楽を止めておく
            MultiMedia.SoundStop();

            //CW起動前には打ち落としを止めておく
            timer1.Enabled = false;

            ////int dmWidth = 640;
            ////int dmHeight = 480;
            ////bool fullScreen = false;
            //////KeyHook keyhook = null;
            ////if (this.endBeforeExecVariant == false && this.fullScreenWhenExecVariant)
            ////{
            ////    // PM起動のままCW起動の場合

            ////    // フルスクリーンモードで起動の場合
            ////    DEVMODE dm = new DEVMODE();
            ////    if (0 == EnumDisplaySettings(null, ENUM_CURRENT_SETTINGS, ref dm))
            ////    {
            ////        MessageDlg.ShowInfoDialog("擬似フルスクリーンに出来ませんでした。");
            ////    }
            ////    else
            ////    {
            ////        // 現在の解像度保持
            ////        dmWidth = dm.dmPelsWidth;
            ////        dmHeight = dm.dmPelsHeight;
            ////        // 擬似フルスクリーン
            ////        dm.dmPelsWidth = 640;
            ////        dm.dmPelsHeight = 480;
            ////        dm.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT;
            ////        if (ChangeDisplaySettings(ref dm, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
            ////        {
            ////            MessageDlg.ShowInfoDialog("擬似フルスクリーンに出来ませんでした。");
            ////        }
            ////        else
            ////        {
            ////            fullScreen = true;

            ////            // キーフック設定
            ////            //効いていないのでコメントアウト
            ////            //keyhook = new KeyHook();
            ////            //keyhook.Set();
            ////            //keyhook.KeyDown += new KeyEvent(Hook_KeyPress);

            ////            // タスクバー
            ////            Int32 hWnd = FindWindow("Shell_TrayWnd", null);
            ////            if (hWnd != 0)
            ////            {
            ////                Int32 hWndChild;
            ////                //システムトレイ
            ////                hWndChild = FindWindowEx(hWnd, IntPtr.Zero, "TrayNotifyWnd", null);
            ////                if (hWndChild != 0)
            ////                {
            ////                    ShowWindow(hWndChild, SW_HIDE);//非表示
            ////                }

            ////                //スタートボタン
            ////                hWndChild = FindWindowEx(hWnd, IntPtr.Zero, "Button", null);
            ////                if (hWndChild != 0)
            ////                {
            ////                    ShowWindow(hWndChild, SW_HIDE);//非表示
            ////                }
            ////                else
            ////                {
            ////                    // すでにChildですらないのだが
            ////                    hWndChild = FindWindowEx(0, IntPtr.Zero, "Button", "スタート");
            ////                    if (hWndChild != 0)
            ////                    {
            ////                        ShowWindow(hWndChild, SW_HIDE);//非表示
            ////                    }
            ////                }

            ////                ShowWindow(hWnd, SW_HIDE);//非表示

            ////            }
            ////        }
            ////    }

            ////}

            Process ps = null;
            if (File.Exists(this.execVariant))
            {
                ps = Process.Start(this.execVariant);
            }
            else if (File.Exists(FilePath.VariantExe))
            {
                ps = Process.Start(FilePath.VariantExe);
            }
            else
            {
                PartyMaker.CWIErrorMessage("CW起動時にエラー");
            }

            if (this.endBeforeExecVariant)
            {
                // 終了
                closingDelete = true;
                this.Close();
            }
            else
            {
                // 今まではここでクローズしていたが
                // そのまま続行！

                // 位置調整に失敗したのでコメントアウト。直してうまくいったらいいな
                //SetWindowPos(ps.Handle, HWND_TOPMOST, 0, 0, 640, 480, SWP_SHOWWINDOW);

                ps.WaitForExit();
                ps.Close();
                ps.Dispose();

                ////if (fullScreen)
                ////{
                ////    // キーフック解除
                ////    ////効いていないのでコメントアウト
                ////    //keyhook.UnSet();
                ////    // 擬似フルスクリーンを元に戻す
                ////    DEVMODE dm = new DEVMODE();
                ////    if (0 == EnumDisplaySettings(null, ENUM_CURRENT_SETTINGS, ref dm))
                ////    {
                ////        MessageDlg.ShowInfoDialog("擬似フルスクリーンを元に戻せませんでした。");
                ////    }
                ////    else
                ////    {
                ////        // 元の解像度に戻す
                ////        dm.dmPelsWidth = dmWidth;
                ////        dm.dmPelsHeight = dmHeight;
                ////        if (ChangeDisplaySettings(IntPtr.Zero, 0) != DISP_CHANGE_SUCCESSFUL)
                ////        {
                ////            MessageDlg.ShowInfoDialog("擬似フルスクリーンを元に戻せませんでした。");
                ////        }
                ////        else
                ////        {
                ////            // タスクバー
                ////            Int32 hWnd = FindWindow("Shell_TrayWnd", null);
                ////            if (hWnd != 0)
                ////            {
                ////                ShowWindow(hWnd, SW_SHOW);//表示

                ////                Int32 hWndChild;
                ////                //システムトレイ
                ////                hWndChild = FindWindowEx(hWnd, IntPtr.Zero, "TrayNotifyWnd", null);
                ////                if (hWndChild != 0)
                ////                {
                ////                    ShowWindow(hWndChild, SW_SHOW);//表示
                ////                }

                ////                //スタートボタン
                ////                hWndChild = FindWindowEx(hWnd, IntPtr.Zero, "Button", null);
                ////                if (hWndChild != 0)
                ////                {
                ////                    ShowWindow(hWndChild, SW_SHOW);//表示
                ////                }
                ////                else
                ////                {
                ////                    // すでにChildですらないのだが
                ////                    hWndChild = FindWindowEx(0, IntPtr.Zero, "Button", "スタート");
                ////                    if (hWndChild != 0)
                ////                    {
                ////                        ShowWindow(hWndChild, SW_SHOW);//表示
                ////                    }
                ////                }
                ////            }

                ////        }
                ////    }

                ////    // 中央表示
                ////    SetWindowCenter();
                ////}


                // CW起動時に選択中の宿があったか
                if (string.IsNullOrEmpty(yadoName) == false)
                {
                    if (File.Exists(FilePath.GetYadoEnvironmentFile(FilePath.GetSystemYadoFolder() + "\\" + yadoName)))
                    {
                        // 選択中の宿があった場合は復活
                        inn = new InnData(yadoName);
                        LoadNewInn();

                        //打ち落とし再開
                        timer1.Enabled = true;
                    }
                    else
                    {
                        // 選択中の宿がなくなっている場合
                        PartyMaker.CWIErrorMessage("PM再起動時:宿がなくなっています。");
                    }
                }
                else
                {
                    // 選択中の宿が無い場合というのは考えがたい
                    PartyMaker.CWIErrorMessage("PM再起動時:宿がなくなっています。");
                }
            }
        }

        #region コメントアウト
        /////// <summary>
        /////// フックが検出したKeyPress
        /////// </summary>
        /////// <param name="key"></param>
        ////private bool Hook_KeyPress(Keys key)
        ////{

        ////    if (
        ////        ((key & Keys.Control) == Keys.Control && (key & Keys.KeyCode) == Keys.Escape)
        ////       ||
        ////        ((key & Keys.Alt) == Keys.Alt && (key & Keys.KeyCode) == Keys.Tab)
        ////       ||
        ////        ((key & Keys.KeyCode) == Keys.LWin)
        ////       ||
        ////        ((key & Keys.KeyCode) == Keys.RWin)
        ////       )
        ////    {
        ////        // WindowsキーとALT+TABとCtrl+ESCを他に回さないようにする。
        ////        return true;
        ////    }
        ////    else
        ////    {
        ////        return false;
        ////    }
        ////}
        #endregion
        #endregion

        #region CWIエラーメッセージ
        /// <summary>
        /// .NETのメッセージボックスを出して強制終了する
        /// </summary>
        /// <param name="mess">表示するメッセージ</param>
        public static void CWIErrorMessage(string mess)
        {
            System.Windows.Forms.MessageBox.Show(mess);

            System.Environment.Exit(1);
        }
        #endregion

        #region ~宿削除
        public static void DeleteAllTempFolder()
        {
            foreach (string folder 
                in Directory.GetDirectories(FilePath.GetSystemYadoFolder()))
            {
                string[] split = folder.Split('\\');
                string strDir = split[split.Length - 1];
                if (strDir.Length != 0
                    && strDir.StartsWith("~"))
                {
                    FilePath.DeleteFolder(folder);
                }
            }
        }
        #endregion

        #region 拡張機能ボタン

        /// <summary>
        /// 拡張機能実行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExtendExec_Click(object sender, EventArgs e)
        {
            switch (this.cmbExtend.SelectedIndex)
            {
                case 0:
                    using (YadoExtendDlg yadoExt = new YadoExtendDlg(this, inn))
                    {
                        yadoExt.ShowDialog();
                    }
                    break;
                case 1:
                    btnExtend_Click(sender, e);
                    break;
                case 2:
                    btnExtend2_Click(sender, e);
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// 拡張ボタン押下
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExtend_Click(object sender, EventArgs e)
        {
            using (FunctionDlg func = new FunctionDlg(this, inn))
            {
                func.ShowDialog();
                
                //チェック反映
                SaveInitXml();

                switch (func.SelectedButtonID)
                {
                    case 4:
                        btnF9_Click(sender, e);
                        break;
                    case 5:
                        btnBmpGet_Click(sender, e);
                        break;
                    case 6:
                        btnSearch_Click(sender, e);
                        break;
                    case 7:
                        btnAllBreakUp_Click(sender, e);
                        break;
                    case 8:
                        btnNewParty_Click(sender, e);
                        break;
                    case 9:
                        btnMakeShelf_Click(sender, e);
                        break;
                    case 10:
                        btnItemFromShelf_Click(sender, e);
                        break;
                    case 11:
                        PCSort(func.SelectedSortType);
                        break;
                    case 13:
                        btnMusic_Click(sender, e);
                        break;
                    case 14:
                        btnBGM_Click(sender, e);
                        break;
                    case 15:
                        selectedFont = func.SelectFont;
                        btnFontVariant_Click(sender, e);
                        break;
                    case 16:
                        selectedMoveTo = func.SelectedMoveTo;
                        btnIkinari_Click(sender, e);
                        break;
                    case 17:
                        IkinariScenario(func.SelectedFolder);
                        break;
                    default:
                        break;
                }

            }
        }

        /// <summary>
        /// 拡張２ボタン押下
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExtend2_Click(object sender, EventArgs e)
        {

            using (FunctionDlg2 func = new FunctionDlg2(this, inn))
            {
                func.ShowDialog();
                switch (func.SelectedButtonID)
                {
                    case 1:
                        this.btnPCtoParty_Click(sender, e);
                        break;
                    case 2:
                        this.btnCharaCoupon_Click(sender, e);
                        break;
                    case 3:
                        this.PartyDataShow();
                        break;
                    case 4:
                        btnCard_Click(sender, e);
                        break;
                    case 5:
                        btnMove_Click(sender, e);
                        break;
                    case 6:
                        btnPartyMove_Click(sender, e);
                        break;
                    case 7:
                        PartyMemberList();
                        break;
                    case 8:
                        CardGet();
                        break;
                }

                //チェック反映
                SaveInitXml();
            }
        }

        
        #endregion

        #region いきなり機能（シナリオ内の安全地帯からのスタート）

        #region いきなり機能　メイン
        /// <summary>
        /// いきなり機能（シナリオ内の安全地帯からのスタート）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnIkinari_Click(object sender, EventArgs e)
        {
            // 1.ゲーム宿であること OK
            // 2.カレントパーティーは宿にいること OK
            // 3.行き先に終了印や進行中がないこと 
            // 4.1-3を満たした上でシナリオキャッシュから行き先が
            //　見つかること
            // 5.そのフォルダがまだあること、サマリファイルもきちんと
            //　読み込めることを確認
            // 6.誰がどういうゴシップやクーポンやLV条件やキャラ条件や
            //　アイテム条件やその他もろもろがあっても変わらない
            //　セーブ状態（シナリオのエリアやセルの状態も落ち着いている）
            //　を再現する。
            // 7.宿の設定なども進行中シナリオリストに加えてその上で
            //　セーブし、起動したらオートロードONのときはオートロード
            //　するようにする。
            // 8.のちにはCardWirth終了時に戻ってこれるようにしたい。
            //　起動前に打ち落とし対象から外し、宿のファイルや
            //  フォルダをつかんでないようにして、Process#WaitForExit()して
            //  戻ってきたら宿再読み込みの予定
            
            //3

            // 当然カレントパーティーがいることが前提
            PartyData party = inn.CurrentParty;

            // _H_ は - に置き換えます。
            // _E_ は ! に置き換えます。
            // _K_は（、_T_は）に置き換えます。
            // _S_は#に置き換えます。
            // _M_は。に置き換えます。
            // _C_は　、_G_は―
            string scenaName = selectedMoveTo.ToString()
                    .Replace("_E_", "!").Replace("_H_", "-").Replace("_C_","　")
                    .Replace("_K_", "（").Replace("_T_", "）").Replace("_G_", "―")
                    .Replace("_S_", "#").Replace("_M_", "。").Replace("_B_", "\"")
                    .Replace("_L_", "「").Replace("_N_", "」").Replace("_A_", "＆").Replace("_Q_", "？")
                    .Replace("_F_", "！").Replace("_Z_", "=").Replace("_Y_", " ").Replace("_D_", "・")
                    .Replace("_I_", "5");
            if (inn.FinishScenarios
                .Contains(scenaName))
            {
                MessageDlg.ShowInfoDialog(scenaName + " はすでに終了しています。");
                return;
            }

            if (inn.ProgressScenarios.Contains(scenaName))
            {
                MessageDlg.ShowInfoDialog(scenaName + " は他のPTが進行中です。");
                return;
            }

            if (MessageDlg.ShowYesNoDialog("宿を " + scenaName + 
                " に到着した状態で\r\n保存しバリアントを起動します。よろしいですか？")
                == DialogResult.Yes)
            {

                //4
                DataTable cacheDt = null;
                using (ScenarioSearchDlg ssd = new ScenarioSearchDlg(this, inn, this))
                {

                    if (File.Exists(FilePath.ApplicationFolder + "\\Summary.cache"))
                    {
                        ssd.ReadCache();
                        cacheDt = ssd.ScenarioCacheDt;

                    }
                    else
                    {
                        MessageDlg.ShowInfoDialog("シナリオキャッシュがありません。");
                        return;
                    }
                }

                bool isScenario = false;
                int scenaNo = -1;
                for (int i = 0;i < cacheDt.Rows.Count; i++)
                {
                    if (cacheDt.Rows[i][0].Equals(scenaName))
                    {
                        isScenario = true;
                        scenaNo = i;
                        break;
                    }
                }

                if (!isScenario)
                {
                    MessageDlg.ShowInfoDialog("シナリオキャッシュに " + scenaName + " がありません。");
                    return;
                }
                else
                {
                    //5
                    //シナリオはあった！
                    string scenaFolder = cacheDt.Rows[scenaNo][7].ToString();

                    bool isCAB = scenaFolder.StartsWith("\"");
                    if (isCAB)
                    {
                        // CAB圧縮されている
                        MessageDlg.ShowInfoDialog("この機能はCAB圧縮シナリオは対象外です。");
                        return;
                    }

                    if (!Directory.Exists(scenaFolder))
                    {
                        // シナリオはなくなっていた！
                        MessageDlg.ShowInfoDialog(scenaName + " は移動しています。");
                        return;
                    }

                    SummaryData sd = null;
                    try
                    {
                        // サマリファイル読み出しチェック
                        sd = new SummaryData();
                        sd.Read(FilePath.GetSummaryFile(scenaFolder));
                    }
                    catch (Exception)
                    {
                        // シナリオは壊れていた！
                        MessageDlg.ShowInfoDialog("シナリオ：" + scenaName + " が壊れています。");
                        return;
                    }

                    //6
                    // F9時に戻るための金額情報
                    inn.CurrentParty.MoneyAtStart = inn.CurrentParty.Money;

                    ScenarioData scena = new ScenarioData();
                    scena.ScenarioFolder = scenaFolder;
                    scena.StepValues = new StringCollection();
                    scena.FlagValues = new StringCollection();
                    scena.PossessInfoCards = new StringCollection();
                    scena.PossessNpcCards = new StringCollection();

                    bool fukusuuFlg = party.Characters.Count > 1;

                    CellData cell;

                    switch(selectedMoveTo)
                    {

                        #region 交易都市リューン

                        case Ikinari.交易都市リューン:
                            //交易都市リューンに着いたところ
                            scena.AreaNumber = 1;
                            scena.FlagValues.Add("メニューの表示=1");
                            scena.BGMFile = "RYUNE.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "交易都市リューン.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region ゴブリンの洞窟

                        case Ikinari.ゴブリンの洞窟:

                            //いきなりすでに依頼を受けていて
                            //見張りの前
                            if (MessageDlg.ShowYesNoDialog("すでに600spで依頼を\r\n"
                                + "受けて見張りの前です。よろしいですか？")
                                != DialogResult.Yes) return;

                            scena.AreaNumber = 2;
                            scena.StepValues.Add("場面=1");
                            scena.BGMFile = "CHIPBLUE.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "背景＿洞窟.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            break;

                        #endregion

                        #region リューンの鍛冶屋

                        case Ikinari.リューンの鍛冶屋:

                            //いきなり行く・行かないの確認から
                            scena.AreaNumber = 2;

                            scena.StepValues.Add("みっしょんの状態=0");
                            scena.StepValues.Add("黒曜石投入数=0");
                            scena.StepValues.Add("紅曜石投入数=0");
                            scena.StepValues.Add("緑曜石投入数=0");
                            scena.StepValues.Add("碧曜石投入数=0");
                            scena.StepValues.Add("金鉱石投入数=0");
                            scena.StepValues.Add("合計投入数=0");
                            scena.StepValues.Add("剣は？=0");
                            scena.StepValues.Add("お菓子を与え=0");
                            scena.StepValues.Add("お菓子何個？=0");
                            scena.StepValues.Add("でぃふぇんす=0");

                            scena.FlagValues.Add("行くか止めるか=1");
                            scena.FlagValues.Add("鍛冶屋=0");
                            scena.FlagValues.Add("鍛冶屋の親父=0");
                            scena.FlagValues.Add("黒曜石=0");
                            scena.FlagValues.Add("金鉱石=0");
                            scena.FlagValues.Add("紅曜石=0");
                            scena.FlagValues.Add("緑曜石=0");
                            scena.FlagValues.Add("碧曜石=0");
                            scena.FlagValues.Add("溶鉱炉=0");
                            scena.FlagValues.Add("鉱石を入れた=1");
                            scena.FlagValues.Add("黒髪少女=0");
                            scena.FlagValues.Add("隕鉄を……=1");
                            scena.FlagValues.Add("黒髪少女(2)=0");
                            scena.FlagValues.Add("青龍の矛を=1");
                            scena.FlagValues.Add("Ｇクレイモアを=1");
                            scena.FlagValues.Add("弟子=0");
                            scena.FlagValues.Add("能書き=1");
                            scena.FlagValues.Add("ワンダーエッグ=1");
                            scena.FlagValues.Add("ちんぴら=0");
                            scena.FlagValues.Add("鎧を投げ込む=1");
                            scena.FlagValues.Add("盾を=1");
                            scena.FlagValues.Add("聖騎士=0");
                            scena.FlagValues.Add("聖騎士レイ=0");
                            scena.FlagValues.Add("聖騎士(キセル)=0");
                            scena.FlagValues.Add("聖騎士オラトリオ=0");
                            scena.FlagValues.Add("聖珠石=0");
                            scena.FlagValues.Add("老剣士=0");
                            scena.FlagValues.Add("若剣士=0");
                            scena.FlagValues.Add("メニューを……=1");
                            scena.FlagValues.Add("女騎士=0");

                            scena.BGMFile = "DNCE1.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "MapOfWirth.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Left = 116;
                            cell.Top = 15;
                            cell.Version = 5;
                            cell.Width = 400;
                            cell.Height = 260;
                            cell.FileName = "AdventurersInn.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            break;

                        #endregion

                        #region 隠者の庵

                        case Ikinari.隠者の庵:

                            if (inn.CurrentParty.Characters.Count < 3)
                            {
                                MessageDlg.ShowInfoDialog(
                                    "本シナリオは三人以上のパーティによる" +
                                    "プレイを基準としています。\r\n" +
                                     "誠に申し訳ありませんが、" +
                                     "三人未満のパーティではプレイできない仕様になっています。");
                                return;
                            }

                            //フラグ・ステップを初期状態に
                            Util3.SetInitScenarioData(scena, sd);
                            // 安全地帯はArea:1
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);


                            if (inn.Gossips.Contains("シュレックの最後の望み"))
                            {
                                // シュレックの最後の望みを聞いた後ならば

                                // 解読の巻物
                                WidCard injaScroll2 = null;

                                try
                                {
                                    injaScroll2 = new WidCard(WidIDtype.アイテム);

                                    injaScroll2.Read(scenaFolder +
                                                "\\Item2.wid");
                                }
                                catch
                                {
                                    MessageDlg.ShowInfoDialog("解読の巻物が読み込めません。");
                                    return;
                                }


                                if (Util1.CheckItemAll(inn, injaScroll2.CardName, injaScroll2.Expression))
                                {
                                    // 解読の巻物交換
                                    WidCard injaScroll3 = null;

                                    try
                                    {
                                        injaScroll3 = new WidCard(WidIDtype.アイテム);

                                        injaScroll3.Read(scenaFolder +
                                                    "\\Item3.wid");
                                    }
                                    catch
                                    {
                                        MessageDlg.ShowInfoDialog("解読の巻物が読み込めません。");
                                        return;
                                    }

                                    // パーティー全体で１枚入手
                                    inn.AddRenamedCardData(party.BagCards
                                        , injaScroll3
                                        , 1);

                                    // １枚削除
                                    Util1.DeleteCardOne(inn, CardIndexData.Kind.item,
                                        injaScroll2.CardName, injaScroll2.Expression);

                                }

                                // 背景変更
                                scena.Cells = new List<CellData>();

                                cell = new CellData();
                                cell.Left = 0;
                                cell.Top = 0;
                                cell.Version = 5;
                                cell.Width = 632;
                                cell.Height = 420;
                                cell.FileName = "bg_nightcastle04.jpg";
                                cell.MaskOn = false;
                                cell.FlagName = null;
                                cell.Visible = true;

                                scena.Cells.Add(cell);

                                cell = new CellData();
                                cell.Left = 116;
                                cell.Top = 15;
                                cell.Version = 5;
                                cell.Width = 400;
                                cell.Height = 260;
                                cell.FileName = "bg_nightcastle04.jpg";
                                cell.MaskOn = false;
                                cell.FlagName = null;
                                cell.Visible = true;

                                scena.Cells.Add(cell);

                                cell = new CellData();
                                cell.Left = 116;
                                cell.Top = 15;
                                cell.Version = 5;
                                cell.Width = 400;
                                cell.Height = 260;
                                cell.FileName = "back_frame.bmp";
                                cell.MaskOn = true;
                                cell.FlagName = null;
                                cell.Visible = true;

                                scena.Cells.Add(cell);


                                scena.SetFlag(@"シュレック氏・生", FLAG_TRUE);

                                scena.BGMFile = "時の舟.mid";

                                scena.SetFlag(@"二回目以降来店フラグ", FLAG_TRUE);
                                scena.SetFlag(@"表示切り替え", FLAG_TRUE);

                                if (inn.Gossips.Contains("アナザー技能表示"))
                                {
                                    scena.SetFlag(@"もう追加イベントは止めようと思っていたのに", FLAG_TRUE);
                                }

                            }
                            else
                            {
                                // シュレックの最後の望みを聞く前ならば
                                // 依頼受けて村長の話を聞いた後、

                                if (MessageDlg.ShowYesNoDialog("すでに300spで依頼を\r\n"
                                    + "受けて村長の話を聞いた後です。よろしいですか？")
                                    != DialogResult.Yes) return;

                                scena.BGMFile = "時の舟.mid";


                                // 背景変更
                                scena.Cells = new List<CellData>();

                                cell = new CellData();
                                cell.Left = 0;
                                cell.Top = 0;
                                cell.Version = 5;
                                cell.Width = 632;
                                cell.Height = 420;
                                cell.FileName = "back_castle.jpg";
                                cell.MaskOn = false;
                                cell.FlagName = null;
                                cell.Visible = true;

                                scena.Cells.Add(cell);

                                cell = new CellData();
                                cell.Left = 116;
                                cell.Top = 15;
                                cell.Version = 5;
                                cell.Width = 400;
                                cell.Height = 260;
                                cell.FileName = "Back_shop1.BMP";
                                cell.MaskOn = true;
                                cell.FlagName = null;
                                cell.Visible = true;

                                scena.Cells.Add(cell);

                                cell = new CellData();
                                cell.Left = 116;
                                cell.Top = 15;
                                cell.Version = 5;
                                cell.Width = 400;
                                cell.Height = 260;
                                cell.FileName = "back_frame.bmp";
                                cell.MaskOn = true;
                                cell.FlagName = null;
                                cell.Visible = true;

                                scena.Cells.Add(cell);

                                scena.SetFlag(@"村長表示ャー", FLAG_TRUE);
                                scena.SetFlag(@"オンリーザフラグ", FLAG_TRUE);
                            }

                            break;

                        #endregion

                        #region 悪魔の酒蔵

                        case Ikinari.悪魔の酒蔵:

                            scena.BGMFile = "iranai.mid";

                            //フラグ・ステップを初期状態に
                            Util3.SetInitScenarioData(scena, sd);
                            // 安全地帯はArea:1 祭壇のエリアイベント
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.SetFlag(@"悪魔", FLAG_TRUE);
                            scena.SetFlag(@"メニュー", FLAG_TRUE);


                            break;

                        #endregion

                        #region 花散里

                        case Ikinari.花散里:

                            //店内到着
                            scena.AreaNumber = 1;

                            scena.FlagValues.Add("技能①=0");

                            if (inn.Gossips.Contains("＿花散里の剣術指南"))
                            {
                                scena.FlagValues.Add("剣術指南=1");
                            }
                            else
                            {
                                scena.FlagValues.Add("剣術指南=0");
                            }

                            scena.FlagValues.Add("技能②=0");
                            scena.FlagValues.Add("剣術①=0");
                            scena.FlagValues.Add("表示切替用=1");
                            scena.FlagValues.Add("剣術②=0");
                            scena.FlagValues.Add("移動=1");

                            scena.BGMFile = "int39xg.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "_BLACK.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);


                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 420;
                            cell.Height = 300;
                            cell.FileName = "背景.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            break;

                        #endregion

                        #region 槍弓術指南道場

                        case Ikinari.槍弓術指南道場:

                            //道場に到着
                            scena.AreaNumber = 1;

                            scena.FlagValues.Add("メニューの表示=1");

                            scena.BGMFile = "αΩ_sakura.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "古山_japan08.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region 兵法指南所ミフネ

                        case Ikinari.兵法指南所ミフネ:

                            //初回のオープニングタイトルを飛ばす
                            //店内到着いらっしゃいの後から

                            string mifuneRaiten = "ミフネ来店";
                            string mifuneAdd = "ミフネ：追加"; 

                            //店内
                            scena.AreaNumber = 2;

                            scena.StepValues.Add("skill\\skill(page)=0");
                            scena.StepValues.Add("sp-100=0");
                            scena.StepValues.Add("sp-1000=0");
                            scena.StepValues.Add("aputitude1=0");
                            scena.StepValues.Add("aputitude2=0");


                            scena.FlagValues.Add("menu-main=1");
                            scena.FlagValues.Add("skill\\skill-1=0");
                            scena.FlagValues.Add("skill\\skill-2=0");
                            scena.FlagValues.Add("skill\\skill-3=0");
                            scena.FlagValues.Add("skill\\skill-4=0");
                            scena.FlagValues.Add("skill\\skill-5=0");
                            scena.FlagValues.Add("skill\\skill-6=0");
                            scena.FlagValues.Add("money=1");
                            scena.FlagValues.Add("back\\credit：Ask=0");
                            scena.FlagValues.Add("back\\credit：ComLink=0");

                            if (inn.Gossips.Contains(mifuneRaiten))
                            {
                                //２回目以降
                                scena.FlagValues.Add("back\\mask=0");
                            }
                            else
                            {
                                //初回
                                scena.FlagValues.Add("back\\mask=1");

                                inn.Gossips.Add(mifuneRaiten);
                            }

                            if (inn.Gossips.Contains(mifuneAdd))
                            {
                                //追加がある
                                scena.FlagValues.Add("menu-plus=1");
                            }
                            else
                            {
                                //追加が無い
                                scena.FlagValues.Add("menu-plus=0");
                            }
                            
                            scena.FlagValues.Add("skill\\skill-plus=0");

                            scena.BGMFile = "bm3.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "black.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Left = 10;
                            cell.Top = 20;
                            cell.Version = 5;
                            cell.Width = 430;
                            cell.Height = 240;
                            cell.FileName = "bg_1.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            break;

                        #endregion

                        #region 師匠屋閉店セール

                        case Ikinari.師匠屋閉店セール:

                            //店内到着：「いらっしゃいませ～♪」の後。

                            scena.AreaNumber = 4;

                            scena.FlagValues.Add("到着=1");

                            scena.BGMFile = "apple.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "MapOfWirth.BMP";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Left = 116;
                            cell.Top = 15;
                            cell.Version = 5;
                            cell.Width = 400;
                            cell.Height = 260;
                            cell.FileName = "AdventurersInn_.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            break;

                        #endregion

                        #region 武の街キラナン

                        case Ikinari.武の街キラナン:

                            //武の街到着

                            for (char c = 'B'; c <= 'G'; c++)
                            {
                                if (inn.Gossips.Contains("武の街ｷﾗﾅﾝver." + c))
                                {
                                    MessageDlg.ShowInfoDialog("上位バージョン（ver.Ｂ以降）を\r\n"
                                        + "プレイした宿ではプレイ出来ません。");
                                    return;
                                }
                            }

                            ////string bunomachiOuhou = "武の街往訪";
                            ////if (Util1.CheckCoupon(party, bunomachiOuhou))
                            ////{
                            ////    //"武の街往訪"がある
                            ////}
                            ////else
                            ////{
                            ////    //"武の街往訪"が無い
                            ////    //終了確認があるがスルー
                            ////}
                            
                            scena.AreaNumber = 21;

                            if (fukusuuFlg)
                            {
                                scena.StepValues.Add("複数形=2");
                            }
                            else
                            {
                                scena.StepValues.Add("複数形=0");
                            }

                            scena.StepValues.Add("汎用ｶｳﾝﾀ1=0");
                            scena.StepValues.Add("訪問釈華力=0");
                            scena.StepValues.Add("訪問蓮=0");
                            scena.StepValues.Add("訪問WB=0");
                            scena.StepValues.Add("汎用ｶｳﾝﾀ2=0");


                            scena.FlagValues.Add("@主=0");
                            scena.FlagValues.Add("@商品1=0");
                            scena.FlagValues.Add("@EXIT=0");
                            scena.FlagValues.Add("酒娘=0");
                            scena.FlagValues.Add("@商品2=0");
                            scena.FlagValues.Add("@商品3=0");
                            scena.FlagValues.Add("@商品物色中=0");
                            scena.FlagValues.Add("@主名無=0");
                            scena.FlagValues.Add("@主と会話=0");
                            scena.FlagValues.Add("*wb会話ﾌﾞﾀ=0");
                            scena.FlagValues.Add("①蓮刀/wb掌1/毒蛇6=0");
                            scena.FlagValues.Add("①蓮本/wb掌2/毒瓶=0");
                            scena.FlagValues.Add("①wb掌3/毒掌1=0");
                            scena.FlagValues.Add("①wb掌4/毒掌2=0");
                            scena.FlagValues.Add("①wb掌5/毒掌3=0");
                            scena.FlagValues.Add("①wb掌6/毒掌4=0");
                            scena.FlagValues.Add("①wbﾌﾞﾀ/毒壷=0");
                            scena.FlagValues.Add("@主2=0");
                            scena.FlagValues.Add("①釈2木/wb兎01/毒蛇眼=0");
                            scena.FlagValues.Add("①釈2木枯/wb兎02=0");
                            scena.FlagValues.Add("①釈2攻1/wb兎03=0");
                            scena.FlagValues.Add("①釈2攻2/wb兎04=0");
                            scena.FlagValues.Add("①釈2攻3/wb兎05=0");
                            scena.FlagValues.Add("①釈瓶/蓮壷a/wb兎06=0");
                            scena.FlagValues.Add("①蓮壷b/wb兎07/毒蛇1=0");
                            scena.FlagValues.Add("①蓮壷c/wb兎08/毒蛇2=0");
                            scena.FlagValues.Add("①蓮壷d/wb兎09/毒蛇3=0");
                            scena.FlagValues.Add("①釈盃1/蓮壷1/wb兎10=0");
                            scena.FlagValues.Add("①釈盃2/蓮壷2/wb兎11=0");
                            scena.FlagValues.Add("①釈盃3/蓮壷3/wb兎12=0");
                            scena.FlagValues.Add("①釈盃4/蓮壷4/wb兎13=0");
                            scena.FlagValues.Add("①釈盃5/蓮壷5/wb兎14=0");
                            scena.FlagValues.Add("①釈盃6/蓮壷6/wb兎15=0");
                            scena.FlagValues.Add("①蓮壷e/wb兎16/毒蛇4=0");
                            scena.FlagValues.Add("①蓮壷f/wb兎17/毒蛇5=0");
                            scena.FlagValues.Add("①wb兎18/毒掌6=0");
                            scena.FlagValues.Add("@flip1=0");
                            scena.FlagValues.Add("@flip2=0");
                            scena.FlagValues.Add("@flip3=0");
                            scena.FlagValues.Add("@flip4=0");
                            scena.FlagValues.Add("@flip5=0");
                            scena.FlagValues.Add("@flip6=0");


                            bool isBunomachiB =
                                Util1.CheckCoupon(party, "毒手の使い手") ||
                                Util1.CheckCoupon(party, "悪役") ||
                                Util1.CheckSkill(party, "暗殺の一撃",
                                "レベル５技能カード（技）。盗賊の技。敵一体の背後に忍び寄り、致命的な一撃を与える。暗殺を生業とする盗賊が好んで使用する技である。実体を持たない敵には効果がない。") ||
                                Util1.CheckSkill(party, "盗賊の手",
                                "レベル３技能カード（技）。盗賊の熟練の技。宝箱や扉に仕掛けられた鍵や罠を安全に解除する。") ||
                                Util1.CheckSkill(party, "闇に隠れる",
                                "レベル１技能カード（技）。盗賊の技。暗闇に同化して姿を消し、敵の攻撃を避ける。このカードを使用しているラウンドは武器による攻撃を無力化できる。") ||
                                Util1.CheckSkill(party, "盗賊の眼",
                                "レベル１技能カード（技）。盗賊の技の一つ。箱や扉などの物体を注意深く観察し、罠や常人では気付き得ない特徴を発見する技術。") ||
                                Util1.CheckSkillAll(inn, "烈掌破",
                                "レベル３技能カード（技）。怒りの感情で練った気弾を掌より発し、敵一体を打ち据える。この紅蓮の気弾は高熱を放ち、触れた敵の身を焦がす。術者には、烈火の気性が要求される。しかし、同時に冷静さを持ち合わせていない場合、怒りを制御できず激昂状態となる。") ||
                                Util1.CheckSkillAll(inn, "陰掌破",
                                "レベル３技能カード（技）。憎悪の感情で練った気弾を掌より発し、敵一体を打ち据える。この高密度の邪気は、攻撃と同時にその精神を蝕み、混乱状態に陥れる。術者には、邪悪な精神が要求される。しかし、同時に冷静さを持ち合わせてない場合、己の心をも蝕む事となる。") ||
                                Util1.CheckSkillAll(inn, "蝕心瘴",
                                "レベル３技能カード（技）。負の感情を込めた気を全身より放つ。放たれた瘴気は、敵意ある魔術より己の身を護ると同時に、周囲の者の精神を衰弱させ、抵抗力を減少させる。しかし、味方にも同様の効果を及ぼしてしまう。この術の使い手には、邪な魂が要求される。") ||
                                Util1.CheckSkillAll(inn, "蛇眼",
                                "レベル３技能カード（技）。憎悪の念を込めた瞳で、相手の瞳を凝視する。これを受けた者は極度の恐怖を感じ、まるで蛇に睨まれた蛙の如く、暫くその身を硬直させてしまう。しかし、視線を一点に集中させる為、使用時には大きな隙ができてしまう。邪な精神が求められる。") ||
                                Util1.CheckSkillAll(inn, "吸精陰手", 
                                "レベル５技能カード（技）。敵の肉体に手をかざし、その気を吸収して己の体力を回復させる。この術は、気血を持たない対象には効果がない。使用には、邪な精神が求められる。") ||
                                Util1.CheckSkillAll(inn, "陰気封殺",
                                "レベル５技能カード（技）。極限まで高めた負の感情を凝縮した気の球で対象を包み込み、その精神を凍てつかせて特殊技能を封じる。但し、術者の純真な気質が強いと負の気を制御しきれず自らの精神を凍てつかせてしまう危険がある。使い手には邪悪な精神が要求される。") ||
                                Util1.CheckSkillAll(inn, "殺気滅心",
                                "レベル７技能カード（技）。殺意を凝縮した気の球で対象を包み込む。この殺意に耐え切れぬ者は、心が死を選択し、擬似的な死亡状態に陥る。但し術者の甘い気質が強いと、殺意を制御しきれず自らの精神を蝕んでしまう危険がある。使い手には、残虐な精神が求められる。") ||
                                Util1.CheckSkillAll(inn, "怒髪咆",
                                "レベル７技能カード（技）。特殊な呼吸法で肺に大量の気を溜め、爆音と共に吐き出す。敵陣を襲う強烈な衝撃波がその平衡感覚を狂わせ、召喚獣を消し去る。しかし爆音は味方にも悪影響を及ぼし、更には術者自身の咽をも破壊する。使用には荒ぶる魂が求められる。");

                            if (isBunomachiB)
                            {
                                scena.FlagValues.Add("@建物A=0");
                            }
                            else
                            {
                                scena.FlagValues.Add("@建物A=1");
                            }

                            if (isVariant128())
                            {
                                scena.FlagValues.Add("@ver=0");
                                if (!inn.Gossips.Contains("武街覚居穿薙梟狼"))
                                {
                                    if (inn.Gossips.Contains("武街見居"))
                                    {
                                        inn.Gossips.Remove("武街見居");
                                        inn.Gossips.Add("武街覚居");
                                    }

                                    if (inn.Gossips.Contains("武街見穿"))
                                    {
                                        inn.Gossips.Remove("武街見穿");
                                        inn.Gossips.Add("武街覚穿");
                                    }

                                    if (inn.Gossips.Contains("武街見薙"))
                                    {
                                        inn.Gossips.Remove("武街見薙");
                                        inn.Gossips.Add("武街覚薙");
                                    }

                                    if (inn.Gossips.Contains("武街見梟"))
                                    {
                                        inn.Gossips.Remove("武街見梟");
                                        inn.Gossips.Add("武街覚梟");
                                    }

                                    if (inn.Gossips.Contains("武街見狼"))
                                    {
                                        inn.Gossips.Remove("武街見狼");
                                        inn.Gossips.Add("武街覚狼");
                                    }
                                }
                            }
                            else
                            {
                                scena.FlagValues.Add("@ver=1");
                            }
                            
                            
                            scena.FlagValues.Add("@商品2.5=0");
                            scena.FlagValues.Add("①wb掌A/毒掌5=0");
                            scena.FlagValues.Add("*入養生所=0");
                            scena.FlagValues.Add("@商品4=0");
                            scena.FlagValues.Add("*棍師口上=0");
                            scena.FlagValues.Add("*入寺=0");
                            scena.FlagValues.Add("*入邪拳処=0");


                            if (isBunomachiB)
                            {
                                scena.FlagValues.Add("@建物B=1");
                            }
                            else
                            {
                                scena.FlagValues.Add("@建物B=0");
                            }

                            scena.FlagValues.Add("@街出=1");
                            scena.FlagValues.Add("酒ｸﾗ=0");
                            scena.FlagValues.Add("ｸﾗ剣見=0");
                            scena.FlagValues.Add("ｸﾗ梟=0");
                            scena.FlagValues.Add("ｸﾗ狼=0");
                            scena.FlagValues.Add("ｸﾗ薙=0");
                            scena.FlagValues.Add("ｸﾗ居=0");
                            scena.FlagValues.Add("ｸﾗ穿=0");
                            scena.FlagValues.Add("ｸﾗ技披露=0");

                            scena.BGMFile = "CW29.MID";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "MapOfWirth.BMP";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);


                            cell = new CellData();
                            cell.Left = 116;
                            cell.Top = 15;
                            cell.Version = 5;
                            cell.Width = 400;
                            cell.Height = 260;
                            cell.FileName = "bousou03.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);


                            cell = new CellData();
                            cell.Left = 115;
                            cell.Top = 15;
                            cell.Version = 5;
                            cell.Width = 401;
                            cell.Height = 260;
                            cell.FileName = "BS_MoW.BMP";
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region ボサの散歩

                        case Ikinari.ボサの散歩:

                            //到着してレベル判定後
                            scena.AreaNumber = 1;

                            scena.StepValues.Add("あいさつ=0");
                            scena.StepValues.Add("娘=0");
                            scena.StepValues.Add("戦闘=0");
                            scena.StepValues.Add("盗賊団=0");
                            scena.StepValues.Add("仕事=0");
                            scena.StepValues.Add("ねずみ=0");
                            scena.StepValues.Add("猪=0");
                            scena.StepValues.Add("Ｘ軸=0");
                            scena.StepValues.Add("Ｙ軸=0");

                            scena.FlagValues.Add("占い=0");
                            scena.FlagValues.Add("金運=0");
                            scena.FlagValues.Add("散歩=0");
                            scena.FlagValues.Add("村娘=0");
                            scena.FlagValues.Add("野良犬=0");
                            scena.FlagValues.Add("盗賊=0");
                            scena.FlagValues.Add("盗賊団の笑い=0");
                            scena.FlagValues.Add("ボサ=1");
                            scena.FlagValues.Add("依頼=0");
                            scena.FlagValues.Add("貼り紙=0");
                            scena.FlagValues.Add("へぼへぼ=0");

                            if (Util1.CheckPartyLevel(party, 6))
                            {
                                scena.FlagValues.Add("ＬＶ６=1");
                            }
                            else
                            {
                                scena.FlagValues.Add("ＬＶ６=0");
                            }
                            
                            scena.FlagValues.Add("北=0");
                            scena.FlagValues.Add("ボサ１=0");
                            scena.FlagValues.Add("ボサ２=0");
                            scena.FlagValues.Add("ボサ３=0");
                            scena.FlagValues.Add("げるち=0");
                            scena.FlagValues.Add("ボルグ=0");

                            if (Util1.CheckPartyLevel(party, 3))
                            {
                                scena.FlagValues.Add("ＬＶ３=1");
                            }
                            else
                            {
                                scena.FlagValues.Add("ＬＶ３=0");
                            }
                            
                            scena.FlagValues.Add("ゾーン=0");
                            scena.FlagValues.Add("探索=0");

                            if (Util1.CheckCoupon(party, "＿カプカプ"))
                            {
                                scena.PossessInfoCards.Add("カプカプ情報");
                            }

                            scena.BGMFile = "DefInn.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "AdventurersInn.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region 酒と露店の街デューン
                        case Ikinari.酒と露店の街デューン:

                            //酒と露店の街デューン
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "ivysoshitebokuha.mid";

                            break;

                        #endregion

                        #region 落日の村オランジェル
                        case Ikinari.落日の村オランジェル:

                            //落日の村オランジェル
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            if (inn.Gossips.Contains("落日の丘の情報済"))
                            {
                                scena.SetFlag("丘へ", FLAG_TRUE);
                            }
                            scena.BGMFile = "足長td- 稲穂と鍬と草笛と.mid";

                            break;

                        #endregion
                            
                        #region 夜の交易都市
                        case Ikinari.夜の交易都市:

                            //到着後
                            scena.AreaNumber = 1;

                            scena.FlagValues.Add("メニューの表示=1");
                            scena.FlagValues.Add("工事中=0");
                            scena.FlagValues.Add("買い物=1");
                            scena.FlagValues.Add("入る=0");
                            scena.FlagValues.Add("雑貨屋－表から着た=1");
                            scena.FlagValues.Add("雑貨屋－裏から着た=0");

                            scena.BGMFile = "gr1.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "交易都市リューン.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion
                            
                        #region 冒険者の街カーゾ

                        case Ikinari.冒険者の街カーゾ:
                            
                            //到着後

                            //エリアは全景
                            scena.AreaNumber = 3;

                            //ステップ・フラグに変更無し

                            scena.BGMFile = "town3.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "bg_007h.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region 冒険者の都市フォルザーヴ

                        case Ikinari.冒険者の都市フォルザーヴ:

                            //エリアは冒険者の都市フォルザーヴ
                            scena.AreaNumber = 2;

                            scena.FlagValues.Add("メニューの表示=1");

                            scena.BGMFile = "town1.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "フォルザーヴ全景.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region カードマニア
                        case Ikinari.カードマニア:

                            //カードマニアの館
                            Util3.SetInitScenarioData(scena, sd);

                            scena.AreaNumber = 3;

                            scena.SetFlag("ロディ", FLAG_TRUE);
                            scena.SetFlag("行動開始", FLAG_TRUE);

                            if (!inn.Gossips.Contains("カードマニアの館"))
                            {
                                inn.Gossips.Add("カードマニアの館");
                            }

                            scena.BGMFile = "cw_und.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "fr002.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region ようこそ札闘亭へ

                        case Ikinari.ようこそ札闘亭へ:

                            //月明かりの夜に到着時
                            scena.AreaNumber = 4;

                            #region ようこそ札闘亭へ　ステップ・フラグべた書き

                            scena.StepValues.Add("＊カードの力　Ｍｅ=0");
                            scena.StepValues.Add("＊カードの力　Ｙｏｕ=0");
                            scena.StepValues.Add("＊エレメント　Ｍｅ=0");
                            scena.StepValues.Add("＊エレメント　Ｙｏｕ=0");
                            scena.StepValues.Add("＊エレメント　ExMe=0");
                            scena.StepValues.Add("＊総合力　Ｍｅ=0");
                            scena.StepValues.Add("＊総合力　Ｙｏｕ=0");
                            scena.StepValues.Add("＊エレメント　ExYou=0");
                            scena.StepValues.Add("対戦相手=0");
                            scena.StepValues.Add("＊ダメージ=0");
                            scena.StepValues.Add("～火のエレメント=5");
                            scena.StepValues.Add("～風のエレメント=5");
                            scena.StepValues.Add("～地のエレメント=5");
                            scena.StepValues.Add("～水のエレメント=5");
                            scena.StepValues.Add("～光のエレメント=5");
                            scena.StepValues.Add("～闇のエレメント=5");
                            scena.StepValues.Add("～無のエレメント=5");
                            scena.StepValues.Add("～ライフ　Ｍｅ=0");
                            scena.StepValues.Add("～ライフ　Ｙｏｕ=0");
                            scena.StepValues.Add("＊会話=0");
                            scena.StepValues.Add("～ライフ　Ｍｅ×１０=0");
                            scena.StepValues.Add("～ライフ　Ｙｏｕ×１０=0");
                            scena.StepValues.Add("勝敗=0");
                            scena.StepValues.Add("＊ラウンド×１０=0");
                            scena.StepValues.Add("＊ラウンド=1");
                            scena.StepValues.Add("＃ティシューのカード=0");
                            scena.StepValues.Add("＃サリットのカード=0");
                            scena.StepValues.Add("＃コーパスのカード=0");
                            scena.StepValues.Add("＃ペナンスのカード=0");
                            scena.StepValues.Add("＊Ｓパワー　Ｙｏｕ=0");
                            scena.StepValues.Add("＊Ｓパワー　Ｍｅ=0");
                            scena.StepValues.Add("＊特殊能力　Ｍｅ=0");
                            scena.StepValues.Add("＊特殊光　Ｍｅ=0");
                            scena.StepValues.Add("＊特殊闇　Ｍｅ=0");
                            scena.StepValues.Add("＊特殊人　Ｍｅ=0");
                            scena.StepValues.Add("＊特殊能力　Ｙｏｕ=0");
                            scena.StepValues.Add("＊特殊人　Ｙｏｕ=0");
                            scena.StepValues.Add("＊特殊光　Ｙｏｕ=0");
                            scena.StepValues.Add("＊特殊闇　Ｙｏｕ=0");
                            scena.StepValues.Add("＊タイム封魔=0");
                            scena.StepValues.Add("＊タイム異世界=0");
                            scena.StepValues.Add("＊レインボー　Ｍｅ=0");
                            scena.StepValues.Add("＊レインボー　Ｙｏｕ=0");

                            bool sattouteiHenna = Util1.CheckCoupon(party, "椎名リオとの交渉終了") ||
                                Util1.CheckCoupon(party, "椎名リオ怒る");

                            if (sattouteiHenna)
                            {
                                scena.StepValues.Add("状況ステップ=3");
                            }
                            else
                            {
                                scena.StepValues.Add("状況ステップ=0");
                            }

                            scena.StepValues.Add("思考パターン=0");
                            scena.StepValues.Add("＊エレメント描画=0");
                            scena.StepValues.Add("＃シェファーのカード=0");
                            scena.StepValues.Add("ヘルプ内容=0");
                            scena.StepValues.Add("＊さかしま=0");
                            scena.StepValues.Add("＊ラウンド×１００=0");
                            scena.StepValues.Add("＊コンボ　Ｍｅ=0");
                            scena.StepValues.Add("＊コンボ　Ｙｏｕ=0");
                            scena.StepValues.Add("＊どうでも良いステップ=0");
                            scena.StepValues.Add("＊勝利×１=0");
                            scena.StepValues.Add("＊勝利×１０=0");
                            scena.StepValues.Add("＊敗北×１=0");
                            scena.StepValues.Add("＊敗北×１０=0");
                            scena.StepValues.Add("＃ウィウィのカード=0");

                            scena.FlagValues.Add("＠ティシュー=0");
                            scena.FlagValues.Add("パターン=1");
                            scena.FlagValues.Add("エレメント減少　Ｍｅ=1");
                            scena.FlagValues.Add("ダメージを受けるのは=1");
                            scena.FlagValues.Add("＠常に表示=1");
                            scena.FlagValues.Add("エレメント減少　Ｙｏｕ=1");
                            scena.FlagValues.Add("逆パターン=0");
                            scena.FlagValues.Add("＠サリット=0");
                            scena.FlagValues.Add("サリット(2)=0");
                            scena.FlagValues.Add("ティシュー(2)=0");
                            scena.FlagValues.Add("＠ペナンス=0");
                            scena.FlagValues.Add("ペナンス(2)=0");
                            scena.FlagValues.Add("＠コーパス=0");
                            scena.FlagValues.Add("コーパス(2)=0");
                            scena.FlagValues.Add("＠汎用フラグ=0");
                            scena.FlagValues.Add("＊キャラカード Ｍｅ=0");

                            if (inn.Gossips.Contains("デュエル台詞省略"))
                            {
                                scena.FlagValues.Add("会話=0");
                            }
                            else
                            {
                                scena.FlagValues.Add("会話=1");
                            }

                            if (fukusuuFlg)
                            {
                                scena.FlagValues.Add("ら=1");
                            }
                            else
                            {
                                scena.FlagValues.Add("ら=0");
                            }

                            scena.FlagValues.Add("＠椎名リオ=0");

                            if (fukusuuFlg)
                            {
                                scena.FlagValues.Add("達=1");
                            }
                            else
                            {
                                scena.FlagValues.Add("達=0");
                            }

                            scena.FlagValues.Add("＊無属性所有=0");
                            scena.FlagValues.Add("＊キャラカード　You=0");
                            scena.FlagValues.Add("データは極力=0");
                            scena.FlagValues.Add("ルールブック処理=0");
                            scena.FlagValues.Add("カウンター=1");
                            scena.FlagValues.Add("星増加方法=1");
                            scena.FlagValues.Add("＠シェファー=0");
                            scena.FlagValues.Add("シェファー(2)=0");

                            int sattouteiType = 0;

                            if (inn.Gossips.Contains("数値完全隠蔽"))
                            {
                                sattouteiType = 0;
                            }
                            else if (inn.Gossips.Contains("メモリ型表示"))
                            {
                                sattouteiType = 1;
                            }
                            else
                            {
                                sattouteiType = 2;
                            }

                            switch (sattouteiType)
                            {
                                case 0:
                                    //数値隠蔽
                                    scena.FlagValues.Add("＠数値型=0");
                                    scena.FlagValues.Add("＠メモリ型=0");
                                    break;
                                case 1:
                                    //メモリ型
                                    scena.FlagValues.Add("＠数値型=0");
                                    scena.FlagValues.Add("＠メモリ型=1");
                                    break;
                                case 2:
                                    //数値型
                                    scena.FlagValues.Add("＠数値型=1");
                                    scena.FlagValues.Add("＠メモリ型=0");
                                    break;
                            }

                            scena.FlagValues.Add("＠汎用フラグ(2)=0");
                            scena.FlagValues.Add("＠汎用フラグ(3)=1");
                            scena.FlagValues.Add("一時的フラグ(1)=0");
                            scena.FlagValues.Add("一時的フラグ(2)=0");
                            scena.FlagValues.Add("サリット酔う=0");
                            scena.FlagValues.Add("一時的フラグ(3)=0");
                            scena.FlagValues.Add("一時的フラグ(4)=0");
                            scena.FlagValues.Add("ホミッチ=1");
                            scena.FlagValues.Add("ラウンド無効=0");
                            scena.FlagValues.Add("＠うさグリーン=0");

                            if (sattouteiHenna)
                            {
                                scena.FlagValues.Add("＠キース=1");
                            }
                            else
                            {
                                scena.FlagValues.Add("＠キース=0");
                            }

                            scena.FlagValues.Add("＠ヘルプ用=0");

                            switch (sattouteiType)
                            {
                                case 0:
                                    //数値隠蔽
                                    scena.FlagValues.Add("画像封印=1");
                                    scena.FlagValues.Add("画像解除=0");
                                    break;
                                case 1:
                                    //メモリ型
                                    scena.FlagValues.Add("画像封印=0");
                                    scena.FlagValues.Add("画像解除=1");
                                    break;
                                case 2:
                                    //数値型
                                    scena.FlagValues.Add("画像封印=0");
                                    scena.FlagValues.Add("画像解除=1");
                                    break;
                            }

                            scena.FlagValues.Add("カード宮殿=0");
                            scena.FlagValues.Add("カードの種=0");
                            scena.FlagValues.Add("高速描画=1");
                            scena.FlagValues.Add("高速描画２=1");
                            scena.FlagValues.Add("＊カニモイの判決=0");
                            scena.FlagValues.Add("＠瞬間ダメージ=0");
                            scena.FlagValues.Add("デバグモード=0");
                            scena.FlagValues.Add("＠ウィウィ=0");
                            scena.FlagValues.Add("ウィウィ(2)=0");


                            #endregion

                            // ベヒーモス交換
                            WidCard sattouteiBehi = null;

                            try
                            {
                                sattouteiBehi = new WidCard(WidIDtype.アイテム);

                                sattouteiBehi.Read(scenaFolder +
                                            "\\Item79.wid");
                            }
                            catch
                            {
                                MessageDlg.ShowInfoDialog("ベヒーモスが読み込めません。");
                                return;
                            }

                            while (Util1.DeleteCardOne(inn, CardIndexData.Kind.item,
                                "べヒーモス",
                                "モンスターカード(普通・獣)\r\nレベル：7\r\n体力：140\r\nアライメント：Neutral\r\n出現場所：サイファーの館\r\n使用時の効果：敵全体にダメージ(？)\r\n肉体属性\r\n\r\n\r\n"))
                            {
                                inn.AddRenamedCardData(party.BagCards
                                    , sattouteiBehi
                                    , 1);
                            }

                            scena.BGMFile = "forgoten.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "DARK.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;


                            cell = new CellData();
                            cell.Left = 117;
                            cell.Top = 16;
                            cell.Version = 5;
                            cell.Width = 400;
                            cell.Height = 260;
                            cell.FileName = "ho-17.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;
                        #endregion

                        #region スティープルチェイサー
                        case Ikinari.スティープルチェイサー:

                            //スティープルチェイサー
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            if (!inn.Gossips.Contains("スティープルチェイサー"))
                            {
                                if (inn.CurrentParty.Characters.Count < 4)
                                {
                                    MessageDlg.ShowInfoDialog(
                                        "申し訳ありませんが、\r\n" +
                                     "4人以上のパーティで臨んで下さい。\r\n" +
                                  "一旦終了致します。");
                                    return;
                                }

                                // ストーリーモード
                                scena.SetFlag("BACK_Rhun", FLAG_TRUE);
                                scena.SetFlag("BACK_title2", FLAG_TRUE);
                                scena.SetFlag("MENU_rider", FLAG_TRUE);
                            }
                            else
                            {

                                if (inn.CurrentParty.Characters.Count < 4)
                                {
                                    MessageDlg.ShowInfoDialog(
                                        "4人未満なので\r\n" +
                                        "レースモードで起動します。");
                                    // レースモード
                                    Util1.SetCouponAll(party, "：レース版", 0);
                                    scena.SetFlag("BACK_title4", FLAG_TRUE);
                                    scena.SetFlag("BACK_Rhun", FLAG_TRUE);
                                    scena.SetFlag("MENU_rider(2)", FLAG_TRUE);
                                }
                                else
                                {
                                    DialogResult ret = MessageDlg.ShowYesNoDialog(
                                        "Yes:ストーリーモード、No:レースモード");
                                    if (ret == DialogResult.Yes)
                                    {
                                        // ストーリーモード
                                        scena.SetFlag("BACK_Rhun", FLAG_TRUE);
                                        scena.SetFlag("BACK_title2", FLAG_TRUE);
                                        scena.SetFlag("MENU_rider", FLAG_TRUE);
                                    }
                                    else
                                    {
                                        // レースモード
                                        Util1.SetCouponAll(party, "：レース版", 0);
                                        scena.SetFlag("BACK_title4", FLAG_TRUE);
                                        scena.SetFlag("BACK_Rhun", FLAG_TRUE);
                                        scena.SetFlag("MENU_rider(2)", FLAG_TRUE);
                                    }
                                }
                            }

                            scena.BGMFile = "MID_steeple.mid";

                            break;

                        #endregion             
                            
                        #region 特殊な種
                        case Ikinari.特殊な種:

                            //特殊な種
                            if (!isVariant128())
                            {
                                MessageDlg.ShowInfoDialog(
                                "このシナリオはVer1.28エンジン専用です。\r\n" +
                                "2009年1月12日現在、配布の最新版に\r\n" +
　                              "対応しています。");
                                return;
                            }
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            break;

                        #endregion             
                            
                        #region 悪魔の書
                        case Ikinari.悪魔の書:

                            //悪魔の書
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            break;

                        #endregion             
                            
                        #region 冒険者支援センター
                        case Ikinari.冒険者支援センター:

                            //冒険者支援センター
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.Cells.Clear();
                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "室内.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            scena.SetFlag("メニュー表示用フラグ", FLAG_TRUE);

                            break;

                        #endregion             
                            
                        #region 天使養成所第1課程第１話
                        case Ikinari.天使養成所第1課程第１話:

                            //天使養成所第1課程第１話
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 18);

                            scena.BGMFile = null;

                            break;

                        #endregion

                        #region ジョブの街
                        case Ikinari.ジョブの街:

                            //ジョブの街
                            Util3.SetInitScenarioData(scena, sd);
                            
                            Util3.SetAreaInitCell(scena, scenaFolder, 5);

                            // 転職の間に着いた時：最短で昼４=3
                            scena.SetStep("town\\時間",3);

                            if (inn.Gossips.Contains("ジョブの街"))
                            {
                                string job = string.Empty;
                                CouponData jobCoup = null;
                                // 全員の称号修正・ジョブ判定・表示フラグ変更
                                for(int i = 0;i < party.Characters.Count;i++)
                                {
                                    CharacterData cd = party.Characters[i];

                                    #region 称号修正
                                    if (!inn.Gossips.Contains("＿j称号修正済"))
                                    {
                                        for (int j = 0; j <= (int)Job.騎士; j++)
                                        {
                                            job = ((Job)j).ToString();
                                            if ((jobCoup = cd.GetCouponData(job)) != null)
                                            {
                                                cd.Coupons.Remove(jobCoup);
                                                cd.SetCoupon("["
                                                    + job + "]", 0);
                                            }
                                        }
                                    }
                                    #endregion

                                    #region ジョブ判定・表示フラグ変更

                                    for (int j = 0; j <= (int)Job.騎士; j++)
                                    {
                                        job = ((Job)j).ToString();
                                        if ((jobCoup = cd.GetCouponData("["
                                                + job + "]")) != null)
                                        {
                                            if (job.Equals("格闘家") ||
                                                job.Equals("召喚師") ||
                                                job.Equals("剣神"))
                                            {
                                                // まだフラグが無い
                                                continue;
                                            }

                                            // Jobについている
                                            scena.SetFlag("job\\job non(" + (i+1) + ")", FLAG_FALSE);
                                            // ジョブの表示フラグを立てる
                                            scena.SetFlag("job\\job "
                                                + job + "(" + (i + 1) + ")", FLAG_TRUE);

                                            break;
                                        }
                                    }
                                    #endregion
                                }
                            }

                            scena.BGMFile = "PSMAT064.mid";

                            if (inn.Gossips.Contains("もう見ねぇよ"))
                            {
                                scena.SetFlag("転職アニメ", FLAG_FALSE);
                            }
                            break;

                        #endregion

                        #region RP5出張所
                        case Ikinari.RP5出張所:
                            //RP5出張所
                            Util3.SetInitScenarioData(scena, sd);
                            
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "Festival.mid";

                            if (!Util1.CheckCoupon(party, "＿RP5のテントを覗く"))
                            {
                                Util1.SetCouponAll(party, "＿RP5のテントを覗く", 0);
                                scena.SetFlag("リオ", FLAG_TRUE);
                            }
                            else
                            {
                                if (DiceRoll.RandomInt(100) < 70)
                                {
                                    scena.SetFlag("リオ", FLAG_TRUE);
                                }
                                else
                                {
                                    scena.SetFlag("メモ", FLAG_TRUE);
                                }
                            }
                            break;
                        #endregion

                        #region ジュークボックス
                        case Ikinari.ジュークボックス:
                            //ジュークボックス
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "bgm1.mid";
                            scena.SetFlag("f1", FLAG_TRUE);
                            if (inn.Gossips.Contains("郵便屋：落とした手紙"))
                            {
                                if (!inn.Gossips.Contains("郵便屋：手紙を拾う"))
                                {
                                    scena.SetFlag("letter", FLAG_TRUE);
                                }
                            }

                            if (inn.Gossips.Contains("音札：白鍋通り"))
                            {
                                scena.SetFlag("f2", FLAG_TRUE);
                            }
                            break;
                        #endregion

                        #region チュートリアル
                        case Ikinari.チュートリアル:
                            //チュートリアル
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "bgm1.mid";
                            scena.SetFlag("f1", FLAG_TRUE);
                            if (inn.Gossips.Contains("メレンダ値引き配布済"))
                            {
                                scena.SetFlag("f3", FLAG_TRUE); 
                            } else {
                                scena.SetFlag("f3", FLAG_TRUE);  // ベヒーモス交換
                                WidCard discount = null;

                                try
                                {
                                    discount = new WidCard(WidIDtype.スキル);

                                    discount.Read(scenaFolder +
                                                "\\Skill1.wid");

                                    inn.AddRenamedCardData(party.BagCards
                                        , discount
                                        , 1);
                                }
                                catch
                                {
                                    MessageDlg.ShowInfoDialog("discountが読み込めません。");
                                    return;
                                }
                                inn.Gossips.Add("メレンダ値引き配布済");
                            }
                            break;
                        #endregion   
                            
                        #region ペルラの市
                        case Ikinari.ペルラの市:
                            //ペルラの市
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bgm1.mid";
                            scena.SetFlag("normal", FLAG_TRUE);
                            if (DiceRoll.RandomInt(100) < 80)
                            {
                                scena.SetFlag("master1", FLAG_TRUE);
                                inn.Gossips.Add("＿1on");
                            }
                            if (DiceRoll.RandomInt(100) < 50)
                            {
                                scena.SetFlag("master2", FLAG_TRUE);
                                inn.Gossips.Add("＿2on");
                            }
                            if (DiceRoll.RandomInt(100) < 20)
                            {
                                scena.SetFlag("master3", FLAG_TRUE);
                                inn.Gossips.Add("＿3on");
                            }
                            scena.SetFlag("master4", FLAG_TRUE);
                            break;
                        #endregion   

                        #region アロマ＆ハーブの店
                        case Ikinari.アロマ_A_ハーブの店:
                            //アロマ＆ハーブの店
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "sn.mid";
                            scena.SetFlag("来店", FLAG_TRUE);
                            if (Util1.CheckCouponAll(party, "＿メレンダ数え歌"))
                            {
                                scena.SetFlag("祭り中", FLAG_TRUE);
                            }
                            if (!Util1.CheckCoupon(party, "＿はらへりシスター（お弁当）") &&
                                Util1.CheckCouponAll(party, "＿シスターにおやつを"))
                            {
                                scena.SetFlag("餌付け", FLAG_TRUE);
                            }
                            break;
                        #endregion   

                        #region おべんと屋"のばら"
                        case Ikinari.おべんと屋_B_のばら_B_:
                            //おべんと屋"のばら"
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bgm1.mid";
                            scena.SetFlag("f1", FLAG_TRUE);
                            scena.SetFlag("f2", FLAG_TRUE);
                            if (inn.Gossips.Contains("郵便屋：手紙の行方→のばら") &&
                                !inn.Gossips.Contains("郵便屋：手紙の行方→井戸端"))
                            {
                                scena.SetFlag("letter", FLAG_TRUE);
                            }
                            break;
                        #endregion   

                        #region 雑貨店ナナメナ
                        case Ikinari.雑貨店ナナメナ:
                            //雑貨店ナナメナ
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            scena.BGMFile = "funny.mid";
                            scena.SetFlag("客？", FLAG_TRUE);
                            int alice = Util1.CheckItem(party, "アリス",
                                "名前　Alice SunFlower\r\n妖精人形である。\r\nなんだかよく分からないけど、今の主に仕えることになった人形。\r\n得意な能力は召喚詩歌で\r\n妖精の力を借りれること。\r\nただし性格は可愛くない。\r\nver02");
                            if (alice != -1)
                            {
                                party.Characters[alice].SetCoupon("：アリスと一緒", 0);
                            }
                            break;
                        #endregion

                        #region 枝折屋
                        case Ikinari.枝折屋:
                            //枝折屋
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "gauche.mid";
                            scena.SetFlag("来店", FLAG_TRUE);
                            if (Util1.CheckCoupon(party, "＿枝折屋来店"))
                            {
                                if (Util1.CheckCoupon(party, "＿枝折屋店主お昼寝中"))
                                {
                                    scena.SetFlag("商品表示", FLAG_TRUE);
                                    scena.SetFlag("眠りミア表示", FLAG_TRUE);
                                    scena.SetFlag("ミア眠った？", FLAG_TRUE);
                                }
                                else
                                {
                                    if (DiceRoll.RandomInt(100) < 70)
                                    {
                                        scena.SetFlag("ミア表示", FLAG_TRUE);
                                        scena.SetFlag("商品表示", FLAG_TRUE);
                                    }
                                    else
                                    {
                                        scena.SetFlag("ミア表示", FLAG_TRUE);
                                        scena.SetFlag("商品表示", FLAG_TRUE);
                                        scena.SetFlag("ミア転んだ？", FLAG_TRUE);
                                    }
                                }
                            }
                            else
                            {
                                scena.SetFlag("ミア表示", FLAG_TRUE);
                                scena.SetFlag("商品表示", FLAG_TRUE);
                                Util1.SetCouponAll(party, "＿枝折屋来店", 0);
                            }
                            break;
                        #endregion

                        #region タンスの春木屋
                        case Ikinari.タンスの春木屋:
                            //タンスの春木屋
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "up0211.mid";
                            Util1.SetCouponAll(party, "＿春木屋ご来店", 0);
                            break;
                        #endregion   

                        #region 白鍋通りの話
                        case Ikinari.白鍋通りの話:
                            //白鍋通りの話
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bgm1.mid";
                            scena.SetFlag("normal", FLAG_TRUE);
                            if (Util1.CheckCoupon(party, "＿メレンダ数え歌"))
                            {
                                scena.SetFlag("表示＿ペシュカ", FLAG_TRUE);
                            }
                            if (DiceRoll.RandomInt(100) < 30)
                            {
                                scena.SetFlag("通行人カーミラ", FLAG_TRUE);
                            }

                            scena.SetFlag("冒険者s", FLAG_TRUE);
                            break;
                        #endregion 
                            
                        #region 　 道端で老婆が――
                        case Ikinari._C__Y_道端で老婆が_G__G_:
                            //　 道端で老婆が――
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "白鍋.mid";
                            if (party.Characters.Count == 1)
                            {
                                scena.SetStep("人数", 1);
                            }

                            break;
                        #endregion             
                       
                        #region Millionth Edition
                        case Ikinari.Millionth_Y_Edition:
                            //Millionth Edition
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "snowfield.mid";
                            scena.SetFlag("normal", FLAG_TRUE);
                            if (inn.Gossips.Contains("＠ウロコの指輪"))
                            {
                                if (inn.Gossips.Contains("竜鱗の騎士@MillionthEdition"))
                                {
                                    scena.SetFlag("dragonbook", FLAG_TRUE);
                                }
                            }
                            else
                            {
                                if (inn.Gossips.Contains("竜鱗の騎士@MillionthEdition"))
                                {
                                    scena.SetFlag("フラグ管理めんどい", FLAG_TRUE);
                                }
                            }
                            scena.SetFlag("book-1", FLAG_TRUE);

                            break;
                        #endregion                  

                        #region カフェ-ニャニャ-
                        case Ikinari.カフェ_H_ニャニャ_H_:
                            //カフェ-ニャニャ-
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "up0221.mid";
                            scena.SetFlag("店員", FLAG_TRUE);

                            if (MessageDlg.ShowYesNoDialog("「いらっしゃいませ、\r\nお食事でいらっしゃいますか？」") 
                                == DialogResult.Yes)
                            {
                                scena.SetFlag("めにゅぅ", FLAG_TRUE);
                                scena.SetFlag("べる", FLAG_TRUE);
                                scena.SetFlag("店員", FLAG_FALSE);
                            }
                            else
                            {
                                scena.SetFlag("お持ち帰り", FLAG_TRUE);
                            }
                            break;
                        #endregion             

                        #region 黄金の釜
                        case Ikinari.黄金の釜:
                            //黄金の釜
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "furo_after_doom.mid";
                            scena.SetFlag("f1", FLAG_TRUE);
                            break;
                        #endregion

                        #region 銀のトルトル
                        case Ikinari.銀のトルトル:
                            //銀のトルトル
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bgm1.mid";
                            scena.SetFlag("normal", FLAG_TRUE);
                            break;
                        #endregion      
       
                        #region 色硝子の窓
                        case Ikinari.色硝子の窓:
                            //色硝子の窓
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "frozen-castle.mid";
                            scena.SetFlag("normal", FLAG_TRUE);
                            scena.SetFlag("per", FLAG_TRUE);
                            if (Util1.CheckCoupon(party, "＿旧知@色硝子の窓"))
                            {
                                scena.SetFlag("know", FLAG_TRUE);
                            }
                            break;
                        #endregion                     

                        #region 青果商グラニュート
                        case Ikinari.青果商グラニュート:
                            //青果商グラニュート
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            scena.BGMFile = "sw.mid";
                            if (Util1.CheckCoupon(party, "＿グラニュート製菓"))
                            {
                                Util1.SetCouponAll(party, "＿グラニュート製菓", 0);
                                scena.SetFlag("称号＿製菓", FLAG_TRUE);
                            }
                            else
                            {
                                Util1.SetCouponAll(party, "＿グラニュート青果", 0);
                            }

                            scena.SetFlag("表示＿バクさん", FLAG_TRUE);
                            scena.SetFlag("表示＿店内", FLAG_TRUE);
                            scena.SetFlag("表示＿青果アイテム", FLAG_TRUE);

                            if (Util1.CheckCoupon(party, "＿グラニュート製菓"))
                            {
                                scena.SetFlag("表示＿表裏切替", FLAG_TRUE);
                            }
                            break;
                        #endregion                     
                            
                        #region -茶々-
                        case Ikinari._H_茶々_H_:
                            //-茶々-
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            if (party.Characters.Count == 1)
                            {
                                scena.SetFlag("人数", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "＿御茶屋の約束"))
                            {
                                scena.SetFlag("ハーブ確認", FLAG_TRUE);
                            }
                            break;
                        #endregion 
                        
                        #region 瓶屋
                        case Ikinari.瓶屋:
                            //瓶屋
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bgm.mid";
                            if (!inn.Gossips.Contains("＿瓶屋来店"))
                            {
                                inn.Gossips.Add("＿瓶屋来店");
                            }

                            scena.SetFlag("来店", FLAG_TRUE);

                            if (!inn.Gossips.Contains("＿澄んだ瓶の入手"))
                            {
                                if (inn.Gossips.Contains("＿黒い瓶の噂"))
                                {
                                    if (Util1.CheckItemAll(inn, "黒い瓶", "瓶屋の奥にぽつんと置かれていた黒い小瓶。真っ黒で中に何が入っているのかはわからない。"))
                                    {
                                        scena.SetFlag("黒い瓶", FLAG_FALSE);
                                    }
                                    else
                                    {
                                        scena.SetFlag("黒い瓶", FLAG_TRUE);
                                    }
                                    scena.SetFlag("開花", FLAG_FALSE);
                                }

                            } else {
                                scena.SetFlag("黒い瓶", FLAG_FALSE);
                                scena.SetFlag("開花", FLAG_TRUE);
                            }
                            
                            break;
                        #endregion

                        #region 木苺通りの話
                        case Ikinari.木苺通りの話:
                            //木苺通りの話
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bgm1.mid";
                            scena.SetFlag("normal", FLAG_TRUE);
                            break;
                        #endregion

                        #region かえる屋
                        case Ikinari.かえる屋:
                            //かえる屋
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            if (inn.Gossips.Contains("＿かえる屋来店"))
                            {
                                scena.BGMFile = "kuro.mid";
                                scena.SetFlag("販売中", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\帰る", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\リチャード", FLAG_TRUE);
                            }
                            else
                            {
                                scena.BGMFile = "Uno.mid";
                                scena.SetFlag("表示フラグ\\脱走カエル1", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\脱走カエル2", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\脱走カエル3", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\脱走カエル4", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\脱走カエル5", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\脱走カエル6", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\脱走カエル7", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\リチャード", FLAG_TRUE);
                                scena.SetFlag("表示フラグ\\帰る", FLAG_TRUE);
                            }
                            break;
                        #endregion

                        #region ゲームで一休み
                        case Ikinari.ゲームで一休み:
                            //ゲームで一休み
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            
                            scena.BGMFile = "bs.mid";
                            scena.SetFlag("来店", FLAG_TRUE);
                            break;
                        #endregion

                        #region ゴミの山？宝の山？
                        case Ikinari.ゴミの山_Q_宝の山_Q_:
                            //ゴミの山？宝の山？
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            if (party.Characters.Count == 1)
                            {
                                MessageDlg.ShowInfoDialog("すみませんが一人ではプレイできません。\r\n二人以上でお越しください。");
                                return;
                            }
                            scena.BGMFile = "bs.mid";
                            scena.SetFlag("店番", FLAG_TRUE);
                            break;
                        #endregion

                        #region サンダース農場
                        case Ikinari.サンダース農場:
                            //サンダース農場
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "bs.mid";
                            break;
                        #endregion

                        #region スキル屋ぎょらん
                        case Ikinari.スキル屋ぎょらん:
                            //スキル屋ぎょらん
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            if (inn.Gossips.Contains("＿ぎょらん来店"))
                            {
                                scena.SetFlag("営業中", FLAG_TRUE);
                                scena.SetFlag("おんなのこ", FLAG_TRUE);
                                if (!inn.Gossips.Contains("ぎょらん店主仮死中"))
                                {
                                    scena.SetFlag("店主生存中", FLAG_TRUE);
                                    scena.BGMFile = "bgm.mid";
                                }
                                else
                                {
                                    scena.SetFlag("店主死亡中", FLAG_TRUE);
                                    scena.BGMFile = "DefOver.mid";
                                }
                            }
                            else
                            {
                                scena.SetFlag("店主生存中", FLAG_TRUE);
                                scena.SetFlag("おんなのこ", FLAG_TRUE);
                                scena.BGMFile = "bgm.mid";
                                inn.Gossips.Add("＿ぎょらん来店");
                                scena.SetFlag("営業中", FLAG_TRUE);
                            }
                            break;
                        #endregion

                        #region 音箱
                        case Ikinari.音箱:
                            //音箱
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "迷いの森.mid";
                            if (!Util1.CheckCoupon(party, "＿音箱来訪"))
                            {
                                scena.SetFlag("店主", FLAG_TRUE);
                            }
                            else
                            {
                                scena.SetFlag("店主", FLAG_TRUE);
                                scena.SetFlag("品物出しますか？", FLAG_TRUE);
                                scena.SetFlag("あにろに", FLAG_TRUE);
                            }
                            if (party.Characters.Count == 1)
                            {
                                scena.SetFlag("人数", FLAG_FALSE);
                            }
                            break;
                        #endregion

                        #region 玉猪竜商店
                        case Ikinari.玉猪竜商店:
                            //玉猪竜商店
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "bgm01.mid";
                            scena.SetFlag("表示", FLAG_TRUE);
                           
                            break;
                        #endregion

                        #region A RAG MAN
                        case Ikinari.A_Y_RAG_Y_MAN:
                            //A RAG MAN
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "黒砂糖路地.mid";
                            scena.SetFlag("背景", FLAG_FALSE);
                            scena.SetFlag("背景2", FLAG_TRUE);
                            scena.SetFlag("表示1", FLAG_TRUE);
                            scena.SetFlag("表示2", FLAG_TRUE);
                            break;
                        #endregion  
        
                        #region 傷薬屋さん
                        case Ikinari.傷薬屋さん:
                            //傷薬屋さん
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "bs.mid";
                            break;
                        #endregion

                        #region 路地裏の隠れ酒屋
                        case Ikinari.路地裏の隠れ酒屋:
                            //路地裏の隠れ酒屋
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bs.mid";
                            if (Util1.CheckCoupon(party, "隠れ酒屋/来店済"))
                            {
                                if (Util1.CheckCoupon(party, "隠れ酒屋の裏メニュー"))
                                {
                                    scena.SetFlag("隠し商品", FLAG_FALSE);
                                    scena.SetFlag("基本商品", FLAG_TRUE);
                                    scena.SetFlag("商品切り替え", FLAG_TRUE);
                                }
                                else
                                {
                                    scena.SetFlag("隠し商品", FLAG_FALSE);
                                    scena.SetFlag("基本商品", FLAG_TRUE);
                                    scena.SetFlag("商品切り替え", FLAG_FALSE);
                                }
                            }
                            else
                            {
                                Util1.SetCouponAll(party, "隠れ酒屋/来店済", 0);
                            }
                            break;
                        #endregion
                            
                        #region 黒砂糖路地の話
                        case Ikinari.黒砂糖路地の話:
                            //黒砂糖路地の話
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "bgm1.mid";
                            if (!inn.Gossips.Contains("レナード死亡"))
                            {
                                if (!inn.Gossips.Contains("レナードと会話（黒砂糖路地の話）"))
                                {
                                    if (party.Characters.Count >= 2)
                                    {
                                        scena.SetFlag("表示：レナード", FLAG_TRUE);
                                    }
                                }
                            }

                            scena.SetFlag("小此木\\逆さ吊りの人", FLAG_TRUE);
                            if (DiceRoll.RandomInt(100) < 40)
                            {
                                scena.SetFlag("bet", FLAG_TRUE);
                            }

                            scena.SetFlag("normal", FLAG_TRUE);
                            scena.SetFlag("擬態", FLAG_TRUE);
                            scena.SetFlag("trap", FLAG_TRUE);

                            if (Util1.CheckCoupon(party, "＿メレンダ数え歌"))
                            {
                                scena.SetFlag("灰色少年＿表示", FLAG_TRUE);
                            }
                            break;
                        #endregion  
                            
                        #region メレンダ祭り準備中
                        case Ikinari.メレンダ祭り準備中:
                            //メレンダ祭り準備中
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "白鍋通り.mid";
                            scena.SetFlag("背景", FLAG_FALSE);
                            scena.SetFlag("背景2", FLAG_TRUE);
                            scena.SetFlag("表示1", FLAG_TRUE);

                            break;
                        #endregion  
                         
                        #region メレンダ祭り開催中
                        case Ikinari.メレンダ祭り開催中:
                            //メレンダ祭り開催中
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "白鍋通り.mid";
                            if (party.Characters.Count >= 2)
                            {
                                scena.SetStep("人数分岐", 1);
                            }
                            scena.SetFlag("背景", FLAG_FALSE);
                            scena.SetFlag("背景2", FLAG_TRUE);
                            scena.SetFlag("吟遊詩人", FLAG_TRUE);

                            break;
                        #endregion

                        #region メレンダ数え歌会議
                        case Ikinari.メレンダ数え歌会議:
                            //メレンダ数え歌会議
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            if (!Util1.CheckCoupon(party, "＿メレンダ数え歌"))
                            {
                                MessageDlg.ShowInfoDialog("このシナリオは『メレンダ数え歌』の捜索依頼を受けた\r\n" +
                                                          "冒険者が一人でもパーティ内に居ることが開始条件です。");
                                return;
                            }
                            scena.BGMFile = "town8.mid";
                            if (!Util1.CheckCoupon(party, "＿数え歌会議"))
                            {
                                foreach (CharacterData cd in party.Characters)
                                {
                                    if (cd.CheckCoupon("＿吟遊詩人") ||
                                       cd.CheckCoupon("吟遊詩人") ||
                                       cd.CheckCoupon("歌い手"))
                                    {
                                        cd.SetCoupon("歌：メレンダ", 0);
                                        scena.SetFlag("詩人居る？", FLAG_FALSE);
                                    }
                                }
                                Util1.SetCouponAll(party, "＿数え歌会議", 0);
                            }
                            else
                            {
                                if (Util1.CheckCoupon(party, "＿アルグレバム面識"))
                                {
                                    scena.AddStep("通行人ステップ");
                                    if (Util1.CheckCoupon(party, "＿二人の地元民"))
                                    {
                                        scena.AddStep("通行人ステップ");
                                        if (Util1.CheckCoupon(party, "＿夫婦に聞き込み"))
                                        {
                                            scena.AddStep("通行人ステップ");
                                        }
                                    }
                                }
                            }

                            break;
                        #endregion      
                            
                        #region 希望の都 「Ｆ=ベル」
                        case Ikinari.希望の都_Y__L_Ｆ_Z_ベル_N_:

                            //希望の都 「Ｆ=ベル」
                            Util3.SetInitScenarioData(scena, sd);
                            
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            // 判定
                            if (inn.CurrentParty.Characters.Count > 1)
                            {
                                scena.SetStep("複数形", 1);
                            }

                            if (Util1.CheckCoupon(party, "Ｆ＝ベル全依頼達成"))
                            {
                                scena.SetFlag("表示・おまけの部屋", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "＿免許皆伝候補"))
                            {
                                scena.SetFlag("ローズレイク奥義継承", FLAG_TRUE);
                            }
                            if (isVariant128())
                            {
                                scena.SetFlag("バージョン1.28", FLAG_TRUE);
                            }
                            if (inn.Gossips.Contains("＿セラヴィ見破る"))
                            {
                                scena.SetFlag("セラヴィ見破る", FLAG_TRUE);
                                scena.SetFlag("ウイングとセラヴィ", FLAG_TRUE);
                                scena.SetFlag("セラヴィの秘密", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "ウルフ退治（F=ベル）"))
                            {
                                scena.SetFlag("鐘\\依頼・ウルフ", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "ゴブリン退治（F=ベル）"))
                            {
                                scena.SetFlag("鐘\\依頼・ゴブリン", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "海賊退治（F=ベル）"))
                            {
                                scena.SetFlag("鐘\\依頼・海賊", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "オーガ退治（F=ベル）"))
                            {
                                scena.SetFlag("鐘\\依頼・オーガ", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "バジリスク退治（F=ベル）"))
                            {
                                scena.SetFlag("呼び鈴\\依頼・バジリスク", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "ダークエルフ退治（F=ベル）"))
                            {
                                scena.SetFlag("呼び鈴\\依頼・グルネル", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "バグベアード退治（F=ベル）"))
                            {
                                scena.SetFlag("呼び鈴\\依頼・バグベアード", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "巨人退治（F=ベル）"))
                            {
                                scena.SetFlag("呼び鈴\\依頼・巨人", FLAG_TRUE);
                            }
                            if (inn.Gossips.Contains("フォーチュン＝ベル完全攻略"))
                            {
                                scena.SetFlag("おまけ\\完全攻略", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "ローズレイク流免許皆伝"))
                            {
                                scena.SetFlag("ローズレイク免許皆伝", FLAG_TRUE);
                            }
                            if (Util1.CheckCoupon(party, "＿レフの薬"))
                            {
                                scena.SetFlag("レフの薬", FLAG_TRUE);
                            }
                            
                            scena.BGMFile = "BeyondMyHeart.mid";

                            break;

                        #endregion             

                        #region TSUKE CLICKER
                        case Ikinari.TSUKE_Y_CLICKER:

                            //TSUKE CLICKER
                            Util3.SetInitScenarioData(scena, sd);
                            
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "bgm_maoudamashii_orchestra08.mp3";
                            scena.SetFlag("新しいフォルダ (2)\\1桁", FLAG_TRUE);

                            #region 背景描画
                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "a.png";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 500;
                            cell.Top = 0;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"失った時間:$クリック数\05_00,000$$クリック数\04_0,000$,$クリック数");
                            cell.DispText.Add(@"\03_000$$クリック数\02_00$$クリック数\01_0$click");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x04;
                            cell.FlagName = @"クリック数\ケタ数_5";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 507;
                            cell.Top = 0;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"失った時間:$クリック数\04_0,000$,$クリック数\03_000$$クリック数\02_00");
                            cell.DispText.Add(@"$$クリック数\01_0$click");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x04;
                            cell.FlagName = @"クリック数\ケタ数_4";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 518;
                            cell.Top = 0;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"失った時間:$クリック数\03_000$$クリック数\02_00$$クリック数\01_0$click");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x04;
                            cell.FlagName = @"クリック数\ケタ数_3";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 525;
                            cell.Top = 0;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"失った時間:$クリック数\02_00$$クリック数\01_0$click");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x04;
                            cell.FlagName = @"クリック数\ケタ数_2";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 532;
                            cell.Top = 0;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"失った時間:$クリック数\01_0$click");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x04;
                            cell.FlagName = @"クリック数\ケタ数_1";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 2;
                            cell.Top = 10;
                            cell.Width = 58;
                            cell.Height = 154;
                            cell.FileName = "b.png";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 0;
                            cell.Top = 13;
                            cell.Width = 216;
                            cell.Height = 36;
                            cell.FileName = @"jpy\1.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = @"z\酒豪";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 0;
                            cell.Top = 51;
                            cell.Width = 216;
                            cell.Height = 36;
                            cell.FileName = @"jpy\2.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = @"z\馬鹿騒ぎ";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 0;
                            cell.Top = 89;
                            cell.Width = 216;
                            cell.Height = 36;
                            cell.FileName = @"jpy\3.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = @"z\今日も休み";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 0;
                            cell.Top = 127;
                            cell.Width = 216;
                            cell.Height = 36;
                            cell.FileName = @"jpy\4.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = @"z\親父が嫌い";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = "";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 235;
                            cell.Top = 35;
                            cell.Width = 391;
                            cell.Height = 50;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = "";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 400;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\1桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 395;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\2桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 390;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\000$$1\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\3桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 380;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\0000$,$1\000$$1\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\4桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 374;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\00000$$1\0000$,$1\000$$1\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\5桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 369;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\000000$$1\00000$$1\0000$,$1\000$$1\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\6桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 362;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\0000000$,$1\000000$$1\00000$$1\0000$,$1\000$$1\00$$1\0$ tsuke ");
                            cell.DispText.Add(@"sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\7桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 356;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\00000000$$1\0000000$,$1\000000$$1\00000$$1\0000$,$1\000$$1");
                            cell.DispText.Add(@"\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\8桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 350;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\000000000$$1\00000000$$1\0000000$,$1\000000$$1\00000$$1");
                            cell.DispText.Add(@"\0000$,$1\000$$1\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\9桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 342;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\0000000000$,$1\000000000$$1\00000000$$1\0000000$,$1\000000");
                            cell.DispText.Add(@"$$1\00000$$1\0000$,$1\000$$1\00$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\10桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 335;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\00000000000$$1\0000000000$,$1\000000000$$1\00000000$$1");
                            cell.DispText.Add(@"\0000000$,$1\000000$$1\00000$$1\0000$,$1\000$$1\00$$1\0$ tsuke ");
                            cell.DispText.Add(@"sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\11桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 331;
                            cell.Top = 46;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$1\000000000000$$1\00000000000$$1\0000000000$,$1\000000000$$1");
                            cell.DispText.Add(@"\00000000$$1\0000000$,$1\000000$$1\00000$$1\0000$,$1\000$$1\00");
                            cell.DispText.Add(@"$$1\0$ tsuke sp");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 18;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"新しいフォルダ (2)\12桁";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 40;
                            cell.Top = 194;
                            cell.Width = 305;
                            cell.Height = 83;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"TSUKE CLICKER");
                            cell.DispText.Add(@"クリックでツケを");
                            cell.DispText.Add(@"貯めよう！");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 19;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = null;
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"z\酒豪";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"z\今日も休み";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"z\親父が嫌い";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 70;
                            cell.Top = 193;
                            cell.Width = 200;
                            cell.Height = 26;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"ツケ増量中");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 16;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"z\酒豪";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 76;
                            cell.Top = 222;
                            cell.Width = 200;
                            cell.Height = 26;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"高確率中");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 16;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"z\今日も休み";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 76;
                            cell.Top = 222;
                            cell.Width = 200;
                            cell.Height = 26;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"倍率上昇中");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 16;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"z\親父が嫌い";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"説明1";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 9;
                            cell.Top = 193;
                            cell.Width = 305;
                            cell.Height = 83;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"酒豪");
                            cell.DispText.Add(@"金のない酒好き仲間を呼び込む。");
                            cell.DispText.Add(@"クリック時のツケの増加量が上昇。");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 14;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"説明1";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"説明2";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 9;
                            cell.Top = 193;
                            cell.Width = 305;
                            cell.Height = 83;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"馬鹿騒ぎ");
                            cell.DispText.Add(@"皆で酒を飲み狂い、騒ぐ。");
                            cell.DispText.Add(@"一度にツケ金を増やせる。");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 14;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"説明2";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"説明3";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 9;
                            cell.Top = 193;
                            cell.Width = 305;
                            cell.Height = 83;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"今日も休日");
                            cell.DispText.Add(@"昨日も今日も明日もお休み。");
                            cell.DispText.Add(@"酒を飲んだり騒いだりするには");
                            cell.DispText.Add(@"うってつけ。");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 14;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"説明3";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 5;
                            cell.Top = 191;
                            cell.Width = 222;
                            cell.Height = 90;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"説明4";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 9;
                            cell.Top = 193;
                            cell.Width = 305;
                            cell.Height = 83;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"親父が嫌い");
                            cell.DispText.Add(@"その嫌悪感がツケの貯まるｽﾋﾟｰﾄﾞ");
                            cell.DispText.Add(@"を加速させる。");
                            cell.DispText.Add(@"全てのツケ金に倍率補正。");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 14;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"説明4";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 116;
                            cell.Top = 15;
                            cell.Width = 400;
                            cell.Height = 260;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"ダスキン\ダスキン";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 130;
                            cell.Top = 27;
                            cell.Width = 200;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"差し入れ量");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 21;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"ダスキン\ダスキン";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 65;
                            cell.Top = 51;
                            cell.Width = 500;
                            cell.Height = 200;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0x00;
                            cell.Argb = 0xC8000000;
                            cell.FlagName = @"z\馬鹿騒ぎ";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 130;
                            cell.Top = 31;
                            cell.Width = 87;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"+$酒宴\9桁(2)$$酒宴\8桁(2)$$酒宴\7桁(2)$,000,000");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"酒宴\レベル3";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 130;
                            cell.Top = 31;
                            cell.Width = 87;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"+$酒宴\6桁(2)$$酒宴\5桁(2)$$酒宴\4桁(2)$,000");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"酒宴\レベル2";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 130;
                            cell.Top = 31;
                            cell.Width = 87;
                            cell.Height = 50;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"+$酒宴\3桁(2)$$酒宴\2桁(2)$$酒宴\1桁(2)$");
                            cell.FontName = "メイリオ";
                            cell.FontSize = 12;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x00;
                            cell.FlagName = @"酒宴\レベル1";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = @"参戦\kananngorannsinn.png";
                            cell.MaskOn = false;
                            cell.FlagName = @"カナン\カナンモード";
                            cell.Visible = true;
                            scena.Cells.Add(cell);
                            #endregion 

                            break;
                        #endregion             

                        #region ある小さなお店
                        case Ikinari.ある小さなお店:

                            //ある小さなお店
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            if (isVariant128())
                            {
                                scena.SetFlag("Ver1.28", FLAG_TRUE);
                            }

                            scena.BGMFile = "Black_Ive_demon.mid";

                            scena.SetFlag("カード枠表示", FLAG_TRUE);
                            scena.SetFlag("カード表示", FLAG_TRUE);

                            if (Util1.CheckCoupon(party, "Ｆ＝ベル全依頼達成"))
                            {
                                scena.SetFlag("カード表示(2)", FLAG_TRUE);
                            }

                            break;

                        #endregion

                        #region 蚤の市
                        case Ikinari.蚤の市:

                            //蚤の市
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            scena.BGMFile = "Tapo.mid";
                            scena.SetFlag("暫定メニュー", FLAG_TRUE);

                            break;

                        #endregion

                        #region 交易都市リューン#Cecil
                        case Ikinari.交易都市リューン_S_Cecil:

                            //交易都市リューン#Cecil
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            if (DiceRoll.RandomInt(100) < 10)
                            {
                                scena.SetFlag("sys_\\02Rare在庫", FLAG_TRUE);
                            }
                            if (Util1.CheckSkillAll(inn, "氷柱の槍", "レベル１技能カード（呪文）。指先より魔法の光線を発し、敵一体を攻撃する。この光線は目標の温度を急激に下げる効力を持ち、生命体に対しては凍傷によるダメージを与える事が出来る。") ||
                               Util1.CheckSkillAll(inn, "風の刃", "レベル１技能カード（呪文）。一閃させた掌より真空の刃を発し、敵一体を切り裂く攻撃魔法。実体を持たない敵に対しては効果を発揮しないものの、大量の出血を伴うため血肉を持つ目標に対しては特に効果が大きい。その使用には知力と好戦さが要求される。") ||
                               Util1.CheckSkillAll(inn, "風刃乱舞", "レベル７技能カード（呪文）。一閃させた掌より無数の真空の刃を発し、四方の敵を切り裂く攻撃魔法。実体を持たない敵に対しては効果を発揮しないものの、大量の出血を伴うため血肉を持つ目標に対しては特に効果が大きい。その使用には知力と好戦さが要求される。"))
                            {
                                scena.SetFlag("sys_\\02Rare技能", FLAG_TRUE);
                            }
                            scena.SetFlag("sys_\\00背景表示", FLAG_TRUE);
                            if (Util1.CheckItemAll(inn, "賢者の杖", "アイテムカード（道具）。特殊な魔法の施された古杖。所有者の集中力を高める力があり、戦闘中に使用すると次ラウンドに技能カードが配布される。ただし、技能カードを所有しない、あるいは使用残数が尽きている場合には効果を発揮しない。") ||
                                Util1.CheckItemAll(inn, "隠者の杖", "アイテムカード（道具）。特殊な魔法の施された古杖。所有者の集中力を高める力があり、戦闘中に使用すると次ラウンドに技能カードが配布される。ただし、技能カードを所有しない、あるいは使用残数が尽きている場合には効果を発揮しない。") ||
                                Util1.CheckItemAll(inn, "ｶﾅﾝの魔剣", "アイテムカード（武器）。刀身１ｍを超える両手持ちの大剣。淡く青白い光を放つ刀身は冷気をまとい、斬りつけた相手に凍傷を負わせる事が出来る。旧文明期に滅びし王国ソドムの最期の王カナンが愛用した魔法の剣。") ||
                                Util1.CheckItemAll(inn, "ｶﾅﾝの鎧", "アイテムカード（防具）。旧文明期の王国ソドム最期の王カナンの鎧。強力な魔力が秘められており、防御力はもちろん、身に付けた者の魔法に対する抵抗力を増大させる力を持つ。") ||
                                Util1.CheckItemAll(inn, "魔法の護符", "アイテムカード（防具）。銀の円盤に魔法陣を描いたお守り。所有者の魔法に対する抵抗力を上昇させる魔法が施されている。持っているだけで効果を発揮するが、戦闘中に使用すると、そのラウンドの抵抗力は飛躍的に増大する事だろう。"))
                            {
                                scena.SetFlag("sys_\\02RareItem", FLAG_TRUE);
                            }

                            scena.SetFlag("sys_\\00メニューの表示", FLAG_TRUE);

                            if (inn.Gossips.Contains("Cecil_R"))
                            {
                                if (DiceRoll.RandomInt(100) < 34)
                                {
                                    scena.BGMFile = "Ryune.mid";
                                }
                                else
                                {
                                    if (DiceRoll.RandomInt(100) < 51)
                                    {
                                        scena.BGMFile = "Bgm_shimo_night_hymn.mid";
                                    }
                                    else
                                    {
                                        scena.BGMFile = "Bgm_gjm_019v100.mid";
                                    }
                                }
                            }
                            else if (inn.Gossips.Contains("Cecil_HM"))
                            {
                                scena.BGMFile = "Bgm_shimo_night_hymn.mid";
                            }
                            else if (inn.Gossips.Contains("Cecil_MS"))
                            {
                                scena.BGMFile = "Bgm_gjm_019v100.mid";
                            }
                            else
                            {
                                scena.BGMFile = "Ryune.mid";
                            }

                            break;

                        #endregion

                        #region 交易都市リューン#C2
                        case Ikinari.交易都市リューン_S_C2:

                            //交易都市リューン#C2
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            if (DiceRoll.RandomInt(100) < 10)
                            {
                                scena.SetFlag("sys_\\02Rare在庫", FLAG_TRUE);
                            }
                            if (Util1.CheckSkillAll(inn, "氷柱の槍", "レベル１技能カード（呪文）。指先より魔法の光線を発し、敵一体を攻撃する。この光線は目標の温度を急激に下げる効力を持ち、生命体に対しては凍傷によるダメージを与える事が出来る。") ||
                               Util1.CheckSkillAll(inn, "風の刃", "レベル１技能カード（呪文）。一閃させた掌より真空の刃を発し、敵一体を切り裂く攻撃魔法。実体を持たない敵に対しては効果を発揮しないものの、大量の出血を伴うため血肉を持つ目標に対しては特に効果が大きい。その使用には知力と好戦さが要求される。") ||
                               Util1.CheckSkillAll(inn, "風刃乱舞", "レベル７技能カード（呪文）。一閃させた掌より無数の真空の刃を発し、四方の敵を切り裂く攻撃魔法。実体を持たない敵に対しては効果を発揮しないものの、大量の出血を伴うため血肉を持つ目標に対しては特に効果が大きい。その使用には知力と好戦さが要求される。"))
                            {
                                scena.SetFlag("sys_\\02Rare技能", FLAG_TRUE);
                            }
                            scena.SetFlag("sys_\\00背景表示", FLAG_TRUE);
                            scena.SetFlag("sys_\\00メニューの表示", FLAG_TRUE);

                            if (inn.Gossips.Contains("Cecil_R"))
                            {
                                if (DiceRoll.RandomInt(100) < 34)
                                {
                                    scena.BGMFile = "Ryune.mid";
                                }
                                else
                                {
                                    if (DiceRoll.RandomInt(100) < 51)
                                    {
                                        scena.BGMFile = "Bgm_shimo_night_hymn.mid";
                                    }
                                    else
                                    {
                                        scena.BGMFile = "Bgm_gjm_019v100.mid";
                                    }
                                }
                            }
                            else if (inn.Gossips.Contains("Cecil_HM"))
                            {
                                scena.BGMFile = "Bgm_shimo_night_hymn.mid";
                            }
                            else if (inn.Gossips.Contains("Cecil_MS"))
                            {
                                scena.BGMFile = "Bgm_gjm_019v100.mid";
                            }
                            else
                            {
                                scena.BGMFile = "Ryune.mid";
                            }

                            break;

                        #endregion

                        #region 古城の老兵
                        case Ikinari.古城の老兵:
                            //古城の老兵
                            Util3.SetInitScenarioData(scena, sd);

                            if (inn.Gossips.Contains("古城の老兵面識"))
                            {
                                Util3.SetAreaInitCell(scena, scenaFolder, 1);
                                scena.BGMFile = "tamhe07.mid";
                                scena.SetFlag("イベント\\真っ黒", FLAG_TRUE);
                                if (inn.Gossips.Contains("ｓ古城【埋葬の剣】"))
                                {
                                    scena.SetFlag("隠・埋葬の剣", FLAG_TRUE);
                                } 
                            }
                            else
                            {
                                Util3.SetAreaInitCell(scena, scenaFolder, 2);
                                scena.BGMFile = "tz・wind.MID";
                                scena.SetFlag("イベント\\城遠景", FLAG_TRUE);
                            }
                            break;

                        #endregion

                        #region 旧街道
                        case Ikinari.旧街道:
                            //旧街道
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 8);
                            scena.BGMFile = "「馬車に揺られて」Ｖ.mid";
                            break;

                        #endregion

                        #region 異端なる魔剣の入手法
                        case Ikinari.異端なる魔剣の入手法:
                            //異端なる魔剣の入手法
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "「尖塔の姫君」ラプンツェル.mid";
                            if (!inn.Gossips.Contains("異端来店"))
                            {
                                inn.Gossips.Add("異端来店");
                            }
                            break;

                        #endregion

                        #region 武具・「夕日の鉄撃」
                        case Ikinari.武具_D__L_夕日の鉄撃_N_:
                            //武具・「夕日の鉄撃」
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "「ジャズ」おいらはみなしご.mid";
                            if (!inn.Gossips.Contains("夕日の鉄撃来店"))
                            {
                                inn.Gossips.Add("夕日の鉄撃");
                            }
                            break;

                        #endregion  
                            
                        #region アタノオル
                        case Ikinari.アタノオル:
                            //アタノオル
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "「真理探究者」リイ.mid";
                            if (inn.Gossips.Contains("錬金術・風の刃"))
                            {
                                scena.SetFlag("Ask技能・風の刃", FLAG_TRUE);
                            }
                            if (inn.Gossips.Contains("錬金術・風刃乱舞"))
                            {
                                scena.SetFlag("Ask技能・風刃乱舞", FLAG_TRUE);
                            }
                            break;

                        #endregion

                        #region 調合屋ケムス
                        case Ikinari.調合屋ケムス:

                            //調合屋ケムス
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            scena.SetFlag("ネイ", FLAG_TRUE);
                            scena.SetFlag("調合屋", FLAG_TRUE);

                            if (Util1.CheckCoupon(party, "調合屋 － 顧客"))
                            {
                                scena.SetFlag("ケムス", FLAG_TRUE);
                            }
                            else
                            {
                                scena.SetFlag("ネイ", FLAG_FALSE);
                                scena.SetFlag("ケムス", FLAG_TRUE);
                            }
                            break;

                        #endregion

                        #region ｱｲﾃﾑｼｮｯﾌﾟ仔馬の尻尾
                        case Ikinari.ｱｲﾃﾑｼｮｯﾌﾟ仔馬の尻尾:

                            //ｱｲﾃﾑｼｮｯﾌﾟ仔馬の尻尾
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "Coffee.mid";
                            scena.SetFlag("表示", FLAG_TRUE);

                            break;

                        #endregion
                            
                        #region 暗黒街ペテンザム
                        case Ikinari.暗黒街ペテンザム:

                            //暗黒街ペテンザム
                            if (party.Characters.Count < 2)
                            {
                                MessageDlg.ShowInfoDialog(
                                    "しかし" + party.Characters[0].CardName 
                                    + "は暗黒街の夜に\r\n" +
                                  "一人で散策するのは危険すぎると判断し、\r\n" +
　　                                "一晩泊まって" + inn.Name + "に" +
　　                                "帰ることにした。");
                                return;
                            }

                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "horror04.mid";
                            scena.SetFlag("商品閲覧", FLAG_FALSE);
                            scena.SetFlag("商品を見る", FLAG_TRUE);
                            scena.SetFlag("千里眼・阿修羅", FLAG_FALSE);
                            scena.SetFlag("剣術・魔法", FLAG_FALSE);
                            scena.SetFlag("忍術・アイテム", FLAG_FALSE);
                            scena.SetFlag("格闘術", FLAG_FALSE);
                            if (Util1.CheckCoupon(party, "＿ドクター"))
                            {
                                scena.SetFlag("ドクター", FLAG_FALSE);
                            }

                            break;

                        #endregion

                        #region ゴブリンの収集
                        case Ikinari.ゴブリンの収集:

                            //ゴブリンの収集
                            MessageDlg.ShowInfoDialog(
                                "ペンギン兵を持っていると\r\n" +
                              "LV5以上で低確率で戦闘がありますが、\r\n" +
                              "省略します。");

                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            scena.SetFlag("遠征", FLAG_TRUE);
                            Util1.SetCouponAll(party, "ゴブリンの収集", 0);

                            scena.BGMFile = "CITY03.mid";

                            if (Util1.CheckCoupon(party, "七男との対面"))
                            {
                                scena.SetFlag("七男目覚め", FLAG_TRUE);
                                scena.SetFlag("七男眠り", FLAG_FALSE);
                            }
                            else
                            {
                                scena.SetFlag("七男目覚め", FLAG_FALSE);
                                scena.SetFlag("七男眠り", FLAG_TRUE);
                            }

                            break;

                        #endregion

                        #region 新装開店！（君が夢，海を越えて）
                        case Ikinari.新装開店_F_:
                            //君が夢，海を越えて
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 18);

                            if (Util1.CheckCoupon(party, "赤い爪を撃退") 
                                || DiceRoll.RandomInt(100) < 80)
                            {
                                scena.BGMFile = "Ryune.mid";
                                scena.SetFlag("町", FLAG_TRUE);
                            }
                            else
                            {
                                scena.BGMFile = "DefBattle.mid";
                                scena.SetFlag("非常事態", FLAG_TRUE);
                            }

                            break;

                        #endregion

                        #region 釣りに行こうっ!!
                        case Ikinari.釣りに行こうっ_E__E_:

                            //最初のおやじの自慢話をすっとばす！
                            if (!Util1.CheckCoupon(party, "釣りの穴場"))
                            {
                                //パーティー全員初めて

                                if (
                                !Util1.CheckItemAll(inn, "釣り竿",
"　なぜか洞窟内に落ちていた釣り竿。　　　……よく見ると、いい仕事しているねぇ～な一品。名人甚五郎作「餌いらず」の銘がうってある。　この竿を使って、魚を釣れないわけはないだろう……。　　　　　　釣りの基本は、じっくり我慢できる精神力と、平和な心が必要。") 
                                    &&
                                !Util1.CheckItemAll(inn, "ルアーロッド",
"　投げ釣りが可能になった釣り竿。しかしレベル４以上の熟練さ必要。主に近場と中くらいの釣り場に最適。遠投も可能だが、狙った所へ飛ばないことが多い。")
                                    &&
                                !Util1.CheckItemAll(inn, "グラスロッド",
"　遠投用の釣り竿。遠くへ仕掛けを飛ばすには、熟練した技(レベル６以上が望ましいと、筋力が必要になる。しかし、遠くに仕掛けを飛ばすことは可能だが、近場の釣りには向いていない。"))
                                {
                                    //釣り道具がない
                                    WidCard tsuriniTsurizao = null;
                                    
                                    try{
                                        tsuriniTsurizao 
                                            = new WidCard(WidIDtype.アイテム);

                                        //Version2(エンジン1.15用)データでは
                                        //シナリオ名、作者名はカード情報に出ないし、
                                        //使用時イベントが無い。
                                        //バイト数から考えるとホールドも無し
                                        //対応させるためにWidCard.csを修正
                                        tsuriniTsurizao.Read(scenaFolder +
                                            "\\Item1.wid");
                                    }
                                    catch
                                    {
                                        MessageDlg.ShowInfoDialog("釣り竿が読み込めません。");
                                        return;
                                    }

                                    //釣竿Get
                                    inn.AddRenamedCardData(party.BagCards,
                                        tsuriniTsurizao, 1);
                                          
                                }

                                Util1.SetCouponAll(party, "釣りの穴場", 0);
                            }
                            
                            
                            
                            scena.AreaNumber = 4;

                            scena.StepValues.Add("釣り熟練度=0");

                            scena.FlagValues.Add("大きな魚=0");
                            scena.FlagValues.Add("釣りからの帰還=1");
                            scena.FlagValues.Add("釣り竿=0");
                            scena.FlagValues.Add("ルアーロッド=0");
                            scena.FlagValues.Add("グラスロッド=0");

                            if (!Util1.CheckCoupon(party, "釣り吉") ||
                                Util1.CheckCoupon(party, "釣り吉三筒と勝負"))
                            {
                                scena.FlagValues.Add("釣り人=1");
                                scena.FlagValues.Add("釣り人A=0");
                            }
                            else
                            {
                                scena.FlagValues.Add("釣り人=0");
                                scena.FlagValues.Add("釣り人A=1");
                            }

                            scena.FlagValues.Add("魚魚震=0");
                            scena.FlagValues.Add("釣り人A(2)=0");
                            scena.FlagValues.Add("会話=1");
                            scena.FlagValues.Add("勝負の釣り場=0");
                            scena.FlagValues.Add("ブルー=0");
                            scena.FlagValues.Add("グラス=0");
                            scena.FlagValues.Add("ペンギン兵=0");
                            scena.FlagValues.Add("解説兄さん=0");
                            scena.FlagValues.Add("ペンギンイベント=1");
                            scena.FlagValues.Add("レッド=0");
                            scena.FlagValues.Add("グリーン=0");
                            scena.FlagValues.Add("ピンク=0");
                            scena.FlagValues.Add("イエロー=0");

                            scena.BGMFile = "砂塵.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "MapOfWirth.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 300;
                            cell.FileName = "harbor.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            break;

                        #endregion

                        #region 幸せ運ぶお店です。1.4
                        case Ikinari.幸せ運ぶお店です_M_:

                            //幸せ運ぶお店です。
                            Util3.SetInitScenarioData(scena, sd);

                            if (inn.Gossips.Contains("カリビアGET"))
                            {
                                scena.SetFlag("看板娘", FLAG_FALSE);
                                scena.SetFlag("卿", FLAG_TRUE);
                            }

                            bool isFirst = false;
                            if (!Util1.CheckCoupon(party, "「卿のお家」"))
                            {
                                if (!Util1.CheckCoupon(party, "通な客"))
                                {
                                    isFirst = true;
                                }
                            }

                            if (isFirst)
                            {
                                Util3.SetAreaInitCell(scena, scenaFolder, 3);
                                Util1.SetCouponAll(party, "「卿のお家」", 0);
                            }
                            else
                            {
                                Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            }

                            scena.BGMFile = "cw2.mid";
                            scena.SetStep("いらっしゃーい♪", 1);

                            break;

                        #endregion

                        #region 天空都市タラント
                        case Ikinari.天空都市タラント:

                            //天空都市タラント
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            if (Util1.CheckCoupon(party, "物資搬入警護依頼終了"))
                            {
                                scena.SetFlag("依頼用\\物資搬入警護依頼", FLAG_FALSE);
                                
                            }
                            if (!Util1.CheckCoupon(party, "天空都市タラント"))
                            {
                                Util1.SetCouponAll(party, "天空都市タラント", 0);
                            }
                            scena.SetStep("基本\\タラント到着", 1);

                            break;

                        #endregion

                        #region 斜陽の魔法店
                        case Ikinari.斜陽の魔法店:

                            //斜陽の魔法店
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            if (!inn.Gossips.Contains("必要な店_来店"))
                            {
                                inn.Gossips.Add("必要な店_来店");
                            }
                            else
                            {
                                if (Util1.CheckCoupon(party, "アルルカ"))
                                {
                                    MessageDlg.ShowInfoDialog(
                                        "アルルカをパーティから外して、\r\n再度入店してください。");
                                    return;
                                }

                                if (inn.Gossips.Contains("必要な店_アルルカ購入"))
                                {
                                    scena.SetFlag("アルルカ購入可能", FLAG_FALSE);
                                    scena.SetFlag("アルルカコメント", FLAG_TRUE);
                                }
                            }

                            scena.SetFlag("品物表示", FLAG_TRUE);
                            if (inn.Gossips.Contains("必要な店_Csk026購入"))
                            {
                                scena.SetFlag("品物表示(3)", FLAG_TRUE);
                            }
                            else
                            {
                                scena.SetFlag("品物表示(2)", FLAG_TRUE);
                            }

                            scena.BGMFile = "BeyondMyHeart.mid";
                            break;

                        #endregion

                        #region 魔道具バザー
                        case Ikinari.魔道具バザー:

                            //魔道具バザー
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = "give_take.mid";

                            scena.SetFlag("表示（キャスト）", FLAG_TRUE);

                            if (!inn.Gossips.Contains("＿魔道具バザーに行かなかった"))
                            {
                                inn.CurrentParty.Money += 1000;
                            }
                            else
                            {
                                inn.Gossips.Remove("＿魔道具バザーに行かなかった");
                            }

                            break;

                        #endregion

                        #region ムーランルージュ
                        case Ikinari.ムーランルージュ:

                            //ムーランルージュ
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            scena.BGMFile = "展覧会の絵.mid";

                            Util1.RemoveCouponAll(party, "＿桂冠詩人失格");
                            break;

                        #endregion

                        #region お寿司始めました
                        case Ikinari.お寿司始めました:

                            //お寿司始めました
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            if (DiceRoll.RandomInt(100) < 40)
                            {
                                scena.SetFlag("謎の商品表示", FLAG_TRUE);
                            }
                            break;

                        #endregion    
                            
                        #region くじびき屋
                        case Ikinari.くじびき屋:

                            //くじびき屋
                            Util3.SetInitScenarioData(scena, sd);

                            if (inn.Gossips.Contains("＿くじびき屋おまけ"))
                            {
                                MessageDlg.ShowInfoDialog(
                                    "オマケモードになっているので\r\nいきなりを使わずに起動してください。");
                                return;
                            }

                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            scena.SetFlag("めにう", FLAG_TRUE);
                            scena.SetFlag("少女", FLAG_FALSE);

                            if (Util1.CheckCoupon(party, "＿くじびきスキップ"))
                            {
                                scena.SetFlag("スキップ", FLAG_TRUE);
                            }
                            else
                            {
                                if (!Util1.CheckCoupon(party, "くじびき屋の客"))
                                {
                                    Util1.SetCouponAll(party, "くじびき屋の客", 0);
                                }
                            }
                            break;

                        #endregion  
  
                        #region 愚者の楽園
                        case Ikinari.愚者の楽園:

                            //愚者の楽園
                            Util3.SetInitScenarioData(scena, sd);

                            MessageDlg.ShowInfoDialog(
                                    "依頼を受けて村についたところから始まります。");
                            
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            scena.SetStep("ストーリー進行度", 1);
                            scena.BGMFile = "みょしだ_村のテーマ.MID";

                            scena.Cells.Clear();
                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "MapOfWirth.BMP";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Left = 116;
                            cell.Top = 15;
                            cell.Version = 5;
                            cell.Width = 400;
                            cell.Height = 260;
                            cell.FileName = "伊波 曜_村１.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            scena.SetFlag("村人フラグ\\村人１", FLAG_TRUE);
                            scena.SetFlag("初期説明\\村の概観", FLAG_TRUE);

                            break;

                        #endregion      

                        #region 友好都市モンブラン
                        case Ikinari.友好都市モンブラン:
                            //友好都市モンブラン
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            //scena.BGMFile = "DefReset.mid";
                            //scena.SetFlag("都市概観\\セル(暗転)表示", FLAG_TRUE);
                            //scena.SetFlag("都市概観\\セル(暗転)表示", FLAG_FALSE);
                            scena.SetFlag("都市概観\\セル(概観)表示4", FLAG_TRUE);
                            scena.BGMFile = "furo_late_fall.mid";
                            scena.SetFlag("都市概観\\カード(概観)表示", FLAG_TRUE);
                            scena.SetFlag("都市概観\\都市初入場", FLAG_TRUE);

                            break;

                        #endregion

                        #region レインの雑貨屋Ｒ
                        case Ikinari.レインの雑貨屋Ｒ:
                            //レインの雑貨屋Ｒ
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            if (!Util1.CheckCoupon(party, "レインの雑貨屋の場所を知る。"))
                            {
                                Util1.SetCouponAll(party, "レインの雑貨屋の場所を知る。", 0);
                            }
                            else
                            {
                                if (Util1.CheckCoupon(party, "裏・レインの雑貨屋来店"))
                                {
                                    if (DiceRoll.RandomInt(100) < 40)
                                    {
                                        scena.SetFlag("ＮＰＣ\\裏レイン", FLAG_TRUE);
                                    }
                                }

                                if (Util1.CheckCoupon(party, "闇・レインの雑貨屋来店"))
                                {
                                    if (DiceRoll.RandomInt(100) < 30)
                                    {
                                        scena.SetFlag("ＮＰＣ\\闇レイン", FLAG_TRUE);
                                    }
                                    else
                                    {
                                        if (Util1.CheckCoupon(party, "猫・レインの雑貨屋来店"))
                                        {
                                            if (DiceRoll.RandomInt(100) < 70)
                                            {
                                                scena.SetFlag("ＮＰＣ\\闇レイン", FLAG_TRUE);
                                            }
                                        }
                                    }
                                }

                                if (Util1.CheckCoupon(party, "猫・レインの雑貨屋来店"))
                                {
                                    if (DiceRoll.RandomInt(100) < 70)
                                    {
                                        scena.SetFlag("ＮＰＣ\\猫レイン", FLAG_TRUE);
                                    }
                                }

                                if (inn.Gossips.Contains("レイン・猫＆闇"))
                                {
                                    scena.SetFlag("商品表示\\商品エア・隠し", FLAG_TRUE);
                                }

                                if (Util1.CheckCoupon(party, "偽・レインの雑貨屋常連"))
                                {
                                    scena.SetFlag("商品表示\\ゴールデンコボルト", FLAG_TRUE);
                                }

                                if (Util1.CheckCoupon(party, "影・レインの雑貨屋来店"))
                                {
                                    if (DiceRoll.RandomInt(100) < 50)
                                    {
                                        scena.SetFlag("ＮＰＣ\\ミルミー", FLAG_TRUE);

                                        if (DiceRoll.RandomInt(100) < 40)
                                        {
                                            scena.SetFlag("ＮＰＣ\\エリシィ", FLAG_TRUE);
                                        }
                                    }
                                    else
                                    {
                                        if (DiceRoll.RandomInt(100) < 30)
                                        {
                                            scena.SetFlag("ＮＰＣ\\エリシィ", FLAG_TRUE);
                                        }
                                    }
                                }

                                //メルル達出現
                                if (!Util1.CheckCoupon(party, "＿メルル"))
                                {
                                    if (!Util1.CheckCoupon(party, "＿エレナ"))
                                    {
                                        if (!Util1.CheckCoupon(party, "＿リルル"))
                                        {
                                            if (Util1.CheckCoupon(party, "双子と魔水晶４クリア"))
                                            {
                                                if (DiceRoll.RandomInt(100) < 55)
                                                {
                                                    scena.SetFlag("ＮＰＣ\\メルル・リルル・エレナ", FLAG_TRUE);
                                                }
                                            }
                                        }
                                    }
                                }

                                //レイリス出現
                                if (!Util1.CheckCoupon(party, "裏レインと同一の存在"))
                                {
                                    if (Util1.CheckCoupon(party, "記憶の水晶"))
                                    {
                                        if (DiceRoll.RandomInt(100) < 50)
                                        {
                                            scena.SetFlag("ＮＰＣ\\レイリス", FLAG_TRUE);
                                            if (DiceRoll.RandomInt(100) < 20)
                                            {
                                                scena.SetFlag("ＮＰＣ\\エリシィ", FLAG_TRUE);
                                            }
                                        }
                                        else
                                        {
                                            if (DiceRoll.RandomInt(100) < 30)
                                            {
                                                scena.SetFlag("ＮＰＣ\\エリシィ", FLAG_TRUE);
                                            }
                                        }
                                    }
                                }

                                //冒険者出現
                                if (DiceRoll.RandomInt(100) < 50)
                                {
                                    scena.SetFlag("ＮＰＣ\\冒険者Ａ", FLAG_TRUE);
                                }
                                if (DiceRoll.RandomInt(100) < 50)
                                {
                                    scena.SetFlag("ＮＰＣ\\冒険者Ｂ", FLAG_TRUE);
                                }
                                if (DiceRoll.RandomInt(100) < 60)
                                {
                                    scena.SetFlag("ＮＰＣ\\冒険者Ｄ", FLAG_TRUE);
                                }
                                if (DiceRoll.RandomInt(100) < 50)
                                {
                                    scena.SetFlag("ＮＰＣ\\冒険者E", FLAG_TRUE);
                                }
                                if (DiceRoll.RandomInt(100) < 50)
                                {
                                    scena.SetFlag("ＮＰＣ\\冒険者Ｆ", FLAG_TRUE);
                                }
                                if (DiceRoll.RandomInt(100) < 50)
                                {
                                    scena.SetFlag("ＮＰＣ\\冒険者Ｃ", FLAG_TRUE);
                                }
                                //ユフィ出現
                                if (Util1.CheckCoupon(party, "裏レインに勝利"))
                                {
                                    if (!Util1.CheckCoupon(party, "＿ユフィ専用クーポン"))
                                    {
                                        if (DiceRoll.RandomInt(100) < 40)
                                        {
                                            scena.SetFlag("ＮＰＣ\\ユフィ", FLAG_TRUE);
                                        }
                                    }
                                }

                                //ザナス出現
                                if (Util1.CheckCoupon(party, "裏･･･レインの雑貨屋入店"))
                                {
                                    if (DiceRoll.RandomInt(100) < 45)
                                    {
                                        scena.SetFlag("ＮＰＣ\\ザナス", FLAG_TRUE);
                                    }
                                }
                                //レイティ出現

                                if (inn.Gossips.Contains("ラスーク支店イベント終了"))
                                {
                                    if (DiceRoll.RandomInt(100) < 50)
                                    {
                                        scena.SetFlag("ＮＰＣ\\レイティ", FLAG_TRUE);
                                    }
                                }
                            }
                            scena.BGMFile = "BeyondMyHeart.mid";
                            scena.SetFlag("システム\\到着", FLAG_TRUE);
                            
                            break;

                        #endregion

                        #region 鉱石集め
                        case Ikinari.鉱石集め:
                            //鉱石集め
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 2);

                            scena.BGMFile = "gm_m_068_town_005.mid";
                            if (Util1.CheckCoupon(party, "代表者（鉱石集め）"))
                            {
                                scena.SetFlag("代表者の存在", FLAG_TRUE);
                            }

                            break;

                        #endregion

                        #region 湖の妖美
                        case Ikinari.湖の妖美:
                            //湖の妖美
                            Util3.SetInitScenarioData(scena, sd);

                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            scena.BGMFile = "DefInn.mid";
                            scena.SetFlag("汎用メニュー", FLAG_TRUE);
                            break;

                        #endregion

                        #region Cafeteria-聖樹館-

                        case Ikinari.Cafeteria_H_聖樹館_H_:

                            //行く途中エリア到着後
                            scena.AreaNumber = 8;

                            scena.StepValues.Add("βレベル=0");

                            scena.FlagValues.Add("特製コーヒー=0");
                            scena.FlagValues.Add("ココア=0");
                            scena.FlagValues.Add("ハーブティー=0");
                            scena.FlagValues.Add("ロイヤルミルクティー=0");
                            scena.FlagValues.Add("ほっとミルク=0");
                            scena.FlagValues.Add("アイスコーヒー=0");
                            scena.FlagValues.Add("アイスココア=0");
                            scena.FlagValues.Add("オレンジジュース=0");
                            scena.FlagValues.Add("アイスティー=0");
                            scena.FlagValues.Add("ミルクスープ=0");
                            scena.FlagValues.Add("ミネストローネ=0");
                            scena.FlagValues.Add("ケーキ=0");
                            scena.FlagValues.Add("ようかん=0");
                            scena.FlagValues.Add("はじめて来た=1");
                            scena.FlagValues.Add("βと話した=0");
                            scena.FlagValues.Add("βが仲間ではない=1");

                            scena.BGMFile = null;

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "外観.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region MTC-魔物の戦術
                        case Ikinari.MTC_H_魔物の戦術:

                            //伊織庵・店頭　到着時
                            Util3.SetInitScenarioData(scena, sd);

                            scena.AreaNumber = 2;

                            if (party.Characters[0].GetCouponData("伊織庵来店")
                                != null)
                            {
                                //ゴシップクーポン再現
                                string[] mtc_monster
                                = { 
                                    "ゴブリンシャーマン",
                                    "コボルトリーダー",
                                    "ゴブリンロード",
                                    "ホブゴブリン",
                                    "ミノタウロス",
                                    "リザードマン",
                                    "ビホルダー",
                                    "ゴブリン",
                                    "コボルト",
                                    "ウィード",
                                    "トレント",
                                    "トロール",
                                    "ウィスプ",
                                    "ゾンビー",
                                    "バンシー",
                                    "ラット",
                                    "ウルフ",
                                    "エント",
                                    "オーガ",
                                    "ワイト",
                                    "インプ",
                                    "オークロード",
                                    "グリズリー",
                                    "グレムリン",
                                    "ゴースト",
                                    "オーク",
                                    "サラマンダー",
                                    "シルフ",
                                    "ウンディーネ",
                                    "ノーム",
                                    "フォウ",
                                    "スライム"};

                                string[] mtc_monster_step
                                ={ 
                                    @"カード\所持\伏兵_ｺﾞﾌﾞﾘﾝｼｬｰﾏﾝ",
                                    @"カード\所持\防御_ｺﾎﾞﾙﾄﾘｰﾀﾞｰ",
                                    @"カード\所持\攻撃_ｺﾞﾌﾞﾘﾝﾛｰﾄﾞ",
                                    @"カード\所持\攻撃_ホブゴブリン",
                                    @"カード\所持\攻撃_ミノタウロス",
                                    @"カード\所持\補助_リザードマン",
                                    @"カード\所持\伏兵_ビホルダー",
                                    @"カード\所持\攻撃_ゴブリン",
                                    @"カード\所持\防御_コボルト",
                                    @"カード\所持\防御_ウィード",
                                    @"カード\所持\防御_トレント",
                                    @"カード\所持\防御_トロール",
                                    @"カード\所持\補助_ウィスプ",
                                    @"カード\所持\補助_ゾンビー",
                                    @"カード\所持\補助_バンシー",
                                    @"カード\所持\攻撃_ラット",
                                    @"カード\所持\攻撃_ウルフ",
                                    @"カード\所持\防御_エント",
                                    @"カード\所持\補助_オーガ",
                                    @"カード\所持\補助_ワイト",
                                    @"カード\所持\補助_インプ",
                                    @"カード\所持\防御_オークロード",
                                    @"カード\所持\攻撃_グリズリー",
                                    @"カード\所持\攻撃_グレムリン",
                                    @"カード\所持\補助_ゴースト",
                                    @"カード\所持\防御_オーク",
                                    @"カード\所持\伏兵_サラマンダー",
                                    @"カード\所持\伏兵_シルフ",
                                    @"カード\所持\伏兵_ウンディーネ",
                                    @"カード\所持\伏兵_ノーム",
                                    @"カード\所持\伏兵_フォウ",
                                    @"カード\所持\伏兵_スライム"
                                };

                                string[] mtc_count
                                = { "＊１",
                                    "＊２",
                                    "＊３",
                                    "＊４",
                                    "＊５",
                                    "＊６",
                                    "＊７",
                                    "＊８",
                                    "＊９"};

                                for (int j = 0; j < mtc_monster.Length; j++)
                                {
                                    for (int i = 0; i < 9; i++)
                                    {
                                        if (inn.Gossips.Contains("MT_" 
                                            + mtc_monster[j] 
                                            + mtc_count[i]))
                                        {
                                            scena.SetStep(mtc_monster_step[j], (i + 1));
                                            break;
                                        }
                                    }
                                }

                                //"伊織庵来店"済み
                                scena.SetFlag("伊織庵来店", FLAG_TRUE);
                            }

                            Util1.SetCouponAll(party, "伊織庵来店", 0);

                            scena.BGMFile = "bgm_tom_shop.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.SetDefault();
                            cell.FileName = "MapOfWirth.BMP";

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Left = 116;
                            cell.Top = 15;
                            cell.Version = 5;
                            cell.Width = 400;
                            cell.Height = 260;
                            cell.FileName = "bg_伊織庵・店頭.jpg";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Left = 375;
                            cell.Top = 40;
                            cell.Version = 5;
                            cell.Width = 125;
                            cell.Height = 225;
                            cell.FileName = "bg_伊織.bmp";
                            cell.MaskOn = true;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.SetDefault();
                            cell.FileName = "bg_背景フレーム.bmp";
                            cell.MaskOn = true;

                            scena.Cells.Add(cell);
                            break;

                        #endregion

                        #region 5Cards
                        case Ikinari._I_Cards:

                            //5Cards
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = @"DefInn.mid";

                            #region 背景描画
                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Version = 5;
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "MapOfWirth.BMP";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 463;
                            cell.Top = 13;
                            cell.Width = 157;
                            cell.Height = 139;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0;
                            cell.Argb = (uint)180 * 0x1000000 + 0xFFFFFF;
                            cell.FlagName = @"";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 458;
                            cell.Top = 8;
                            cell.Width = 157;
                            cell.Height = 139;
                            cell.ExtCellType = 0x03;
                            cell.MaskOn = false;
                            cell.Gradation = 0;
                            cell.Argb = (uint)170 * 0x1000000;
                            cell.FlagName = @"";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 467;
                            cell.Top = 16;
                            cell.Width = 144;
                            cell.Height = 22;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"現在成績");
                            cell.FontName = "ＭＳ Ｐゴシック";
                            cell.FontSize = 17;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x01;
                            cell.FlagName = @"";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 471;
                            cell.Top = 41;
                            cell.Width = 137;
                            cell.Height = 55;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"$system\勝ち点10$$system\勝ち点01$勝点");
                            cell.DispText.Add(@"$system\負け点10$$system\負け点01$負点");
                            cell.DispText.Add(@"$system\引き分け数$分点");
                            cell.FontName = "ＭＳ Ｐゴシック";
                            cell.FontSize = 17;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x01;
                            cell.FlagName = @"";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            cell = new CellData();
                            cell.Version = 6;
                            cell.Left = 469;
                            cell.Top = 103;
                            cell.Width = 143;
                            cell.Height = 39;
                            cell.ExtCellType = 0x02;
                            cell.MaskOn = false;
                            cell.DispText = new StringCollection();
                            cell.DispText.Add(@"モード：$system\ルール$");
                            cell.DispText.Add(@"GK：%system\GiantKilling%");
                            cell.FontName = "ＭＳ Ｐゴシック";
                            cell.FontSize = 17;
                            cell.Argb = 0xFFFFFFFF;
                            cell.DecoFont = 0x01;
                            cell.FlagName = @"";
                            cell.Visible = true;
                            scena.Cells.Add(cell);

                            break;
                            #endregion

                        #endregion

                        #region 知識と技術の店
                        case Ikinari.知識と技術の店:

                            //知識と技術の店
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = @"room-under-moonlight.mid";

                            scena.SetFlag("0001盗賊", FLAG_TRUE);
                            break;
                        #endregion

                        #region 山稜都市ハルガン
                        case Ikinari.山稜都市ハルガン:

                            //山稜都市ハルガン
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            if (inn.CurrentParty.Characters.Count < 2)
                            {
                                scena.SetStep("人数分岐", 0);
                            }
                            else
                            {
                                scena.SetStep("人数分岐", 1);

                            }
                            scena.BGMFile = @"wmh_8year01_battle.mid";
                            break;
                        #endregion

                        #region 宵楽都市サテラバス
                        case Ikinari.宵楽都市サテラバス:

                            //宵楽都市サテラバス
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            if (inn.CurrentParty.Characters.Count < 2)
                            {
                                scena.SetStep("人数分岐", 0);
                            }
                            else
                            {
                                scena.SetStep("人数分岐", 1);

                            }


                            if (!Util1.CheckCoupon(party, "：ルイド起こしたよ"))
                            {
                                int rValue = DiceRoll.RandomInt(100);
                                if (rValue < 16)
                                {
                                    scena.SetFlag("ルイド", FLAG_TRUE);
                                }
                            }

                            scena.BGMFile = @"mamo_k_city＠rico様&ツカサ様.mid";
                            break;
                        #endregion

                        #region 蒼天都市アクイラ
                        case Ikinari.蒼天都市アクイラ:

                            //蒼天都市アクイラ
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            if (inn.CurrentParty.Characters.Count < 2)
                            {
                                scena.SetStep("人数分岐", 0);
                            }
                            else
                            {
                                scena.SetStep("人数分岐", 1);

                            }
                            scena.BGMFile = @"wmh_mb006_mysterious.mid";
                            break;
                        #endregion

                        #region 蔦纏う街アイヴィドーン
                        case Ikinari.蔦纏う街アイヴィドーン:

                            //蔦纏う街アイヴィドーン
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 2);
                            if (inn.CurrentParty.Characters.Count < 2)
                            {
                                scena.SetStep("人数分岐", 0);
                            }
                            else
                            {
                                scena.SetStep("人数分岐", 1);

                            }

                            if (Util1.CheckPartyLevel(party, 6))
                            {
                                scena.SetFlag("６レベル以上？", FLAG_TRUE);
                            }

                            scena.BGMFile = @"furo_seaside_town.mid";
                            break;
                        #endregion

                        #region 碧海の都アレトゥーザ

                        case Ikinari.碧海の都アレトゥーザ:

                            //碧海の都アレトゥーザ
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = @"tribal bolero_夢音楽館.mid";

                            scena.SetFlag("エリア*広場", FLAG_TRUE);
                            // 全員分のクーポン名
                            List<CouponData> coupons = new List<CouponData>();
                            coupons.Clear();
                            foreach (CharacterData cd in inn.CurrentParty.Characters)
                            {
                                foreach (CouponData coup in cd.Coupons)
                                {
                                    coupons.Add(coup);
                                }
                            }

                            var coupNames = coupons.Select(co => co.Name);
                            if (coupNames.Contains("アレトゥーザの英雄"))
                            {
                                scena.SetStep("Step_Inn", 9);
                                break;
                            }
                            if (coupNames.Contains("碧海の魔女○"))
                            {
                                scena.SetStep("Step_Inn", 8);
                                break;
                            }
                            if (scena.StepValues.Contains("Step_Inn=6")) break;
                            if (coupNames.Contains("＿マスターの依頼×"))
                            {
                                scena.SetStep("Step_Inn", 7); 
                                break;
                            }
                            if (coupNames.Contains("＿マスターの依頼△"))
                            {
                                scena.SetStep("Step_Inn", 5);
                                break;
                            }
                            if (coupNames.Contains("闇窟の妖魔○"))
                            {
                                scena.SetStep("Step_Inn", 5);
                                break;
                            }
                            if (scena.StepValues.Contains("Step_Inn=4")) break;
                            if (coupNames.Contains("デオタトの依頼○"))
                            {
                                scena.SetStep("Step_Inn", 3);
                                break;
                            }
                            if (coupNames.Contains("＿デオタトの依頼△"))
                            {
                                scena.SetStep("Step_Inn", 2);
                                coupons.Reverse();
                                foreach (CouponData coup in coupons)
                                {
                                    if(coup.Name == "＿デオタトの依頼")
                                    {
                                        coupons.Remove(coup);
                                    }
                                }
                                break;
                            }
                            
                            
                            break;
                        #endregion

                        #region 砂港都市グレムリン
                        case Ikinari.砂港都市グレムリン:

                            //砂港都市グレムリン
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);
                            scena.BGMFile = @"田舎町.mp3";
                            break;
                        #endregion

                        #region 傷薬を作ろう
                        case Ikinari.傷薬を作ろう:

                            //傷薬を作ろう
                            Util3.SetInitScenarioData(scena, sd);
                             
                            if (inn.Gossips.Contains("傷薬を作ろう＿01"))
                            {
                                scena.SetFlag("フラグ01", FLAG_TRUE);
                                scena.SetFlag("フラグ02", FLAG_FALSE);
                            }

                            if (DiceRoll.RandomInt(100) < 50)
                            {
                                if (DiceRoll.RandomInt(100) < 40)
                                {
                                    if (DiceRoll.RandomInt(100) < 50)
                                    {
                                        scena.SetStep("傷薬+", 2);
                                    }
                                    else
                                    {
                                        scena.SetStep("傷薬+", 1);
                                    }
                                }
                                else
                                {
                                    if (DiceRoll.RandomInt(100) < 67)
                                    {

                                        if (DiceRoll.RandomInt(100) < 50)
                                        {
                                            scena.SetStep("傷薬+", 5);
                                        }
                                        else
                                        {
                                            scena.SetStep("傷薬+", 4);
                                        }
                                    }
                                    else
                                    {
                                        scena.SetStep("傷薬+", 3);
                                    }

                                }

                            }


                            if (DiceRoll.RandomInt(100) < 30)
                            {
                                if (DiceRoll.RandomInt(100) < 67)
                                {
                                    if (DiceRoll.RandomInt(100) < 50)
                                    {
                                        scena.SetStep("傷薬++", 3);
                                    }
                                    else
                                    {
                                        scena.SetStep("傷薬++", 2);
                                    }
                                }
                                else
                                {
                                    scena.SetStep("傷薬++", 1);
                                }
                            }

                            if (DiceRoll.RandomInt(100) < 4)
                            {
                                scena.SetStep("回復の薬", 1);
                            }
                            scena.SetFlag("表示01", FLAG_TRUE);
                            inn.Gossips.Add("傷薬を作ろう");
                            Util3.SetAreaInitCell(scena, scenaFolder, 1);

                            break;
                            #endregion                            
                            
                        //#region 青を漕ぐ銀の櫂
                        //case Ikinari.青を漕ぐ銀の櫂:

                        //    //青を漕ぐ銀の櫂
                        //    if(inn.CurrentParty.Characters.Count == 6)
                        //    { 
                        //        Util3.SetInitScenarioData(scena, sd);
                        //        scena.BGMFile = "";
                        //        scena.SetFlag("舞台照明", FLAG_TRUE);
                        //        scena.SetFlag("汎用1", FLAG_TRUE);
                        //        if (inn.Gossips.Contains("青を漕ぐ銀の櫂"))
                        //        {
                        //            scena.SetFlag("実は放棄しました", FLAG_TRUE);
                        //        }

                        //        Util3.SetAreaInitCell(scena, scenaFolder, 1);
                        //    }
                        //    else
                        //    {
                        //        MessageDlg.ShowInfoDialog("当シナリオは6人用となっております。\r\n" +
                        //            "人数調整の後、またお越しください。");
                        //    }
                        //    break;
                        //#endregion


                        //#region オクスペレド闘技場V2
                        //case Ikinari.オクスペレド闘技場V2:

                        //    //オクスペレド闘技場V2
                        //    Util3.SetInitScenarioData(scena, sd);
                        //    Util3.SetAreaInitCell(scena, scenaFolder ,1);

                        //    bool hasGossip = false;
                        //    foreach (string gos in inn.Gossips)
                        //    {
                        //        if (gos.Equals("ox更新処理_v190531"))
                        //        {
                        //            hasGossip = true;
                        //            break;
                        //        }
                        //        if (!hasGossip)
                        //        {
                        //            List<string> strList = new List<string>();
                        //            strList.Add("ox闘技場ダブルタッグマッチ開放");
                        //            strList.Add("ox闘技場開放：ネカネ");
                        //            strList.Add("ox闘技場開放：ウォーレス");
                        //            strList.Add("ox闘技場開放：コリネット");
                        //            strList.Add("ox闘技場開放：モーリッツ");
                        //            strList.Add("ox闘技場開放：エドリック");
                        //            strList.Add("ox闘技場開放：ロビーナ");
                        //            strList.Add("ox更新処理_v190531");
                        //            inn.Gossips.AddRange(strList.ToArray());
                        //        }
                        //    }

                        //    inn.Gossips.Remove("ox闘技大会カスタムＯＦＦ");

                        //    int countStep = inn.CurrentParty.Characters.Count;
                        //    scena.SetStep("Sys：PT人数", countStep);
                        //    if (countStep > 1)
                        //    {
                        //        scena.SetFlag("表示：編成_2", FLAG_TRUE);
                        //    } if (countStep > 2)
                        //    {
                        //        scena.SetFlag("表示：編成_3", FLAG_TRUE);
                        //    } if (countStep > 3)
                        //    {
                        //        scena.SetFlag("表示：編成_4", FLAG_TRUE);
                        //    } if (countStep > 4)
                        //    {
                        //        scena.SetFlag("表示：編成_5", FLAG_TRUE);
                        //    } if (countStep > 5)
                        //    {
                        //        scena.SetFlag("表示：編成_6", FLAG_TRUE);
                        //    }


                        //    for (int j = 0; j < 6; j++)
                        //    {
                        //        hasGossip = false;
                        //        for(int i=0;i<10;i++){
                        //            foreach(string gos in inn.Gossips){
                        //                if(gos.Equals("oxBP10^"+ j +"-"+ i)){
                        //                    scena.SetStep(@"BP\10^"+j, i);
                        //                    hasGossip = true;
                        //                    break;
                        //                }
                        //            }
                        //            if (hasGossip) break;
                        //        }
                        //    }


                        //    scena.BGMFile = @"【音楽・効果音】\_DOVA-SYNDROME様\こおろぎ様\騎士団.ogg";
                        //    break;
                        //#endregion            

                        #region 冒険者の自由市２
                        case Ikinari.冒険者の自由市２:

                            //冒険者の自由市２
                            Util3.SetInitScenarioData(scena, sd);
                            Util3.SetAreaInitCell(scena, scenaFolder, 46);

                            scena.BGMFile = @"mariko+占星術\Ombra mai fu.MID";
                            break;
                        #endregion

                        #region ある日の買い物２
                        case Ikinari.ある日の買い物２:

                            //ある日の買い物２
                            //フラグ・ステップを初期状態に
                            Util3.SetInitScenarioData(scena, sd);
                            // 安全地帯はArea:102_移動画面のエリアイベント終了後
                            Util3.SetAreaInitCell(scena, scenaFolder, 102);
                            // Area:102_移動画面に移動してきてスタート：エリアイベントが終了するまでの
                            // ステップ・フラグ・背景・BGM変化などを反映
                            // Area:101_シナリオ開始
                            // エンジンver1.28チェック
                            if (isVariant128() == false)
                            {
                                MessageDlg.ShowInfoDialog("＜エラー発生＞\r\n\r\nこのシナリオはVer1.28専用です。\r\n" +
　                                "Ver1.28以上のエンジンで\r\n" +
                                 "シナリオを再起動してください。");
                                return;
                            }
                            // Package:109_ANTICS処理
                            for (int i = 0; i < party.Characters.Count; i++)
                            {
                                CharacterData cd = party.Characters[i];

                                #region ANTICS処理
                                //ペルソナ
                                if (cd.CheckCoupon("【P:Aquarius】") || 
                                    cd.CheckCoupon("＿【P:Aquarius】") ||
                                    cd.CheckCoupon("【P:Sagittarius】") ||
                                    cd.CheckCoupon("＿【P:Sagittarius】"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.子供口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("【P:Pisces】") ||
                                    cd.CheckCoupon("＿【P:Pisces】") ||
                                    cd.CheckCoupon("【P:Virgo】") ||
                                    cd.CheckCoupon("＿【P:Virgo】") ||
                                    cd.CheckCoupon("【P:Libra】") ||
                                    cd.CheckCoupon("＿【P:Libra】"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.丁寧口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("【P:Aries】") ||
                                    cd.CheckCoupon("＿【P:Aries】") ||
                                    cd.CheckCoupon("【P:Cancer】") ||
                                    cd.CheckCoupon("＿【P:Cancer】"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.粗野口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("【P:Taurus】") ||
                                    cd.CheckCoupon("＿【P:Taurus】"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.老人口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("【P:Vir】") ||
                                    cd.CheckCoupon("＿【P:Vir】"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.男性口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("【P:Femina】") ||
                                    cd.CheckCoupon("＿【P:Femina】"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.女性口調);
                                    continue;
                                }

                                //各種称号
                                if (cd.CheckCoupon("＿老人口調") ||
                                    cd.CheckCoupon("＿老人言葉"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.老人口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("＿粗野口調") ||
                                    cd.CheckCoupon("＿スラム口調"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.粗野口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("＿友達口調") ||
                                    cd.CheckCoupon("＿子供口調") ||
                                    cd.CheckCoupon("＿子供言葉") ||
                                    cd.CheckCoupon("＿お喋り"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.子供口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("＿丁寧口調") ||
                                    cd.CheckCoupon("＿丁寧言葉"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.丁寧口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("＿女性口調"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.女性口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("＿男性口調") ||
                                    cd.CheckCoupon("＿大人口調"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.男性口調);
                                    continue;
                                }
                                // 特徴
                                if (cd.CheckCoupon(CWData.STR_OLD))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.老人口調);
                                    continue;
                                }
                                if (cd.CheckCoupon("＿上品"))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.丁寧口調);
                                    continue;
                                }

                                if (cd.CheckCoupon(CWData.STR_MALE))
                                {
                                    SetTalkCoupon(cd, AnticsTalk.男性口調);
                                }
                                else
                                {
                                    SetTalkCoupon(cd, AnticsTalk.女性口調);
                                }
                                #endregion

                            }
                            // イベントフラグ
                            int randomValue = DiceRoll.RandomInt(4);
                            switch (randomValue)
                            {
                                case 0:
                                    scena.SetFlag(@"02_EVE\F013_ツルギ一騎討ち", FLAG_TRUE);
                                    break;
                                case 1:
                                    scena.SetFlag(@"02_EVE\F014_ウォーズ一騎討ち", FLAG_TRUE);
                                    break;
                                case 2:
                                    scena.SetFlag(@"02_EVE\F015_ういろうイベント", FLAG_TRUE);
                                    break;
                                default:
                                    scena.SetFlag(@"02_EVE\F016_サロメイベント", FLAG_TRUE);
                                    break;
                            }
                            // Area:102_移動画面
                            // Package:102_メニュー初期化
                            // シナリオ初期値FALSE(0)なので、無くていいのでコメントアウト
                            ////for (int i = 1; i <= 10; i++)
                            ////{
                            ////    scena.SetFlag(@"01_GEN\F001_メニュー" + i.ToString("00"), FLAG_FALSE);
                            ////}
                            ////scena.SetFlag(@"迷宮\F001_メニュー", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F002_移動↑", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F003_移動←", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F004_移動→", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F005_移動↓", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F006_扉↑", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F007_扉←", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F008_扉→", FLAG_FALSE);
                            ////scena.SetFlag(@"迷宮\F009_扉↓", FLAG_FALSE);

                            // BGMを停止
                            scena.BGMFile = null;

                            break;

                        #endregion 

                        #region ジーベック銀行
                        case Ikinari.ジーベック銀行:

                            //着いてどういったご用件でしょうかの確認の後
                            scena.AreaNumber = 1;
                            scena.StepValues.Add("所持金確認=0");
                            scena.StepValues.Add("所持金確認２=0");
                            scena.StepValues.Add("所持金確認３=0");
                            scena.BGMFile = "DefInn.mid";

                            scena.Cells = new List<CellData>();

                            cell = new CellData();
                            cell.Left = 0;
                            cell.Top = 0;
                            cell.Version = 5;
                            cell.Width = 632;
                            cell.Height = 420;
                            cell.FileName = "背景＿銀行.bmp";
                            cell.MaskOn = false;
                            cell.FlagName = null;
                            cell.Visible = true;

                            scena.Cells.Add(cell);
                            break;
                        #endregion

                        default:
                            return;
                    }

                    //シナリオに入ったらそれぞれ＿１などの
                    //クーポンが付いている。
                    for (int i = 0; i < party.Characters.Count; i++)
                    {
                        party.Characters[i].SetCoupon(CWData.CouponEnumerateStr[i], 0);
                    }

                    // 実行中シナリオデータに付加
                    inn.CurrentParty.PlayingScenarioData = scena;

                    // 進行中シナリオに追加
                    inn.ProgressScenarios.Add(scenaName);

                    //7 さあ行って来い！
                    inn.Save();

                    MessageDlg.ShowInfoDialog(
                        "正常に保存されました。\r\nバリアントエンジンを実行します。");

                    CWExec();

                }

            }
        }
        #endregion

        #region いきなりSUB
        /// <summary>
        /// ANTICS処理：口調クーポン配布
        /// </summary>
        /// <param name="talkType"></param>
        private void SetTalkCoupon(CharacterData cd ,AnticsTalk talkType)
        {
            for (int i = 0; i <= (int)AnticsTalk.老人口調; i++)
            {
                cd.RemoveCouponData("：" + ((AnticsTalk)i).ToString());
            }

            switch (talkType)
            {
                case AnticsTalk.男性口調:
                    cd.SetCoupon("：男性口調", 0);
                    break;
                case AnticsTalk.女性口調:
                    cd.SetCoupon("：女性口調", 0);
                    break;
                case AnticsTalk.丁寧口調:
                    cd.SetCoupon("：丁寧口調", 0);
                    if (cd.CheckCoupon(CWData.STR_MALE))
                    {
                        cd.SetCoupon("：男性口調", 0);
                    }
                    else
                    {
                        cd.SetCoupon("：女性口調", 0);
                    }
                    break;
                case AnticsTalk.子供口調:
                    cd.SetCoupon("：子供口調", 0);
                    if (cd.CheckCoupon(CWData.STR_MALE))
                    {
                        cd.SetCoupon("：男性口調", 0);
                    }
                    else
                    {
                        cd.SetCoupon("：女性口調", 0);
                    }
                    break;
                case AnticsTalk.粗野口調:
                    cd.SetCoupon("：粗野口調", 0);
                    cd.SetCoupon("：男性口調", 0);
                    break;
                case AnticsTalk.老人口調:
                    cd.SetCoupon("：老人口調", 0);
                    if (cd.CheckCoupon(CWData.STR_MALE))
                    {
                        cd.SetCoupon("：男性口調", 0);
                    }
                    else
                    {
                        cd.SetCoupon("：女性口調", 0);
                    }
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// ver 1.28以上判定
        /// </summary>
        /// <returns></returns>
        private bool isVariant128()
        {
            FileVersionInfo vi =
            FileVersionInfo.GetVersionInfo(FilePath.VariantExe);
            
            bool ret = 
                vi.ProductMajorPart == 1 &&
                (vi.ProductMinorPart > 2 ||
                (vi.ProductMinorPart == 2 &&
                vi.FileBuildPart >= 8));
            
            vi = null;

            return ret;
        }
        #endregion
        
        #endregion

        #region いきなりシナリオに開始ポイントを追加してそこから開始機能
        private void IkinariScenario(string scenaFolder)
        {

            // 1.サマリファイルもきちんと読み込めることを確認
            // 2.Area9999.wid（シナリオ開始セーブポイントエリア）があるか確認
            //   なければ追加し、（シナリオの開始エリアIDを9999に変更はやめておく）、
            //   メニューのシナリオ開始のエリア移動の移動先を元の開始エリアIDにする。
            // 3.ステップ、フラグなどがスタートから変更のない状態にする。
            // 4.宿の設定なども進行中シナリオリストに加えてその上で
            //　セーブし、起動したらオートロードONのときはオートロード
            //　するようにする。

            // 1
            SummaryData sd = null;
            try
            {
                // サマリファイル読み出しチェック
                sd = new SummaryData();
                sd.Read(FilePath.GetSummaryFile(scenaFolder));
            }
            catch (Exception)
            {
                // シナリオは壊れていた！
                MessageDlg.ShowInfoDialog( scenaFolder + " が壊れています。");
                return;
            }

            //2

            // 当然カレントパーティーがいることが前提
            PartyData party = inn.CurrentParty;


            //Area9999.wid があるか
            if (File.Exists(scenaFolder + "\\Area9999.wid") == false)
            {

                if (MessageDlg.ShowYesNoDialog("シナリオの最初にセーブ用のエリアを追加します。" +
                    Environment.NewLine + "よろしいですか？")
                    != DialogResult.Yes) return;

                byte[] allBytes = CWIEngine.Properties.Resources.Area9999;
                string areaFile = scenaFolder + "\\Area9999.wid";
                try
                {
                    // ファイル書き出し
                    File.WriteAllBytes(areaFile, allBytes);

                    // そのファイルをWidAreaとして読み込み
                    WidArea area9999 = new WidArea();
                    area9999.Read(areaFile);
                    // スタート開始メニューのエリア移動のエリアIDを本来の開始エリアIDにする
                    MenuData scenaStart = area9999.Menus[0];
                    EventData eve = scenaStart.Events[0];
                    ContentData startCont = eve.Starts[0];
                    ContentData areaMove = startCont.Children[0];
                    areaMove.Data.UInt = sd.AreaNumber;

                    // 書き出し
                    area9999.Write(areaFile);
                }
                catch (Exception)
                {
                    PartyMaker.CWIErrorMessage("セーブエリアを書き込めませんでした。");
                    return;
                }
            }

            // サマリファイルは AreaNumber = 9999 に書き換えてもいいのだが
            // あとで元に戻したい時に面倒なのであえてそのままにしておくためコメントアウト。
            // サマリを書き換えなければ、戻す時はArea9999.widを削除すればよい。
            ////sd.AreaNumber = 9999;
            ////sd.Write(FilePath.GetSummaryFile(scenaFolder));

            // 3

            //シナリオに入ったらそれぞれ＿１などの
            //クーポンが付いている。

            for (int i = 0; i < party.Characters.Count; i++)
            {
                party.Characters[i].SetCoupon(CWData.CouponEnumerateStr[i], 0);
            }

            ScenarioData scena = new ScenarioData();
            scena.ScenarioFolder = scenaFolder;
            scena.StepValues = new StringCollection();
            scena.FlagValues = new StringCollection();
            scena.PossessInfoCards = new StringCollection();
            scena.PossessNpcCards = new StringCollection();

            Util3.SetInitScenarioData(scena, sd);
            Util3.SetAreaInitCell(scena, scenaFolder, 9999);

            // 4
            // 実行中シナリオデータに付加
            inn.CurrentParty.PlayingScenarioData = scena;

            // F9時に戻るための金額情報
            inn.CurrentParty.MoneyAtStart = inn.CurrentParty.Money;

            inn.ProgressScenarios.Add(sd.ScenarioName);

            // さあ行って来い！
            inn.Save();

            MessageDlg.ShowInfoDialog(
                "正常に保存されました。\r\nバリアントエンジンを実行します。");

            CWExec();

        }

        #endregion

        #region ソート
        /// <summary>
        /// ソートします
        /// </summary>
        /// <param name="sortType"></param>
        private void PCSort(PCソート種類 sortType)
        {            
            // 一応安全のため確認後
            if (DialogResult.Yes == MessageDlg.ShowYesNoDialog(
            "待機PCをソートします。\r\n時間がかかるかもしれません。\r\nよろしいですか？"))
            {
                if (inn.Characters.Count > 0)
                {
                    int count = inn.Characters.Count;

                    // 処理
                    string selectedString = string.Empty;
                    if (sortType == PCソート種類.クーポン || sortType == PCソート種類.カード名)
                    {
                        using (CouponStringDlg csd = new CouponStringDlg(this,sortType))
                        {
                            if (csd.ShowDialog() == DialogResult.OK)
                            {
                                selectedString = csd.SelectedText;
                            }
                            else
                            {
                                return;
                            }
                        }

                    }

                    // 進捗ダイアログ開始
                    StartProgress();
                    // 進捗ダイアログ初期処理待ち
                    WaitProgress();
                    progress.Message = "待機PCをソート中です。";
                    progress.Maximum = 100;
                    progress.Minimum = 0;
                    progress.Value = 0;

                    inn.PCSort(sortType, selectedString);
                    
                    // 表示反映
                    // 個数が同じなのでクリアせずにSetCdする。
                    GC.Collect(2);

                    progress.Value = 50;

                    // 表示用カード追加・配置修正
                    int nowX = -pnlPc.AutoScrollPosition.X;
                    int nowY = -pnlPc.AutoScrollPosition.Y;

                    pnlPc.AutoScrollPosition = new Point(0, 0);

                    DispCard dc;
                    for (int i = 0; i < count; i++)
                    {
                        // プログレスバーのメッセージを変更する
                        progress.Message = "待機PC：" + (i + 1) + "人目を表示しています。";

                        // DispCard は　New　せずにSetHcする。
                        dc = (DispCard)(pnlPc.Controls[i]);
                        dc.SetCd(inn.Characters[i]);

                        //プログレスバーの値を変更する

                        progress.Value = 50 + (i + 1) * 50 / count;
                    }

                    pnlPc.AutoScrollPosition = new Point(nowX, nowY);

                    // 進捗ダイアログ終了
                    EndProgress();
                }
                else
                {
                    MessageDlg.ShowInfoDialog(
                        "待機PCはいません。");

                }
            }
        }
        #endregion

        #region バリアントエンジン打ち落とし
        /// <summary>
        /// バリアントエンジン打ち落とし
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            // カードワースバリアントエンジンが起動してきたら打ち落とす！
            Process[] pss;


            foreach (string engineName in variants)
            {

                pss = Process.GetProcesses();
                foreach (Process ps in pss)
                {
                    // 全プロセスからフォントバリアントを含めて
                    if (Regex.IsMatch(ps.ProcessName,
                        engineName.Substring(0, engineName.Length - 4)) &&
                        ps.ProcessName != "CardWirthEditor")
                    {
                        if (!ps.HasExited)
                        {
                            try
                            {
                                ps.Kill();
                            }
                            catch (InvalidOperationException)
                            {
                                // スルー
                            }
                        }
                    }
                }

            }
        }
        #endregion

        #region MIDI ジュークボックス機能
        /// <summary>
        /// MIDI ジュークボックス機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMusic_Click(object sender, EventArgs e)
        {
            using (JukuBoxDlg jbd = new JukuBoxDlg(this, inn))
            {
                jbd.ShowDialog();
            }
        }
        #endregion

        #region パーティーデータ表示
        /// <summary>
        /// パーティーデータ表示
        /// </summary>
        private void PartyDataShow()
        {
            using (PartyDataListDlg dlg = new PartyDataListDlg(inn.CurrentParty))
            {
                dlg.ShowDialog();
            }
        }
        #endregion

        #region BGM選択
        /// <summary>
        /// BGM選択
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBGM_Click(object sender, EventArgs e)
        {
            using (BGMSelectDlg bgd = new BGMSelectDlg(this))
            {
                bgd.ShowDialog();
            }

        }
        #endregion

        #region フォントバリアント
        /// <summary>
        /// フォントバリアント作成機能
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFontVariant_Click(object sender, EventArgs e)
        {
            //CardWirth MidiPatch はバリアントエンジンではない
            //CardWirthNext はフォントバリアントを作成できない
            bool variantCheck = FilePath.VariantExe.EndsWith(CWData.Variant_CW_MP) ||
                FilePath.VariantExe.EndsWith(CWData.Variant_CWN);

            if (variantCheck)
            {
                MessageDlg.ShowInfoDialog(Path.GetFileNameWithoutExtension(FilePath.VariantExe)
                + "はフォントバリアントを作成できません。");
                return;
            }

            string variant = FilePath.VariantExe;

            MessageDlg.ShowInfoDialog(Path.GetFileNameWithoutExtension(variant) + 
                " バリアントを置き換えます。");

            string variantString = selectedFont.ToString();
            
            string fontVariant = variant.Replace(".exe",
                "_" + variantString + ".exe");

            if (File.Exists(fontVariant))
            {
                MessageDlg.ShowInfoDialog(variantString 
                    + "バリアントはすでに存在します。");

                return;
            }

            try
            {
                byte[] reads = File.ReadAllBytes(variant);
                //バリアントがギガ単位でない事を願う
                int size = reads.Length;
                if (size == 0) return;
                byte[] writes = new byte[size];

                for (int i = 0; i < size; i++)
                {
                    writes[i] = reads[i];

                    // DefInn, DefOpen, DefOver, DefBattleをmp3に変換
                    if (variantMp3)
                    {
                        int mp3Str = SetMp3VariantBytes(writes, reads, i, size);
                        if (mp3Str != 0)
                        {
                            i += mp3Str;
                            continue;
                        }
                    }

                    if (((int)selectedFont) < ((int)SelectedFont.メイリオ))
                   {
                        // ＭＳ 明朝などのプロポーショナルフォント
                        // と普通のフォントが両方ある場合
                        if (reads[i] != 0x82 && reads[i] != 0x4D)
                        {
                            // "ＭＳ "や
                            // "MS UI G"の最初の１バイトと一致していない
                            continue;

                        }
                        else if (i + 11 < size
                                && reads[i + 0] == 0x4D
                                && reads[i + 1] == 0x53
                                && reads[i + 2] == 0x20
                                && reads[i + 3] == 0x55
                                && reads[i + 4] == 0x49
                                && reads[i + 5] == 0x20
                                && reads[i + 6] == 0x47
                                )
                        {
                            //"MS UI G" => "MS UI Gothic"
                            //UIにはとりあえずＵＩフォントが
                            //あってもなくてもプロポーショナルフォント
                            //にしておく
                            // なので "ＭＳ 明朝 　"などへ
                            SetPFontNameBytes(writes,
                                 selectedFont,
                                 i,
                                 11 + 1);
                            i += 11;

                        } else if (i + 9 < size
                                && reads[i + 1] == 0x82
                                && reads[i + 1] == 0x6C
                                && reads[i + 2] == 0x82
                                && reads[i + 3] == 0x72
                                && reads[i + 4] == 0x20
                                )
                        {
                            // "ＭＳ ○○"がファイルにスペースがある。
                            // "ＭＳ "が確定
                            if (reads[i + 5] == 0x83
                                && reads[i + 6] == 0x53)
                            {
                                //"ＭＳ ゴ" => "ＭＳ ゴシック"
                                // なので "ＭＳ 明朝　　"などへ
                                SetFontNameBytes(writes,
                                    selectedFont,
                                    i,
                                    12 + 1);
                                i += 12;

                            }
                            else if (reads[i + 5] == 0x96
                                          && reads[i + 6] == 0xBE)
                            {
                                //"ＭＳ 明" => "ＭＳ 明朝"
                                // なので "HG明朝E  "などへ
                                SetFontNameBytes(writes,
                                   selectedFont,
                                   i,
                                   8 + 1);
                                i += 8;
                            }
                            if (reads[i + 5] == 0x82
                                && reads[i + 6] == 0x6F
                                )
                            {
                                // "ＭＳ Ｐ"が確定
                                if (reads[i + 7] == 0x83
                                    && reads[i + 8] == 0x53
                                    )
                                {
                                    //"ＭＳ Ｐゴ" => "ＭＳ Ｐゴシック" 
                                    // なので "ＭＳ Ｐ明朝　　"などへ
                                    SetPFontNameBytes(writes,
                                         selectedFont,
                                         i,
                                         14 + 1);
                                    i += 14;

                                }
                                else if (reads[i + 7] == 0x96
                                    && reads[i + 8] == 0xBE
                                    )
                                {
                                    //"ＭＳ Ｐ明" => "ＭＳ Ｐ明朝" 
                                    // なので "HGP明朝E 　"などへ
                                    SetPFontNameBytes(writes,
                                         selectedFont,
                                         i,
                                         10 + 1);
                                    i += 10;

                                }

                            }

                        }
                        
                        // 明朝終了
                    }
                    else
                    {
                        //メイリオ以降の１フォントで対応するもの
                        if (reads[i] != 0x82 && reads[i] != 0x4D)
                        {
                            // "ＭＳ "や
                            // "MS UI G"の最初の１バイトと一致していない
                            continue;

                        }
                        else if (i + 11 < size
                                && reads[i + 0] == 0x4D
                                && reads[i + 1] == 0x53
                                && reads[i + 2] == 0x20
                                && reads[i + 3] == 0x55
                                && reads[i + 4] == 0x49
                                && reads[i + 5] == 0x20
                                && reads[i + 6] == 0x47
                                )
                        {
                            //"MS UI G" => "MS UI Gothic"
                            // なので "メイリオ　　"などへ
                            SetFontNameBytes(writes,
                                 selectedFont,
                                 i,
                                 11 + 1);
                            i += 11;
                        }
                        else
                        {
                            if (i + 9 < size
                                && reads[i + 0] == 0x82
                                && reads[i + 1] == 0x6C
                                && reads[i + 2] == 0x82
                                && reads[i + 3] == 0x72
                                && reads[i + 4] == 0x20
                                )
                            {
                                // "ＭＳ "が確定

                                if (reads[i + 5] == 0x82
                                    && reads[i + 6] == 0x6F
                                    )
                                {
                                    // "ＭＳ Ｐ"が確定
                                    if (reads[i + 7] == 0x83
                                        && reads[i + 8] == 0x53
                                        )
                                    {
                                        //"ＭＳ Ｐゴ" => "ＭＳ Ｐゴシック" 
                                        // なので "メイリオ 　　　"などへ
                                        SetFontNameBytes(writes,
                                             selectedFont,
                                             i,
                                             14 + 1);
                                        i += 14;

                                    }
                                    else if (reads[i + 7] == 0x96
                                        && reads[i + 8] == 0xBE
                                        )
                                    {
                                        //"ＭＳ Ｐ明" => "ＭＳ Ｐ明朝" 
                                        // なので "メイリオ 　"などへ
                                        SetFontNameBytes(writes,
                                             selectedFont,
                                             i,
                                             10 + 1);
                                        i += 10;

                                    }

                                }
                                else
                                {
                                    // "ＭＳ Ｐ"ではない
                                    // ＭＳ 明朝、ＭＳ ゴシックも変えてみる

                                    if (reads[i + 5] == 0x83
                                        && reads[i + 6] == 0x53)
                                    {
                                        //"ＭＳ ゴ" => "ＭＳ ゴシック"
                                        // なので "メイリオ 　　"などへ
                                        SetFontNameBytes(writes,
                                             selectedFont,
                                             i,
                                             12 + 1);
                                        i += 12;


                                    }
                                    else if (reads[i + 5] == 0x96
                                          && reads[i + 6] == 0xBE)
                                    {
                                        //"ＭＳ 明" => "ＭＳ 明朝"
                                        // なので "メイリオ "などへ
                                        SetFontNameBytes(writes,
                                           selectedFont,
                                           i,
                                           8 + 1);
                                        i += 8;
                                    }
                                }
                            }
                        }

                        //明朝以外終了
                    }
                }

                File.WriteAllBytes(fontVariant, writes);

                #region コメントアウト
                ////if (this.variantPatch)
                ////{
                ////    if (File.Exists(FilePath.ApplicationFolder + "\\expatch.exe") == false)
                ////    {
                ////        MessageDlg.ShowInfoDialog("expatch.exeが存在しません。");
                ////        return;
                ////    }
                ////    Process ps = null;
                ////    try
                ////    {
                ////        //syncで同期をとってみるver
                ////        ProcessStartInfo info = new ProcessStartInfo(FilePath.ApplicationFolder + "\\expatch.exe", fontVariant);
                ////        info.WindowStyle = ProcessWindowStyle.Hidden;
                ////        ps = Process.Start(info);
                ////        ps.WaitForExit();
                ////        ps.Close();
                ////        ps.Dispose();
                ////    }
                ////    catch (Exception)
                ////    {
                ////        MessageDlg.ShowInfoDialog("expatch.exeの適用に失敗しました。");
                ////        return;
                ////    }
                ////}
                #endregion

                MessageDlg.ShowInfoDialog(variantString + "バリアントを作成しました。" + 
                    (this.variantMp3 ? "(mp3対応化しました。)" : "")
                    //+(this.variantPatch ? "(expatch.exeを適用しました。)" : "")
                    );

            } 
            catch(Exception)
            {
                MessageDlg.ShowInfoDialog("フォントの置き換えに失敗しました。");
            }

        }

        #region バイト変換

        /// <summary>
        /// Def○○○.midをmp3に置換
        /// </summary>
        /// <param name="writes"></param>
        /// <param name="reads"></param>
        /// <param name="index"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        private int SetMp3VariantBytes(
              byte[] writes
            , byte[] reads
            , int index
            , int size)
        {
            int i = index;
            // DefInn.mid
            //44 65 66 49 6E 6E 2E 6D 69 64
            if (i + 9 < size
                                 && reads[i + 0] == 0x44
                                 && reads[i + 1] == 0x65
                                 && reads[i + 2] == 0x66
                                 && reads[i + 3] == 0x49
                                 && reads[i + 4] == 0x6E
                                 && reads[i + 5] == 0x6E
                                 && reads[i + 6] == 0x2E
                                 && reads[i + 7] == 0x6D
                                 && reads[i + 8] == 0x69
                                 && reads[i + 9] == 0x64
                                 )
            {
                // DefInn.mp3
                //44 65 66 49 6E 6E 2E 6D 70 33
                writes[i + 0] = 0x44;
                writes[i + 1] = 0x65;
                writes[i + 2] = 0x66;
                writes[i + 3] = 0x49;
                writes[i + 4] = 0x6E;
                writes[i + 5] = 0x6E;
                writes[i + 6] = 0x2E;
                writes[i + 7] = 0x6D;
                writes[i + 8] = 0x70;
                writes[i + 9] = 0x33;

                return 9;
            }

            //DefOpen.mid
            //44 65 66 4F 70 65 6E 2E 6D 69 64
            if (i + 10 < size
                                 && reads[i + 0] == 0x44
                                 && reads[i + 1] == 0x65
                                 && reads[i + 2] == 0x66
                                 && reads[i + 3] == 0x4F
                                 && reads[i + 4] == 0x70
                                 && reads[i + 5] == 0x65
                                 && reads[i + 6] == 0x6E
                                 && reads[i + 7] == 0x2E
                                 && reads[i + 8] == 0x6D
                                 && reads[i + 9] == 0x69
                                 && reads[i + 10] == 0x64
                                 )
            {
                //DefOpen.mp3
                //44 65 66 4F 70 65 6E 2E 6D 70 33
                writes[i + 0] = 0x44;
                writes[i + 1] = 0x65;
                writes[i + 2] = 0x66;
                writes[i + 3] = 0x4F;
                writes[i + 4] = 0x70;
                writes[i + 5] = 0x65;
                writes[i + 6] = 0x6E;
                writes[i + 7] = 0x2E;
                writes[i + 8] = 0x6D;
                writes[i + 9] = 0x70;
                writes[i + 10] = 0x33;

                return 10;
            }

            //DefOver.mid
            //44 65 66 4F 76 65 72 2E 6D 69 64
            if (i + 10 < size
                                 && reads[i + 0] == 0x44
                                 && reads[i + 1] == 0x65
                                 && reads[i + 2] == 0x66
                                 && reads[i + 3] == 0x4F
                                 && reads[i + 4] == 0x76
                                 && reads[i + 5] == 0x65
                                 && reads[i + 6] == 0x72
                                 && reads[i + 7] == 0x2E
                                 && reads[i + 8] == 0x6D
                                 && reads[i + 9] == 0x69
                                 && reads[i + 10] == 0x64
                                 )
            {
                //DefOver.mp3
                //44 65 66 4F 76 65 72 2E 6D 70 33
                writes[i + 0] = 0x44;
                writes[i + 1] = 0x65;
                writes[i + 2] = 0x66;
                writes[i + 3] = 0x4F;
                writes[i + 4] = 0x76;
                writes[i + 5] = 0x65;
                writes[i + 6] = 0x72;
                writes[i + 7] = 0x2E;
                writes[i + 8] = 0x6D;
                writes[i + 9] = 0x70;
                writes[i + 10] = 0x33;

                return 10;
            }

            //DefBattle.mid
            //44 65 66 42 61 74 74 6C 65 2E 6D 69 64
            if (i + 12 < size
                                 && reads[i + 0] == 0x44
                                 && reads[i + 1] == 0x65
                                 && reads[i + 2] == 0x66
                                 && reads[i + 3] == 0x42
                                 && reads[i + 4] == 0x61
                                 && reads[i + 5] == 0x74
                                 && reads[i + 6] == 0x74
                                 && reads[i + 7] == 0x6C
                                 && reads[i + 8] == 0x65
                                 && reads[i + 9] == 0x2E
                                 && reads[i + 10] == 0x6D
                                 && reads[i + 11] == 0x69
                                 && reads[i + 12] == 0x64
                                 )
            {
                //DefBattle.mp3
                //44 65 66 42 61 74 74 6C 65 2E 6D 70 33
                writes[i + 0] = 0x44;
                writes[i + 1] = 0x65;
                writes[i + 2] = 0x66;
                writes[i + 3] = 0x42;
                writes[i + 4] = 0x61;
                writes[i + 5] = 0x74;
                writes[i + 6] = 0x74;
                writes[i + 7] = 0x6C;
                writes[i + 8] = 0x65;
                writes[i + 9] = 0x2E;
                writes[i + 10] = 0x6D;
                writes[i + 11] = 0x70;
                writes[i + 12] = 0x33;

                return 12;
            }

            return 0;
        }

        /// <summary>
        /// フォント名をバイトで設定
        /// </summary>
        /// <param name="writes"></param>
        /// <param name="fnt"></param>
        /// <param name="index"></param>
        /// <param name="count"></param>
        private void SetFontNameBytes(byte[] writes
            , SelectedFont fnt
            , int index
            , int count)
        {
            int i = index;
            switch (fnt)
            {
                case SelectedFont.ＭＳ明朝:
                    //"ＭＳ 明朝"のみ
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0x6C;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0x72;
                    writes[i + 4] = 0x20;
                    writes[i + 5] = 0x96;
                    writes[i + 6] = 0xBE;
                    writes[i + 7] = 0x92;
                    writes[i + 8] = 0xA9;
                    break;

                case SelectedFont.花鳥風月:
                    //花鳥風月
                    //89 D4 92 B9 95 97 8C 8E
                    writes[i + 0] = 0x89;
                    writes[i + 1] = 0xD4;
                    writes[i + 2] = 0x92;
                    writes[i + 3] = 0xB9;
                    writes[i + 4] = 0x95;
                    writes[i + 5] = 0x97;
                    writes[i + 6] = 0x8C;
                    writes[i + 7] = 0x8E;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.HG明朝B:
                    //48 47 96 BE 92 A9 42
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x96;
                    writes[i + 3] = 0xBE;
                    writes[i + 4] = 0x92;
                    writes[i + 5] = 0xA9;
                    writes[i + 6] = 0x42;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.HG明朝E:
                    //48 47 96 BE 92 A9 45
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x96;
                    writes[i + 3] = 0xBE;
                    writes[i + 4] = 0x92;
                    writes[i + 5] = 0xA9;
                    writes[i + 6] = 0x45;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.HG行書体:
                    //48 47 8D 73 8F 91 91 CC
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x8D;
                    writes[i + 3] = 0x73;
                    writes[i + 4] = 0x8F;
                    writes[i + 5] = 0x83;
                    writes[i + 6] = 0x91;
                    writes[i + 7] = 0xCC;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.ふい字:
                    //82 D3 82 A2 8E 9A
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xD3;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xA2;
                    writes[i + 4] = 0x8E;
                    writes[i + 5] = 0x9A;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.Arisaka:
                    //41 72 69 73 61 6B 61
                    writes[i + 0] = 0x41;
                    writes[i + 1] = 0x72;
                    writes[i + 2] = 0x69;
                    writes[i + 3] = 0x73;
                    writes[i + 4] = 0x61;
                    writes[i + 5] = 0x6B;
                    writes[i + 6] = 0x61;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.Osaka:
                    //4F 73 61 6B 61
                    writes[i + 0] = 0x4F;
                    writes[i + 1] = 0x73;
                    writes[i + 2] = 0x61;
                    writes[i + 3] = 0x6B;
                    writes[i + 4] = 0x61;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                // 以下１フォント
                case SelectedFont.メイリオ:
                    //83 81 83 43 83 8A 83 49
                    writes[i + 0] = 0x83;
                    writes[i + 1] = 0x81;
                    writes[i + 2] = 0x83;
                    writes[i + 3] = 0x43;
                    writes[i + 4] = 0x83;
                    writes[i + 5] = 0x8A;
                    writes[i + 6] = 0x83;
                    writes[i + 7] = 0x49;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.MeiryoAA:
                    //MeiryoAA
                    //4D 65 69 72 79 6F 41 41
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x65;
                    writes[i + 2] = 0x69;
                    writes[i + 3] = 0x72;
                    writes[i + 4] = 0x79;
                    writes[i + 5] = 0x6F;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.KugimmyAA:
                    //KugimmyAA
                    //4B 75 67 69 6D 6D 79 41 41
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x75;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x69;
                    writes[i + 4] = 0x6D;
                    writes[i + 5] = 0x6D;
                    writes[i + 6] = 0x79;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x41;
                    break;


                case SelectedFont.HenryAA:
                    //48 65 6E 72 79 41 41
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x65;
                    writes[i + 2] = 0x6E;
                    writes[i + 3] = 0x72;
                    writes[i + 4] = 0x79;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.YukarryAA:
                    //59 75 6B 61 72 72 79 41 41
                    writes[i + 0] = 0x59;
                    writes[i + 1] = 0x75;
                    writes[i + 2] = 0x6B;
                    writes[i + 3] = 0x61;
                    writes[i + 4] = 0x72;
                    writes[i + 5] = 0x72;
                    writes[i + 6] = 0x79;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x41;
                    break;


                case SelectedFont.Yukarina:
                    //59 75 6B 61 72 69 6E 61
                    writes[i + 0] = 0x59;
                    writes[i + 1] = 0x75;
                    writes[i + 2] = 0x6B;
                    writes[i + 3] = 0x61;
                    writes[i + 4] = 0x72;
                    writes[i + 5] = 0x69;
                    writes[i + 6] = 0x6E;
                    writes[i + 7] = 0x61;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.Osaka_AA:
                    //Osaka-AA
                    //4F 73 61 6B 61 2D 41 41
                    writes[i + 0] = 0x4F;
                    writes[i + 1] = 0x73;
                    writes[i + 2] = 0x61;
                    writes[i + 3] = 0x6B;
                    writes[i + 4] = 0x61;
                    writes[i + 5] = 0x2D;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.Tarisaka:
                    //Tarisaka
                    //54 61 72 69 73 61 6B 61
                    writes[i + 0] = 0x54;
                    writes[i + 1] = 0x61;
                    writes[i + 2] = 0x72;
                    writes[i + 3] = 0x69;
                    writes[i + 4] = 0x73;
                    writes[i + 5] = 0x61;
                    writes[i + 6] = 0x6B;
                    writes[i + 7] = 0x61;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.Axis_AA:
                    //41 78 69 73 5F 41 41
                    writes[i + 0] = 0x41;
                    writes[i + 1] = 0x78;
                    writes[i + 2] = 0x69;
                    writes[i + 3] = 0x73;
                    writes[i + 4] = 0x5F;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.AXIS_R_AA:
                    //AXIS-R_AA
                    //41 58 49 53 2D 52 5F 41 41
                    writes[i + 0] = 0x41;
                    writes[i + 1] = 0x58;
                    writes[i + 2] = 0x49;
                    writes[i + 3] = 0x53;
                    writes[i + 4] = 0x2D;
                    writes[i + 5] = 0x52;
                    writes[i + 6] = 0x5F;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x41;
                    break;


                case SelectedFont.BGSGAAV:
                    //42 47 53 47 41 41 56
                    writes[i + 0] = 0x42;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x53;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x41;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x56;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.CMG_AA:
                    //CMG-AA
                    //43 4D 47 2D 41 41
                    writes[i + 0] = 0x43;
                    writes[i + 1] = 0x4D;
                    writes[i + 2] = 0x47;
                    writes[i + 3] = 0x2D;
                    writes[i + 4] = 0x41;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.DFNT:
                    //44 46 4E 54
                    writes[i + 0] = 0x44;
                    writes[i + 1] = 0x46;
                    writes[i + 2] = 0x4E;
                    writes[i + 3] = 0x54;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.FMBO:
                    //46 4D 42 4F
                    writes[i + 0] = 0x46;
                    writes[i + 1] = 0x4D;
                    writes[i + 2] = 0x42;
                    writes[i + 3] = 0x4F;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.IWGAA:
                    //49 57 47 41 41
                    writes[i + 0] = 0x49;
                    writes[i + 1] = 0x57;
                    writes[i + 2] = 0x47;
                    writes[i + 3] = 0x41;
                    writes[i + 4] = 0x41;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.HKGAA:
                    //HKGAA
                    //48 4B 47 41 41
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x4B;
                    writes[i + 2] = 0x47;
                    writes[i + 3] = 0x41;
                    writes[i + 4] = 0x41;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.HRAA:
                    //HRAA
                    //48 52 41 41
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x52;
                    writes[i + 2] = 0x41;
                    writes[i + 3] = 0x41;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.KGCG_NAA:
                    //4B 47 43 47 5F 4E 41 41
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x43;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x5F;
                    writes[i + 5] = 0x4E;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.KGCG_NAA2:
                    //KGCG_NAA2
                    //4B 47 43 47 5F 4E 41 41 32
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x43;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x5F;
                    writes[i + 5] = 0x4E;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x32;
                    break;

                case SelectedFont.KGCG_Web:
                    //4B 47 43 47 5F 57 65 62
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x43;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x5F;
                    writes[i + 5] = 0x57;
                    writes[i + 6] = 0x65;
                    writes[i + 7] = 0x62;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.KGCG_AA:
                    //4B 47 43 47 5F 41 41
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x43;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x5F;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.KGFT:
                    //4B 47 46 54
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x46;
                    writes[i + 3] = 0x54;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.KGHC_AA:
                    //KGHC-AA
                    //4B 47 48 43 2D 41 41
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x48;
                    writes[i + 3] = 0x43;
                    writes[i + 4] = 0x2D;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.KGWa:
                    //4B 47 57 61
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x57;
                    writes[i + 3] = 0x61;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.KleeAA:
                    //4B 6C 65 65 41 41
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x6C;
                    writes[i + 2] = 0x65;
                    writes[i + 3] = 0x65;
                    writes[i + 4] = 0x41;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.KozGoM_AA:
                    //KozGoM-AA
                    //4B 6F 7A 47 6F 4D 2D 41 41
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x6F;
                    writes[i + 2] = 0x7A;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x6F;
                    writes[i + 5] = 0x4D;
                    writes[i + 6] = 0x2D;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x41;
                    break;


                case SelectedFont.KozGoR_AA:
                    //KozGoR-AA
                    //4B 6F 7A 47 6F 52 2D 41 41
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x6F;
                    writes[i + 2] = 0x7A;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x6F;
                    writes[i + 5] = 0x52;
                    writes[i + 6] = 0x2D;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x41;
                    break;

                case SelectedFont.MillyAA:
                    //MillyAA
                    //4D 69 6C 6C 79 41 41
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x6C;
                    writes[i + 3] = 0x6C;
                    writes[i + 4] = 0x79;
                    writes[i + 5] = 0x41;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;

                    break;

                case SelectedFont.Migu_1M:
                    //Migu 1M
                    //4D 69 67 75 20 31 4D
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x75;
                    writes[i + 4] = 0x20;
                    writes[i + 5] = 0x31;
                    writes[i + 6] = 0x4D;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;

                    break;

                case SelectedFont.Migu_1P:
                    //Migu 1P
                    //4D 69 67 75 20 31 50
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x75;
                    writes[i + 4] = 0x20;
                    writes[i + 5] = 0x31;
                    writes[i + 6] = 0x50;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;

                    break;

                case SelectedFont.Migu_1VS:
                    //Migu_1VS
                    //4D 69 67 75 20 31 56 53
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x75;
                    writes[i + 4] = 0x20;
                    writes[i + 5] = 0x31;
                    writes[i + 6] = 0x56;
                    writes[i + 7] = 0x53;
                    writes[i + 8] = 0x00;

                    break;

                case SelectedFont.MigMix_1M:
                    //MigMix 1M
                    //4D 69 67 4D 69 78 20 31 4D
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x4D;
                    writes[i + 4] = 0x69;
                    writes[i + 5] = 0x78;
                    writes[i + 6] = 0x20;
                    writes[i + 7] = 0x31;
                    writes[i + 8] = 0x4D;

                    break;

                case SelectedFont.MigMix_1P:
                    //MigMix 1P
                    //4D 69 67 4D 69 78 20 31 50
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x4D;
                    writes[i + 4] = 0x69;
                    writes[i + 5] = 0x78;
                    writes[i + 6] = 0x20;
                    writes[i + 7] = 0x31;
                    writes[i + 8] = 0x50;

                    break;

                case SelectedFont.MigMix_2M:
                    //MigMix 2M
                    //4D 69 67 4D 69 78 20 32 4D
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x4D;
                    writes[i + 4] = 0x69;
                    writes[i + 5] = 0x78;
                    writes[i + 6] = 0x20;
                    writes[i + 7] = 0x32;
                    writes[i + 8] = 0x4D;

                    break;

                case SelectedFont.MigMix_2P:
                    //MigMix 2P
                    //4D 69 67 4D 69 78 20 32 50
                    writes[i + 0] = 0x4D;
                    writes[i + 1] = 0x69;
                    writes[i + 2] = 0x67;
                    writes[i + 3] = 0x4D;
                    writes[i + 4] = 0x69;
                    writes[i + 5] = 0x78;
                    writes[i + 6] = 0x20;
                    writes[i + 7] = 0x32;
                    writes[i + 8] = 0x50;

                    break;

                case SelectedFont.SherryAA:
                    //SherryAA
                    //53 68 65 72 72 79 41 41 
                    writes[i + 0] = 0x53;
                    writes[i + 1] = 0x68;
                    writes[i + 2] = 0x65;
                    writes[i + 3] = 0x72;
                    writes[i + 4] = 0x72;
                    writes[i + 5] = 0x79;
                    writes[i + 6] = 0x41;
                    writes[i + 7] = 0x41;
                    writes[i + 8] = 0x00;

                    break;

                case SelectedFont.Takao明朝:
                    //Takao明朝
                    //54 61 6B 61 6F 96 BE 92 A9 
                    writes[i + 0] = 0x54;
                    writes[i + 1] = 0x61;
                    writes[i + 2] = 0x6B;
                    writes[i + 3] = 0x61;
                    writes[i + 4] = 0x6F;
                    writes[i + 5] = 0x96;
                    writes[i + 6] = 0xBE;
                    writes[i + 7] = 0x92;
                    writes[i + 8] = 0xA9;

                    break;

                case SelectedFont.T1号:
                    //54 31 8D 86
                    writes[i + 0] = 0x54;
                    writes[i + 1] = 0x31;
                    writes[i + 2] = 0x8D;
                    writes[i + 3] = 0x86;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.Konatu:
                    //4B 6F 6E 61 75 76
                    writes[i + 0] = 0x4B;
                    writes[i + 1] = 0x6F;
                    writes[i + 2] = 0x6E;
                    writes[i + 3] = 0x61;
                    writes[i + 4] = 0x75;
                    writes[i + 5] = 0x76;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

#if DEBUG

                case SelectedFont.あんずもじ:　// apjfont
                    //61 70 6A 66 6F 6E 74
                    writes[i + 0] = 0x61;
                    writes[i + 1] = 0x70;
                    writes[i + 2] = 0x6A;
                    writes[i + 3] = 0x66;
                    writes[i + 4] = 0x6F;
                    writes[i + 5] = 0x6E;
                    writes[i + 6] = 0x74;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.あんずもじ奏:　// apjkfont
                    //61 70 6A 6B 66 6F 6E 74
                    writes[i + 0] = 0x61;
                    writes[i + 1] = 0x70;
                    writes[i + 2] = 0x6A;
                    writes[i + 3] = 0x6B;
                    writes[i + 4] = 0x66;
                    writes[i + 5] = 0x6F;
                    writes[i + 6] = 0x6E;
                    writes[i + 7] = 0x74;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.あんずもじ湛:　// apjffont
                    //61 70 6A 66 66 6F 6E 74
                    writes[i + 0] = 0x61;
                    writes[i + 1] = 0x70;
                    writes[i + 2] = 0x6A;
                    writes[i + 3] = 0x66;
                    writes[i + 4] = 0x66;
                    writes[i + 5] = 0x6F;
                    writes[i + 6] = 0x6E;
                    writes[i + 7] = 0x74;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.女の子フォント:// 女の子
                    //8F 97 82 CC 8E 71
                    writes[i + 0] = 0x8F;
                    writes[i + 1] = 0x97;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xCC;
                    writes[i + 4] = 0x8E;
                    writes[i + 5] = 0x71;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.たぬき油性マジック: //たぬき油
                    //82 BD 82 CA 82 AB 96 FB
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xBD;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xCA;
                    writes[i + 4] = 0x82;
                    writes[i + 5] = 0xAB;
                    writes[i + 6] = 0x96;
                    writes[i + 7] = 0xFB;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.ほにゃ字ふとっちょ:
                    //ほにゃ字F
                    //82 D9 82 C9 82 E1 8E 9A 46
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xD9;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xC9;
                    writes[i + 4] = 0x82;
                    writes[i + 5] = 0xE1;
                    writes[i + 6] = 0x8E;
                    writes[i + 7] = 0x9A;
                    writes[i + 8] = 0x46;
                    break;

                case SelectedFont.ほにゃ字ひょろり:
                    //ほにゃ字H
                    //82 D9 82 C9 82 E1 8E 9A 48
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xD9;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xC9;
                    writes[i + 4] = 0x82;
                    writes[i + 5] = 0xE1;
                    writes[i + 6] = 0x8E;
                    writes[i + 7] = 0x9A;
                    writes[i + 8] = 0x48;
                    break;

#endif
                case SelectedFont.ほにゃ字:
                    //82 D9 82 C9 82 E1 8E 9A
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xD9;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xC9;
                    writes[i + 4] = 0x82;
                    writes[i + 5] = 0xE1;
                    writes[i + 6] = 0x8E;
                    writes[i + 7] = 0x9A;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.イバラ字:
                    //83 43 83 6F 83 89 8E 9A
                    writes[i + 0] = 0x83;
                    writes[i + 1] = 0x43;
                    writes[i + 2] = 0x83;
                    writes[i + 3] = 0x6F;
                    writes[i + 4] = 0x83;
                    writes[i + 5] = 0x89;
                    writes[i + 6] = 0x8E;
                    writes[i + 7] = 0x9A;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.切絵字:
                    //90 D8 8A 47 8E 9A
                    writes[i + 0] = 0x90;
                    writes[i + 1] = 0xD8;
                    writes[i + 2] = 0x8A;
                    writes[i + 3] = 0x47;
                    writes[i + 4] = 0x8E;
                    writes[i + 5] = 0x9A;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.水面字:
                    //90 85 96 CA 8E 9A
                    writes[i + 0] = 0x90;
                    writes[i + 1] = 0x85;
                    writes[i + 2] = 0x96;
                    writes[i + 3] = 0xCA;
                    writes[i + 4] = 0x8E;
                    writes[i + 5] = 0x9A;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.モフ字:
                    //83 82 83 74 8E 9A
                    writes[i + 0] = 0x83;
                    writes[i + 1] = 0x82;
                    writes[i + 2] = 0x83;
                    writes[i + 3] = 0x74;
                    writes[i + 4] = 0x8E;
                    writes[i + 5] = 0x9A;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.へた字:
                    //82 D6 82 BD 8E 9A
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xD6;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xBD;
                    writes[i + 4] = 0x8E;
                    writes[i + 5] = 0x9A;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.雑字:
                    //8E 47 8E 9A
                    writes[i + 0] = 0x8E;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x8E;
                    writes[i + 3] = 0x9A;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;


                case SelectedFont.怨霊:
                    //89 85 97 EC
                    writes[i + 0] = 0x89;
                    writes[i + 1] = 0x85;
                    writes[i + 2] = 0x97;
                    writes[i + 3] = 0xEC;
                    writes[i + 4] = 0x00;
                    writes[i + 5] = 0x00;
                    writes[i + 6] = 0x00;
                    writes[i + 7] = 0x00;
                    writes[i + 8] = 0x00;
                    break;

                case SelectedFont.あくびん:
                    //82 A0 82 AD 82 D1 82 F1
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xA0;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xAD;
                    writes[i + 4] = 0x82;
                    writes[i + 5] = 0xD1;
                    writes[i + 6] = 0x82;
                    writes[i + 7] = 0xF1;
                    writes[i + 8] = 0x00;
                    break;

                default:
                    PartyMaker.CWIErrorMessage("フォントバリアントの定義がありません。");
                    break;
            }

            for (int j = 9; j < count; j++)
            {
                writes[i + j] = 0x00; 
            }

            return;
        }

        /// <summary>
        /// プロポーショナルフォントをバイトで設定（等幅と異なる場合）
        /// </summary>
        /// <param name="writes"></param>
        /// <param name="fnt"></param>
        /// <param name="index"></param>
        /// <param name="count"></param>
        private void SetPFontNameBytes(byte[] writes
            , SelectedFont fnt
            , int index
            , int count)
        {
            //プロポーショナルフォントが
            //別名のときに呼ぶメソッド
            int i = index;
            switch (fnt)
            {

                case SelectedFont.ＭＳ明朝:
                    //"ＭＳ Ｐ明朝"
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0x6C;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0x72;
                    writes[i + 4] = 0x20;
                    writes[i + 5] = 0x82;
                    writes[i + 6] = 0x6F;
                    writes[i + 7] = 0x96;
                    writes[i + 8] = 0xBE;
                    writes[i + 9] = 0x92;
                    writes[i + 10] = 0xA9;
                    break;

                case SelectedFont.花鳥風月:
                    //花鳥風月Ｐ
                    //89 D4 92 B9 95 97 8C 8E 82 6F
                    writes[i + 0] = 0x89;
                    writes[i + 1] = 0xD4;
                    writes[i + 2] = 0x92;
                    writes[i + 3] = 0xB9;
                    writes[i + 4] = 0x95;
                    writes[i + 5] = 0x97;
                    writes[i + 6] = 0x8C;
                    writes[i + 7] = 0x8E;
                    writes[i + 8] = 0x82;
                    writes[i + 9] = 0x6F;
                    writes[i + 10] = 0x00;
                    break;

                case SelectedFont.HG明朝B:
                    //HGP明朝B
                    //48 47 50 96 BE 92 A9 42
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x50;
                    writes[i + 3] = 0x96;
                    writes[i + 4] = 0xBE;
                    writes[i + 5] = 0x92;
                    writes[i + 6] = 0xA9;
                    writes[i + 7] = 0x42;
                    writes[i + 8] = 0x00;
                    writes[i + 9] = 0x00;
                    writes[i + 10] = 0x00;
                    break;

                case SelectedFont.HG明朝E:
                    //HGP明朝E
                    //48 47 50 96 BE 92 A9 45
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x50;
                    writes[i + 3] = 0x96;
                    writes[i + 4] = 0xBE;
                    writes[i + 5] = 0x92;
                    writes[i + 6] = 0xA9;
                    writes[i + 7] = 0x45;
                    writes[i + 8] = 0x00;
                    writes[i + 9] = 0x00;
                    writes[i + 10] = 0x00;
                    break;

                case SelectedFont.HG行書体:
                    //HGP行書体
                    //48 47 50 8D 73 8F 91 91 CC
                    writes[i + 0] = 0x48;
                    writes[i + 1] = 0x47;
                    writes[i + 2] = 0x50;
                    writes[i + 3] = 0x8D;
                    writes[i + 4] = 0x73;
                    writes[i + 5] = 0x8F;
                    writes[i + 6] = 0x83;
                    writes[i + 7] = 0x91;
                    writes[i + 8] = 0xCC;
                    writes[i + 9] = 0x00;
                    writes[i + 10] = 0x00;
                    break;

                case SelectedFont.ふい字:
                    //ふい字Ｐ
                    //82 D3 82 A2 8E 9A 82 6F
                    writes[i + 0] = 0x82;
                    writes[i + 1] = 0xD3;
                    writes[i + 2] = 0x82;
                    writes[i + 3] = 0xA2;
                    writes[i + 4] = 0x8E;
                    writes[i + 5] = 0x9A;
                    writes[i + 6] = 0x82;
                    writes[i + 7] = 0x6F;
                    writes[i + 8] = 0x00;
                    writes[i + 9] = 0x00;
                    writes[i + 10] = 0x00;
                    break;

                case SelectedFont.Arisaka:
                    //Arisaka-AA
                    //41 72 69 73 61 6B 61 2D 41 41
                    writes[i + 0] = 0x41;
                    writes[i + 1] = 0x72;
                    writes[i + 2] = 0x69;
                    writes[i + 3] = 0x73;
                    writes[i + 4] = 0x61;
                    writes[i + 5] = 0x6B;
                    writes[i + 6] = 0x61;
                    writes[i + 7] = 0x2D;
                    writes[i + 8] = 0x41;
                    writes[i + 9] = 0x41;
                    writes[i + 10] = 0x00;
                    break;

                case SelectedFont.Osaka:
                    //Osakaー等幅
                    //4F 73 61 6B 61 81 5B 93 99 95 9D
                    writes[i + 0] = 0x4F;
                    writes[i + 1] = 0x73;
                    writes[i + 2] = 0x61;
                    writes[i + 3] = 0x6B;
                    writes[i + 4] = 0x61;
                    writes[i + 5] = 0x81;
                    writes[i + 6] = 0x5B;
                    writes[i + 7] = 0x93;
                    writes[i + 8] = 0x99;
                    writes[i + 9] = 0x95;
                    writes[i + 10] = 0x9D;
                    break;
            }

            for (int j = 11; j < count; j++)
            {
                writes[i + j] = 0x00;
            }

            return;
        }

        #endregion

        private void cmbPartyMake_SelectedIndexChanged(object sender, EventArgs e)
        {
            SaveInitXml();
        }

        #endregion

        #endregion
    }

    #region 選択されたフォント
    public enum SelectedFont
    {
        ＭＳ明朝,
        花鳥風月,
        HG明朝B,
        HG明朝E,
        HG行書体,
        ふい字,
        Arisaka,
        Osaka,
        // メイリオ以降は１フォントで置換
        メイリオ,
        MeiryoAA,
        KugimmyAA,
        HenryAA,
        YukarryAA,
        Yukarina,
        Osaka_AA,
        Tarisaka,
        Axis_AA,
        AXIS_R_AA,
        BGSGAAV,
        CMG_AA,
        DFNT,
        FMBO,
        IWGAA,
        HKGAA,
        HRAA,
        KGCG_NAA,
        KGCG_NAA2,
        KGCG_Web,
        KGCG_AA,
        KGFT,
        KGHC_AA,
        KGWa,
        KleeAA,
        KozGoM_AA,
        KozGoR_AA,
        MillyAA,
        Migu_1M,
        Migu_1P,
        Migu_1VS,
        MigMix_1M,
        MigMix_1P,
        MigMix_2M,
        MigMix_2P,
        SherryAA,
        Takao明朝,
        T1号,
        Konatu,
#if DEBUG
        // デバッグ時のみ使用可能
        // あんずもじ,あんずもじ奏,あんずもじ湛 の書体名を apjfont,apjkfont,apjffontに置換
        // 女の子フォント の書体名を 女の子 に置換
        // ほにゃ字ふとっちょ,ほにゃ字ひょろり の FamilyとUniqueIdentifier をほにゃ字F,Hに変換
        あんずもじ,
        あんずもじ奏,
        あんずもじ湛,
        女の子フォント,
        たぬき油性マジック,
        ほにゃ字ふとっちょ,
        ほにゃ字ひょろり,
#endif
        ほにゃ字,
        イバラ字,
        切絵字,
        水面字,
        モフ字,
        へた字,
        雑字,
        怨霊,
        あくびん
    }
    #endregion

    #region いきなりの遷移先
    public enum Ikinari
    {
        交易都市リューン,
        ゴブリンの洞窟,
        リューンの鍛冶屋,
        隠者の庵,
        悪魔の酒蔵,
        花散里,
        槍弓術指南道場,
        兵法指南所ミフネ,
        師匠屋閉店セール,
        武の街キラナン,
        ボサの散歩,
        酒と露店の街デューン,
        落日の村オランジェル,
        夜の交易都市,
        冒険者の街カーゾ,
        冒険者の都市フォルザーヴ,
        カードマニア,
        ようこそ札闘亭へ,
        スティープルチェイサー,
        特殊な種,
        悪魔の書,
        冒険者支援センター,
        天使養成所第1課程第１話,
        ジョブの街,
        RP5出張所,
        ジュークボックス,
        チュートリアル,
        ペルラの市,
        // _A_は＆に置き換えます。
        アロマ_A_ハーブの店,
        // _B_は"に置き換えます。
        おべんと屋_B_のばら_B_,
        雑貨店ナナメナ,
        枝折屋,
        タンスの春木屋,
        白鍋通りの話,
        //_C_は　、_G_は―
        _C__Y_道端で老婆が_G__G_,
        Millionth_Y_Edition,
        カフェ_H_ニャニャ_H_,
        黄金の釜,
        銀のトルトル,
        色硝子の窓,
        青果商グラニュート,
        _H_茶々_H_,
        瓶屋,
        木苺通りの話,
        かえる屋,
        ゲームで一休み,
        // _Q_は？
        ゴミの山_Q_宝の山_Q_,
        サンダース農場,
        スキル屋ぎょらん,
        音箱,
        玉猪竜商店,
        A_Y_RAG_Y_MAN,
        傷薬屋さん,
        路地裏の隠れ酒屋,
        黒砂糖路地の話,
        メレンダ祭り準備中,
        メレンダ祭り開催中,
        メレンダ数え歌会議,
        // _Y_は 、_Z_は=
        // _L_は「に置き換えます。
        // _N_は」に置き換えます。
        希望の都_Y__L_Ｆ_Z_ベル_N_,
        TSUKE_Y_CLICKER,
        ある小さなお店,
        // _K_は（、_T_は）に置き換えます。
        蚤の市,
        // _S_は#に置き換えます。
        交易都市リューン_S_Cecil,
        交易都市リューン_S_C2,
        古城の老兵,
        旧街道,
        異端なる魔剣の入手法,
        武具_D__L_夕日の鉄撃_N_,
        アタノオル,
        調合屋ケムス,
        ｱｲﾃﾑｼｮｯﾌﾟ仔馬の尻尾,
        暗黒街ペテンザム,
        ゴブリンの収集,
        // _F_は！に置き換えます。
        新装開店_F_,
        // _E_は!に置き換えます。
        釣りに行こうっ_E__E_,
        // _M_は。に置き換えます。
        幸せ運ぶお店です_M_,
        天空都市タラント,
        斜陽の魔法店,
        魔道具バザー,
        ムーランルージュ,
        お寿司始めました,
        くじびき屋,
        愚者の楽園,
        友好都市モンブラン,
        レインの雑貨屋Ｒ,
        鉱石集め,
        湖の妖美,
        // _H_ は - に置き換えます。
        Cafeteria_H_聖樹館_H_,
        MTC_H_魔物の戦術,
        //心のカクテル診断,
        //_I_は5に置き換えます。
        _I_Cards,
        知識と技術の店,
        山稜都市ハルガン,
        宵楽都市サテラバス,
        蒼天都市アクイラ,
        蔦纏う街アイヴィドーン,
        碧海の都アレトゥーザ,
        砂港都市グレムリン,
        傷薬を作ろう,
        //青を漕ぐ銀の櫂,
        //オクスペレド闘技場V2,
        冒険者の自由市２,
        ある日の買い物２,
        ジーベック銀行
    }

    /// <summary>
    /// ANTICS処理：口調
    /// </summary>
    public enum AnticsTalk
    {
        男性口調,
        女性口調,
        丁寧口調,
        子供口調,
        粗野口調,
        老人口調
    }

    /// <summary>
    /// ジョブの街のジョブ
    /// </summary>
    public enum Job
    {
        // ジョブの街のジョブ
        剣士,
        魔法使い,
        シーフ,
        アーチャー,
        忍者,
        侍,
        // 格闘家　はまだフラグが無い
        格闘家,
        // 召喚師　はまだフラグが無い
        召喚師,
        ガンマン,
        ヘルパー,
        魔法剣士,
        // 剣神　はまだフラグが無い
        剣神,
        エルフ戦士,
        精霊術士,
        騎士
    }
    #endregion

}